      program sepmesh
c ======================================================================
c
c        programmer     Guus Segal
c        version 2.1    date   30-04-1995 Removal of buffer
c        version 2.0    date   11-11-1990 Complete reorganization
c        version 1.0    date   26-03-1988
c
c
c
c   copyright (c) 1988-1995  "Ingenieursbureau SEPRA"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "Ingenieursbureau SEPRA".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c      Program sepmesh reads an input file containing information of the
c      mesh, and generates a SEPRAN mesh
c      The mesh is written to the meshoutput file as indicated in the
c      sepran.env file
c **********************************************************************
c
c                       KEYWORDS
c
c    create
c    mesh
c    plot
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c    SEPMSH  Performs the actual mesh generation
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call sepmsh(0)
      end
      subroutine msh096 ( kmesh, kmeshh, kmeshq, mapar, coor, ineigf,
     +                    isurfc, iref, kmeshl, kmeshm )
c ======================================================================
c
c        programmer     Guus Segal
c        version 3.0    date   01-04-1996 Extra parameters kmeshl and kmeshm
c        version 2.1    date   12-09-1995 Allow unformatted files
c        version 2.0    date   21-09-1994 Extra parameter kmeshq
c        version 1.1    date   16-08-1994 Adaptation format
c        version 1.0    date   19-07-1994
c
c
c
c   copyright (c) 1994-1996  "Ingenieursbureau SEPRA"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "Ingenieursbureau SEPRA".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                         DESCRIPTION
c
c   Write co-ordinates for 2D finite volume grid to file finvol.new
c   Write extra integer information for SEPRAN and pre-processor to finvol.new
c   Special subroutine meant for ISNaS
c **********************************************************************
c
c                      KEYWORDS
c
c    finite_volumes
c    write_mesh
c **********************************************************************
c
c                      INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer kmesh(*), kmeshh(6,*), mapar(*), ineigf(10,*),
     +        isurfc(4,*), iref, kmeshq(*), kmeshl(*), kmeshm(*)
      double precision coor(2,*)

c     coor    i    array of length ndim x npoint containing the co-ordinates
c                  of the nodal points with respect to the global numbering
c     ineigf  i    In this array of length 6 x ncurvs for each curve the
c                  following information is stored:
c                  ineigf(1,i) first surface number curve i is lying in
c                  ineigf(2,i) corresponding edge sequence number
c                  ineigf(3,i) corresponding subedge sequence number
c                  ineigf(4-5,i) Parameters i1 and i2 with respect to face
c                  ineigh(6-10,i) contain the same information for the second
c                              surface
c                  A zero means no surface found
c                  A negative sign denotes the reversed direction
c     iref    i    File reference number
c     isurfc  i    In this array of length 4 x nsurfs for each surface the
c                  four curves surrounding it are stored
c     kmesh   i    Standard SEPRAN array containing information of the mesh
c     kmeshh  i    kmesh part h
c     kmeshl  i    kmesh part l
c     kmeshm  i    kmesh part m
c     kmeshq  i    kmesh part q
c     mapar        mapping array: contains the mapping from local to
c                  global numbering of the nodal points
c **********************************************************************
c
c                      COMMON BLOCKS
c
      integer nbuffr, kbuffr, intlen, ibfree
      common /cbuffr/ nbuffr, kbuffr, intlen, ibfree
      save /cbuffr/

c                 /cbuffr/
c    Several variables related to the common buffer ibuffr
c
c    nbuffr     Declared length of array ibuffr
c    kbuffr     Last position used in ibuffr
c    intlen     Length of a real variable divided by the length of
c               an integer variable
c    ibfree     Next free position in array ibuffr
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      integer  machin, lenwor, numchr, nrec4, iref10, iref73, iref74,
     +         irefpl, iplotf, icarcn, iref17, isite, idumma(17)
      common /cmachn/  machin, lenwor, nrec4, numchr, iref10, iref73,
     +                 iref74, irefpl, iplotf, icarcn, iref17, isite,
     +                 idumma
      save /cmachn/
c
c                 /cmachn/
c     Common block for determination of computer system
c     as well as some machine-dependent quantities and references
c
c     icarcn    Indicates if the computer uses the carriage control parameter
c               for output to a file (1) or not (0)
c     idumma    Dummy array, not yet used
c     iref10    Dataset reference number with respect to file meshoutput
c               If negative the file is unformatted
c     iref17    Dataset reference number with respect to file menumsg
c     iref73    Dataset reference number with respect to file sepcomp.inf
c     iref74    Dataset reference number with respect to file sepcomp.out
c               If negative the file is unformatted
c     irefpl    Dataset reference number with respect to SEPRAN plot file
c               If negative the file is unformatted
c     iplotf    Indicates if a copy of the plot must be made to a file (1)
c               or not (0)
c     isite     Sequence number of site to call special site adapted subroutines
c               Possible values:
c               0:  general
c               1:  Eindhoven University of Technology
c               2:  Eindhoven University of Technology (VIP)
c               3:  TNO/KRI (VIP)
c     lenwor    This parameter is used in the open statement of direct access
c               files. The record length in the statement rec =
c               is given by lenwor times the number of words
c               So if the machine requires that the length is given in words
c               then lenwor should be one. If the record length must be
c               given in bytes, lenwor gives the number of bytes per word
c     machin    Type of computer system. Possible values:
c                1    IBM
c                2    Cyber   (NOS/VE)
c                3    Apollo  (Unis/Aegis)
c                4    HP 9000 (HP/UX)
c                5    .....    Unix
c                6    VAX      VMS
c                7    Harris
c                8    Cray
c                9    IBM PC 386   MSDOS + FTN77/386
c               10    CONVEX
c               13    Alliant
c     nrec4     Number of words that can be stored in one record of the
c               error message file
c     numchr    Number of characters that can be stored in a word
c  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c **********************************************************************
c
c                    LOCAL PARAMETERS
c
      integer i, idim, isurf, j, ncurvs, npoint, nsurfs, ih, lenkmsm
      integer ni, nj, lxarr, lyarr
      double precision lx, ly, xarr(2), yarr(2), xeps, yeps

c     i          Counting variable
c     idim       Loop parameter over dimension of the space
c     ih         General integer help variable
c     isurf      Surface sequence number
c     j          Counting variable
c     lenkmsm    Length of array kmeshm
c     ncurvs     Number of curves in mesh
c     npoint     Number of nodal points in local grid
c     nsurfs     Number of surfaces in mesh
c **********************************************************************
c
c             SUBROUTINES CALLED
c
      integer inilen

c     INILEN    Compute length of array in IBUFFR
c     READFL    Read array from a direct access SEPRAN file
c **********************************************************************
c
c                    I/O
c
c     The real part is written to the file finvol.new
c     Also some extra information is written
c     The following information is written:
c
c     For isurf := 1 (1) nsurfs
c        Read mapar
c        For idim := 1, 2
c           For i = 1, npoint_local
c              write coor(idim,mapar(i))
c     Write array isurfc
c     Write array ineigf
c     Write length of array kmeshq
c     Write array kmeshq
c **********************************************************************
c
c                    ERROR MESSAGES
c
c **********************************************************************
c
c                    PSEUDO CODE
c
c     For isurf := 1 (1) nsurfs
c        Read mapar
c        For idim := 1, 2
c           write(coor(idim,mapar(i)), i:=1 (1) npoint_local
c     Write array isurfc
c     Write array ineigf
c     Write length of array kmeshq
c     Write array kmeshq
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data xarr /0, 20/, yarr /0, 20/
c
c ======================================================================
c
      ncurvs = kmesh(11)
      nsurfs = kmesh(12)
      if ( iref10.gt.0 ) write(iref,100)
100   format(/'# Part 4 co-ordinates')
      do 200 isurf = 1, nsurfs
         if ( iref10.gt.0 ) write(iref,110) isurf
110      format(/'# Block ',i4)
         npoint = kmeshh(2,1+isurf)/(2*intlen)

c        --- Read mapar

         call readfl ( kmeshh(5,1+isurf), 1, npoint, 1, 3, mapar )

         ni    = 20
         nj    = 20
         lx    = 4d0
         ly    = 1d0
         lxarr = 2
         lyarr = 2
c        Arrays xarr and yarr have been filled in data statement 
         xeps   = 0.1
         yeps   = 0.03
         call disturb( coor, ni, nj, lx, ly, xarr, yarr, lxarr, lyarr,
     +                 xeps, yeps )

         if ( iref10.gt.0 ) then
            do 130 idim = 1, 2
               write(iref,120) idim, (coor(idim,mapar(i)),i=1,npoint)
120            format(/'# x',i1,' co-ordinates'/6(ss,1pe13.5e2))
130         continue
         else
            write(iref) ((coor(idim,mapar(i)),i=1,npoint),idim=1,2)
         end if
200   continue

      if ( iref10.gt.0 ) then
         write(iref,210) nsurfs,((isurfc(j,i),j=1,4),i,i=1,nsurfs)
210      format(/'# Array surfaces'/i4,26x,'# nblocks'/
     +          (4i8,5x,'#  block no',i5))
         write(iref,220) ncurvs,((ineigf(j,i),j=1,10),i,i=1,ncurvs)
220      format(/'# Array ineigh_face'/i4,26x,'# ncurvs'/
     +           ((10i6),5x,'# curve no',i6))
      else
         write(iref) nsurfs
         write(iref) ((isurfc(j,i),j=1,4),i=1,nsurfs)
         write(iref) ncurvs
         write(iref) ((ineigf(j,i),j=1,10),i=1,ncurvs)
      end if
      if ( kmesh(31).gt.0 ) then

c     --- kmeshq exists, hence there are subcurves

         if ( iref10.gt.0 ) then
            write(iref,230) kmeshq(ncurvs+1)
230         format(i6,10x,'# Length of array kmeshq')
            write(iref,240) (kmeshq(i),i=1,kmeshq(ncurvs+1))
240         format(/'# Array kmeshq'/(10i6))
         else
            write(iref) kmeshq(ncurvs+1)
            write(iref) (kmeshq(i),i=1,kmeshq(ncurvs+1))
         end if

      else

c     --- kmeshq does not exist

         if ( iref10.gt.0 ) then
            write(iref,230) 0
         else
            ih = 0
            write(iref) ih
         end if

      end if

c     --- Write special information for isnaspost
c         First kmeshl

      if ( iref10.gt.0 ) then
         write(iref,250) kmesh(10)
250      format(i6,10x,'# Length of array kmeshl')
         write(iref,260) (kmeshl(i),i=1,kmesh(10))
260      format(/'# Array kmeshl'/(10i6))
      else
         write(iref) kmesh(10)
         write(iref) (kmeshl(i),i=1,kmesh(10))
      end if

c     --- Next kmeshm

      lenkmsm = inilen(kmesh(27))
      if ( iref10.gt.0 ) then
         write(iref,270) kmesh(11), lenkmsm
270      format(2i6,4x,'# ncurvs, Length of array kmeshm')
         write(iref,280) (kmeshm(i),i=1, lenkmsm)
280      format(/'# Array kmeshm'/(10i6))
      else
         write(iref) kmesh(11), lenkmsm
         write(iref) (kmeshm(i),i=1,lenkmsm)
      end if

c     --- Finally the arrays mapar

      do 300 isurf = 1, nsurfs
         if ( iref10.gt.0 ) write(iref,110) isurf
         npoint = kmeshh(2,1+isurf)/(2*intlen)

c        --- Read mapar

         call readfl ( kmeshh(5,1+isurf), 1, npoint, 1, 3, mapar )

         if ( iref10.gt.0 ) then
            write(iref,290) (mapar(i),i=1,npoint)
290         format(/'# mapar'/(10i6))
         else
            write(iref) (mapar(i),i=1,npoint)
         end if
300   continue

c     --- Write npoint

      if ( iref10.gt.0 ) then
         write(iref,310) kmesh(8)
310      format(/'# npoint'/(10i6))
      else
         write(iref) kmesh(8)
      end if

      end
      subroutine disturb ( coor, ni, nj, lx, ly, xarr, yarr, lxarr,
     +                     lyarr, xeps, yeps )
c ********************************************************************
c
c        programmer :    Marcel Zijlema
c        version    :    1.0    ISNaS/Sepran
c        date       :    22-08-1996
c        computer   :    HP 9000/735
c        reference  :
c
c ********************************************************************
c
c     * TUD subroutine to change a SEPRAN mesh. 
c      
c       The mesh is supposed to be created by SEPMESH and stored in
c       the unformatted file meshoutput
c
c ********************************************************************
c
c           INPUT / OUTPUT PARAMETERS
c
      integer  ni, nj, lxarr, lyarr
      double precision coor(1:2, 0:ni, 0:nj), lx, ly, xarr(1:lxarr),
     +                 yarr(1:lyarr), xeps, yeps
c
c     COOR     o    Double precision three-dimensional array.
c
c                   COOR(k, i, j) k=1,2  ; i=0(1)ni ; j=0(1)nj 
c
c                   COOR(1, i, j) x-coordinate of vertex (i,j),
c                                 being the left-under
c                                 point of cell(i,j).
c                   COOR(2, i, j) y-coordinate of vertex(i,j).
c
c***********************************************************************
c
c                   COMMON BLOCKS
c
c ********************************************************************
c
c                   LOCAL PARAMETERS
c
      integer i, j, k
      double precision xp, yp
c
c     i, j, k     General loop variables
c     xp, yp      Computed coordinates
c
c ********************************************************************
c
c                         I/O
c
c
c ********************************************************************
c
c                      SUBROUTINES CALLED
c
c
c ********************************************************************
c
c                     ERROR MESSAGES
c
c ********************************************************************

      do 10 j = 0, nj
      do 10 i = 0, ni

        xp = i*lx/ni
        yp = j*ly/nj

        if ( mod(j+1,2).eq.0 ) then

          do k=1, lxarr-1
             if ( i.eq.0 .or. i.eq.ni ) then
               xp = xp
             else if ( i.ge.xarr(k) .and. i.lt.xarr(k+1) ) then
               xp = xp+mod(k,2)*xeps-mod(k+1,2)*xeps
             end if
          end do

        end if

        if ( mod(i+1,2).eq.0 ) then

          do k=1, lyarr-1
             if ( j.eq.0 .or. j.eq.nj ) then
               yp = yp
             else if ( j.ge.yarr(k) .and. j.lt.yarr(k+1) ) then
               yp = yp+mod(k,2)*yeps-mod(k+1,2)*yeps
             end if
          end do

        end if

        coor(1,i,j) = xp
        coor(2,i,j) = yp

10    continue

      end
