      subroutine isalloc ( nblocks, neq, isiinput, isrinput,
     +                     isaverstr,  isinfbnd, isbndcon,
     +                     ismatrix, isgmat, isrhside,
     +                     iadmin, isigeom, isdensup )
c ======================================================================
c
c     programmer   Kees Kassels
c     version      3.6   date    08-02-1999 Kees K: isgeom and iseddy
c                                removed here, will be created now in
c                                subroutine isrdint
c     version      3.5   date    04-09-1998 Kees K: isactive isrinput
c     version      3.4   date    22-07-1998 Duncan: iterative press. corr
c                                                    save coef dens upwind
c     version      3.3   date    04-06-1998 Charles: mdisc removed
c     version      3.2   date    29-05-1998 Charles: isdist2, isdist3 removed
c     version      3.1   date    03-03-1998 Charles: minor correction
c     version      3.0   date    16-02-1998 Charles: isalloc2 in isalloc
c     version      2.0   date    22-08-1997 Fred T: new array igeom
c                                to store integer info. on closest wall
c                                and geometry array extended to store
c                                the averaged shear stress at the wall
c     version      1.9   date    18-08-1997 Fred T: array length geom
c                                increased to incorporate the wall dist.
c     version      1.8   date    27-03-1996 Kees K bndcon per equation
c     version      1.7   date    27-06-1995 Hester: arraylength eddy increased
c                                to incorporate density in compressible flow.
c     version      1.6   date    26-06-1995 Guus: clear array geom
c     version      1.5   date    21-04-1995 RvN: extension infbnd and bndcon
c     version      1.4   date    31-01-1995 Erik: error removed
c     version      1.3   date    16-01-1995 Erik: added allocation of
c                                preconditioner if saveprec = 1.
c                                New dimensions for ismatrix.
c     version      1.2   date    31-10-1994 Kees K array geom added
c     version      1.1   date    17-08-1994 RvN
c                                linfbnd and lbndcon adjusted
c     version      1.0   date    30-04-1994
c
c   copyright (c) 1994-1998  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Create space for arrays eddy, infbnd, bndcon, matrix,
c     gmat, and rhside.
c **********************************************************************
c
c                       KEYWORDS
c
c     parallel
c     receive
c     start_information
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer nblocks, neq
      integer iadmin(1:nblocks),     isiinput(1:nblocks),
     +        isrinput(1:nblocks),
     +        isinfbnd(1:nblocks),
     +        isbndcon(1:nblocks,1:neq),ismatrix(1:nblocks,1:neq,*),
     +        isgmat(1:nblocks),     isrhside(1:nblocks),
     +        isigeom(1:nblocks),
     +        isaverstr(1:nblocks), isdensup(1:nblocks)

c     iadmin    i    reserved
c     isaverstr i/o  Array of sequence numbers (nblocks) of array averstr
c     isbndcon  i    Array of sequence numbers (nblocks) of array bndcon
c     isdensup  i/o  Array of sequence numbers (nblocks) of array densup
c     isgmat    i    Array of sequence numbers (nblocks) of array gmat
c     isigeom   i    Array of sequence numbers (nblocks) of array igeom
c     isiinput  i    Array of sequence numbers (nblocks) of array iinput
c     isinfbnd  i    Array of sequence numbers (nblocks) of array infbnd
c     ismatrix  i    Array of sequence numbers (nblocks) of array matrix
c     isrhside  i    Array of sequence numbers (nblocks) of array rhside
c     isrinput  i    Array of sequence numbers (nblocks) of array rinput
c     nblocks   i    Number of blocks
c     neq       i    Number of equations
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisnas'
      include 'IScommon/cisparal'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer iblock, idummy, iequa, ipiinblk, ipiinput, lbndcon,
     +      lgmat, linfbnd, linfmg, linfmm, linfmt,
     +      lmatrix, lprec, lrhside, lsolutnv, lwork, maxb, maxcon,
     +      maxdim, maxdx, maxpoints, maxvpnts, ndegfd, ndim,
     +      ni, nj, nk, ntimlv, nvirtual, nxmax, saveprec,
     +      ligeom, ipiincom, mturb, ipiintur, viscorr,
     +      iprinput, ipigeom, iprinbc, ipiinbc, ipiindsc,
     +      idiscret, ntrnsp, nturb, nwindowmx, lenaver, ldensup

c     iblock    Block sequence number
c     idiscret  Type of discretization
c     idummy    Integer dummy
c     iequa     General equation sequence number
c     ipigeom   Starting address of array igeom in ibuffr
c     ipiinbc   Starting address of array iinbc in array iinput
c     ipiinblk  Starting address of array IINBLK in array IINPUT
c     ipiincom  Starting address of array IINCOM in array IINPUT
c     ipiindsc  Starting address of array IINDSC in array IINPUT
c     ipiinput  Starting address of array iinput in ibuffr
c     ipiintur  Starting address of array IINTUR in array IINPUT
c     iprinbc   Starting address of array rinbc in array iinput
c     iprinput  Starting address of array rinput in buffer
c     lbndcon   Length of array bndcon
c     ldensup   Length of array densup
c     lenaver   Help parameter for the length of array averstr
c     lgmat     Length of array gmat
c     ligeom    Length of array igeom
c     linfbnd   Length of array infbnd
c     linfmg    Length of array infmg
c     linfmm    Length of array infmm
c     linfmt    Length of array infmt
c     lmatrix   Length of array matrix
c     lprec     Length of array prec
c     lrhside   Length of array rhside
c     lsolutnv  Length of temporary solution array
c     lwork     Length of array work
c     maxb      Maximum number of real values for one boundary condition
c     maxcon    Maximum number of non zero diagonals in a matrix
c     maxdim    Highest physical space dimension for which ISNaS can work
c     maxdx     Maximum relative shift between cell indices
c     maxpoints Maximum number of points in a block
c     maxvpnts  Maximum number of points per block including virtual ones
c     mturb     Indicates type of turbulence modelling
c     ndegfd    Number of degrees of freedom
c     ndim      Dimension of space (2 or 3)
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     ntimlv    Number of time levels
c     ntrnsp    Number of transport equations
c     nturb     Number of extra transport equations for turbulence
c     nvirtual  Number of rows of virtual cells
c     nwindowmx Maximum size of the window to be used for time-averaging
c     nxmax     Maximum of ni, nj, nk
c     saveprec  Save preconditioner across domain decomposition
c               iterations (=1) or not (=0)
c     viscorr   Indicate whether to use viscous corrections or not
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isingetp, isdpgetp

c     ISACTIVE  Activate old permanent array
c     ISDCREAT  Create permanent space in array IBUFFR (Double precision array)
c     ISDPGETP  get the pointer corresponding to a double precision array
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISICREAT  Create permanent space in array IBUFFR (Integer array)
c     ISINGETP  Get the pointer corresponding to an integer array
c     ISLENAV3  Calculate lenaver
c     ISLSDS02  Compute lengths of temporary arrays
c     ISSETICO  Make integer array equal to a constant
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c      for iblock = 1 (1) nblocks do
c
c         if block(iblock) on this node  then
c
c            for arrays  eddy, infbnd, bndcon, matrix,
c                       gmat, and  rhside  do
c
c               get length of array
c
c               create space for array
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call iseropen('isalloc')

c     --- determine SAVEPREC

      call isactive(isiinput(1))
      ipiinput  = isingetp(isiinput(1))
      ipiinblk  = ibuffr(ipiinput + 7 - 1) + ipiinput - 1
      ipiintur  = ibuffr(ipiinput + 8 - 1) + ipiinput - 1
      ipiindsc  = ibuffr(ipiinput + 6 - 1)
      ipiincom  = ibuffr(ipiinput + 9 - 1) + ipiinput - 1
      saveprec  = ibuffr(ipiinblk + 3 - 1)
      mturb     = ibuffr(ipiintur + 2 - 1)
      viscorr   = ibuffr(ipiintur - 1 + 3)
      idiscret  = ibuffr(ipiinput-1+ipiindsc+1)

      do iblock = 1, nblocks

         iblockrf = iblock

         if( iadmin(iblock).eq.inodenr) then

            call isactive(isiinput(iblock))
            ipiinput = isingetp( isiinput(iblock) )

            ni         = ibuffr( ipiinput + 51 -1 )
            nj         = ibuffr( ipiinput + 52 -1 )
            nk         = ibuffr( ipiinput + 53 -1 )
            ndim       = ibuffr( ipiinput + 57 -1 )
            ndegfd     = ibuffr( ipiinput + 55 -1 )
            nvirtual   = ibuffr( ipiinput + 54 -1 )
            maxb       = ibuffr( ipiinput + 61 -1 )
            maxcon     = ibuffr( ipiinput + 60 -1 )
            maxvpnts   = ibuffr( ipiinput + 69 -1 )
            maxpoints  = ibuffr( ipiinput + 71 -1 )

            nxmax   = max(ni, nj, nk)

c           --- temporary length of array infbnd

            linfbnd = 2*ndim*(1+2*(ndim-1)) + 2 *
     +          ((2*nxmax+2+4*nvirtual)**(ndim-1)) * (2*ndim)

c           --- temporary

            lbndcon = maxb *
     +          ((2*nxmax+2+4*nvirtual)**(ndim-1)) * (2*ndim)
            lmatrix = maxpoints * ndim * maxcon
            lgmat   = maxpoints * ndim * maxcon
            lrhside = maxpoints * ndegfd
            ligeom  = 0
            if ( mturb.gt.0 ) then
               ligeom  = ligeom + maxvpnts
            end if

            if ( idiscret.eq.4 ) then
               ligeom  = ligeom + maxvpnts * 2**ndim
            end if

            ldensup =(ni+1)*(nj+1)*6
            call isdcreat(isdensup(iblock),ldensup,'densup')

            call isicreat( isinfbnd(iblock), linfbnd, 'infbnd' )

            if ( ligeom.gt.0 )
     +         call isicreat( isigeom(iblock), ligeom, 'igeom' )

            do iequa = 1, neq
               call isdcreat(isbndcon(iblock,iequa),lbndcon,'bndcon')
            end do

            call isdcreat( ismatrix(iblock,1,1), lmatrix, 'matrix' )
            call isdcreat( isgmat(iblock),       lgmat  , 'gmat' )
            call isdcreat( isrhside(iblock),     lrhside, 'rhside' )

            if ( ligeom.gt.0 ) then
               ipigeom = isingetp(isigeom(iblock))
               call issetico ( ibuffr(ipigeom), ligeom, 0 )
            end if

            if ( saveprec.eq.0 ) then

                  ismatrix(iblock,1,2) = 0

            else

               lprec = 0
               do iequa = 1, neq

                  call islsds02( iequa, ibuffr(ipiinput),
     +                           linfmm, linfmg, linfmt,
     +                           idummy, lsolutnv,  lwork,
     +                           ni, nj, nk, ndim ,
     +                           nvirtual, ndegfd , ntimlv,
     +                           maxdx, maxdim )

                 lprec = max( lprec, idummy)

               end do

               call isdcreat( ismatrix(iblock,1,2), lprec, 'prec' )

            end if

c           --- all matrices  and preconditioners for a subdomain share
c               the same storage for all equations
c               This may change in the future if for instance the
c               pressure matrix is saved between consecutive
c               time steps.

            do iequa = 2, neq
               ismatrix(iblock,iequa,1) = ismatrix(iblock,1,1)
               ismatrix(iblock,iequa,2) = ismatrix(iblock,1,2)
            end do

            if (((mturb.eq.1).or.(mturb.eq.4)).and.viscorr.ne.0) then

               call isactive(isrinput(iblock))
               iprinput = isdpgetp( isrinput(iblock) )
               iprinbc    = ibuffr( ipiinput + 27 -1 )
               ipiinbc    = ibuffr( ipiinput + 2  -1 )
               ntrnsp     = ibuffr( ipiinput + 63 -1 )
               nturb      = ibuffr( ipiinput + 70 -1 )

               call islenav3 ( ibuffr( ipiinput + ipiinbc-1),
     +                         ni, nj, nk, ntrnsp, nturb,
     +                         buffer(iprinput+iprinbc-1),
     +                         ibuffr(ipiintur-1+9) )

               nwindowmx = ibuffr(ipiintur - 1 + 6)
               lenaver   = ibuffr(ipiintur - 1 + 9)

               if ( lenaver.gt.0 )
     +              call isdcreat ( isaverstr(iblock),
     +                            (lenaver+1)*nwindowmx, 'averstr' )

            end if

         end if

      end do

      call iserclos('isalloc')

      end
