      subroutine ischecktop1 ( iblock, nblocks, nfaces, maxnsubf,
     +                         nsubfaces, subfaces, iinput )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.1    date 04-01-2000 Minor correction error message
c        version 1.0    date 25-11-1999
c
c
c   copyright (c) 1999-2000  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Check the block topology from the grid input file.
c
c **********************************************************************
c
c                       KEYWORDS
c
c   check
c   topology
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iblock, maxnsubf, nblocks, nfaces, iinput(*)
      integer nsubfaces(1:nblocks, 1:nfaces)
      integer subfaces(1:nblocks, 1:nfaces, 1:maxnsubf, *)

c     iblock         i    Block sequence number
c     iinput         i    Contains the integer user input for one block.
c                         For a description see the programmers guide
c     maxnsubf       i    Maximum number of subfaces encountered on all subfaces
c     nblocks        i    Number of blocks
c     nfaces         i    Number of faces
c     nsubfaces      i    Array containing the number of subfaces of each face,
c                         see array itopol for an extended description
c     subfaces       i    This array contains a description of all subfaces
c                         subfaces(iblock,iface,isubface,.) refers to block
c                         iblock, face iface and subface isubface
c                         Contents of the last index
c                         Pos. 1  i1
c                         Pos. 2  i2
c                         Pos. 3  j1
c                         Pos. 4  j2
c                         Pos. 5  block number neighbour block
c                         Pos. 6  face number neighbour face
c                         Pos. 7  subface number neighbour subface
c                         Pos. 8  iorient
c                         Pos. 9  Reserved for preprocessor only
c                         Pos. 10 Reserved for preprocessor only
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer iface, isubface, nsubf, i1, i2,
     +        j1, j2, ni, nj, jf1, if1

c     i1             Lower index number along first direction of a face or
c                    edge
c     i2             Upper index number along first direction of a face
c                    or edge
c     if1            Last index used along first direction
c     iface          Face number
c     isubface       Subface number
c     j1             Lower index number along second direction of a face
c     j2             Upper index number along second direction of a face
c     jf1            Last index used along second direction
c     ni             Number of non-virtual cells in the i-direction
c                    (one block)
c     nj             Number of non-virtual cells in the j-direction
c                    (one block)
c     nsubf          Number of subfaces
c **********************************************************************
c
c                       I/O
c
c      none
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS       Resets old name of previous subroutine of higher level
c     ISEROPEN       Produces concatenated name of local subroutine
c     ISERRINT       Put integer value in error message
c     ISERRSUB       Error messages
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     721   i1 incorrect
c     722   i2 <= i1
c     723   j1 incorrect
c     724   j2 <= j1
c     725   if1 # nj
c     726   if1 # ni
c **********************************************************************
c
c                       PSEUDO CODE
c
c     For iblock := 1 (1) nblocks do
c       For iface := 1 (1) nfaces do
c          nsubf := nsubfaces(iblock,iface)
c          For isubfaces := 1 (1) nsubf do
c             Check position 5 6 and 7 of subfaces
c         End
c       End
c     End
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen ( 'ischecktop1' )

c     --- Loop over faces, subfaces

      ni = iinput(51)
      nj = iinput(52)
      do iface = 1, nfaces
         if1 = 0
         jf1 = 0
         nsubf = nsubfaces(iblock,iface)
         do isubface = 1, nsubf
            i1 = subfaces(iblock,iface,isubface,1)
            i2 = subfaces(iblock,iface,isubface,2)
            j1 = subfaces(iblock,iface,isubface,3)
            j2 = subfaces(iblock,iface,isubface,4)
            if ( i1.ne.if1+1 ) then

c           --- if1 # if1+1

               call iserrint ( i1, 1 )
               call iserrint ( if1+1, 2 )
               call iserrint ( iblock, 3 )
               call iserrint ( iface, 4 )
               call iserrint ( isubface, 5 )
               call iserrsub ( 721, 5, 0, 0 )

            end if
            if ( i2.lt.i1 ) then

c           --- i2 <= i1

               call iserrint ( i2, 1 )
               call iserrint ( i1, 2 )
               call iserrint ( iblock, 3 )
               call iserrint ( iface, 4 )
               call iserrint ( isubface, 5 )
               call iserrsub ( 722, 5, 0, 0 )

            end if
            if1 = i2
            if ( j1.gt.0 .or. jf1.gt.0 ) then
               if ( j1.ne.jf1+1 ) then

c              --- jf1 # jf1+1

                  call iserrint ( j1, 1 )
                  call iserrint ( jf1+1, 2 )
                  call iserrint ( iblock, 3 )
                  call iserrint ( iface, 4 )
                  call iserrint ( isubface, 5 )
                  call iserrsub ( 723, 5, 0, 0 )

               end if
               if ( j2.lt.j1 ) then

c              --- j2 <= j1

                  call iserrint ( j2, 1 )
                  call iserrint ( j1, 2 )
                  call iserrint ( iblock, 3 )
                  call iserrint ( iface, 4 )
                  call iserrint ( isubface, 5 )
                  call iserrsub ( 724, 5, 0, 0 )

               end if
               jf1 = j2
            end if

         end do
         if ( j1.eq.0 ) then
            if ( iface.le.2 ) then
               if ( if1.ne.nj ) then

c              --- if1 # ni

                  call iserrint ( iface, 1 )
                  call iserrint ( if1, 2 )
                  call iserrint ( nj, 3 )
                  call iserrint ( iblock, 4 )
                  call iserrsub ( 725, 4, 0, 0 )

               end if
            else
               if ( if1.ne.ni ) then

c              --- if1 # nj

                  call iserrint ( iface, 1 )
                  call iserrint ( if1, 2 )
                  call iserrint ( ni, 3 )
                  call iserrint ( iblock, 4 )
                  call iserrsub ( 726, 4, 0, 0 )

               end if
            end if
         end if
      end do

      call iserclos ( 'ischecktop1' )

      end
