      subroutine iscoeftu( ichois, mturb, ncoefs, npoint, ndegfd, ndim,
     +                     ntimlv, ni, nj, nk, nvirtual, ntrnsp, nturb,
     +                     lwork, iincof, rincof, iindsc, coefs, rintur,
     +                     geom, solut, eddy, coor, work, iinbc ,
     +                     rinbc, time, leniindsc, viscorr, maniso )
c ======================================================================
c
c        programmer     Marcel Zijlema
c        version 3.9    date   04-08-1997 Guus: remove stop
c        version 3.8    date   14-07-1997 Guus: replace iindsc(7) by (2)
c        version 3.7    date   11-06-1997 Guus: change call to coefficient routs
c        version 3.6    date   21-08-1996 Marcel: parameter list isprod2
c                                                 and isprod3 changed
c        version 3.5    date   11-07-1996 Marcel: parameter list changed
c        version 3.4    date   23-05-1996 Marcel: parameter list isprod2
c                                                 and isprod3 changed
c        version 3.3    date   15-12-1995 Marcel: introduce iscofkw3
c        version 3.2    date   08-12-1995 Marcel: call isprod now
c                                               depends on dimension
c        version 3.1    date   31-08-1995 Guus: introduce iscofkw2
c        version 3.0    date   28-02-1995 Guus: extra parameter leniindsc
c        version 2.2    date   22-02-1995 Guus: extended length of iindsc
c        version 2.1    date   25-01-1995 Marcel: parameter list iscofkwf
c                                               changed
c        version 2.0    date   19-01-1995 Marcel: fill coefs adjacent to
c                                               boundary using wall functions
c        version 1.3    date   10-01-1995 Marcel: extension parameter list
c                                               with iindsc, nturb, ntrnsp
c        version 1.2    date   30-11-1994 Marcel: new calls: iscoftu2 and
c                                               iscoftu3
c        version 1.1    date   29-11-1994 Marcel: usfil2 and usfil3 changed
c                                               into isflc2 and isflc3
c        version 1.0    date   22-09-1994
c
c
c
c   copyright (c) 1994-1997  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Fills time, place and solution dependent coefficients of
c     several turbulence equations
c **********************************************************************
c
c                       KEYWORDS
c
c     turbulence
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ichois, mturb, ncoefs, npoint, ndegfd, ntrnsp, nturb,
     +        ndim, ntimlv, ni, nj, nk, nvirtual, lwork, iinbc(*),
     +        iincof(1:ncoefs), leniindsc,
     +        iindsc(leniindsc,ndegfd-ndim), viscorr, maniso
      double precision rincof(1:ncoefs), coefs(1:npoint,1:ncoefs),
     +                 solut(*), eddy(*), rintur(*), geom(*),
     +                 work(lwork), coor(*), rinbc(*),
     +                 time

c     coefs     o    reserved
c     coor      i    reserved
c     eddy      i    reserved
c     geom      i    reserved
c     ichois    i    General choice parameter
c     iinbc     i    reserved
c     iincof    i    reserved
c     iindsc    i    reserved
c     leniindsc i    Length of first component of array iindsc
c     lwork     i    Length of array work
c     maniso    i    Indicates type of nonlinear turbulence (anisotropic)
c                    modelling
c     mturb     i    Indicates type of turbulence modelling
c     ncoefs    i    Number of coefficients
c     ndegfd    i    Number of degrees of freedom
c     ndim      i    Dimension of space (2 or 3)
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     npoint    i    Number of points for one block
c     ntimlv    i    Number of time levels
c     ntrnsp    i    Number of transport equations
c     nturb     i    Number of extra transport equations for turbulence
c     nvirtual  i    Number of rows of virtual cells
c     rinbc     i    reserved
c     rincof    i    reserved
c     rintur    i    reserved
c     solut     i    reserved
c     time      i    Actual time of time step
c     viscorr   i    Indicate whether to use viscous corrections or not
c     work      i    Work array
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, icoef, maxcoefs, choice, ipinvjac

c     choice    General choice parameter
c     i         General loop variable
c     icoef     Sequence number of coefficient
c     ipinvjac  Starting address of array invjac in geom
c     maxcoefs  Maximum number of coefficients allowed
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISCOFKW2  fill coefs for k-equation using wall functions in 2D case
c     ISCOFKW3  fill coefs for k-equation using wall functions in 3D case
c     ISCOFTU2  fill coefs for turbulence equations in 2D case
c     ISCOFTU3  fill coefs for turbulence equations in 3D case
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRCHR  Puts values of strings in character error array
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c     ISFLC2    fill one coefficient in coefs with a function of space,
c               time and posssible preceding solutions, in 2D case
c     ISPROD2   Computes production of turbulent kinetic energy in 2D case
c     ISPROD3   Computes production of turbulent kinetic energy in 3D case
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c       6   iincof(i) out of range
c      25   Value of CHOICE has not yet been implemented
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen('iscoeftu')

c     --- First, initialize the coefficients

      do icoef = 1, ncoefs
         do i = 1, npoint
            coefs(i,icoef) = 0d0
         end do
      end do

c     --- startaddress for inverse of Jacobian in array geom

      if ( ndim.eq.2) then
         ipinvjac = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)*4 + 1
      else if ( ndim.eq.3) then
         ipinvjac = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)*
     +              (nk+1+2*nvirtual)*8 + 1
      end if

      maxcoefs = 2

c     --- Fill first two coefficients with density and molecular
c         viscosity, respectively

      do icoef = 1, maxcoefs

         choice = iincof(icoef)

         if ( choice.lt.-1 .and. choice.ge.-100 .or.
     +        choice.ge.200 ) then

c        --- IINCOF(icoef) wrong value

            call iserrint( icoef, 1 )
            call iserrint( choice, 2 )
            call iserrchr( 'iscoeftu',1 )
            call iserrsub ( 6, 2, 0, 1 )

            go to 1000

         else if (choice.eq.-1 ) then

c        --- Fill COEFS with constant

            do i = 1, npoint
              coefs(i,icoef) = rincof(icoef)
            end do

         else if ( choice.gt.0 .and. choice.lt.200 ) then

c        --- Fill COEFS with a function of space, time and possible
c            preceding solutions

            if ( ndim.eq.2) then
               call isflc2 ( choice, coefs, ncoefs, icoef, ni, nj, ndim,
     +                       nvirtual, coor, solut , ndegfd, time )
            else

               call iserrint ( choice, 1 )
               call iserrsub ( 25, 1, 0, 0 )

            end if

         end if

      end do

c     --- Fill the 4th, 6th and 9th position of COEFS w.r.t. turbulence
c         equations with molecular viscosity

      do i = 1, npoint
        coefs(i,4)      = coefs(i,2)
        coefs(i,6)      = coefs(i,2)
        coefs(i,3*ndim) = coefs(i,2)
      end do

c     --- Compute the production of turbulent kinetic energy

      if ( ndim.eq.2 ) then

        call isprod2(work, solut, coor, coefs(1,1), eddy,
     +               geom(ipinvjac), lwork, ndegfd, ni, nj,
     +               npoint, iindsc(2,ntrnsp+nturb+ichois+2),
     +               nvirtual, rintur, maniso)

      else if ( ndim.eq.3 ) then

        call isprod3(work, solut, coor, coefs(1,1), eddy,
     +               geom(ipinvjac), lwork, ndegfd, ni, nj, nk,
     +               npoint, iindsc(2,ntrnsp+nturb+ichois+2),
     +               nvirtual, rintur, maniso)

      end if

c     --- Fill the remaining COEFS, depending on turbulence model

      if ( ndim.eq.2 ) then

        call iscoftu2 (ichois, mturb, ncoefs, ndegfd,
     +                 ntimlv, ni, nj, nvirtual,
     +                 coefs, rintur, solut, eddy, work, viscorr )

      else if ( ndim.eq.3 ) then

        call iscoftu3 (ichois, mturb, ncoefs, ndegfd,
     +                 ntimlv, ni, nj, nk, nvirtual,
     +                 coefs, rintur, solut, eddy, work)

      end if

      if ( ichois.eq.-2 ) then

c     --- Afterwards, fill the coefficients D and f^* in wall-adjacent
c         cells corresponding to the turbulent kinetic energy equation
c         with source terms determined by wall functions

         if ( mturb.eq.1 .or. mturb.eq.2 .or. mturb.eq.5 ) then

            if ( ndim.eq.2 ) then
               call iscofkw2 ( coefs, iinbc, rinbc, solut, coor,
     +                         eddy, ni, nj, ntrnsp, nturb,
     +                         ndegfd, nvirtual, ntimlv, ncoefs, rintur,
     +                         geom(ipinvjac) )
            else

               call iscofkw3 ( coefs, iinbc, rinbc, solut, coor,
     +                         eddy, ndim, ni, nj, nk, ntrnsp, nturb,
     +                         ndegfd, nvirtual, ntimlv, ncoefs, rintur)
            end if

         end if

      end if

1000  call iserclos('iscoeftu')

      end
