      subroutine iscofkw3( coefs , iinbc , rinbc , solut   , coor  ,
     +                     eddy  , ndim  , ni    , nj      , nk    ,
     +                     ntrnsp, nturb , ndegfd, nvirtual, ntimlv,
     +                     ncoefs, rintur )
c ======================================================================
c
c        programmer     Marcel Zijlema
c        version 1.3    date   19-03-1997 Marcel: correction address rinbc
c        version 1.2    date   20-02-1996 Marcel: bug fixed
c        version 1.1    date   23-01-1996 Marcel: bug fixed pointer iinbc,
c                                                 loop over cells corrected
c        version 1.0    date   15-12-1995 extracted from iscofkwf
c
c
c
c   copyright (c) 1995-1996  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Fill some coefficients with respect to k-equation using
c     wall functions (3D version)
c **********************************************************************
c
c                       KEYWORDS
c
c     3d
c     coefficients
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ndim, ni, nj, nk, ntrnsp, nturb, ndegfd, nvirtual,
     +        ntimlv, ncoefs, iinbc(*)
      double precision coefs(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1-nvirtual:nk+1+nvirtual,
     +                       1:ncoefs),
     +                 solut(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1-nvirtual:nk+1+nvirtual,
     +                       1:ndegfd, 1:ntimlv),
     +                 eddy(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual),
     +                 coor(*), rinbc(*), rintur(*)

c     coefs     o    reserved
c     coor      i    reserved
c     eddy      i    reserved
c     iinbc     i    reserved
c     ncoefs    i    Number of coefficients
c     ndegfd    i    Number of degrees of freedom
c     ndim      i    Dimension of space (2 or 3)
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     ntimlv    i    Number of time levels
c     ntrnsp    i    Number of transport equations
c     nturb     i    Number of extra transport equations for turbulence
c     nvirtual  i    Number of rows of virtual cells
c     rinbc     i    reserved
c     rintur    i    reserved
c     solut     i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer nside, ntdir(1:3), ntsig(1:3), carea(1:2,1:3),
     +        varea(1:2,1:3), ndir, t1dir, t2dir,
     +        ipstart, nseg, i, j, i1, i2, j1, j2, ip, jp, kp,
     +        k, nsign, iplrinbc
      double precision ndist, physut, rhow, mu, tke, tauw,
     +                 uplus, facs(1:2)

c     carea     Index bounds on set of grid cells adjacent to
c               this boundary
c     facs      Multiplication factor
c     i         General loop variable
c     i1        Lower index number along first direction of a face or edge
c     i2        Upper index number along first direction of a face or edge
c     ip        Local counting variable i-direction
c     iplrinbc  Start address
c     ipstart   Starting address
c     j         General loop variable
c     j1        Lower index number along second direction of a face
c     j2        Upper index number along second direction of a face
c     jp        Local counting variable j-direction
c     k         General loop variable
c     kp        Local counting variable k-direction
c     mu        The laminar viscosity
c     ndir      Direction of normal vector on this side
c     ndist     Distance normal to a solid wall
c     nseg      Number of segments the boundary is divided in
c     nside     Number of boundary for which we need the
c               information
c     nsign     Inner product of outward normal in computational domain
c               with the base vector perpendicular to the face
c     ntdir     Contains the ksi-alpha indices of the normal and
c               tangential directions
c               ntdir(1) refers to the normal direction
c               ntdir(2) refers to the first tangential direction
c               ntdir(3) refers to the second tangential direction
c     ntsig     Gives the direction of the normal or tangential vectors
c               with respect to the ksi co-ordinate axis
c               The normal is supposed to be the outward normal and
c               the tangential directions are taken into a counter
c               clockwise way
c               ntsig(1) refers to the normal direction
c               ntsig(2) refers to the first tangential direction
c               ntsig(3) refers to the second tangential direction
c     physut    The physical velocity tangent to the wall
c     rhow      Specific weight
c     t1dir     Direction of first tangential vector on this side
c     t2dir     Direction of second tangential vector on this side
c     tauw      The wall shear stress
c     tke       The turbulent kinetic energy
c     uplus     The dimensionless tangential velocity
c     varea     Area of arrays
c **********************************************************************
c
c                       I/O
c
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISBNBVI   Gives information on the relation between the ISNaS
c               basis vectors and the normal and tangential vectors on
c               a given boundary. Moreover it gives lower and upper bou
c     ISLOGW    Compute the wall shear stress with log-law based
c               wall condition depending on smoothness or roughness
c               of the boundary
c     ISWLLQN3  Determine essential wall quantities in R3
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      iplrinbc = 0

c     --- loop over boundaries  1........6

      do nside = 1, 6

c     --- Get information on boundary 'nside'

        call isbnbvi(ndim,ni,nj,nk,nside,ntdir,ntsig,carea,varea)

        ndir  = ntdir(1)
        t1dir = ntdir(2)
        t2dir = ntdir(3)
        nsign = ntsig(1)

        nseg = iinbc( iinbc(nside) )

c       --- loop over segments

        do k = 1, nseg

          ipstart = iinbc(nside) +(k-1) *( 4 + (ntrnsp+nturb+1) * 4 )

c         --- Get cell index bounds and change to vertex bounds

          i1 = iinbc(ipstart+1)
          j1 = iinbc(ipstart+2)
          i2 = iinbc(ipstart+1+2)+1
          j2 = iinbc(ipstart+2+2)+1

          if ( iinbc( ipstart + 5 ).eq.9 ) then

c         --- Fill coefs in wall adjacent cells for turbulent
c             kinetic energy equation using wall functions

c             --- loop over cel

            do j = 2*j1-1, 2*j2-1
              do i = 2*i1-1, 2*i2-1

                if ( nside.eq.1 ) then
                  ip = 1
                  jp = int((i+1)/2)
                  kp = int((j+1)/2)
                else if ( nside.eq.2 ) then
                  ip = ni
                  jp = int((i+1)/2)
                  kp = int((j+1)/2)
                else if ( nside.eq.3 ) then
                  ip = int((j+1)/2)
                  jp = 1
                  kp = int((i+1)/2)
                else if ( nside.eq.4 ) then
                  ip = int((j+1)/2)
                  jp = nj
                  kp = int((i+1)/2)
                else if ( nside.eq.5 ) then
                  ip = int((i+1)/2)
                  jp = int((j+1)/2)
                  kp = 1
                else if ( nside.eq.6 ) then
                  ip = int((i+1)/2)
                  jp = int((j+1)/2)
                  kp = nk
                end if
                if ( ip.gt.ni ) ip = ni
                if ( jp.gt.nj ) jp = nj
                if ( kp.gt.nk ) kp = nk

c               --- first, compute two essential wall quantities:
c                   distance perpendicular to the wall and physical
c                   tangential velocity

                call iswllqn3( physut, ndist, facs, coor, solut,
     +                         ni, nj, nk, nvirtual, ndir, ip,
     +                         jp, kp, nside, nsign, t1dir, t2dir )

                rhow = coefs(ip,jp,kp,1)
                mu = coefs(ip,jp,kp,4) -
     +               coefs(ip,jp,kp,1)*eddy(ip,jp,kp)/rintur(4)
                tke = solut(ip,jp,kp,ndegfd-1,1)

                call islogw(tauw, uplus, physut, ndist, tke,
     +                      rintur, rhow, mu, rinbc(iplrinbc+1))

                coefs(ip,jp,kp,3) = tauw*physut/ndist
                coefs(ip,jp,kp,2) = rhow*( rintur(3)**(75d-2) )*
     +                              sqrt( tke )*uplus/ndist

              end do
            end do

          end if

          iplrinbc = iplrinbc +(ntrnsp+nturb+2)*3

        end do

      end do

      end
