      subroutine isconloc ( nblocks, iadmin, issolut, isiinput,
     +                      isrinput, locconv, ilocstr, rlocstr )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.1    date 05-10-1999  Jason: send-requests added
c        version 1.0    date 20-04-1999
c
c
c   copyright (c) 1999-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Check if local convergence of the iteration process has been reached
c    This subroutine has not yet been implemented
c **********************************************************************
c
c                       KEYWORDS
c
c   structure
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer issolut(*), isiinput(*), isrinput(*), nblocks,
     +        iadmin(1:nblocks), ilocstr(*)
      double precision rlocstr(*)
      logical locconv

c     iadmin         i    contains information about
c                         which blocks on which computer are stored.
c     ilocstr        i    Contains information of the structure of the main
c                         program for each cluster
c                         See the programmers Guide for a description
c     isiinput       i    Array of sequence numbers (nblocks) of array iinput
c     isrinput       i    Array of sequence numbers (nblocks) of array rinput
c     issolut        i    Array of sequence numbers (nblocks) of array solut
c     locconv       i/o   If true local convergence has been reached
c     nblocks        i    Number of blocks
c     rlocstr        i    Contains information of the structure of the main
c                         program for each cluster
c                         See the programmers Guide for a description
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisparal'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer iblock, ipiinput, ndegfd,
     +        ndim, icrit, imax, ind, ipsolut, ni, nj, nk,
     +        node, npointsv, nvirtual, ireq
      double precision eps, am, dpinfnorm

c     am             Help variable to compute a maximum
c     dpinfnorm      infinite norm of the iterative pressure correction
c     eps            Accuracy with respect to the time parameter tout
c     iblock         Block sequence number
c     icrit          Integer defining what criterion should be used to
c                    check convergence.
c                    Possible values:
c                    0:  Standard method, the norm of the difference of two
c                        succeeding iterations is considered
c                    1:  The norm of the pressure correction is considered
c     imax           Maximum position in table
c     ind            Sequence number of node
c     ipiinput       Starting address of array iinput in ibuffr
c     ipsolut        Starting address of array solut in buffer
c     ireq           array of send requests
c     ndegfd         Number of degrees of freedom
c     ndim           Dimension of space (2 or 3)
c     ni             Number of non-virtual cells in the i-direction
c                    (one block)
c     nj             Number of non-virtual cells in the j-direction
c                    (one block)
c     nk             Number of non-virtual cells in the k-direction
c                    (one block)
c     node           Node actually received from
c     npointsv       Number of points in the mesh with virtual points
c     nvirtual       Number of rows of virtual cells
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer idamax, isdpgetp, isingetp

c     IDAMAX         Blas routine to find the index in an array corresponding
c                    to the element with largest absolute value
c     ISACTIVE       Activate old permanent array
c     ISDPGETP       get the pointer corresponding to a double precision array
c     ISINGETP       get the pointer corresponding to an integer array
c     ISRECMPI       Receive data from node or host
c     ISSENMPI       Send data to node or host
c     ISWAITALL      Wait for all sends to complete
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     Get starting positions of the array ilocstruct and rlocstruct in
c     ibuffr/buffer
c     Get icrit from array ilocstruct
c     If ( icrit=0 ) then
c        locconv := true
c     else ( icrit=1 )
c        Get accuracy eps from array rlocstruct
c        Norm := norm of Delta p
c        locconv := Norm < eps
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- Get starting positions of the array ilocstruct and rlocstruct in
c         ibuffr/buffer

      call isactive(isiinput(1))
      call isactive(isrinput(1))
      ipiinput  = isingetp(isiinput(1))
      ndegfd    = ibuffr(ipiinput-1 + 55)
      ndim      = ibuffr(ipiinput-1 + 57)

c     --- Get icrit from array ilocstruct

      icrit = ilocstr(4)
      if ( icrit.eq.0 ) then

c     --- icrit = 0

         locconv = .true.

      else

c     --- icrit=1
c         Get accuracy eps from array rlocstruct

         eps = rlocstr(1)
         dpinfnorm = 0d0

c        --- Norm := norm of Delta p (max norm)

         do iblock = 1, nblocks
            if ( iadmin(iblock).eq.inodenr ) then

c           --- activate arrays

               call isactive(isiinput(iblock))
               call isactive(issolut(iblock))
               ipsolut  = isdpgetp(issolut(iblock))
               ipiinput = isingetp(isiinput(iblock))
               nvirtual = ibuffr(ipiinput + 54 -1)
               ni       = ibuffr(ipiinput + 51 -1)
               nj       = ibuffr(ipiinput + 52 -1)

               if ( ndim.eq.2 ) then

                  nk       = 1
                  npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)* nk

               else if ( ndim.eq.3 ) then

                  nk       = ibuffr(ipiinput + 53 -1)
                  npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)*
     +                                             (nk+1+2*nvirtual)

               end if
               ipsolut = ipsolut+npointsv*(ndegfd+ndim)
               imax = idamax ( npointsv, buffer(ipsolut), 1 )
               dpinfnorm = max(dpinfnorm,abs(buffer(ipsolut-1+imax)))

            end if

         end do

c        --- exchange information

         if ( iacnodes.gt.0 .and. inodenr.eq.ISHOST ) then

c        --- locconv := Norm < eps

            do ind = 1, iacnodes

c           --- for all nodes not on the host, receive norm

               node = IS_ANY
               call isrecmpi ( am, 1, ISDOUBLE, node,
     +                         ICSTCR )
               dpinfnorm = max(dpinfnorm,am)

            end do

         else if ( iacnodes.gt.0  ) then

c        --- not on the host, send norms to the host

            call issenmpi ( dpinfnorm, 1, ISDOUBLE, ISHOST,
     +                      ICSTCR, ireq )
            call iswaitall ( 1, ireq )

         end if

         locconv = dpinfnorm.lt.eps

      end if
      end
