      subroutine iscorn ( iadmin, isiinput, issolut, isitopol )
c ======================================================================
c
c        programmer     Erik Brakkee
c        version 1.6    date 02-11-1999 Jason: replaced with call to ismbexch
c        version 1.5    date 05-12-1998 Guus: Compute mean values
c        version 1.4    date 23-11-1998 Jason: removed calls to isini* isterm*
c        version 1.3    date 31-08-1998 Jason: remove check nodeid
c        version 1.2    date 27-02-1997 Kees K: extension 3D
c        version 1.1    date 13-03-1996 Guus: New call to iscorn02
c        version 1.0    date 07-04-1994
c
c
c   copyright (c) 1994-1998  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Correct normal fluxes on the host. After a call to this subroutine,
c     all normal fluxes on the block interfaces will have exactly the same
c     values for each pair of neighbouring blocks
c     This step in the multi-block algorithm occurs between solving
c     the momentum equations and solving the pressure equations.
c     It is essential for obtaining a right-hand side of the pressure
c     equation which is compatible with divergence freedom.
c **********************************************************************
c
c                       KEYWORDS
c
c     multiblock
c     correction_fluxes
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iadmin(*), isiinput(*), isitopol, issolut(*)

c     iadmin   i  reserved
c     isiinput i  Array of sequence numbers (nblocks) of array iinput
c     isitopol i  Sequence number of array itopol
c     issolut  i  Array of sequence numbers (nblocks) of array solut
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ilev, ipiinput, itrans, isol,  ipiinblk
      logical mean

c     ilev      Time level of the multi-block transport of information
c     ipiinblk  Starting address of array iinblk in ibuffr
c     ipiinput  Starting address of array iinput in ibuffr
c     isol      Solution type (accurate=0), (inaccurate=1), (ilu=2)
c     itrans    reserved
c     mean      logical specifying that mean fluxes should be computed
c **********************************************************************
c
c                       I/O
c
c      none
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isingetp

c     ISACTIVE  Activate permanent array
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISINGETP  get the pointer corresponding to an integer array
c     ISMBEXCH  Transport unknowns to neighbouring blocks, using mean values
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen('iscorn')

      call isactive(isiinput(1))
      ipiinput = isingetp(isiinput(1))
      ilev     = ibuffr(ipiinput + 64 - 1)
      itrans = 2
      ipiinblk = ibuffr(ipiinput + 7 - 1) + ipiinput - 1
      isol     = ibuffr(ipiinblk + 2 - 1)

      mean = .true.
      call ismbexch ( itrans, ilev, isiinput, issolut,
     +                iadmin, isitopol, isol.eq.0, mean )

      call iserclos('iscorn')

      end
