      subroutine isdivmas ( iadmin, isiinput, isrinput, issolut,
     +                      iscoefs )
c ======================================================================
c
c     programmer   Guus Segal
c     version      1.1  09-06-1998 Charles: nvpoints changed to npointsv
c     version      1.0  19-11-1997
c
c   copyright (c) 1997-1997  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Divide the velocity vector by the mass fraction
c
c **********************************************************************
c
c                       KEYWORDS
c
c     mass_fraction
c     velocity
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iadmin(*), isiinput(*), isrinput(*), issolut(*),
     +        iscoefs(*)

c     iadmin    i   reserved
c     iscoefs   i   Array of sequence numbers (nblocks) of array coefs
c     isiinput  i   Array of sequence numbers (nblocks) of array iinput
c     isrinput  i   Array of sequence numbers (nblocks) of array rinput
c     issolut   i   Array of sequence numbers (nblocks) of array solut
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisparal'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer iblock, ipiinput, ipsolut, nblocks, ndegfd, mequat,
     +        ndim, ni, nj, nk, ntimlv, nvirtual, npointsv, ipcoefs,
     +        ipiincom

c     iblock    Block sequence number
c     ipcoefs   Starting address of array coefs in buffr
c     ipiincom  Starting address of array IINCOM in array IINPUT
c     ipiinput  Starting address of array iinput in ibuffr
c     ipsolut   Starting address of array solut in buffr
c     mequat    Indicates the type of momentum equation to be solved:
c               0:  standard equation
c               1:  multi-phase gas flow
c     nblocks   Number of blocks
c     ndegfd    Number of degrees of freedom
c     ndim      Dimension of space (2 or 3)
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     npointsv  Number of points in the mesh with virtual points
c     ntimlv    Number of time levels
c     nvirtual  Number of rows of virtual cells
c **********************************************************************
c
c                       I/O
c
c      none
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isingetp, isdpgetp

c     ISACTIVE  Activate old permanent array
c     ISDPGETP  get the pointer corresponding to a double precision array
c     ISDVMAFR  Correct velocity by dividing by the mass fraction
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISINGETP  get the pointer corresponding to an integer array
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     For all blocks do
c        If ( block at this node ) then
c           Activate arrays
c           Divide velocity by mass fraction
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen ( 'isdivmas' )

c     --- shift arrays of block iblock
c         determine some general parameters

      call isactive(isiinput(1))
      ipiinput = isingetp(isiinput(1))
      ipiincom = ibuffr(ipiinput +  9 - 1)
      mequat   = ibuffr(ipiinput + ipiincom + 1)
      if ( mequat.ne.1 ) go to 1000

      nblocks  = ibuffr(ipiinput + 73 - 1)
      ndegfd   = ibuffr(ipiinput + 55 - 1)
      ndim     = ibuffr(ipiinput + 57 - 1)
      ntimlv   = ibuffr(ipiinput + 64 - 1)
      nvirtual = ibuffr(ipiinput + 54 - 1)

      do iblock = 1, nblocks

         if ( iadmin(iblock).eq.inodenr .or. inodenr.eq.0 ) then

            call isactive(isiinput(iblock))
            call isactive(isrinput(iblock))
            call isactive(issolut(iblock))
            call isactive(iscoefs(iblock))

            ipiinput = isingetp(isiinput(iblock))
            ipsolut  = isdpgetp(issolut(iblock))
            ipcoefs  = isdpgetp(iscoefs(iblock))

            ni       = ibuffr(ipiinput + 51 - 1)
            nj       = ibuffr(ipiinput + 52 - 1)
            nk       = ibuffr(ipiinput + 53 - 1)
            npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)
            if ( ndim.eq.3 ) then
               npointsv = npointsv*(nk+1+2*nvirtual)
            end if

            call isdvmafr ( ni, nj, ndegfd, nvirtual,ntimlv,
     +                      buffer(ipsolut),
     +                      buffer(ipcoefs+5*npointsv) )

         end if

      end do

1000  call iserclos ( 'isdivmas' )

      end
