      subroutine isexvc02(iblock, itrans, ni, nj, nvirtual,
     +                    ndegfd, itimlv, ntimlv, nblocks, nfaces,
     +                    maxnsubf, solut, nsubfaces, subfaces)
c=====================================================================
c
c
c        programmer   Kees Kassels
c
c        version 2.2  date 19-11-97 Kees K: iperiod removed
c        version 2.1  date 08-05-95 Michiel K: copy of isexso02, but
c                     only for corners
c        version 2.0  date  27-12-1994 code cleanup
c        version 1.0  date  17-08-1994
c
c   copyright (c) 1994  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c***********************************************************************
c
c                  DESCRIPTION
c
c     Initialize virtual unknowns in 2D corners
c
c***********************************************************************
c
c                  KEYWORDS
c
c   mapping_array
c   topology
c***********************************************************************
c
c                  INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iblock, itrans, maxnsubf, nblocks, ndegfd, nfaces,
     +        ni, nj, nvirtual, itimlv, ntimlv

      double precision solut(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1:ndegfd, 1:ntimlv)

      integer nsubfaces(1:nblocks, 1:nfaces)

      integer subfaces(1:nblocks, 1:nfaces, 1:maxnsubf, *)

c     iblock    i    Block sequence number
c     itimlv    i    Loop variable with respect to a given time level
c     itrans    i    reserved
c     maxnsubf  i    Maximum number of subfaces encountered on all subfaces
c     nblocks   i    Number of blocks
c     ndegfd    i    Number of degrees of freedom
c     nfaces    i    Number of faces
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nsubfaces i    Array containing the number of subfaces of each face,
c                    see array itopol for an extended description
c     ntimlv    i    Number of time levels
c     nvirtual  i    Number of rows of virtual cells
c     solut     i    reserved
c     subfaces  i    reserved
c***********************************************************************
c
c                   COMMON BLOCKS
c
      include 'IScommon/cisnas'
c***********************************************************************
c
c                   LOCAL PARAMETERS
c
      integer i, j, k, ifi, jfi, klow, khigh

      double precision w1, w2

c     i         General loop variable
c     ifi       Starting value of general loop variable i
c     j         General loop variable
c     jfi       Starting value of general loop variable j
c     k         General loop variable
c     khigh     Upper index of unknowns
c     klow      Lower index of unknowns
c     w1        Weight factor for extrapolation in direction 1 at the
c               faces/edges/vertices
c     w2        Weight factor for extrapolation in direction 2 at the
c               faces/edges/vertices
c**********************************************************************
c
c                   I/O
c
c
c**********************************************************************
c
c                   CALLED SUBROUTINES
c
c
c**********************************************************************
c
c                   ERROR MESSAGES
c
c
c***********************************************************************
c
c                   PSEUDO CODE
c
c=======================================================================
c
c     --- determine index range for unknowns
c
      if ( itrans .eq. 1 ) then

c        --- assume solut, tangential fluxes across the boundary,
c            followed by normal fluxes across the boundary

         klow = 1
         khigh = 2

      else

         klow = itrans
         khigh = itrans

      endif

c     --- handle corner points

      do k = klow, khigh

c     --- bottom-left corner

         w1 = 5d-1
         if ( subfaces(iblock, idown, 1, 5 ) .ne. 0 ) w1 = w1 + 5d-1
         if ( subfaces(iblock, ileft, 1, 5 ) .ne. 0 ) w1 = w1 - 5d-1
         w2 = 1d0 - w1

         do i = 0, 1-nvirtual, -1
            do j = 0, 1-nvirtual, -1
               solut(i,j,k,itimlv) =
     +            ( 2d0*solut(i+1,j,k,itimlv)   -
     +                  solut(i+2,j,k,itimlv)   ) * w1 +
     +            ( 2d0*solut(i,j+1,k,itimlv)   -
     +                  solut(i,j+2,k,itimlv)   ) * w2
            enddo
         enddo

c        --- bottom-right corner

         w1 = 5d-1
         if ( subfaces(iblock,idown,nsubfaces(iblock,idown), 5) .ne. 0)
     +           w1 = w1 + 5d-1
         if ( subfaces(iblock,iright,1,5) .ne. 0 ) w1 = w1 - 5d-1
         w2 = 1d0 - w1

         ifi = ni+1
         if ( k .eq. 1 ) ifi = ifi + 1
         do i = ifi, ni+1+nvirtual
            do j = 0, 1-nvirtual, -1
               solut(i,j,k,itimlv) =
     +           ( 2d0*solut(i-1,j,k,itimlv)    -
     +                 solut(i-2,j,k,itimlv)    ) * w1 +
     +           ( 2d0*solut(i,j+1,k,itimlv)    -
     +                 solut(i,j+2,k,itimlv)    ) * w2
            enddo
         enddo

c        --- top-left corner

         w1 = 5d-1
         if ( subfaces(iblock,iup,1,5) .ne. 0 ) w1 = w1 + 5d-1
         if ( subfaces(iblock,ileft,nsubfaces(iblock,ileft),5) .ne. 0 )
     +           w1 = w1 -5d-1
         w2 = 1d0 - w1

         jfi = nj+1
         if( k .eq. 2 ) jfi = jfi + 1
         do i = 0, 1-nvirtual, -1
            do j = jfi, nj+1+nvirtual
               solut(i,j,k,itimlv) =
     +            ( 2d0*solut(i+1,j,k,itimlv)   -
     +                  solut(i+2,j,k,itimlv)   ) * w1 +
     +            ( 2d0*solut(i,j-1,k,itimlv)   -
     +                  solut(i,j-2,k,itimlv)   ) * w2
            enddo
         enddo

c        --- top-right corner

         w1 = 5d-1
         if ( subfaces(iblock,iup,nsubfaces(iblock,iup),5) .ne. 0 )
     +          w1 = w1 + 5d-1
         if ( subfaces(iblock,iright,nsubfaces(iblock,iright),5)
     +         .ne. 0)
     +          w1 = w1 - 5d-1
         w2 = 1d0 - w1
         ifi = ni+1
         jfi = nj+1
         if ( k .eq. 1 ) ifi = ifi + 1
         if ( k .eq. 2 ) jfi = jfi + 1

         do i = ifi, ni+1+nvirtual
            do j = jfi, nj+1+nvirtual
               solut(i,j,k,itimlv) =
     +            ( 2d0*solut(i-1,j,k,itimlv)   -
     +                  solut(i-2,j,k,itimlv)   ) * w1 +
     +            ( 2d0*solut(i,j-1,k,itimlv)   -
     +                  solut(i,j-2,k,itimlv)   ) * w2
            enddo
         enddo

      enddo

      end
