      subroutine isfilb2 ( iinbc, rinbc, iside, ibinfor, rincof, bndcon,
     +                     coor, ni, nj, ncoefs, ndegfd,
     +                     iequa, nvirtual, time, iprinbc, coefs, eddy,
     +                     solut, mturb, invjac, viscorr , rintur,
     +                     itimeact, iseqpres, mequat, massfrac,
     +                     iincom )
c ======================================================================
c
c        programmer     Marcel Zijlema
c        version 4.4    date 15-12-1999 Guus: minor change in loop bounds
c        version 4.3    date 07-07-1999 Guus: new energy equation
c        version 4.2    date 01-12-1998 Guus: new approach common points of
c                                       two sections
c        version 4.1    date 11-12-1997 Hester:correction condition
c                                       for multiplication with massfr
c        version 4.0    date 26-11-1997 Guus: remove ntrnsp, nturb
c                                       New definition of sequence
c        version 3.1    date 13-11-1997 Kees K. no action for perodic b.c.
c        version 3.0    date 05-11-1997 Guus: extra parameters
c        version 2.8    date 10-09-1997 Charles extra parameter iseqpres
c        version 2.7    date 04-06-1997 Kees K. initialize allinflow
c        version 2.6    date 11-07-1996 Marcel b.c. eps (Lam-Bremhorst)
c        version 2.5    date 04-06-1996 Marcel extension parameter list
c        version 2.4    date 31-05-1996 Kees K inflow for nseg
c        version 2.3    date 24-05-1996 Marcel extended k-eps model
c        version 2.2    date 10-04-1996 Kees K space for press.b.c.
c        version 2.1    date 27-03-1996 Kees K itimeact added
c        version 2.0    date 28-08-1995 Guus: complete update to increase spee
c        version 1.1    date 21-04-1995 RvN virtual point test version
c        version 1.0    date 31-01-1995
c
c
c
c   copyright (c) 1995-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     fill arrays infbnd and bndcon from arrays iinbc and rinbc
c
c **********************************************************************
c
c                       KEYWORDS
c
c     2d
c     boundary_conditions
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual,
     +        ncoefs, ndegfd, viscorr, mturb,
     +        iequa, iinbc(*), iside,
     +        ibinfor(1-nvirtual:*), iprinbc, itimeact,
     +        iseqpres, mequat, iincom(*)
      double precision coor(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,1:2),
     +                 invjac((ni+1+2*nvirtual)*(nj+1+2*nvirtual),4),
     +                 coefs(*), eddy(*), solut(*),
     +                 bndcon(2,1-nvirtual:*),
     +                 massfrac(1-nvirtual:ni+1+nvirtual,
     +                          1-nvirtual:nj+1+nvirtual),
     +                 rinbc(*), time, rintur(*), rincof(*)

c     bndcon         o    Array BNDCON is used to indicate the value of the
c                         boundary conditions for each point at the boundary of
c                         each single block for each unknown.
c                         For a description see the programmers guide
c     coefs          i    COEFS(i,k) contains the k-th coefficient in
c                         centroid i. The nodes are ordered in exactly the
c                         same way as the scalar equations in SOLUT.
c                         The coefficients array is extended with virtual
c                         cells in exactly the same way as the locally
c                         copied solution array is.
c     coor           i    Array containing the coordinates for one block
c                         In 2d coor(i,j,.) contains the x resp y-coordinate
c                         of point i,j
c                         in 3d: coor(i,j,k,.) corresponds to (i,j,k)
c     eddy           i    array EDDY contains the values of the
c                         eddy viscosity of that block in the centroids
c                         of the elements. The storage of such
c                         an array EDDY is identical to that of all other
c                         coefficients.
c     ibinfor        o    Third part of infbnd
c     iequa          i    General equation sequence number
c     iinbc          i    Array IINBC is used to indicate the type of
c                         boundary conditions for each segment at the
c                         boundary of each single block for each unknown.
c                         IINBC is defined as sub array of array IINPUT,
c                         which means that for each block there is a
c                         separate array IINBC.
c                         For a description see the programmers guide
c     iincom         i    Array IINCOM is part of array IINPUT and hence
c                         coupled to one block only. It contains information
c                         with respect to the compressible flow
c                         For a description see the programmers guide
c     invjac         i    1/sqrt(g) in pnttyp
c     iprinbc        i    Starting address of array rinbc in array iinput
c     iseqpres       i    Sequence number of pressure equation
c     iside          i    Sequence number of side
c     itimeact       i    Actual time step
c     massfrac       i    Array containing the mass fraction
c     mequat         i    Indicates the type of momentum equation to be solved:
c                         0:  standard equation
c                         1:  multi-phase gas flow
c     mturb          i    Indicates type of turbulence modelling
c     ncoefs         i    Number of coefficients
c     ndegfd         i    Number of degrees of freedom
c     ni             i    Number of non-virtual cells in the i-direction
c                         (one block)
c     nj             i    Number of non-virtual cells in the j-direction
c                         (one block)
c     nvirtual       i    Number of rows of virtual cells
c     rinbc          i    array RINBC is used to store constants for the
c                         computation of boundary conditions for each segment at
c                         the boundary of each single block for unknown.
c                         The contents of RINBC are defined by IINBC
c                         For a description see the programmers guide
c     rincof         i    in rincof real information about the coefficients
c                         is stored. The contents is defined by iincof.
c                         For a description see the programmers guide
c     rintur         i    Array RINTUR is part of array RINPUT and hence is
c                         coupled to one block only. At this moment RINTUR
c                         is filled with fixed constants, however, since in
c                         practice these turbulence constants may depend on
c                         the type of model to be solved, it has been
c                         decided to let RINTUR be part of RINPUT. In this
c                         way it may be possible in the future to adapt the
c                         series of constants in the pre-processing phase.
c                         For a description see the programmers guide
c     solut          i    Solution array
c                         For a description see the programmers guide
c     time           i    Actual time of time step
c     viscorr        i    Indicate whether to use viscous corrections or not
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer outside
      parameter (outside = 101)
      logical allinflow
      integer i, k, idim, nseg, ipstart, ndegfdlc, ilowloop,
     +        i1, i2, btype, bctyp(1:2), nij, menergy,
     +        ip, jp, inci, incj, m
      double precision coorx, coory, coorz, bcval(1:2), bvalue, usfunb,
     +                 massmean, help, rho

c     allinflow      Indicating inflow over all segments
c     bctyp          A given boundary type
c     bcval          A given boundary condition value
c     btype          Type of boundary condition
c     bvalue         Real boundary value
c     coorx          X-coordinate
c     coory          Y-coordinate
c     coorz          Z-coordinate
c     help           Help variable to store a real temporarily
c     i              General loop variable
c     i1             Lower index number along first direction of a face or
c                    edge
c     i2             Upper index number along first direction of a face
c                    or edge
c     idim           Loop variable used for dimension
c     ilowloop       Lower bound for loop
c     inci           Offset between vertices in i-direction
c     incj           Offset between vertices in j-direction
c     ip             Local counting variable i-direction
c     ipstart        Starting address
c     jp             Local counting variable j-direction
c     k              General loop variable
c     m              General loop variable
c     massmean       Mean value of mass fraction
c     menergy        Defines the type of energy equation used.
c                    Possible values:
c                    0: none
c                    1: the primary unknown is the enthalpy
c                    2: the primary unknown is the total enthalpy
c                    3: the primary unknown is the internal energy
c     ndegfdlc       Local number of degrees of freedom
c     nij            ni or nj
c     nseg           Number of segments the boundary is divided in
c     outside        Type of boundary condition for points outside the
c                    ibinfor array
c     rho            Estimate of the asymptotic convergence rate
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISBNDP         Set inflow profile
c     ISERCLOS       Resets old name of previous subroutine of higher level
c     ISEROPEN       Produces concatenated name of local subroutine
c     ISERRSUB       Error messages
c     ISMULMFR       Multiply array bndcon by the mass fraction for a specific s
c                    and segment
c     ISOMEGB2       Fill boundary values for omega in wall adjacent
c                    cells (2D)
c     ISWLLFN2       Fill boundary values for momentum  and coefs in
c                    wall adjacent cells for turbulence equations
c                    using wall functions
c     USFUNB         user written subroutine
c     USPCOMPINTEN   user written subroutine to compute the internal energy
c                    from the temperature per point
c **********************************************************************
c
c                       ERROR MESSAGES
c
c      27   Multi phase gas flow has not yet been implemented
c    1202   Only combination of k-eps type models and wall functions is
c           allowed
c    1203   No viscous corrections should be used when wall functions
c           are applied
c    1204   Viscous corrections should be used in case of low-Reynolds
c           number model
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen( 'isfilb2' )

      menergy  = iincom(10)
      if ( iside.le.2 ) then
         nij  = nj
      else
         nij  = ni
      end if
      coorz = 0d0

c     --- Initialize the arrays

      if ( itimeact.eq.1 ) then
         do i = 1-nvirtual, 2*nij+1+nvirtual
            ibinfor(i) = outside
            bndcon(1,i) = 0d0
            bndcon(2,i) = 0d0
         end do
      end if

c     --- Loop over segments on this side
c         initialize allinflow

      allinflow = .false.

      nseg = iinbc( iinbc(iside) )
      if ( iseqpres.eq.2 ) then

c     --- Coupled approach

         ndegfdlc = ndegfd-1

      else

c     --- Decoupled approach

         ndegfdlc = ndegfd

      end if

      do k = 1, nseg

         ipstart = iinbc(iside) +(k-1) *(2 +ndegfdlc *3)

c        --- Get cell index bounds and change to vertex bounds

         i1 = iinbc(ipstart+1)
         i2 = iinbc(ipstart+2)+1

         btype = iinbc( ipstart +(iequa-1)*3 +3 )
         do idim = 1, 2
            bctyp(idim) = iinbc( ipstart +(iequa-1)*3 +3 +idim)
            if (bctyp(idim).eq.-1) then
               bcval(idim) = rinbc( iprinbc +(iequa-1)*2 +idim)
            end if
         end do

         if ( btype.eq.3 .or. btype.eq.9 .or. btype.eq.34 ) then

c        --- If wall functions, then fill bndcon for momentum, temperature
c            and dissipation rate equations
c            Check if no viscous corrections are used

           if ( viscorr.ne.0 ) then
              call iserrsub ( 1203, 0, 0, 0 )
           end if
           if ( mequat.eq.1 ) then

c          --- multiphase gas flow has not yet been implemented

              call iserrsub ( 27, 0, 0, 0 )

           end if

c          --- Check if only k-eps type models are used

           if ( mturb.ne.1 .and. mturb.ne.2 .and. mturb.ne.5 ) then
             call iserrsub ( 1202, 0, 0, 0 )
           end if

           call iswllfn2 ( iequa, iside, i1, i2, bndcon, rincof, coefs,
     +                     coor, eddy, solut, rintur, ni, nj, ndegfd,
     +                     nvirtual, ncoefs, invjac(1,4),
     +                     rinbc(iprinbc+1), bcval(1), ibinfor, btype,
     +                     iseqpres )

        else if ( btype.eq.12 ) then

           if ( k.eq.1 .and. nseg.gt.1 ) then
              allinflow = .true.
              do m = 2, nseg
                 ipstart = iinbc(iside) +(m-1) *(2 +ndegfdlc *3)
                 btype = iinbc( ipstart +(iequa-1)*3 +3 )
                 if ( btype.ne.12 ) then
                    allinflow = .false.
                 end if
              end do
              if ( allinflow ) then
                 ipstart = iinbc(iside) +(nseg-1) *
     +                                          (2 +ndegfdlc *3)
                 i2 = iinbc(ipstart+2)+1
              end if
           end if

c          --- parabolic inflow

           if ( allinflow ) then
              if ( k.eq.1 ) then
                 bvalue = rinbc( iprinbc +(iequa-1)*2 +1)
                 call isbndp ( bndcon, coor, ni, nj, nvirtual, i1, i2-1,
     +                         iside, bvalue, ibinfor, mequat,
     +                         massfrac )
              end if
           else
              bvalue = rinbc( iprinbc +(iequa-1)*2 +1)
              call isbndp ( bndcon, coor, ni, nj, nvirtual, i1, i2-1,
     +                      iside, bvalue, ibinfor, mequat, massfrac )
           end if

        else if ( btype.eq.27 ) then

c       --- analytical solution of omega in viscous sublayer

            call isomegb2 ( iside, i1, i2, bndcon, coefs, coor, solut,
     +                      rintur, ni, nj, nvirtual, ncoefs,
     +                      eddy, invjac(1,4), viscorr, ibinfor )

        else if ( btype.eq.48 ) then

c       --- boundary condition for dissipation rate at wall (homogeneous
c           Neumann condition), according to Lam and Bremhorst (1981)
c           Check if viscous corrections are used

            if ( viscorr.eq.0 ) then
              call iserrsub ( 1204, 0, 0, 0 )
            end if

            do i = 2*i1-1, 2*i2-1
              ibinfor(i) = 2
              bndcon(1,i) = 0d0
              bndcon(2,i) = 0d0
            end do

        else if ( btype.eq.98 .or. btype.eq.99 ) then

           if ( k.gt.1 .and. ibinfor(2*i1-1).eq.5 ) then
              ibinfor(2*i1-1) = 89
           else if ( k.gt.1 .and. (ibinfor(2*i1-1).eq.4 .or.
     +               ibinfor(2*i1-1).eq.10 .or.
     +               ibinfor(2*i1-1).eq.12) ) then
              ibinfor(2*i1-1) = 88
           else
              ibinfor(2*i1-1) = btype
           end if
           do i = 2*i1, 2*i2-1
              ibinfor(i) = btype
           end do

c          --- do nothing

        else

           ilowloop = 2*i1-1
           if ( k.gt.1 .and. (ibinfor(ilowloop).eq.4 .or.
     +          ibinfor(ilowloop).eq.10 .or. ibinfor(ilowloop).eq.12) )
     +        then

c          --- Previous value is Dirichlet, keep it

              ilowloop = ilowloop + 1

           else if ( k.gt.1 .and. btype.eq.5 .and.
     +               (ibinfor(ilowloop).eq.98 .or.
     +                ibinfor(ilowloop).eq.99) ) then

c          --- Previous value is periodical, present value is free-slip
c              Flag the point as transit point

              ibinfor(2*i1-1) = 89
              ilowloop = ilowloop + 1

           else if ( k.gt.1 .and. (btype.eq.4 .or. btype.eq.10 .or.
     +               btype.eq.12) .and.
     +               (ibinfor(ilowloop).eq.98 .or.
     +                ibinfor(ilowloop).eq.99) ) then

c          --- Previous value is periodical, present value is Dirichlet
c              Flag the point as transit point

              ibinfor(2*i1-1) = 88
              ilowloop = ilowloop + 1

           else if ( k.gt.1 .and. (ibinfor(ilowloop).eq.98 .or.
     +               ibinfor(ilowloop).eq.99) ) then

c          --- Previous value is periodical and present not dirichlet
c              Keep periodical

              ilowloop = ilowloop + 1

           else if ( k.gt.1 .and. ibinfor(ilowloop).eq.5 .and. .not.
     +               (btype.eq.4 .or. btype.eq.10 .or. btype.eq.12) )
     +        then

c          --- Previous value is free-slip and present not dirichlet
c              Keep free-slip

              ilowloop = ilowloop + 1

           end if
           do i = ilowloop, 2*i2-1
              ibinfor(i) = btype
           end do

           do idim = 1, 2

              if ( bctyp(idim).eq.-1 ) then

                 do i = ilowloop, 2*i2
                    bndcon(idim,i) = bcval(idim)
                 end do

                 if ( mequat.eq.1 .and. abs(bcval(idim)).gt.0d0
     +                .and. iequa.lt.iseqpres ) then

c                --- Multiphase gas flow, multiply by the mass fraction

                    call ismulmfr ( bndcon, massfrac, iside, i1, i2, ni,
     +                              nj, idim, nvirtual )

                 end if

              else if ( bctyp(idim).gt.0 ) then

                if ( itimeact.eq.1 ) then

                    if ( iside.le.2 ) then
                       jp = i1-1
                       inci  = 0
                       incj  = 1
                       if ( iside.eq.1 ) then
                          ip = 1
                       else
                          ip = ni+1
                       end if
                    else
                       ip = i1-1
                       inci  = 1
                       incj  = 0
                       if ( iside.eq.3 ) then
                          jp = 1
                       else
                          jp = nj+1
                       end if
                    end if
                    do i = i1, i2

                       ip = ip+inci
                       jp = jp+incj
                       coorx = coor(ip,jp,1)
                       coory = coor(ip,jp,2)
                       if ( i.gt.i1 .or. ilowloop.eq.2*i1-1) then
                          bndcon(idim,2*i-1) = usfunb(bctyp(idim),
     +                                        coorx,coory,coorz,time)
                          if ( mequat.eq.1 ) bndcon(idim,2*i-1) =
     +                       bndcon(idim,2*i-1)*massfrac(ip,jp)
                       end if
                       if ( i.ne.i2 ) then
                          coorx = 0.5d0*(coor(ip,jp,1)+
     +                                coor(ip+inci,jp+incj,1))
                          coory = 0.5d0*(coor(ip,jp,2)+
     +                                coor(ip+inci,jp+incj,2))
                          bndcon(idim,2*i) = usfunb(bctyp(idim),
     +                                     coorx,coory,coorz,time)
                          if ( mequat.eq.1 ) then
                             massmean = 0.5d0*(massfrac(ip,jp)+
     +                               massfrac(ip+inci,jp+incj))
                             bndcon(idim,2*i) = bndcon(idim,2*i)*
     +                                       massmean
                          end if
                       end if

                    end do

                 end if

              else if ( bctyp(idim).lt.-100 ) then

                 if ( iside.le.2 ) then
                    jp = i1-1
                    inci  = 0
                    incj  = 1
                    if ( iside.eq.1 ) then
                       ip = 1
                    else
                       ip = ni+1
                    end if
                 else
                    ip = i1-1
                    inci  = 1
                    incj  = 0
                    if ( iside.eq.3 ) then
                       jp = 1
                    else
                       jp = nj+1
                    end if
                 end if
                 do i = i1, i2

                    ip = ip+inci
                    jp = jp+incj
                    coorx = coor(ip,jp,1)
                    coory = coor(ip,jp,2)
                    if ( i.gt.i1 .or. ilowloop.eq.2*i-1 ) then
                       bndcon(idim,2*i-1) = usfunb(abs(100+bctyp(idim)),
     +                                     coorx,coory,coorz,time)
                       if ( mequat.eq.1 ) bndcon(idim,2*i-1) =
     +                    bndcon(idim,2*i-1)*massfrac(ip,jp)
                    end if
                    coorx = 0.5d0*(coor(ip,jp,1)+
     +                             coor(ip+inci,jp+incj,1))
                    coory = 0.5d0*(coor(ip,jp,2)+
     +                             coor(ip+inci,jp+incj,2))
                    bndcon(idim,2*i) = usfunb(abs(100+bctyp(idim)),
     +                                  coorx,coory,coorz,time)
                    if ( mequat.eq.1 ) then
                       massmean = 0.5d0*(massfrac(ip,jp)+
     +                            massfrac(ip+inci,jp+incj))
                       bndcon(idim,2*i) = bndcon(idim,2*i)*
     +                                    massmean
                    end if

                 end do

              else

                 do i = ilowloop, 2*i2-1
                    bndcon(idim,i) = 0d0
                 end do

              end if
              if ( idim.eq.1 .and. btype.eq.1 .and. iequa.eq.3 .and.
     +             menergy.eq.3 ) then

c             --- transform T to e

                 if ( iside.le.2 ) then
                    jp = i1-1
                    inci  = 0
                    incj  = 1
                    if ( iside.eq.1 ) then
                       ip = 1
                    else
                       ip = ni+1
                    end if
                 else
                    ip = i1-1
                    inci  = 1
                    incj  = 0
                    if ( iside.eq.3 ) then
                       jp = 1
                    else
                       jp = nj+1
                    end if
                 end if
                 do i = i1, i2

                    ip = ip+inci
                    jp = jp+incj
                    rho = massfrac(ip,jp)
                    if ( i.gt.i1 .or. ilowloop.eq.2*i1-1) then
                       call uspcompinten ( rho, bndcon(idim,2*i-1),
     +                                     help )
                       bndcon(idim,2*i-1) = help
                    end if
                    rho = (massfrac(ip+inci,jp+incj)+rho)*0.5d0
                    call uspcompinten ( rho, bndcon(idim,2*i),
     +                                  help )
                    bndcon(idim,2*i) = help

                 end do

              end if

           end do

        end if

        iprinbc = iprinbc +ndegfdlc*2

      end do

      call iserclos( 'isfilb2' )

      end
