      subroutine isfilb5 (  ni, nj, nvirtual, iface, infbnd, ibinfor,
     +                      bndcon )
c ======================================================================
c
c
c        programmer     Guus Segal
c        version 1.0    date 06-09-95
c
c   copyright (c) 1995-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c ********************************************************************
c
c                         DESCRIPTION
c
c    Fill parts of the arrays ibinfor and bndcon for outside points
c    in case of periodical boundary conditions
c ********************************************************************
c
c                      KEYWORDS
c
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, iface, ibinfor(*), infbnd(*)
      double precision bndcon(2,*)

c     bndcon   i/o   reserved
c     ibinfor   i    Third part of infbnd
c     iface     i    Face number
c     infbnd   i/o   reserved
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c ********************************************************************
c
c                    COMMON BLOCKS
c
c ********************************************************************
c
c                    LOCAL PARAMETERS
      integer periodic
      parameter (periodic = 98)
      integer i, nij, lowneighb, highneigh

c     highneigh Starting address in ibinfor of side adjacent to upper part
c               of boundary
c     i         General loop variable
c     lowneighb Starting address in ibinfor of side adjacent to lower part
c               of boundary
c     nij       ni or nj
c     periodic  Periodic boundary conditions
c ********************************************************************
c
c             SUBROUTINES CALLED
c
c ********************************************************************
c
c                              I/O
c
c ********************************************************************
c
c                    ERROR MESSAGES
c
c ********************************************************************
c
c                    PSEUDO CODE
c
c    Consider the rectangular region below
c
c
c
c               |                       |
c               |          4            |
c            ------------------------------
c               |                       |
c               |                       |
c               |                       |
c               |                       |
c               |                       |
c            1  |                       | 2
c               |                       |
c               |                       |
c               |                       |
c               |                       |
c            ------------------------------
c               |         3             |
c               |                       |
c
c
c
c   If for example sides 3 and 4 are periodical then the parts of sides 1 and 2
c   below side 3 are copied from the top of sides 1 and 2 (below 4)
c   In the same way the parts above 4 are copied from the parts above 3
c
c ********************************************************************
c
c                    DATA STATEMENTS
c
c ======================================================================
c
      if ( iface.le.2 ) then

         nij = nj
         if ( iface.eq.1 ) then

c        --- face 1

            lowneighb  = infbnd(3)+nvirtual
            highneigh = infbnd(4)+nvirtual

         else

c        --- face 2

            lowneighb  = infbnd(3)+nvirtual+2*ni
            highneigh = infbnd(4)+nvirtual+2*ni

         end if

      else

         nij = ni
         if ( iface.eq.3 ) then

c        --- face 3

            lowneighb  = infbnd(1)+nvirtual
            highneigh = infbnd(2)+nvirtual

         else

c        --- face 4

            lowneighb  = infbnd(1)+nvirtual+2*nj
            highneigh = infbnd(2)+nvirtual+2*nj

         end if

      end if

      if ( ibinfor(lowneighb).eq.periodic .and.
     +     ibinfor(highneigh).eq.periodic ) then

c     --- Boundary is periodic, copy information
c         First lower part

         do i = infbnd(iface), infbnd(iface)+nvirtual-1
            ibinfor(i) = ibinfor(i+2*nij)
            bndcon(1,i) = bndcon(1,i+2*nij)
            bndcon(2,i) = bndcon(2,i+2*nij)
         end do

c        --- Next upper part

         do i = infbnd(iface)+2*nij+1+nvirtual,
     +          infbnd(iface)+2*nij+2*nvirtual
            ibinfor(i) = ibinfor(i-2*nij)
            bndcon(1,i) = bndcon(1,i-2*nij)
            bndcon(2,i) = bndcon(2,i-2*nij)
         end do
      end if
      end
