      subroutine isfilg03 (  ni, nj, nk, nvirtual, coor, geom, work )
c ======================================================================
c
c     programmer     Guus Segal
c     version 1.0    date 11-06-1996
c
c   copyright (c) 1996-1996  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     computes the square root of the determinant of the metric
c     tensor upto nvirtual-1 cells beyond the boundary for
c     all points (R^3)
c     Alternative 1
c     First the covariant base-vectors are computed in the
c     points where the determinant must be computed, then
c     from these base-vectors the determinant is computed.
c **********************************************************************
c
c                       KEYWORDS
c
c     jacobian
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer  ni, nj, nk, nvirtual
      double precision  coor(*), geom(*), work(*)

c     coor       i   reserved
c     geom       o   reserved
c     ni         i   Number of non-virtual cells in the i-direction (one block)
c     nj         i   Number of non-virtual cells in the j-direction (one block)
c     nk         i   Number of non-virtual cells in the k-direction (one block)
c     nvirtual   i   Number of rows of virtual cells
c     work           Work array
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer npf(0:3,0:4,1:8)
      integer rspv(1:3,1:8,0:3,0:4,1:8)
      double precision cv(1:8,0:3,0:4,1:8)

c     cv        reserved
c     npf       reserved
c     rspv      reserved
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISCOVA3D  Compute covariant base vectors in all points of the grid
c               three-dimensional case
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISPNDF    Make tables for interpolation and finite difference formulas
c     ISSQRG3D  Compute Jacobian and its inverse in R3
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call iseropen ( 'isfilg' )

      call ispndf(npf,rspv,cv,3)

      call iscova3d ( coor,  work, ni, nj, nk, nvirtual,
     +                npf , rspv, cv )

      call issqrg3d ( geom, work, ni, nj, nk, nvirtual )

      call iserclos ( 'isfilg' )

      end
