      subroutine isfilm02(mmct,gmct,smct,nconct,ndim,maxdx)
c ======================================================================
c
c        programmer   Ronald van Nooyen
c        version 2.3    date 09-10-1996 Correction 17-point stencil (alpha=2)
c        version 2.2    date 20-04-1994 DG and transport stencil stored at
c                                       (alphat,betat)=(ndim+1,ndim+1)
c        version 2.1    date 01-11-1993 choice between 13, 17 and 21 point
c                                       stencil for momentum equations
c                                       on the basis of the value of nconct
c        version 2.0    date 11-10-1993 comments updated and description
c                                       of 3D gradient matrix and the divgrad
c                                       or transport matrix included
c        version 1.0    date 23-3-1993
c
c
c   copyright (c) 1994-1996  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     The routine iscsvmd2 i.e.
c     ISnas Cell Shift and Variable type to Matrix Diagonal in 2D
c     ^^    ^    ^         ^                ^      ^           ^
c                 initialises a code table mct which gives
c                 the relation between velocity components and
c                 pressure values in
c                 given locations and offsets from the diagonal
c                 in the momentum, gradient and  divgrad or
c                 transport matrix (in this routine for ndim=2 only)
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer nconct, ndim, maxdx
      integer mmct(1:ndim,1:ndim,
     +     -maxdx:maxdx,-maxdx:maxdx,-maxdx:maxdx)
      integer gmct(1:ndim,1:1,
     +     -maxdx:maxdx,-maxdx:maxdx,-maxdx:maxdx)
      integer smct(1:1,1:1,
     +     -maxdx:maxdx,-maxdx:maxdx,-maxdx:maxdx)

c     gmct    o   Matrix code table gradient matrix see array infmg
c     maxdx   i   Maximum relative shift between cell indices
c     mmct    o   Matrix code table transport equation see array infmt
c     nconct  i   Number of connections in a row of the matrix
c     ndim    i   Dimension of space (2 or 3)
c     smct    o   Column index+row degree of freedom to connection
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer alphat, betat, di, dj, dk

c     alphat    Variable or cell indices
c                     1...ndim: flux variable or flux matrix
c                     ndim+1:   pressure variable or
c                               divgrad/transport matrix
c     betat     Variable or cell indices
c                     1...ndim: flux variable or flux matrix
c                     ndim+1:   pressure variable or
c                               divgrad/transport matrix
c     di        Shift in stencil in i-direction
c     dj        Shift in stencil in j-direction
c     dk        Shift in stencil in k-direction
c **********************************************************************
c
c                       I/O
c
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c

c     --- Initialize all diagonal numbers to -1, so we can allways
c         recognize an attempt to find a code for an unused diagonal

      do 10000 dk=-maxdx,maxdx
         do 11000 dj=-maxdx,maxdx
            do 11100 di=-maxdx,maxdx
               do 11110 alphat=1,ndim
                  do 11111 betat=1,ndim
                     mmct(alphat,betat,di,dj,dk) = -1
11111             continue
11110          continue
               do 11120 alphat=1,ndim
                  do 11121 betat=1,1
                     gmct(alphat,betat,di,dj,dk) = -1
11121             continue
11120          continue
               do 11130 alphat=1,1
                  do 11131 betat=1,1
                     smct(alphat,betat,di,dj,dk) = -1
11131             continue
11130          continue
11100       continue
11000    continue
10000 continue

c     --- Momentum matrix

      do 2000 alphat=1,ndim

c     --- Set the diagonal number for diagonals in the
c         momentum matrix for V^alphat with shift di,dj,dk
c         in a V^alphat cell

         mmct(alphat,alphat,  0,  0, 0)   =   1
         mmct(alphat,alphat, -1, -1, 0)   =   2
         mmct(alphat,alphat,  0, -1, 0)   =   3
         mmct(alphat,alphat,  1, -1, 0)   =   4
         mmct(alphat,alphat, -1,  0, 0)   =   5
         mmct(alphat,alphat,  1,  0, 0)   =   6
         mmct(alphat,alphat, -1,  1, 0)   =   7
         mmct(alphat,alphat,  0,  1, 0)   =   8
         mmct(alphat,alphat,  1,  1, 0)   =   9

         if (alphat.eq.1) then

c        --- Set the diagonal number for diagonals in the
c            momentum matrix for V^betat with shift di,dj,dk
c            in a V^1 cell

            betat = 2
            mmct(alphat, betat, -1,  0, 0)   =  10
            mmct(alphat, betat,  0,  0, 0)   =  11
            mmct(alphat, betat, -1,  1, 0)   =  12
            mmct(alphat, betat,  0,  1, 0)   =  13
            if (nconct.gt.13) then
               mmct(alphat, betat, -1, -1, 0)   =  14
               mmct(alphat, betat,  0, -1, 0)   =  15
               mmct(alphat, betat, -1,  2, 0)   =  16
               mmct(alphat, betat,  0,  2, 0)   =  17
            end if
            if (nconct.gt.17) then
               mmct(alphat, betat, -2,  0, 0)   =  18
               mmct(alphat, betat,  1,  0, 0)   =  19
               mmct(alphat, betat, -2,  1, 0)   =  20
               mmct(alphat, betat,  1,  1, 0)   =  21
            end if

         else if (alphat.eq.2) then

c        --- Set the diagonal number for diagonals in the
c            momentum matrix for V^betat with shift di,dj,dk
c            in a V^2 cell

            betat = 1
            mmct(alphat, betat,  0, -1, 0)   =  10
            mmct(alphat, betat,  1, -1, 0)   =  11
            mmct(alphat, betat,  0,  0, 0)   =  12
            mmct(alphat, betat,  1,  0, 0)   =  13

            if (nconct.gt.13) then
               mmct(alphat, betat, -1, -1, 0)   =  14
               mmct(alphat, betat,  2, -1, 0)   =  15
               mmct(alphat, betat, -1,  0, 0)   =  16
               mmct(alphat, betat,  2,  0, 0)   =  17
            end if

            if (nconct.gt.17) then
               mmct(alphat, betat,  0, -2, 0)   =  18
               mmct(alphat, betat,  1, -2, 0)   =  19
               mmct(alphat, betat,  0,  1, 0)   =  20
               mmct(alphat, betat,  1,  1, 0)   =  21
            end if

         end if

 2000 continue

c     --- Gradient matrix

      betat = 1

c     --- diagonal code for di,dj,dk pressure in V^1 integration cell

      alphat = 1

      gmct(alphat, betat, -1, -1, 0)   =   1
      gmct(alphat, betat,  0, -1, 0)   =   2
      gmct(alphat, betat, -1,  0, 0)   =   3
      gmct(alphat, betat,  0,  0, 0)   =   4
      gmct(alphat, betat, -1,  1, 0)   =   5
      gmct(alphat, betat,  0,  1, 0)   =   6

c     --- diagonal code for di,dj,dk pressure in V^1 integration cell

      alphat = 2

      gmct(alphat, betat, -1, -1, 0)   =   1
      gmct(alphat, betat, -1,  0, 0)   =   2
      gmct(alphat, betat,  0, -1, 0)   =   3
      gmct(alphat, betat,  0,  0, 0)   =   4
      gmct(alphat, betat,  1, -1, 0)   =   5
      gmct(alphat, betat,  1,  0, 0)   =   6

c     --- Divgrad/Transport matrix

      alphat = 1
      betat  = 1

      smct(alphat,alphat,  0,  0, 0)   =   1
      smct(alphat,alphat, -1, -1, 0)   =   2
      smct(alphat,alphat,  0, -1, 0)   =   3
      smct(alphat,alphat,  1, -1, 0)   =   4
      smct(alphat,alphat, -1,  0, 0)   =   5
      smct(alphat,alphat,  1,  0, 0)   =   6
      smct(alphat,alphat, -1,  1, 0)   =   7
      smct(alphat,alphat,  0,  1, 0)   =   8
      smct(alphat,alphat,  1,  1, 0)   =   9

      end
