      subroutine isfilvcf ( iblock, nblocks, isiinput, iscoefs,
     +                      isitopol, mequat )
c ======================================================================
c
c        programmer     Kees Kassels
c        version 2.0    date 20-11-1997 Guus: extra parameter mequat
c        version 1.1    date 18-01-1995 Erik: ncoefs = iinput(66) instead
c                                       of iinput(57)
c        version 1.0    date 24-11-1994
c
c   copyright (c) 1994-1997  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     This routine extrapolates the virtual points of the local
c     coefficients arrays of the subdomains for outer boundaries.
c **********************************************************************
c
c                       KEYWORDS
c
c   extrapolation
c   virtual
c   coefficients
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iblock, nblocks, iscoefs, isiinput, isitopol, mequat

c     iblock    i    Block sequence number
c     iscoefs   i    Array of sequence numbers (nblocks) of array coefs
c     isiinput  i    Array of sequence numbers (nblocks) of array iinput
c     isitopol  i    Sequence number of array itopol
c     mequat    i    Indicates the type of momentum equation to be solved:
c                    0:  standard equation
c                    1:  multi-phase gas flow
c     nblocks   i    Number of blocks
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ipcoefs, ipiinput, ipitopol, ipnsubf,
     +        ipsubf, maxnsubf, ndim, nfaces, ni, nj, nk,
     +        nvirtual, ncoefs

c     ipcoefs   Starting address of array coefs in buffr
c     ipiinput  Starting address of array iinput in ibuffr
c     ipitopol  Starting address of array itopol in ibuffr
c     ipnsubf   Starting address of the nsubfaces array
c     ipsubf    Starting address of the subfaces array
c     maxnsubf  Maximum number of subfaces encountered on all subfaces
c     ncoefs    Number of coefficients
c     ndim      Dimension of space (2 or 3)
c     nfaces    Number of faces
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     nvirtual  Number of rows of virtual cells
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isdpgetp, isingetp

c     ISACTIVE  Activate old permanent array
c     ISDCRPRI  Decrease priorities of all arrays in ibuffr
c     ISDPGETP  Get the pointer corresponding to a double precision array
c     ISEXCF    Compute virtual coordinates not coming from a
c               neighbouring block by extrapolation
c     ISINGETP  Get the pointer corresponding to an integer array
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c      fill parameters from array itopol
c
c      for nblocks do
c         if block on this node
c            fill parameters from array iinput
c            --- actual extrapolation
c            call isexcf
c         end if
c      end do
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- compute pointers and some parameters for the itopol array

      call isactive(isitopol)
      ipitopol     = isingetp(isitopol)
      nfaces       = ibuffr(ipitopol + 2 - 1)
      maxnsubf = ibuffr(ipitopol + 3 - 1)
      ipnsubf  = ibuffr(ipitopol+5-1) + ipitopol - 1
      ipsubf   = ibuffr(ipitopol+6-1) + ipitopol - 1

c     --- fill the  virtual coefficients by extrapolation

      call isactive(isiinput)
      call isactive(iscoefs)
      ipiinput = isingetp(isiinput)
      ipcoefs  = isdpgetp(iscoefs)

      ni       = ibuffr(ipiinput + 51 - 1)
      nj       = ibuffr(ipiinput + 52 - 1)
      nk       = ibuffr(ipiinput + 53 - 1)
      nvirtual = ibuffr(ipiinput + 54 - 1)
      ndim     = ibuffr(ipiinput + 57 - 1)
      ncoefs   = ibuffr(ipiinput + 66 - 1)

      call  isexcf ( iblock, ni, nj, nk, nvirtual, ndim,
     +               ncoefs, nblocks, nfaces, maxnsubf,
     +               buffer(ipcoefs),
     +               ibuffr(ipnsubf), ibuffr(ipsubf), mequat )

      call isdcrpri

      end
