      subroutine isflc2 ( choice, coefs, ncoefs, icoef, ni, nj, ndim,
     +                    nvirtual, coor, solut, ndegfd, time )
c ======================================================================
c
c        programmer     Kees Kassels
c        version 2.2    date 01-12-1998 Kees K.: use cartesian velocities
c                                       in function usfunc1
c        version 2.1    date 10-06-1997 Guus: remove usfilc
c        version 2.0    date 31-10-1995 Marcel: new call to USFILC
c        version 1.0    date 22-11-1994
c
c   copyright (c) 1994-1997  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     coefficients will be filled with a user supplied function
c
c **********************************************************************
c
c                       KEYWORDS
c
c     coefficients
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer choice, ni, nj, ncoefs, icoef, ndim, ndegfd, nvirtual
      double precision coefs(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,1:ncoefs),
     +                 coor(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,1:ndim),
     +                 solut(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,1:ndegfd,1:2),
     +                 time

c     choice    i    General choice parameter
c     coefs     o    reserved
c     coor      i    reserved
c     icoef     i    Sequence number of coefficient
c     ncoefs    i    Number of coefficients
c     ndegfd    i    Number of degrees of freedom
c     ndim      i    Dimension of space (2 or 3)
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     solut     i    reserved
c     time      i    Actual time of time step
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ip, jp, i, k
      double precision coorx, coory, coorz, work(100),
     +                  output1, output2, a1(2), a2(2), sqrtg

c     a1        Covariant xi-1 base vector
c     a2        Covariant xi-2 base vector
c     coorx     X-coordinaat
c     coory     Y-coordinaat
c     coorz     Z-coordinaat
c     i         General loop variable
c     ip        Local counting variable i-direction
c     jp        Local counting variable j-direction
c     k         General loop variable
c     output1   Temporary storage for output(i,j,k,1)
c     output2   Temporary storage for output(i,j,k,2)
c     sqrtg     The Jacobian in the center of a cell
c     work      Work array
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      double precision usfunc, usfunc1

c     USFUNC    User supplied function to fill coefficients
c     USFUNC1   User supplied function to fill coefficients
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      do jp = 1, nj
         do ip = 1, ni

            coorx = 25d-2 * ( coor(ip,jp,1) + coor(ip+1,jp,1)
     +                       +coor(ip,jp+1,1)+coor(ip+1,jp+1,1))
            coory = 25d-2 * ( coor(ip,jp,2) + coor(ip+1,jp,2)
     +                       +coor(ip,jp+1,2)+coor(ip+1,jp+1,2))
            coorz = 0d0

            if ( choice.lt.100 ) then

               coefs(ip,jp,icoef) = usfunc(choice, coorx, coory,
     +                                                 coorz, time)

            else if ( choice.gt.100 .and. choice.lt.200 ) then

               output1 = 0.5d0*(solut(ip,jp,1,1)+solut(ip+1,jp,1,1))
               output2 = 0.5d0*(solut(ip,jp,2,1)+solut(ip,jp+1,2,1))

               do k = 1, 2
                  a1(k) = 0.5d0*( coor(ip+1,jp+1,k) - coor(ip,jp+1,k) +
     +                             coor(ip+1,jp,k)   - coor(ip,jp,k) )
                  a2(k) = 0.5d0*( coor(ip+1,jp+1,k) - coor(ip+1,jp,k) +
     +                             coor(ip,jp+1,k)   - coor(ip,jp,k) )
               end do

               sqrtg = a1(1)*a2(2) - a2(1)*a1(2)

               do k = 1, 2
                  work(k) =
     +               ( a1(k) * output1 + a2(k) * output2 ) / sqrtg
               end do

               do i = ndim+1, ndegfd
                  work(i) = solut(ip,jp,i,1)
               end do

               coefs(ip,jp,icoef) = usfunc1(choice-100, coorx, coory,
     +                                      coorz, time , work, ndegfd )

            end if

         end do

      end do

      end
