      subroutine ishdd( ndim    , nblocks , ieq    , isiinput,isrinput,
     +                  iscoor  , issolut , iscoefs, iseddy , isinfbnd,
     +                  isbndcon, ismatrix, isgmat , isrhside, isgeom ,
     +                  isitopol, isdensup, iadmin )
c ======================================================================
c
c     programmer   Kees Kassels
c     version      1.8   date    31-08-1998 Jason: remove check nodeid
c     version      1.7   date    15-05-1998 Duncan: new call to isnmainh
c     version      1.6   date    13-10-1997 Charles: uncoupled allowed
c     version      1.5   date    14-08-1997 Kees K: return placed after
c                                           terminate message
c     version      1.4   date    06-08-1997 Guus: return in case of error
c     version      1.3   date    24-04-1997 Kees K: ndim added
c     version      1.2   date    06-02-1995 Michiel K:array raction removed
c     version      1.1   date    03-11-1994 coefs and geom added
c     version      1.0   date    16-05-1994
c
c   copyright (c) 1994-1998  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     perform parallel domain decomposition
c
c **********************************************************************
c
c                       KEYWORDS
c
c     host
c     domain_decompostion
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ieq        , iadmin(*)  , isbndcon(*), iscoor(*)  ,
     +        iseddy(*)  , isgmat(*)  , isiinput(*), isinfbnd(*),
     +        isitopol   , ismatrix(*), isrhside(*), isrinput(*),
     +        issolut(*) , iscoefs(*),  isgeom(*)  , isdensup(*),
     +        nblocks, ndim

c     iadmin    i   reserved
c     ieq       i   Equation number
c     isbndcon  i   Array of sequence numbers (nblocks) of array bndcon
c     iscoefs   i   Array of sequence numbers (nblocks) of array coefs
c     iscoor    i   Array of sequence numbers (nblocks) of array coor
c     isdensup  i   Array of sequence numbers (nblocks) of array densup
c     iseddy    i   Array of sequence numbers (nblocks) of array eddy
c     isgeom    i   Array of sequence numbers (nblocks) of array geom
c     isgmat    i   Array of sequence numbers (nblocks) of array gmat
c     isiinput  i   Array of sequence numbers (nblocks) of array iinput
c     isinfbnd  i   Array of sequence numbers (nblocks) of array infbnd
c     isitopol  i   Sequence number of array itopol
c     ismatrix  i   Array of sequence numbers (nblocks) of array matrix
c     isrhside  i   Array of sequence numbers (nblocks) of array rhside
c     isrinput  i   Array of sequence numbers (nblocks) of array rinput
c     issolut   i   Array of sequence numbers (nblocks) of array solut
c     nblocks   i   Number of blocks
c     ndim      i   Dimension of space (2 or 3)
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisparal'
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ilev, ind, itrans, idummy, icoupled, iseqpres,
     +        ipiinput
      logical done

c     done      Indicates node domain decomposition should stop
c     icoupled  If 0 the momentum equations are coupled, if 1 they are not
c     idummy    Integer dummy
c     ilev      Time level of the multi-block transport of information
c     ind       Sequence number of node
c     ipiinput  Starting address of array iinput in ibuffr
c     iseqpres  Sequence number of pressure equation
c     itrans    reserved
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isingetp

c     ISACTIVE  Activate old permanent array
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISINGETP  get the pointer corresponding to an integer array
c     ISRCDDHS  Receives the multi-block information from each node
c               and place it at the correct location in the arrays
c               of virtual unknowns.
c     ISSDD     Sequential domain decomposition routine
c     ISSNDDHS  Copy the arrays of virtual unknowns for all
c               blocks to the local copies at the nodes, using PVM
c               if necessary.
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c      for all nodes not at the host do
c
c         send action information
c         send domain decomposition information
c
c      end do
c
c
c      perform sequential domain decomposition at the host
c
c
c      for all nodes not at the host do
c
c         receive domain decomposition information
c
c      end do
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen('ishdd')

      call isactive( isiinput(1) )
      ipiinput = isingetp(isiinput(1))
      ndim     = ibuffr(ipiinput-1+57)
      icoupled = ibuffr(ipiinput-1+76)
      if ( icoupled.eq.0 ) then
         iseqpres = 2
      else
         iseqpres = ndim + 1
      end if

      done = .false.

      do ind = 1, iacnodes

c     --- for all nodes not on the host

         if ( ieq.lt.iseqpres ) then
            itrans = ieq
         else
            itrans = ieq + ( ndim - 1 ) * ( 1 - icoupled )
         end if

         ilev = 2

         call issnddhs( nblocks, itrans, ilev, ind, isiinput,
     +                  issolut, isitopol, iadmin, done)
         if ( nerror.ne.0 ) go to 1000

      end do

      if ( nerror.ne.0 ) go to 1000
            
      call issdd ( ieq, nblocks , isiinput, isrinput,
     +             iscoor  , issolut , ismatrix, isgmat  ,
     +             isrhside, isitopol, iadmin, isinfbnd, isbndcon,
     +             isgeom, isdensup)

      do ind = 1, iacnodes

c     --- for all nodes not on the host
c        --- receive boundary info

         call isrcddhs( idummy, isiinput, issolut, isitopol )

         if ( nerror.ne.0 ) go to 1000

      end do

1000  call iserclos('ishdd')

      end
