      subroutine isincd2 ( iincnd, rincnd, solut, coor, ndegfd,
     +                     nvirtual, ni, nj, mshtyp )
c ======================================================================
c
c        programmer     Marcel Zijlema
c        version 3.1    date 08-09-1999 Guus: fill more virtual cells by
c                                       extrapolation
c        version 3.0    date 19-02-1999 Guus: new parameter list
c        version 2.8    date 27-07-1998 Duncan: new call to iscmrho2
c        version 2.7    date 05-03-1998 Duncan: extension for cavity
c        version 2.6    date 09-02-1998 Hester: indices corrected
c        version 2.5    date 03-12-1997 Kees K: computation of
c                                       covariant base vectors changed
c        version 2.4    date 12-11-1997 Hester: param. mscal added
c        version 2.3    date 05-11-1997 Charles: collocated allowed
c        version 2.2    date 10-10-1995 Guus: new call to iscmvel2
c        version 2.1    date 05-07-1995 Hester: cleaned up
c        version 2.0    date 30-05-1995 Hester: adjusted to incorporate
c                                            compressible flow.
c        version 1.1    date 01-03-1995 Kees K: copy timelevels
c        version 1.0    date 01-08-1994
c
c   copyright (c) 1994-1999  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c **********************************************************************
c
c                       DESCRIPTION
c
c     fill array solut from arrays iincnd and rincnd (2D)
c
c **********************************************************************
c
c                       KEYWORDS
c
c     initial_conditions
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      integer ni, nj, ndegfd, nvirtual, iincnd(1:ndegfd), mshtyp
      double precision rincnd(1:ndegfd),
     +                 coor(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,*),
     +                 solut(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1:ndegfd)

c     coor       i   reserved
c     iincnd     i   reserved
c     mshtyp     i   Type of grid arrangement
c     ndegfd     i   Number of degrees of freedom
c     ni         i   Number of non-virtual cells in the i-direction (one block)
c     nj         i   Number of non-virtual cells in the j-direction (one block)
c     nvirtual   i   Number of rows of virtual cells
c     rincnd     i   reserved
c     solut     i/o  reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c     none
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer  idegfd, lasti, lastj, ip, jp, j, choice, p, q
      double precision coord(1:2), acov(1:3,1:3), u(1:2)

c     acov      Covariant base vectors.
c               Meaning: a(i,j) gives the value of the i-th
c                        component of the j-th base vector
c     choice    General choice parameter
c     coord     reserved
c     idegfd    Loop variable with respect to a given unknown
c     ip        Local counting variable i-direction
c     j         General loop variable
c     jp        Local counting variable j-direction
c     lasti     Last index in i-direction
c     lastj     Last index in j-direction
c     p         Index in covariant base vectors
c     q         Index in covariant base vectors
c     u         Cartesian velocities
c **********************************************************************
c
c                       I/O
c
c     none
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      double precision usfuni

c     ISERCLOS      removes name of subroutine from error list
c     ISEROPEN      adds name of subroutine to error list
c     ISERRCHR      auxiliary subroutine to iserrsub
c     ISERRSUB      gives warning messages
c     USFUNI        user written subroutine to define the initial
c                   condition as a function of space
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     8      Reading from backing storage not yet available
c     9      Reading from restart file not yet available
c **********************************************************************
c
c                       PSEUDO CODE
c
c     For all solution vectors do
c
c       reset a given solution vector to zero
c
c       If solution = velocity then
c
c         compute the contravariant velocity components as follows:
c
c            p            p            (p)
c           V  = sqrt(g) U  = sqrt(g) a   . u
c                                     -     -
c
c         whereas
c
c                    (p)
c           sqrt(g) a     = a   x  a
c                   -       -(q)   -(r)
c
c         Hence, first compute the covariant base vectors, Cartesian
c         velocities from array rincnd or function USFUNI and then
c         the contravariant fluxes.
c
c       else if solution = scalar then
c
c         compute scalar from array rincnd or function USFUNI
c
c       end if
c
c       copy timelevel 1 to the other timelevels
c
c       If compressible then
c
c          compute initial density : eddy(*,*,2)
c
c          extrapolate density to virtual unknowns
c
c          compute initial velocity = initial flux/density
c
c       end if
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call iseropen('isincd2')

c     --- Determine all solution vectors

      do idegfd = 1, ndegfd

c     --- First reset a solution vector to zero

         lasti = ni + 1 + nvirtual
         lastj = nj + 1 + nvirtual

         do jp = 0, nj+1
            do ip = 0, ni+1
               solut( ip, jp, idegfd ) = 0d0
            end do
         end do
         do jp = 1-nvirtual, -1
            do ip = 1-nvirtual,ni+1+nvirtual
               solut( ip, jp, idegfd ) = 0d0
            end do
         end do
         do jp = nj+2, nj+1+nvirtual
            do ip = 1-nvirtual,ni+1+nvirtual
               solut( ip, jp, idegfd ) = 0d0
            end do
         end do
         do jp = 1-nvirtual, nj+1+nvirtual
            do ip = 1-nvirtual, -1
               solut( ip, jp, idegfd ) = 0d0
            end do
         end do
         do jp = 1-nvirtual, nj+1+nvirtual
            do ip = ni+2, ni+1+nvirtual
               solut( ip, jp, idegfd ) = 0d0
            end do
         end do

         if ( idegfd.le.2 ) then

            if ( mshtyp.eq.0 ) then
               if ( idegfd.eq.1 ) then
                  lasti = ni + 1
                  lastj = nj
               else if ( idegfd.eq.2 ) then
                  lasti = ni
                  lastj = nj + 1
               end if
            else
               lasti = ni
               lastj = nj
            end if

c           --- Compute covariant base vectors, coordinates needed for
c               computation of Cartesian velocities with function USFUNI
c               and then finally compute contravariant fluxes.

            do jp = 1, lastj
               do ip = 1, lasti

                  do j = 1, 2
                     coord(j) = 0.5d0*( coor(ip,jp,j)+
     +                          coor(ip+idegfd-1,jp+2-idegfd,j) )

                     if ( idegfd.eq.1 ) then
                        acov(j,1) = 2d-2 * (
     +                              (coor(ip+1,jp,j)-coor(ip,jp,j))+
     +                              (coor(ip+1,jp+1,j)-coor(ip,jp+1,j))+
     +                              (coor(ip,jp,j)-coor(ip-1,jp,j))+
     +                              (coor(ip,jp,j)-coor(ip-1,jp,j)) )
                        acov(j,2) = coor(ip,jp+1,j)-coor(ip,jp,j)
                        acov(j,3) = 0d0
                     else if ( idegfd.eq.2) then
                        acov(j,1) = coor(ip+1,jp,j)-coor(ip,jp,j)
                        acov(j,2) = 2d-2 * (
     +                              (coor(ip,jp+1,j)-coor(ip,jp,j))+
     +                              (coor(ip+1,jp+1,j)-coor(ip+1,jp,j))+
     +                              (coor(ip,jp,j)-coor(ip,jp-1,j))+
     +                              (coor(ip+1,jp,j)-coor(ip+1,jp-1,j)))
                        acov(j,3) = 0d0
                     else
                        acov(j,1) = 5d-1 *(
     +                              (coor(ip+1,jp,j)-coor(ip,jp,j))+
     +                              (coor(ip+1,jp+1,j)-coor(ip,jp+1,j)))

                        acov(j,2) = 5d-1 *(
     +                              (coor(ip,jp+1,j)-coor(ip,jp,j))+
     +                              (coor(ip+1,jp+1,j)-coor(ip+1,jp,j)))
                        acov(j,3) = 0d0
                     end if

                  end do

                  acov(3,1) = 0d0
                  acov(3,2) = 0d0
                  acov(3,3) = 1d0

                  do j = 1, 2
                     u(j) = 0d0

                     if ( iincnd(j).eq.-1 ) then

                        u(j) = rincnd(j)

                     else if ( iincnd(j).lt.-100 ) then

                        choice = abs( iincnd(j)+100 )
                        u(j) = usfuni(choice, coord(1), coord(2), 0d0 )

                     else if ( iincnd(j).gt.100 ) then

c                    --- Read solution vector idegfd from backing storage
c                            NOT YET AVAILABLE ***

                        call iserrchr('isincd2',1 )
                        call iserrsub ( 8, 0, 0, 1 )

                        go to 1000

                     else if ( iincnd(j).gt.0 .and.
     +                         iincnd(j).lt.100 ) then

c                    --- Read complete solution vector from the restart file
c                            NOT YET AVAILABLE ***

                        call iserrchr('isincd2',1 )
                        call iserrsub ( 9, 0, 0, 1 )

                        go to 1000

                     end if

                  end do

                  p = mod( idegfd+1,3 )
                  q = mod( idegfd+2,3 )
                  if ( p.eq.0 ) p = 3
                  if ( q.eq.0 ) q = 3

                  if ( mshtyp.eq.0 ) then
                     solut(ip,jp,idegfd) = (acov(2,p)*acov(3,q)-
     +                                      acov(3,p)*acov(2,q)) * u(1)
     +                                   + (acov(3,p)*acov(1,q)-
     +                                      acov(1,p)*acov(3,q)) * u(2)
                  else
                     solut(ip,jp,idegfd) = u(idegfd)
                  end if

               end do

            end do

         else

            if ( iincnd(idegfd).eq.-1 ) then

c           --- Fill solution vector idegfd with a constant

               do jp = 1-nvirtual, nj+1+nvirtual
                  do ip = 1-nvirtual, ni+1+nvirtual
                     solut(ip,jp,idegfd) = rincnd(idegfd)
                  end do
               end do

            else if ( iincnd(idegfd).lt.-100 ) then

c           --- Fill solution vector idegfd with function USFUNI

               choice = abs( iincnd(idegfd) + 100 )

               do jp = 1-nvirtual, nj+nvirtual
                  do ip = 1-nvirtual, ni+nvirtual

                     do j = 1, 2

                        coord(j) = 0.25d0*( coor(ip,jp,j) +
     +                                      coor(ip,jp+1,j) +
     +                                      coor(ip+1,jp,j) +
     +                                      coor(ip+1,jp+1,j) )

                     end do

                     solut(ip,jp,idegfd) = usfuni( choice, coord(1),
     +                                             coord(2), 0d0 )

                  end do

               end do
               jp = nj+nvirtual+1
               do ip = 1-nvirtual, ni+nvirtual
                  solut(ip,jp,idegfd) = solut(ip,jp-1,idegfd)
               end do
               ip = ni+nvirtual+1
               solut(ip,jp,idegfd) = solut(ip-1,jp,idegfd)
               do jp = 1-nvirtual, nj+nvirtual
                  solut(ip,jp,idegfd) = solut(ip-1,jp,idegfd)
               end do

            else if ( iincnd(j).gt.100 ) then

c           --- Read solution vector idegfd from backing storage
c                   NOT YET AVAILABLE ***

               call iserrchr('isincd2',1 )
               call iserrsub ( 8, 0, 0, 1 )

               go to 1000

            else if ( iincnd(j).gt.0 .and. iincnd(j).lt.100 ) then

c           --- Read complete solution vector from the restart file
c                   NOT YET AVAILABLE ***

               call iserrchr('isincd2',1 )
               call iserrsub ( 9, 0, 0, 1 )

               go to 1000

            end if

         end if

      end do

1000  call iserclos('isincd2')

      end
