      subroutine ismblk ( iblock, itrans, ilev, isiinput, istraq,
     +                    iadmin, isitopol)
c ======================================================================
c
c        programmer     Erik Brakkee
c        version 1.7    date 21-09-1999 Guus: allow ilev<0
c        version 1.6    date 13-01-1999 Guus: Layout
c        version 1.5    date 20-10-1997 Kees K:  call ismblk02 with isolv
c        version 1.4    date 28-02-1997 Kees K:  extension 3D
c        version 1.3    date 07-02-1997 Kees K:  lensmsg changed
c        version 1.2    date 12-12-1996 Kees K:  idiscret added.
c        version 1.1    date 22-11-1994 Erik:  extended for coefs.
c                                       changed solut into traq
c        version 1.0    date 07-04-1994 Erik:  initial revision
c
c   copyright (c) 1994-1999  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Transport certain variables (itrans,ilev) from block iblock
c     to the traq arrays of the neighbouring blocks on the same node.
c **********************************************************************
c
c                       KEYWORDS
c
c     multiblock
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iadmin(*), iblock, ilev, isiinput(*), isitopol,
     +        istraq(*), itrans

c     iadmin   i  reserved
c     iblock   i  Block sequence number
c     ilev     i  Time level of the multi-block transport of information
c     isiinput i  Array of sequence numbers (nblocks) of array iinput
c     isitopol i  Sequence number of array itopol
c     istraq   i  Array of sequence numbers (nblocks) of array traq
c     itrans   i  reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisparal'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, ipiinput, ipitopol, ipnsubf, ipsmsg, iptraq,
     +        ipsubf,  issmsg, lensmsg, maxnsubf,
     +        nblocks, ntraq1, nfaces, ni, nj, nk, ntraq2, nvirtual,
     +        ipiindsc, idiscret, isolv

c     i         General loop variable
c     idiscret  Type of discretization
c     ipiindsc  Starting address of array IINDSC in array IINPUT
c     ipiinput  Starting address of array iinput in ibuffr
c     ipitopol  Starting address of array itopol in ibuffr
c     ipnsubf   Starting address of the nsubfaces array
c     ipsmsg    Starting address of array smsg in buffer
c     ipsubf    Starting address of the subfaces array
c     iptraq    Starting address of array traq in buffer
c     isolv     Called by solver for periodic b.c. isolv=1 else isolv=0
c     issmsg    Sequence number of array smsg
c     lensmsg   Length of array smsg
c     maxnsubf  Maximum number of subfaces encountered on all subfaces
c     nblocks   Number of blocks
c     nfaces    Number of faces
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     ntraq1    Dimensioning parameter of the TRAQ array.
c               (ntraq1 = ndegfd for solut, ncoefs for coefs)
c     ntraq2    Dimensioning parameter of the TRAQ array.
c               (ntraq2 = ntimlv for solut, 1 for coefs)
c     nvirtual  Number of rows of virtual cells
c **********************************************************************
c
c                       I/O
c
c      none
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isdpgetp, isingetp

c     ISACTIVE  Activate permanent array
c     ISDCREAT  Create permanent space in array IBUFFR (Double precision array)
c     ISDCRPRI  Decrease priorities of all arrays
c     ISDELARR  Delete a permanent array
c     ISDPGETP  get the pointer corresponding to a double precision array
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISINGETP  get the pointer corresponding to an integer array
c     ISMBLK02  does all the work for this routine
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     1. compute some general parameters
c     2. compute an upper bound for a multi-block submessage (smsg)
c     3. activate and create arrays
c     4. compute pointers
c     5. actual multi-block transport (ismblk02)
c     6. delete array smsg
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c

      call iseropen('ismblk')

c     --- compute nblocks, nfaces and maxnsubfaces

      call isactive(isitopol)
      ipitopol = isingetp(isitopol)
      nblocks  = ibuffr(ipitopol + 1 - 1)
      nfaces   = ibuffr(ipitopol + 2 - 1)
      maxnsubf = ibuffr(ipitopol + 3 - 1)

c     --- compute an upper bound for the length of a multi-block submessage

      call isactive(isiinput(iblock))
      ipiinput = isingetp(isiinput(iblock))
      ni       = ibuffr(ipiinput + 51 - 1)
      nj       = ibuffr(ipiinput + 52 - 1)
      nk       = ibuffr(ipiinput + 53 - 1)
      ipiindsc = ibuffr(ipiinput-1+6)
      idiscret = ibuffr(ipiinput-1+ipiindsc+1)

c     --- if ilev<0, then isdiscret is set equal to 10 indicating that
c         nvirtual rows of virtual cells must be copied

      if ( ilev.lt.0 ) idiscret = 10

      if ( itrans.lt.0 ) then

c     --- coefs array: ntraq1 = ncoefs, ntraq2 = 0

         ntraq1 = ibuffr(ipiinput + 66 - 1)
         ntraq2 = 1

      else

c     --- solut array, ntraq1 = ndegfd, ntraq2 = ntimlv

         ntraq1 = ibuffr(ipiinput + 55 - 1)
         ntraq2 = ibuffr(ipiinput + 64 - 1)

      end if
      nvirtual = ibuffr(ipiinput + 54 - 1)
      if ( nk.eq.0 ) then
         lensmsg = 4*(max(ni,nj)+1+2*nvirtual)
      else
         lensmsg = 6*(max(ni,nj,nk)+1+2*nvirtual)*
     +              (max(ni,nj,nk)+1+2*nvirtual)
      end if

c     --- activate and create arrays

      issmsg = 0
      call isdcreat(issmsg, lensmsg, 'smsg' )
      do i = 1, nblocks
         if ( inodenr.eq.0 .or. iadmin(i).eq.inodenr ) then
            call isactive(isiinput(i))
            call isactive(istraq(i))
         end if
      end do
      call isactive(isitopol)

c     --- compute pointers

      ipsmsg   = isdpgetp(issmsg)
      iptraq   = isdpgetp(istraq(iblock))
      ipitopol = isingetp(isitopol)
      ipnsubf  = ibuffr(ipitopol+5-1) + ipitopol - 1
      ipsubf   = ibuffr(ipitopol+6-1) + ipitopol - 1

c     --- indicating that ismblk02 is not called in the linear solver

      isolv = 0

      call ismblk02 ( iblock, itrans, abs(ilev), ni, nj, nk,ntraq1,
     +                ntraq2, nvirtual, nblocks,  nfaces, maxnsubf,
     +                idiscret, isolv, buffer(iptraq), isiinput, istraq,
     +                iadmin, ibuffr(ipnsubf), ibuffr(ipsubf),
     +                buffer(ipsmsg) )

      call isdelarr(issmsg)
      call isdcrpri

      call iserclos('ismblk')

      end
