      subroutine istemperat01 ( isiinput, isrinput, issolut, iseddy )
c ======================================================================
c
c     programmer  Guus Segal
c     version      1.1    date 23-09-1999 Guus: delete array work
c     version      1.0    date 01-07-1999
c
c   copyright (c) 1999-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Compute the temperature as function of the internal energy
c
c **********************************************************************
c
c                       KEYWORDS
c
c     compressible
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer isiinput, isrinput, issolut, iseddy

c     iseddy         i    Array of sequence numbers (nblocks) of array eddy
c     isiinput       i    Array of sequence numbers (nblocks) of array iinput
c     isrinput       i    Array of sequence numbers (nblocks) of array rinput
c     issolut        i    Array of sequence numbers (nblocks) of array solut
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ipiinput,iprinput, ipiincom, iprincom, ipsolut,
     +        ipeddy, ni, nj, ndegfd, ndim, nvirtual, npointsv,
     +        iswork, meqstate, nk

c     ipeddy         Starting address of array eddy in buffer
c     ipiincom       Starting address of array IINCOM in array IINPUT
c     ipiinput       Starting address of array iinput in ibuffr
c     iprincom       Starting address of array RINCOM in array RINPUT
c     iprinput       Starting address of array rinput in buffer
c     ipsolut        Starting address of array solut in buffer
c     iswork         Sequence number of array work
c     meqstate       Defines the type of equation of state
c                    See the programmers Guide array IINCOM for the
c                    possible values
c     ndegfd         Number of degrees of freedom
c     ndim           Dimension of space (2 or 3)
c     ni             Number of non-virtual cells in the i-direction
c                    (one block)
c     nj             Number of non-virtual cells in the j-direction
c                    (one block)
c     nk             Number of non-virtual cells in the k-direction
c                    (one block)
c     npointsv       Number of points in the mesh with virtual points
c     nvirtual       Number of rows of virtual cells
c **********************************************************************
c
c                       I/O
c
c     none
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isingetp, isdpgetp

c     ISACTIVE       Activate old permanent array
c     ISDCREAT       Create permanent space in array IBUFFR (Double precision ar
c     ISDPGETP       get the pointer corresponding to a double precision array
c     ISERCLOS       Resets old name of previous subroutine of higher level
c     ISEROPEN       Produces concatenated name of local subroutine
c     ISINGETP       get the pointer corresponding to an integer array
c     USCOMPTEMP     user subroutine to compute the temperature
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     none
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call iseropen('istemperat01')

      call isactive(isiinput)
      call isactive(isrinput)
      call isactive(issolut)
      call isactive(iseddy)

      ipiinput = isingetp(isiinput)

      ni       = ibuffr(ipiinput + 51 - 1)
      nj       = ibuffr(ipiinput + 52 - 1)
      nk       = 0
      nvirtual = ibuffr(ipiinput + 54 - 1)
      npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)

      iswork   = 0
      call isdcreat ( iswork, npointsv, 'work' )
      ipiinput = isingetp(isiinput)
      iprinput = isdpgetp(isrinput)
      ipsolut  = isdpgetp(issolut )
      ipeddy   = isdpgetp(iseddy  )
      ipiincom = ibuffr(ipiinput +  9 - 1)
      iprincom = ibuffr(ipiinput + 34 - 1)

      ndegfd   = ibuffr(ipiinput + 55 - 1)
      ndim     = ibuffr(ipiinput + 57 - 1)
      meqstate = ibuffr(ipiinput-1 + ipiincom+9-1)

      if ( ndim.eq.2 ) then

c     --- compute density from equation of state

        if ( meqstate.eq.0 ) then

c       --- meqstate = 0, perfect gas, T = e/cv

        else

c       --- meqstate = 1, user provided

           call uscomptemp ( buffer(ipeddy), buffer(ipsolut), ni, nj,
     +                       nk, nvirtual, ndim, ndegfd,
     +                       buffer(iprinput-1+iprincom) )

         end if

      end if
      call isdelarr ( iswork )

      call iserclos('istemperat01')

      end
