      subroutine isthta ( ndim, ndegfd, npointsv, ntimlv, solut, theta )
c ======================================================================
c
c     programmer   Kees Kassels
c
c     version      1.3  09-06-1998 Charles: nvpoints changed to npointsv
c     version      1.2  21-10-1994  M. Kooper Theta method in New version
c     version      1.1  29-07-1992  Zdenek: extension for 3D,
c     version      1.01 ISNAS     date    06-05-1992
c     version      1.0  ISNAS     date    08-11-1991
c     computer     HP 9000/735
c
c   copyright (c) 1993  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     This subroutine extrapolates the  solution array according to
c     the Theta - method
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      integer ndim, npointsv, ndegfd, ntimlv
      double precision solut(1:npointsv,1:ndegfd,1:ntimlv), theta

c     ndegfd     i    Number of degrees of freedom
c     ndim       i    Dimension of space (2 or 3)
c     npointsv   i    Number of points in the mesh with virtual points
c     ntimlv     i    Number of time levels
c     solut     i/o   reserved
c     theta      i    Parameter theta for theta method
c **********************************************************************
c
c                       COMMON BLOCKS
c
c      none
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer idegfd, ipointl, itimlv

c     idegfd    Loop variable with respect to a given unknown
c     ipointl   Loop variable
c     itimlv    Loop variable with respect to a given time level
c **********************************************************************
c
c                       I/O
c
c      none
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     none
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     none
c **********************************************************************
c
c                       PSEUDO CODE
c
c              n+1        1          n+theta                   n
c             V       =  --- * (  V        - ( 1 - theta) * V )
c              m        theta
c
c              n+1        1          n+theta                   n
c             T       =  --- * (  T        - ( 1 - theta) * T )
c              m        theta
c
c              n+1       n+theta      1               n+theta     n
c             P       = P        + ( --- - 1 ) * (  P        -  P )
c              m                    theta
c
c              n+1   n    1          n+theta     n
c             P   - P =  --- * (  P        -  P )
c              m     m  theta
c
c
c
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c

c     --- handle all quantities except pressure

      do itimlv = ntimlv, 2, -1
         do idegfd = 1, ndim
            do ipointl = 1, npointsv
               solut(ipointl, idegfd, itimlv) = ( 1d0/theta ) *
     +              ( solut(ipointl, idegfd, itimlv)          -
     +              ( 1d0 - theta)*solut(ipointl,idegfd,itimlv-1) )
            end do
         end do
         do idegfd = ndim+2, ndegfd
            do ipointl = 1, npointsv
               solut(ipointl, idegfd, itimlv) = ( 1d0/theta ) *
     +              ( solut(ipointl, idegfd, itimlv)          -
     +              ( 1d0 - theta)*solut(ipointl,idegfd,itimlv-1) )
            end do
         end do
      end do

c     --- handle the pressure
c         at level NTIMLV   Delta P is stored,
c         at level NTIMLV-1 P^(n+theta) is stored

      idegfd = ndim+1
      do itimlv = 1, ntimlv-1
         do ipointl = 1, npointsv
            solut(ipointl, idegfd, itimlv  ) =
     +      solut(ipointl, idegfd, itimlv  ) + ( 1d0/theta - 1d0 ) *
     +      solut(ipointl, idegfd, itimlv+1)
         end do
      end do
      do ipointl = 1, npointsv
         solut(ipointl, idegfd, ntimlv ) = ( 1d0/theta ) *
     +   solut(ipointl, idegfd, ntimlv )
      end do

      end
