      subroutine istuvsc3 ( mturb, ndegfd  , ntimlv, rintur, ni, nj,
     +                      nk, nvirtual, solut , eddy, work, lwork,
     +                      coor, geom, ichois,idiscret, npoint,igeom,
     +                      molvis, rho, ntrnsp, nturb, iinbc, rinbc,
     +                      itimeact, averstr, icount, nwinsta, nwinend,
     +                      timewind, nwindowmx, deltat)
c ======================================================================
c
c        programmer     Marcel Zijlema
c        version 2.1    date   24-08-1998  Kees K: work added in call
c                                          of issmagor
c        version 2.0    date   19-08-1997  Fred T: Smagorinsky model
c        version 1.1    date   24-05-1996  Marcel extended k-eps model
c        version 1.0    date   29-11-1994
c
c
c   copyright (c) 1994-1997  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Computes eddy viscosity, which depends on the choice
c     of a turbulence model (3D)
c **********************************************************************
c
c                       KEYWORDS
c
c     turbulence
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer mturb, ndegfd, ntimlv, nvirtual, ni, nj, nk, lwork,
     +ichois,idiscret, npoint, igeom(*),ntrnsp,nturb,iinbc(*),itimeact,
     + icount, nwinsta, nwindowmx, nwinend
      double precision rintur(*), work(*), coor(*), geom(*), molvis,
     +                 eddy(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual),
     +                 solut(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1-nvirtual:nk+1+nvirtual,
     +                       1:ndegfd, 1:ntimlv), rho,
     +                       rinbc(*), averstr(*), timewind, deltat

c     averstr   i    reserved
c     coor      i    reserved
c     deltat    i    Product of theta and dt
c     eddy      o    reserved
c     geom      i    reserved
c     ichois    i    General choice parameter
c     icount    i    Counts the number of positions necessary for the number
c     idiscret  i    Type of discretization
c     igeom     i    Integer array that returns the closest cell on a wall
c     iinbc     i    reserved
c     itimeact  i    Actual time step
c     lwork     i    Length of array work
c     molvis    i    Molecular viscosity
c     mturb     i    Indicates type of turbulence modelling
c     ndegfd    i    Number of degrees of freedom
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     npoint    i    Number of points for one block
c     ntimlv    i    Number of time levels
c     ntrnsp    i    Number of transport equations
c     nturb     i    Number of extra transport equations for turbulence
c     nvirtual  i    Number of rows of virtual cells
c     nwindowmx i    Maximum size of the window to be used for time-averaging
c     nwinend   i    End position of time window
c     nwinsta   i    Starting position of time window
c     rho       i    Estimate of the asymptotic convergence rate
c     rinbc     i    reserved
c     rintur    i    reserved
c     solut     i    reserved
c     timewind  i    Time interval for time window
c     work           Work array
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ip, jp, kp, rspv(1:3,1:8,0:3,0:4,1:8),
     +        npf(0:3,0:4,1:8), ipcntdef, ipcva, ipjcta,
     +        ipjctg, ipchsym, ippderiv, ipveloc, nwork,
     +        i,pnttyp, ipjac, ipinvjac, ipwdis, alphad,
     +        betad, idir, jdir, vcellc
      double precision cv(1:8,0:3,0:4,1:8), ksign

c     alphad    Direction index of integration.
c               Hence if a V1-cell is used, alphad = 1 and so on
c     betad     Direction index, usually the direction perpendicular to the
c               integration direction of the boundary integral.
c               For example betad in local point (1,0) is 1 and in (0,1): 2
c     cv        reserved
c     i         General loop variable
c     idir      General loop variable for 1...ndim loop
c     ip        Local counting variable i-direction
c     ipchsym   Starting address of array CHSYM in array WORK
c     ipcntdef  Starting address of array CNTDEF in array WORK
c     ipcva     Starting address of array CVA in array WORK
c     ipinvjac  Starting address of array invjac in geom
c     ipjac     Starting address of array jac in geom
c     ipjcta    Starting address of array JCTA in array WORK
c     ipjctg    Starting address of array JCTG in array WORK
c     ippderiv  Starting address of array PDERIV in array WORK
c     ipveloc   Starting address of array VELOC in array WORK
c     ipwdis    Starting address of array wdis in array geom
c     jdir      General loop variable for 1...ndim loop
c     jp        Local counting variable j-direction
c     kp        Local counting variable k-direction
c     ksign     Help parameter to define 1 or -1
c     npf       reserved
c     nwork     Second dimension of array work
c     pnttyp    reserved
c     rspv      reserved
c     vcellc    Point type of integration cell centre for a given
c               variable type
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISCHRYS3  Compute the chrystoffel symbols in integration point
c               three-dimensional case
c     ISCOVA3D  Compute covariant base vectors in all points of the grid
c               three-dimensional case
c     ISDOUB3D  Compute the double contraction of rate-of-deformation tensor or
c               rotation tensor
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRCHR  Puts values of strings in character error array
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c     ISINTCF3  Compute interpolation of a coefficient in one integration
c               point in R^3
c     ISJCTA3D  Compute sqrt(g) x contra variant base vectors in integration
c               point three-dimensional case
c     ISJCTG3D  Compute sqrt(g) x contra variant metric tensors in integration
c               point three-dimensional case
c     ISPNDF    Make tables for interpolation and finite difference formulas
c     ISSMAGOR  Computes eddy viscosity, for the smagorinsky model
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c      17   internal error: array size too large
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call iseropen('istuvsc3')

      if ( mturb.eq.0 ) then

c     --- No turbulence model

        do kp = 1, nk
          do jp = 1, nj
            do ip = 1, ni
              eddy(ip,jp,kp) = 0d0
            end do
          end do
        end do

      else if ( mturb.eq.1 .or. mturb.eq.2 .or. mturb.eq.5 ) then

c     --- Standard high-Re k-eps model or RNG k-eps model or
c         extended k-eps model

        do kp = 1, nk
          do jp = 1, nj
            do ip = 1, ni
              eddy(ip,jp,kp) = rintur(3)*solut(ip,jp,kp,ndegfd-1,1)*
     +                         solut(ip,jp,kp,ndegfd-1,1) /
     +                         solut(ip,jp,kp,ndegfd,1)
            end do
          end do
        end do

      else if ( mturb.eq.3 ) then

c     --- Wilcox k-omega model

        do kp = 1, nk
          do jp = 1, nj
            do ip = 1, ni
              eddy(ip,jp,kp) = solut(ip,jp,kp,ndegfd-1,1) /
     +                         solut(ip,jp,kp,ndegfd,1)
            end do
          end do
        end do

      else if ( mturb.eq.4 ) then

c     --- LES Smargorinsky model.

         if (ichois.eq.1) then

c        --- construct finite difference formulas

            call ispndf(npf,rspv,cv,3)

c           --- allocate local arrays

            ipcntdef    = 1
            ipcva       = ipcntdef +  1 * npoint
            ipjcta      = ipcva    + 72 * npoint
            ipjctg      = ipjcta   +  9 * npoint
            ipchsym     = ipjctg   +  6 * npoint
            ippderiv    = ipchsym  + 18 * npoint
            ipveloc     = ippderiv +  9 * npoint
            nwork       = ipveloc  +  12* npoint

c           --- compute some pointers

            ipjac    = 1
            ipinvjac = ipjac    +   8 * npoint
            ipwdis   = ipinvjac +   8 * npoint

            if (nwork.gt.lwork) then
               call iserrchr ( 'work' , 1 )
               call iserrint ( nwork  , 1 )
               call iserrint ( lwork  , 2 )
               call iserrsub ( 17 , 2 , 0 , 1 )
            end if

c           --- initialize working array

            do i = 1 , nwork
               work ( i ) = 0d0
            end do

c           --- fill covariant basis vectors in cva for all integration pts

            call iscova3d (coor, work(ipcva), ni, nj, nk, nvirtual,
     +                     npf, rspv, cv )

c           --- Compute the contraction of the deformation rate tensor
c               cell center

            pnttyp = 8

c           --- Compute contravariant b.v, metric tensors and Christof. symbols

            call isjcta3d ( work(ipcva+9*npoint*(pnttyp-1)),
     +                      work(ipjcta), ni, nj, nk, nvirtual )

            call isjctg3d ( geom(ipinvjac+npoint*(pnttyp-1)),
     +                      work(ipjcta),
     +                      work(ipjctg), ni, nj, nk, nvirtual )

            call ischrys3 ( work(ipcva), work(ipjcta), work(ipchsym),
     +                      geom(ipinvjac+npoint*(pnttyp-1)), ni, nj,
     +                      nk, nvirtual, pnttyp )

c           --- alpha
c               Compute spatial derivative of V      in direction beta

            do alphad = 1 , 3
               do betad = 1 , 3

                  call isintcf3(
     +              solut(1-nvirtual,1-nvirtual,1-nvirtual, alphad, 2),
     +              work (ippderiv+npoint*(alphad-1+3*(betad-1)))     ,
     +              ni, nj, nk, nvirtual, npf, rspv, cv, betad, alphad,
     +              pnttyp     )

               end do
            end do

c           --- Interpolate velocities to center of cell faces and pnttyp

            do jdir = 1 , 3
               vcellc = 8 - 2**(jdir-1)
               do idir = 1 , 3
                  call isintcf3(
     +                solut(1-nvirtual,1-nvirtual,1-nvirtual, idir, 2),
     +                work (ipveloc+npoint*(idir-1+3*(jdir-1)))     ,
     +                ni, nj, nk, nvirtual, npf, rspv, cv, 0, idir,
     +                vcellc     )

               end do

               call isintcf3(
     +               solut(1-nvirtual,1-nvirtual,1-nvirtual, jdir, 2),
     +               work (ipveloc+npoint*(jdir+8)), ni, nj, nk,
     +               nvirtual, npf,rspv,cv, 0, jdir, pnttyp )

            end do

c           --- determine the double contraction of the deformation rate tensor

            ksign = 1

            call isdoub3d(work(ipcntdef), work(ippderiv),
     +                    work(ipveloc), solut, work(ipcva),
     +                    geom(ipinvjac), work(ipjcta),
     +                    work(ipjctg), work(ipchsym), ni, nj,
     +                    nk, ndegfd, nvirtual, idiscret, pnttyp,
     +                    ksign )

c           --- calculate the covariant metric tensor at cell center
c               WARNING : using the routine for contravariant metric tensor
c                     so the output is store at the same place and the
c                     actual value is g     /  Jacobian
c                                      (ij)

            call isjctg3d ( geom(ipinvjac+npoint*(pnttyp-1)),
     +                      work(ipcva+9*npoint*(pnttyp-1)),
     +                      work(ipjctg), ni, nj, nk, nvirtual )

c           --- calculate the eddy viscosity

            call issmagor(eddy,rintur(14),geom(ipjac+npoint*(pnttyp-1)),
     +                   geom(ipwdis), work(ipcntdef) ,
     +                   ni, nj, nk, nvirtual, igeom, averstr,
     +                   geom(ipinvjac+npoint*(pnttyp-1)),
     +                   work(ipveloc+npoint*9), work(ipjctg), molvis,
     +                   rho, ntrnsp, nturb, iinbc,rinbc, work, icount,
     +                   nwinsta, nwinend, timewind, nwindowmx, deltat,
     +                   itimeact  )

         end if

      end if

      call iserclos('istuvsc3')

      end
