      subroutine isbcs2 ( alphad, ni, nj, nvirtual,  prems, rhside,
     +                    locdx, ibinfor, ibipoint, bndcon, bcgeo )
c ======================================================================
c
c        programmer     Kees Kassels
c        version 1.4    date 02-12-1998 Guus: iprec removed
c        version 1.3    date 09-12-1997 change for periodic
c        version 1.2    date 06-01-1997 multiblock
c        version 1.1    date 15-10-1996 only for dirichlet-dirichlet
c        version 1.0    date 03-09-1996
c
c   copyright (c) 1996-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c      Wesbeek discretization stress term on the boundary,
c      tangential cell. If dirichlet-dirichlet
c      boundary condition then put undistributed value in the
c      integration point on the boundary directly in right-hand side.
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, ibipoint(*), ibinfor(*), alphad,
     +        locdx(2)
      double precision rhside(ni+1,nj+1,2),
     +                 prems(ni+1,nj+1,2,4), bndcon(2,*), bcgeo(8,*)

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     bcgeo     i    reserved
c     bndcon    i    reserved
c     ibinfor   i    Third part of infbnd
c     ibipoint  i    ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     locdx     i    Position of the integration point with respect to
c                    the centre of the integration cell
c                    The cell sizes are assumed to be 2 times 2, hence the
c                    distance of a midside point to a cell centre is always 1
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     prems    i/o   Intermediate matrix for building the stress term according
c                    to the Wesbeek discretization.
c     rhside   i/o   reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer unut, uxuy, parabolic
      parameter( unut = 4, uxuy = 10, parabolic= 12 )
      integer ipibinf, ipoint, ip, jp, xp(2), iside, iparea(2,2),tdir,
     +        ndir, nsign, tsign, ipnt, istwo(2,2), btype
      double precision at1, at2, an1, an2, gttdivsg, gtt, gnn,
     +                 fac11, fac12, fac21, fac22, det, un, ut
      double precision u1, u2

c     an1       Component 1 of contravariant base vector in normal direction
c     an2       Component 2 of contravariant base vector in normal direction
c     at1       Component 1 of covariant base vector in tangential direction
c     at2       Component 2 of covariant base vector in tangential direction
c     btype     Type of boundary condition
c     det       Determinant
c     fac11     Component of 2x2 matrix
c     fac12     Component of 2x2 matrix
c     fac21     Component of 2x2 matrix
c     fac22     Component of 2x2 matrix
c     gnn       Contravariant metric tensor
c     gtt       Contravariant metric tensor in tt direction
c     gttdivsg  Contravariant metric tensor in tt direction divided by sqrt(g)
c     ip        Local counting variable i-direction
c     iparea    Area of integration points where a contribution to
c               the matrix is to be calculated
c     ipibinf   Starting address of array infor for present side minus one
c     ipnt      Integration point number
c     ipoint    Address in array IBUFFR
c     iside     Sequence number of side
c     istwo     Translation table to store and retrieve the
c               elements of a symmetric ndim-dimensional
c               rank 2 tensor from a vector with length
c               (ndim*(ndim+1))/2
c     jp        Local counting variable j-direction
c     ndir      Direction of normal vector on this side
c     nsign     Inner product of outward normal in computational domain
c               with the base vector perpendicular to the face
c     parabolic Parabolic boundary conditions
c     tdir      Direction of tangent vector on this side
c     tsign     Inner product of tangential vector defined counter
c               clockwise with respect to the outward normal and the
c               base vector along the side
c     u1        Cartesian velocity component
c     u2        Cartesian velocity component
c     un        Normal velocity (physical)
c     unut      U_n and u_t prescribed
c     ut        Tangential velocity (physical)
c     uxuy      U_x and u_y prescribed
c     xp        Auxiliary array to calculate boundary point number
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data istwo / 1, 2,
     +             2, 3 /
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- loop over the normal cells at a distance 1
c         of the boundary

      iparea(1,1) = 1
      iparea(2,1) = -1
      iparea(1,2) = 1
      iparea(2,2) = -1
      iside = 0

      if ( alphad.eq.1 ) then

         if ( locdx(1).ne.0 ) then

            tdir = 2
            ndir = 1
            if ( locdx(1).eq.-1 ) then
               iside = 1
               nsign = -1
               tsign = -1
               iparea(1,1) = 2
               iparea(2,1) = 2
               iparea(1,2) = 1
               iparea(2,2) = nj
               ipnt = 1
            else
               iside = 2
               nsign = 1
               tsign = 1
               iparea(1,1) = ni
               iparea(2,1) = ni
               iparea(1,2) = 1
               iparea(2,2) = nj
               ipnt = 2
            end if

         end if

      else

         if ( locdx(2).ne.0 ) then

            tdir = 1
            ndir = 2
            if ( locdx(2).eq.-1 ) then
               iside = 3
               nsign = -1
               tsign = 1
               iparea(1,1) = 1
               iparea(2,1) = ni
               iparea(1,2) = 2
               iparea(2,2) = 2
               ipnt = 1
            else
               iside = 4
               nsign = 1
               tsign = -1
               iparea(1,1) = 1
               iparea(2,1) = ni
               iparea(1,2) = nj
               iparea(2,2) = nj
               ipnt = 2
            end if

         end if

      end if

      ipibinf = ibipoint(iside)+nvirtual-1

      do  jp = iparea(1,2), iparea(2,2)
         do  ip = iparea(1,1), iparea(2,1)

            xp(1) = ip
            xp(2) = jp
            ipoint = ipibinf+2*xp(tdir)

            btype = ibinfor(ipoint)

            if ( btype.eq.uxuy .or. btype.eq.unut .or.
     +           btype.eq.parabolic .or. btype.eq.88 ) then

               if ( btype.ne.uxuy ) then

                  if( tdir.eq.1 ) then
                     at1 = bcgeo(7,ipoint)
                     at2 = -bcgeo(6,ipoint)
                  else
                     at1 = -bcgeo(5,ipoint)
                     at2 = bcgeo(4,ipoint)
                  end if
                  an1 = bcgeo((2+2*ndir),ipoint)/bcgeo(8,ipoint)
                  an2 = bcgeo((3+2*ndir),ipoint)/bcgeo(8,ipoint)
                  gttdivsg = bcgeo(istwo(ndir,ndir),ipoint)
                  gnn = gttdivsg/bcgeo(8,ipoint)
                  gtt = bcgeo(8,ipoint)*gttdivsg
                  fac11 = at1/sqrt(gtt)
                  fac12 = at2/sqrt(gtt)
                  fac21 = an1/sqrt(gnn)
                  fac22 = an2/sqrt(gnn)
                  det = fac11*fac22-fac12*fac21
                  un = bndcon(1,ipoint)*nsign
                  ut = bndcon(2,ipoint)*tsign
                  u1 = (fac22*ut -fac12*un)/det
                  u2 = (-fac21*ut +fac11*un)/det
               else
                  u1 = bndcon(1,ipoint)
                  u2 = bndcon(2,ipoint)
               end if

               rhside(ip,jp,alphad ) = rhside(ip,jp,alphad ) -
     +            (  prems(ip,jp,1,ipnt) * u1 +
     +               prems(ip,jp,2,ipnt) * u2 )
               prems(ip,jp,1,ipnt) = 0d0
               prems(ip,jp,2,ipnt) = 0d0

            end if

         end do
      end do

c     --- loop over tangential cells at the boundary
c         Compute loop bounds

      iparea(1,1) = 1
      iparea(2,1) = -1
      iparea(1,2) = 1
      iparea(2,2) = -1

      if ( alphad.eq.1 ) then

c     --- iside 3 and 4

         do iside = 3, 4

            tdir = 1
            ndir = 2
            if ( iside.eq.3 ) then
               nsign = -1
               tsign = 1

               ipibinf = ibipoint(1)+nvirtual-1
               ipoint  = ipibinf+1
               btype = ibinfor(ipoint)
               if ( btype.ge.89 .and. btype.le.99 ) then
                  iparea(1,1) = 1
               else
                  iparea(1,1) = 2
               end if
               ipibinf = ibipoint(2)+nvirtual-1
               ipoint  = ipibinf+1
               btype = ibinfor(ipoint)
               if ( btype.ge.89 .and. btype.le.99 ) then
                  iparea(2,1) = ni+1
               else
                  iparea(2,1) = ni
               end if

               iparea(1,2) = 1
               iparea(2,2) = 1
               ipnt = 3
            else
               nsign = 1
               tsign = -1

               ipibinf = ibipoint(1)+nvirtual-1
               ipoint  = ipibinf+1
               btype = ibinfor(ipoint)
               if ( btype.ge.89 .and. btype.le.99 ) then
                  iparea(1,1) = 1
               else
                  iparea(1,1) = 2
               end if
               ipibinf = ibipoint(2)+nvirtual-1
               ipoint  = ipibinf+1
               btype = ibinfor(ipoint)
               if ( btype.ge.89 .and. btype.le.99 ) then
                  iparea(2,1) = ni+1
               else
                  iparea(2,1) = ni
               end if

               iparea(1,2) = nj
               iparea(2,2) = nj
               ipnt = 4
            end if

            if ( locdx(1).ne.0 ) then

               ipibinf = ibipoint(iside)+nvirtual-1

               do  jp = iparea(1,2), iparea(2,2)
                  do  ip = iparea(1,1), iparea(2,1)

                     xp(1) = ip
                     xp(2) = jp
                     if ( locdx(1).eq.-1 ) then
                        ipoint = ipibinf+2*(xp(tdir)-1)
                     else
                        ipoint = ipibinf+2*xp(tdir)
                     end if

                     btype = ibinfor(ipoint)

                     if ( btype.eq.uxuy .or. btype.eq.unut .or.
     +                    btype.eq.parabolic .or. btype.eq.88 ) then

                        if ( btype.ne.uxuy ) then
                           if( tdir.eq.1 ) then
                              at1 = bcgeo(7,ipoint)
                              at2 = -bcgeo(6,ipoint)
                           else
                              at1 = -bcgeo(5,ipoint)
                              at2 = bcgeo(4,ipoint)
                           end if
                           an1 = bcgeo((2+2*ndir),ipoint)/
     +                           bcgeo(8,ipoint)
                           an2 = bcgeo((3+2*ndir),ipoint)/
     +                           bcgeo(8,ipoint)
                           gttdivsg = bcgeo(istwo(ndir,ndir),ipoint)
                           gnn = gttdivsg/bcgeo(8,ipoint)
                           gtt = bcgeo(8,ipoint)*gttdivsg
                           fac11 = at1/sqrt(gtt)
                           fac12 = at2/sqrt(gtt)
                           fac21 = an1/sqrt(gnn)
                           fac22 = an2/sqrt(gnn)
                           det = fac11*fac22-fac12*fac21
                           un = bndcon(1,ipoint)*nsign
                           ut = bndcon(2,ipoint)*tsign
                           u1 = (fac22*ut -fac12*un)/det
                           u2 = (-fac21*ut +fac11*un)/det
                        else
                           u1 = bndcon(1,ipoint)
                           u2 = bndcon(2,ipoint)
                        end if

                        rhside(ip,jp,alphad ) = rhside(ip,jp,alphad ) -
     +                 (  prems(ip,jp,1,ipnt) * u1 +
     +                    prems(ip,jp,2,ipnt) * u2 )
                          prems(ip,jp,1,ipnt) = 0d0
                          prems(ip,jp,2,ipnt) = 0d0

                     end if

                     btype = ibinfor(ipoint)

                  end do
               end do

            end if

            if ( (iside.eq.3 .and. locdx(2).eq.-1) .or.
     +           (iside.eq.4 .and. locdx(2).eq. 1) ) then

               ipibinf = ibipoint(iside)+nvirtual-1

               do  jp = iparea(1,2), iparea(2,2)
                  do  ip = iparea(1,1), iparea(2,1)

                     xp(1) = ip
                     xp(2) = jp
                     ipoint = ipibinf+2*xp(tdir) - 1

                     btype = ibinfor(ipoint)

                     if ( btype.eq.uxuy .or. btype.eq.unut .or.
     +                    btype.eq.parabolic .or. btype.eq.88 ) then

                        if ( btype.ne.uxuy ) then
                           if( tdir.eq.1 ) then
                              at1 = bcgeo(7,ipoint)
                              at2 = -bcgeo(6,ipoint)
                           else
                              at1 = -bcgeo(5,ipoint)
                              at2 = bcgeo(4,ipoint)
                           end if
                           an1 = bcgeo((2+2*ndir),ipoint)/
     +                           bcgeo(8,ipoint)
                           an2 = bcgeo((3+2*ndir),ipoint)/
     +                           bcgeo(8,ipoint)
                           gttdivsg = bcgeo(istwo(ndir,ndir),ipoint)
                           gnn = gttdivsg/bcgeo(8,ipoint)
                           gtt = bcgeo(8,ipoint)*gttdivsg
                           fac11 = at1/sqrt(gtt)
                           fac12 = at2/sqrt(gtt)
                           fac21 = an1/sqrt(gnn)
                           fac22 = an2/sqrt(gnn)
                           det = fac11*fac22-fac12*fac21
                           un = bndcon(1,ipoint)*nsign
                           ut = bndcon(2,ipoint)*tsign
                           u1 = (fac22*ut -fac12*un)/det
                           u2 = (-fac21*ut +fac11*un)/det
                        else
                           u1 = bndcon(1,ipoint)
                           u2 = bndcon(2,ipoint)
                        end if

                        rhside(ip,jp,alphad ) = rhside(ip,jp,alphad ) -
     +                  (  prems(ip,jp,1,ipnt) * u1 +
     +                     prems(ip,jp,2,ipnt) * u2 )
                           prems(ip,jp,1,ipnt) = 0d0
                           prems(ip,jp,2,ipnt) = 0d0

                    end if

                    btype = ibinfor(ipoint)

                 end do
              end do
           end if

         end do

      else

c     --- alphad =  2

         do iside = 1, 2

            tdir = 2
            ndir = 1
            if ( iside.eq.1) then
               nsign = -1
               tsign = -1

               ipibinf = ibipoint(3)+nvirtual-1
               ipoint  = ipibinf+1
               btype = ibinfor(ipoint)
               if ( btype.ge.89 .and. btype.le.99 ) then
                  iparea(1,2) = 1
               else
                  iparea(1,2) = 2
               end if
               ipibinf = ibipoint(4)+nvirtual-1
               ipoint  = ipibinf+1
               btype = ibinfor(ipoint)
               if ( btype.ge.89 .and. btype.le.99 ) then
                  iparea(2,2) = nj+1
               else
                  iparea(2,2) = nj
               end if

               iparea(1,1) = 1
               iparea(2,1) = 1
               ipnt = 3
            else
               nsign = 1
               tsign = 1

               ipibinf = ibipoint(3)+nvirtual-1
               ipoint  = ipibinf+1
               btype = ibinfor(ipoint)
               if ( btype.ge.89 .and. btype.le.99 ) then
                  iparea(1,2) = 1
               else
                  iparea(1,2) = 2
               end if
               ipibinf = ibipoint(4)+nvirtual-1
               ipoint  = ipibinf+1
               btype = ibinfor(ipoint)
               if ( btype.ge.89 .and. btype.le.99 ) then
                  iparea(2,2) = nj+1
               else
                  iparea(2,2) = nj
               end if

               iparea(1,1) = ni
               iparea(2,1) = ni
               ipnt = 4
            end if

            if ( locdx(2).ne.0 ) then

               ipibinf = ibipoint(iside)+nvirtual-1

               do  jp = iparea(1,2), iparea(2,2)
                  do  ip = iparea(1,1), iparea(2,1)

                     xp(1) = ip
                     xp(2) = jp
                     if ( locdx(2).eq.-1 ) then
                        ipoint = ipibinf+2*(xp(tdir)-1)
                     else
                        ipoint = ipibinf+2*xp(tdir)
                     end if

                     btype = ibinfor(ipoint)

                     if ( btype.eq.uxuy .or. btype.eq.unut .or.
     +                    btype.eq.parabolic .or. btype.eq.88 ) then

                        if ( btype.ne.uxuy ) then
                           if( tdir.eq.1 ) then
                              at1 = bcgeo(7,ipoint)
                              at2 = -bcgeo(6,ipoint)
                           else
                              at1 = -bcgeo(5,ipoint)
                              at2 = bcgeo(4,ipoint)
                           end if
                           an1 = bcgeo((2+2*ndir),ipoint)/
     +                           bcgeo(8,ipoint)
                           an2 = bcgeo((3+2*ndir),ipoint)/
     +                           bcgeo(8,ipoint)
                           gttdivsg = bcgeo(istwo(ndir,ndir),ipoint)
                           gnn = gttdivsg/bcgeo(8,ipoint)
                           gtt = bcgeo(8,ipoint)*gttdivsg
                           fac11 = at1/sqrt(gtt)
                           fac12 = at2/sqrt(gtt)
                           fac21 = an1/sqrt(gnn)
                           fac22 = an2/sqrt(gnn)
                           det = fac11*fac22-fac12*fac21
                           un = bndcon(1,ipoint)*nsign
                           ut = bndcon(2,ipoint)*tsign
                           u1 = (fac22*ut -fac12*un)/det
                           u2 = (-fac21*ut +fac11*un)/det
                        else
                           u1 = bndcon(1,ipoint)
                           u2 = bndcon(2,ipoint)
                        end if

                        rhside(ip,jp,alphad ) = rhside(ip,jp,alphad ) -
     +                 (  prems(ip,jp,1,ipnt) * u1 +
     +                    prems(ip,jp,2,ipnt) * u2 )
                          prems(ip,jp,1,ipnt) = 0d0
                          prems(ip,jp,2,ipnt) = 0d0

                     end if

                     btype = ibinfor(ipoint)

                  end do
               end do

            end if

            if ( (iside.eq.1 .and. locdx(1).eq.-1) .or.
     +           (iside.eq.2 .and. locdx(1).eq. 1) ) then

              ipibinf = ibipoint(iside)+nvirtual-1

               do  jp = iparea(1,2), iparea(2,2)
                  do  ip = iparea(1,1), iparea(2,1)

                     xp(1) = ip
                     xp(2) = jp
                     ipoint = ipibinf+2*xp(tdir) - 1

                     btype = ibinfor(ipoint)

                     if ( btype.eq.uxuy .or. btype.eq.unut .or.
     +                    btype.eq.parabolic .or. btype.eq.88 ) then

                        if ( btype.ne.uxuy ) then
                           if( tdir.eq.1 ) then
                              at1 = bcgeo(7,ipoint)
                              at2 = -bcgeo(6,ipoint)
                           else
                              at1 = -bcgeo(5,ipoint)
                              at2 = bcgeo(4,ipoint)
                           end if
                           an1 = bcgeo((2+2*ndir),ipoint)/
     +                           bcgeo(8,ipoint)
                           an2 = bcgeo((3+2*ndir),ipoint)/
     +                           bcgeo(8,ipoint)
                           gttdivsg = bcgeo(istwo(ndir,ndir),ipoint)
                           gnn = gttdivsg/bcgeo(8,ipoint)
                           gtt = bcgeo(8,ipoint)*gttdivsg
                           fac11 = at1/sqrt(gtt)
                           fac12 = at2/sqrt(gtt)
                           fac21 = an1/sqrt(gnn)
                           fac22 = an2/sqrt(gnn)
                           det = fac11*fac22-fac12*fac21
                           un = bndcon(1,ipoint)*nsign
                           ut = bndcon(2,ipoint)*tsign
                           u1 = (fac22*ut -fac12*un)/det
                           u2 = (-fac21*ut +fac11*un)/det
                        else
                           u1 = bndcon(1,ipoint)
                           u2 = bndcon(2,ipoint)
                        end if

                        rhside(ip,jp,alphad ) = rhside(ip,jp,alphad ) -
     +                  (  prems(ip,jp,1,ipnt) * u1 +
     +                  prems(ip,jp,2,ipnt) * u2 )
                        prems(ip,jp,1,ipnt) = 0d0
                        prems(ip,jp,2,ipnt) = 0d0

                     end if

                     btype = ibinfor(ipoint)

                  end do
               end do

            end if

         end do

      end if

      end
