      subroutine isbmomc3 ( ni, nj, nk, nvirtual, ilow1, ihigh1,
     +                      ilow2, ihigh2, veloc, vsum, alphad,
     +                      densrho, invjac, locdx, dcell,
     +                      ibinfor, iindsc, iface)
c=======================================================================
c
c
c        programmer     Kees Kassels
c        version 1.0    date 23-06-95 (3D version of isbmomc2)
c
c   copyright (c) 1995-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c*********************************************************************
c
c                      DESCRIPTION
c
c    determines the coefficients of the velocity components in a given
c    integration point within a boundary flux cell for the convection term in R3
c*********************************************************************
c
c                      KEYWORDS
c
c    boundary_conditions
c    convection
c    matrix
c    right_hand_side
c*********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer ni, nj, nk, nvirtual, ilow1, ihigh1, ilow2, ihigh2,
     +        locdx(3), dcell(3), iface, iindsc(*),
     +        alphad, ibinfor(ilow1:ihigh1,ilow2:ihigh2)


      double precision invjac(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,
     +                        1-nvirtual:nk+1+nvirtual),
     +                 densrho(1-nvirtual:ni+1+nvirtual,
     +                         1-nvirtual:nj+1+nvirtual,
     +                         1-nvirtual:nk+1+nvirtual),
     +                 vsum(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual,3),
     +                 veloc(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1-nvirtual:nk+1+nvirtual,3)

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     dcell     i    reserved
c     densrho   i    Array containing the density rho. The size and structure is
c                    the same as for the scalar unknowns
c     ibinfor   i    Third part of infbnd
c     iface     i    Face number
c     ihigh1    i    First upper index of an array
c     ihigh2    i    Second upper index of an array
c     iindsc    i    reserved
c     ilow1     i    First lower index of an array
c     ilow2     i    Second lower index of an array
c     invjac    i    1/sqrt(g) in pnttyp
c     iparea    i    Area of integration points where a contribution to
c                    the matrix is to be calculated
c     locdx     i    Position of the integration point with respect to
c                    the centre of the integration cell
c                    The cell sizes are assumed to be 2 times 2, hence the
c                    distance of a midside point to a cell centre is always 1
c     mlin      i    Type of linearization (1=newton, 2=picard)
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     veloc     i    Array for the old velocities (used in convection
c                    term calculation)
c     vsum      o    Flux contributions to the matrix
c*********************************************************************
c
c                      COMMON BLOCKS
c
c*********************************************************************
c
c                      LOCAL PARAMETERS
c

      integer unut, periodic, antiper, multiblk,
     +        uxuy, unsnt, perjump, antijump,
     +        oldturb, parabolic, skip, masspar
      parameter( unut = 4,  periodic = 98,
     +           antiper = 97, perjump = 96, antijump =95,
     +           multiblk = 99, uxuy = 10, unsnt = 5,
     +           oldturb = 9, parabolic = 12)
      parameter ( skip = 300, masspar = 100)
      logical ungiven
      integer mip, mjp, mkp, xp(3), ip, jp, kp, mxp(3),
     +        btype,  iparea(2,3), tdir(2),
     +        ndir, nsig,  mlin

      double precision rmult1


c     antijump  Anti-periodic boundary conditions with jump in
c               pressure
c     antiper   Anti-periodic boundary conditions
c     btype     Type of boundary condition
c     ip        Local counting variable i-direction
c     jp        Local counting variable j-direction
c     kp        Local counting variable k-direction
c     masspar   Boundary condition for massive parallel computations
c     mip       Integration cell indices
c     mjp       Integration cell indices
c     mkp       Integration cell indices
c     multiblk  Multi-block boundary conditions
c     mxp       Auxiliary array to calculate boundary point number
c     ndir      Direction of normal vector on this side
c     nsig      Sign of inner product of the basis vector
c               for the above mentioned internal direction
c               and the normal
c     oldturb   Old turbulence boundary condtion
c               (equivalent to unsnt)
c     parabolic Parabolic boundary conditions
c     periodic  Periodic boundary conditions
c     perjump   Periodic boundary conditions with jump in pressure
c     rmult1    Multiplication factor
c     skip      If true boundary condition must be skipped
c     tdir      Direction of tangent vector on this side
c     ungiven   If true un is given
c     unsnt     U_n and sigma_nt prescribed
c     unut      U_n and u_t prescribed
c     uxuy      U_x and u_y prescribed
c     xp        Auxiliary array to calculate boundary point number
c*********************************************************************
c
c                      SUBROUTINES CALLED
c
c*********************************************************************
c
c                      I/O
c
c*********************************************************************
c
c                      ERROR MESSAGES
c
c*********************************************************************
c
c                      PSEUDO CODE
c
c       Relevant formula for an internal cell
c
c
c    The convective terms in case of the classical discretization are given by:
c
c
c    rho      a  1 |i+1/2,j,k     rho      a  2 |i,j+1/2,k
c   -----    V  V  |         +   -----    V  V  |          +
c   sqrt(g)        |i-1/2,j,k    sqrt(g)        |i,j-1/2,k
c
c
c     rho      a  3 |i,j,k+1/2      rho    |  a  |   c  b |
c    -----    V  V  |          +   -----   |     |  V  V  |
c    sqrt(g)        |i,j,k-1/2     sqrt(g) | b c |        |i,j,k
c
c
c   Application of Newton linearization:
c
c   a  b    _a  b      a _b     _a _b
c  V  V  =  V  V   +  V  V   -  V  V
c                                                 1
c   results in the following contribution of the V  component to the matrix:
c
c
c    rho      1 |i+1/2,j,k     rho      2 |i,j+1/2,k      rho      a |i+1/2,j,k
c   -----    V  |           + -----    V  |            + -----    V  |         +
c   sqrt(g)     |i-1/2,j,k    sqrt(g)     |i,j-1/2,k     sqrt(g)     |i-1/2,j,k
c
c
c    rho      1 |i,j,k+1/2      2 rho    |  a  |   b |
c   -----    V  |           +    -----   |     |  V  |
c   sqrt(g)     |i,j,k-1/2       sqrt(g) | 1 b |     |i,j,k
c
c                                              2      3
c  and of course a similar expression for the V  and V components
c
c  Application of Picard linearization (instead of newton lin.) gives for
c  the b-th momentum equation:
c
c     a  b      _a  b              b  b     _b  b
c    V  V   =   V  V      and     V  V   =  V  V
c   In case of a tangential boundary cell there is no need to adapt these
c   formulae and the subroutine is actually skipped
c
c   In case of a normal half cell some adaptation is necessary
c
c   Consider a normal half cell at the lower boundary, i.e.
c       ntdir=(3,1,2)
c       ntsig=(-1,1,1)
c
c   The picture shows the intersection with the plane ksi2 = constant (0)
c
c         ------------------------------------------
c         |              |            |            |
c         |              |            |            |
c         |              |            |            |
c         |              +-----+------+            |
c         |              |            |            |
c         |              |            |            |
c         |              |            |            |
c         ---------------+-----*------+-------------
c
c   The cell centre is indicated by *, the possible integration points by +
c
c   The integration points that are chosen depend on the type of boundary
c   conditions
c
c   If un* is given the normal integration cell is not used and the
c   matrix contribution is zero
c
c   if Snn is given we must distinguish between Snt given and ut given
c   If Snt is given the integration points are:
c     (0,0,0), (0,0,1), (1,0,1), (-1,0,1), (0,-1,1), (0,1,1)
c   If ut is given the integration points are:
c     (0,0,0), (0,0,1), (1,0,0), (-1,0,0), (0,-1,0), (0,1,0)
c
c   A special situation is if one of the points (1,0,0), (-1,0,0),
c   (0,1,0) or (0,-1,0) has itself ut given.
c   In that case we use that point as integration point
c   instead of the point at ksi2 = 1
c
c   So the integration becomes either:
c
c    rho      3  1 |1,0,1        rho      3  2 |0,1,1     rho      3  3 |0,0,1
c   -----    V  V  |         +  -----    V  V  |     +   -----    V  V  |     +
c  2sqrt(g)        |-1,0,1      2sqrt(g)       |0,-1,1   sqrt(g)        |0,0,0
c
c
c    rho    |  2  |   c  b |
c   -----   |     |  V  V  |        (Snt given)
c  2sqrt(g) | b c |        |0,0,0
c
c   or
c
c   in case ut given at a point of the boundary: Ksi_3 = 1 replaced by 0c
c
c   An extra problem is that Vt can not be evaluated in (0,0,0)
c   If this value is needed, it is replaced by Vt(0,0,1)
c=======================================================================

      mlin = iindsc(8)

         iparea(1,1) = 1
         iparea(2,1) = 1
         iparea(1,2) = 1
         iparea(2,2) = 1
         iparea(1,3) = 1
         iparea(2,3) = 1

         if ( iface.le.2 ) then

c        --- iface = 1, 2:  back and front plane

            iparea(2,2) = nj
            iparea(2,3) = nk
            ndir    = 1
            tdir(1) = 2
            tdir(2) = 3
            nsig    = -1


            if ( iface.eq.2 ) then

c           --- iface = 2:  front

               iparea(1,1) = ni+1
               iparea(2,1) = ni+1
               nsig    =  1

            end if

         else if ( iface.gt.2  .and. iface .lt. 5) then

c        --- iface = 3, 4:  left- and right plane

            iparea(2,1) = ni
            iparea(2,3) = nk
            ndir = 2
            tdir(1) = 3
            tdir(2) = 1
            nsig    = -1


            if ( iface.eq.4 ) then

c           --- iface = 4:  right side

               iparea(1,2) = nj+1
               iparea(2,2) = nj+1
               nsig    =  1


            end if

         else if ( iface.gt.4  ) then

c        --- iface = 5, 6:  bottom and top

            iparea(2,1) = ni
            iparea(2,2) = nj
            ndir = 3
            tdir(1) = 1
            tdir(2) = 2
            nsig    = -1

            if ( iface.eq.6 ) then

c           --- iface = 6:  upper side

               iparea(1,3) = nk+1
               iparea(2,3) = nk+1
               nsig    =  1

            end if

         end if
c
      if ( ndir.eq.alphad ) then

c        --- Determine multiplication factors due
c            to linearisation type

            if ( mlin.eq.1 ) then
               rmult1 = 2d0
            else
               rmult1 = 1d0
            end if


c     --- Normal half cell
c         loop over cells

         do 200 mkp = iparea(1,3), iparea(2,3)
            do 180 mjp = iparea(1,2), iparea(2,2)
               do 150 mip = iparea(1,1), iparea(2,1)

c                 --- Get integration cell index

                  mxp(1) = mip
                  mxp(2) = mjp
                  mxp(3) = mkp

c                 --- Get boundary type for integration cell centre

                  btype = ibinfor(2*mxp(tdir(1)),2*mxp(tdir(2)))

c                 --- Check for Dirichlet condition in half cell

                  ungiven = btype.eq.uxuy .or. btype.eq.unut .or.
     +                      btype.eq.parabolic .or. btype.eq.unsnt .or.
     +                      btype.eq.oldturb

c                 --- Get reduced boundary condition type

                  if ( btype.eq.multiblk .or. btype.eq.periodic .or.
     +                 btype.eq.antiper .or.
     +                 btype.eq.perjump .or. btype.eq.antijump .or.
     +                 btype.eq.masspar ) btype = skip

c                 --- Get integration point index

                  xp(ndir)    = mxp(ndir) + dcell(ndir)
                  xp(tdir(1)) = mxp(tdir(1)) + dcell(tdir(1))
                  xp(tdir(2)) = mxp(tdir(2)) + dcell(tdir(2))

                  ip = xp(1)
                  jp = xp(2)
                  kp = xp(3)

                  if ( ungiven ) then

c                 --- Normal velocity given, no contribution to stencil

                     vsum(ip,jp,kp,1) =0d0
                     vsum(ip,jp,kp,2) =0d0
                     vsum(ip,jp,kp,3) =0d0

                  else if ( btype.ne.skip ) then

c                 --- Centre of cell is not adjacent to another block,
c                     construct basic stencil for half cell

                     if ( locdx(ndir).eq.0 ) then

                        vsum(ip,jp,kp,1) = .5d0 * vsum(ip,jp,kp,1)
                        vsum(ip,jp,kp,2) = .5d0 * vsum(ip,jp,kp,2)
                        vsum(ip,jp,kp,3) = .5d0 * vsum(ip,jp,kp,3)

                     end if


                     if (locdx(ndir)   .eq.0  .and.
     +                        locdx(tdir(1)).eq.0  .and.
     +                        locdx(tdir(2)).eq.0  ) then

c                    --- Cell centre (0,0,0)
c                        add term which for the inner area is
c                        computed in the integration point outside
c                        the region

                        vsum(ip,jp,kp,ndir) = 
     +                       vsum(ip,jp,kp,ndir) + 
     +                       nsig*rmult1*densrho(ip,jp,kp)*
     +                       invjac(ip,jp,kp)*veloc(ip,jp,kp,ndir)

                     end if

                     if (locdx(ndir).eq.nsig) then

c                    --- Point outside region

                        vsum(ip,jp,kp,1) = 0d0
                        vsum(ip,jp,kp,2) = 0d0
                        vsum(ip,jp,kp,3) = 0d0

                     end if

                  end if

150            continue

180         continue

200      continue


      end if
      end
