      subroutine isbmomt2 ( ni, nj, nvirtual, matrix, rhside, alphad,
     +                      ibinfor, ibipoint )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.1    date 17-02-1998 Change of names
c        version 1.0    date 07-03-1995
c
c   copyright (c) 1995-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     correct the time derivative part of the matrix diagonal with respect to
c     the boundary cells where sigma^nn is given
c     R^2 only
c     Remark: the present version of this subroutines assumes that boundary
c             conditions  may differ in each point
c             The subroutine may be more efficient if information is stored
c             as sections or per side
c **********************************************************************
c
c                       KEYWORDS
c
c     2d
c     matrix
c     right_hand_side
c     time_derivative
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer alphad, ni, nj, ibinfor(*), ibipoint(*), nvirtual
      double precision matrix(ni+1,nj+1), rhside(ni+1,nj+1)

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     ibinfor   i    Third part of infbnd
c     ibipoint  i    ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     matrix   i/o   reserved
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     rhside   i/o   reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ip, jp, snnsnt, snnut, ndir, ilow, jlow, iupp, jupp, tdir,
     +        iside, xp(2), btype, ipibinf
      parameter( snnsnt = 7, snnut = 6 )

c     btype     Type of boundary condition
c     ilow      Lower index of subface
c     ip        Local counting variable i-direction
c     ipibinf   Starting address of array infor for present side minus one
c     iside     Sequence number of side
c     iupp      Upper i-index of subface
c     jlow      Lower j-index for a subface
c     jp        Local counting variable j-direction
c     jupp      Upper j-index of subface
c     ndir      Direction of normal vector on this side
c     snnsnt    Sigma_nn and sigma_nt prescribed
c     snnut     Sigma_nn and u_t prescribed
c     tdir      Direction of tangent vector on this side
c     xp        Auxiliary array to calculate boundary point number
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c      This subroutine only treats those half cells where the normal stress
c      is given.
c      In those cells it is necessary to divide the value of the
c      matrix element as well as the right-hand side element by 2
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- loop over boundaries

      do iside = 1, 4
         ilow = 1
         iupp = 1
         jlow = 1
         jupp = 1
         ipibinf = ibipoint(iside)-1+nvirtual
         if ( iside.le.2 ) then

c        --- iside = 1, 2:  left- and right-hand side

            jupp = nj
            ndir = 1
            tdir = 2
            if ( iside.eq.2 ) then

c           --- iside = 2:  right-hand side

               ilow = ni+1
               iupp = ni+1

            end if

         else

c        --- iside = 3, 4:  lower- and upper side

            iupp = ni
            ndir = 2
            tdir = 1
            if ( iside.eq.4 ) then

c           --- iside = 4:  upper side

               jlow = nj+1
               jupp = nj+1

            end if

         end if
         if ( ndir.eq.alphad ) then

c        --- alphad = normal direction, hence half cell found

            do jp = jlow, jupp
               do ip =  ilow, iupp
                  xp(1) = ip
                  xp(2) = jp
                  btype = ibinfor(ipibinf+2*xp(tdir))
                  if ( btype.eq.snnut .or. btype.eq.snnsnt ) then
                     matrix(ip,jp) = matrix(ip,jp) * 0.5d0
                     rhside(ip,jp) = rhside(ip,jp) * 0.5d0
                  end if
               end do
            end do
         end if
      end do

      end
