      subroutine isbms13d ( ni, nj, nk, nvirtual, ilow1, ihigh1,
     +                      ilow2, ihigh2 , vsum, dvsum , alphad,
     +                      mu, nzvsum, nzdvsum , locjac, locinvjac,
     +                      jcta, jctg , chsym , cva, locdx, dcell,
     +                      ibinfor , bndcon, maxb, rhside, iface )
c ======================================================================
c
c        programmer  Kees Kassels
c        version 1.2    date 20-05-1996 Kees K: cva added, vfac changed
c        version 1.1    date 04-01-1996 Marcel: correction vfac
c        version 1.0    date 14-11-1995 normal half cell removed
c        version 1.0    date 29-06-1995 (extracted from ismmsb3 of
c                                      Marcel Zijlema)
c
c   copyright (c) 1995-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c **********************************************************************
c
c                       DESCRIPTION
c
c     ISnas Momentum Matrix Stress at the Boundary in 3 dimensions
c
c **********************************************************************
c
c                       KEYWORDS
c
c     3d
c     boundary_conditions
c     matrix
c     stress
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nk, nvirtual, ilow1, ihigh1, ilow2,
     +        ihigh2, maxb,  locdx(3), dcell(3), alphad,
     +        ibinfor(ilow1:ihigh1,ilow2:ihigh2)
      double precision locjac(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,
     +                        1-nvirtual:nk+1+nvirtual),
     +                 locinvjac(1-nvirtual:ni+1+nvirtual,
     +                           1-nvirtual:nj+1+nvirtual,
     +                           1-nvirtual:nk+1+nvirtual),
     +                 cva(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,
     +                    1-nvirtual:nk+1+nvirtual,3,3),
     +                 bndcon(maxb,ilow1:ihigh1,ilow2:ihigh2)
      logical nzvsum(3),nzdvsum(3,3)
      double precision
     +   mu(1-nvirtual:ni+1+nvirtual,1-nvirtual:nj+1+nvirtual,
     +      1-nvirtual:nk+1+nvirtual),
     +   vsum(1-nvirtual:ni+1+nvirtual,1-nvirtual:nj+1+nvirtual,
     +        1-nvirtual:nk+1+nvirtual,3),
     +   dvsum(1-nvirtual:ni+1+nvirtual,1-nvirtual:nj+1+nvirtual,
     +         1-nvirtual:nk+1+nvirtual,3,3),
     +   jcta(1-nvirtual:ni+1+nvirtual,1-nvirtual:nj+1+nvirtual,
     +         1-nvirtual:nk+1+nvirtual,3,3),
     +   jctg(1-nvirtual:ni+1+nvirtual,1-nvirtual:nj+1+nvirtual,
     +        1-nvirtual:nk+1+nvirtual,6),
     +   chsym(1-nvirtual:ni+1+nvirtual,1-nvirtual:nj+1+nvirtual,
     +         1-nvirtual:nk+1+nvirtual,3,6),
     +   rhside(ni+1,nj+1,nk+1,3)

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     bndcon    i    reserved
c     chsym     i    Christoffel symbols of the second kind in the
c                    point given by pnttyp
c                    (symmetric contravariant part stored as vector
c                    with index encoding given by istwo)
c     cva       i    Array containing the covariant basis vectors
c                    in 2D we have
c                                                 l
c                    cva(i,j,l,alpha,p) contains a        in cell i,j in
c                                                 (alpha)       point type p
c                    in 3D it is cva(i,j,k,l,alpha,p) referring to cell i,j,k
c     dcell     i    reserved
c     dvsum     o    Flux derivative contributions to the matrix
c     ibinfor   i    Third part of infbnd
c     iface     i    Face number
c     ihigh1    i    First upper index of an array
c     ihigh2    i    Second upper index of an array
c     ilow1     i    First lower index of an array
c     ilow2     i    Second lower index of an array
c     iparea    i    Area of integration points where a contribution to
c                    the matrix is to be calculated
c     istwo     i    Translation table to store and retrieve the
c                    elements of a symmetric ndim-dimensional
c                    rank 2 tensor from a vector with length
c                    (ndim*(ndim+1))/2
c     jcta      i    Jacobian times contravariant basis vectors
c                    in point pnttyp
c                    (stored as vector index encoding given by istwo)
c     jctg      i    Jacobian times symmetric contravariant metric
c                    tensor in point given by pnttyp
c                    (stored as vector index encoding given by istwo)
c     locdx     i    Position of the integration point with respect to
c                    the centre of the integration cell
c                    The cell sizes are assumed to be 2 times 2, hence the
c                    distance of a midside point to a cell centre is always 1
c     locinvjac i    Array containing the inverse of the Jacobian for all
c                    integration points of one type
c     locjac    i    Array containing the Jacobian for all integration
c                    points of one type
c     maxb      i    Maximum number of real values for one boundary condition
c     mu        i    The laminar viscosity
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     nzdvsum   o    Indicates whether or not a partial derivative in a
c                    certain direction occurs in the stencil.
c                    For example the derivative dU2/dksi1 in the point (0,1)
c                    or the V1-stencil of the 13-point molecule does not exist
c                    nzdvsum should therefore be false in that point
c     nzvsum    o    Indicates whether or not a velocity in a
c                    certain direction has a contribution in the stencil.
c     rhside    i    reserved
c     vsum      o    Flux contributions to the matrix
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer istwo(3,3), iparea(2,3), iface
      integer unut, snnsnt, periodic, antiper, multiblk,
     +        uxuy, unsnt, perjump, antijump,
     +        oldturb, parabolic, skip, masspar
      parameter ( skip = 300, masspar = 100)
      parameter( unut = 4, snnsnt = 7, periodic = 98,
     +           antiper = 97, perjump = 96, antijump =95,
     +           multiblk = 99, uxuy = 10, unsnt = 5,
     +           oldturb = 9, parabolic= 12)
      integer i, j, btype, mip, mjp, mkp, mxp(3),
     +        ict1, ict2, icbtype, ip, jp, kp, xp(3), ndir,
     +        tdir(2),  nsig, tsig(2),
     +        gammal, deltal
      double precision vval(3), vfac(3,3), gcontra, stress(3,3),
     +                 gcov(3,3)
      data istwo / 1, 2, 3,
     +             2, 4, 5,
     +             3, 5, 6 /

c     antijump  Anti-periodic boundary conditions with jump in
c               pressure
c     antiper   Anti-periodic boundary conditions
c     btype     Type of boundary condition
c     deltal    Loop index
c     gammal    Loop index
c     gcontra   Contravariant metric tensor
c     gcov      Covarariant metric tensor
c     i         General loop variable
c     icbtype   Type of boundary condition with respect to centre of integration
c               cell
c     ict1      Pointer in ibinfor with respect to boundary point
c     ict2      Pointer in ibinfor with respect to boundary point
c     ip        Local counting variable i-direction
c     j         General loop variable
c     jp        Local counting variable j-direction
c     kp        Local counting variable k-direction
c     masspar   Boundary condition for massive parallel computations
c     mip       Integration cell indices
c     mjp       Integration cell indices
c     mkp       Integration cell indices
c     multiblk  Multi-block boundary conditions
c     mxp       Auxiliary array to calculate boundary point number
c     ndir      Direction of normal vector on this side
c     nsig      Sign of inner product of the basis vector
c               for the above mentioned internal direction
c               and the normal
c     oldturb   Old turbulence boundary condtion
c               (equivalent to unsnt)
c     parabolic Parabolic boundary conditions
c     periodic  Periodic boundary conditions
c     perjump   Periodic boundary conditions with jump in pressure
c     skip      If true boundary condition must be skipped
c     snnsnt    Sigma_nn and sigma_nt prescribed
c     stress    A local array containing physical stresses
c     tdir      Direction of tangent vector on this side
c     tsig      Sign of inner product of the true tangent vector
c               on this side and the ISNaS vector with direction TDIR
c     unsnt     U_n and sigma_nt prescribed
c     unut      U_n and u_t prescribed
c     uxuy      U_x and u_y prescribed
c     vfac      Factors for derivation of flux if not known
c               explicitly
c     vval      Flux values
c     xp        Auxiliary array to calculate boundary point number
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c   First we consider the formulae at an internal cell:
c
c
c    The stress terms in case of the classical discretization are given by:
c
c
c             a1 |i+1/2,j,k              a2 |i,j+1/2,k
c   sqrt(g)  T   |           + sqrt(g)  T   |          +
c                |i-1/2,j,k                 |i,j-1/2,k
c
c
c              a3 |i,j,k+1/2              |  a  |   cb |
c     sqrt(g) T   |           +   sqrt(g) |     |  T   |
c                 |i,j,k-1/2              | b c |      |i,j,k
c
c
c             ab
c   The term T   can be written as:
c
c
c    ab         ac b      bc a
c  -T   = mu ( g  U    + g  U   )
c                  ,c        ,c
c
c           aa         ac a
c   Hence -T   = 2 mu g  U
c                         ,c
c
c
c                    a                  a        b     |  a   |  c
c    The derivative U   is equal to  d U  / d Ksi    + |      | U
c                    ,b                                |b   c |
c
c   Consider a tangential cell at the lower boundary, i.e.
c       ntdir=(3,1,2)
c       ntsig=(-1,1,1)
c
c
c         ---------------+--------------------------
c         |              |            |            |
c         |      |       |     |      |            |
c         |              |            |            |
c         |      +       *     +      |            |
c         |              |            |            |
c         |      |       |     |      |            |
c         |              |            |            |
c         ---------------+--------------------------
c
c   The following boundary conditions have the following effects:
c
c
c   Dirichlet boundary conditions:  do not change the matrix elements
c                                   Use the folding subroutine for the
c                                   virtual velocities
c
c   Sigma nn, ut1 and ut2 given:    no sigmann term at the boundary
c                                   Folding routine for ut1 and ut2
c
c   We assumed, however, that ut1 and ut2 are both zero (for the moment)
c
c   Sigma nt, un given:
c      The only point at the boundary is the point (0,0,-1)
c      un does not need any precautions
c
c      sigma^nt1 = sqrt(g^nn) nsig S^nt alpha(t1) -
c
c                   g_t2t2 g_nt1 - g_t1t2 g_nt2
c                  ----------------------------- sigma^nn
c                  g_t1t1 g_t2t2 - g_t1t2 g_t1t2
c
c
c      sigma^nt2 = sqrt(g^nn) nsig S^nt alpha(t2) -
c
c                   g_t1t1 g_nt2 - g_t1t2 g_nt1
c                  ----------------------------- sigma^nn
c                  g_t1t1 g_t2t2 - g_t1t2 g_t1t2
c
c      So S^nt to right-hand side and the effect of sigma^nn to the
c      matrix
c
c   Sigma nt, Sigma nn given:
c      The only point at the boundary is the point (0,0,-1)
c      sigma^nt in this point may be computed directly and must be added to
c      the right-hand side
c      We assumed that both sigma's are zero (for the moment)
c      The contribution to the matrix is zero
c
c
c    The comments refer to a cell at the lower boundary
c
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call iseropen ( 'isbms13d' )

         iparea(1,1) = 1
         iparea(2,1) = 1
         iparea(1,2) = 1
         iparea(2,2) = 1
         iparea(1,3) = 1
         iparea(2,3) = 1

         if ( iface.le.2 ) then

c        --- iface = 1, 2:  back and front plane

            iparea(2,2) = nj
            iparea(2,3) = nk
            ndir    = 1
            tdir(1) = 2
            tdir(2) = 3
            nsig    = -1
            tsig(1) = -1
            tsig(2) =  1

            if ( iface.eq.2 ) then

c           --- iface = 2:  front

               iparea(1,1) = ni
               iparea(2,1) = ni
               nsig    =  1
               tsig(1) =  1
               tsig(2) =  1

            end if

         else if ( iface.gt.2  .and. iface.lt.5) then

c        --- iface = 3, 4:  left- and right plane

            iparea(2,1) = ni
            iparea(2,3) = nk
            ndir = 2
            tdir(1) = 3
            tdir(2) = 1
            nsig    = -1
            tsig(1) = -1
            tsig(2) =  1

            if ( iface.eq.4 ) then

c           --- iface = 4:  right side

               iparea(1,2) = nj
               iparea(2,2) = nj
               nsig    =  1
               tsig(1) =  1
               tsig(2) =  1

            end if

         else if ( iface.gt.4  ) then

c        --- iface = 5, 6:  bottom and top

            iparea(2,1) = ni
            iparea(2,2) = nj
            ndir = 3
            tdir(1) = 1
            tdir(2) = 2
            nsig    = -1
            tsig(1) = -1
            tsig(2) =  1

            if ( iface.eq.6 ) then

c           --- iface = 6:  upper side

               iparea(1,3) = nk
               iparea(2,3) = nk
               nsig    =  1
               tsig(1) =  1
               tsig(2) =  1

            end if

         end if

c        --- loop over cells

      do mkp = iparea(1,3), iparea(2,3)
        do mjp = iparea(1,2), iparea(2,2)
          do mip = iparea(1,1), iparea(2,1)

c         --- Get integration cell index

            mxp(1) = mip
            mxp(2) = mjp
            mxp(3) = mkp

c           --- Get integration cell position for ibinfor

            if ( alphad.eq.tdir(1) ) then
               ict1 = 2*mxp(tdir(1))-1
            else
               ict1 = 2*mxp(tdir(1))
            end if
            if ( alphad.eq.tdir(2) ) then
               ict2 = 2*mxp(tdir(2))-1
            else
               ict2 = 2*mxp(tdir(2))
            end if
            icbtype = ibinfor(ict1,ict2)

c           --- Get reduced boundary condition type

            btype = icbtype
            if ( btype.eq.parabolic ) btype = unut
            if ( btype.eq.oldturb ) btype = unsnt

c           --- Get integration point index

            xp(tdir(1)) = mxp(tdir(1)) + dcell(tdir(1))
            xp(tdir(2)) = mxp(tdir(2)) + dcell(tdir(2))
            xp(ndir )   = mxp(ndir )   + dcell(ndir )

            ip = xp(1)
            jp = xp(2)
            kp = xp(3)

            if ( alphad.eq.tdir(1) ) then

c           --- First tangential velocity integration cell

               if ( btype.eq.unsnt ) then

c              --- un and Snt given
c                  Get boundary condition information
c                  vval contains some contribution for right-hand side
c                  vfac contains the multiplication factor for sigma^nn

                  if ( locdx(tdir(1)).eq.0 .and.
     +                 locdx(tdir(2)).eq.0 .and.
     +                 locdx(ndir).eq.nsig    ) then

c                 --- Point (0,0,-1) found
c                     Put contribution of S^nt into right-hand side


                    gcontra = jctg(ip,jp,kp,istwo(ndir,ndir)) *
     +                         locinvjac(ip,jp,kp)

                    vfac(tdir(1),tdir(1)) = -locjac(ip,jp,kp)*
     +                                       sqrt(gcontra)

                    stress(ndir,tdir(1))=bndcon(2,ict1,ict2)

                    vval(tdir(1))=vfac(tdir(1),tdir(1)) *
     +                            stress(ndir,tdir(1))

                    rhside(mip,mjp,mkp,alphad) =
     +                     rhside(mip,mjp,mkp,alphad) - vval(tdir(1))

                    vsum(ip,jp,kp,tdir(1)) = 0d0
                    vsum(ip,jp,kp,tdir(2)) = 0d0
                    vsum(ip,jp,kp,ndir) = 0d0
                    dvsum(ip,jp,kp,tdir(1),tdir(1)) = 0d0
                    dvsum(ip,jp,kp,tdir(1),tdir(2)) = 0d0
                    dvsum(ip,jp,kp,tdir(1),ndir) = 0d0

c                   --- Compute contribution to sigma^nn term
c                       Multiply by vfac

                    do gammal = 1, 3
                      nzvsum(gammal) = .true.
                      nzdvsum(ndir,gammal) = .true.
                    end do

                   do i = 1, 3
                      do j = 1, 3
                         gcov(i,j) = cva(ip,jp,kp,1,i)*
     +                               cva(ip,jp,kp,1,j) +
     +                               cva(ip,jp,kp,2,i)*
     +                               cva(ip,jp,kp,2,j) +
     +                               cva(ip,jp,kp,3,i)*
     +                               cva(ip,jp,kp,3,j)
                      end do
                   end do

                    vfac(ndir,tdir(1))=nsig*locjac(ip,jp,kp)*
     +                 (gcov(tdir(2),tdir(2))*
     +                  gcov(ndir,tdir(1)) -
     +                  gcov(tdir(1),tdir(2))*
     +                  gcov(ndir,tdir(2))) /
     +                 (gcov(tdir(1),tdir(1))*
     +                  gcov(tdir(2),tdir(2)) -
     +                  gcov(tdir(2),tdir(1))*
     +                  gcov(tdir(1),tdir(2)))

                    dvsum(ip,jp,kp,ndir,ndir) =
     +                  vfac(ndir,tdir(1))*2d0*mu(ip,jp,kp)*
     +                  locinvjac(ip,jp,kp)*
     +                  jctg(ip,jp,kp,istwo(ndir,ndir))
                    dvsum(ip,jp,kp,ndir,tdir(1)) =
     +                  vfac(ndir,tdir(1))*2d0*mu(ip,jp,kp)*
     +                  locinvjac(ip,jp,kp)*
     +                  jctg(ip,jp,kp,istwo(ndir,tdir(1)))
                    dvsum(ip,jp,kp,ndir,tdir(2)) =
     +                  vfac(ndir,tdir(1))*2d0*mu(ip,jp,kp)*
     +                  locinvjac(ip,jp,kp)*
     +                  jctg(ip,jp,kp,istwo(ndir,tdir(2)))

                    do i = 1, 3

                      vsum(ip,jp,kp,1) = vsum(ip,jp,kp,1)+
     +                                   dvsum(ip,jp,kp,ndir,i)*
     +                                   chsym(ip,jp,kp,ndir,istwo(i,1))
                      vsum(ip,jp,kp,2) = vsum(ip,jp,kp,2)+
     +                                   dvsum(ip,jp,kp,ndir,i)*
     +                                   chsym(ip,jp,kp,ndir,istwo(i,2))
                      vsum(ip,jp,kp,3) = vsum(ip,jp,kp,3)+
     +                                   dvsum(ip,jp,kp,ndir,i)*
     +                                   chsym(ip,jp,kp,ndir,istwo(i,3))

                    end do

                  end if

               else if ( btype.eq.snnsnt ) then

c              --- Known stresses

                  if ( locdx(tdir(1)).eq.0 .and.
     +                 locdx(tdir(2)).eq.0 .and.
     +                 locdx(ndir).eq.nsig    ) then

c                 --- Point (0,0,-1)
c                     Contribution of matrix is zero
c                     Get boundary condition information
c                     At this momentwe assume that only outflow
c                     is prescribed, i.e.
c                      nn    nt1    nt2
c                     S   = S    = S    = 0

                    do gammal = 1, 3
                      vsum(ip,jp,kp,gammal) = 0d0
                      do deltal = 1, 3
                        dvsum(ip,jp,kp,gammal,deltal) = 0d0
                      end do
                    end do

                    vval(tdir(1)) = 0d0

                    rhside(mip,mjp,mkp,alphad) =
     +                               rhside(mip,mjp,mkp,alphad) +
     +                               nsig*locjac(ip,jp,kp)*vval(tdir(1))

                  end if

               end if

            else if ( alphad.eq.tdir(2) ) then

c           --- Second tangential velocity integration cell

               if ( btype.eq.unsnt ) then

c              --- un and Snt given
c                  Get boundary condition information
c                  vval contains some contribution for right-hand side
c                  vfac contains the multiplication factor for sigma^nn

                  if ( locdx(tdir(1)).eq.0 .and.
     +                 locdx(tdir(2)).eq.0 .and.
     +                 locdx(ndir).eq.nsig    ) then

c                 --- Point (0,0,-1) found
c                     Put contribution of S^nt into right-hand side

                    gcontra = jctg(ip,jp,kp,istwo(ndir,ndir)) *
     +                         locinvjac(ip,jp,kp)

                    vfac(tdir(2),tdir(2)) = -locjac(ip,jp,kp)*
     +                                       sqrt(gcontra)

                    stress(ndir,tdir(2))=bndcon(3,ict1,ict2)

                    vval(tdir(2))=vfac(tdir(2),tdir(2)) *
     +                            stress(ndir,tdir(2))

                    rhside(mip,mjp,mkp,alphad) =
     +                     rhside(mip,mjp,mkp,alphad) - vval(tdir(2))

                    vsum(ip,jp,kp,tdir(1)) = 0d0
                    vsum(ip,jp,kp,tdir(2)) = 0d0
                    vsum(ip,jp,kp,ndir) = 0d0
                    dvsum(ip,jp,kp,tdir(2),tdir(1)) = 0d0
                    dvsum(ip,jp,kp,tdir(2),tdir(2)) = 0d0
                    dvsum(ip,jp,kp,tdir(2),ndir) = 0d0

c                   --- Compute contribution to sigma^nn term
c                       Multiply by vfac

                    do gammal = 1, 3
                      nzvsum(gammal) = .true.
                      nzdvsum(ndir,gammal) = .true.
                    end do

                   do i = 1, 3
                      do j = 1, 3
                         gcov(i,j) = cva(ip,jp,kp,1,i)*
     +                               cva(ip,jp,kp,1,j) +
     +                               cva(ip,jp,kp,2,i)*
     +                               cva(ip,jp,kp,2,j) +
     +                               cva(ip,jp,kp,3,i)*
     +                               cva(ip,jp,kp,3,j)
                      end do
                   end do

                    vfac(ndir,tdir(2))=nsig*locjac(ip,jp,kp)*
     +                 (gcov(tdir(1),tdir(1))*
     +                  gcov(ndir,tdir(2)) -
     +                  gcov(tdir(1),tdir(2))*
     +                  gcov(ndir,tdir(1))) /
     +                 (gcov(tdir(1),tdir(1))*
     +                  gcov(tdir(2),tdir(2)) -
     +                  gcov(tdir(2),tdir(1))*
     +                  gcov(tdir(1),tdir(2)))

                    dvsum(ip,jp,kp,ndir,ndir) =
     +                  vfac(ndir,tdir(2))*2d0*mu(ip,jp,kp)*
     +                  locinvjac(ip,jp,kp)*
     +                  jctg(ip,jp,kp,istwo(ndir,ndir))
                    dvsum(ip,jp,kp,ndir,tdir(1)) =
     +                  vfac(ndir,tdir(2))*2d0*mu(ip,jp,kp)*
     +                  locinvjac(ip,jp,kp)*
     +                  jctg(ip,jp,kp,istwo(ndir,tdir(1)))
                    dvsum(ip,jp,kp,ndir,tdir(2)) =
     +                  vfac(ndir,tdir(2))*2d0*mu(ip,jp,kp)*
     +                  locinvjac(ip,jp,kp)*
     +                  jctg(ip,jp,kp,istwo(ndir,tdir(2)))

                    do i = 1, 3

                      vsum(ip,jp,kp,1) = vsum(ip,jp,kp,1)+
     +                                   dvsum(ip,jp,kp,ndir,i)*
     +                                   chsym(ip,jp,kp,ndir,istwo(i,1))
                      vsum(ip,jp,kp,2) = vsum(ip,jp,kp,2)+
     +                                   dvsum(ip,jp,kp,ndir,i)*
     +                                   chsym(ip,jp,kp,ndir,istwo(i,2))
                      vsum(ip,jp,kp,3) = vsum(ip,jp,kp,3)+
     +                                   dvsum(ip,jp,kp,ndir,i)*
     +                                   chsym(ip,jp,kp,ndir,istwo(i,3))

                    end do

                  end if

               else if ( btype.eq.snnsnt ) then

c              --- Known stresses

                  if ( locdx(tdir(1)).eq.0 .and.
     +                 locdx(tdir(2)).eq.0 .and.
     +                 locdx(ndir).eq.nsig    ) then

c                 --- Point (0,0,-1)
c                     Contribution of matrix is zero
c                     Get boundary condition information
c                     At this momentwe assume that only outflow
c                     is prescribed, i.e.
c                      nn    nt1    nt2
c                     S   = S    = S    = 0

                    do gammal = 1, 3
                      vsum(ip,jp,kp,gammal) = 0d0
                      do deltal = 1, 3
                        dvsum(ip,jp,kp,gammal,deltal) = 0d0
                      end do
                    end do

                    vval(tdir(2)) = 0d0

                    rhside(mip,mjp,mkp,alphad) =
     +                              rhside(mip,mjp,mkp,alphad) +
     +                              nsig*locjac(ip,jp,kp)*vval(tdir(2))

                  end if

               end if

            end if

          end do
        end do
      end do

      call iserclos ( 'isbms13d' )

      end
