      subroutine isbwbs1 ( ni, nj, nvirtual, iside, dvsum, alphad,
     +                     locdx, mu, dcell, ibinfor, ibipoint,
     +                     bndcon, rhside, adir, adiri, adirii, cva )
c ======================================================================
c
c        programmer     Kees Kassels
c        version 1.3    date 24-12-1998 Guus: adaptation for transition
c                                       multiblock fixed boundary
c        version 1.2    date 03-12-1998 adirii: mip --> ip
c        version 1.1    date 25-11-1998 iparea changed
c        version 1.0    date 09-10-1996
c
c   copyright (c) 1996-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c **********************************************************************
c
c                       DESCRIPTION
c
c     ISnas Momentum Matrix Stress at the Boundary in 2 dimensions
c     Tangential cells only, Wesbeek discretization
c **********************************************************************
c
c                       KEYWORDS
c
c     2d
c     boundary_conditions
c     matrix
c     stress
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, iside, dcell(2), alphad,
     +        ibinfor(*), ibipoint(*), locdx(2)
      double precision
     +   mu(1-nvirtual:ni+1+nvirtual,1-nvirtual:nj+1+nvirtual),
     +   dvsum(1-nvirtual:ni+1+nvirtual,1-nvirtual:nj+1+nvirtual,2,2),
     +   adir(1-nvirtual:ni+1+nvirtual,1-nvirtual:nj+1+nvirtual,2),
     +   adiri(1-nvirtual:ni+1+nvirtual,1-nvirtual:nj+1+nvirtual,2),
     +   adirii(1-nvirtual:ni+1+nvirtual,1-nvirtual:nj+1+nvirtual,2,2),
     +   cva(1-nvirtual:ni+1+nvirtual,1-nvirtual:nj+1+nvirtual,2,2,4),
     +   rhside(ni+1,nj+1,2),  bndcon(2,*)

c     adir      i    Contravariant base vector in integration cell center
c     adiri     i    Contravariant base vector in integration point
c     adirii    i    Contravariant base vector in integration point
c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     bndcon    i    reserved
c     cva       i    Array containing the covariant basis vectors
c                    in 2D we have
c                                                 l
c                    cva(i,j,l,alpha,p) contains a        in cell i,j in
c                                                 (alpha)       point type p
c                    in 3D it is cva(i,j,k,l,alpha,p) referring to cell i,j,k
c     dcell     i    reserved
c     dvsum     i    Flux derivative contributions to the matrix
c     ibinfor   i    Third part of infbnd
c     ibipoint  i    ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     iside     i    Sequence number of side
c     locdx     i    Position of the integration point with respect to
c                    the centre of the integration cell
c                    The cell sizes are assumed to be 2 times 2, hence the
c                    distance of a midside point to a cell centre is always 1
c     mu        i    The laminar viscosity
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     rhside    i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer snnsnt, unsnt, iparea(2,2), walllaw
      parameter( snnsnt = 7, unsnt = 5, walllaw = 9 )
      integer j, btype, mip, mjp, mxp(2), ict1, ip, jp, xp(2), ndir,
     +        tdir, nsign, tsign, ipibinf, ipibinsd, betad,
     +        gammad, sign, pnttyp
      double precision factor, gtt, t1, t2, c1(2), c2(2), det
      logical noskip

c     betad     Direction index, usually the direction perpendicular to the
c               integration direction of the boundary integral.
c               For example betad in local point (1,0) is 1 and in (0,1): 2
c     btype     Type of boundary condition
c     c1        General help variable
c     c2        General help variable
c     det       Determinant
c     factor    factors for linear interpolation or
c               difference formula in direction compon
c     gammad    Direction index
c     gtt       Contravariant metric tensor in tt direction
c     ict1      Pointer in ibinfor with respect to boundary point
c     ip        Local counting variable i-direction
c     iparea    Area of integration points where a contribution to
c               the matrix is to be calculated
c     ipibinf   Starting address of array infor for present side minus one
c     ipibinsd  Starting address of array infor for adjacent side minus one
c     j         General loop variable
c     jp        Local counting variable j-direction
c     mip       Integration cell indices
c     mjp       Integration cell indices
c     mxp       Auxiliary array to calculate boundary point number
c     ndir      Direction of normal vector on this side
c     noskip    Indicate whether a given equation is skipped or not
c     nsign     Inner product of outward normal in computational domain
c               with the base vector perpendicular to the face
c     pnttyp    reserved
c     sign      sign for multiplication (1 or -1)
c     snnsnt    Sigma_nn and sigma_nt prescribed
c     t1        Component 1 of tangent vector
c     t2        Component 2 of tangent vector
c     tdir      Direction of tangent vector on this side
c     tsign     Inner product of tangential vector defined counter
c               clockwise with respect to the outward normal and the
c               base vector along the side
c     unsnt     U_n and sigma_nt prescribed
c     walllaw   Law of the wall boundary condition
c     xp        Auxiliary array to calculate boundary point number
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISORIENT2 Computes ndir, nsign, tdir, tsign for a 2-dimensional
c               problem
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c   First we consider the formulae at an internal cell:
c
c
c    The stress terms in case of the Wesbeek discretization are given by:
c
c
c    (a)                (1)  b |i+1/2,j     (2)          b |i,j+1/2
c   a      . { sqrt(g) a    T  |         + a   sqrt(g)  T  |        }
c    (0,0)              b      |i-1/2,j     b              |i,j-1/2
c
c
c
c   Consider a tangential cell at the lower boundary, i.e.
c       ntdir=(2,1)
c       ntsign=(-1,1)
c
c
c
c
c         ---------------+--------------------------
c         |              |            |            |
c         |      |       |     |      |            |
c         |              |            |            |
c         |      +       *     +      |            |
c         |              |            |            |
c         |      |       |     |      |            |
c         |              |            |            |
c         ---------------+--------------------------
c
c   The following boundary conditions have the following effects:
c
c
c   Dirichlet boundary conditions:  do not change the matrix elements
c                                   Use the folding subroutine for the
c                                   virtual velocities
c
c   Sigma nn, ut given:             no sigmann term at the boundary
c                                   Folding routine for ut
c
c   Sigma nt, un given:
c
c      Formulae:
c
c
c     (2)          b |      ||  (2)        ||       b
c    a   sqrt(g)  T  |    = || a    sqrt(g)|| { n (T.n)n + sigma^nt t }
c     b              |i,j   ||             ||    b - - -            -
c
c     with  sigma^nt given
c
c    This subroutine uses all tangential unknowns along the side,
c    except when the first and last unknown (corresponding to adjacent sides),
c    do not correspond to periodical or multiblock boundary conditions
c    In that case the boundary unknown is skipped
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      iparea(1,1) = 1
      iparea(1,2) = 1
      iparea(2,1) = 1
      iparea(2,2) = 1
      ipibinf = ibipoint(iside)+nvirtual-1

c     --- Compute tdir and ndir

      call isorient2 ( iside, tdir, ndir, tsign, nsign )

      if ( iside.le.2 ) then

c     --- iside = 1, 2:  left- and right-hand side

         iparea(1,2) = 1
         iparea(2,2) = nj+1
         if ( iside.eq.1 ) then

c        --- iside = 1:  left-hand side

            ipibinsd = ibipoint(3)+nvirtual-1
            btype = ibinfor(ipibinsd+2)
            noskip = btype.ge.89 .and. btype.le.99
            if ( .not. noskip ) iparea(1,2) = 1
            ipibinsd = ibipoint(4)+nvirtual-1
            btype = ibinfor(ipibinsd+2)
            noskip = btype.ge.89 .and. btype.le.99
            if ( .not. noskip ) iparea(2,2) = nj+1

         else

c        --- iside = 2:  right-hand side

            iparea(1,1) = ni
            iparea(2,1) = ni
            ipibinsd = ibipoint(3)+nvirtual-1
            btype = ibinfor(ipibinsd+2*ni)
            noskip = btype.ge.89 .and. btype.le.99
            if ( .not. noskip ) iparea(1,2) = 1
            ipibinsd = ibipoint(4)+nvirtual-1
            btype = ibinfor(ipibinsd+2*ni)
            noskip = btype.ge.89 .and. btype.le.99
            if ( .not. noskip ) iparea(2,2) = nj+1

         end if

      else

c     --- iside = 3, 4:  lower- and upper side

         iparea(1,1) = 1
         iparea(2,1) = ni+1
         if ( iside.eq.3 ) then

c        --- iside = 3:  lower side

            ipibinsd = ibipoint(1)+nvirtual-1
            btype = ibinfor(ipibinsd+2)
            noskip = btype.ge.89 .and. btype.le.99
            if ( .not. noskip ) iparea(1,1) = 1
            ipibinsd = ibipoint(2)+nvirtual-1
            btype = ibinfor(ipibinsd+2)
            noskip = btype.ge.89 .and. btype.le.99
            if ( .not. noskip ) iparea(2,1) = ni+1

         else

c        --- iside = 4:  upper side

            iparea(1,2) = nj
            iparea(2,2) = nj
            ipibinsd = ibipoint(1)+nvirtual-1
            btype = ibinfor(ipibinsd+2*nj)
            noskip = btype.ge.89 .and. btype.le.99
            if ( .not. noskip ) iparea(1,1) = 1
            ipibinsd = ibipoint(2)+nvirtual-1
            btype = ibinfor(ipibinsd+2*nj)
            noskip = btype.ge.89 .and. btype.le.99
            if ( .not. noskip ) iparea(2,1) = ni+1

         end if

      end if

c     --- loop over cells

      do mjp = iparea(1,2), iparea(2,2)
         do mip = iparea(1,1), iparea(2,1)

c        --- Get integration cell index

            mxp(1) = mip
            mxp(2) = mjp

c           --- Get integration point index

            xp(1) = mxp(1) + dcell(1)
            xp(2) = mxp(2) + dcell(2)

            ip = xp(1)
            jp = xp(2)

c           --- Get integration point position for ibinfor

            ict1 = ipibinf+2*xp(tdir)-1
            btype = ibinfor(ict1)
            if ( btype.eq.walllaw ) btype = unsnt

            if ( btype.eq.unsnt .or. btype.eq.snnsnt ) then

               dvsum(mip,mjp,1,1) = 0d0
               dvsum(mip,mjp,1,2) = 0d0
               dvsum(mip,mjp,2,1) = 0d0
               dvsum(mip,mjp,2,2) = 0d0

c              --- fill dvsum

               if ( abs(locdx(1)).ne.0 ) betad = 1
               if ( abs(locdx(2)).ne.0 ) betad = 2
               sign = -locdx(betad)

               do gammad = 1, 2
                  do j = 1, 2

                     factor = (adir(mip,mjp,1)*adiri(ip,jp,1) +
     +                         adir(mip,mjp,2)*adiri(ip,jp,2) ) *
     +                   (1d0/(adiri(ip,jp,1)*adiri(ip,jp,1)+
     +                         adiri(ip,jp,2)*adiri(ip,jp,2)) )

                     if ( mip.eq. iparea(2,1) ) factor = factor * 5d-1

                     dvsum(mip,mjp,gammad,j) = factor*sign*mu(mip,mjp)*
     +                  adiri(ip,jp,j)*adiri(ip,jp,gammad)
                  end do
               end do

c              --- fill rhside

               gtt = cva(ip,jp,1,tdir,1)*cva(ip,jp,1,tdir,1) +
     +               cva(ip,jp,2,tdir,1)*cva(ip,jp,2,tdir,1)
               t1 = cva(ip,jp,1,tdir,1)/sqrt(gtt)
               t2 = cva(ip,jp,2,tdir,1)/sqrt(gtt)
               factor = (adir(mip,mjp,1)*t1+adir(mip,mjp,2)*t2) *
     +                   sqrt(adiri(ip,jp,1)*adiri(ip,jp,1)+
     +                        adiri(ip,jp,2)*adiri(ip,jp,2))

               if (mip.eq. iparea(2,1) ) then
                  rhside(mip,mjp,alphad) = rhside(mip,mjp,alphad) +
     +                              0.5d0*factor*tsign*bndcon(2,ict1)
               else
                  rhside(mip,mjp,alphad) = rhside(mip,mjp,alphad) +
     +                                 factor * tsign * bndcon(2,ict1)

               end if

c              --- change adirii, now no one side approximation.
c                  In case of diriclet boundary conditions the
c                  integration path for the velocity gradient is
c                  computed with one side approximation.
c                  That is not wanted in this case.

               pnttyp = 2
               c1(1) = 5d-1 * ( cva(ip,jp,1,1,pnttyp) +
     +                                 cva(ip-1,jp,1,1,pnttyp))
               c1(2) = 5d-1 * ( cva(ip,jp,2,1,pnttyp)  +
     +                                 cva(ip-1,jp,2,1,pnttyp))

               pnttyp = 3
               c2(1) = 5d-1 * ( cva(ip,jp,1,2,pnttyp) +
     +                                 cva(ip,jp-1,1,2,pnttyp))
               c2(2) = 5d-1 * ( cva(ip,jp,2,2,pnttyp) +
     +                                 cva(ip,jp-1,2,2,pnttyp))

               det = c2(2)*c1(1)-c1(2)*c2(1)

               adirii(ip,jp,1,1) =  c2(2)/det
               adirii(ip,jp,1,2) = -c2(1)/det
               adirii(ip,jp,2,1) = -c1(2)/det
               adirii(ip,jp,2,2) =  c1(1)/det

            end if

         end do
      end do

      end
