      subroutine iscconv ( alphad, iinput, solut, coor, work, ipveloc,
     +                      iprho, ipvecup1, ipvecup2, pnttyp, locdx,
     +                      npointsv, ibinfor, ibipoint )
c ======================================================================
c
c        programmer     Kees Kassels
c        version 1.0    date 01-03-1999 extracted from ismom2dwb
c
c   copyright (c) 1998-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c   Fill help array vecup1 in array work with start-point ipvecup1,
c   with the coefficient rho*V for the convective term, depending
c   on upwind and conservative or not.
c   Fill help array vecup2 in array work with start-point ipvecup2,
c   with the factor which  determines the direction of upwind.
c **********************************************************************
c
c                       KEYWORDS
c
c    convection
c    upwind
c    conservative
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer alphad, npointsv, iinput(*), ibipoint(*), ibinfor(*),
     +         ipveloc, iprho, ipvecup1, ipvecup2, pnttyp, locdx(2)
      double precision  work(*), coor(*), solut(npointsv,*)

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     coor      i    reserved
c     ibinfor   i    Third part of infbnd
c     ibipoint  i    ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     iinput    i    reserved
c     iprho     i    Starting address of array rho either in BUFFER or WORK
c     ipvecup1  i    Starting address of array VECUP1 in BUFFER
c     ipvecup2  i    Starting address of array VECUP2 in BUFFER
c     ipveloc   i    Starting address of array VELOC in array WORK
c     locdx     i    Position of the integration point with respect to
c                    the centre of the integration cell
c                    The cell sizes are assumed to be 2 times 2, hence the
c                    distance of a midside point to a cell centre is always 1
c     npointsv  i    Number of points in the mesh with virtual points
c     pnttyp    i    reserved
c     solut     i    reserved
c     work      i    Work array
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer  ni, nj, nvirtual, ndegfd,
     +          iptemp,  idir, ipiindsc, vartyp, acttyp,
     +          iconv, ipiintim,  iupwind, typp, vtime, ivt, iconserv

c     acttyp    Type of action to be taken
c               0      : interpolation for point value
c               1..ndim: finite difference for partial derivative
c                        in the direction given by value of acttyp
c     iconserv  If 1 the conservative scheme is used, otherwise (0) the
c               non-conservative one
c     iconv     If 1 the timestep is iterated till converged, otherwise (0)
c               apply only one iterative step
c     idir      General loop variable for 1...ndim loop
c     ipiindsc  Starting address of array IINDSC in array IINPUT
c     ipiintim  Starting address of array IINTIM in array IINPUT
c     iptemp    Starting address of temporary array in IBUFFR
c     iupwind   Type of upwinding
c     ivt       Local counter
c     ndegfd    Number of degrees of freedom
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nvirtual  Number of rows of virtual cells
c     typp      Help variable to store pnttype in case of upwind
c     vartyp    Variable to be interpolated/extrapolated
c               vartyp: 1 for V1, 2 for V2 and 3 for pressure,
c               0 for vertex (for ndim=2)
c               or
c               vartyp: 1 for V1, 2 for V2 3, for V3 and
c               4 for pressure, 0 for vertex  (for ndim=3)
c     vtime     Help variable to compute the velocity at the old time level
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISINTCF2  Compute interpolation of a coefficient in an integration point
c     ISVECUP1  Compute help variable, in cell centre, for first order
c               conservative upwind
c     ISVECUP2  Compute help variable, in cell centre, for first order
c               conservative upwind
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c

      ipiindsc = iinput(6)
      ipiintim = iinput(4)
      ni       = iinput(51)
      nj       = iinput(52)
      nvirtual = iinput(54)
      ndegfd   = iinput(55)
      iupwind  = iinput(ipiindsc-1+5)
      iconserv = iinput(ipiindsc-1+9)
      iconv    = iinput(ipiintim -1 + 17)

c     --- if upwind and conservative then compute twice the velocity
c         once in the integration point and once in the cell centre

      if ( iupwind.eq.1 .and. iconserv.eq. 1 ) then
         vtime = 2
      else
         vtime = 1
      end if

      do ivt = 1, vtime

         if ( ivt.eq.1) then

c        --- compute V in integration point

            typp = pnttyp

         else if ( ivt.eq.2) then

c        --- compute V in cell centre

            typp = 4-alphad

         end if

c        --- Fill velocity componentwise
c            Use second timelevel of velocity for
c            iterative pressure correction

         acttyp = 0
         if ( iconv.eq.0 ) then

c        --- Standard situation the old time level is used

            do idir = 1, 2
               vartyp = idir
               iptemp = ipveloc+(idir-1)*npointsv
               call isintcf2 ( solut(1,idir), work(iptemp),
     +                          ni, nj, nvirtual, vartyp,
     +                          acttyp, typp, 0, coor )
            end do

         else if ( iconv.eq.1 ) then

c        --- iterative pressure correction the new time level is used
c            In the future this may be applied always
c            This situation is only possible once the initial
c            conditions are copied to the new time levels in the
c            correct way, see subroutine isincnd

            do idir = 1, 2
               vartyp = idir
               iptemp = ipveloc+(idir-1)*npointsv
               call isintcf2 ( solut(1,idir+ndegfd),work(iptemp),
     +                          ni, nj, nvirtual, vartyp,
     +                          acttyp, typp, 0, coor )
            end do

         end if

         if ( iupwind.eq.1 .and. ivt.eq.1 ) then

c        --- compute vecup2, using the flux in
c            the integration point

            call isvecup2 ( ni, nj, nvirtual, work(ipveloc),
     +                       work(ipvecup2), alphad, locdx,
     +                       ibinfor, ibipoint )

            if ( iconserv.eq.0 ) then

c           --- compute vecup1, using the flux in
c               the integration point

               call isvecup1 ( alphad, ni, nj, nvirtual,
     +                         work(ipveloc), work(iprho),
     +                         work(ipvecup1), locdx,
     +                         iconserv, ibinfor, ibipoint )
            end if

         else if ( ivt.eq.2 ) then

c        --- compute vecup1, using the flux in
c            the cell centre

            call isvecup1 ( alphad, ni, nj, nvirtual,
     +                      work(ipveloc), work(iprho),
     +                      work(ipvecup1), locdx, iconserv,
     +                      ibinfor, ibipoint )

         end if

      end do

      end
