      subroutine iscdu22d( vold , mnew, pold, switch, shiftr,
     +                   shiftrfo, densup, ni, nj, nvirtual, rinput,
     +                   iinput, eddy, nblocks, nfaces, maxnsubf,
     +                   nsubfaces, subfaces)
c ======================================================================
c
c     programmer     Duncan van der Heul
c     version      1.5    date   02-12-1999  Duncan: revision of supersonic
c                                                    schemes
c     version      1.4    date   01-04-1999  Duncan: Change of density bias
c                                                      gradient_based
c     version      1.3    date   25-03-1999  Duncan: inclusion supersonic mix
c                                                    scheme
c     version      1.2    date   18-03-1999  Duncan: inclusion of Supersonic
c                                                    scheme
c     version      1.1    date   04-03-1999  Duncan: total revision; new
c                                                    call to isdstfl/ls
c                                                    fixed bugs
c     version      1.0    date   22-07-1998
c
c
c   copyright (c) 1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Subroutine ISCDUP2D calculates the contributions to the pressure
c     matrix due to the application of the density bias in the continuity
c     equation.
c     Currently two versions are available:
c     1) based on strict linearisation of the upwinded continuity
c        which only gives an upwind bias in the case of a strong gradient
c        in the pressure
c     2) A variation of the formulation of Karki and Patankar
c     WARNING: the case of density bias is not implemented for the boundaries
c     this means it is expected the density shift will not be activated.
c     Only the SUPERSDC scheme is implemented for the boundaries
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, iinput(*), nblocks, nfaces, maxnsubf,
     +                 nsubfaces(*), subfaces(*)
      double precision shiftr(ni+1,nj+1), switch(ni+1,nj+1),
     +                 vold(1-nvirtual:ni+1+nvirtual,
     +                    1-nvirtual:nj+1+nvirtual),
     +                 densup(ni+1,nj+1,3), rinput(*),
     +                 mnew(1-nvirtual:ni+1+nvirtual,
     +                    1-nvirtual:nj+1+nvirtual),
     +                 pold(1-nvirtual:ni+1+nvirtual,
     +                    1-nvirtual:nj+1+nvirtual),
     +                 eddy(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,*),
     +                 shiftrfo(ni+1,nj+1)

c     densup   i/o   coefficients of density upwind in pressure matrix
c     eddy      i    reserved
c     iinput    i    reserved
c     maxnsubf  i    Maximum number of subfaces encountered on all subfaces
c     mnew      i    Momentum flux at the present time level
c     nblocks   i    Number of blocks
c     nfaces    i    Number of faces
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nsubfaces i    Array containing the number of subfaces of each face,
c                    see array itopol for an extended description
c     nvirtual  i    Number of rows of virtual cells
c     pold      i    Pressure at the old time level
c     rinput    i    reserved
c     shiftr    i    Shift variable used in compressible density upwinding.
c     shiftrfo  i    Shift variable used in compressible density upwinding,
c                    first order.
c     subfaces  i    reserved
c     switch    i    Continuous compressible density upwind switch
c                      switch = 0 for Mach numbers < 1-ls
c                      switch=0.5 for Mach numbers > 1+ls
c     vold      i    Velocity at previous time step
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j, k, mscal, ichoicedb, iprincom, istartup, mcav,
     +                      iprincav
      logical periodic

c     i         General loop variable
c     ichoicedb if 1 a gradient based, if 2 a mach based,
c               if 3 an unconditional density bias is applied
c     iprincav  Starting address of array RINCAV in array RINPUT
c     iprincom  Starting address of array RINCOM in array RINPUT
c     istartup  If 1 the cavity calculation is in the startup procedure,
c               if 0 not
c     j         General loop variable
c     k         General loop variable
c     mcav      If 1 a cavity model is used, if 0 not
c     mscal     Indicates use of non-dimensionalisation for compr. flow:
c               0:   Equations are non-dimensionalised
c               1:   Equations are in dimensional formulation
c     periodic  Periodic boundary conditions
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      double precision isdstfll, isdstlss

c     ISCHECKPERJ Check for periodic boundary conditions
c     ISDSTFLL    Derivative of dimensional equation of state
c     ISDSTLSS    Derivative of dimensionless equation of state
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      ichoicedb =  iinput(iinput(9)-1 +7)
      mscal     =  iinput(iinput(9)+4)
      mcav      =  iinput(77)
      if( mcav.eq.1)
     +      istartup=iinput(iinput(11)-1+2)
      iprincom  =  iinput(34)
      iprincav  =  iinput(35)

c     --- reset densup

      do k=1,3
         do j=1,nj+1
            do i=1,ni+1
               densup(i,j,k)=0d0
            end do
         end do
      end do

      if( ichoicedb.eq.1) then

         do j=2, nj-1

           if( mscal.eq.0) then

              do i=1, ni+1
                 if( abs(mnew(i,j)).gt.1d-12) then
                    densup(i,j,1) = -1d0*abs(mnew(i,j))*
     +              1d0/(eddy(i,j-1,2)+eddy(i,j,2))**2*(
     +              isdstlss((pold(i,j-1)+pold(i,j))/2d0,
     +                          rinput(iprincom), istartup,
     +                                        rinput(iprincav))*
     +              (pold(i,j)-pold(i,j-1))+1d0*
     +               (eddy(i,j-1,2)+eddy(i,j,2)))
     +                              -1d0*mnew(i,j)*
     +              (shiftrfo(i,j)-shiftr(i,j))/
     +               (eddy(i,j-1,2)+eddy(i,j,2))
                 end if
              end do

            else if( mscal.eq.1) then

              do i=1, ni+1

                 if( abs(mnew(i,j)).gt.1d-12) then
                    densup(i,j,1) =  -1d0*abs(mnew(i,j))*
     +              1d0/(eddy(i,j-1,2)+eddy(i,j,2))**2*(
     +              isdstfll((pold(i,j-1)+pold(i,j))*0.5d0,
     +                                            istartup,
     +                                        rinput(iprincav))*
     +              (pold(i,j)-pold(i,j-1))+1d0*
     +               (eddy(i,j-1,2)+eddy(i,j,2)))
     +                              -1d0*mnew(i,j)*
     +              (shiftrfo(i,j)-shiftr(i,j))/
     +               (eddy(i,j-1,2)+eddy(i,j,2))
                 end if

               end do

            end if

         end do

         do j=2, nj-1

           if( mscal.eq. 0) then

              do i=1, ni+1

                 densup(i,j,2) = 0d0
                 if( abs(mnew(i,j)).gt.1d-12) then

                    densup(i,j,2) = -1d0*abs(mnew(i  ,j))*
     +              1d0/(eddy(i,j-1,2)+eddy(i  ,j,2))**2*(
     +              isdstlss((pold(i,j-1)+pold(i  ,j))*0.5d0,
     +                          rinput(iprincom), istartup,
     +                                        rinput(iprincav))*
     +              (pold(i  ,j)-pold(i,j-1))-1d0*
     +               (eddy(i,j-1,2)+eddy(i  ,j,2)))
     +                              -1d0*mnew(i,j)*
     +              (shiftrfo(i,j)-shiftr(i,j))/
     +               (eddy(i,j-1,2)+eddy(i,j,2))

                 end if

                 if( abs(mnew(i,j+1)).gt.1d-12) then

                    densup(i,j,2) =  densup(i,j,2)+
     +                               1d0*abs(mnew(i,j+1))*
     +              1d0/(eddy(i,  j,2)+eddy(i,j+1,2))**2*(
     +              isdstlss((pold(i,  j)+pold(i,j+1))*0.5d0,
     +                          rinput(iprincom), istartup,
     +                                        rinput(iprincav))*
     +              (pold(i,j+1)-pold(i,  j))+1d0*
     +               (eddy(i,  j,2)+eddy(i,j+1,2)))
     +                              +1d0*mnew(i,j+1)*
     +              (shiftrfo(i,j+1)-shiftr(i,j+1))/
     +               (eddy(i,j+1,2)+eddy(i,j,2))

                 end if
              end do

           else if( mscal.eq.1) then

              do i=1, ni+1

                 if( abs(mnew(i,j)).gt.1d-12) then

                    densup(i,j,2) = -1d0*abs(mnew(i  ,j))*
     +              1d0/(eddy(i,j-1,2)+eddy(i  ,j,2))**2*(
     +              isdstfll((pold(i,j-1)+pold(i  ,j))*0.5d0,
     +                                            istartup,
     +                                        rinput(iprincav))*
     +              (pold(i  ,j)-pold(i,j-1))-1d0*
     +               (eddy(i,j-1,2)+eddy(i  ,j,2)))
     +                              -1d0*mnew(i,j)*
     +              (shiftrfo(i,j)-shiftr(i,j))/
     +               (eddy(i,j-1,2)+eddy(i,j,2))

                 end if

                 if( abs(mnew(i,j+1)).gt.1d-12) then

                    densup(i,j,2) =  densup(i,j,2)+
     +                               1d0*abs(mnew(i,j+1))*
     +              1d0/(eddy(i,  j,2)+eddy(i,j+1,2))**2*(
     +              isdstfll((pold(i,  j)+pold(i,j+1))*0.5d0,
     +                                            istartup,
     +                                        rinput(iprincav))*
     +              (pold(i,j+1)-pold(i,j  ))+1d0*
     +               (eddy(i,  j,2)+eddy(i,j+1,2)))
     +                              +1d0*mnew(i,j+1)*
     +              (shiftrfo(i,j+1)-shiftr(i,j+1))/
     +               (eddy(i,j+1,2)+eddy(i,j,2))

                 end if

              end do

           end if

         end do

         do j=2, nj-1

           if(mscal.eq.0) then

              do i=1, ni+1

                 if( abs(mnew(i,j+1)).gt.1d-12) then

                    densup(i,j,3) = 1d0*abs(mnew(i,j+1))*
     +              1d0/(eddy(i,  j,2)+eddy(i,j+1,2))**2*(
     +              isdstlss((pold(i,  j)+pold(i,j+1))*0.5d0,
     +                          rinput(iprincom), istartup,
     +                                        rinput(iprincav))*
     +              (pold(i,j+1)-pold(i,j  ))-1d0*
     +               (eddy(i,  j,2)+eddy(i,j+1,2)))
     +                              +1d0*mnew(i,j+1)*
     +              (shiftrfo(i,j+1)-shiftr(i,j+1))/
     +               (eddy(i,j+1,2)+eddy(i,j,2))

                 end if

              end do

           else if( mscal.eq.1) then

              do i=1, ni+1

                 if( abs(mnew(i,j+1)).gt.1d-12) then

                    densup(i,j,3) =  1d0*abs(mnew(i,j+1))*
     +              1d0/(eddy(i,  j,2)+eddy(i,j+1,2))**2*(
     +              isdstfll((pold(i,  j)+pold(i,j+1))*0.5d0,
     +                                            istartup,
     +                                        rinput(iprincav))*
     +              (pold(i,j+1)-pold(i,  j))-1d0*
     +               (eddy(i,j  ,2)+eddy(i,j+1,2)))
     +                              +1d0*mnew(i,j+1)*
     +              (shiftrfo(i,j+1)-shiftr(i,j+1))/
     +               (eddy(i,j+1,2)+eddy(i,j,2))

                 end if

              end do

           end if

         end do

      else if(ichoicedb.ge.2.and.ichoicedb.le.3) then

         do i=1, ni

           periodic = .FALSE.

           call ischeckperj(iblockrf, ni, nj, nblocks,
     +                     nfaces, maxnsubf, nsubfaces,
     +                     subfaces, i, 1, periodic)
           if( periodic) then

             densup(i,1,1) = -1d0*(
     +                 vold(i,1)*(.5d0*(1d0-shiftr(i,1))
     +                                         +switch(i,1)) )
           else

             densup(i,1,1)   = 0d0

           end if

         end do

         do j=2, nj-1

           do i=1, ni

             densup(i,j,1) = -1d0*(
     +                 vold(i,j)*(.5d0*(1d0-shiftr(i,j))+switch(i,j)) )

           end do

         end do

         do i=1, ni

           periodic = .FALSE.

           call ischeckperj(iblockrf, ni, nj, nblocks,
     +                     nfaces, maxnsubf, nsubfaces,
     +                     subfaces, i, nj, periodic)
           if( periodic) then

             densup(i,nj,1) = -1d0*(
     +                 vold(i,nj)*(.5d0*(1d0-shiftr(i,nj))
     +                                          +switch(i,nj)) )

            else

             densup(i,nj,1)   = 0d0

           end if

         end do

         do i=1, ni

           periodic = .FALSE.

           call ischeckperj(iblockrf, ni, nj, nblocks,
     +                     nfaces, maxnsubf, nsubfaces,
     +                     subfaces, i, 1, periodic)
           if( periodic) then

              densup(i,1,2) = 1d0*(
     +               vold(i,1+1)*(.5d0*(1d0-shiftr(i,1+1))+
     +               switch(i,1+1))
     +               -vold(i,1)*(.5d0*(1d0-shiftr(i,1))-
     +               switch(i,1)))
           else

              densup(i,1,2)   = 0d0

           end if

         end do

         do j=2, nj-1

           do i=1, ni

             densup(i,j,2) = 1d0*(
     +                 vold(i,j+1)*(.5d0*(1d0-shiftr(i,j+1))+
     +                 switch(i,j+1))
     +                 -vold(i,j)*(.5d0*(1d0-shiftr(i,j))-
     +                 switch(i,j)))
           end do

         end do

         do i=1, ni

           periodic = .FALSE.

           call ischeckperj(iblockrf, ni, nj, nblocks,
     +                     nfaces, maxnsubf, nsubfaces,
     +                     subfaces, i, nj, periodic)
           if( periodic) then

              densup(i,1,2) = 1d0*(
     +               vold(i,nj+1)*(.5d0*(1d0-shiftr(i,nj+1))+
     +               switch(i,nj+1))
     +               -vold(i,nj)*(.5d0*(1d0-shiftr(i,nj))-
     +               switch(i,nj)))

           else

              densup(i,nj,2)   = 0d0

           end if

         end do

         do i=1, ni

           periodic = .FALSE.

           call ischeckperj(iblockrf, ni, nj, nblocks,
     +                     nfaces, maxnsubf, nsubfaces,
     +                     subfaces, i, 1, periodic)
           if( periodic) then

             densup(i,1,3) = 1d0*(vold(i,1+1)*
     +                    (.5d0*(1d0-shiftr(i,1+1))
     +                 -switch(i,1+1)) )
           else
             densup(i,1,3) = 0d0
           end if

         end do

         do j=2, nj-1

           do i=1, ni

             densup(i,j,3) = 1d0*(vold(i,j+1)*
     +                    (.5d0*(1d0-shiftr(i,j+1))
     +                 -switch(i,j+1)) )

           end do

         end do

         do i=1, ni

           periodic = .FALSE.

           call ischeckperj(iblockrf, ni, nj, nblocks,
     +                     nfaces, maxnsubf, nsubfaces,
     +                     subfaces, i, nj, periodic)
           if( periodic) then
             densup(i,nj,3) = 1d0*(vold(i,nj+1)*
     +                    (.5d0*(1d0-shiftr(i,nj+1))
     +                 -switch(i,nj+1)) )
           else
             densup(i,nj,3) = 0d0
           end if

         end do

      else if(ichoicedb.eq.4) then

         do j=1, nj

           do i=1, ni+1

             densup(i,j,1) =  -switch(i,j)*
     +                       vold(i,j)*(0.5d0*(1d0+(vold(i,j)+1d-12)/
     +                                        (abs(vold(i,j))+1d-12)))

           end do

         end do

         do j=1, nj

           do i=1, ni

             densup(i,j,2) =  -switch(i,j)*
     +                  vold(i,j  )*(0.5d0*(1d0-(vold(i,  j)+1d-12)/
     +                                      (abs(vold(i,  j))+1d-12)))
     +                 +     switch(i,j+1)*
     +                  vold(i,j+1)*(0.5d0*(1d0+(vold(i,j+1)+1d-12)/
     +                                      (abs(vold(i,j+1))+1d-12)))
           end do

         end do

         do j=1, nj

           do i=1, ni

             densup(i,j,3) =  switch(i,j+1)*
     +                  vold(i,j+1)*(0.5d0*(1d0-(vold(i,j+1)+1d-12)/
     +                                      abs(vold(i,j+1)+1d-12)))

           end do

         end do

      else if( ichoicedb.eq.5 .or. ichoicedb.eq.6) then

         do i=1, ni

           periodic = .FALSE.

           call ischeckperj(iblockrf, ni, nj, nblocks,
     +                     nfaces, maxnsubf, nsubfaces,
     +                     subfaces, i, 1, periodic)

           if( periodic ) then

             densup(i,1,1) =(1d0-switch(i,1  ))*( -1d0*(
     +           vold(i,1)*(.5d0*(1d0-shiftr(i,1))+sign(0.5d0,
     +                   vold(i,1))) ))+
     +
     +                            switch(i,1  ) *(
     +                   -vold(i,1)*(0.5d0*(1d0+(vold(i,1)+1d-12)/
     +                                   (abs(vold(i,1))+1d-12))))
           else

             densup(i,1,1)   = 0d0

           end if

         end do

         do j=2, nj

           do i=1, ni

             densup(i,j,1) =(1d0-switch(i,j  ))*( -1d0*(
     +           vold(i,j)*(.5d0*(1d0-shiftr(i,j))+sign(0.5d0,
     +                   vold(i,j))) ))+
     +
     +                            switch(i,j  ) *(
     +                   -vold(i,j)*(0.5d0*(1d0+(vold(i,j)+1d-12)/
     +                                   (abs(vold(i,j))+1d-12))))
           end do

         end do

         do i=1, ni

           densup(i,1,2) = (1d0-switch(i,1+1))*(
     +              vold(i,1+1)*(.5d0*(1d0-shiftr(i,1+1))+
     +              sign(0.5d0,vold(i,1+1))))+
     +                          switch(i,1+1) *(
     +              vold(i,1+1)*(0.5d0*(1d0+(vold(i,1+1)+1d-12)/
     +                                 (abs(vold(i,1+1))+1d-12))))

           periodic = .FALSE.

           call ischeckperj(iblockrf, ni, nj, nblocks,
     +                     nfaces, maxnsubf, nsubfaces,
     +                     subfaces, i, 1, periodic)

           if( periodic ) then

              densup(i,1,2) = densup(i,1,2) +
     +                        (1d0-switch(i,1  ))*(
     +                -vold(i,1)*(.5d0*(1d0-shiftr(i,1))-
     +                 sign(0.5d0,vold(i,1  ))))+
     +
     +                              switch(i,  1) *(
     +                -vold(i,  1)*(0.5d0*(1d0-(vold(i  ,1)+1d-12)/
     +                                  (abs(vold(i  ,1))+1d-12))))
           end if

         end do

         do j=2, nj-1

           do i=1, ni

             densup(i,j,2) = (1d0-switch(i,j+1))*(
     +                 vold(i,j+1)*(.5d0*(1d0-shiftr(i,j+1))+
     +                 sign(0.5d0,vold(i,j+1))))+
     +                        (1d0-switch(i,j  ))*(
     +                -vold(i,j)*(.5d0*(1d0-shiftr(i,j))-
     +                 sign(0.5d0,vold(i,j  ))))+
     +
     +                              switch(i,  j) *(
     +                -vold(i  ,j)*(0.5d0*(1d0-(vold(i  ,j)+1d-12)/
     +                                  (abs(vold(i  ,j))+1d-12))))+
     +                             switch(i,j+1) *(
     +                 vold(i,j+1)*(0.5d0*(1d0+(vold(i,j+1)+1d-12)/
     +                                    (abs(vold(i,j+1))+1d-12))))
           end do

         end do

         do i=1, ni

           densup(i,nj,2) = (1d0-switch(i,nj  ))*(
     +             -vold(i,nj)*(.5d0*(1d0-shiftr(i,nj))-
     +              sign(0.5d0,vold(i,nj  ))))+
     +
     +                           switch(i, nj) *(
     +             -vold(i,  nj)*(0.5d0*(1d0-(vold(i,  nj)+1d-12)/
     +                               (abs(vold(i  ,nj))+1d-12))))

           periodic = .FALSE.

           call ischeckperj(iblockrf, ni, nj, nblocks,
     +                     nfaces, maxnsubf, nsubfaces,
     +                     subfaces, i, nj, periodic)

           if( periodic ) then

              densup(i,nj,2) = densup(i,nj,2) +
     +                        (1d0-switch(i,nj+1))*(
     +                 vold(i,nj+1)*(.5d0*(1d0-shiftr(i,nj+1))+
     +                 sign(0.5d0,vold(i,nj+1))))+
     +                             switch(i,nj+1) *(
     +                 vold(i,nj+1)*(0.5d0*(1d0+(vold(i,nj+1)+1d-12)/
     +                                    (abs(vold(i,nj+1))+1d-12))))

           end if

         end do

         do j=1, nj-1

           do i=1, ni

             densup(i,j,3) = (1d0-switch(i,j+1))*(
     +                 vold(i,j+1)*(.5d0*(1d0-shiftr(i,j+1))
     +                 -sign(0.5d0,vold(i,j+1)))         )+
     +
     +                            switch(i,j+1) *(
     +                 vold(i,j+1)*(0.5d0*(1d0-(vold(i,j+1)+1d-12)/
     +                                    (abs(vold(i,j+1))+1d-12))))

           end do

         end do

         do i=1, ni

           periodic = .FALSE.

           call ischeckperj(iblockrf, ni, nj, nblocks,
     +                     nfaces, maxnsubf, nsubfaces,
     +                     subfaces, i, nj, periodic)

           if( periodic ) then

             densup(i,nj,3) = (1d0-switch(i,nj+1))*(
     +                 vold(i,nj+1)*(.5d0*(1d0-shiftr(i,nj+1))
     +                 -sign(0.5d0,vold(i,nj+1)))         )+
     +
     +                            switch(i,nj+1) *(
     +                 vold(i,nj+1)*(0.5d0*(1d0-(vold(i,nj+1)+1d-12)/
     +                                    (abs(vold(i,nj+1))+1d-12))))

           else

             densup(i,nj,3) = 0d0

           end if

         end do

      end if

      end
