      subroutine ischeckperj(iblock, ni, nj, nblocks, nfaces,
     +            maxnsubf, nsubfaces, subfaces, i, j, periodic)
c ======================================================================
c
c     programmer   Duncan van der Heul
c
c     version      1.0  7-5-1999
c
c   copyright (c) 1994-1999  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    regulate the shift of the unknown for the nonlinear Gauss-Seidel
c    algorithm in case of periodical boundary conditions. Only
c    implementation for the case when the connection is made across
c    identical running indices, an j= constant face connected to an
c    j = constant face , or an i =constant face connected to an i is
c    constant face.
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iblock, maxnsubf, nblocks,  nfaces, ni, nj,
     +        nsubfaces(1:nblocks, 1:nfaces),
     +        subfaces(1:nblocks, 1:nfaces, 1:maxnsubf, *),
     +        i, j
      logical ininterv, periodic

c     i         i    General loop variable
c     iblock    i    Block sequence number
c     j         i    General loop variable
c     maxnsubf  i    Maximum number of subfaces encountered on all subfaces
c     nblocks   i    Number of blocks
c     nfaces    i    Number of faces
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nsubfaces i    Array containing the number of subfaces of each face,
c                    see array itopol for an extended description
c     periodic  o    Periodic boundary conditions
c     subfaces  i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ibto(4), iface, isubface, bpi,
     +        jhigh, jlow, ihigh, ilow, neigsface

c     bpi       boundary index
c     ibto      Target block
c     iface     Face number
c     ihigh     Upper index of subface
c     ilow      Lower index of subface
c     ininterv  Logical, .true. of boundary point belongs to subface
c               under consideration
c     isubface  Subface number
c     jhigh     Upper j-index for a subface
c     jlow      Lower j-index for a subface
c     neigsface Index of neighbouring subface
c **********************************************************************
c
c                       I/O
c
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c

c     --- determine which faces are internal to the domain

      do iface=1, nfaces
        ibto(iface)= 0
        do isubface = 1, nsubfaces(iblock, iface)
          ibto(iface) = max(ibto(iface),
     +             subfaces(iblock, iface, isubface, 5))
        end do

      end do

c     --- determine if a point is a boundary point, bpi indicates
c         what the position of the face is

      bpi = 0
      if ( j.eq.1 )  bpi = 3
      if ( j.eq.nj ) bpi = 4

c     --- take care of the corner points, they have to be put in the
c          faces with the periodical boundary condition

c         first corner point i=1, normally in face 1, but if face 3 is
c         periodical
c         in face 3

       if(j.eq.1 .and. i.eq.1) bpi = 1
       if(j.eq.1 .and. i.eq.1 .and. ibto(3).ne.0) bpi = 3

c      --- second corner point i=ni, normally in face 2, but if face 3 is
c          --- periodical
c          in face 3

       if(j.eq.1 .and. i.eq.ni) bpi = 2
       if(j.eq.1 .and. i.eq.ni .and. ibto(3).ne.0) bpi = 3

c      --- third corner point j=(nj-1)*(ni+1)+1, normally in face 1,
c          but if face 4 is periodical
c          in face 4

       if(j.eq.nj .and. i.eq.1) bpi = 1
       if(j.eq.nj .and. i.eq.1 .and. ibto(4).ne.0) bpi = 4

c      --- fourth corner point j=(nj-1)*(ni+1)+ni, normally in face 2,
c          but if face 3 is periodical
c          in face 3

       if(j.eq.nj .and. i.eq.ni) bpi = 2
       if(j.eq.nj .and. i.eq.ni .and. ibto(4).ne.0) bpi = 4

c      --- if the point under consideration is a boundary point and the face
c          contains a periodical boundary condition , special action has to be
c          taken, otherwise standard matrix vector multiplication

      if(ibto(bpi).ne.0 .and. bpi.ne.0) then

c     --- the point lies on a face with a periodical part. Determine the
c         interval boundaries of the defining subfaces of the face and
c         check if this point lies on a periodical subface

          if( bpi.eq.1) then

c         --- left face of the block

             do isubface = 1, nsubfaces(iblock, bpi)
                ininterv= .FALSE.
                jlow  = subfaces(iblock, bpi, isubface, 1)
                jhigh = subfaces(iblock, bpi, isubface, 2)

c               --- note that the first points of the subfaces next to the
c                   --- periodic
c                   subface are included in the periodic subface. This is
c                   possible because
c                   the matrix has been folded, and matrix connections to non
c                   periodical
c                   points have been set to zero

                ininterv= ( j.ge.(jlow-1) .and.
     +                           j.le.(jhigh+1) )

                if( ininterv ) then

c               --- if in interval then obtain the neigbouring face and
c                   --- subface

                   neigsface=
     +                     subfaces(iblock, bpi, isubface, 7)

                   periodic=(neigsface.ne.0)

                end if

             end do

          end if

          if( bpi.eq.2) then

c         --- right face of the block, special treatment for connections
c              4, 6 and 9

             do isubface = 1, nsubfaces(iblock, bpi)
                ininterv= .FALSE.
                jlow  = subfaces(iblock, bpi, isubface, 1)
                jhigh = subfaces(iblock, bpi, isubface, 2)

c               --- note that the first points of the subfaces next to the
c                   --- periodic
c                   subface are included in the periodic subface. This is
c                   possible because
c                   the matrix has been folded, and matrix connections to non
c                   periodical
c                   points have been set to zero

                ininterv= ( j.ge.(jlow-1) .and.
     +                         j.le.(jhigh+1) )

                if( ininterv ) then

c               --- if in interval then obtain the neigbouring face and
c                   --- subface

                   neigsface=
     +                     subfaces(iblock, bpi, isubface, 7)

                   periodic=(neigsface.ne.0)

                end if

             end do

          end if

          if( bpi.eq.3) then

c         --- lower face of the block, special treatment for connections
c              2, 3 and 4

             do isubface = 1, nsubfaces(iblock, bpi)
                ininterv= .FALSE.
                ilow  = subfaces(iblock, bpi, isubface, 1)
                ihigh = subfaces(iblock, bpi, isubface, 2)

c               --- note that the first points of the subfaces next to the
c                   --- periodic
c                   subface are included in the periodic subface. This is
c                   possible because
c                   the matrix has been folded, and matrix connections to non
c                   periodical
c                   points have been set to zero

                ininterv= ( i.ge.(ilow-1) .and.
     +                          i.le.(ihigh+1) )

                if( ininterv ) then

c               --- if in interval then obtain the neigbouring face and
c                   --- subface

                   neigsface=
     +                     subfaces(iblock, bpi, isubface, 7)

                   periodic=(neigsface.ne.0)

                end if

             end do

          end if

          if( bpi.eq.4) then

c         --- upper face of the block, special treatment for connections
c              7, 8 and 9

             do isubface = 1, nsubfaces(iblock, bpi)
                ininterv= .FALSE.
                ilow  = subfaces(iblock, bpi, isubface, 1)
                ihigh = subfaces(iblock, bpi, isubface, 2)

c               --- note that the first points of the subfaces next to the
c                   --- periodic
c                   subface are included in the periodic subface. This is
c                   possible because
c                   the matrix has been folded, and matrix connections to non
c                   periodical
c                   points have been set to zero

                ininterv= ( i.ge.(ilow) .and.
     +                           i.le.(ihigh) )

                if( ininterv ) then

c               --- if in interval then obtain the neigbouring face and
c                   --- subface

                   neigsface=
     +                     subfaces(iblock, bpi, isubface, 7)

                   periodic=(neigsface.ne.0)

                end if

             end do

          end if

      end if

      end
