      subroutine iscont3d ( adir, jac, cva, ni, nj, nk,
     +                      nvirtual, alphad )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.0    date 12-02-1998
c
c
c   copyright (c) 1998-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Compute contravariant base vectors in centroids of alphad cells
c    three-dimensional case
c **********************************************************************
c
c                       KEYWORDS
c
c    base_vectors
c    interpolation
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nk, nvirtual, alphad
      double precision adir(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual,*),
     +                 cva(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,
     +                     1-nvirtual:nk+1+nvirtual,3,3,*),
     +                 jac(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,
     +                     1-nvirtual:nk+1+nvirtual,*)

c     adir      o    Contravariant base vector in integration cell center
c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     cva       i    Array containing the covariant basis vectors
c                    in 2D we have
c                                                 l
c                    cva(i,j,l,alpha,p) contains a        in cell i,j in
c                                                 (alpha)       point type p
c                    in 3D it is cva(i,j,k,l,alpha,p) referring to cell i,j,k
c     jac       i    sqrt(g) in pnttyp
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j, k, pnttyp, valpha(3), beta, vbeta(3), iupploop,
     +        jupploop, kupploop

c     beta      Parameter beta for turbulence modelling
c     i         General loop variable
c     iupploop  Upper bound for loop
c     j         General loop variable
c     jupploop  Upper bound for loop
c     k         General loop variable
c     kupploop  Upper bound for loop
c     pnttyp    reserved
c     valpha    Array for cyclic changing ordering
c               alphal = 1       alphal = 2   alphal = 3
c               valpha(1) =1 ---> valpha(1)=2  valpha(1)=3
c               valpha(2) =2 ---> valpha(2)=3  valpha(2)=1
c               valpha(3) =3 ---> valpha(3)=1  valpha(3)=2
c     vbeta     Array for cyclic changing ordering
c               betal = 1        betal = 2    betal = 3
c               vbeta(1) =1 ---> vbeta(1)=2  vbeta(1)=3
c               vbeta(2) =2 ---> vbeta(2)=3  vbeta(2)=1
c               vbeta(3) =3 ---> vbeta(3)=1  vbeta(3)=2
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     The contravariant base vectors may be computed from the covariant ones
c     in the following way:
c
c     sqrt(g) x contravariant base vector in 3D is given by:
c
c     sqrt(g) a^alpha     = a_beta x a_ gamma   for alpha, beta, gamma cyclic
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      valpha(1) = alphad
      valpha(2) = mod(alphad  ,3) + 1
      valpha(3) = mod(alphad+1,3) + 1
      iupploop = ni
      jupploop = nj
      kupploop = nk
      if ( alphad.eq.1 ) then
         pnttyp = 7
         iupploop = ni+1
      else if ( alphad.eq.2 ) then
         pnttyp = 6
         jupploop = nj+1
      else
         pnttyp = 4
         kupploop = nk+1
      end if

      do beta = 1, 3
         vbeta(1) = beta
         vbeta(2) = mod(beta  ,3) + 1
         vbeta(3) = mod(beta+1,3) + 1
         do k = 1, kupploop
            do j = 1, jupploop
               do i = 1, iupploop
                  adir(i,j,k,beta) =
     +                (cva(i,j,k,vbeta(2),valpha(2),pnttyp)*
     +                 cva(i,j,k,vbeta(3),valpha(3),pnttyp)-
     +                 cva(i,j,k,vbeta(3),valpha(2),pnttyp)*
     +                 cva(i,j,k,vbeta(2),valpha(3),pnttyp))/
     +                 jac(i,j,k,pnttyp)
               end do
            end do
         end do
      end do

      end
