      subroutine iscontg ( adirii, invjac, cva, ni, nj,
     +                     nvirtual, locdx, alphad )
c ======================================================================
c
c        programmer     Kees Kassels
c        version 1.0    date 11-07-1996
c
c
c   copyright (c) 1996-1996  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Compute integration path coefficients
c    two-dimensional case
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, alphad, locdx(2)
      double precision adirii(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,2,2),
     +                 cva(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,2,2,4),
     +                 invjac(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,4)

c     adirii    i    Contravariant base vector in integration point
c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     cva       i    Array containing the covariant basis vectors
c                    in 2D we have
c                                                 l
c                    cva(i,j,l,alpha,p) contains a        in cell i,j in
c                                                 (alpha)       point type p
c                    in 3D it is cva(i,j,k,l,alpha,p) referring to cell i,j,k
c     invjac    i    1/sqrt(g) in pnttyp
c     locdx     i    Position of the integration point with respect to
c                    the centre of the integration cell
c                    The cell sizes are assumed to be 2 times 2, hence the
c                    distance of a midside point to a cell centre is always 1
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j, pnttyp
      double precision c1(2), c2(2), det

c     c1        General help variable
c     c2        General help variable
c     det       Determinant
c     i         General loop variable
c     j         General loop variable
c     pnttyp    reserved
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     The contravariant base vectors may be computed from the covariant ones
c     in the following way:
c
c      (1)                  2     1
c     a    =  1/sqrt(g) ( a    ,-a   )
c                          (2)    (2)
c
c
c      (2)                  2     1
c     a    =  1/sqrt(g) (-a    , a   )
c                          (1)    (1)
c
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      if ( alphad.eq.1 ) then

c     --- alphad = 1 cell

         if ( locdx(1).ne.0 ) then

            pnttyp = 4

            do j = 1, nj +1
               do i = 0, ni+1
                  adirii(i,j,1,1) =  cva(i,j,2,2,pnttyp) *
     +                             invjac(i,j,pnttyp)
                  adirii(i,j,1,2) = -cva(i,j,1,2,pnttyp) *
     +                             invjac(i,j,pnttyp)
                  adirii(i,j,2,1) = -cva(i,j,2,1,pnttyp) *
     +                             invjac(i,j,pnttyp)
                  adirii(i,j,2,2) =  cva(i,j,1,1,pnttyp) *
     +                             invjac(i,j,pnttyp)
               end do
            end do

         else

            do j = 2, nj
               do i = 0, ni+1

                  pnttyp = 2
                  c1(1) = 5d-1 * ( cva(i,j,1,1,pnttyp) +
     +                                     cva(i-1,j,1,1,pnttyp))
                  c1(2) = 5d-1 * ( cva(i,j,2,1,pnttyp)  +
     +                                     cva(i-1,j,2,1,pnttyp))

                  pnttyp = 3
                  c2(1) = 5d-1 * ( cva(i,j,1,2,pnttyp) +
     +                                     cva(i,j-1,1,2,pnttyp))
                  c2(2) = 5d-1 * ( cva(i,j,2,2,pnttyp) +
     +                                     cva(i,j-1,2,2,pnttyp))

                  det = c2(2)*c1(1)-c1(2)*c2(1)

                  adirii(i,j,1,1) =  c2(2)/det
                  adirii(i,j,1,2) = -c2(1)/det
                  adirii(i,j,2,1) = -c1(2)/det
                  adirii(i,j,2,2) =  c1(1)/det

               end do
            end do

            j = 1
            do i = 0, ni+1

               pnttyp = 2
               c1(1) = 5d-1 * ( cva(i,j,1,1,pnttyp) +
     +                                     cva(i-1,j,1,1,pnttyp))
               c1(2) = 5d-1 * ( cva(i,j,2,1,pnttyp)  +
     +                                     cva(i-1,j,2,1,pnttyp))

               pnttyp = 3
               c2(1) = 5d-1 * ( cva(i,j,1,2,pnttyp) )
               c2(2) = 5d-1 * ( cva(i,j,2,2,pnttyp) )

               det = c2(2)*c1(1)-c1(2)*c2(1)

               adirii(i,j,1,1) =  c2(2)/det
               adirii(i,j,1,2) = -c2(1)/det
               adirii(i,j,2,1) = -c1(2)/det
               adirii(i,j,2,2) =  c1(1)/det

            end do

            j = nj+1
            do i = 0, ni+1

               pnttyp = 2
               c1(1) = 5d-1 * ( cva(i,j,1,1,pnttyp) +
     +                                     cva(i-1,j,1,1,pnttyp))
               c1(2) = 5d-1 * ( cva(i,j,2,1,pnttyp)  +
     +                                     cva(i-1,j,2,1,pnttyp))

               pnttyp = 3
               c2(1) = 5d-1 * ( cva(i,j-1,1,2,pnttyp) )
               c2(2) = 5d-1 * ( cva(i,j-1,2,2,pnttyp) )

               det = c2(2)*c1(1)-c1(2)*c2(1)

               adirii(i,j,1,1) =  c2(2)/det
               adirii(i,j,1,2) = -c2(1)/det
               adirii(i,j,2,1) = -c1(2)/det
               adirii(i,j,2,2) =  c1(1)/det

            end do

         end if

      else

c     --- alphad = 2 cell

          if ( locdx(2).ne.0 ) then

             pnttyp = 4

             do j = 0, nj+1
                do i = 1, ni +1
                  adirii(i,j,1,1) =  cva(i,j,2,2,pnttyp) *
     +                             invjac(i,j,pnttyp)
                  adirii(i,j,1,2) = -cva(i,j,1,2,pnttyp) *
     +                             invjac(i,j,pnttyp)
                  adirii(i,j,2,1) = -cva(i,j,2,1,pnttyp) *
     +                             invjac(i,j,pnttyp)
                  adirii(i,j,2,2) =  cva(i,j,1,1,pnttyp) *
     +                             invjac(i,j,pnttyp)
                end do
             end do

          else

             do j = 0, nj+1
                do i = 2, ni

                  pnttyp = 2
                  c1(1) = 5d-1 * ( cva(i,j,1,1,pnttyp) +
     +                                     cva(i-1,j,1,1,pnttyp))
                  c1(2) = 5d-1 * ( cva(i,j,2,1,pnttyp)  +
     +                                     cva(i-1,j,2,1,pnttyp))

                  pnttyp = 3
                  c2(1) = 5d-1 * ( cva(i,j,1,2,pnttyp) +
     +                                     cva(i,j-1,1,2,pnttyp))
                  c2(2) = 5d-1 * ( cva(i,j,2,2,pnttyp) +
     +                                     cva(i,j-1,2,2,pnttyp))

                  det = c2(2)*c1(1)-c1(2)*c2(1)

                  adirii(i,j,1,1) =  c2(2)/det
                  adirii(i,j,1,2) = -c2(1)/det
                  adirii(i,j,2,1) = -c1(2)/det
                  adirii(i,j,2,2) =  c1(1)/det

                end do
             end do

             i = 1
             do j = 0, nj+1

                  pnttyp = 2
                  c1(1) = 5d-1 * ( cva(i,j,1,1,pnttyp) )
                  c1(1) = 5d-1 * ( cva(i,j,1,1,pnttyp) )
                  c1(2) = 5d-1 * ( cva(i,j,2,1,pnttyp) )

                  pnttyp = 3
                  c2(1) = 5d-1 * ( cva(i,j,1,2,pnttyp) +
     +                                     cva(i,j-1,1,2,pnttyp))
                  c2(2) = 5d-1 * ( cva(i,j,2,2,pnttyp) +
     +                                     cva(i,j-1,2,2,pnttyp))

                  det = c2(2)*c1(1)-c1(2)*c2(1)

                  adirii(i,j,1,1) =  c2(2)/det
                  adirii(i,j,1,2) = -c2(1)/det
                  adirii(i,j,2,1) = -c1(2)/det
                  adirii(i,j,2,2) =  c1(1)/det

             end do

             i = ni+1
             do j = 0, nj+1

                  pnttyp = 2
                  c1(1) = 5d-1 * ( cva(i-1,j,1,1,pnttyp) )
                  c1(2) = 5d-1 * ( cva(i-1,j,2,1,pnttyp) )

                  pnttyp = 3
                  c2(1) = 5d-1 * ( cva(i,j,1,2,pnttyp) +
     +                                     cva(i,j-1,1,2,pnttyp))
                  c2(2) = 5d-1 * ( cva(i,j,2,2,pnttyp) +
     +                                     cva(i,j-1,2,2,pnttyp))

                  det = c2(2)*c1(1)-c1(2)*c2(1)

                  adirii(i,j,1,1) =  c2(2)/det
                  adirii(i,j,1,2) = -c2(1)/det
                  adirii(i,j,2,1) = -c1(2)/det
                  adirii(i,j,2,2) =  c1(1)/det

             end do

         end if

      end if

      end
