      subroutine isdiss2d ( intn, ni, nj, nvirtual, matrix, vsum, dvsum,
     +                      invjac, mct, dcell, alphad, iarea, nintpol,
     +                      dij, nconct, invvar, mcom, eddy, mequat,
     +                      massfrac )
c ======================================================================
c
c        programmer     Guus Segal
c        version 3.1    date 09-06-1998 Charles: nvpoints changed to npointsv
c        version 3.0    date 08-10-1997 Hester: extra parameters
c        version 2.3    date 03-07-1997 Guus: new call to isintcf2
c        version 2.2    date 30-01-1997 Hester: extra parameters invvar,
c                                       mcom, eddy for compr. case
c        version 2.1    date 09-10-1996 Guus: extra parameter nconct
c        version 2.0    date 30-10-1995 Guus: new parameter list
c        version 1.0    date 23-05-1995 Extracted from subroutine isdstndf
c                                       of Ronald van Nooyen
c
c   copyright (c) 1995-1997  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Distribution of interpolation or finite difference formulas
c    over integration point with respect to the stress term
c **********************************************************************
c
c                       KEYWORDS
c
c     distribution
c     discretization
c     matrix
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, dcell(2), intn,
     +        alphad, iarea(1:2,2), dij(2,4,2), nintpol(2),
     +        mct(2,2,-2:2,-2:2,-2:2), nconct, mcom, mequat
      double precision matrix((ni+1)*(nj+1),2,*),
     +                 vsum((ni+1+2*nvirtual)*(nj+1+2*nvirtual),*),
     +                 dvsum((ni+1+2*nvirtual)*(nj+1+2*nvirtual),2,2),
     +                 invjac((ni+1+2*nvirtual)*(nj+1+2*nvirtual),*),
     +                 eddy((ni+1+2*nvirtual)*(nj+1+2*nvirtual)),
     +                 massfrac((ni+1+2*nvirtual)*(nj+1+2*nvirtual)),
     +                 invvar((ni+1+2*nvirtual)*(nj+1+2*nvirtual))

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     dcell     i    reserved
c     dij       i    Array containing the distances of the interpolation
c                    points to the cell centra in global co-ordinates
c                    dij(alpha,i,j) denotes the distance of the i-th
c                    interpolation point, component j and with respect to
c                    the V-alpha unknown (distribution)
c     dvsum     i    Flux derivative contributions to the matrix
c     eddy      i    reserved
c     iarea     i    Area of arrays
c     intn      i    Integration point number
c     invjac    i    1/sqrt(g) in pnttyp
c     invvar    i    Array with 1/variable
c     massfrac  i    Array containing the mass fraction
c     matrix   i/o   reserved
c     mcom      i    If zero the flow is incompressible else (1) compressible
c     mct       i    Integer matrix information
c     mequat    i    Indicates the type of momentum equation to be solved:
c                    0:  standard equation
c                    1:  multi-phase gas flow
c     nconct    i    Number of connections in a row of the matrix
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nintpol   i    Number of interpolation points in the distribution
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     vsum      i    Flux contributions to the matrix
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, jdir, iconct, kdir, nintder(2,2),
     +        didj(2,2,4,2), acttyp, vartyp, pnttyp, npointsv
      double precision cv(2,2,4), fact, dummy(1)

c     acttyp    Type of action to be taken
c               0      : interpolation for point value
c               1..ndim: finite difference for partial derivative
c                        in the direction given by value of acttyp
c     cv        reserved
c     didj      Array containing the distances of the finite difference
c               points to the cell centra in global co-ordinates
c               didj(alpha,beta,i,j) denotes the distance of the i-th
c               finite difference point, component j and with respect to
c               the d V-alpha/ d ksi beta derivative
c     dummy     Dummy
c     fact      Multiplication factor
c     i         General loop variable
c     iconct    Loop variable for loop over all connections in the matrx
c     jdir      General loop variable for 1...ndim loop
c     kdir      General loop variable for 1...ndim loop
c     nintder   Number of points in the distribution of the finite
c               difference
c     npointsv  Number of points in the mesh with virtual points
c     pnttyp    reserved
c     vartyp    Variable to be interpolated/extrapolated
c               vartyp: 1 for V1, 2 for V2 and 3 for pressure,
c               0 for vertex (for ndim=2)
c               or
c               vartyp: 1 for V1, 2 for V2 3, for V3 and
c               4 for pressure, 0 for vertex  (for ndim=3)
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISAXYZ2D  Perform a vector*vector update for ISNaS over a
c               2-dimensional region
c     ISINTCF2  Compute interpolation of a coefficient in an integration point
c     ISMOMDS2  Make a list in R2 of the number of points for the
c               differentiation and the corresponding connections for
c               each direction in one integration point
c     ISSETCON  Make array equal to a constant
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c    This is the main distribution subroutine with respect to the stress
c    This subroutine is called in a loop over the integration points and
c    also in a loop over the unknowns
c    The contribution of the integration point to the molecule has been stored
c    in the arrays vsum and dvsum which have the length of all points
c    in the mesh including the virtual points
c    vsum contains the coefficient for the unknown velocities in the
c    integration points
c    dvsum contains the coefficients for the partial derivatives of the
c    velocities in the computational domain
c    This array is build as if the integration point is an entry of
c    the matrix.
c    However, in general the integration point does not contain the
c    unknown to which the matrix corresponds.
c
c    For example consider the V1 component in the V1 row of the convection
c    matrix in R2 (Newton linearization)
c
c          *----------------*-----------------*
c          |                |                 |
c          |                |                 |
c          |                |                 |
c          *----------------*-----------------*
c          |        |       |        |        |
c          |                -i,j              |
c          |        |       |        |        |
c          *----------------*-----------------*
c
c    One of the contributions to the molecule is the term
c
c    rho/sqrt(g) V2 in the integration point i,j+1/2
c
c    This contribution has been computed before and stored in array update
c
c    However in point i,j+1/2 there is no V1 unknown and this contribution
c    must be distributed over the two surrounding points i,j and i,j+1
c
c    In this subroutine the distribution takes place and the corresponding
c    matrix elements (rho/sqrt(g) V2 )/2 are placed in the matrix in the
c    correct positions
c
c    The following tables and parameters define how the distrution must be
c    carried out:
c
c
c    dcell contains the global coordinate distance from integration point
c          to the centre of the integration cell (in the example (0,1))
c    dij   contains the global coordinate distance from the distribution points
c          to the centre of the integration cell (in the example (0,1))
c    nintpol contains the number of distribution points
c    mct   contains the column number in the matrix where the value
c          must be stored
c    didj  contains the global coordinate distance from the distribution points
c          to the centre of the integration cell in the case of a finite
c          difference
c    nintder contains the number of distribution points for a finite difference
c    cv    contains the multiplication factors for a finite difference
c
c                    PSEUDO CODE
c
c    Compute finite difference formula for partial derivatives of velocities
c    For each velocity component in the integration point do
c       fact := 1/(number of distribution points)
c       for i := 1 (1) number of distribution points do
c          Compute diagonal in matrix in which the coefficient must be put
c          Add vsum to matrix
c       For each derivative in the integration point do
c       for i := 1 (1) number of finite differnece points do
c          Compute diagonal in matrix in which the coefficient must be put
c          Add dvsum to matrix
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- Compute finite difference formula for partial derivatives of
c         velocities

      call ismomds2 ( intn, alphad, nintder, cv, didj, nconct )

      npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)

      do kdir = 1, 2

          call issetcon(invvar, npointsv, 1d0)

c         --- Loop over points in interpolation formula

         fact = 1d0/nintpol(kdir)

         if ( mcom.eq.1 ) then

c        --- compressible case, use rho U

            acttyp = 0
            vartyp = 3
            pnttyp = 4-kdir
            call isintcf2 ( eddy(1), invvar(1), ni, nj, nvirtual,
     +                      vartyp, acttyp, pnttyp, 0, dummy )

c           --- in the compressible case we work with density*velocity
c               as unknown, instead of the velocity. Here, we therefore
c               divide the coefficient by the density at the previous
c               time step: U = 1/rho * (rho * U)

            do i = 1, npointsv

              invvar(i) = invjac(i,4-kdir)/invvar(i)

            end do

         else if ( mequat.eq.1 ) then

c        --- two phase flow, use eps U

            acttyp = 0
            vartyp = 3
            pnttyp = 4-kdir
            call isintcf2 ( massfrac, invvar, ni, nj, nvirtual,
     +                      vartyp, acttyp, pnttyp, 0, dummy )

            do i = 1, npointsv

              invvar(i) = invjac(i,4-kdir)/invvar(i)

            end do

         else

c        --- standard

            do i = 1, npointsv
               invvar(i) = invjac(i,4-kdir)
            end do

         end if

         do i = 1, nintpol(kdir)

c        --- determine corresponding diagonal in the matrix

            iconct = mct(alphad,kdir,dij(kdir,i,1),
     +                   dij(kdir,i,2),0)

c           --- distribute vsum

            call isaxyz2d ( ni, nj, nvirtual, dcell(1), dcell(2),
     +                      dij(kdir,i,1), dij(kdir,i,2),
     +                      matrix(1,alphad,iconct), fact,
     +                      invvar(1), vsum(1,kdir), iarea )

         end do

         do jdir = 1, 2

            do i = 1, nintder(kdir,jdir)

c           --- determine corresponding diagonal in the matrix

               iconct = mct(alphad,kdir,didj(kdir,jdir,i,1),
     +                      didj(kdir,jdir,i,2),0)

c              --- distribute dvsum

            call isaxyz2d ( ni, nj, nvirtual, dcell(1), dcell(2),
     +                      didj(kdir,jdir,i,1), didj(kdir,jdir,i,2),
     +                      matrix(1,alphad,iconct), cv(kdir,jdir,i),
     +                      invvar(1), dvsum(1,kdir,jdir),
     +                      iarea )

            end do

         end do

      end do

      end
