      subroutine isdswb3d ( alphad, ni, nj, nk, nvirtual, locdx, iarea,
     +                      prems, matrix, cva, invjac, mct )
c ======================================================================
c
c        programmer     Kees Kassels
c        version 1.0    date 19-05-1998
c
c   copyright (c) 1998-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c **********************************************************************
c
c                       DESCRIPTION
c
c     Distribution of the intermediate matrix 'prems' into
c     the  matrix, Wesbeek discretization 3D.
c **********************************************************************
c
c                       KEYWORDS
c
c    stress
c    matrix
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nk, nvirtual, locdx(3), iarea(2,3),
     +        mct(1:3,1:3,-2:+2,-2:+2,-2:+2), alphad
      double precision cva(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,
     +                     1-nvirtual:nk+1+nvirtual,3,3,8),
     +                 invjac(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,
     +                        1-nvirtual:nk+1+nvirtual,8),
     +                 matrix(1:ni+1,1:nj+1,1:nk+1,3,*),
     +                 prems(1:ni+1,1:nj+1,1:nk+1,3,6)

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     cva       i    Array containing the covariant basis vectors
c                    in 2D we have
c                                                 l
c                    cva(i,j,l,alpha,p) contains a        in cell i,j in
c                                                 (alpha)       point type p
c                    in 3D it is cva(i,j,k,l,alpha,p) referring to cell i,j,k
c     iarea     i    Area of arrays
c     invjac    i    1/sqrt(g) in pnttyp
c     locdx     i    Position of the integration point with respect to
c                    the centre of the integration cell
c                    The cell sizes are assumed to be 2 times 2, hence the
c                    distance of a midside point to a cell centre is always 1
c     matrix    i    reserved
c     mct       i    Integer matrix information
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     prems     i    Intermediate matrix for building the stress term according
c                    to the Wesbeek discretization.
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, ip, jp, kp, betad, gammad, deltad,
     +        pnttyp1, iconct,  upnt, ispnt,
     +        onepnt, fourpnt(3), locu(3), locug(3), loc4(3), loc4g(3),
     +        locupnt(1:6,1:3,1:3), locspnt(1:4,1:2), valpha(3), fpd(3),
     +        nfp, ifp, pnttyput
      double precision  fact1pnt, fact4pnt
      data locupnt / -1, +1,  0,  0,  0,  0,
     +                0,  0, -1, +1,  0,  0,
     +                0,  0,  0,  0, -1, +1,
     +
     +                0,  0,  0,  0, -1, +1,
     +               -1, +1,  0,  0,  0,  0,
     +                0,  0, -1, +1,  0,  0,
     +
     +                0,  0, -1, +1,  0,  0,
     +                0,  0,  0,  0, -1, +1,
     +               -1, +1,  0,  0,  0,  0 /
      data locspnt / -1, +1, -1, +1,
     +               +1, +1, -1, -1 /

c     betad     Direction index, usually the direction perpendicular to the
c               integration direction of the boundary integral.
c               For example betad in local point (1,0) is 1 and in (0,1): 2
c     deltad    Direction index
c     fact1pnt  Factor for one point contribution
c     fact4pnt  Factor for four point contribution
c     fourpnt   Variable indicating what kind of point the four points are
c     fpd       Array for cyclic changing ordering
c     gammad    Direction index
c     i         General loop variable
c     iconct    Loop variable for loop over all connections in the matrx
c     ifp       Counting variable for four points
c     ip        Local counting variable i-direction
c     ispnt     Counting variable for surrounding points
c     jp        Local counting variable j-direction
c     kp        Local counting variable k-direction
c     loc4      Help variable to change from local shift to shift in stencil
c     loc4g     Global loc4
c     locspnt   Shift of four surrounding V-points in units of half a cell
c     locu      Shift of u-point w.r.t. cell centre
c     locug     Global shift of u-point w.r.t. cell centre
c     locupnt   Shift of four surrounding u-points in units of half a cell
c     nfp       Number of fourpoints
c     onepnt    Variable indicating what kind of point the one points is
c     pnttyp1   Point type of u-point
c     pnttyput  Point type of point where the shift goes to
c     upnt      Counting variable for u points
c     valpha    Array for cyclic changing ordering
c               alphal = 1       alphal = 2   alphal = 3
c               valpha(1) =1 ---> valpha(1)=2  valpha(1)=3
c               valpha(2) =2 ---> valpha(2)=3  valpha(2)=1
c               valpha(3) =3 ---> valpha(3)=1  valpha(3)=2
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISGLALSH3
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c      The Wesbeek discrtization is carried out by a loop over
c      the cells. For each cell we do a loop over the integration
c      points.
c
c      In a integration point we need the gradient of the
c      velocity vector. Using the path-integral method the
c      gradient of the velocity vector is expressed in
c      terms of the cartesian velocities in the 6
c      surrounding points.
c
c      The coefficients for these 6 cartesian velocities are
c      stored in array prems(i,j,k,l,gamma)(l=1:3;gamma=1:6),
c      where l refers to the vector and gamma to the 6 points.
c
c      In this subroutine these contributions are distributed
c                alpha
c      over the V      components and stored at the right positions
c      in the matrix.
c
c      The shifts of the 6 surrounding points with respect to
c      the integration point are stored in array
c      locupnt(1:6,1:3,1:3)
c
c      locupnt(i,j,alphad) refers to the i-th point, shift in
c      j direction for an alphad cell. For example in an
c      alphad=1 cell point 1 is at the left of the integration
c      point and has a local shift of (-1,0,0). Hence
c      locupnt(1,.,1)=(-1,0,0).
c
c      The shift of the integration point with respect to the
c      integration cell center is stored in locdx.
c      Combination of locupnt and locdx gives the shift of the
c      new points with respect to the integration cell center.
c                                         alpha
c      Each cartesian velocity component u      may be expressed
c      into the contravariant fluxes by
c
c
c                alpha  1       alpha  2       alpha  3
c               a      V   +   a      V   +   a      V
c       alpha    (1)            (2)            (3)
c      u      = ---------------------------------------
c
c                              sqrt(g)
c
c      In each point where the velocity must be evaluated either
c           1        2         3
c      the V or the V  or the V  unknown is present if the point
c      is situated in the middle of a face. If the point is situated
c      in a vertex no velocity component is present.
c      The unknowns not present  must be interpolated from
c      4 surrounding points.
c      The local shifts for this interpolation is stored in
c      array locspnt(1:4,1:2) where the first component refers to
c      the point and the second one to the component of the shift.
c
c      Once the shifts with respect to the integration cell center
c      are known the global i,j position of the point can be
c      computed by transforming the local shift into the global
c      shift.
c      These global shifts are used in array mct. Array mct gives
c      withe these shifts the diagonal number in the matrix.
c
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- cyclic

      valpha(1) = alphad
      valpha(2) = mod(alphad  ,3) + 1
      valpha(3) = mod(alphad+1,3) + 1

      betad   = valpha(2)
      gammad  = valpha(3)

      do upnt = 1, 6

c     --- location u point

         do i =1, 3
             locu(i) =  locdx(i) + locupnt(upnt,i,alphad)
         end do

c        --- check the u-point, is it an alphad-point or a betad-point
c            or a gammad-point or none of these, in other words is the
c            u-point situated in the middle of a face or in a vertex.
c            the onepnt contribution can be distributed directly
c            the fourpnt contribution must be distributed over 4
c            surrounding points

         onepnt = 0
         nfp    = 2

         if ( mod( locu( alphad),2).eq.0 .and.
     +        mod( locu( betad),2).eq.0  .and.
     +        mod( locu( gammad),2).eq.0) then
            onepnt     = alphad
            fourpnt(1) = betad
            fourpnt(2) = gammad
         else if ( mod( locu( gammad),2).eq.0 .and.
     +              mod( locu( betad),2).ne.0) then
            onepnt     = betad
            fourpnt(1) = gammad
            fourpnt(2) = alphad
         else if ( mod( locu( betad),2).eq.0 .and.
     +              mod( locu( gammad),2).ne.0) then
            onepnt     = gammad
            fourpnt(1) = alphad
            fourpnt(2) = betad
         else if ( locu( betad).ne.0 .and.
     +              locu( gammad).ne.0) then
            fourpnt(1) = alphad
            fourpnt(2) = betad
            fourpnt(3) = gammad
            nfp        = 3
         end if

c        --- pnttype for the u-point

         if ( onepnt.eq.1) then
            pnttyp1 = 7
         else if( onepnt.eq.2) then
            pnttyp1 = 6
         else if( onepnt.eq.3) then
            pnttyp1 = 4
         else
            pnttyp1 = 1
         end if

c        --- contribution for one point (onepnt)

         if ( onepnt.gt.0 ) then

            call isglalsh3 ( alphad, locu, locug, pnttyput )

            iconct = mct (alphad, onepnt, locug(1), locug(2), locug(3))

            do deltad = 1, 3

               do kp = iarea(1,3), iarea(2,3)
                  do jp = iarea(1,2), iarea(2,2)
                     do ip = iarea(1,1), iarea(2,1)

                        fact1pnt =
     +                      cva(ip+locug(1),jp+locug(2),kp+locug(3),
     +                          deltad,onepnt,pnttyp1) *
     +                      invjac(ip+locug(1),jp+locug(2),kp+locug(3),
     +                             pnttyp1)
                        matrix(ip,jp,kp,alphad,iconct) =
     +                       matrix(ip,jp,kp,alphad,iconct) +
     +                       prems(ip,jp,kp,deltad,upnt) * fact1pnt
                     end do
                  end do
               end do
            end do

         end if

         do ifp = 1, nfp

c        --- nfp x contribution for four points ( fourpnt)

            do ispnt = 1,4

c           --- location 4 surrounding points

               if ( onepnt.gt.0 ) then
                  do i =1, 3
                     loc4(i) =  locu(i)
                  end do
                  loc4(onepnt) =  locu(onepnt)+locspnt(ispnt,1)
                  loc4(fourpnt(ifp)) = locu(fourpnt(ifp)) +
     +                                 locspnt(ispnt,2)
               else

c              --- cyclic

                  fpd(1) = fourpnt(ifp)
                  fpd(2) = mod(fourpnt(ifp),3) + 1
                  fpd(3) = mod(fourpnt(ifp)+1,3) + 1

                  do i =1, 3
                     loc4(i) =  locu(i)
                  end do
                  loc4(fpd(2)) = locu(fpd(2))+locspnt(ispnt,1)
                  loc4(fpd(3)) = locu(fpd(3))+locspnt(ispnt,2)
               end if

               call isglalsh3 (alphad, loc4, loc4g, pnttyput )

               iconct = mct ( alphad, fourpnt(ifp), loc4g(1), loc4g(2),
     +                        loc4g(3) )

               do deltad = 1, 3

                  do kp = iarea(1,3), iarea(2,3)
                     do jp = iarea(1,2), iarea(2,2)
                        do ip = iarea(1,1), iarea(2,1)

                           fact4pnt =
     +                       cva(ip+locug(1),jp+locug(2),kp+locug(3),
     +                           deltad,fourpnt(ifp),pnttyp1) *
     +                       invjac(ip+locug(1),jp+locug(2),kp+locug(3),
     +                              pnttyp1) * 25d-2
                           matrix(ip,jp,kp,alphad,iconct) =
     +                        matrix(ip,jp,kp,alphad,iconct) +
     +                        prems(ip,jp,kp,deltad,upnt) * fact4pnt
                        end do
                     end do
                  end do
               end do
            end do
         end do

      end do

      end
