      subroutine isexbo23 ( ni, nj, nvirtual, ndegfd, ntimlv, solut,
     +                      ibinfor,  bcgeo, bndcon,
     +                      iside, ip, jp, ishift, jshift, ipibinf,
     +                      tdir, ndir, tsign, nsign )
c ======================================================================
c
c        programmer     Kees Vuik
c        version 1.2    date 29-10-1998 Duncan: new boundary condtions
c                                               added
c        version 1.1    date 15-02-1996 Kees V: Implementation of all
c                                     Dirichlet boundary conditions
c        version 1.0    date 17-10-1995
c
c   copyright (c) 1995-1996  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Computes the tangential velocities and extrapolates them
c     to the virtual points
c **********************************************************************
c
c                       KEYWORDS
c
c     boundary_conditions
c     explicit
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer          ni, nj, nvirtual, ndegfd, ntimlv,
     +                 ibinfor(*), iside, ip, jp, ishift, jshift,
     +                 ipibinf, tdir, ndir, tsign, nsign
      double precision bcgeo(8,*), bndcon(2,*),
     +                 solut(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1:ndegfd, 1:ntimlv)

c     bcgeo     i    reserved
c     bndcon    i    reserved
c     ibinfor   i    Third part of infbnd
c     ip        i    Local counting variable i-direction
c     ipibinf   i    Starting address of array infor for present side minus one
c     ishift    i    Shift in i-direction
c     iside     i    Sequence number of side
c     jp        i    Local counting variable j-direction
c     jshift    i    Shift in j-direction
c     ndegfd    i    Number of degrees of freedom
c     ndir      i    Direction of normal vector on this side
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nsign     i    Inner product of outward normal in computational domain
c                    with the base vector perpendicular to the face
c     ntimlv    i    Number of time levels
c     nvirtual  i    Number of rows of virtual cells
c     solut     i    reserved
c     tdir      i    Direction of tangent vector on this side
c     tsign     i    Inner product of tangential vector defined counter
c                    clockwise with respect to the outward normal and the
c                    base vector along the side
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer          btype, ipoint, istwo(2,2), unut, parabolic,
     +                 uxuy, multiblk, sgnn, unstt, multibltr
      double precision gnn, gnt, gtt, gttdivsg, gntdivsg, un, ut
      logical          skip
      parameter ( unut = 4, uxuy = 10, parabolic = 12,
     +            multiblk = 99, sgnn = 7, unstt = 5, multibltr=89 )

c     btype     Type of boundary condition
c     gnn       Contravariant metric tensor
c     gnt       Contravariant metric tensor in nt direction
c     gntdivsg  Contravariant metric tensor in nt direction divided by sqrt(g)
c     gtt       Contravariant metric tensor in tt direction
c     gttdivsg  Contravariant metric tensor in tt direction divided by sqrt(g)
c     ipoint    Address in array IBUFFR
c     istwo     Translation table to store and retrieve the
c               elements of a symmetric ndim-dimensional
c               rank 2 tensor from a vector with length
c               (ndim*(ndim+1))/2
c     multiblk  Multi-block boundary conditions
c     multibltr Multi-block/solid wall transition boundary condition
c     parabolic Parabolic boundary conditions
c     sgnn      Outflow boundary condition ( for Euler)
c     skip      If true boundary condition must be skipped
c     un        Normal velocity (physical)
c     unstt     U_n and s_tt prescribed
c     unut      U_n and u_t prescribed
c     ut        Tangential velocity (physical)
c     uxuy      U_x and u_y prescribed
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     712   unknown boundary condition for theta = 0
c **********************************************************************
c
c                       PSEUDO CODE
c
c     In the explicit time discretization the matrix vector
c     multiplication is carried out directly after the construction
c     of its elements. In this multiplication virtual tangential
c     velocities are used. The tangential velocity given on the
c     boundary is used to extrapolate the value of the tangential
c     velocity in these virtual points.
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data istwo / 1, 2,
     +             2, 3 /
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c

      call iseropen ( 'isexbo23' )

      if ( iside.le.2 ) then
         ipoint = ipibinf + 2*jp - 1
      else
         ipoint = ipibinf + 2*ip - 1
      end if

      btype = ibinfor(ipoint)
      skip  = (btype.eq.multiblk).or.(btype.eq.multibltr)

      if ( .not. skip ) then

         if ( btype.eq.unut .or. btype.eq.parabolic ) then

c        --- un and ut given

            un = bndcon(1,ipoint)*nsign
            ut = bndcon(2,ipoint)*tsign
            gttdivsg = bcgeo(istwo(ndir,ndir),ipoint)
            gntdivsg =-bcgeo(istwo(ndir,tdir),ipoint)
            gnn = gttdivsg/bcgeo(8,ipoint)
            gtt = bcgeo(8,ipoint)*gttdivsg
            gnt = bcgeo(8,ipoint)*gntdivsg
            un = sqrt(gnn)*un
            ut = (sqrt(gtt)*ut-gnt*un)/gtt
            ut = bcgeo(8,ipoint)*ut

         else if ( btype.eq.uxuy ) then

c        --- ux and uy given

            ut = bcgeo(2+2*tdir,ipoint)*bndcon(1,ipoint)+
     +           bcgeo(3+2*tdir,ipoint)*bndcon(2,ipoint)

         else if (btype.eq.unstt.or.btype.eq.sgnn) then

c        --- freeslip boundary condition, unstt given or
c               outflow boundary (zeroth order extrapolation)

            ut = solut(ip+ishift,jp+jshift,tdir,ntimlv)

c           --- error message for other boundary conditions

         else

            call iserrint ( btype, 1 )
            call iserrsub ( 712, 1, 0, 0)
            go to 1000

         end if

c        --- filling of the virtual tangential velocity component

         solut(ip,jp,tdir,ntimlv) = 2*ut-
     +                   solut(ip+ishift,jp+jshift,tdir,ntimlv)
      end if

1000  call iserclos ( 'isexbo23' )

      end
