      subroutine isexbou2 ( ni, nj, nvirtual, ndegfd, ntimlv, solut,
     +                      ibinfor, ibipoint, bcgeo, bndcon )
c ======================================================================
c
c        programmer     Kees Vuik
c                                               added
c        version 1.2    date 29-10-1998 Duncan: extra boundary conditions
c        version 1.1    date 15-02-1996 Kees V: Implementation of all
c                                     Dirichlet boundary conditions
c        version 1.0    date 28-09-1995
c
c   copyright (c) 1995-1996  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Fills the boundary conditions for theta = 0
c
c **********************************************************************
c
c                       KEYWORDS
c
c     boundary_conditions
c     explicit
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, ndegfd, ntimlv, ibipoint(*), ibinfor(*)
      double precision bcgeo(8,*), bndcon(2,*),
     +                 solut(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1:ndegfd, 1:ntimlv)

c     bcgeo     i    reserved
c     bndcon    i    reserved
c     ibinfor   i    Third part of infbnd
c     ibipoint  i    ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     ndegfd    i    Number of degrees of freedom
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     ntimlv    i    Number of time levels
c     nvirtual  i    Number of rows of virtual cells
c     solut     i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer          btype, ihigh, ilow, ip, ipibinf, ipoint, iside,
     +                 istwo(2,2), ishift, jshift,
     +                 jhigh, jlow, jp, ndir, nsign,
     +                 tdir, tsign, unut, parabolic,
     +                 uxuy, multiblk, unstt, sgnn, multibltr
      double precision gnn, gttdivsg, un
      logical          skip
      parameter ( unut = 4, uxuy = 10, parabolic = 12,
     +            multiblk = 99, unstt = 5, sgnn= 7,
     +            multibltr = 98 )

c     btype     Type of boundary condition
c     gnn       Contravariant metric tensor
c     gttdivsg  Contravariant metric tensor in tt direction divided by sqrt(g)
c     ihigh     Upper index of subface
c     ilow      Lower index of subface
c     ip        Local counting variable i-direction
c     ipibinf   Starting address of array infor for present side minus one
c     ipoint    Address in array IBUFFR
c     ishift    Shift in i-direction
c     iside     Sequence number of side
c     istwo     Translation table to store and retrieve the
c               elements of a symmetric ndim-dimensional
c               rank 2 tensor from a vector with length
c               (ndim*(ndim+1))/2
c     jhigh     Upper j-index for a subface
c     jlow      Lower j-index for a subface
c     jp        Local counting variable j-direction
c     jshift    Shift in j-direction
c     multiblk  Multi-block boundary conditions
c     multibltr Multi-block/solid wall transition boundary condition
c     ndir      Direction of normal vector on this side
c     nsign     Inner product of outward normal in computational domain
c               with the base vector perpendicular to the face
c     parabolic Parabolic boundary conditions
c     sgnn      Outflow boundary condition ( for Euler)
c     skip      If true boundary condition must be skipped
c     tdir      Direction of tangent vector on this side
c     tsign     Inner product of tangential vector defined counter
c               clockwise with respect to the outward normal and the
c               base vector along the side
c     un        Normal velocity (physical)
c     unstt     U_n and s_tt prescribed
c     unut      U_n and u_t prescribed
c     uxuy      U_x and u_y prescribed
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c     ISEXBO23  Computes the tangential velocities and extrapolates them
c               to the virtual points
c     ISORIENT2 Computes ndir, nsign, tdir, tsign for a 2-dimensional
c               problem
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     712   unknown boundary condition for theta = 0
c **********************************************************************
c
c                       PSEUDO CODE
c
c     This subroutine fills the boundary conditions for the explicit
c     time discretization. The normal velocity given on the boundary
c     is copied to the solution array. The tangential velocities are
c     done by a call to isexbou23.f
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data istwo / 1, 2,
     +             2, 3 /
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c

      call iseropen ( 'isexbou2' )

c     --- loop over faces

      do iside = 1, 4

c     --- Compute tdir and ndir

         call isorient2 ( iside, tdir, ndir, tsign, nsign )
         if ( nerror.ne.0 ) go to 1000

c        --- adapt normal velocities

         if ( iside.le.2 ) then

            jlow = 1
            jhigh = nj

            if ( iside.eq.1 ) then
               ilow = 1
               ihigh = 1
            else
               ilow = ni+1
               ihigh = ni+1
            end if

         else

            ilow = 1
            ihigh = ni

            if ( iside.eq.3 ) then
               jlow = 1
               jhigh = 1
            else
               jlow = nj+1
               jhigh = nj+1
            end if

         end if

         ipibinf = ibipoint(iside)+nvirtual-1

         do jp = jlow, jhigh

            do ip = ilow, ihigh

               if ( iside.le.2 ) then
                  ipoint = ipibinf + 2*jp
               else
                  ipoint = ipibinf + 2*ip
               end if

               btype = ibinfor(ipoint)
               skip = btype.eq.multiblk.or.btype.eq.multibltr
               if ( .not. skip ) then

                  if ( btype.eq.unut .or. btype.eq.parabolic ) then

c                 --- un and ut given

                     un = bndcon(1,ipoint)*nsign
                     gttdivsg = bcgeo(istwo(ndir,ndir),ipoint)
                     gnn = gttdivsg/bcgeo(8,ipoint)
                     un = sqrt(gnn)*un
                     un = bcgeo(8,ipoint)*un

                  else if ( btype.eq.uxuy ) then

c                 --- ux and uy given

                     un = bcgeo(2+2*ndir,ipoint)*bndcon(1,ipoint)+
     +                    bcgeo(3+2*ndir,ipoint)*bndcon(2,ipoint)

                  else if( btype.eq.unstt) then

c                 --- freeslip boundary condition or un stt given

                     un = bndcon(1,ipoint)*nsign
                     gttdivsg = bcgeo(istwo(ndir,ndir),ipoint)
                     gnn = gttdivsg/bcgeo(8,ipoint)
                     un = sqrt(gnn)*un
                     un = bcgeo(8,ipoint)*un

                  else  if( btype.eq.sgnn) then

c                 --- Outflow boundary condition ( Euler)

                     un=solut(ip,jp,ndir,ntimlv)

                  else

c                 --- error message for other boundary conditions

                     call iserrint ( btype, 1 )
                     call iserrsub ( 712, 1, 0, 0)
                     go to 1000

                  end if
                  solut(ip,jp,ndir,ntimlv) = un

               end if

            end do
         end do

c        --- adapt tangential velocities

         if ( iside.le.2 ) then

            jlow = 2
            jhigh = nj
            jshift = 0

            if ( iside.eq.1 ) then
               ilow = 0
               ihigh = 0
               ishift = 1
            else
               ilow = ni+1
               ihigh = ni+1
               ishift = -1
            end if

         else

            ilow = 2
            ihigh = ni
            ishift = 0

            if ( iside.eq.3 ) then
               jlow = 0
               jhigh = 0
               jshift = 1
            else
               jlow = nj+1
               jhigh = nj+1
               jshift = -1
            end if

         end if

         ipibinf = ibipoint(iside)+nvirtual-1

         do jp = jlow, jhigh

            do ip = ilow, ihigh

               call isexbo23 ( ni, nj, nvirtual, ndegfd, ntimlv, solut,
     +                         ibinfor,  bcgeo, bndcon,
     +                         iside, ip, jp, ishift, jshift, ipibinf,
     +                         tdir, ndir, tsign, nsign )
               if ( nerror.ne.0 ) go to 1000

            end do
         end do

      end do

1000  call iserclos ( 'isexbou2' )

      end
