      subroutine isextcf2 ( coef, ni, nj, nvirtual, vartyp )
c ======================================================================
c
c
c     programmer     Hester Bijl
c
c     version      1.1    date 05-07-95 Hester: cleaned up
c     version      1.0    date 30-05-95 (Subroutine programmed like isfilvso)
c
c
c   copyright (c) 1995-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c ********************************************************************
c
c                      DESCRIPTION
c
c    Compute extrapolation of a coefficient in one integration point in R^2
c
c    This subroutine may be made more efficient by using the fact that a
c    coefficient is constant
c    Another possibility to increase efficiency is to extrapolate to all
c    integration points in one step.
c ********************************************************************
c
c                      KEYWORDS
c
c    coefficients
c
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual
      double precision coef(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual)
      integer vartyp


c     coef      i    A specific coefficient
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     vartyp    i    Variable to be interpolated/extrapolated
c                    vartyp: 1 for V1, 2 for V2 and 3 for pressure,
c                    0 for vertex (for ndim=2)
c                    or :
c                    vartyp: 1 for V1, 2 for V2 3, for V3 and
c                    4 for pressure, 0 for vertex  (for ndim=3)
c ********************************************************************
c
c                      COMMON BLOCKS
c
c ********************************************************************
c
c                      LOCAL PARAMETERS

      integer i, j


c     i         General loop variable
c     j         General loop variable
c ********************************************************************
c
c                      SUBROUTINES CALLED
c
c ********************************************************************
c
c                      I/O
c
c ********************************************************************
c
c                      ERROR MESSAGES
c
c ********************************************************************
c
c                      PSEUDO CODE
c
c ======================================================================
c

c
c     --- interpolate coefficient in pressure integration points to
c         virtual points
c

      if (vartyp.eq.3) then

        do 100 j = 1, nj

          do i = ni+1, ni+1+nvirtual
            coef(i,j) = 2d0*coef(i-1,j) - coef(i-2,j)
          end do

          do i = 0, 1-nvirtual, -1
             coef(i,j) = 2d0*coef(i+1,j) - coef(i+2,j)
          end do

 100    end do


        do 200 i = 1, ni

          do j = nj+1, nj+1+nvirtual
            coef(i,j) = 2d0*coef(i,j-1) - coef(i,j-2)
          end do

          do j = 0, 1-nvirtual, -1
             coef(i,j) = 2d0*coef(i,j+1) - coef(i,j+2)
          end do

 200    continue

c
c     corners
c

c     --- bottom-left corner

         do i = 0, 1-nvirtual, -1
            do j = 0, 1-nvirtual, -1
               coef(i,j) =
     +            ( 2d0*coef(i+1,j) -
     +                  coef(i+2,j) ) * 5d-1 +
     +            ( 2d0*coef(i,j+1) -
     +                  coef(i,j+2) ) * 5d-1
            enddo
         enddo

c        --- bottom-right corner

         do i = ni+1, ni+1+nvirtual
            do j = 0, 1-nvirtual, -1
               coef(i,j) =
     +           ( 2d0*coef(i-1,j) -
     +                 coef(i-2,j) ) * 5d-1 +
     +           ( 2d0*coef(i,j+1) -
     +                 coef(i,j+2) ) * 5d-1
            enddo
         enddo

c        --- top-left corner

         do i = 0, 1-nvirtual, -1
            do j = nj+1, nj+1+nvirtual
               coef(i,j) =
     +            ( 2d0*coef(i+1,j) -
     +                  coef(i+2,j) ) * 5d-1 +
     +            ( 2d0*coef(i,j-1) -
     +                  coef(i,j-2) ) * 5d-1
            enddo
         enddo


c        --- top-left corner

         do i = ni+1, ni+1+nvirtual
            do j = nj+1, nj+1+nvirtual
               coef(i,j) =
     +            ( 2d0*coef(i-1,j) -
     +                  coef(i-2,j) ) * 5d-1 +
     +            ( 2d0*coef(i,j-1) -
     +                  coef(i,j-2) ) * 5d-1
            enddo
         enddo


      end if

      end

