      subroutine isgmat04 ( ni, nj, nvirtual, gmat, cva, jac,
     +                      alphad, adir, ibipoint, ibinfor )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.4    date 03-12-1998 Guus: remove iprec
c        version 1.3    date 08-12-1997 Kees K, change for periodic
c        version 1.2    date 19-12-1996 Kees K, ibipoint, ibinfor added
c        version 1.1    date 25-10-1996 Kees K, one sided approximations
c        version 1.0    date 20-09-1996
c
c   copyright (c) 1996-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Compute internal part of gradient matrix in R^2 for one direction
c    WesBeek discretization
c **********************************************************************
c
c                       KEYWORDS
c
c    grad_matrix
c    matrix
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, alphad, ibipoint(*), ibinfor(*)
      double precision gmat(ni+1,nj+1,2,6),
     +                 cva(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,2,2,4),
     +                 adir(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,2),
     +                 jac(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual)

c     adir      i    Contravariant base vector in integration cell center
c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     cva       i    Array containing the covariant basis vectors
c                    in 2D we have
c                                                 l
c                    cva(i,j,l,alpha,p) contains a        in cell i,j in
c                                                 (alpha)       point type p
c                    in 3D it is cva(i,j,k,l,alpha,p) referring to cell i,j,k
c     gmat      o    Gradient matrix
c     ibinfor   i    Third part of infbnd
c     ibipoint  i    ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     jac       i    sqrt(g) in pnttyp
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ip, jp, betad, i, iupploop, jupploop
      integer iside, ipibinf, btype, ipoint, btypenb
      double precision fact, c1(2), c2(2), c
      logical skip

c     betad     Direction index, usually the direction perpendicular to the
c               integration direction of the boundary integral.
c               For example betad in local point (1,0) is 1 and in (0,1): 2
c     btype     Type of boundary condition
c     btypenb   Type of boundary condition of neighbour side
c     c         General help variable
c     c1        General help variable
c     c2        General help variable
c     fact      Multiplication factor
c     i         General loop variable
c     ip        Local counting variable i-direction
c     ipibinf   Starting address of array infor for present side minus one
c     ipoint    Address in array IBUFFR
c     iside     Sequence number of side
c     iupploop  Upper bound for loop
c     jp        Local counting variable j-direction
c     jupploop  Upper bound for loop
c     skip      If true boundary condition must be skipped
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c   The following discretization is used in the WesBeek case
c
c   V^1-cell:
c
c      a^(1) . sqrt(g) grad p (0,0), with
c
c      sqrt(g) (0,0) = 0.5d0*(sqrt(g)(1,0)+sqrt(g)(-1,0))
c
c      grad p (0,0) = c^1 (p(1,0)-p(-1,0)) +
c                     c^2 (p(-1,2)-p(-1,-2) + p(1,2)-p(1,-2))
c
c                | c_2(2) |            |-c_1(2) |
c      c^1 = 1/C |        |  c^2 = 1/C |        |
c                |-c_2(1) |            | c_1(1) |
c
c
c      C = c_2(2) c_1(1) - c_2(1) c_1(2)
c
c      c_1 = a_1 (0,0)
c
c      c_2 = 1/2 ( a_2(-1,2) + a_2(-1,-2) + a_2(1,2) + a_2(1,-2) ) +
c                  a_2(-1,0) + a_2(1,0)
c
c
c   V^2-cell:
c
c      a^(2) . sqrt(g) grad p (0,0), with
c
c      sqrt(g) (0,0) = 0.5d0*(sqrt(g)(01,)+sqrt(g)(0,-1))
c
c      grad p (0,0) = c^1 (p(0,1)-p(0,-1)) +
c                     c^2 (p(2,-1)-p(-2,-1) + p(2,1)-p(-2,1))
c
c                | c_2(2) |            |-c_1(2) |
c      c^1 = 1/C |        |  c^2 = 1/C |        |
c                |-c_2(1) |            | c_1(1) |
c
c
c      C = c_2(2) c_1(1) - c_2(1) c_1(2)
c
c      c_1 = a_2 (0,0)
c
c      c_2 = 1/2 ( a_1(-2,1) + a_1(-2,-1) + a_1(2,1) + a_1(-2,1) ) +
c                  a_1(0,-1) + a_1(0,1)
c
c
c                    PSEUDO CODE
c
c
c    First clear part corresponding to virtual unknowns
c    Loop over Va unknown  (a=alpha)
c       Compute factor before (1,0) and (-1,0) (a=1) or (0,1) and (0,-1)
c       Store factor in gmat(a,4) and -gmat(a,3)
c       Compute factor before (0,1) and (0,-1) (a=1) or (1,0) and (-1,0)
c       and multiply by .25
c       Store factor in gmat(a,5), gmat(a,6) and -gmat(a,1), -gmat(a,2)
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- First clear part corresponding to virtual unknowns

      if ( alphad.eq.1 ) then

c     --- V1-cell

         do ip = 1, ni+1
            do i = 1, 6
               gmat(ip,nj+1,alphad,i) = 0d0
            end do
         end do

         iupploop = ni+1
         jupploop = nj
         betad = 2

c        --- Actually filling of gmat

         do ip = 1, iupploop
            do jp = 1, jupploop

               fact = jac(ip,jp)
               c1(1) = cva(ip,jp,1,alphad,3)
               c1(2) = cva(ip,jp,2,alphad,3)

               c2(1) = 0.5d0*(cva(ip-1,jp-1,1,betad,4)+
     +                        cva(ip  ,jp-1,1,betad,4)+
     +                        cva(ip  ,jp+1,1,betad,4)+
     +                        cva(ip-1,jp+1,1,betad,4))+
     +                 cva(ip-1,jp,1,betad,4)+cva(ip,jp,1,betad,4)
               c2(2) = 0.5d0*(cva(ip-1,jp-1,2,betad,4)+
     +                        cva(ip  ,jp-1,2,betad,4)+
     +                        cva(ip  ,jp+1,2,betad,4)+
     +                        cva(ip-1,jp+1,2,betad,4))+
     +                 cva(ip-1,jp,2,betad,4)+cva(ip,jp,2,betad,4)

               c = fact/(c2(2)*c1(1)-c2(1)*c1(2))
               fact = c*(adir(ip,jp,1)*c2(2)-adir(ip,jp,2)*c2(1))
               gmat(ip,jp,alphad,3) = -fact
               gmat(ip,jp,alphad,4) =  fact
               fact = c*(-adir(ip,jp,1)*c1(2)+adir(ip,jp,2)*c1(1))

               gmat(ip,jp,alphad,1) = -fact
               gmat(ip,jp,alphad,2) = -fact
               gmat(ip,jp,alphad,5) =  fact
               gmat(ip,jp,alphad,6) =  fact

            end do
         end do

c        --- one sided approximation for tangential cells at the boundary

         jp = 1
         do ip = 1, iupploop

            iside   = 3
            ipibinf = ibipoint(iside)-1+nvirtual
            ipoint  = ipibinf + 2*ip -1
            btype   = ibinfor(ipoint)
            skip    = btype.ge.88 .and. btype.le.100

            if ( .not. skip ) then

               fact = jac(ip,jp)
               c1(1) = cva(ip,jp,1,alphad,3)
               c1(2) = cva(ip,jp,2,alphad,3)

               c2(1) = 0.5d0*(cva(ip-1,jp+1,1,betad,4)+
     +                        cva(ip-1,jp  ,1,betad,4)+
     +                        cva(ip  ,jp+1,1,betad,4)+
     +                        cva(ip  ,jp  ,1,betad,4))
               c2(2) = 0.5d0*(cva(ip-1,jp+1,2,betad,4)+
     +                        cva(ip-1,jp  ,2,betad,4)+
     +                        cva(ip  ,jp+1,2,betad,4)+
     +                        cva(ip  ,jp  ,2,betad,4))

               c = fact/(c2(2)*c1(1)-c2(1)*c1(2))
               fact = c*(adir(ip,jp,1)*c2(2)-adir(ip,jp,2)*c2(1))
               gmat(ip,jp,alphad,3) = -fact
               gmat(ip,jp,alphad,4) =  fact
               fact = c*(-adir(ip,jp,1)*c1(2)+adir(ip,jp,2)*c1(1))

               gmat(ip,jp,alphad,3) = gmat(ip,jp,alphad,3) - fact
               gmat(ip,jp,alphad,4) = gmat(ip,jp,alphad,4) - fact
               gmat(ip,jp,alphad,5) =  fact
               gmat(ip,jp,alphad,6) =  fact
               gmat(ip,jp,alphad,1) =  0d0
               gmat(ip,jp,alphad,2) =  0d0

            else if ( btype.lt.90 ) then

c           --- transition from multiblock boundary to fixed boundary

               btypenb = ibinfor(ipoint-1)
               if ( btypenb.gt.90 ) then

c              --- Previous point is multiblock
c                  eliminate point 2

                  fact = jac(ip,jp)
                  c1(1) = cva(ip,jp,1,alphad,3)
                  c1(2) = cva(ip,jp,2,alphad,3)

                  c2(1) = 0.5d0*(cva(ip-1,jp+1,1,betad,4)+
     +                           cva(ip-1,jp-1,1,betad,4)+
     +                           cva(ip  ,jp+1,1,betad,4)+
     +                           cva(ip  ,jp  ,1,betad,4))
                  c2(2) = 0.5d0*(cva(ip-1,jp+1,2,betad,4)+
     +                           cva(ip-1,jp-1,2,betad,4)+
     +                           cva(ip  ,jp+1,2,betad,4)+
     +                           cva(ip  ,jp  ,2,betad,4))

                  c = fact/(c2(2)*c1(1)-c2(1)*c1(2))
                  fact = c*(adir(ip,jp,1)*c2(2)-adir(ip,jp,2)*c2(1))
                  gmat(ip,jp,alphad,3) = -fact
                  gmat(ip,jp,alphad,4) =  fact
                  fact = c*(-adir(ip,jp,1)*c1(2)+adir(ip,jp,2)*c1(1))

                  gmat(ip,jp,alphad,4) = gmat(ip,jp,alphad,4) - fact
                  gmat(ip,jp,alphad,5) =  fact
                  gmat(ip,jp,alphad,6) =  fact
                  gmat(ip,jp,alphad,1) = -fact
                  gmat(ip,jp,alphad,2) =  0d0

               else

c              --- Next point is multiblock
c                  eliminate point 1

                  fact = jac(ip,jp)
                  c1(1) = cva(ip,jp,1,alphad,3)
                  c1(2) = cva(ip,jp,2,alphad,3)

                  c2(1) = 0.5d0*(cva(ip-1,jp+1,1,betad,4)+
     +                           cva(ip-1,jp  ,1,betad,4)+
     +                           cva(ip  ,jp+1,1,betad,4)+
     +                           cva(ip  ,jp-1,1,betad,4))
                  c2(2) = 0.5d0*(cva(ip-1,jp+1,2,betad,4)+
     +                           cva(ip-1,jp  ,2,betad,4)+
     +                           cva(ip  ,jp+1,2,betad,4)+
     +                           cva(ip  ,jp-1,2,betad,4))

                  c = fact/(c2(2)*c1(1)-c2(1)*c1(2))
                  fact = c*(adir(ip,jp,1)*c2(2)-adir(ip,jp,2)*c2(1))
                  gmat(ip,jp,alphad,3) = -fact
                  gmat(ip,jp,alphad,4) =  fact
                  fact = c*(-adir(ip,jp,1)*c1(2)+adir(ip,jp,2)*c1(1))

                  gmat(ip,jp,alphad,3) = gmat(ip,jp,alphad,3) - fact
                  gmat(ip,jp,alphad,5) =  fact
                  gmat(ip,jp,alphad,6) =  fact
                  gmat(ip,jp,alphad,2) = -fact
                  gmat(ip,jp,alphad,1) =  0d0

               end if

            end if

            btype   = ibinfor(ipoint)

         end do

         jp = nj
         do ip = 1, iupploop

            iside   = 4
            ipibinf = ibipoint(iside)-1+nvirtual
            ipoint  = ipibinf + 2*ip -1
            btype   = ibinfor(ipoint)

            skip    = btype.ge.88 .and. btype.le.100

            if ( .not. skip ) then

               fact = jac(ip,jp)
               c1(1) = cva(ip,jp,1,alphad,3)
               c1(2) = cva(ip,jp,2,alphad,3)

               c2(1) = 0.5d0*(cva(ip-1,jp-1,1,betad,4)+
     +                        cva(ip-1,jp  ,1,betad,4)+
     +                        cva(ip  ,jp-1,1,betad,4)+
     +                        cva(ip  ,jp  ,1,betad,4))
               c2(2) = 0.5d0*(cva(ip-1,jp-1,2,betad,4)+
     +                        cva(ip-1,jp  ,2,betad,4)+
     +                        cva(ip  ,jp-1,2,betad,4)+
     +                        cva(ip  ,jp  ,2,betad,4))

               c = fact/(c2(2)*c1(1)-c2(1)*c1(2))
               fact = c*(adir(ip,jp,1)*c2(2)-adir(ip,jp,2)*c2(1))
               gmat(ip,jp,alphad,3) = -fact
               gmat(ip,jp,alphad,4) =  fact
               fact = c*(-adir(ip,jp,1)*c1(2)+adir(ip,jp,2)*c1(1))

               gmat(ip,jp,alphad,1) = -fact
               gmat(ip,jp,alphad,2) = -fact
               gmat(ip,jp,alphad,3) = gmat(ip,jp,alphad,3) + fact
               gmat(ip,jp,alphad,4) = gmat(ip,jp,alphad,4) + fact
               gmat(ip,jp,alphad,5) =  0d0
               gmat(ip,jp,alphad,6) =  0d0

            else if ( btype.lt.90 ) then

c           --- transition from multiblock boundary to fixed boundary

               btypenb = ibinfor(ipoint-1)
               if ( btypenb.gt.90 ) then

c              --- Previous point is multiblock
c                  eliminate point 6

                  fact = jac(ip,jp)
                  c1(1) = cva(ip,jp,1,alphad,3)
                  c1(2) = cva(ip,jp,2,alphad,3)

                  c2(1) = 0.5d0*(cva(ip-1,jp+1,1,betad,4)+
     +                           cva(ip-1,jp-1,1,betad,4)+
     +                           cva(ip  ,jp-1,1,betad,4)+
     +                           cva(ip  ,jp  ,1,betad,4))
                  c2(2) = 0.5d0*(cva(ip-1,jp+1,2,betad,4)+
     +                           cva(ip-1,jp-1,2,betad,4)+
     +                           cva(ip  ,jp-1,2,betad,4)+
     +                           cva(ip  ,jp  ,2,betad,4))

                  c = fact/(c2(2)*c1(1)-c2(1)*c1(2))
                  fact = c*(adir(ip,jp,1)*c2(2)-adir(ip,jp,2)*c2(1))
                  gmat(ip,jp,alphad,3) = -fact
                  gmat(ip,jp,alphad,4) =  fact
                  fact = c*(-adir(ip,jp,1)*c1(2)+adir(ip,jp,2)*c1(1))

                  gmat(ip,jp,alphad,4) = gmat(ip,jp,alphad,4) - fact
                  gmat(ip,jp,alphad,5) =  fact
                  gmat(ip,jp,alphad,2) = -fact
                  gmat(ip,jp,alphad,1) = -fact
                  gmat(ip,jp,alphad,6) =  0d0

               else

c              --- Next point is multiblock
c                  eliminate point 5

                  fact = jac(ip,jp)
                  c1(1) = cva(ip,jp,1,alphad,3)
                  c1(2) = cva(ip,jp,2,alphad,3)

                  c2(1) = 0.5d0*(cva(ip-1,jp  ,1,betad,4)+
     +                           cva(ip-1,jp-1,1,betad,4)+
     +                           cva(ip  ,jp+1,1,betad,4)+
     +                           cva(ip  ,jp-1,1,betad,4))
                  c2(2) = 0.5d0*(cva(ip-1,jp  ,2,betad,4)+
     +                           cva(ip-1,jp-1,2,betad,4)+
     +                           cva(ip  ,jp+1,2,betad,4)+
     +                           cva(ip  ,jp-1,2,betad,4))

                  c = fact/(c2(2)*c1(1)-c2(1)*c1(2))
                  fact = c*(adir(ip,jp,1)*c2(2)-adir(ip,jp,2)*c2(1))
                  gmat(ip,jp,alphad,3) = -fact
                  gmat(ip,jp,alphad,4) =  fact
                  fact = c*(-adir(ip,jp,1)*c1(2)+adir(ip,jp,2)*c1(1))

                  gmat(ip,jp,alphad,3) = gmat(ip,jp,alphad,3) - fact
                  gmat(ip,jp,alphad,1) = -fact
                  gmat(ip,jp,alphad,6) =  fact
                  gmat(ip,jp,alphad,2) = -fact
                  gmat(ip,jp,alphad,5) =  0d0

               end if

            end if

            btype   = ibinfor(ipoint)

         end do

c        --- one sided approximation for normal half cells at the boundary

         ip = 1
         do jp = 1, jupploop

            iside   = 1
            ipibinf = ibipoint(iside)-1+nvirtual
            ipoint  = ipibinf + 2*jp
            btype   = ibinfor(ipoint)

            skip    = btype.ge.88 .and. btype.le.100

            if ( .not. skip ) then

c           --- for half cell use 0.5 * sqrt(g) !

               fact = 0.5d0*jac(ip,jp)
               c1(1) = 0.5d0*cva(ip,jp,1,alphad,3)
               c1(2) = 0.5d0*cva(ip,jp,2,alphad,3)

               c2(1) = 0.5d0*(cva(ip  ,jp-1,1,betad,4)+
     +                        cva(ip  ,jp+1,1,betad,4))
     +                        +cva(ip,jp,1,betad,4)
               c2(2) = 0.5d0*(cva(ip  ,jp-1,2,betad,4)+
     +                        cva(ip  ,jp+1,2,betad,4))
     +                        +cva(ip,jp,2,betad,4)

               c = fact/(c2(2)*c1(1)-c2(1)*c1(2))
               fact = c*(adir(ip,jp,1)*c2(2)-adir(ip,jp,2)*c2(1))
               gmat(ip,jp,alphad,3) =  0d0
               gmat(ip,jp,alphad,4) =  fact
               fact = c*(-adir(ip,jp,1)*c1(2)+adir(ip,jp,2)*c1(1))

               gmat(ip,jp,alphad,1) =  0d0
               gmat(ip,jp,alphad,2) = -fact
               gmat(ip,jp,alphad,5) =  0d0
               gmat(ip,jp,alphad,6) =  fact

            end if

         end do

         ip = ni+1
         do jp = 1, jupploop

            iside   = 2
            ipibinf = ibipoint(iside)-1+nvirtual
            ipoint  = ipibinf + 2*jp
            btype   = ibinfor(ipoint)

            skip    = btype.ge.88 .and. btype.le.100

            if ( .not. skip ) then

c           --- for half cell use 0.5 * sqrt(g) !

               fact = 0.5d0*jac(ip,jp)
               c1(1) = 0.5d0*cva(ip,jp,1,alphad,3)
               c1(2) = 0.5d0*cva(ip,jp,2,alphad,3)

               c2(1) = 0.5d0*(cva(ip-1,jp-1,1,betad,4)+
     +                        cva(ip-1,jp+1,1,betad,4))+
     +                        cva(ip-1,jp,1,betad,4)
               c2(2) = 0.5d0*(cva(ip-1,jp-1,2,betad,4)+
     +                        cva(ip-1,jp+1,2,betad,4))+
     +                        cva(ip-1,jp,2,betad,4)

               c = fact/(c2(2)*c1(1)-c2(1)*c1(2))
               fact = c*(adir(ip,jp,1)*c2(2)-adir(ip,jp,2)*c2(1))
               gmat(ip,jp,alphad,3) = -fact
               gmat(ip,jp,alphad,4) =  0d0
               fact = c*(-adir(ip,jp,1)*c1(2)+adir(ip,jp,2)*c1(1))

               gmat(ip,jp,alphad,1) = -fact
               gmat(ip,jp,alphad,2) =  0d0
               gmat(ip,jp,alphad,5) =  fact
               gmat(ip,jp,alphad,6) =  0d0

            end if

         end do

      else

c     --- V2-cell

         do jp = 1, nj+1
            do i = 1, 6
               gmat(ni+1,jp,alphad,i) = 0d0
            end do
         end do
         iupploop = ni
         jupploop = nj+1
         betad = 1

c        --- Actually filling of gmat

         do ip = 1, iupploop
            do jp = 1, jupploop

               fact = jac(ip,jp)
               c1(1) = cva(ip,jp,1,alphad,2)
               c1(2) = cva(ip,jp,2,alphad,2)

               c2(1) = 0.5d0*(cva(ip-1,jp-1,1,betad,4)+
     +                        cva(ip-1,jp  ,1,betad,4)+
     +                        cva(ip+1,jp  ,1,betad,4)+
     +                        cva(ip+1,jp-1,1,betad,4))+
     +                 cva(ip,jp-1,1,betad,4)+cva(ip,jp,1,betad,4)
               c2(2) = 0.5d0*(cva(ip-1,jp-1,2,betad,4)+
     +                        cva(ip-1,jp  ,2,betad,4)+
     +                        cva(ip+1,jp  ,2,betad,4)+
     +                        cva(ip+1,jp-1,2,betad,4))+
     +                 cva(ip,jp-1,2,betad,4)+cva(ip,jp,2,betad,4)

               c = fact/(c2(2)*c1(1)-c2(1)*c1(2))
               fact = c*(adir(ip,jp,1)*c2(2)-adir(ip,jp,2)*c2(1))
               gmat(ip,jp,alphad,3) = -fact
               gmat(ip,jp,alphad,4) =  fact
               fact = c*(-adir(ip,jp,1)*c1(2)+adir(ip,jp,2)*c1(1))

               gmat(ip,jp,alphad,1) = -fact
               gmat(ip,jp,alphad,2) = -fact
               gmat(ip,jp,alphad,5) =  fact
               gmat(ip,jp,alphad,6) =  fact

            end do
         end do

c        --- one sided approximation for tangential cells at the boundary

         ip = 1
         do jp = 1, jupploop

            iside   = 1
            ipibinf = ibipoint(iside)-1+nvirtual
            ipoint  = ipibinf + 2*jp -1
            btype   = ibinfor(ipoint)

            skip    = btype.ge.88 .and. btype.le.100

            if ( .not. skip ) then

               fact = jac(ip,jp)
               c1(1) = cva(ip,jp,1,alphad,2)
               c1(2) = cva(ip,jp,2,alphad,2)

               c2(1) = 0.5d0*(cva(ip  ,jp  ,1,betad,4)+
     +                        cva(ip+1,jp  ,1,betad,4)+
     +                        cva(ip  ,jp-1,1,betad,4)+
     +                        cva(ip+1,jp-1,1,betad,4))
               c2(2) = 0.5d0*(cva(ip  ,jp  ,2,betad,4)+
     +                        cva(ip+1,jp  ,2,betad,4)+
     +                        cva(ip  ,jp-1,2,betad,4)+
     +                        cva(ip+1,jp-1,2,betad,4))

               c = fact/(c2(2)*c1(1)-c2(1)*c1(2))
               fact = c*(adir(ip,jp,1)*c2(2)-adir(ip,jp,2)*c2(1))
               gmat(ip,jp,alphad,3) = -fact
               gmat(ip,jp,alphad,4) =  fact
               fact = c*(-adir(ip,jp,1)*c1(2)+adir(ip,jp,2)*c1(1))

               gmat(ip,jp,alphad,3) = gmat(ip,jp,alphad,3) -fact
               gmat(ip,jp,alphad,4) = gmat(ip,jp,alphad,4) -fact
               gmat(ip,jp,alphad,5) =  fact
               gmat(ip,jp,alphad,6) =  fact
               gmat(ip,jp,alphad,1) =  0d0
               gmat(ip,jp,alphad,2) =  0d0

            else if ( btype.lt.90 ) then

c           --- transition from multiblock boundary to fixed boundary

               btypenb = ibinfor(ipoint-1)
               if ( btypenb.gt.90 ) then

c              --- Previous point is multiblock
c                  eliminate point 2

                  fact = jac(ip,jp)
                  c1(1) = cva(ip,jp,1,alphad,2)
                  c1(2) = cva(ip,jp,2,alphad,2)

                  c2(1) = 0.5d0*(cva(ip  ,jp  ,1,betad,4)+
     +                           cva(ip+1,jp  ,1,betad,4)+
     +                           cva(ip-1,jp-1,1,betad,4)+
     +                           cva(ip+1,jp-1,1,betad,4))
                  c2(2) = 0.5d0*(cva(ip  ,jp  ,2,betad,4)+
     +                           cva(ip+1,jp  ,2,betad,4)+
     +                           cva(ip-1,jp-1,2,betad,4)+
     +                           cva(ip+1,jp-1,2,betad,4))

                  c = fact/(c2(2)*c1(1)-c2(1)*c1(2))
                  fact = c*(adir(ip,jp,1)*c2(2)-adir(ip,jp,2)*c2(1))
                  gmat(ip,jp,alphad,3) = -fact
                  gmat(ip,jp,alphad,4) =  fact
                  fact = c*(-adir(ip,jp,1)*c1(2)+adir(ip,jp,2)*c1(1))

                  gmat(ip,jp,alphad,4) = gmat(ip,jp,alphad,4) -fact
                  gmat(ip,jp,alphad,5) =  fact
                  gmat(ip,jp,alphad,6) =  fact
                  gmat(ip,jp,alphad,1) = -fact
                  gmat(ip,jp,alphad,2) =  0d0

               else

c              --- Next point is multiblock
c                  eliminate point 1

                  fact = jac(ip,jp)
                  c1(1) = cva(ip,jp,1,alphad,2)
                  c1(2) = cva(ip,jp,2,alphad,2)

                  c2(1) = 0.5d0*(cva(ip-1,jp  ,1,betad,4)+
     +                           cva(ip+1,jp  ,1,betad,4)+
     +                           cva(ip  ,jp-1,1,betad,4)+
     +                           cva(ip+1,jp-1,1,betad,4))
                  c2(2) = 0.5d0*(cva(ip-1,jp  ,2,betad,4)+
     +                           cva(ip+1,jp  ,2,betad,4)+
     +                           cva(ip  ,jp-1,2,betad,4)+
     +                           cva(ip+1,jp-1,2,betad,4))

                  c = fact/(c2(2)*c1(1)-c2(1)*c1(2))
                  fact = c*(adir(ip,jp,1)*c2(2)-adir(ip,jp,2)*c2(1))
                  gmat(ip,jp,alphad,3) = -fact
                  gmat(ip,jp,alphad,4) =  fact
                  fact = c*(-adir(ip,jp,1)*c1(2)+adir(ip,jp,2)*c1(1))

                  gmat(ip,jp,alphad,3) = gmat(ip,jp,alphad,3) -fact
                  gmat(ip,jp,alphad,5) =  fact
                  gmat(ip,jp,alphad,6) =  fact
                  gmat(ip,jp,alphad,2) = -fact
                  gmat(ip,jp,alphad,1) =  0d0

               end if

            end if

            btype   = ibinfor(ipoint)

         end do

         ip = ni
         do jp = 1, jupploop

            iside   = 2
            ipibinf = ibipoint(iside)-1+nvirtual
            ipoint  = ipibinf + 2*jp -1
            btype   = ibinfor(ipoint)

            skip    = btype.ge.88 .and. btype.le.100

            if ( .not. skip ) then

               fact = jac(ip,jp)
               c1(1) = cva(ip,jp,1,alphad,2)
               c1(2) = cva(ip,jp,2,alphad,2)

               c2(1) = 0.5d0*(cva(ip  ,jp  ,1,betad,4)+
     +                        cva(ip-1,jp  ,1,betad,4)+
     +                        cva(ip  ,jp-1,1,betad,4)+
     +                        cva(ip-1,jp-1,1,betad,4))
               c2(2) = 0.5d0*(cva(ip  ,jp  ,2,betad,4)+
     +                        cva(ip-1,jp  ,2,betad,4)+
     +                        cva(ip  ,jp-1,2,betad,4)+
     +                        cva(ip-1,jp-1,2,betad,4))

               c = fact/(c2(2)*c1(1)-c2(1)*c1(2))
               fact = c*(adir(ip,jp,1)*c2(2)-adir(ip,jp,2)*c2(1))
               gmat(ip,jp,alphad,3) = -fact
               gmat(ip,jp,alphad,4) =  fact
               fact = c*(-adir(ip,jp,1)*c1(2)+adir(ip,jp,2)*c1(1))

               gmat(ip,jp,alphad,3) = gmat(ip,jp,alphad,3) +fact
               gmat(ip,jp,alphad,4) = gmat(ip,jp,alphad,4) +fact
               gmat(ip,jp,alphad,1) = -fact
               gmat(ip,jp,alphad,2) = -fact
               gmat(ip,jp,alphad,5) =  0d0
               gmat(ip,jp,alphad,6) =  0d0

            else if ( btype.lt.90 ) then

c           --- transition from multiblock boundary to fixed boundary

               btypenb = ibinfor(ipoint-1)
               if ( btypenb.gt.90 ) then

c              --- Previous point is multiblock
c                  eliminate point 6

                  fact = jac(ip,jp)
                  c1(1) = cva(ip,jp,1,alphad,2)
                  c1(2) = cva(ip,jp,2,alphad,2)

                  c2(1) = 0.5d0*(cva(ip  ,jp  ,1,betad,4)+
     +                           cva(ip-1,jp  ,1,betad,4)+
     +                           cva(ip-1,jp-1,1,betad,4)+
     +                           cva(ip+1,jp-1,1,betad,4))
                  c2(2) = 0.5d0*(cva(ip  ,jp  ,2,betad,4)+
     +                           cva(ip-1,jp  ,2,betad,4)+
     +                           cva(ip-1,jp-1,2,betad,4)+
     +                           cva(ip+1,jp-1,2,betad,4))

                  c = fact/(c2(2)*c1(1)-c2(1)*c1(2))
                  fact = c*(adir(ip,jp,1)*c2(2)-adir(ip,jp,2)*c2(1))
                  gmat(ip,jp,alphad,3) = -fact
                  gmat(ip,jp,alphad,4) =  fact
                  fact = c*(-adir(ip,jp,1)*c1(2)+adir(ip,jp,2)*c1(1))

                  gmat(ip,jp,alphad,4) = gmat(ip,jp,alphad,4) -fact
                  gmat(ip,jp,alphad,5) =  fact
                  gmat(ip,jp,alphad,2) =  fact
                  gmat(ip,jp,alphad,1) = -fact
                  gmat(ip,jp,alphad,6) =  0d0

               else

c              --- Next point is multiblock
c                  eliminate point 5

                  fact = jac(ip,jp)
                  c1(1) = cva(ip,jp,1,alphad,2)
                  c1(2) = cva(ip,jp,2,alphad,2)

                  c2(1) = 0.5d0*(cva(ip+1,jp  ,1,betad,4)+
     +                           cva(ip-1,jp  ,1,betad,4)+
     +                           cva(ip-1,jp-1,1,betad,4)+
     +                           cva(ip  ,jp-1,1,betad,4))
                  c2(2) = 0.5d0*(cva(ip+1,jp  ,2,betad,4)+
     +                           cva(ip-1,jp  ,2,betad,4)+
     +                           cva(ip  ,jp-1,2,betad,4)+
     +                           cva(ip-1,jp-1,2,betad,4))

                  c = fact/(c2(2)*c1(1)-c2(1)*c1(2))
                  fact = c*(adir(ip,jp,1)*c2(2)-adir(ip,jp,2)*c2(1))
                  gmat(ip,jp,alphad,3) = -fact
                  gmat(ip,jp,alphad,4) =  fact
                  fact = c*(-adir(ip,jp,1)*c1(2)+adir(ip,jp,2)*c1(1))

                  gmat(ip,jp,alphad,3) = gmat(ip,jp,alphad,3) -fact
                  gmat(ip,jp,alphad,1) =  fact
                  gmat(ip,jp,alphad,6) =  fact
                  gmat(ip,jp,alphad,2) = -fact
                  gmat(ip,jp,alphad,5) =  0d0

               end if

            end if

            btype   = ibinfor(ipoint)

         end do

c        --- one sided approximation for normal half cells at the boundary

         jp = 1
         do ip = 1, iupploop

            iside   = 3
            ipibinf = ibipoint(iside)-1+nvirtual
            ipoint  = ipibinf + 2*ip
            btype   = ibinfor(ipoint)

            skip    = btype.ge.88 .and. btype.le.100

            if ( .not. skip ) then

c           --- for half cell use 0.5 * sqrt(g) !

               fact = 0.5d0*jac(ip,jp)
               c1(1) = 0.5d0*cva(ip,jp,1,alphad,2)
               c1(2) = 0.5d0*cva(ip,jp,2,alphad,2)

               c2(1) = 0.5d0*(cva(ip-1,jp  ,1,betad,4)+
     +                        cva(ip+1,jp  ,1,betad,4))
     +                        +cva(ip,jp,1,betad,4)
               c2(2) = 0.5d0*(cva(ip-1,jp  ,2,betad,4)+
     +                        cva(ip+1,jp  ,2,betad,4))+
     +                        cva(ip,jp,2,betad,4)

               c = fact/(c2(2)*c1(1)-c2(1)*c1(2))
               fact = c*(adir(ip,jp,1)*c2(2)-adir(ip,jp,2)*c2(1))
               gmat(ip,jp,alphad,3) =  0d0
               gmat(ip,jp,alphad,4) =  fact
               fact = c*(-adir(ip,jp,1)*c1(2)+adir(ip,jp,2)*c1(1))

               gmat(ip,jp,alphad,1) =  0d0
               gmat(ip,jp,alphad,2) = -fact
               gmat(ip,jp,alphad,5) =  0d0
               gmat(ip,jp,alphad,6) =  fact

            end if

         end do

         jp = nj+1
         do ip = 1, iupploop

            iside   = 4
            ipibinf = ibipoint(iside)-1+nvirtual
            ipoint  = ipibinf + 2*ip
            btype   = ibinfor(ipoint)

            skip    = btype.ge.88 .and. btype.le.100

            if ( .not. skip ) then

c           --- for half cell use 0.5 * sqrt(g) !

               fact = 0.5d0*jac(ip,jp)
               c1(1) = 0.5d0*cva(ip,jp,1      ,alphad,2)
               c1(2) = 0.5d0*cva(ip,jp,2      ,alphad,2)

               c2(1) = 0.5d0*(cva(ip-1,jp-1,1,betad,4)+
     +                        cva(ip+1,jp-1,1,betad,4))+
     +                        cva(ip,jp-1,1,betad,4)
               c2(2) = 0.5d0*(cva(ip-1,jp-1,2,betad,4)+
     +                        cva(ip+1,jp-1,2,betad,4))+
     +                        cva(ip,jp-1,2,betad,4)

               c = fact/(c2(2)*c1(1)-c2(1)*c1(2))
               fact = c*(adir(ip,jp,1)*c2(2)-adir(ip,jp,2)*c2(1))
               gmat(ip,jp,alphad,3) = -fact
               gmat(ip,jp,alphad,4) =  0d0
               fact = c*(-adir(ip,jp,1)*c1(2)+adir(ip,jp,2)*c1(1))

               gmat(ip,jp,alphad,1) = -fact
               gmat(ip,jp,alphad,2) =  0d0
               gmat(ip,jp,alphad,5) =  fact
               gmat(ip,jp,alphad,6) =  0d0

            end if

         end do

      end if

      end
