      subroutine isgmat23 ( iface, ni, nj, nk, nvirtual, gmat, cva,
     +                      invjac, ilow1, ihigh1, ilow2, ihigh2,
     +                      ibinfor )
c ======================================================================
c
c        programmer     Kees Kassels
c        version 1.1    date 07-06-1996 for tangential cell fact * 0.25
c                                       and geometry in velocity point
c        version 1.0    date 19-01-1996
c
c   copyright (c) 1995-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                         DESCRIPTION
c
c **********************************************************************
c
c                      KEYWORDS
c
c **********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer ni, nj, nk, nvirtual, ilow1, ihigh1,ilow2, ihigh2,
     +        ibinfor(ilow1:ihigh1,ilow2:ihigh2), iface

      double precision gmat(ni+1,nj+1,nk+1,3,10),
     +                 cva(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,
     +                     1-nvirtual:nk+1+nvirtual,3,3,8),
     +                 invjac(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,
     +                        1-nvirtual:nk+1+nvirtual,8)

c     cva       i    Array containing the covariant basis vectors
c                    in 2D we have
c                                                 l
c                    cva(i,j,l,alpha,p) contains a        in cell i,j in
c                                                 (alpha)       point type p
c                    in 3D it is cva(i,j,k,l,alpha,p) referring to cell i,j,k
c     gmat      i    Gradient matrix
c     ibinfor   i    Third part of infbnd
c     iface     i    Face number
c     ihigh1    i    First upper index of an array
c     ihigh2    i    Second upper index of an array
c     ilow1     i    First lower index of an array
c     ilow2     i    Second lower index of an array
c     invjac    i    1/sqrt(g) in pnttyp
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c **********************************************************************
c
c                    COMMON BLOCKS
c
c **********************************************************************
c
c                    LOCAL PARAMETERS
c
      integer unut, snnsnt, snnut, periodic, antiper, multiblk,
     +        uxuy, unsnt,  perjump, antijump,
     +        oldturb, parabolic, skip
      parameter( unut = 4, snnsnt = 7, periodic = 98,
     +           antiper = 97, perjump = 96, antijump =95,
     +           multiblk = 99, uxuy = 10, unsnt = 5, snnut = 6,
     +           oldturb = 9, parabolic = 12)
      parameter ( skip = 300)
      logical  ungiven
      integer  ndir, tdir(2), nsign, iparea(2,3), nijk(3), ktdir,
     +         pchr(2), xp(3), cxp(3), pnttyp, alphal, betal,
     +         valpha(3), vbeta(3), ict1, ict2, btype, chpnt, chdir,
     +         chxp1(3), chxp2(3), ip, jp, kp, inside, i, j,
     +         alphad, betad

      double precision chsym(5), ctva(3,3), fact, gcov(3,3)

c     alphad    Direction index of integration.
c               Hence if a V1-cell is used, alphad = 1 and so on
c     alphal    Loop index
c     antijump  Anti-periodic boundary conditions with jump in
c               pressure
c     antiper   Anti-periodic boundary conditions
c     betad     Direction index, usually the direction perpendicular to the
c               integration direction of the boundary integral.
c               For example betad in local point (1,0) is 1 and in (0,1): 2
c     betal     Loop index
c     btype     Type of boundary condition
c     chdir     Direction for christoffel symbols
c     chpnt     Pointtype for christoffel symbols
c     chsym     Christoffel symbols of the second kind in the
c               point given by pnttyp
c               (symmetric contravariant part stored as vector
c               with index encoding given by istwo)
c     chxp1     Auxiliary array for christoffel symbols
c     chxp2     Auxiliary array for christoffel symbols
c     ctva      Contravariant base vectors times jacobian
c     cxp       Auxiliary array containing cellindex for coefficients
c     fact      Multiplication factor
c     gcov      Covarariant metric tensor
c     i         General loop variable
c     ict1      Pointer in ibinfor with respect to boundary point
c     ict2      Pointer in ibinfor with respect to boundary point
c     inside    Position of connection corresponding to point inside the inner
c               region opposite the boundary point
c     ip        Local counting variable i-direction
c     iparea    Area of integration points where a contribution to
c               the matrix is to be calculated
c     j         General loop variable
c     jp        Local counting variable j-direction
c     kp        Local counting variable k-direction
c     ktdir     Sequence number of tangential direction (1 or 2)
c     multiblk  Multi-block boundary conditions
c     ndir      Direction of normal vector on this side
c     nijk      Vector of number of points in the i,j and k-directions
c     nsign     Inner product of outward normal in computational domain
c               with the base vector perpendicular to the face
c     oldturb   Old turbulence boundary condtion
c               (equivalent to unsnt)
c     parabolic Parabolic boundary conditions
c     pchr      Auxiliary array containing pointtype for christoffel sybols
c     periodic  Periodic boundary conditions
c     perjump   Periodic boundary conditions with jump in pressure
c     pnttyp    reserved
c     skip      If true boundary condition must be skipped
c     snnsnt    Sigma_nn and sigma_nt prescribed
c     snnut     Sigma_nn and u_t prescribed
c     tdir      Direction of tangent vector on this side
c     ungiven   If true un is given
c     unsnt     U_n and sigma_nt prescribed
c     unut      U_n and u_t prescribed
c     uxuy      U_x and u_y prescribed
c     valpha    Array for cyclic changing ordering
c               alphal = 1       alphal = 2   alphal = 3
c               valpha(1) =1 ---> valpha(1)=2  valpha(1)=3
c               valpha(2) =2 ---> valpha(2)=3  valpha(2)=1
c               valpha(3) =3 ---> valpha(3)=1  valpha(3)=2
c     vbeta     Array for cyclic changing ordering
c               betal = 1        betal = 2    betal = 3
c               vbeta(1) =1 ---> vbeta(1)=2  vbeta(1)=3
c               vbeta(2) =2 ---> vbeta(2)=3  vbeta(2)=1
c               vbeta(3) =3 ---> vbeta(3)=1  vbeta(3)=2
c     xp        Auxiliary array to calculate boundary point number
c **********************************************************************
c
c             SUBROUTINES CALLED
c
c
c **********************************************************************
c
c                              I/O
c
c **********************************************************************
c
c                    ERROR MESSAGES
c
c **********************************************************************
c
c                    PSEUDO CODE
c
c     The discretization of a normal half cell is computed completely in this
c     subroutine, the discretization of a tangential cell has been
c     carried out before. This subroutine only changes elements for
c     the tangential cells
c
c   Consider a normal half cell at the lower boundary, i.e.
c       ntdir=(3,1,2)
c       ntsig=(-1,1,1)
c
c
c   The picture shows the intersection with the plane ksi2 = constant (0)
c
c         ------------------------------------------
c         |              |            |            |
c         |              |            |            |
c         |              |            |            |
c         |              +-----+------+            |
c         |              |            |            |
c         |              |            |            |
c         |              |            |            |
c         ---------------+-----*------+-------------
c
c
c   The stress term for this cell has the following shape:
c
c  1          a1 |i+1/2,j,k    1         a2 |i,j+1/2,k            a3 |i,j,k+1/2
c  - sqrt(g) T   |          +  -sqrt(g) T   |          + sqrt(g) T   |
c  2             |i-1/2,j,k    2            |i,j-1/2,k               |i,j,k
c
c
c      1         |  a  |  bc |
c    + - sqrt(g) |     | T   |
c      2         | b c |     |i,j,k
c
c                                         ab    ab
c   With respect to the pressure we have T   = g  p
c
c   The following boundary conditions have the following effects for the
c   pressure
c
c
c   un given in boundary centroid:  matrix elements equal to 0
c
c   Sigma nn, Sigma nt given:
c
c   The terms involving the pressure that remain are:
c
c            33                         | 3 | 11   | 3 | 22   | 3 | 21
c   sqrt(g) g   p |       + 0.5 sqrt(g)(|   |g p + |   |g p +2|   |g p)|
c                 |(0,0,1)              |1 1|      |2 2|      |2 1|    |(0,0,0)
c
c   However, in order to make things simple (0,0,0) is replaced by (0,0,1)
c
c   Sigma nn, ut given:
c
c   The terms involving the pressure that remain are:
c
c                31  |(1,0,0)               32  |(0,1,0)           33
c   0.5 sqrt(g) g  p |       + 0.5 sqrt(g) g  p |       + sqrt(g) g  p |      +
c                    |(-1,0,0)                  |(0,-1,0)              |(0,0,1)
c
c
c                 | 3 |  ab
c    0.5 sqrt(g) (|   | g  p |)
c                 |a b|      |(0,0,0)
c                  a,b #3,3
c   However, in order to make things simple (0,0,0) is replaced by (0,0,1),
c   (1,0,0) by (1,0,1) and (-1,0,0) by (-1,0,1) and so on
c
c
c
c   Consider a tangential cell at the lower boundary, i.e.
c       ntdir=(3,1,2)
c       ntsig=(-1,1,1)
c
c
c
c
c         ---------------+--------------------------
c         |              |            |            |
c         |      |       |     |      |            |
c         |              |            |            |
c         |      +       *     +      |            |
c         |              |            |            |
c         |      |       |     |      |            |
c         |              |            |            |
c         ---------------+--------------------------
c
c    The only position influenced by the boundary conditions is
c    the point (0,0,-1), where we have the term:
c
c    -sqrt(g) sigma^3t |
c                      | (0,0,-1)
c
c    Hence only in the case sigma^nt given some action should be carried out
c
c    In the case sigma^nn and sigma^nt given we have sigma^3t given.
c    It is sufficient to remove the pressure term from the point (0,0,-1)
c
c    In the case sigma^nt given we have:
c
c      sigma^nt1 = sqrt(g^nn) S^nt alpha(t1) -
c
c                   g_t2t2 g_nt1 - g_t1t2 g_nt2
c                  ----------------------------- sigma^nn
c                  g_t1t1 g_t2t2 - g_t1t2 g_t1t2
c
c
c      sigma^nt2 = sqrt(g^nn) S^nt alpha(t2) -
c
c                   g_t1t1 g_nt2 - g_t1t2 g_nt1
c                  ----------------------------- sigma^nn
c                  g_t1t1 g_t2t2 - g_t1t2 g_t1t2
c
c    Hence first we have to remove the pressure term from the point (0,0,-1)
c    and then we have to add the contribution:
c
c    -f g^33 sqrt(g), where f is the factor given above before sigma^nn
c
c **********************************************************************
c
c                    DATA STATEMENTS
c
c ======================================================================
c
c     --- loop over faces

      nijk(1) = ni
      nijk(2) = nj
      nijk(3) = nk

c     --- Compute tdir and ndir

      if ( iface.le.2 ) then

c     --- iface = 1, 2:  back and front plane

         ndir    = 1
         tdir(1) = 2
         tdir(2) = 3
         pchr(1) = 6
         pchr(2) = 4
         if ( iface.eq.1 ) then

c        --- iface = 1:  back

            nsign = -1

         else

c        --- iface = 2:  front

            nsign = 1

         end if

      else if ( iface.gt.2  .and. iface.lt.5) then

c     --- iface = 3, 4:  left- and right plane

         ndir = 2
         tdir(1) = 3
         tdir(2) = 1
         pchr(1) = 4
         pchr(2) = 7
         if ( iface.eq.3 ) then

c        --- iface = 3:  left

            nsign = -1

         else

c        --- iface = 4:  right

            nsign = 1

         end if

      else

c     --- iface = 5, 6:  bottom and top

         ndir = 3
         tdir(1) = 1
         tdir(2) = 2
         pchr(1) = 7
         pchr(2) = 6

         if ( iface.eq.5 ) then

c        --- iface = 5:  bottom

            nsign = -1

         else

c        --- iface = 6:  top

            nsign = 1

         end if

      end if

c     --- loop over normal half cells
c         Compute loop bounds

      if ( mod(iface,2).eq.0 ) then
         iparea(1,ndir) = nijk(ndir)+1
         iparea(2,ndir) = nijk(ndir)+1
      else
         iparea(1,ndir) = 1
         iparea(2,ndir) = 1
      end if

      iparea(1,tdir(1)) = 1
      iparea(2,tdir(1)) = nijk(tdir(1))
      iparea(1,tdir(2)) = 1
      iparea(2,tdir(2)) = nijk(tdir(2))

      do 400 kp = iparea(1,3), iparea(2,3)
         do 300 jp = iparea(1,2), iparea(2,2)
            do 200 ip = iparea(1,1), iparea(2,1)

c           --- Store integration cell indices in vector

               xp(1) = ip
               xp(2) = jp
               xp(3) = kp

c              --- Get corresponding index into ibinfor
c                  Get boundary information

               ict1 = 2*xp(tdir(1))
               ict2 = 2*xp(tdir(2))
               btype = ibinfor(ict1,ict2)

c              --- Get reduced boundary condition type

               if ( btype.eq.parabolic ) then
                  btype = unut
               else if ( btype.eq.oldturb ) then
                  btype = unsnt
               else if ( btype.eq.multiblk .or. btype.eq.periodic .or.
     +                   btype.eq.antiper .or.
     +                   btype.eq.perjump .or. btype.eq.antijump ) then
                  btype = skip
               end if

               ungiven = (btype.eq.uxuy .or.
     +                    btype.eq.unut .or. btype.eq.parabolic .or.
     +                    btype.eq.unsnt .or. btype.eq.oldturb  )

               if ( ungiven ) then

c              --- Dirichlet half cell, set contribution equal to zero

                  do i = 1, 10
                     gmat(ip,jp,kp,ndir,i) = 0d0
                  end do

               else if ( btype.ne.skip  )
     +            then

c              --- integration half cell for normal direction

                  do i = 1, 10
                     gmat(ip,jp,kp,ndir,i) = 0d0
                  end do

                  do i = 1, 3
                     cxp(i) = xp(i)
                  end do

                  pnttyp = 8
                  inside = 4
                  if ( nsign.eq.1 ) then
                     cxp(ndir) = cxp(ndir) - 1
                     inside = 3
                  end if

c                 --- contravariant base vectors

                  do alphal = 1, 3
                     do betal = 1, 3
                        valpha(1) = alphal
                        valpha(2) = mod(alphal  ,3) + 1
                        valpha(3) = mod(alphal+1,3) + 1
                        vbeta(1)  = betal
                        vbeta(2)  = mod(betal  ,3) + 1
                        vbeta(3)  = mod(betal+1,3) + 1
                    ctva(alphal,betal) =
     +              cva(cxp(1),cxp(2),cxp(3),vbeta(2),valpha(2),pnttyp)*
     +              cva(cxp(1),cxp(2),cxp(3),vbeta(3),valpha(3),pnttyp)-
     +              cva(cxp(1),cxp(2),cxp(3),vbeta(3),valpha(2),pnttyp)*
     +              cva(cxp(1),cxp(2),cxp(3),vbeta(2),valpha(3),pnttyp)
                     end do
                  end do

c                 --- christoffel symbols
c                     chsym(1)     (2)       (3)         (4)        (5)
c                     |  n    |  |  n    |  |  n    |  |  n    |  |  n    |
c                     |       |  |       |  |       |  |       |  |       |
c                     | t1 t1 |  | n  t1 |  | t2 t2 |  | t1 t2 |  | n  t2 |

                  do i = 1, 5

                     if ( i.le.2 ) then
                        chpnt = pchr(1)
                        do j = 1, 3
                           chxp1(j) = xp(j)
                           chxp2(j) = xp(j)
                        end do
                        chxp1(tdir(1)) = chxp1(tdir(1)) + 1
                     else
                        chpnt = pchr(2)
                        do j = 1, 3
                           chxp1(j) = xp(j)
                           chxp2(j) = xp(j)
                        end do
                        chxp1(tdir(2)) = chxp1(tdir(2)) + 1
                     end if
                     if ( i.eq.1 ) then
                        chdir = tdir(1)
                     else if ( i.eq.2 ) then
                        chdir = ndir
                     else if ( i.eq.3 ) then
                        chdir = tdir(2)
                     else if ( i.eq.4 ) then
                        chdir = tdir(1)
                     else
                        chdir = ndir
                     end if

                     chsym(i) = invjac(cxp(1),cxp(2),cxp(3),pnttyp) *
     +                 (ctva(ndir,1) *
     +                 (cva(chxp1(1),chxp1(2),chxp1(3),1,chdir,chpnt)-
     +                  cva(chxp2(1),chxp2(2),chxp2(3),1,chdir,chpnt)) +
     +                  ctva(ndir,2) *
     +                 (cva(chxp1(1),chxp1(2),chxp1(3),2,chdir,chpnt)-
     +                  cva(chxp2(1),chxp2(2),chxp2(3),2,chdir,chpnt)) +
     +                  ctva(ndir,3) *
     +                 (cva(chxp1(1),chxp1(2),chxp1(3),3,chdir,chpnt)-
     +                  cva(chxp2(1),chxp2(2),chxp2(3),3,chdir,chpnt)))

                  end do

                  if ( btype.eq.snnsnt .or. btype.eq.snnut) then

                     gmat(ip,jp,kp,ndir,inside) = -nsign *
     +                  invjac(cxp(1),cxp(2),cxp(3),pnttyp) *
     +                  (ctva(ndir,1)*ctva(ndir,1) +
     +                   ctva(ndir,2)*ctva(ndir,2) +
     +                   ctva(ndir,3)*ctva(ndir,3))  +
     +               .5d0 * chsym(1) *
     +               invjac(cxp(1),cxp(2),cxp(3),pnttyp) *
     +                 (ctva(tdir(1),1)*ctva(tdir(1),1) +
     +                  ctva(tdir(1),2)*ctva(tdir(1),2) +
     +                  ctva(tdir(1),3)*ctva(tdir(1),3))  +
     +               .5d0 * chsym(3) *
     +               invjac(cxp(1),cxp(2),cxp(3),pnttyp) *
     +                 (ctva(tdir(2),1)*ctva(tdir(2),1) +
     +                  ctva(tdir(2),2)*ctva(tdir(2),2) +
     +                  ctva(tdir(2),3)*ctva(tdir(2),3))  +
     +               chsym(4) *
     +               invjac(cxp(1),cxp(2),cxp(3),pnttyp) *
     +                 (ctva(tdir(1),1)*ctva(tdir(2),1) +
     +                  ctva(tdir(1),2)*ctva(tdir(2),2) +
     +                  ctva(tdir(1),3)*ctva(tdir(2),3))

                     if  (  btype.eq.snnut) then

                        gmat(ip,jp,kp,ndir,inside) =
     +                             gmat(ip,jp,kp,ndir,inside) +
     +                  chsym(2) *
     +                  invjac(cxp(1),cxp(2),cxp(3),pnttyp) *
     +                    (ctva(ndir,1)*ctva(tdir(1),1) +
     +                     ctva(ndir,2)*ctva(tdir(1),2) +
     +                     ctva(ndir,3)*ctva(tdir(1),3))  +
     +                  chsym(5) *
     +                   invjac(cxp(1),cxp(2),cxp(3),pnttyp) *
     +                    (ctva(ndir,1)*ctva(tdir(2),1) +
     +                     ctva(ndir,2)*ctva(tdir(2),2) +
     +                     ctva(ndir,3)*ctva(tdir(2),3))

c                       --- n t1  |(1,0,1)
c                           --- 0.5 sq
c                                 |(-1,0,1)
c                           ---

                         do i = 1, 3
                            cxp(i) = xp(i)
                         end do
                         if ( nsign.eq.1 ) then
                            cxp(ndir) = cxp(ndir) - 1
                         end if
                         if ( ndir.eq.1) then
                            pnttyp = 6
                         else if ( ndir.eq.2 ) then
                            pnttyp = 4
                         else
                            pnttyp = 7
                         end if

c                        --- contravariant base vectors

                        do alphal = 1, 3
                           do betal = 1, 3
                              valpha(1) = alphal
                              valpha(2) = mod(alphal  ,3) + 1
                              valpha(3) = mod(alphal+1,3) + 1
                              vbeta(1)  = betal
                              vbeta(2)  = mod(betal  ,3) + 1
                              vbeta(3)  = mod(betal+1,3) + 1
                    ctva(alphal,betal) =
     +              cva(cxp(1),cxp(2),cxp(3),vbeta(2),valpha(2),pnttyp)*
     +              cva(cxp(1),cxp(2),cxp(3),vbeta(3),valpha(3),pnttyp)-
     +              cva(cxp(1),cxp(2),cxp(3),vbeta(3),valpha(2),pnttyp)*
     +              cva(cxp(1),cxp(2),cxp(3),vbeta(2),valpha(3),pnttyp)
                           end do
                        end do

                        fact = .5d0 *
     +                     invjac(cxp(1),cxp(2),cxp(3),pnttyp) *
     +                     (ctva(ndir,1)*ctva(tdir(1),1) +
     +                      ctva(ndir,2)*ctva(tdir(1),2) +
     +                      ctva(ndir,3)*ctva(tdir(1),3))
                        if ( nsign.eq.1 ) then
                           gmat(ip,jp,kp,ndir,1) =
     +                        gmat(ip,jp,kp,ndir,1) - fact
                           gmat(ip,jp,kp,ndir,3) =
     +                        gmat(ip,jp,kp,ndir,3) - fact
                        else
                           gmat(ip,jp,kp,ndir,2) =
     +                        gmat(ip,jp,kp,ndir,2) - fact
                           gmat(ip,jp,kp,ndir,4) =
     +                        gmat(ip,jp,kp,ndir,4) - fact
                        end if

c                       --- point (1,0,1)

                        cxp(tdir(1)) = cxp(tdir(1)) + 1

c                       --- contravariant base vectors

                        do alphal = 1, 3
                           do betal = 1, 3
                              valpha(1) = alphal
                              valpha(2) = mod(alphal  ,3) + 1
                              valpha(3) = mod(alphal+1,3) + 1
                              vbeta(1)  = betal
                              vbeta(2)  = mod(betal  ,3) + 1
                              vbeta(3)  = mod(betal+1,3) + 1
                    ctva(alphal,betal) =
     +              cva(cxp(1),cxp(2),cxp(3),vbeta(2),valpha(2),pnttyp)*
     +              cva(cxp(1),cxp(2),cxp(3),vbeta(3),valpha(3),pnttyp)-
     +              cva(cxp(1),cxp(2),cxp(3),vbeta(3),valpha(2),pnttyp)*
     +              cva(cxp(1),cxp(2),cxp(3),vbeta(2),valpha(3),pnttyp)
                           end do
                        end do

                        fact =  .5d0 *
     +                     invjac(cxp(1),cxp(2),cxp(3),pnttyp) *
     +                     (ctva(ndir,1)*ctva(tdir(1),1) +
     +                      ctva(ndir,2)*ctva(tdir(1),2) +
     +                      ctva(ndir,3)*ctva(tdir(1),3))
                        if ( nsign.eq.1 ) then
                           gmat(ip,jp,kp,ndir,3) =
     +                        gmat(ip,jp,kp,ndir,3) + fact
                           gmat(ip,jp,kp,ndir,5) =
     +                        gmat(ip,jp,kp,ndir,5) + fact
                        else
                           gmat(ip,jp,kp,ndir,4) =
     +                        gmat(ip,jp,kp,ndir,4) + fact
                           gmat(ip,jp,kp,ndir,6) =
     +                        gmat(ip,jp,kp,ndir,6) + fact
                        end if

c                       --- n t2  |(0,0,1)
c                           --- 0.5 sq
c                                 |(0,-1,1)
c                           ---

                         do i = 1, 3
                            cxp(i) = xp(i)
                         end do
                         if ( nsign.eq.1 ) then
                            cxp(ndir) = cxp(ndir) - 1
                         end if
                         if ( ndir.eq.1) then
                            pnttyp = 4
                         else if ( ndir.eq.2 ) then
                            pnttyp = 7
                         else
                            pnttyp = 6
                         end if

c                        --- contravariant base vectors

                        do alphal = 1, 3
                           do betal = 1, 3
                              valpha(1) = alphal
                              valpha(2) = mod(alphal  ,3) + 1
                              valpha(3) = mod(alphal+1,3) + 1
                              vbeta(1)  = betal
                              vbeta(2)  = mod(betal  ,3) + 1
                              vbeta(3)  = mod(betal+1,3) + 1
                       ctva(alphal,betal) =
     +              cva(cxp(1),cxp(2),cxp(3),vbeta(2),valpha(2),pnttyp)*
     +              cva(cxp(1),cxp(2),cxp(3),vbeta(3),valpha(3),pnttyp)-
     +              cva(cxp(1),cxp(2),cxp(3),vbeta(3),valpha(2),pnttyp)*
     +              cva(cxp(1),cxp(2),cxp(3),vbeta(2),valpha(3),pnttyp)
                           end do
                        end do

                        fact = .5d0 *
     +                     invjac(cxp(1),cxp(2),cxp(3),pnttyp) *
     +                     (ctva(ndir,1)*ctva(tdir(2),1) +
     +                      ctva(ndir,2)*ctva(tdir(2),2) +
     +                      ctva(ndir,3)*ctva(tdir(2),3))
                        if ( nsign.eq.1 ) then
                           gmat(ip,jp,kp,ndir,3) =
     +                        gmat(ip,jp,kp,ndir,3) - fact
                           gmat(ip,jp,kp,ndir,7) =
     +                        gmat(ip,jp,kp,ndir,7) - fact
                        else
                           gmat(ip,jp,kp,ndir,4) =
     +                        gmat(ip,jp,kp,ndir,4) - fact
                           gmat(ip,jp,kp,ndir,8) =
     +                        gmat(ip,jp,kp,ndir,8) - fact
                        end if

c                       --- point (1,0,1)

                        cxp(tdir(2)) = cxp(tdir(2)) + 1

c                       --- contravariant base vectors

                        do alphal = 1, 3
                           do betal = 1, 3
                              valpha(1) = alphal
                              valpha(2) = mod(alphal  ,3) + 1
                              valpha(3) = mod(alphal+1,3) + 1
                              vbeta(1)  = betal
                              vbeta(2)  = mod(betal  ,3) + 1
                              vbeta(3)  = mod(betal+1,3) + 1
                    ctva(alphal,betal) =
     +              cva(cxp(1),cxp(2),cxp(3),vbeta(2),valpha(2),pnttyp)*
     +              cva(cxp(1),cxp(2),cxp(3),vbeta(3),valpha(3),pnttyp)-
     +              cva(cxp(1),cxp(2),cxp(3),vbeta(3),valpha(2),pnttyp)*
     +              cva(cxp(1),cxp(2),cxp(3),vbeta(2),valpha(3),pnttyp)
                           end do
                        end do

                        fact =  .5d0 *
     +                     invjac(cxp(1),cxp(2),cxp(3),pnttyp) *
     +                     (ctva(ndir,1)*ctva(tdir(2),1) +
     +                      ctva(ndir,2)*ctva(tdir(2),2) +
     +                      ctva(ndir,3)*ctva(tdir(2),3))
                        if ( nsign.eq.1 ) then
                           gmat(ip,jp,kp,ndir,3)  =
     +                        gmat(ip,jp,kp,ndir,3) + fact
                           gmat(ip,jp,kp,ndir,9)  =
     +                        gmat(ip,jp,kp,ndir,9) + fact
                        else
                           gmat(ip,jp,kp,ndir,4)  =
     +                        gmat(ip,jp,kp,ndir,4) + fact
                           gmat(ip,jp,kp,ndir,10) =
     +                        gmat(ip,jp,kp,ndir,10) + fact
                        end if

                     end if

                  end if
              end if

200         continue
300      continue
400   continue

c     --- loop over two sets of tangential cells
c         Compute loop bounds

      if ( mod(iface,2).eq.0 ) then
         iparea(1,ndir) = nijk(ndir)
         iparea(2,ndir) = nijk(ndir)
      else
         iparea(1,ndir) = 1
         iparea(2,ndir) = 1
      end if

      do ktdir = 1, 2

         iparea(1,tdir(1)) = 1
         iparea(2,tdir(1)) = nijk(tdir(1))
         iparea(1,tdir(2)) = 1
         iparea(2,tdir(2)) = nijk(tdir(2))
         iparea(1,tdir(ktdir)) = 2
         alphad = tdir(ktdir)

         do 700 kp = iparea(1,3), iparea(2,3)
            do 600 jp = iparea(1,2), iparea(2,2)
              do 500 ip = iparea(1,1), iparea(2,1)

                 xp(1) = ip
                 xp(2) = jp
                 xp(3) = kp

                 do i = 1, 3
                    cxp(i) = xp(i)
                 end do

                 if ( alphad.eq.tdir(1) ) then
                    ict1 = 2*cxp(tdir(1))-1
                 else
                    ict1 = 2*cxp(tdir(1))
                 end if
                 if ( alphad.eq.tdir(2) ) then
                    ict2 = 2*cxp(tdir(2))-1
                 else
                    ict2 = 2*cxp(tdir(2))
                 end if
                 btype = ibinfor(ict1,ict2)

                 if ( btype.eq.unsnt .or. btype.eq.snnsnt ) then
c                --- sigmant given
c                    remove pressure contribution to S^nt

                    if ( alphad.eq.tdir(1) ) then

                       if ( nsign.eq.-1 ) then

                          fact = gmat(ip,jp,kp,alphad,7)
                          gmat(ip,jp,kp,alphad,3) =
     +                                    gmat(ip,jp,kp,alphad,3) - fact
                          gmat(ip,jp,kp,alphad,4) =
     +                                    gmat(ip,jp,kp,alphad,4) - fact
                          gmat(ip,jp,kp,alphad,7) = 0d0
                          gmat(ip,jp,kp,alphad,8) = 0d0

                       else

                          fact = gmat(ip,jp,kp,alphad,9)
                          gmat(ip,jp,kp,alphad,3) =
     +                                    gmat(ip,jp,kp,alphad,3) - fact
                          gmat(ip,jp,kp,alphad,4) =
     +                                    gmat(ip,jp,kp,alphad,4) - fact
                          gmat(ip,jp,kp,alphad,9)  = 0d0
                          gmat(ip,jp,kp,alphad,10) = 0d0

                       end if

                    else

                       if ( nsign.eq.-1 ) then

                          fact = gmat(ip,jp,kp,alphad,1)
                          gmat(ip,jp,kp,alphad,3) =
     +                                    gmat(ip,jp,kp,alphad,3) - fact
                          gmat(ip,jp,kp,alphad,4) =
     +                                    gmat(ip,jp,kp,alphad,4) - fact
                          gmat(ip,jp,kp,alphad,1) = 0d0
                          gmat(ip,jp,kp,alphad,2) = 0d0

                       else

                          fact = gmat(ip,jp,kp,alphad,5)
                          gmat(ip,jp,kp,alphad,3) =
     +                                    gmat(ip,jp,kp,alphad,3) - fact
                          gmat(ip,jp,kp,alphad,4) =
     +                                    gmat(ip,jp,kp,alphad,4) - fact
                          gmat(ip,jp,kp,alphad,5) = 0d0
                          gmat(ip,jp,kp,alphad,6) = 0d0

                       end if

                    end if

                    if ( btype.eq.unsnt  ) then

c                   --- sigmant and un given
c                       Add pressure contribution to S^nn

                       if ( ktdir.eq.1 ) then
                          if( tdir(1).eq.1 ) then
                             pnttyp = 7
                          else if ( tdir(1).eq.2 ) then
                             pnttyp = 6
                          else
                             pnttyp = 4
                          end if
                       else
                          if( tdir(2).eq.1 ) then
                             pnttyp = 7
                          else if ( tdir(2).eq.2 ) then
                             pnttyp = 6
                          else
                             pnttyp = 4
                          end if
                       end if

                       do 720 i = 1, 3
                          gcov(i,i) =
     +                         cva(cxp(1),cxp(2),cxp(3),1,i,pnttyp)**2+
     +                         cva(cxp(1),cxp(2),cxp(3),2,i,pnttyp)**2+
     +                         cva(cxp(1),cxp(2),cxp(3),3,i,pnttyp)**2
                          do 710 j = 1, i-1
                             gcov(i,j) =
     +                         cva(cxp(1),cxp(2),cxp(3),1,i,pnttyp)*
     +                         cva(cxp(1),cxp(2),cxp(3),1,j,pnttyp)+
     +                         cva(cxp(1),cxp(2),cxp(3),2,i,pnttyp)*
     +                         cva(cxp(1),cxp(2),cxp(3),2,j,pnttyp)+
     +                         cva(cxp(1),cxp(2),cxp(3),3,i,pnttyp)*
     +                         cva(cxp(1),cxp(2),cxp(3),3,j,pnttyp)
                             gcov(j,i) = gcov(i,j)
710                       continue
720                    continue

                       if ( alphad.eq.tdir(1) ) then
                          betad = tdir(2)
                       else
                          betad = tdir(1)
                       end if
                       fact = (gcov(betad,betad)*
     +                        gcov(ndir,alphad)-
     +                        gcov(tdir(1),tdir(2))*
     +                        gcov(ndir,betad))/
     +                       (gcov(tdir(1),tdir(1))*
     +                        gcov(tdir(2),tdir(2))-
     +                        gcov(tdir(1),tdir(2))**2)

c                      --- contravariant base vectors

                       do alphal = 1, 3
                          do betal = 1, 3
                             valpha(1) = alphal
                             valpha(2) = mod(alphal  ,3) + 1
                             valpha(3) = mod(alphal+1,3) + 1
                             vbeta(1)  = betal
                             vbeta(2)  = mod(betal  ,3) + 1
                             vbeta(3)  = mod(betal+1,3) + 1
                      ctva(alphal,betal) =
     +              cva(cxp(1),cxp(2),cxp(3),vbeta(2),valpha(2),pnttyp)*
     +              cva(cxp(1),cxp(2),cxp(3),vbeta(3),valpha(3),pnttyp)-
     +              cva(cxp(1),cxp(2),cxp(3),vbeta(3),valpha(2),pnttyp)*
     +              cva(cxp(1),cxp(2),cxp(3),vbeta(2),valpha(3),pnttyp)
                          end do
                       end do

                       fact = - nsign *
     +                        fact * 25d-2 *
     +                        invjac(cxp(1),cxp(2),cxp(3),pnttyp) *
     +                        (ctva(ndir,1)*ctva(ndir,1) +
     +                        ctva(ndir,2)*ctva(ndir,2) +
     +                        ctva(ndir,3)*ctva(ndir,3))

                       gmat(ip,jp,kp,alphad,3) =
     +                        gmat(ip,jp,kp,alphad,3) +fact
                       gmat(ip,jp,kp,alphad,4) =
     +                        gmat(ip,jp,kp,alphad,4) +fact

                       if ( alphad.eq.tdir(1) ) then

                          if ( nsign.eq.-1 ) then

                             gmat(ip,jp,kp,alphad,7) =
     +                                    gmat(ip,jp,kp,alphad,7) + fact
                             gmat(ip,jp,kp,alphad,8) =
     +                                    gmat(ip,jp,kp,alphad,8) + fact
                          else
                             gmat(ip,jp,kp,alphad,9)  =
     +                                    gmat(ip,jp,kp,alphad,9) + fact
                             gmat(ip,jp,kp,alphad,10) =
     +                                   gmat(ip,jp,kp,alphad,10) + fact
                          end if

                       else
                          if ( nsign.eq.-1 ) then

                             gmat(ip,jp,kp,alphad,1) =
     +                                    gmat(ip,jp,kp,alphad,1) + fact
                             gmat(ip,jp,kp,alphad,2) =
     +                                    gmat(ip,jp,kp,alphad,2) + fact
                          else
                             gmat(ip,jp,kp,alphad,5) =
     +                                    gmat(ip,jp,kp,alphad,5) + fact
                             gmat(ip,jp,kp,alphad,6) =
     +                                    gmat(ip,jp,kp,alphad,6) + fact
                          end if

                       end if

                    end if

                 end if
500            continue
600         continue
700      continue

      end do

      end
