      subroutine isgmat3d ( ni, nj, nk, nvirtual, gmat, cva, invjac,
     +                      ibipoint, ibindex, ibinfor, mctg, mctinvg )
c ======================================================================
c
c
c        programmer     Kees Kassels
c        version 1.0    date 11-01-96
c
c   copyright (c) 1995-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c ********************************************************************
c
c                         DESCRIPTION
c
c    Compute gradient matrix in R^3
c    At this moment the discretization is restricted to standard ISNaS
c ********************************************************************
c
c                      KEYWORDS
c
c    grad_matrix
c    matrix
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer ni, nj, nk, nvirtual, ibipoint(*), ibindex(2,2,*),
     +        ibinfor(*), mctg(*), mctinvg(*)
      double precision gmat(*),
     +                 cva((ni+1+2*nvirtual)*(nj+1+2*nvirtual)*
     +                     (nk+1+2*nvirtual),3,3,8),
     +                 invjac((ni+1+2*nvirtual)*(nj+1+2*nvirtual)*
     +                        (nk+1+2*nvirtual),8)

c     cva       i    Array containing the covariant basis vectors
c                    in 2D we have
c                                                 l
c                    cva(i,j,l,alpha,p) contains a        in cell i,j in
c                                                 (alpha)       point type p
c                    in 3D it is cva(i,j,k,l,alpha,p) referring to cell i,j,k
c     gmat      o    Gradient matrix
c     ibindex   i    ibindex(1,1,side)
c                                    first point index where information
c                                    is available on this side in
c                                    direction tdir (2D) or tdir(1) (3D)
c                    ibindex(2,1,side) last point index where information
c                                    is available on this side in
c                                    direction tdir (2D) or tdir(1) (3D
c                    ibindex(1,2,side)
c                                    first point index where information
c                                    is available on this side in
c                                    direction tdir or tdir(2) (3D)
c                    ibindex(2,2,side)
c                                    last point index where information
c                                    is available on this side in
c     ibinfor   i    Third part of infbnd
c     ibipoint  i    ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     invjac    i    1/sqrt(g) in pnttyp
c     mctg      i    Integer matrix information
c     mctinvg   i    Matrix information
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c ********************************************************************
c
c                    COMMON BLOCKS
c
c ********************************************************************
c
c                    LOCAL PARAMETERS

      integer iface, ilow1, ihigh1, ilow2, ihigh2

c     iface     Face number
c     ihigh1    First upper index of an array
c     ihigh2    Second upper index of an array
c     ilow1     First lower index of an array
c     ilow2     Second lower index of an array
c
c ********************************************************************
c
c             SUBROUTINES CALLED
c
c     ISGMAT03  Compute internal part of gradient matrix in R^3 for
c               one direction
c     ISGMAT23  Correct the gradient matrix for boundary cells in 3D
c     ISGMF3D   Folding subroutine for the gradient matrix in 3D
c ********************************************************************
c
c                              I/O
c
c ********************************************************************
c
c                    ERROR MESSAGES
c
c ********************************************************************
c
c                    PSEUDO CODE
c
c    Fill gmat for all V1-unknowns
c    Fill gmat for all V2-unknowns
c    Correct gmat for boundary cells as function of the boundary conditions
c    Fold all virtual pressures inside the region
c ********************************************************************
c
c                    DATA STATEMENTS
c
c ======================================================================
c
c     --- Fill gmat for all V1-unknowns

      call isgmat03 ( ni, nj, nk, nvirtual, gmat, cva(1,1,1,7),
     +                invjac(1,7), 1 )

c     --- Fill gmat for all V2-unknowns

      call isgmat03 ( ni, nj, nk, nvirtual, gmat, cva(1,1,1,6),
     +                invjac(1,6), 2 )

c     --- Fill gmat for all V3-unknowns

      call isgmat03 ( ni, nj, nk, nvirtual, gmat, cva(1,1,1,4),
     +                invjac(1,4), 3 )


c     --- Correct gmat for boundary cells as function of the boundary conditions

      do iface = 1, 6
         ilow1  = ibindex(1,1,iface)
         ihigh1 = ibindex(2,1,iface)
         ilow2  = ibindex(1,2,iface)
         ihigh2 = ibindex(2,2,iface)
         call isgmat23 ( iface, ni, nj, nk, nvirtual, gmat, cva,
     +                   invjac, ilow1, ihigh1, ilow2, ihigh2,
     +                   ibinfor(ibipoint(iface)) )
      end do
c
c     --- Fold all virtual pressures inside the region
c
      call isgmf3d (  ni, nj, nk, mctg, mctinvg, ibindex, ibipoint,
     +                ibinfor, gmat )

      end
