      subroutine isgmf3d ( ni, nj, nk, mct, mctinv, ibindex, ibipoint,
     +                     ibinfor, matrix )
c=======================================================================
c
c
c        programmer     Ronald van Nooyen
c        version 1.1    date 02-02-96 Kees K. from routine ismcbcf.f
c        version 1.0    date 01-11-94
c
c
c   copyright (c) 1994  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c*********************************************************************
c
c                      DESCRIPTION
c
c   control orutine for the boundary condition folding process
c
c*********************************************************************
c
c                      KEYWORDS
c
c*********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
c
      integer maxdim, maxdx
      parameter ( maxdim = 3, maxdx = 2 )
c
      integer ibindex(1:2,1:2,1:6), ibinfor(*),
     +   ibipoint(1:6),
     +   ni, nj, nk,
     +   mctinv(*), mct(*)

      double precision matrix(*)
c
c     ibindex   i    ibindex(1,1,side)
c                                    first point index where information
c                                    is available on this side in
c                                    direction tdir (2D) or tdir(1) (3D)
c                    ibindex(2,1,side) last point index where information
c                                    is available on this side in
c                                    direction tdir (2D) or tdir(1) (3D
c                    ibindex(1,2,side)
c                                    first point index where information
c                                    is available on this side in
c                                    direction tdir or tdir(2) (3D)
c                    ibindex(2,2,side)
c                                    last point index where information
c                                    is available on this side in
c     ibinfor   i    Third part of infbnd
c     ibipoint  i    ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     matrix    i    reserved
c     mct       i    Integer matrix information
c     mctinv    i    Matrix information
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c
c*********************************************************************
c
c                      COMMON BLOCKS
c
c*********************************************************************
c
c                      LOCAL PARAMETERS
c
c
      integer iconct, idir, iface, ifcarea(1:2,1:maxdim),
     +        ifvarea(1:2,1:maxdim), iphase, jdim,
     +        matarea(1:2,1:maxdim),
     +        it1lo, it1hi, it2lo, it2hi,
     +        matdim, ndir, ntdir(1:maxdim), ntsig(1:maxdim),
     +        ndim, nconct

c     iconct    Loop variable for loop over all connections in the matrx
c     idir      General loop variable for 1...ndim loop
c     iface     Face number
c     ifcarea   Index bounds for cells on this face
c     ifvarea   Index bounds for vertices on this face
c     iphase    Phase in folding process
c     it1hi     Upper bound on first tangential index for destination
c     it1lo     Lower bound on first tangential index for destination
c     it2hi     Upper bound on second tangential index for destination
c     it2lo     Lower bound on second tangential index for destination
c     jdim      Loop variable used for dimension
c     matarea   Index bounds for the active area of the matrix
c     matdim    Number of different tpyes of integration cells for this matrix
c     maxdim    Highest physical space dimension for which ISNaS can work
c     maxdx     Maximum relative shift between cell indices
c     nconct    Number of connections in a row of the matrix
c     ndim      Dimension of space (2 or 3)
c     ndir      Direction of normal vector on this side
c     ntdir     Contains the ksi-alpha indices of the normal and
c               tangential directions
c               ntdir(1) refers to the normal direction
c               ntdir(2) refers to the first tangential direction
c               ntdir(3) refers to the second tangential direction
c     ntsig     Gives the direction of the normal or tangential vectors
c               with respect to the ksi co-ordinate axis
c               The normal is supposed to be the outward normal and
c               the tangential directions are taken into a counter
c               clockwise way
c               ntsig(1) refers to the normal direction
c               ntsig(2) refers to the first tangential direction
c               ntsig(3) refers to the second tangential direction
c
c*********************************************************************
c
c                      SUBROUTINES CALLED
c
c     ISBNBVI   Boundary information
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c
c*********************************************************************
c
c                      I/O
c
c*********************************************************************
c
c                      ERROR MESSAGES
c
c*********************************************************************
c
c                      PSEUDO CODE
c
c     Loop over connections
c        Loop over integration cell types
c
c=======================================================================
c
      ndim = 3
      nconct = 10
c
      call iseropen ( 'isgmf3d' )

c     --- Get number of integration cell types


c     --- Pressure gradient matrix
         matdim   = ndim

c     --- 3D: Three phases
c         phase one: fold inwards along each face
c         phase two: extra fold at sides to catch previously changed
c         elements
c         phase three: extra fold at corners to catch previously changed
c         elements

      do 10000 iphase=1,ndim+1

         do 11000 iface = 1, 2*ndim

c        --- Get the normal and tangential directions (ntdir,ntsig),
c            the grid cells adjacent to the boundary (ifcarea) and
c            the vertices on the boundary (ifvarea)

            call isbnbvi(ndim, ni, nj, nk, iface, ntdir, ntsig,
     +                   ifcarea, ifvarea)

            ndir = ntdir(1)

c           --- Loop over connections

            do 11100 iconct=1,nconct

c              --- Loop over integration cell type

               do 11110 idir=1,matdim

c               --- Initialization of variable and integration cell
c                   centre location.
c                   Initialization of active matrix area.

                  do 11111 jdim=1,maxdim
                     matarea(1,jdim) = ifcarea(1,jdim)
                     matarea(2,jdim) = ifcarea(2,jdim)
11111             continue

c                 --- adjust locations and area for special cases


c                 --- Pressure gradient matrix

                     matarea(2,idir) = ifvarea(2,idir)
                     matarea(1,ndir) = matarea(2,ndir)


c                 --- Call routine that does the work




                     it1lo = ibindex(1,1,iface)
                     it1hi = ibindex(2,1,iface)
                     it2lo = ibindex(1,2,iface)
                     it2hi = ibindex(2,2,iface)

c                 --- G or -RinvG matrix
c                    --- fold elements inwards
                     call isgmbf( ndim, ni, nj, nk,
     +                            nconct, maxdx,
     +                            it1lo, it1hi, it2lo, it2hi,
     +                            mct, mctinv, matrix,
     +                            ibinfor(ibipoint(iface)),
     +                            iface, iconct, idir, matarea, iphase )

11110          continue
11100       continue
11000    continue
10000 continue

      call iserclos ( 'isgmf3d' )
      end

