      subroutine isjctg3d ( invjac, jcta, jctg, ni, nj, nk, nvirtual )
c ======================================================================
c
c
c        programmer     Kees Kassels
c        version 1.0    date 20-04-95 (as isjctg2d extracted from isjctg)
c
c
c   copyright (c) 1995-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c ********************************************************************
c
c                      DESCRIPTION
c
c    Compute sqrt(g) x contra variant metric tensors in integration point
c    three-dimensional case
c ********************************************************************
c
c                      KEYWORDS
c
c    base_vectors
c    interpolation
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer ni, nj, nk, nvirtual
      double precision jctg(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual,6)
      double precision jcta(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual,3,3)
      double precision invjac(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,
     +                        1-nvirtual:nk+1+nvirtual)

c     invjac    i    1/sqrt(g) in pnttyp
c     jcta      i    Jacobian times contravariant basis vectors
c                    in point pnttyp
c                    (stored as vector index encoding given by istwo)
c     jctg      o    Jacobian times symmetric contravariant metric
c                    tensor in point given by pnttyp
c                    (stored as vector index encoding given by istwo)
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c ********************************************************************
c
c                      COMMON BLOCKS
c
c ********************************************************************
c
c                      LOCAL PARAMETERS

      integer i, j, k, istwo(3,3), alphad, betad

      data istwo / 1, 2, 3,
     +             2, 4, 5,
     +             3, 5, 6 /

c     alphad    Direction index of integration.
c               Hence if a V1-cell is used, alphad = 1 and so on
c     betad     Direction index, usually the direction perpendicular to the
c               integration direction of the boundary integral.
c               For example betad in local point (1,0) is 1 and in (0,1): 2
c     i         General loop variable
c     istwo     Translation table to store and retrieve the
c               elements of a symmetric ndim-dimensional
c               rank 2 tensor from a vector with length
c               (ndim*(ndim+1))/2
c     j         General loop variable
c     k         General loop variable
c ********************************************************************
c
c                      SUBROUTINES CALLED
c
c ********************************************************************
c
c                      I/O
c
c ********************************************************************
c
c                      ERROR MESSAGES
c
c ********************************************************************
c
c                      PSEUDO CODE
c
c     sqrt(g) x contravariant geometric tensor in 2D is given by:
c
c              alpha beta    (alpha)    (beta)
c     sqrt(g) g           = a        . a
c
c                                  (.)
c     Since jcta contains sqrt(g) a    the product must be divided by sqrt(g)
c ======================================================================
c
c
         do 70 alphad=1,3
            do 60 betad=alphad,3
               do 50 k = 1-nvirtual, nk+1+nvirtual
                 do 40 j = 1-nvirtual, nj+1+nvirtual
                     do 30 i = 1-nvirtual, ni+1+nvirtual
                        jctg(i,j,k,istwo(betad,alphad)) = (
     +                       jcta(i,j,k,1,betad)*
     +                       jcta(i,j,k,1,alphad)+
     +                       jcta(i,j,k,2,betad)*
     +                       jcta(i,j,k,2,alphad)+
     +                       jcta(i,j,k,3,betad)*
     +                       jcta(i,j,k,3,alphad)
     +                       )*invjac(i,j,k)
30                continue
40           continue
50          continue
60       continue
70    continue


      end
