      subroutine ismcvuwb ( ni, nj, nvirtual, vecup1, vecup2, vsum,
     +                       locdx, dcell, iarea, adir, cva, invjac,
     +                       iconserv )
c ======================================================================
c
c        programmer     Kees Kassels
c
c        version 1.1    date 09-12-1998 iconser and invjac added
c        version 1.0    date 10-08-1998
c
c   copyright (c) 1995-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     determines the coefficients of the velocity components
c     in a flux cell for the convection term in R2, first order
c     upwind, wesbeek discretization
c **********************************************************************
c
c                       KEYWORDS
c
c    convection
c    matrix
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, locdx(2), dcell(2), iarea(2,2),
     +         iconserv
      double precision vecup1(1-nvirtual:ni+1+nvirtual,
     +                         1-nvirtual:nj+1+nvirtual),
     +                  vecup2(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,2)
      double precision vsum(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,2),
     +                  adir(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,2),
     +                  cva(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,2,2),
     +                  invjac(1-nvirtual:ni+1+nvirtual,
     +                         1-nvirtual:nj+1+nvirtual)

c     adir      i    Contravariant base vector in integration cell center
c     cva       i    Array containing the covariant basis vectors
c                    in 2D we have
c                                                 l
c                    cva(i,j,l,alpha,p) contains a        in cell i,j in
c                                                 (alpha)       point type p
c                    in 3D it is cva(i,j,k,l,alpha,p) referring to cell i,j,k
c     dcell     i    reserved
c     iarea     i    Area of arrays
c     iconserv  i    If 1 the conservative scheme is used, otherwise (0) the
c                    non-conservative one
c     invjac    i    1/sqrt(g) in pnttyp
c     locdx     i    Position of the integration point with respect to
c                    the centre of the integration cell
c                    The cell sizes are assumed to be 2 times 2, hence the
c                    distance of a midside point to a cell centre is always 1
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     vecup1    i    Upwind help vector 1, rho*V/sqrt(g) in cell centre
c     vecup2    i    Upwind help vector 2, (1+sign V)/2 and (1-sign V)/2 in
c                    integration point
c     vsum      i    Flux contributions to the matrix
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, ip, lvec, j, jp,
     +        sign, ish(2), jsh(2)

c     i         General loop variable
c     ip        Local counting variable i-direction
c     ish       Shift in i-direction
c     j         General loop variable
c     jp        Local counting variable j-direction
c     jsh       Shift in j-direction
c     lvec      Vector number
c     sign      sign for multiplication (1 or -1)
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c       Relevant formula
c
c
c   In case of the Wesbeek discretization we have the following expressions
c   for the convective terms (alpha cell):
c
c     (alpha)           1  |i+1/2,j          2  |i,j+1/2
c    a  (i,j)  . ( rho V u |         +  rho V u |         )
c                          |i-1/2,j             |i,j-1/2
c
c                   1           2
c   with u = (a    V   +  a    V ) / sqrt(g)
c              (1)         (2)
c
c
c   In the Picard linearization we use V at the old level
c
c   So the contributions to the matrix become:
c
c                                                  (alpha)      _1
c   Picard: locdx = (+/-1, 0):   V1 term: +/-(rho a (0,0)  . a  V ) / sqrt(g)
c                                                             (1)
c
c                                                  (alpha)      _1
c                                V2 term: +/-(rho a (0,0)  . a  V ) / sqrt(g)
c                                                             (2)
c
c                                                  (alpha)      _2
c           locdx = (0, +/-1):   V1 term: +/-(rho a (0,0)  . a  V ) / sqrt(g)
c                                                             (1)
c
c                                                  (alpha)      _2
c                                V2 term: +/-(rho a (0,0)  . a  V ) / sqrt(g)
c                                                             (2)
c
c   with first order conservative upwind  rho V / sqrt(g) is computed
c   in an upwind point in stead of integration point
c
c        (alpha)            1  |i+1/2,j      (alpha)
c        a  (i,j)  . ( rho V u |          =  a  (i,j)  .
c                              |
c
c                -1                  1                 1       2
c        (1+sign V        )/2 { rho V/sqrt(g)    (a   V + a   V )     }
c                 i+1/2,j                   (0,0)  (1)     (2)   (0,0)
c
c
c                -1                  1                 1       2
c   +    (1-sign V        )/2 { rho V/sqrt(g)    (a   V + a   V )     }
c                 i+1/2,j                   (1,0)  (1)     (2)   (1,0)
c
c
c
c
c                -1                               -1
c        (1+sign V        )/2    and      (1-sign V        )/2   are stored
c                 i+1/2,j                          i+1/2,j
c
c
c        in help vector vecup2.
c
c             1
c        rho V/sqrt(g)        is stored in help vector vecup1.
c                    (1,0)
c
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- if ( locdx(1).ne.0 .or. locdx(2).ne.0 ) then

      if ( locdx(1).ne.0 .or. locdx(2).ne.0) then

c     --- Wesbeek discretization
c         The centroid has no contribution at all

c         --- shifts for both upwind points

         ish(1) = min(locdx(1),0)
         ish(2) = max(locdx(1),0)
         jsh(1) = min(locdx(2),0)
         jsh(2) = max(locdx(2),0)
         sign = locdx(1) + locdx(2)

         do jp = iarea(1,2), iarea(2,2)
            do ip = iarea(1,1), iarea(2,1)

c           --- for both upwind points do

               lvec = 0
               do i = ish(1), ish(2)
                  do j = jsh(1), jsh(2)

c                 --- for each upwind point take a different
c                     vector vecup2, indicated by lvec
c                     lvec = 1  --- > (1 + sign V )/2
c                     lvec = 2  --- > (1 - sign V )/2

                     lvec = lvec+1

                     if ( iconserv.eq.0 ) then

c                    --- adir       in cell center.
c                        cva, invjac
c                                   in upwind point, i.e.
c                                   also in cell center but shifted.
c                        vecup1, vecup2
c                                   in integration point.

                        vsum(ip,jp,1) = vsum(ip,jp,1) + sign *
     +                      (adir(ip,jp,1)*cva(ip+i,jp+j,1,1)+
     +                       adir(ip,jp,2)*cva(ip+i,jp+j,2,1))*
     +                       vecup1(ip+dcell(1),jp+dcell(2))*
     +                       vecup2(ip+dcell(1),jp+dcell(2),lvec)*
     +                       invjac(ip+i,jp+j)

                        vsum(ip,jp,2) = vsum(ip,jp,2) + sign *
     +                      (adir(ip,jp,1)*cva(ip+i,jp+j,1,2)+
     +                       adir(ip,jp,2)*cva(ip+i,jp+j,2,2))*
     +                       vecup1(ip+dcell(1),jp+dcell(2))*
     +                       vecup2(ip+dcell(1),jp+dcell(2),lvec)*
     +                       invjac(ip+i,jp+j)

                     else if ( iconserv.eq.1 ) then

c                    --- conservative
c                        adir       in cell center.
c                        cva, invjac and  vecup1
c                                   in upwind point, i.e.
c                                   also in cell center but shifted.
c                        vecup2     in integration point.

                        vsum(ip,jp,1) = vsum(ip,jp,1) + sign *
     +                      (adir(ip,jp,1)*cva(ip+i,jp+j,1,1)+
     +                       adir(ip,jp,2)*cva(ip+i,jp+j,2,1))*
     +                       vecup1(ip+i,jp+j)*
     +                       vecup2(ip+dcell(1),jp+dcell(2),lvec)*
     +                       invjac(ip+i,jp+j)

                        vsum(ip,jp,2) = vsum(ip,jp,2) + sign *
     +                      (adir(ip,jp,1)*cva(ip+i,jp+j,1,2)+
     +                       adir(ip,jp,2)*cva(ip+i,jp+j,2,2))*
     +                       vecup1(ip+i,jp+j)*
     +                       vecup2(ip+dcell(1),jp+dcell(2),lvec)*
     +                       invjac(ip+i,jp+j)

                     end if

                   end do
                end do

           end do
         end do

      end if

      end
