      subroutine ismnw2d ( rhside, solut, coefs, chsym,
     +                     vsum, invjac, coor, mct, ibipoint,
     +                     ibinfor, veloc, rho, eddy, rhsinc,
     +                     iindsc, rindsc, pnttyps, upwvel, iarea,
     +                     ni, nj, iconserv, iupwind, npointsv,
     +                     iconv, iprhside, nvirtual, pnttyp, alphad,
     +                     nintpol, dij, mcom, ndegfd)
c ======================================================================
c
c        programmer     Duncan
c        version 1.2    date 27-08-1999 Duncan: move invjac to distribution
c                                               subroutines
c        version 1.1    date 12-04-1999 Duncan: removed bug call to daxpy
c        version 1.0    date 31-3-1999
c
c   copyright (c) 1995-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Build momentum right-hand side contribution for newton
c    linearisation with compressible flow in R2.
c    subroutine ISMNW2D
c **********************************************************************
c
c                       KEYWORDS
c
c    2d
c    momentum_equation
c    right_hand_side
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer npointsv, ni, nj, iconserv, iupwind, iconv, nvirtual,
     +        mct(2,2,-2:2,-2:2,-2:2), ibipoint(*), ibinfor(*),
     +        iprhside, pnttyps(*),iindsc(*), pnttyp, alphad,
     +        nintpol(2,5), dij(2,4,2,5), iarea(2,*), ndegfd
      double precision rhside(*), coor(*), upwvel(*),
     +                 solut(npointsv,*), coefs(npointsv,*),
     +                 veloc(*), rindsc(*), rho(*),
     +                 vsum(npointsv,*),  invjac(npointsv,*),
     +                 eddy(*), rhsinc(*), chsym(*)
     +

c     chsym     i    Christoffel symbols of the second kind in the
c                    point given by pnttyp
c                    (symmetric contravariant part stored as vector
c                    with index encoding given by istwo)
c     coefs     i    reserved
c     coor      i    Array containing the coordinates for one block
c                    In 2d coor(i,j,.) contains the x resp y-coordinate
c                    of point i,j
c                    in 3d: coor(i,j,k,.) corresponds to (i,j,k)
c     eddy      i    reserved
c     ibinfor   i    Third part of infbnd
c     ibipoint  i    ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     iconserv  i    If 1 the conservative scheme is used, otherwise (0) the
c                    non-conservative one
c     iconv     i    If 1 the timestep is iterated till converged, otherwise (0)
c                    apply only one iterative step
c     iindsc    i    reserved
c     invjac    i    1/sqrt(g) in pnttyp
c     iprhside  i    Starting address of array rhside in buffer
c     iupwind   i    Type of upwinding
c     mct       i    Integer matrix information
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     npointsv  i    Number of points in the mesh with virtual points
c     pnttyps   i    Contains the value of pnttype for all 5 integration points
c     rho       i    Estimate of the asymptotic convergence rate
c     rhside    i    reserved
c     rhsinc    i    Increment in momentum rhs due to newton linearisation
c     rindsc    i    reserved
c     solut     i    reserved
c     upwvel    i    Convection velocity at integration point.
c     veloc     i    Array for the old velocities (used in convection
c                    term calculation)
c     vsum      i    Flux contributions to the matrix
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer vartyp,
     +        iptemp, i, idir, mcom,
     +        acttyp, ichoisdis
      logical reuse

c     acttyp    Type of action to be taken
c               0      : interpolation for point value
c               1..ndim: finite difference for partial derivative
c                        in the direction given by value of acttyp
c     alphad    Direction index of integration.
c               Hence if a V1-cell is used, alphad = 1 and so on
c     dij       Array containing the distances of the interpolation
c               points to the cell centra in global co-ordinates
c               dij(alpha,i,j) denotes the distance of the i-th
c               interpolation point, component j and with respect to
c               the V-alpha unknown (distribution)
c     i         General loop variable
c     iarea     Area of arrays
c     ichoisdis Choice parameter with respect to the convection scheme
c               for discretization of convective term of transport
c               equation(s) including turbulence.
c               Meaning of ichoisdis:
c                 ichoisdis = 0: 2nd order central scheme
c                 ichoisdis = 1: 1st order standard upwind scheme
c                 ichoisdis = 2: Hybrid central/upwind scheme
c                 ichoisdis = 3: Linear kappa-scheme
c                 ichoisdis = 4: Sweby Phi-limiter
c                 ichoisdis = 5: R-kappa limiter
c                 ichoisdis = 6: USR-kappa limiter
c                 ichoisdis = 7: PL-kappa limiter
c                 ichoisdis = 8: USPL-kappa limiter
c                 ichoisdis = 9: Symmetric PL-kappa limiter
c                 ichoisdis = 10: MPL2-kappa limiter
c                 ichoisdis = 11: Symmetric PL-kappa limiter
c     idir      General loop variable for 1...ndim loop
c     iptemp    Starting address of temporary array in IBUFFR
c     mcom      If zero the flow is incompressible else (1) compressible
c     ndegfd    Number of degrees of freedom
c     nintpol   Number of interpolation points in the distribution
c     nvirtual  Number of rows of virtual cells
c     pnttyp    reserved
c     reuse     If true the computed coefficients may be reused again
c     vartyp    Variable to be interpolated/extrapolated
c               vartyp: 1 for V1, 2 for V2 and 3 for pressure,
c               0 for vertex (for ndim=2)
c               or
c               vartyp: 1 for V1, 2 for V2 3, for V3 and
c               4 for pressure, 0 for vertex  (for ndim=3)
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     DAXPY     Computes a vector + a constant * other vector. This
c               is a BLAS routine.
c     ISBMOMCC2 Fill normal half cells for convection in R2
c     ISBUPWVEL Update the multiplication vector which indicates from which
c               side upwind must be taken for half cells where we have an
c               outflow boundary condition
c     ISEXDIC2  Distribution of convective terms, non conservative
c     ISEXDICC2 Distribution of convective terms, conservative central
c     ISEXDICU2 Distribution of convective terms, conservative upwind
c     ISINTCF2  Compute interpolation of a coefficient in an integration point
c     ISMOMCCV2 Fill convective terms for one integration point in R2
c     ISSETCON  Make array equal to a constant
c     ISUPWVEL2 Compute sign vectors corresponding to velocity for upwind
c     ISURHSC2D Update right-hand side vector with higher order upwind
c               scheme (2D)
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c       Clear array rhsinc
c       For all integration points do
c          Fill coefficients for convective terms
c          Fill multiplication factors for convective terms
c          distribute convective terms to rhside
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
            do i = 0, 2

c           --- First clear array vsum and rhsinc

               call issetcon ( vsum, 2*npointsv, 0d0 )
               call issetcon ( rhsinc, (ni+1)*(nj+1)*2, 0d0 )

               reuse = i.eq.0 .or. iconserv.eq.1 .and. i.eq.1 .or.
     +                 iupwind.gt.0 .and. iconserv.eq.1

               if ( .not. reuse ) pnttyp = pnttyps(2*i+1)

c              --- Fill some coefficients

               if ( .not. reuse ) then

c              --- Fill rho except for first integration point
c                  In the case of a central conservative scheme we use also
c                  Centroid points for the second integration point

                  vartyp = 3
                  acttyp = 0
                  call isintcf2 ( coefs(1,1), rho, ni, nj,
     +                            nvirtual, vartyp, acttyp, pnttyp, 0,
     +                            coor )
               end if

c              --- Fill velocity componentwise

               if ( iconserv.eq.0 .or. .not. reuse .or. i.eq.0 ) then

c              --- The velocity is required in all integration points,
c                  but in a central conservative scheme we use also
c                  Centroid points for the second integration point

c                  use second timelevel of velocity for iterative pressure
c                  correction

                  acttyp = 0

                    if ( iconv.eq.0) then

                      do idir = 1, 2
                        vartyp = idir
                        iptemp = (idir-1)*npointsv

                        call isintcf2 ( solut(1,idir),
     +                                  veloc(1+iptemp), ni,
     +                                  nj, nvirtual, vartyp, acttyp,
     +                                  pnttyp, 0, coor )
                      end do

                    else if( iconv.eq.1) then

                      do idir = 1, 2
                        vartyp = idir
                        iptemp = (idir-1)*npointsv
                        call isintcf2 ( solut(1,idir+ndegfd),
     +                                  veloc(1+iptemp), ni,
     +                                  nj, nvirtual, vartyp, acttyp,
     +                                  pnttyp, 0, coor )

                      end do

                    end if
               end if

c              --- Next fill vsum with contribution of convective terms to
c                  integration points

               call ismomccv2 ( ni, nj, nvirtual, rho,
     +                          chsym, veloc, vsum, alphad, i,
     +                          iindsc, mcom )

c              --- Correct contribution of convective terms for half cells

               call isbmomcc2 ( ni, nj, nvirtual, veloc, vsum,
     +                          alphad, rho,
     +                          i, ibinfor, ibipoint,
     +                          iindsc, mcom )

               if ( iconserv.eq.0 ) then

c              --- Non-conservative scheme

c                  --- Distribute computed values over the positions
c                     corresponding to the unknowns

                  call isexdic2( ni, nj, nvirtual, rhsinc,
     +                            vsum, alphad, iarea,
     +                            iindsc, nintpol, dij,
     +                            mct, i, ndegfd, eddy(2*npointsv+1),
     +                            invjac(1,pnttyp))

               else

c              --- Conservative scheme

c                  --- Distribute computed values over the positions
c                     corresponding to the unknowns

                  if ( iupwind.eq.0 ) then

c                 --- central scheme

                     call isexdicc2( ni, nj, nvirtual, rhsinc(
     +                                                   iprhside),
     +                             vsum, alphad, iarea,
     +                             iindsc, nintpol, dij,
     +                             mct, i, ndegfd,eddy(2*npointsv+1),
     +                             invjac(1,pnttyp))
                  else

c                 --- upwind scheme
c                     First store terms like (1-sign(v^alpha)) in array
c                     work from position ipprems

                     if ( i.ne.0 ) then

                     call isupwvel2 ( ni, nj, nvirtual, solut,
     +                                upwvel, alphad, i )

c                    --- Correct for half cells

                        call isbupwvel ( ni, nj, nvirtual, alphad, i,
     +                                   ibinfor, ibipoint,
     +                                    upwvel)
                     end if
                     call isexdicu2( ni, nj, nvirtual, rhsinc(
     +                                                   iprhside),
     +                              vsum, alphad, iarea,
     +                              iindsc, nintpol, dij,
     +                              mct, i, ndegfd,
     +                              upwvel, eddy(2*npointsv+1),
     +                              invjac(1,pnttyp))

                  end if

               end if

               ichoisdis = iindsc(5)

               if ( ichoisdis.gt.2 .and. i.gt.0 ) then

c              --- Higher order upwind scheme

                  if (mcom.eq.0) then

                     call isurhsc2d ( alphad, ni, nj, nvirtual,
     +                                rhsinc(iprhside),
     +                                vsum,
     +                                veloc, solut(1,alphad),
     +                                iarea, i,
     +                                ichoisdis, rindsc(1),
     +                                rindsc(2),
     +                                rindsc(3),
     +                                rindsc(4),
     +                                invjac(1,pnttyp) )
                  else

                     call isurhsc2d ( alphad, ni, nj, nvirtual,
     +                                rhsinc(iprhside),
     +                                vsum,
     +                                veloc,
     +                                eddy((1+alphad)*npointsv+1),
     +                                iarea, i,
     +                                ichoisdis, rindsc(1),
     +                                rindsc(2),
     +                                rindsc(3),
     +                                rindsc(4),
     +                                invjac(1,pnttyp) )

                  end if

               end if

c              --- Add contributions to rhside vector

            call daxpy((ni+1)*(nj+1),-1d0, rhsinc(iprhside),
     +                                     1, rhside(iprhside),1)

            end do

      end
