      subroutine ismomcv3 (ni, nj, nk, nvirtual, densrho, chsym, invjac,
     +                     veloc, vsum, alphad,  locdx, iindsc, iparea )
c======================================================================
c
c
c        programmer     Ronald van Nooyen
c        version 2.1    date 22-01-95 (Kees K: renamed)
c        version 2.0    date 22-01-95 (Guus: New parameter list)
c        version 1.0    date 18-02-93
c
c   copyright (c) 1993-1994  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c ********************************************************************
c
c                         DESCRIPTION
c
c     determines the coefficients of the velocity components in
c     a given integration point within a flux cell for the convection term in R3
c**********************************************************************
c
c                      KEYWORDS
c
c    convection
c    matrix
c    right_hand_side
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer ni, nj, nk, nvirtual, alphad, iparea(2,3),
     +        locdx(3), iindsc(*)
      double precision densrho(1-nvirtual:ni+1+nvirtual,
     +                         1-nvirtual:nj+1+nvirtual,
     +                         1-nvirtual:nk+1+nvirtual),
     +                 veloc(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1-nvirtual:nk+1+nvirtual,3),
     +                 invjac(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,
     +                        1-nvirtual:nk+1+nvirtual)
      double precision vsum(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual,3),
     +                 chsym(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1-nvirtual:nk+1+nvirtual,3,6)

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     chsym     i    Christoffel symbols of the second kind in the
c                    point given by pnttyp
c                    (symmetric contravariant part stored as vector
c                    with index encoding given by istwo)
c     densrho   i    Array containing the density rho. The size and structure is
c                    the same as for the scalar unknowns
c     iindsc    i    reserved
c     invjac    i    1/sqrt(g) in pnttyp
c     iparea    i    Area of integration points where a contribution to
c                    the matrix is to be calculated
c     locdx     i    Position of the integration point with respect to
c                    the centre of the integration cell
c                    The cell sizes are assumed to be 2 times 2, hence the
c                    distance of a midside point to a cell centre is always 1
c     mlin      i    Type of linearization (1=newton, 2=picard)
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     veloc     i    Array for the old velocities (used in convection
c                    term calculation)
c     vsum     i/o   Flux contributions to the matrix
c**********************************************************************
c
c                 COMMON BLOCKS
c
c**********************************************************************
c
c                 LOCAL PARAMETERS
c
      integer ip, jp, kp, betad, i, mlin, istwo(3,3)
      double precision sign, rmult

      data istwo / 1, 2, 3,
     +             2, 4, 5,
     +             3, 5, 6 /

c     betad     Direction index, usually the direction perpendicular to the
c               integration direction of the boundary integral.
c               For example betad in local point (1,0) is 1 and in (0,1): 2
c     i         General loop variable
c     ip        Local counting variable i-direction
c     istwo     Translation table to store and retrieve the
c               elements of a symmetric ndim-dimensional
c               rank 2 tensor from a vector with length
c               (ndim*(ndim+1))/2
c     jp        Local counting variable j-direction
c     kp        Local counting variable k-direction
c     rmult     Multiplication factor due to linearisation and dependent
c               on linearisation type,  2d0 for Newton, 1d0 for Picard
c     sign      sign for multiplication (1 or -1)
c**********************************************************************
c
c                 I/O
c
c**********************************************************************
c
c                 SUBROUTINES CALLED
c
c**********************************************************************
c
c                 ERROR MESSAGES
c
c
c**********************************************************************
c
c                 PSEUDO CODE
c
c
c       Relevant formula
c
c
c    The convective terms in case of the classical discretization are given by:
c
c
c    rho      a  1 |i+1/2,j,k     rho      a  2 |i,j+1/2,k
c   -----    V  V  |         +   -----    V  V  |          +
c   sqrt(g)        |i-1/2,j,k    sqrt(g)        |i,j-1/2,k
c
c
c     rho      a  3 |i,j,k+1/2      rho    |  a  |   c  b |
c    -----    V  V  |          +   -----   |     |  V  V  |
c    sqrt(g)        |i,j,k-1/2     sqrt(g) | b c |        |i,j,k
c
c
c   There are two linearization methods available Newton and Picard.
c
c   Application of Newton linearization:
c
c   a  b    _a  b      a _b     _a _b
c  V  V  =  V  V   +  V  V   -  V  V
c                                                 1
c   results in the following contribution of the V  component to the matrix:
c
c
c    rho      1 |i+1/2,j,k     rho      2 |i,j+1/2,k      rho      1 |i+1/2,j,k
c   -----    V  |           + -----    V  |            + -----    V  |         +
c   sqrt(g)     |i-1/2,j,k    sqrt(g)     |i,j-1/2,k     sqrt(g)     |i-1/2,j,k
c
c
c    rho      3 |i,j,k+1/2      2 rho    |  a  |   b |
c   -----    V  |           +    -----   |     |  V  |
c   sqrt(g)     |i,j,k-1/2       sqrt(g) | 1 b |     |i,j,k
c
c                                              2      3
c  and of course a similar expression for the V  and V components
c
c  Application of the Picard linearization (in the b-th momentum equation) :
c
c   a  b     _a  b
c  V  V  =   V  V
c                                               1
c  results in the following contribution of the V  component to the matrix:
c
c
c    rho      1 |i+1/2,j,k     rho     2 |i,j+1/2,k       rho     3 |i,j,k+1/2
c   -----    V  |           + -----   V  |           +   -----   V  |         +
c   sqrt(g)     |i-1/2,j,k    sqrt(g)    |i,j-1/2,k     sqrt(g)     |i,j,k-1/2
c
c
c    rho    |  a  |   b |
c   -----   |     |  V  |
c   sqrt(g) | 1 b |     |i,j,k
c
c  and of course a similar expression for the V  and V components
c
c
c**********************************************************************
c
c                    DATA STATEMENTS
c
c ======================================================================
c
      mlin = iindsc(8)

      if ( mlin.eq.1 ) then

c     ---mlin=1, Newton lin. : multiplier is 2d0.
c        mlin=2, Picard lin. : multiplier is 1d0.

         rmult = 2d0
      else
         rmult = 1d0
      end if


      if ( abs(locdx(1))+abs(locdx(2))+abs(locdx(3)).eq.1 ) then

c     --- centres of integration cell sides
c         betad in (convection term), betad

         if ( locdx(1).ne.0 ) then
            betad = 1
            sign = locdx(1)
         else if ( locdx(2).ne.0 ) then
            betad = 2
            sign = locdx(2)
         else
            betad = 3
            sign = locdx(3)
         end if
         if ( alphad.eq.betad ) then

c        --- alphad = betad, hence (1,1), (2,2) or (3,3) element in block
c            matrix


            do 130 kp = iparea(1,3),iparea(2,3)
               do 120 jp = iparea(1,2),iparea(2,2)
                  do 110 ip = iparea(1,1),iparea(2,1)
                     vsum(ip,jp,kp,alphad) = vsum(ip,jp,kp,alphad)+
     +                                       sign*rmult*
     +                                       invjac(ip,jp,kp)*
     +                                       densrho(ip,jp,kp)*
     +                                       veloc(ip,jp,kp,alphad)
110               continue
120            continue
130         continue

         else

c        --- alphad # betad

            do 230 kp = iparea(1,3),iparea(2,3)
               do 220 jp = iparea(1,2),iparea(2,2)
                  do 210 ip = iparea(1,1),iparea(2,1)
                     vsum(ip,jp,kp,alphad) = vsum(ip,jp,kp,alphad)+
     +                                       sign*invjac(ip,jp,kp)*
     +                                       densrho(ip,jp,kp)*
     +                                       veloc(ip,jp,kp,betad)
210               continue
220            continue
230         continue


            if ( mlin.eq.1 ) then

c           --- mlin=1, Newton linearisation. No contribution for
c                           Picard.


               do 260 kp = iparea(1,3),iparea(2,3)
                  do 250 jp = iparea(1,2),iparea(2,2)
                     do 240 ip = iparea(1,1),iparea(2,1)
                        vsum(ip,jp,kp,betad) = vsum(ip,jp,kp,betad)+
     +                                         sign*invjac(ip,jp,kp)*
     +                                         densrho(ip,jp,kp)*
     +                                         veloc(ip,jp,kp,alphad)
240                  continue
250               continue
260            continue

             end if

         end if

      else if ( locdx(1).eq.0 .and. locdx(2).eq.0 ) then

c     --- Take care of christoffel symbol term
c         from (convection term),betad derivative in cell centre


         do 340 i = 1, 3
            do 330 kp=iparea(1,3),iparea(2,3)
               do 320 jp=iparea(1,2),iparea(2,2)
                  do 310 ip=iparea(1,1),iparea(2,1)
                     vsum(ip,jp,kp,i) = invjac(ip,jp,kp)*
     +                   densrho(ip,jp,kp)*rmult*
     +                   (chsym(ip,jp,kp,alphad,istwo(i,1))*
     +                   veloc(ip,jp,kp,1)+
     +                   chsym(ip,jp,kp,alphad,istwo(i,2))*
     +                   veloc(ip,jp,kp,2)+
     +                   chsym(ip,jp,kp,alphad,istwo(i,3))*
     +                   veloc(ip,jp,kp,3))
310               continue
320            continue
330         continue
340      continue

      end if

      end

