      subroutine ismomf21 ( ni, nj, nvirtual, mct, matrix, rhside,
     +                      ibinfor, ibipoint, bcgeo, bndcon )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.5    date 03-12-1998 Remove iprec
c        version 1.4    date 19-11-1997 Kees K.: same action for periodic
c                                                as for multiblock
c        version 1.3    date 07-02-1997 Kees K.: change for corners
c        version 1.2    date 07-02-1997 Kees K.: again multiblock
c        version 1.1    date 18-12-1996 Kees K.: fixed for multiblock
c        version 1.0    date 17-06-1996
c
c   copyright (c) 1996-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    All parts of the matrix outside the region are folded to internal
c    domain for the momentum matrix in R2
c    The folding process depends on the types of boundary conditions
c    This subroutine is meant for a 23-point stencil
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, ibipoint(*), ibinfor(*),
     +        mct(2,2,-2:2,-2:2,-2:2)
      double precision bcgeo(8,*), rhside(ni+1,nj+1,2),
     +                 matrix(ni+1,nj+1,2,*), bndcon(2,*)

c     bcgeo     i    reserved
c     bndcon    i    reserved
c     ibinfor   i    Third part of infbnd
c     ibipoint  i    ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     matrix   i/o   reserved
c     mct       i    Integer matrix information
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     rhside   i/o   reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer unut, uxuy, unsnt, snnut, walllaw, parabolic
      parameter( unut = 4, uxuy = 10, unsnt = 5, snnut = 6,
     +           walllaw = 9, parabolic= 12 )
      logical ungiven, skip, utgiven
      integer btype, ipibinf, ipoint, alphad,  tsign,
     +        iconcto, iconctc, iconcti, iconctr, iconctl, k,
     +        ip, jp, xp(2), ndir, icbtype,
     +        tdir, nsign, iside, istwo(2,2), iparea(2,2)
      double precision fact, gtt, gttdivsg, gntdivsg, un, ut, gnn, gnt

c     alphad    Direction index of integration.
c               Hence if a V1-cell is used, alphad = 1 and so on
c     btype     Type of boundary condition
c     fact      Multiplication factor
c     gnn       Contravariant metric tensor
c     gnt       Contravariant metric tensor in nt direction
c     gntdivsg  Contravariant metric tensor in nt direction divided by sqrt(g)
c     gtt       Contravariant metric tensor in tt direction
c     gttdivsg  Contravariant metric tensor in tt direction divided by sqrt(g)
c     icbtype   Type of boundary condition with respect to centre of integration
c               cell
c     iconctc   Connection in matrix for central point
c     iconcti   Connection in matrix for internal point
c     iconctl   Connection in matrix for "left" point
c     iconcto   Connection in matrix for outside point
c     iconctr   Connection in matrix for "right" point
c     ip        Local counting variable i-direction
c     iparea    Area of integration points where a contribution to
c               the matrix is to be calculated
c     ipibinf   Starting address of array infor for present side minus one
c     ipoint    Address in array IBUFFR
c     iside     Sequence number of side
c     istwo     Translation table to store and retrieve the
c               elements of a symmetric ndim-dimensional
c               rank 2 tensor from a vector with length
c               (ndim*(ndim+1))/2
c     jp        Local counting variable j-direction
c     k         General loop variable
c     ndir      Direction of normal vector on this side
c     nsign     Inner product of outward normal in computational domain
c               with the base vector perpendicular to the face
c     parabolic Parabolic boundary conditions
c     skip      If true boundary condition must be skipped
c     snnut     Sigma_nn and u_t prescribed
c     tdir      Direction of tangent vector on this side
c     tsign     Inner product of tangential vector defined counter
c               clockwise with respect to the outward normal and the
c               base vector along the side
c     un        Normal velocity (physical)
c     ungiven   If true un is given
c     unsnt     U_n and sigma_nt prescribed
c     unut      U_n and u_t prescribed
c     ut        Tangential velocity (physical)
c     utgiven   If true U_tis  given in point
c     uxuy      U_x and u_y prescribed
c     walllaw   Law of the wall boundary condition
c     xp        Auxiliary array to calculate boundary point number
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISORIENT2 Computes ndir, nsign, tdir, tsign for a 2-dimensional
c               problem
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c    The folding process will be carried out per side.
c    Some rows corresponding to unknowns in the neighbourhood of the rectangle
c    require special attention
c
c    With respect to the boundary conditions the following combinations
c    are important:
c
c    U^n and U^t given   (U^n = sqrt(g^nn) u_n,                     (1)
c                         U^t=(sqrt(g_tt) u_t - g_nt U^n)/g_tt)     (2)
c
c    u_n given, u_t not
c    u_t given, u_n not
c    u_t nor u_n given
c    multiblock
c    periodical boundary conditions
c
c    For illustration purposes we limit ourselves to the lower side and
c    in case of a corner to the combination of lower side and left-hand side
c    With respect to the tangential cells we limit ourselves to the cells
c    2 until ni, hence all tangential velocities not at other boundaries
c
c    For those cells we have the following situation:
c
c                |(-1,3)       |(1,3)
c
c          --(-2,2)     --(0,2)      --(2,2)
c
c    |(-3,1)     |(-1,1)       |(1,1)       |(3,1)
c
c          --(-2,0)     --(0,0)      --(2,0)
c                 (-1,-1)       (1,-1)
c    | - - - - - | - - - - - - | - - - - - -|- - - -  lower boundary
c
c          --(-2,-2)    --(0,-2)     --(2,-2)
c
c                |(-1,-3)      |(1,-3)
c
c    The tangential unknowns (.,-2) are outside the region and must be
c    folded. The normal components (.,-1) may be known, in which case
c    the known values will be substituted.
c    The normal unknowns (.,-3) are outside the region and must be folded.
c
c    The folding process is as follows:
c
c    a. Periodical and multiblock boundaries: no action should be taken
c    b. U^n and U^t given at the boundary (.,-1):
c            U^t(.,-2) = 2U^t(.,-1) - U^t(.,0)                         (3)
c    c. U^t not given:
c            U^t(.,-2) = 2U^t(.,0) - U^t(.,2)                          (4)
c    d. U^t given and U^n not:
c            At the points (-2,-2) and (2,-2) use formula (4)
c            At the point (0,-2) use a combination of (2) and (3):
c            U^t(0,-2) = 2U^t(0,-1) - U^t(0,0)                         (5)
c            U^t(0,-1) = (sqrt(g_tt) u_t - g_nt (U^n(1,-1)+U^n(-1,-1))/2)/g_tt)
c
c
c    With respect to the normal cells we first consider the cells
c    2 until ni-1, hence all normal velocities with a distance of at least
c    1.5 cell to other boundaries
c
c    For those cells we have the following situation:
c
c                       --(-1,3)     --(1,3)
c
c                |(-2,2)       |(0,2)        |(2,2)
c
c           --(-3,1)    --(-1,1)     --(1,1)      --(3,1)
c                 (-2,0)        (0,0)         (2,0)
c    - - - - - - | - - - - - - | - - - - - - | - - -   lower boundary
c
c           --(-3,-1)   --(-1,-1)    --(1,-1)     --(3,-1)
c
c                |(-2,-2)      |(0,-2)       |(2,-2)
c
c                       --(-1,-3)    --(1,-3)
c
c     Both the tangential unknowns U^t(.,-1), U^t(.,-3),
c     as the normal unknowns U^n(.,-2) should be folded
c
c    The folding process is as follows:
c
c    a. Periodical and multiblock boundaries: no action should be taken
c    b. U^n given at the boundary: diagonal element 1, off-diagonal elements 0
c       Value in right-hand side
c    c. In all other situations we do:
c         U^t(.,-3) = 2U^t(.,-1) - U^t(.,1)
c         U^n(.,-2) = 2U^n(.,0) - U^n(.,2)                              (6)
c       If U^t given: Use formula (5) with adapted indices
c       else
c          at this moment use U^t(.,-1) = U^t(.,1) (See 91-09, page 34)  (7)
c
c    With respect to the normal cells, it is also necessary to consider the
c    cells with center at a distance of 1 cell of the boundary
c    For those cells we have to eliminate two tangential velocities
c    At this moment we do this by linear extrapolation
c
c    Finally we consider a cell at a corner
c
c    Since at this moment we restrict ourselves to Dirichlet boundary
c    conditions, the only special situation is the one with respect to cell
c    corresponding to the normal velocity at 1 cell distance of the under
c    boundary and a half cell distance of the left boundary
c    For that cell we have to eliminate the U^t(.,-3) unknowns.
c
c                        |left boundary
c
c                |(-2,2) |     |(0,2)        |(2,2)
c
c                       --(-1,1)     --(1,1)
c                 (-2,0) |      (0,0)         (2,0)
c                |       - - - | - - - - - - | - - -   lower boundary
c
c                       --(-1,-1)    --(1,-1)
c
c                |(-2,-2)      |(0,-2)       |(2,-2)
c
c
c    The folding process is as follows:
c
c    a. If point (-2,0) does not correspond to periodical or multiblock bc's
c       nor U^n given then
c         - U^n(-2,-2) = ( 2U^n(0,-2)-U^n(2,-2) + 2U^n(-2,0)-U^n(-2,2) )/2   (8)
c         if ( ut given ) then
c            eliminate (-1,-1) using formula (5)
c            In the special case that along the neighbour boundary also only ut
c            is given we may not follow this approach since then U(-2,0) is
c            changed, but this a tangential velocity at the other side
c            which itself changes (-1,-1)
c
c            In this special case we have at both ut given and compute
c            un in the following way:
c
c            u = ut t + un n       = ut t + un n , where 1 refers to
c                  1 1    1 1          2 2    2 2
c            lower side and 2 to the left side
c
c            hence un  = ( ut  - ut (t ,t ) ) / (n ,t )
c                    1       2     1  1  2        1  2
c
c                  un  = ( ut  - ut (t ,t ) ) / (n ,t )
c                    2       1     2  2  1        2  1
c
c                  t  = a    / ||a   ||  t  = a    / ||a   ||
c                   1    (1)      (1)     2    (2)      (2)
c
c                        (2)      (2)          (1)      (1)
c                  n  = a   / ||a    ||  n  = a   / ||a    ||
c                   1                     2
c
c            This special possibility has not yet been programmed since it
c            is not at all clear if this situation ever occurs.
c            In the input subroutine this situation is recognized and
c            an error message is given.
c    b. All other points are eliminated at their respective boundaries
c
c
c                    PSEUDO CODE
c
c      For iside := 1 (1) 4 do
c         For 2 corner normal cells do
c            eliminate the point "(-2,-2)" if necessary
c         For tangential cells do
c            Perform the folding process over all outside connections
c         For normal cells do
c            Perform the folding process over all outside connections
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data istwo / 1, 2,
     +             2, 3 /
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- Loop over tangential cells:

      do iside = 1, 4

c     --- Compute tdir and ndir

         call isorient2 ( iside, tdir, ndir, tsign, nsign )

         ipibinf = ibipoint(iside)+nvirtual-1

c        --- loop over tangential cells
c            Compute loop bounds

         iparea(1,1) = 1
         iparea(1,2) = 1
         iparea(2,1) = 1
         iparea(2,2) = 1

         if ( iside.le.2 ) then

c        --- iside = 1, 2:  left- and right-hand side

            iparea(2,2) = nj+1
            if ( iside.eq.2 ) then

c           --- iside = 2:  right-hand side

               iparea(1,1) = ni
               iparea(2,1) = ni

            end if

         else

c        --- iside = 3, 4:  lower- and upper side

            iparea(2,1) = ni+1
            if ( iside.eq.4 ) then

c           --- iside = 4:  upper side

               iparea(1,2) = nj
               iparea(2,2) = nj

            end if

         end if

c        --- Loop over three tangential unknowns outside the region

         alphad = tdir
         if ( tdir.eq.1 ) then
            if ( nsign.lt.0 ) then
               iconctl = mct ( alphad, ndir, -1, 0, 0 )
               iconctr = mct ( alphad, ndir,  0, 0, 0 )
            else
               iconctl = mct ( alphad, ndir, -1, nsign, 0 )
               iconctr = mct ( alphad, ndir,  0, nsign, 0 )
            end if
         else
            if ( nsign.lt.0 ) then
               iconctl = mct ( alphad, ndir, 0, -1, 0 )
               iconctr = mct ( alphad, ndir, 0,  0, 0 )
            else
               iconctl = mct ( alphad, ndir, nsign, -1, 0 )
               iconctr = mct ( alphad, ndir, nsign,  0, 0 )
            end if
         end if
         do k = -1, 1

            if ( tdir.eq.1 ) then
               iconcto = mct ( alphad, tdir, k, nsign, 0 )
               iconctc = mct ( alphad, tdir, k,     0, 0 )
               iconcti = mct ( alphad, tdir, k,-nsign, 0 )
            else
               iconcto = mct ( alphad, tdir, nsign, k, 0 )
               iconctc = mct ( alphad, tdir,     0, k, 0 )
               iconcti = mct ( alphad, tdir,-nsign, k, 0 )
            end if

c           --- loop over cells

            do jp = iparea(1,2), iparea(2,2)
               do ip = iparea(1,1), iparea(2,1)
                  xp(1) = ip
                  xp(2) = jp
                  ipoint = ipibinf+2*(xp(tdir)+k)-1

                  btype = ibinfor(ipoint)

                  skip = btype.ge.89 .and. btype.le. 99
                  if ( .not. skip .and. btype.eq.88 .and. k.ne.0 ) then

c                 --- Transition point found, not below center of cell
c                     Act as cell center

                     icbtype =ibinfor(ipibinf+2*xp(tdir)-1)
                     skip = icbtype.ge.89 .and. icbtype.le.99

                  end if
                  if ( .not. skip ) then
                     if ( btype.eq.parabolic .or. btype.eq.88 ) then
                        btype = unut
                     else if ( btype.eq.walllaw ) then
                        btype = unsnt
                     end if
                     utgiven = btype.eq.snnut .or. btype.eq.unut .or.
     +                         btype.eq.uxuy
                     fact = matrix(ip,jp,alphad,iconcto)
                     matrix(ip,jp,alphad,iconcto) = 0d0
                     if ( .not. utgiven .or. btype.eq.snnut .and.
     +                    k.ne.0 )then

c                    --- U_t has not been given use formula (4) or
c                        U_t has been given and U_n not in points k = -1, 1

                        matrix(ip,jp,alphad,iconctc) =
     +                     matrix(ip,jp,alphad,iconctc)+2d0*fact
                        matrix(ip,jp,alphad,iconcti) =
     +                     matrix(ip,jp,alphad,iconcti)-fact

                     else if ( btype.eq.snnut ) then

c                    --- U_t has been given and U_n not
c                        use formula (5) at the points k = 0

                        gttdivsg = bcgeo(istwo(ndir,ndir),ipoint)
                        gntdivsg =-bcgeo(istwo(ndir,tdir),ipoint)
                        gtt = bcgeo(8,ipoint)*gttdivsg
                        ut = bndcon(2,ipoint)*tsign*bcgeo(8,ipoint)
                        matrix(ip,jp,tdir,iconctc) =
     +                     matrix(ip,jp,alphad,iconctc)-fact
                        rhside(ip,jp,alphad) = rhside(ip,jp,alphad) -
     +                     2d0*fact/sqrt(gtt)*ut
                        fact = fact*gntdivsg/gttdivsg
                        matrix(ip,jp,tdir,iconctl) =
     +                     matrix(ip,jp,alphad,iconctl)-fact
                        matrix(ip,jp,tdir,iconctr) =
     +                     matrix(ip,jp,alphad,iconctr)-fact
                     else
                        if ( btype.eq.unut ) then

c                       --- un and ut given

                           un = bndcon(1,ipoint)*nsign
                           ut = bndcon(2,ipoint)*tsign
                           gttdivsg = bcgeo(istwo(ndir,ndir),ipoint)
                           gntdivsg =-bcgeo(istwo(ndir,tdir),ipoint)
                           gnn = gttdivsg/bcgeo(8,ipoint)
                           gtt = bcgeo(8,ipoint)*gttdivsg
                           gnt = bcgeo(8,ipoint)*gntdivsg
                           un = sqrt(gnn)*un
                           ut = (sqrt(gtt)*ut-gnt*un)/gtt
                           un = bcgeo(8,ipoint)*un
                           ut = bcgeo(8,ipoint)*ut

                        else

c                       --- ux and uy given

                           ut = bcgeo(2+2*tdir,ipoint)*bndcon(1,ipoint)+
     +                          bcgeo(3+2*tdir,ipoint)*bndcon(2,ipoint)

                        end if

                        matrix(ip,jp,alphad,iconctc) =
     +                     matrix(ip,jp,alphad,iconctc)-fact
                        rhside(ip,jp,alphad) = rhside(ip,jp,alphad) -
     +                     2d0*fact*ut

                     end if

                  end if

c                 --- Only if all tangential velocities are treated we may
c                     substitute the values for the normal velocities if
c                     given, since the matrix elements may have been updated
c                     during elimination

                  if ( k.eq.1 ) then

c                 --- k = 1, eliminate normal velocities at the boundary
c                     if given (All tangential velocities corresponding to
c                     this cell are treated.
c                     "Left" point:

                     ipoint = ipibinf+2*xp(tdir)-2
                     btype = ibinfor(ipoint)
                     ungiven = .false.
                     if ( btype.eq.unut .or. btype.eq.parabolic .or.
     +                    btype.eq.88 .or. btype.eq.unsnt .or.
     +                    btype.eq.walllaw ) then

                        un = bndcon(1,ipoint)*nsign
                        gttdivsg = bcgeo(istwo(ndir,ndir),ipoint)
                        gnn = gttdivsg*bcgeo(8,ipoint)
                        un = sqrt(gnn)*un
                        ungiven = .true.
                     else if ( btype.eq.uxuy ) then
                        un = bcgeo(2+2*ndir,ipoint)*bndcon(1,ipoint)+
     +                       bcgeo(3+2*ndir,ipoint)*bndcon(2,ipoint)
                        ungiven = .true.
                     end if
                     if ( ungiven ) then

c                    --- un is given

                        rhside(ip,jp,alphad) = rhside(ip,jp,alphad) -
     +                         matrix(ip,jp,alphad,iconctl)*un
                        matrix(ip,jp,alphad,iconctl) = 0d0

                     end if

c                    --- "Right" point:

                     ipoint = ipibinf+2*xp(tdir)
                     btype = ibinfor(ipoint)
                     ungiven = .false.
                     if ( btype.eq.unut .or. btype.eq.parabolic .or.
     +                    btype.eq.88 .or. btype.eq.unsnt .or.
     +                    btype.eq.walllaw ) then

                        un = bndcon(1,ipoint)*nsign
                        gttdivsg = bcgeo(istwo(ndir,ndir),ipoint)
                        gnn = gttdivsg*bcgeo(8,ipoint)
                        un = sqrt(gnn)*un
                        ungiven = .true.
                     else if ( btype.eq.uxuy ) then
                        un = bcgeo(2+2*ndir,ipoint)*bndcon(1,ipoint)+
     +                       bcgeo(3+2*ndir,ipoint)*bndcon(2,ipoint)
                        ungiven = .true.
                     end if
                     if ( ungiven ) then

c                    --- un is given

                        rhside(ip,jp,alphad) = rhside(ip,jp,alphad) -
     +                         matrix(ip,jp,alphad,iconctr)*un
                        matrix(ip,jp,alphad,iconctr) = 0d0

                     end if

                  end if

                  ipoint = ipibinf+2*(xp(tdir)+k)-1
                  btype = ibinfor(ipoint)

               end do

            end do

         end do

c        --- Loop over two normal velocities outside the region:

         do k = -1, 0

            if ( tdir.eq.1 ) then
               if ( nsign.lt.0) then
                  iconcto = mct ( alphad, ndir, k, nsign, 0 )
                  iconctc = mct ( alphad, ndir, k,     0, 0 )
                  iconcti = mct ( alphad, ndir, k,-nsign, 0 )
               else
                  iconcto = mct ( alphad, ndir, k, 2*nsign, 0 )
                  iconctc = mct ( alphad, ndir, k, nsign, 0 )
                  iconcti = mct ( alphad, ndir, k, 0, 0 )
               end if
            else
               if ( nsign.lt.0) then
                  iconcto = mct ( alphad, ndir, nsign, k, 0 )
                  iconctc = mct ( alphad, ndir,  0, k, 0 )
                  iconcti = mct ( alphad, ndir,-nsign, k, 0 )
               else
                  iconcto = mct ( alphad, ndir, 2*nsign, k, 0 )
                  iconctc = mct ( alphad, ndir,  nsign, k, 0 )
                  iconcti = mct ( alphad, ndir, 0, k, 0 )
               end if
            end if

c           --- loop over cells

            do jp = iparea(1,2), iparea(2,2)
               do ip = iparea(1,1), iparea(2,1)
                  xp(1) = ip
                  xp(2) = jp
                  ipoint = ipibinf+2*(xp(tdir)+k)
                  btype = ibinfor(ipoint)

                  skip = btype.ge.89 .and. btype.le.99

                  if ( .not. skip ) then

                     fact = matrix(ip,jp,alphad,iconcto)
                     matrix(ip,jp,alphad,iconcto) = 0d0
                     matrix(ip,jp,alphad,iconctc) =
     +               matrix(ip,jp,alphad,iconctc)+2d0*fact
                     matrix(ip,jp,alphad,iconcti) =
     +               matrix(ip,jp,alphad,iconcti)-fact

                  end if

               end do
            end do

         end do

      end do

c     --- Loop over the normal cells at a distance 1 of the boundary

      do iside = 1, 4

c     --- Compute tdir and ndir

         call isorient2 ( iside, tdir, ndir, tsign, nsign )

         ipibinf = ibipoint(iside)+nvirtual-1

c        --- Compute loop bounds

         if ( iside.le.2 ) then

c        --- iside = 1, 2:  left- and right-hand side

            iparea(1,2) = 1
            iparea(2,2) = nj
            if ( iside.eq.1 ) then

c           --- iside = 1:  left-hand side

               iparea(1,1) = 2
               iparea(2,1) = 2

            else

c           --- iside = 2:  right-hand side

               iparea(1,1) = ni
               iparea(2,1) = ni

            end if

         else

c        --- iside = 3, 4:  lower- and upper side

            iparea(1,1) = 1
            iparea(2,1) = ni
            if ( iside.eq.3 ) then

c           --- iside = 3:  under side

               iparea(1,2) = 2
               iparea(2,2) = 2

            else

c           --- iside = 4:  upper side

               iparea(1,2) = nj
               iparea(2,2) = nj

            end if

         end if
         alphad = ndir

c        --- Loop over two tangential velocities outside the region:

         do k = 0, 1

            if ( tdir.eq.1 ) then
               if ( nsign.lt.0 ) then
                  iconcto = mct ( alphad, tdir, k, 2*nsign, 0 )
                  iconctc = mct ( alphad, tdir, k, nsign, 0 )
                  iconcti = mct ( alphad, tdir, k, 0, 0 )
               else
                  iconcto = mct ( alphad, tdir, k, nsign, 0 )
                  iconctc = mct ( alphad, tdir, k, 0, 0 )
                  iconcti = mct ( alphad, tdir, k, -nsign, 0 )
               end if
            else
               if ( nsign .lt.0 ) then
                  iconcto = mct ( alphad, tdir, 2*nsign, k, 0 )
                  iconctc = mct ( alphad, tdir, nsign, k, 0 )
                  iconcti = mct ( alphad, tdir, 0, k, 0 )
               else
                  iconcto = mct ( alphad, tdir, nsign, k, 0 )
                  iconctc = mct ( alphad, tdir, 0, k, 0 )
                  iconcti = mct ( alphad, tdir, -nsign, k, 0 )

               end if
            end if

c           --- loop over cells

            do jp = iparea(1,2), iparea(2,2)
               do ip = iparea(1,1), iparea(2,1)
               xp(1) = ip
               xp(2) = jp

               ipoint = ipibinf+2*(xp(tdir)+k) -1

               btype = ibinfor(ipoint)

               skip = btype.ge.88 .and. btype.le. 99

                  if ( .not. skip ) then

                     fact = matrix(ip,jp,alphad,iconcto)
                     matrix(ip,jp,alphad,iconcto) = 0d0
                     matrix(ip,jp,alphad,iconctc) =
     +               matrix(ip,jp,alphad,iconctc)+2d0*fact
                     matrix(ip,jp,alphad,iconcti) =
     +               matrix(ip,jp,alphad,iconcti)-fact

                  end if

               btype = ibinfor(ipoint)

               end do

            end do

         end do

c        --- loop over normal half cells
c            Compute loop bounds

         iparea(1,1) = 1
         iparea(1,2) = 1
         iparea(2,1) = 1
         iparea(2,2) = 1
         if ( iside.le.2 ) then

c        --- iside = 1, 2:  left- and right-hand side

            iparea(2,2) = nj
            if ( iside.eq.2 ) then

c           --- iside = 2:  right-hand side

               iparea(1,1) = ni+1
               iparea(2,1) = ni+1

            end if

         else

c        --- iside = 3, 4:  lower- and upper side

            iparea(2,1) = ni
            if ( iside.eq.4 ) then

c           --- iside = 4:  upper side

               iparea(1,2) = nj+1
               iparea(2,2) = nj+1

            end if

         end if
         alphad = ndir

c        --- loop over cells

         do jp = iparea(1,2), iparea(2,2)
            do ip = iparea(1,1), iparea(2,1)
               xp(1) = ip
               xp(2) = jp
               ipoint = ipibinf+2*xp(tdir)
               btype = ibinfor(ipoint)

               ungiven = btype.eq.parabolic .or. btype.eq.88 .or.
     +                   btype.eq.unut .or. btype.eq.uxuy .or.
     +                   btype.eq.unsnt .or. btype.eq.walllaw
               if ( ungiven ) then

c              --- Normal velocity given, set diagonal to 1 and
c                  off-diagonal to 0

                  matrix(ip,jp,alphad,1 ) = 1d0
                  do k = 2, 21
                     matrix(ip,jp,alphad,k ) = 0d0
                  end do
                  if ( btype.eq.uxuy ) then
                     un = bcgeo(2+2*ndir,ipoint)*bndcon(1,ipoint)+
     +                    bcgeo(3+2*ndir,ipoint)*bndcon(2,ipoint)
                  else
                     un = bndcon(1,ipoint)*nsign
                     gttdivsg = bcgeo(istwo(ndir,ndir),ipoint)
                     gnn = gttdivsg*bcgeo(8,ipoint)
                     un = sqrt(gnn)*un
                  end if
                  rhside(ip,jp,alphad ) = un

               end if

            end do
         end do

c        --- Loop over two tangential velocities outside the region:

         do k = 0, 1

            if ( tdir.eq.1 ) then
               if ( nsign.lt.0 ) then
                  iconcto = mct ( alphad, tdir, k, 2*nsign, 0 )
                  iconctc = mct ( alphad, tdir, k, nsign, 0 )
                  iconcti = mct ( alphad, tdir, k, 0, 0 )
               else
                  iconcto = mct ( alphad, tdir, k, nsign, 0 )
                  iconctc = mct ( alphad, tdir, k, 0, 0 )
                  iconcti = mct ( alphad, tdir, k, -nsign, 0 )
               end if
            else
               if ( nsign .lt.0 ) then
                  iconcto = mct ( alphad, tdir, 2*nsign, k, 0 )
                  iconctc = mct ( alphad, tdir, nsign, k, 0 )
                  iconcti = mct ( alphad, tdir, 0, k, 0 )
               else
                  iconcto = mct ( alphad, tdir, nsign, k, 0 )
                  iconctc = mct ( alphad, tdir, 0, k, 0 )
                  iconcti = mct ( alphad, tdir, -nsign, k, 0 )

               end if
            end if

c           --- loop over cells

            do jp = iparea(1,2), iparea(2,2)
               do ip = iparea(1,1), iparea(2,1)
                  xp(1) = ip
                  xp(2) = jp

                  ipoint = ipibinf+2*(xp(tdir)+k)-1

                  btype = ibinfor(ipoint)

                  skip = btype.ge.88 .and. btype.le. 99

                  if (  .not. skip ) then

                     fact = matrix(ip,jp,alphad,iconcto)
                     matrix(ip,jp,alphad,iconcto) = 0d0
                     matrix(ip,jp,alphad,iconctc) =
     +                  matrix(ip,jp,alphad,iconctc)+2d0*fact
                     matrix(ip,jp,alphad,iconcti) =
     +                  matrix(ip,jp,alphad,iconcti)-fact
                  end if

               end do
            end do

         end do

c        --- Loop over four tangential velocities outside the region:

         do k = -1, 2

            if ( tdir.eq.1 ) then
               if ( nsign.lt.0 ) then
                  iconcto = mct ( alphad, tdir, k, nsign, 0 )
                  iconcti = mct ( alphad, tdir, k, 0, 0 )
               else
                  iconcto = mct ( alphad, tdir, k, 0, 0 )
                  iconcti = mct ( alphad, tdir, k, -nsign, 0 )
               end if
            else
               if ( nsign .lt.0 ) then
                  iconcto = mct ( alphad, tdir, nsign, k, 0 )
                  iconcti = mct ( alphad, tdir, 0, k, 0 )
               else
                  iconcto = mct ( alphad, tdir, 0, k, 0 )
                  iconcti = mct ( alphad, tdir, -nsign, k, 0 )

               end if
            end if

c           --- loop over cells

            do jp = iparea(1,2), iparea(2,2)
               do ip = iparea(1,1), iparea(2,1)
                  xp(1) = ip
                  xp(2) = jp
                  ipoint = ipibinf+2*(xp(tdir)+k)-1
                  btype = ibinfor(ipoint)

                  skip = btype.ge.88 .and. btype.le. 99

                  if (  .not. skip ) then

                     fact = matrix(ip,jp,alphad,iconcto)
                     matrix(ip,jp,alphad,iconcto) = 0d0
                     matrix(ip,jp,alphad,iconcti) =
     +                  matrix(ip,jp,alphad,iconcti)+fact
                  end if

               end do
            end do

         end do

c        --- Loop over three normal velocities outside the region:

         do k = -1, 1

            if ( tdir.eq.1 ) then
               iconcto = mct ( alphad, ndir, k, nsign, 0 )
               iconctc = mct ( alphad, ndir, k, 0, 0 )
               iconcti = mct ( alphad, ndir, k, -nsign, 0 )
            else

               iconcto = mct ( alphad, ndir, nsign, k, 0 )
               iconctc = mct ( alphad, ndir, 0, k, 0 )
               iconcti = mct ( alphad, ndir, -nsign, k, 0 )
            end if

c           --- loop over cells

            do jp = iparea(1,2), iparea(2,2)
               do ip = iparea(1,1), iparea(2,1)
                  xp(1) = ip
                  xp(2) = jp
                  ipoint = ipibinf+2*(xp(tdir)+k)
                  btype = ibinfor(ipoint)
                  skip = btype.ge.88 .and. btype.le. 99

                  if (  .not. skip ) then

                     fact = matrix(ip,jp,alphad,iconcto)
                     matrix(ip,jp,alphad,iconcto) = 0d0
                     matrix(ip,jp,alphad,iconctc) =
     +                  matrix(ip,jp,alphad,iconctc)+2d0*fact
                     matrix(ip,jp,alphad,iconcti) =
     +                  matrix(ip,jp,alphad,iconcti)-fact
                  end if

               end do
            end do

         end do
      end do

      end
