      subroutine ismomrh2 ( f, rho, coor, xcoor, gmat, gmctinv, rhside,
     +                      work, ni, nj, nvirtual, ibipoint,
     +                      ibinfor, cva )
c ======================================================================
c
c     programmer  Guus Segal
c     version     2.0   date  29-09-1999 Complete update, extra parameters
c     version     1.1   date  30-11-1995 Correction rho interpolation
c     version     1.0   date  04-10-1995
c
c   copyright (c) 1995-1999  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     ISNAS subroutine to build the right-hand side source term in R2
c     for one step in the solution process for the momentum equations.
c     rho    * [ Gx     f1     +  Gy     f2 ]
c       (0,0)      (0,0)  (0,0)     (0,0)  (0,0)
c **********************************************************************
c
c                       KEYWORDS
c
c     force_righthandside
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, gmctinv(0:3,2,6), ibipoint(*),
     +        ibinfor(*)
      double precision gmat(ni+1,nj+1,2,6), rhside(ni+1,nj+1,2),
     +                 xcoor(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,2),
     +                 work(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,5),
     +                 coor(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,2),
     +                 rho(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual),
     +                 f(1-nvirtual:ni+1+nvirtual,
     +                   1-nvirtual:nj+1+nvirtual,2),
     +                 cva(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,2,2,4)

c     coor           i    Array containing the coordinates for one block
c                         In 2d coor(i,j,.) contains the x resp y-coordinate
c                         of point i,j
c                         in 3d: coor(i,j,k,.) corresponds to (i,j,k)
c     cva            i    Array containing the covariant basis vectors
c                         in 2D we have
c                                                      l
c                         cva(i,j,l,alpha,p) contains a        in cell i,j in
c                                                      (alpha)     point type p
c                         in 3D it is cva(i,j,k,l,alpha,p) referring to
c                         cell i,j,k
c     f              i    Array containing the force term
c     gmat           i    Gradient matrix
c     gmctinv        i    Connection+row degree of freedom to column indices
c                         for gradient matrix
c     ibinfor        i    Third part of infbnd
c     ibipoint       i    ipoint(side)
c                                      position of start of ibinfor information
c                                      for SIDE in ibinfor
c     ni             i    Number of non-virtual cells in the i-direction
c                         (one block)
c     nj             i    Number of non-virtual cells in the j-direction
c                         (one block)
c     nvirtual       i    Number of rows of virtual cells
c     rho            i    Estimate of the asymptotic convergence rate
c     rhside        i/o   right-hand side vector
c     work                Work array
c     xcoor               Work array in which the co-ordinates in the centroids
c                         are stored
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer snnsnt, snnut
      parameter ( snnsnt = 7, snnut = 6 )
      integer idim, i, ishift, j, jshift, k, btype, ipibinf, iside
      double precision f1int, f2int, rhoint

c     btype          Type of boundary condition
c     f1int          Value of f_1 in integration point
c     f2int          Value of f_2 in integration point
c     i              General loop variable
c     idim           Loop variable used for dimension
c     ipibinf        Starting address of array infor for present side
c                    minus one
c     ishift         Shift in i-direction
c     iside          Sequence number of side
c     j              General loop variable
c     jshift         Shift in j-direction
c     k              General loop variable
c     rhoint         Value of rho in integration point
c     snnsnt         Sigma_nn and sigma_nt prescribed
c     snnut          Sigma_nn and u_t prescribed
c **********************************************************************
c
c                       I/O
c
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISSETCON       Make array equal to a constant
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c
c                  Mathematical evaluation
c
c
c   In order to balance pressure forces and the right-hand side force
c   the right-hand side is evaluated using the gradient matrix
c
c   Since grad x = (1,0) and grad y = (0,1) the right-hand side is given by:
c
c     rho    * [ Gx     f1     +  Gy     f2 ]
c       (0,0)      (0,0)  (0,0)     (0,0)  (0,0)
c
c
c   However, once sigma_nn is given at a boundary this formula can no longer be
c   applied since the gradient matrix is adapted.
c   For those points we go back to the classical finite volume approach:
c
c   / f^alpha dOmega = sqrt(g) ^alpha / 2 (Only for half cells!)
c
c   Because f^alpha = a^alpha . f we get
c
c   rhs = sqrt(g)/2 * ( a^1 f1 + a^2 f2)
c
c   sqrt(g) * a^alpha can be computed from a_(alpha) as described in isjcta2d
c
c                  PSEUDO CODE
c
c
c    evaluate x in cell centres and store in xcoor(.,1)
c    evaluate y in cell centres and store in xcoor(.,2)
c    for idim := 1, 2 do
c       Compute Gx in integration centre and store in work(.,1)
c       Compute Gy in integration centre and store in work(.,2)
c       evaluate rho in integration centre
c       evaluate f1  in integration centre
c       evaluate f2  in integration centre
c       Update right-hand side
c       treat the boundaries in a special way
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c

c     --- evaluate x and y in cell centres and store in xcoor(.,1)

      do idim = 1, 2
         do i = 0, ni+1
            do j = 0, nj+1
               xcoor(i,j,idim) = 0.25d0*(coor(i,j,idim)+
     +                           coor(i+1,j,idim)+coor(i,j+1,idim)+
     +                           coor(i+1,j+1,idim))
            end do
         end do
      end do

c     --- First evaluation and updating in 1-direction
c         Compute Gx and Gy, store in work

      call issetcon ( work, (ni+1+2*nvirtual)*(nj+1+2*nvirtual)*2, 0d0 )
      do k = 1, 6

         ishift = gmctinv(1,1,k)
         jshift = gmctinv(2,1,k)

         do j = 1, nj
            do i = 1, ni+1
               work(i,j,1) = work(i,j,1) + gmat(i,j,1,k) *
     +                       xcoor(i+ishift,j+jshift,1)
               work(i,j,2) = work(i,j,2) + gmat(i,j,1,k) *
     +                       xcoor(i+ishift,j+jshift,2)
            end do
         end do

      end do

c     --- evaluate rho, f1 and f2 in integration centre and store in work(.,1-3)

      do i = 2, ni
         do j = 1, nj
            rhoint = 0.5d0*(rho(i,j)+rho(i-1,j))
            f1int = 0.5d0*(f(i,j,1)+f(i-1,j,1))
            f2int = 0.5d0*(f(i,j,2)+f(i-1,j,2))
            rhside(i,j,1) = rhside(i,j,1)+rhoint*
     +                      (work(i,j,1)*f1int+work(i,j,2)*f2int)
         end do
      end do

c     --- Loop over sides 1, 2:  left- and right-hand side
c         Only the half cells are considered

      do iside = 1, 2

         ipibinf = ibipoint(iside)-1+nvirtual
         if ( iside.eq.2 ) then

c        --- iside = 2:  right-hand side

            i = ni+1

         else

c        --- iside = 1:  left-hand side

            i = 1

         end if

         do j = 1, nj

c        --- Get integration cell position for ibinfor

            btype = ibinfor(ipibinf+2*j)
            if ( btype.eq.snnut .or. btype.eq.snnsnt ) then

c           --- Normal stress given, use the integral over f

               rhoint = 0.25d0*(rho(i,j)+rho(i-1,j))
               f1int = 0.5d0*(f(i,j,1)+f(i-1,j,1))
               f2int = 0.5d0*(f(i,j,2)+f(i-1,j,2))
               rhside(i,j,1) = rhside(i,j,1)+rhoint*
     +                         (cva(i,j,2,2,3)*f1int-
     +                          cva(i,j,2,1,3)*f2int)

            else

c           --- usual approach

               rhoint = 0.5d0*(rho(i,j)+rho(i-1,j))
               f1int = 0.5d0*(f(i,j,1)+f(i-1,j,1))
               f2int = 0.5d0*(f(i,j,2)+f(i-1,j,2))
               rhside(i,j,1) = rhside(i,j,1)+rhoint*
     +                         (work(i,j,1)*f1int+work(i,j,2)*f2int)

            end if

         end do

      end do

c     --- Next evaluation and updating in 2-direction
c         Compute Gx and Gy, store in work

      call issetcon ( work, (ni+1+2*nvirtual)*(nj+1+2*nvirtual)*2, 0d0 )
      do k = 1, 6

         ishift = gmctinv(1,2,k)
         jshift = gmctinv(2,2,k)

         do j = 1, nj+1
            do i = 1, ni
               work(i,j,1) = work(i,j,1) + gmat(i,j,2,k) *
     +                       xcoor(i+ishift,j+jshift,1)
               work(i,j,2) = work(i,j,2) + gmat(i,j,2,k) *
     +                       xcoor(i+ishift,j+jshift,2)
            end do
         end do

      end do

c     --- evaluate rho, f1 and f2 in integration centre and store in work(.,1-3)

      do i = 1, ni
         do j = 2, nj
            rhoint = 0.5d0*(rho(i,j)+rho(i,j-1))
            f1int = 0.5d0*(f(i,j,1)+f(i,j-1,1))
            f2int = 0.5d0*(f(i,j,2)+f(i,j-1,2))
            rhside(i,j,2) = rhside(i,j,2)+rhoint*
     +                      (work(i,j,1)*f1int+work(i,j,2)*f2int)
         end do
      end do

c     --- Loop over sides 3, 4:  lower- and upper side

      do iside = 3, 4

         ipibinf = ibipoint(iside)-1+nvirtual
         if ( iside.eq.4 ) then

c        --- iside = 4:  upper side

            j = nj+1

         else

c        --- iside = 3:  lower side

            j = 1

         end if

         do i = 1, ni

c        --- Get integration cell position for ibinfor

            btype = ibinfor(ipibinf+2*i)
            if ( btype.eq.snnut .or. btype.eq.snnsnt ) then

c           --- Normal stress given, use the integral over f

               rhoint = 0.25d0*(rho(i,j)+rho(i,j-1))
               f1int = 0.5d0*(f(i,j,1)+f(i,j-1,1))
               f2int = 0.5d0*(f(i,j,2)+f(i,j-1,2))
               rhside(i,j,2) = rhside(i,j,2)+rhoint*
     +                         (-cva(i,j,1,2,2)*f1int+
     +                          cva(i,j,1,1,2)*f2int)
            else

c           --- usual approach

               rhoint = 0.5d0*(rho(i,j)+rho(i,j-1))
               f1int = 0.5d0*(f(i,j,1)+f(i,j-1,1))
               f2int = 0.5d0*(f(i,j,2)+f(i,j-1,2))
               rhside(i,j,2) = rhside(i,j,2)+rhoint*
     +                         (work(i,j,1)*f1int+work(i,j,2)*f2int)

            end if

         end do

      end do

      end
