      subroutine ismomtd2 ( alphad, ni, nj, nvirtual, densrho, veloc, 
     +                       matrix, rhside, coor, deltat, iparea,  
     +                       mcom, massfrac, mequat, zerocoef, axisym )
c ======================================================================
c
c        programmer     Guus Segal
c        version 3.1    date 22-01-1998 Hester: zerocoef for compressible
c        version 3.0    date 05-01-1998 Extra parameter zerocoef
c        version 2.0    date 07-08-1997 Extra parameters massfrac, mequat
c        version 1.0    date 07-03-1995
c
c   copyright (c) 1995-1997  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     adds the time derivative to the diagonal of the matrix and
c     the right-hand side
c     Also the contribution of the zeroth order term in case of multi phase gas
c     flow is added. Mark that this extension is ad-hoc
c     R^2 only
c **********************************************************************
c
c                       KEYWORDS
c
c     2d
c     matrix
c     right_hand_side
c     time_derivative
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, iparea(2,2), mcom, mequat, alphad,
     +         axisym
      double precision densrho(1-nvirtual:ni+1+nvirtual,
     +                         1-nvirtual:nj+1+nvirtual),
     +                 veloc(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual),
     +                 massfrac(1-nvirtual:ni+1+nvirtual,
     +                          1-nvirtual:nj+1+nvirtual),
     +                 zerocoef(1-nvirtual:ni+1+nvirtual,
     +                          1-nvirtual:nj+1+nvirtual),
     +                 coor(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,2),
     +                 matrix(ni+1,nj+1),
     +                 rhside(ni+1,nj+1), deltat

c     deltat    i    Product of theta and dt
c     densrho   i    Array containing the density rho. The size and structure is
c                    the same as for the scalar unknowns
c     iparea    i    Area of integration points where a contribution to
c                    the matrix is to be calculated
c     massfrac  i    Array containing the mass fraction
c     matrix   i/o   reserved
c     mcom      i    If zero the flow is incompressible else (1) compressible
c     mequat    i    Indicates the type of momentum equation to be solved:
c                    0:  standard equation
c                    1:  multi-phase gas flow
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     rhside   i/o   reserved
c     veloc     i    Array for the old velocities (used in convection
c                    term calculation)
c     zerocoef  i    Array containing the coefficient for the zero-th order term
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ip, jp
      double precision deltatin

c     deltatin  Inverse of deltat, i.e. 1/deltat
c     ip        Local counting variable i-direction
c     jp        Local counting variable j-direction
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c      It is assumed that the matrix and right-hand side have been initialized
c      before.
c      Furthermore this subroutine may only be called for centroids of
c      velocity cells.
c
c        The element for the time-derivative in the matrix diagonal is
c
c                   alphad
c          rho     V      / deltat
c             (0,0) (0,0)
c
c
c        The element for the time-derivative in the right-hand side is
c
c                  _alphad
c          rho     V      / deltat
c             (0,0) (0,0)
c
c      In the case of multiphase gas flow we have to multiply the velocity
c      at the previous time level by the mass fraction epsilon
c      Also we introduce the extra term
c
c           c V^alphad in the equation
c
c      Since the unknown is eps V^alphad, we have to divide the coefficient
c      c by eps
c
c      Remark: the present implementation may only be used for a Cartesian grid
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- loop over cells

      deltatin = 1d0/deltat
      if ( mcom.eq.0 ) then

c     --- incompressible code

         do jp = iparea(1,2), iparea(2,2)
            do ip = iparea(1,1), iparea(2,1)

               matrix(ip,jp) = densrho(ip,jp) * deltatin

            end do
         end do

      else

c     --- compressible code (rho=1)

         if ( axisym .eq. 1) then
	 
	    if ( alphad .eq. 1) then
               do jp = iparea(1,2), iparea(2,2)
                  do ip = iparea(1,1), iparea(2,1)
                     matrix(ip,jp) = deltatin * 5d-1 *
     +                                ( coor(ip,jp,1)+coor(ip,jp+1,1))
                  end do
               end do	    
	    else
               do jp = iparea(1,2), iparea(2,2)
                  do ip = iparea(1,1), iparea(2,1)
                     matrix(ip,jp) = deltatin * 5d-1 *
     +                                ( coor(ip,jp,1)+coor(ip+1,jp,1))		     
                  end do
               end do
	    end if
	 
	 else
	 
            do jp = iparea(1,2), iparea(2,2)
               do ip = iparea(1,1), iparea(2,1)
                  matrix(ip,jp) = deltatin
               end do
            end do
	 
	 end if

      end if

      if ( mequat.eq.0 ) then

c     --- standard case

         do jp = iparea(1,2), iparea(2,2)
            do ip = iparea(1,1), iparea(2,1)

               rhside(ip,jp) =  densrho(ip,jp) *
     +                          veloc(ip,jp) * deltatin

            end do
         end do

      else

c     --- two-phase gas flow
c         the right-hand side equation is updated by the standard term
c         multiplied by the mass fraction
c         The matrix is updated by the zeroth order coefficient divided
c         by the mass fraction

         if (mcom.eq.0) then

            do jp = iparea(1,2), iparea(2,2)
              do ip = iparea(1,1), iparea(2,1)

                rhside(ip,jp) =  densrho(ip,jp) * massfrac(ip,jp) *
     +                           veloc(ip,jp) * deltatin
                matrix(ip,jp) = matrix(ip,jp) +
     +                          zerocoef(ip,jp)/massfrac(ip,jp)

             end do
          end do

        else

            do jp = iparea(1,2), iparea(2,2)
              do ip = iparea(1,1), iparea(2,1)

                rhside(ip,jp) =  densrho(ip,jp) * massfrac(ip,jp) *
     +                           veloc(ip,jp) * deltatin
                matrix(ip,jp) = matrix(ip,jp) + zerocoef(ip,jp)/
     +                          (massfrac(ip,jp) * densrho(ip,jp))

             end do
          end do

        end if
      end if

      end
