      subroutine isprhs2d ( ni, nj, nvirtual, vstar, vstar1, vstar2,
     +                      vstartmp1 ,vstartmp2,
     +                      rhside, solut, coor, jcta1, jcta2,
     +                      jac, rinput, iinput, ndegfd, uold, switch,
     +                      shiftr, mshtyp, massfrac, intp,
     +                      idiscret, bicof, ibicof, eddy, axisym,
     +                      drhodh )
c ======================================================================
c
c        programmer     Guus Segal
c        version 3.1    date 10-12-1999 Duncan: modification for supersdc
c        version 3.0    date 30-06-1999 Guus: new parameter list
c        version 2.12   date 04-04-1999 Duncan: Supersonic defect correction
c        version 2.11   date 27-03-1999 Duncan: Supersonic Crank-Nicholson
c        version 2.10   date 18-03-1999 Duncan: extension for new
c                                               Supersonic Scheme
c        version 2.9    date 22-03-1999 Kees K: axisymmetric compressibel
c                                               standard pressure correction
c                                               no-upwind
c        version 2.8    date 04-03-1999 Duncan: new call to isstdfl/isstdls
c        version 2.7    date 02-02-1999 Kees K: axisymmetric
c        version 2.6    date 27-07-1998 Duncan: extension for density
c                                               bias in j-direction
c                                               ( only cavitating flow)
c                                               extension to iterate
c                                               pressure correction
c                                               new indentation
c        version 2.5    date 04-06-1998 Charles: bilinear interpolation
c        version 2.4    date 05-03-1998 Duncan: extension for cavitation
c        version 2.3    date 01-12-1997 Hester: parameter mprcor added
c        version 2.2    date 12-11-1997 Hester: parameter mscal added
c        version 2.1    date 05-11-1997 Charles: correction in collocated part
c        version 2.0    date 30-10-1997 Hester: extra parameter massfrac
c        version 1.4    date 29-10-1997 Charles: change in collocated part
c        version 1.3    date 13-10-1997 Charles: collocated allowed
c        version 1.2    date 15-05-1996 Hester: sign mistake corrected.
c        version 1.1    date 26-02-1996 Hester: upwinded density for
c                                               compressible flow added.
c        version 1.0    date 06-10-1995
c
c   copyright (c) 1995-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Build pressure right-hand-side vector in R2
c
c **********************************************************************
c
c                       KEYWORDS
c
c    2d
c    matrix
c    pressure_equation
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, iinput(*), ndegfd, mshtyp, idiscret,
     +         axisym
      integer ibicof(1-nvirtual:ni+1+nvirtual,
     +               1-nvirtual:nj+1+nvirtual,2,2)
      double precision bicof(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,2,2)
      double precision jac(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual),
     +                 jcta1(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,2,2),
     +                 jcta2(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,2,2),
     +                 vstar(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,2),
     +                 vstar1(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual),
     +                 vstar2(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual),
     +                 vstartmp1(1-nvirtual:ni+1+nvirtual,
     +                          1-nvirtual:nj+1+nvirtual),
     +                 vstartmp2(1-nvirtual:ni+1+nvirtual,
     +                          1-nvirtual:nj+1+nvirtual),
     +                 drhodh(1-nvirtual:ni+1+nvirtual,
     +                          1-nvirtual:nj+1+nvirtual),
     +                 rhside(ni+1,nj+1), rinput(*),
     +                 solut(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,ndegfd,*),
     +                 switch(ni+1,nj+1,2),
     +                 shiftr(ni+1,nj+1,2),
     +                 uold(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,2),
     +                 massfrac(1-nvirtual:ni+1+nvirtual,
     +                          1-nvirtual:nj+1+nvirtual),
     +                 coor(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,2),
     +                 intp(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,2),
     +                 eddy(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,*)

c     axisym         i    In axi-symmetric case axisym=1 else axisym=0
c     bicof          i    Array containing bilinear coefficients
c     coor           i    Array containing the coordinates for one block
c                         In 2d coor(i,j,.) contains the x resp y-coordinate
c                         of point i,j
c                         in 3d: coor(i,j,k,.) corresponds to (i,j,k)
c     drhodh         i    Contains drho/dh in all centres of the cells
c     eddy           i    array EDDY contains the values of the
c                         eddy viscosity of that block in the centroids
c                         of the elements. The storage of such
c                         an array EDDY is identical to that of all other
c                         coefficients.
c     ibicof         i    Integer array containing bilinear coefficients
c     idiscret       i    Type of discretization
c     iinput         i    Contains the integer user input for one block.
c                         For a description see the programmers guide
c     intp           i    Interpolated values of variable
c     jac            i    sqrt(g) in pnttyp
c     jcta1          i    Jacobian times contravariant basis vectors
c                         in point pnttyp=3
c                         (stored as vector index encoding given by istwo)
c     jcta2          i    Jacobian times contravariant basis vectors
c                         in point pnttyp=2
c                         (stored as vector index encoding given by istwo)
c     massfrac       i    Array containing the mass fraction
c     mshtyp         i    Type of grid arrangement
c     ndegfd         i    Number of degrees of freedom
c     ni             i    Number of non-virtual cells in the i-direction
c                         (one block)
c     nj             i    Number of non-virtual cells in the j-direction
c                         (one block)
c     nvirtual       i    Number of rows of virtual cells
c     rhside         o    right-hand side vector
c     rinput         i    The real user input is stored in the array rinput
c                         for each block
c                         For a description see the programmers guide
c     shiftr         i    Shift variable used in compressible density upwinding.
c     solut          i    Solution array
c                         For a description see the programmers guide
c     switch         i    Continuous compressible density upwind switch
c                           switch = 0 for Mach numbers < 1-ls
c                           switch=0.5 for Mach numbers > 1+ls
c     uold           i    Velocity at previous time step.
c     vstar          i    Array containing the predicted velocity V^*
c     vstar1         i    Array containing the 1st component of V^*
c     vstar2         i    Array containing the 2nd component of  V^*
c     vstartmp1      i    Array containing temporarily 1st component of V^*
c     vstartmp2      i    Array containing temporarily 2nd component of V^*
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j, ip, jp, iprincom, mcom, mcomupw, mprcor,
     +        acttyp, pnttyp, vartyp, alphad, mequat, mscal, mcav,
     +        iconv, ipiintim, ipiincav, istartup, iprincav,
     +        mcomsup, ichoicedb
      double precision deltatin, dt, gamcm, mach2, cons, cons1,
     +                 dummy(1), uloc, vloc

c     acttyp         Type of action to be taken
c                    0      : interpolation for point value
c                    1..ndim: finite difference for partial derivative
c                             in the direction given by value of acttyp
c     alphad         Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     cons           Constant used in equation of state for compressible
c                    computations
c     cons1          Constant used in equation of state for compressible
c                    computations
c     deltatin       Inverse of deltat, i.e. 1/deltat
c     dt             Time step
c     dummy          Dummy
c     gamcm          Gas constant gamma (for compressible comput.)
c     i              General loop variable
c     ichoicedb      if 1 a gradient based, if 2 a mach based,
c                    if 3 an unconditional density bias is applied
c     iconv          If 1 the timestep is iterated till converged,
c                    otherwise (0) apply only one iterative step
c     ip             Local counting variable i-direction
c     ipiincav       Starting address of array IINCAV in array IINPUT
c     ipiintim       Starting address of array IINTIM in array IINPUT
c     iprincav       Starting address of array RINCAV in array RINPUT
c     iprincom       Starting address of array RINCOM in array RINPUT
c     istartup       If 1 the cavity calculation is in the startup
c                    procedure, if 0 not
c     j              General loop variable
c     jp             Local counting variable j-direction
c     mach2          Square of the mach number (for compressible comput.)
c     mcav           If 1 a cavity model is used, if 0 not
c     mcom           If zero the flow is incompressible else (1)
c                    compressible
c     mcomsup        If 1 the supersonic scheme is activated if 0 not
c     mcomupw        If zero the rho equation for compressible flow is
c                    solved by a central scheme,
c                    otherwise by a upwind scheme
c     mequat         Indicates the type of momentum equation to be solved:
c                    0:  standard equation
c                    1:  multi-phase gas flow
c     mprcor         Indicates choice of compressible pressure correction
c                    method:
c                    0:   Original formulation
c                    1:   Pressure corr. equation better for contact
c                         discontinuities, eq. obtained from
c                         energy+momentum eq.
c     mscal          Indicates use of non-dimensionalisation for
c                    compr. flow:
c                    0:   Equations are non-dimensionalised
c                    1:   Equations are in dimensional formulation
c     pnttyp         Point where a value or finite difference is
c                    to be calculated within the cell
c                    For a description see the programmers guide
c     uloc           Local u-coordinate for the interpolation
c     vartyp         Variable to be interpolated/extrapolated
c                    vartyp: 1 for V1, 2 for V2 and 3 for pressure,
c                    0 for vertex (for ndim=2)
c                    or
c                    vartyp: 1 for V1, 2 for V2 3, for V3 and
c                    4 for pressure, 0 for vertex  (for ndim=3)
c     vloc           Local v-coordinate for the interpolation
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      double precision isstdfl, isstdls

c     ISCONC2D       Perform the contravariant components form the Cartesian
c                    at the cell faces
c     ISINTCF2       Compute interpolation of a coefficient in an integration
c                    point
c     ISSETCON       Make array equal to a constant
c     ISSTDFL        dimensionfull equation of state
c     ISSTDLS        dimensionless equation of state
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c    The pressure correction right-hand side has the shape:
c
c        *
c     D V
c
c     with:
c
c     D:        the divergence matrix
c     V*:       the predicted velocity
c
c     The divergence per cell looks like
c
c
c                          1
c                   -------|-------
c                   |             |
c                   |             |
c                -1 -             - 1
c                   |             |
c                   |             |
c                   -------|-------
c                         -1
c
c
c                    PSEUDO CODE
c
c
c     Clear pressure right-hand side
c     Construct the pressure right-hand side
c     If ( compressible flow ) then
c        Update right-hand side
c     If ( iterate pressure correction) then
c        Update right-hand side with contributions of previous pressure
c        corrections.
c        currently only implemented for mcomupw =0 ( no density bias)
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c

c     --- Clear pressure right-hand side

      call issetcon ( rhside, (ni+1)*(nj+1), 0d0 )
      call issetcon ( intp, 2*(ni+1+2*nvirtual)*(nj+1+2*nvirtual),
     +                0d0 )

c     --- Construct the pressure right-hand side
c         If collocated, first, interpolation at the faces,
c                    then, construction of the contravariant V*,
c                    finally, treatment at the boundaries

      if ( mshtyp.eq.1 ) then

c     --- Interpolation of V* which is Cartesian

         do j = 1-nvirtual, nj+1+nvirtual
            do i = 1-nvirtual, ni+1+nvirtual

               vstartmp1(i,j) = vstar(i,j,1)
               vstartmp2(i,j) = vstar(i,j,2)

            end do
         end do

         acttyp = 0
         vartyp = 3
         alphad = 1
         pnttyp = 4 - alphad

         call isintcf2 ( vstartmp1, vstar1,
     +                   ni, nj, nvirtual, vartyp,
     +                   acttyp, pnttyp, mshtyp, coor )
         call isintcf2 ( vstartmp2, vstar2,
     +                   ni, nj, nvirtual, vartyp,
     +                   acttyp, pnttyp, mshtyp, coor )

         if ( idiscret.eq.4 ) then
            do jp = 1, nj
               do ip = 2, ni

                  uloc = bicof(ip,jp,1,1)
                  vloc = bicof(ip,jp,2,1)
                  i = ibicof(ip,jp,1,1)
                  j = ibicof(ip,jp,2,1)

                  vstar1(ip,jp)= uloc *     vloc  * vstartmp1(i  ,j  )+
     +                           uloc *(1d0-vloc) * vstartmp1(i  ,j-1)+
     +                      (1d0-uloc)*     vloc  * vstartmp1(i-1,j  )+
     +                      (1d0-uloc)*(1d0-vloc) * vstartmp1(i-1,j-1)

                  vstar2(ip,jp)= uloc *     vloc  * vstartmp2(i  ,j  )+
     +                           uloc *(1d0-vloc) * vstartmp2(i  ,j-1)+
     +                      (1d0-uloc)*     vloc  * vstartmp2(i-1,j  )+
     +                      (1d0-uloc)*(1d0-vloc) * vstartmp2(i-1,j-1)

               end do
            end do
         end if

         call isconc2d ( vstartmp1, vstar1, vstar2,
     +                   jcta2, ni, nj, nvirtual, alphad )

         do j = 1-nvirtual, nj+1+nvirtual
            do i = 1-nvirtual, ni+1+nvirtual

               vstar1(i,j) = vstar(i,j,1)
               vstar2(i,j) = vstar(i,j,2)
               vstar(i,j,1) = vstartmp1(i,j)

            end do
         end do

         alphad = 2
         pnttyp = 4 - alphad

         call isintcf2 ( vstar1, vstartmp1,
     +                   ni, nj, nvirtual, vartyp,
     +                   acttyp, pnttyp, mshtyp, coor )
         call isintcf2 ( vstar2, vstartmp2,
     +                   ni, nj, nvirtual, vartyp,
     +                   acttyp, pnttyp, mshtyp, coor )

         if ( idiscret.eq.4 ) then
            do jp = 2, nj
               do ip = 1, ni

                  uloc = bicof(ip,jp,1,2)
                  vloc = bicof(ip,jp,2,2)
                  i = ibicof(ip,jp,1,2)
                  j = ibicof(ip,jp,2,2)

                  vstartmp1(ip,jp)= uloc *     vloc  * vstar1(i  ,j  )+
     +                              uloc *(1d0-vloc) * vstar1(i  ,j-1)+
     +                         (1d0-uloc)*     vloc  * vstar1(i-1,j  )+
     +                         (1d0-uloc)*(1d0-vloc) * vstar1(i-1,j-1)

                  vstartmp2(ip,jp)= uloc *     vloc  * vstar2(i  ,j  )+
     +                              uloc *(1d0-vloc) * vstar2(i  ,j-1)+
     +                         (1d0-uloc)*     vloc  * vstar2(i-1,j  )+
     +                         (1d0-uloc)*(1d0-vloc) * vstar2(i-1,j-1)

               end do
            end do
         end if

         call isconc2d ( vstar2, vstartmp1, vstartmp2,
     +                   jcta1, ni, nj, nvirtual, alphad )

         do j = 1-nvirtual, nj+1+nvirtual
            do i = 1-nvirtual, ni+1+nvirtual

               vstar(i,j,2) = vstar2(i,j)

            end do
         end do
      end if

      mcomupw    = iinput(iinput(9)+1)
      mprcor     = iinput(iinput(9)+5)
      ichoicedb  = iinput(iinput(9)+7 - 1)

c     --- ichoicedb  = 3

      if ( ichoicedb.ge.4) then
         mcomsup = 1
      else
         mcomsup = 0
      end if

      if ( mprcor.eq.0 ) then

c     --- mprcor = 0, standard pressure correction

        if ( mcomupw.eq.0.or.mcomsup.eq.1 ) then

c       --- mcomupw = 0, central scheme for pressure correction or
c            mcomsup = 1, supersonic scheme for pressure correction

           if ( axisym.eq.1 ) then

c          --- Axi-symmetric coordinates, central scheme for pressure correction

              do j = 1, nj
                 do i = 1, ni

                       rhside(i,j) = -vstar(i+1,j,1) * 5d-1 *
     +                                (coor(i+1,j,1)+coor(i+1,j+1,1))
     +                                +vstar(i,j,1) *  5d-1 *
     +                                (coor(i,j,1)+coor(i,j+1,1))
     +                                -vstar(i,j+1,2) *  5d-1 *
     +                                (coor(i,j+1,1)+coor(i+1,j+1,1))
     +                                +vstar(i,j,2) *  5d-1 *
     +                                (coor(i,j,1)+coor(i+1,j,1))

                 end do
              end do

           else if ( mcomsup.eq.1 ) then

c          --- mcomsup = 1, supersonic scheme for pressure correction

              do j = 1, nj
                 do i = 1, ni

                    rhside(i,j) =
     +                 -(1d0-switch(i+1,j  ,1))*shiftr(i+1,j  ,1)*
     +                                               vstar(i+1,j  ,1)
     +                 +(1d0-switch(i  ,j  ,1))*shiftr(i  ,j  ,1)*
     +                                               vstar(i  ,j  ,1)
     +                 -(1d0-switch(i  ,j+1,2))*shiftr(i  ,j+1,2)*
     +                                               vstar(i  ,j+1,2)
     +                 +(1d0-switch(i  ,j  ,2))*shiftr(i  ,j  ,2)*
     +                                               vstar(i  ,j  ,2)
     +                 -(switch(i+1,j  ,1))*shiftr(i+1,j  ,1)*
     +                                               eddy(i+1,j  ,3)
     +                 +(switch(i  ,j  ,1))*shiftr(i  ,j  ,1)*
     +                                               eddy(i  ,j  ,3)
     +                 -(switch(i  ,j+1,2))*shiftr(i  ,j+1,2)*
     +                                               eddy(i  ,j+1,4)
     +                 +(switch(i  ,j  ,2))*shiftr(i  ,j  ,2)*
     +                                               eddy(i  ,j  ,4)

                 end do
              end do

           else

c          --- Cartesian coordinates, central scheme for pressure correction

              do j = 1, nj
                 do i = 1, ni

                    rhside(i,j) = -vstar(i+1,j,1)+vstar(i,j,1)
     +                            -vstar(i,j+1,2)+vstar(i,j,2)

                 end do
              end do

           end if

        else

c       --- mcomupw = 1, upwind scheme for pressure correction

          do j = 1, nj
            do i = 1, ni

              rhside(i,j) = -shiftr(i+1,j,1)*vstar(i+1,j,1)+
     +                       shiftr(i,j,1)*vstar(i,j,1)
     +                      -shiftr(i,j+1,2)*vstar(i,j+1,2)
     +                      +shiftr(i,j,2)*vstar(i,j,2)

            end do
          end do

        end if

      else if ( mprcor.eq.1 ) then

c     --- mprcor = 1, special pressure correction as described in the thesis
c                     of Hester Bijl.
c                     Only possible in the absence of diffusion.
c                     Furthermore restricted to perfect gas and with negligence
c                     of kinetic energy in the enthalpy equation
c                     h must be used as variable

         vartyp = 3

c        --- interpolate hnew to V-1 integration points

         pnttyp = 3
         acttyp = 0
         call isintcf2 ( solut(1-nvirtual,1-nvirtual,4,2),
     +                   intp(1-nvirtual,1-nvirtual,1), ni, nj,
     +                   nvirtual, vartyp, acttyp, pnttyp, 0,dummy )

c        --- interpolate hnew to V-2 integration points

         pnttyp = 2
         acttyp = 0
         call isintcf2 ( solut(1-nvirtual,1-nvirtual,4,2),
     +                   intp(1-nvirtual,1-nvirtual,2), ni, nj,
     +                   nvirtual, vartyp, acttyp, pnttyp, 0, dummy )

         do j = 1, nj
            do i = 1, ni
               rhside(i,j) = -vstar(i+1,j,1)*intp(i+1,j,1)+vstar(i,j,1)*
     +                       intp(i,j,1)-vstar(i,j+1,2)*intp(i,j+1,2)
     +                       +vstar(i,j,2)*intp(i,j,2)
            end do
         end do

      end if

      mcav     = iinput(77)
      mcom     = iinput(iinput(9))
      mequat   = iinput(iinput(9)+2)
      mscal    = iinput(iinput(9)+4)
      iprincom = iinput(34)-1
      iprincav = iinput(35)

      if ( mcom.eq.1 ) then

c     --- Compressible flow
c         Update right-hand side by drho/dh dh/dt
c         Cavitating flow, compressible but do not update right-hand side

         dt    = rinput(iinput(29)-1+4)
         gamcm = rinput(iprincom+1)
         mach2 = rinput(iprincom+2)**2d0
         deltatin=1d0/dt

         cons  = 1d0 + ((gamcm - 1d0)*mach2)/2d0
         cons1 = (gamcm * mach2) / cons

         if ( mequat.eq.0 ) then

c        --- standard equation

            if ( mcav.eq.0 ) then

c           --- no cavity

               if ( mprcor.eq.0 ) then

c              --- standard pressure correction

                  if ( mcomupw.eq.0 ) then

c                 --- central scheme

                     if ( axisym.eq.1 ) then

c                    --- Axi-symmetric case

                        do j = 1, nj
                           do i = 1, ni
                              rhside(i,j) = rhside(i,j) +
     +                           (solut(i,j,4,2) - solut(i,j,4,1)) *
     +                           deltatin *jac(i,j) * drhodh(i,j) *
     +                           25d-2*( coor(i,j,1) + coor(i+1,j,1)+
     +                           coor(i,j+1,1) + coor(i+1,j+1,1) )
     +

                           end do
                        end do

                     else

c                    --- Cartesian case

                        do j = 1, nj
                           do i = 1, ni
                              rhside(i,j) = rhside(i,j) +
     +                           (solut(i,j,4,2) - solut(i,j,4,1)) *
     +                           deltatin *jac(i,j) * drhodh(i,j)
                           end do
                        end do

                     end if

                  else if ( mcomupw.eq.1 ) then

c                 --- uwpind scheme

                     do j = 1, nj
                        do i = 1, ni
                           rhside(i,j) = rhside(i,j) + deltatin *(
     +                        (solut(i,j,4,2) - solut(i,j,4,1) )*
     +                         jac(i,j) * drhodh(i,j)*
     +                         ( 1d0 + uold(i+1,j,1)*
     +                           (.5d0*(1d0-shiftr(i+1,j,1))+
     +                            switch(i+1,j,1)) - uold(i,j,1)*(
     +                            .5d0*(1d0- shiftr(i,j,1))-
     +                            switch(i,j,1)) )-
     +                        (solut(i-1,j,4,2) - solut(i-1,j,4,1) )*
     +                         jac(i-1,j) * drhodh(i-1,j)*uold(i,j,1)*
     +                         (.5d0*(1d0-shiftr(i,j,1))+switch(i,j,1))+
     +                        (solut(i+1,j,4,2) - solut(i+1,j,4,1) )*
     +                         jac(i+1,j) * drhodh(i+1,j)*uold(i+1,j,1)*
     +                        (.5d0*(1d0-shiftr(i+1,j,1))-
     +                          switch(i+1,j,1)) )

                        end do
                     end do

                  end if

               else if ( mprcor.eq.1 ) then

c              --- Special pressure correction of Hester

                  call issetcon ( intp, 2*(ni+1+2*nvirtual)*
     +                            (nj+1+2*nvirtual), 0d0 )

c                 --- interpolate V-1 to center of cells

                  vartyp = 1
                  pnttyp = 4
                  acttyp = 0
                  call isintcf2 ( uold(1-nvirtual,1-nvirtual,1),
     +                            intp(1-nvirtual,1-nvirtual,1), ni, nj,
     +                            nvirtual, vartyp, acttyp, pnttyp, 0,
     +                            dummy )

c                 --- interpolate V-2 to center of cells

                  vartyp = 2
                  pnttyp = 4
                  acttyp = 0
                  call isintcf2 ( uold(1-nvirtual,1-nvirtual,2),
     +                            intp(1-nvirtual,1-nvirtual,2), ni, nj,
     +                            nvirtual, vartyp, acttyp, pnttyp, 0,
     +                            dummy )

                  if ( mscal.eq.0 ) then

c                 --- dimensionless

                     do j = 1, nj
                        do i = 1, ni

                           if ( intp(i,j,1).gt.0d0 ) then

                              rhside(i,j) = rhside(i,j)*gamcm +
     +                             (gamcm - 1d0)*
     +                                cons1*intp(i,j,1)*
     +                                  (solut(i+1,j,3,1)-
     +                                      solut(i,j,3,1))
                           else

                              rhside(i,j) = rhside(i,j)*gamcm +
     +                             (gamcm - 1d0)*
     +                                cons1*intp(i,j,1)*
     +                                  (solut(i,j,3,1)-
     +                                      solut(i-1,j,3,1))
                           end if

                           if (intp(i,j,2).gt.0d0) then

                              rhside(i,j) = rhside(i,j) +
     +                              (gamcm - 1d0)*
     +                               cons1*intp(i,j,2)*
     +                               (solut(i,j+1,3,1)-solut(i,j,3,1))
                           else

                              rhside(i,j) = rhside(i,j) +
     +                              (gamcm - 1d0)*
     +                               cons1*intp(i,j,2)*
     +                               (solut(i,j,3,1)-solut(i,j-1,3,1))
                           end if

                        end do
                     end do

                  else if ( mscal.eq.1 ) then

c                 --- dimensionfull

                     do j = 1, nj
                        do i = 1, ni

                           if (intp(i,j,1).gt.0d0) then
                              rhside(i,j) = rhside(i,j)*(gamcm-1d0) +
     +                              (gamcm - 1d0)*intp(i,j,1)*
     +                              (solut(i+1,j,3,1)-solut(i,j,3,1))
                           else
                              rhside(i,j) = rhside(i,j)*(gamcm-1d0)+
     +                              (gamcm - 1d0)*intp(i,j,1)*
     +                              (solut(i,j,3,1)-solut(i-1,j,3,1))
                           end if

                           if ( intp(i,j,2).gt.0d0 ) then

                              rhside(i,j) = rhside(i,j) +
     +                              (gamcm - 1d0)*
     +                              intp(i,j,2)*
     +                              (solut(i,j+1,3,1)-solut(i,j,3,1))
                           else

                              rhside(i,j) = rhside(i,j) +
     +                              (gamcm - 1d0)*
     +                              intp(i,j,2)*
     +                              (solut(i,j,3,1)-solut(i,j-1,3,1))
                           end if

                        end do
                     end do

                  end if

               end if

            end if

c           --- if the pressure correction is iterated to convergence
c               add contributions of previous pressure corrections to rhs

            ipiintim=iinput(4)
            iconv=iinput(ipiintim -1 + 17)

            if ( iconv.eq.1 ) then
               if( mcav.eq.1 ) then
                   ipiincav=iinput(11)
                   istartup=iinput(ipiincav -1 + 2)
               end if

               if( mscal.eq.0 ) then
                  do i = 1, ni
                     do j = 1, nj
                        rhside(i,j) = rhside(i,j) -jac(i,j)*deltatin*
     +                  (isstdls(solut(i,j,3,1),rinput(iprincom+1),
     +                                  istartup, rinput(iprincav))-
     +                  eddy(i,j,1))
                     end do
                  end do
               else if ( mscal.eq.1 ) then
                  do i = 1, ni
                     do j = 1, nj
                        rhside(i,j) = rhside(i,j) -jac(i,j)*deltatin*
     +                      (isstdfl(solut(i,j,3,1),
     +                                  istartup, rinput(iprincav))-
     +                  eddy(i,j,1))
                     end do
                  end do
               end if

            end if

         else if ( mequat.eq.1 ) then

c        --- two-phase gas flow

               do j = 1, nj
                  do i = 1, ni
                     rhside(i,j) = rhside(i,j) +
     +                  (solut(i,j,4,2) - solut(i,j,4,1) ) *
     +                   deltatin * massfrac(i,j)*jac(i,j) * drhodh(i,j)
                  end do
               end do

         end if

      end if

      end
