      subroutine isprod3(work    , solut , coor, coef, eddy, invjac  ,
     +                   lwork   , ndegfd, ni  , nj  , nk  , npointsv,
     +                   idiscret, nvirtual, rintur, maniso            )
c ======================================================================
c
c        programmer     Marcel Zijlema
c        version 3.0    date   05-09-1996 Marcel: nonlinear effects
c        version 2.1    date   21-08-1996 Marcel: velocities should also be
c                                         interpolated in cell center
c        version 2.0    date   22-05-1996 Marcel: Kato-Launder modification
c        version 1.1    date   25-01-1996 bug fixed
c        version 1.0    date   08-12-1995
c
c   copyright (c) 1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Compute production of turbulent kinetic energy in R3
c
c **********************************************************************
c
c                       KEYWORDS
c
c     3d
c     turbulence
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer idiscret, lwork, ndegfd, ni, nj, nk, npointsv, nvirtual,
     +        maniso
      double precision coor(*), coef(1:npointsv), eddy(1:npointsv),
     +                 invjac(1:npointsv,1:8), work(1:lwork),
     +                 solut(1:npointsv,1:ndegfd,1:2), rintur(*)

c     coef      i    A specific coefficient
c     coor      i    reserved
c     eddy      i    reserved
c     idiscret  i    Type of discretization
c     invjac    i    1/sqrt(g) in pnttyp
c     lwork     i    Length of array work
c     maniso    i    Indicates type of nonlinear turbulence (anisotropic)
c                    modelling
c     ndegfd    i    Number of degrees of freedom
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     npointsv  i    Number of points in the mesh with virtual points
c     nvirtual  i    Number of rows of virtual cells
c     rintur    i    reserved
c     solut     i    reserved
c     work      i    Work array
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, idir, ipchsym, ipcntdef, ipcntrot, ipcva, ipjcta,
     +        ipjctg, ippderiv, ipprod, ipveloc, iphightens, jdir,
     +        nwork, pnttyp, vcellc, alphad, betad
      double precision mags, magw, ksign, modprod, ctau1, ctau2, ctau3
      integer rspv(1:3,1:8,0:3,0:4,1:8), npf(0:3,0:4,1:8)
      double precision cv(1:8,0:3,0:4,1:8)

c     alphad    Direction index of integration.
c               Hence if a V1-cell is used, alphad = 1 and so on
c     betad     Direction index, usually the direction perpendicular to the
c               integration direction of the boundary integral.
c               For example betad in local point (1,0) is 1 and in (0,1): 2
c     ctau1     Parameter ctau1 for nonlinear two-equation model
c     ctau2     Parameter ctau2 for nonlinear two-equation model
c     ctau3     Parameter ctau3 for nonlinear two-equation model
c     cv        reserved
c     i         General loop variable
c     idir      General loop variable for 1...ndim loop
c     ipchsym   Starting address of array CHSYM in array WORK
c     ipcntdef  Starting address of array CNTDEF in array WORK
c     ipcntrot  Starting address of array CNTROT in array WORK
c     ipcva     Starting address of array CVA in array WORK
c     iphightens Starting address of array HIGHTENS in array WORK
c     ipjcta    Starting address of array JCTA in array WORK
c     ipjctg    Starting address of array JCTG in array WORK
c     ippderiv  Starting address of array PDERIV in array WORK
c     ipprod    Starting address of array PROD in array WORK
c     ipveloc   Starting address of array VELOC in array WORK
c     jdir      General loop variable for 1...ndim loop
c     ksign     Help parameter to define 1 or -1
c     mags      Magnitude of strain rate
c     magw      Magnitude of rotation
c     modprod   Parameter modprod for production of k in
c               k-eps/k-omega models
c     npf       reserved
c     nwork     Second dimension of array work
c     pnttyp    reserved
c     rspv      reserved
c     vcellc    Point type of integration cell centre for a given
c               variable type
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISCHRYS3  Fill chrystoffel symbols in one integration point in R3
c     ISCOVA3D  Fill covariant base vectors in all integration points in R3
c     ISDOUB3D  Compute the double contraction of deformation rate tensor
c               or rotation tensor in R3
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRCHR  Puts values of strings in character error array
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c     ISINTCF3  Compute interpolation of a coefficient in an integration point
c     ISJCTA3D  Fill Jacobian times contravariant base vectors
c               in one integration point in R3
c     ISJCTG3D  Fill Jacobian times contravariant metric tensors
c               in one integration point in R3
c     ISPNDF    Make tables for interpolation and finite difference formulas
c     ISQDTN13  Compute nonlinear effects of the production rate of turbulent
c               energy in R3
c     ISSETCON  Make array equal to a constant
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c      17   internal error: array size too large
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call iseropen('isprod3')

      modprod = rintur(20)

c-----------------TEMPORARY-----------------------------------------------
      call ispndf(npf,rspv,cv,3)

c-----------------TEMPORARY-----------------------------------------------

c     --- allocate local arrays

      ipprod     = 1
      ipcntdef   = ipprod     + npointsv
      ipcntrot   = ipcntdef   + npointsv
      ipcva      = ipcntrot   + npointsv
      ipjcta     = ipcva      + 72*npointsv
      ipjctg     = ipjcta     + 9*npointsv
      ipchsym    = ipjctg     + 6*npointsv
      ippderiv   = ipchsym    + 18*npointsv
      ipveloc    = ippderiv   + 9*npointsv
      iphightens = ipveloc    + 12*npointsv
      nwork      = iphightens + npointsv

      if ( nwork.gt.lwork ) then
        call iserrchr( 'work',1 )
        call iserrint( nwork, 1 )
        call iserrint( lwork, 2 )
        call iserrsub ( 17, 2, 0, 1 )
      end if

c     --- initialize the work array's

      do i = 1, nwork
        work( i ) = 0d0
      end do

c     --- Fill covariant basis vectors in cva for all integration points

      call issetcon ( work(ipcva), 72*npointsv, 0d0 )
      call iscova3d ( coor, work(ipcva), ni, nj, nk, nvirtual, npf,
     +                rspv, cv )

c     --- Determine point type corresponding to the centre of a
c         pressure cell in which the contraction of deformation
c         rate tensor must be computed

      pnttyp = 8

c     --- Compute contravariant basis vectors, metric tensors
c         and Christoffel symbols

      call isjcta3d ( work(ipcva+9*npointsv*(pnttyp-1)),
     +                work(ipjcta), ni, nj, nk, nvirtual )

      call isjctg3d ( invjac(1,pnttyp), work(ipjcta),
     +                work(ipjctg), ni, nj, nk, nvirtual )

      call ischrys3 ( work(ipcva), work(ipjcta), work(ipchsym),
     +                invjac(1,pnttyp), ni, nj, nk, nvirtual,
     +                pnttyp )

c                                        alpha
c     --- Compute partial derivative of V      in direction beta in
c         point of type pnttyp

      do alphad = 1, 3
        do betad = 1, 3
          call isintcf3( solut(1,alphad,2),
     +                   work(ippderiv+npointsv*(alphad-1+3*(betad-1))),
     +                   ni, nj, nk, nvirtual, npf, rspv, cv, betad,
     +                   alphad, pnttyp )
        end do
      end do

c     --- Interpolate velocities to centre of cell faces and pnttyp

      do jdir = 1, 3
        vcellc = 8 - 2**(jdir-1)
        do idir = 1, 3
          call isintcf3( solut(1,idir,2),
     +                   work(ipveloc+npointsv*(idir-1+3*(jdir-1))),
     +                   ni, nj, nk, nvirtual, npf, rspv, cv, 0, idir,
     +                   vcellc )
        end do
        call isintcf3( solut(1,jdir,2), work(ipveloc+npointsv*(jdir+8)),
     +                 ni, nj, nk, nvirtual, npf, rspv, cv, 0, jdir,
     +                 pnttyp )
      end do

c     --- Determine the double contraction of deformation rate tensor

      ksign = 1
      call isdoub3d(work(ipcntdef), work(ippderiv), work(ipveloc),
     +              solut, work(ipcva), invjac, work(ipjcta),
     +              work(ipjctg), work(ipchsym), ni, nj, nk, ndegfd,
     +              nvirtual, idiscret, pnttyp, ksign)

      if ( modprod.ne.0 ) then

c     --- weighted average of standard production term and production term
c         with Kato-Launder correction

c         --- Determine the double contraction of rotation tensor

        ksign = -1
        call isdoub3d(work(ipcntrot), work(ippderiv), work(ipveloc),
     +                solut, work(ipcva), invjac, work(ipjcta),
     +                work(ipjctg), work(ipchsym), ni, nj, nk, ndegfd,
     +                nvirtual, idiscret, pnttyp, ksign)

      end if

c     --- Compute the production of turbulent kinetic energy with or
c         without Kato-Launder modification

      do i = 1, npointsv

        mags = sqrt(2d0*work(i+ipcntdef-1))
        magw = sqrt(2d0*work(i+ipcntrot-1))
        work(i+ipprod-1) = coef(i)*eddy(i)*mags*( (1-modprod)*mags +
     +                                             modprod*magw )
        work(i+ipcntdef-1) = 0d0

      end do

      if ( maniso.gt.0 ) then

c     --- Add nonlinear effects of stress-strain constitutive relation
c         in case of nonlinear turbulence modeling

        do i = 1, npointsv
          work(i+ipcntdef-1) = 0d0
          work(i+ipcntrot-1) = 0d0
        end do

        ctau1 = rintur(22)
        ctau2 = rintur(23)
        ctau3 = rintur(24)

        call isqdtn13(work(ipcntrot), work(ippderiv),
     +                work(ipveloc+9*npointsv),
     +                work(ipcva+9*npointsv*(pnttyp-1)),
     +                invjac(1,pnttyp), work(ipjctg), work(ipchsym),
     +                work(iphightens), ni, nj, nk, nvirtual, idiscret,
     +                ctau1, ctau2, ctau3 )

        do i = 1, npointsv
          work(i+ipcntdef-1)= -coef(i)*(eddy(i)**2)*work(i+ipcntrot-1)/
     +                         (rintur(3)*solut(i,ndegfd-1,1)
     +                          +1d-10)
        end do

      end if

      call iserclos('isprod3')

      end
