      subroutine istrac3d ( ni, nj, nk, nvirtual, cstar, solut, tsum,
     +                      iparea, locdx )
c ======================================================================
c
c        programmer     Marcel Zijlema
c        version 1.0    date   01-05-1996
c
c
c   copyright (c) 1995-1996  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Compute contribution of convection term in R3 for the transport
c    equations
c    one integration point
c **********************************************************************
c
c                       KEYWORDS
c
c     coefficients
c     discretization
c     transport_matrix
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nk, nvirtual, iparea(2,3), locdx
      double precision cstar(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1-nvirtual:nk+1+nvirtual),
     +                 solut(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1-nvirtual:nk+1+nvirtual),
     +                 tsum(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual)

c     cstar     i    The capacity in a given integration point
c     iparea    i    Area of integration points where a contribution to
c                    the matrix is to be calculated
c     locdx     i    Position of the integration point with respect to
c                    the centre of the integration cell
c                    The cell sizes are assumed to be 2 times 2, hence the
c                    distance of a midside point to a cell centre is always 1
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     solut     i    reserved
c     tsum      o    Flux contributions to the matrix
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ip, jp, kp

c     ip        Local counting variable i-direction
c     jp        Local counting variable j-direction
c     kp        Local counting variable k-direction
c **********************************************************************
c
c                       I/O
c
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c      This subroutine may only be called for centres of integration cell
c      sides.
c      The convective terms in both classical and WesBeek discretizations
c      are given by
c
c          *  alphad   |+e(alphad)/2
c         C  V       T |
c                      |-e(alphad)/2
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- Calculate the coefficient in point e(alphad)/2

      do kp = iparea(1,3), iparea(2,3)
        do jp = iparea(1,2), iparea(2,2)
          do ip = iparea(1,1), iparea(2,1)
            tsum(ip,jp,kp) = locdx*cstar(ip,jp,kp)*solut(ip,jp,kp)
          end do
        end do
      end do

      end
