      subroutine istraf2d ( ni, nj, nvirtual, mct, matrix, rhside,
     +                      ibinfor, ibipoint, bndcon )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.2    date 06-05-1996 Marcel: small correction
c        version 1.1    date 20-03-1996 Extension for btype=0
c        version 1.0    date 07-11-1995
c
c   copyright (c) 1995-1996  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                         DESCRIPTION
c
c    All parts of the matrix outside the region are folded to internal
c    domain for the transport matrix in R2
c    The folding process depends on the types of boundary conditions
c **********************************************************************
c
c                      KEYWORDS
c
c **********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, ibipoint(*), ibinfor(*),
     +        mct(-2:2,-2:2,-2:2)
      double precision rhside(ni+1,nj+1),
     +                 matrix(ni+1,nj+1,*), bndcon(2,*)

c     bndcon    i    reserved
c     ibinfor   i    Third part of infbnd
c     ibipoint  i    ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     matrix   i/o   reserved
c     mct       i    Integer matrix information
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     rhside   i/o   reserved
c **********************************************************************
c
c                    COMMON BLOCKS
c
c **********************************************************************
c
c                    LOCAL PARAMETERS
c
      integer diri, robb, dtur
      parameter( diri = 1, robb=2, dtur=3 )

      logical multbn1, multbn2
      integer btype1, btype2, ipibinf, ipoint, tsign, i, btype,
     +        iconcto, iconctc, iconcti, ip, jp, xp(2), ndir, k,
     +        tdir, nsign, iside, iparea(2,2)
      double precision fact

c     btype     Type of boundary condition
c     btype1    Type of boundary condition at one of the boundaries in vertex
c     btype2    Type of boundary condition at one of the boundaries in vertex
c     diri      Dirichlet boundary condition, i.e. T is given
c     dtur      Dirichlet condition for turbulence, i.e. T
c               given at the centroids of boundary cells
c     fact      Multiplication factor
c     i         General loop variable
c     iconctc   Connection in matrix for central point
c     iconcti   Connection in matrix for internal point
c     iconcto   Connection in matrix for outside point
c     ip        Local counting variable i-direction
c     iparea    Area of integration points where a contribution to
c               the matrix is to be calculated
c     ipibinf   Starting address of array infor for present side minus one
c     ipoint    Address in array IBUFFR
c     iside     Sequence number of side
c     jp        Local counting variable j-direction
c     k         General loop variable
c     multbn1   If true the boundary is periodical or multiblock
c     multbn2   If true the boundary is periodical or multiblock
c     ndir      Direction of normal vector on this side
c     nsign     Inner product of outward normal in computational domain
c               with the base vector perpendicular to the face
c     robb      Robbins boundary condition, i.e. aT + (k grad T).n given
c                                                       T
c     tdir      Direction of tangent vector on this side
c     tsign     Inner product of tangential vector defined counter
c               clockwise with respect to the outward normal and the
c               base vector along the side
c     xp        Auxiliary array to calculate boundary point number
c **********************************************************************
c
c             SUBROUTINES CALLED
c
c     ISORIENT2 Computes ndir, nsign, tdir, tsign for a 2-dimensional
c               problem
c **********************************************************************
c
c                              I/O
c
c **********************************************************************
c
c                    ERROR MESSAGES
c
c **********************************************************************
c
c                    METHOD
c
c    The folding process will be carried out per side.
c    Some rows corresponding to unknowns in the neighbourhood of the rectangle
c    require special attention
c
c    For illustration purposes we limit ourselves to the lower side and
c    in case of a corner to the combination of lower side and left-hand side
c
c            -------------------------------------
c            |           |           |           |
c            |     +     |     +     |     +     |
c            |           |           |           |
c            ------------------|------------------
c            |           |           |           |
c            |     +     -     +     -     +     |
c            |           |           |           |
c            ------------------|-------------------------- lower boundary
c
c                  +           +           +
c
c
c
c    The folding process is as follows:
c
c    a. Periodical and multiblock boundaries: no action should be taken
c    b. T given at the boundary (.,-1):
c         T(.,-2) = 2T(.,-1) - T(.,0)                         (3)
c    c. T not given:
c         T(.,-2) = 2T(.,0) - T(.,2)                          (4)
c
c
c    Finally we consider a cell at a corner
c
c                        -------------------------
c                        |           |           |
c                  +     |     +     |     +     |
c                        |           |           |
c                        ------|------------------
c                        |           |           |
c                  +     -     +     -     +     |
c                        |           |           |
c                        ------|-------------------------- lower boundary
c
c                  +           +           +
c
c    The folding process is as follows:
c
c    a. If point (-2,-2) does not correspond to periodical or multiblock bc's
c       - T(-2,-2) = ( 2T(0,-2)-T(2,-2) + 2T(-2,0)-T(-2,2) )/2   (8)
c    b. All other points are eliminated at their respective boundaries
c
c
c                    PSEUDO CODE
c
c      For iside := 1 (1) 4 do
c         For 2 corner normal cells do
c            eliminate the point "(-2,-2)" if necessary
c      For iside := 1 (1) 4 do
c         For all boundary cells do
c            Perform the folding process over all outside connections
c **********************************************************************
c
c                    DATA STATEMENTS
c
c ======================================================================
c
c     --- First the 4 virtual vertex unknowns are eliminated if
c         necessary
c         Vertex (1,1)

      btype1 = ibinfor(ibipoint(1)+nvirtual)
      btype2 = ibinfor(ibipoint(3)+nvirtual)
      multbn1 = btype1.ge.95 .and. btype1.le.99
      multbn2 = btype2.ge.95 .and. btype2.le.99
      if ( btype1.eq.99 .and. btype2.eq.99 ) then

c     --- Both sides are multiblock, treat as eliminate all

         multbn1 = .false.
         multbn2 = .false.

      end if
      fact = matrix(1,1,2)
      if ( .not. multbn1 ) then
         if ( .not. multbn2 ) then

c        --- Standard situation, eliminate -1,-1 in two directions

            matrix(1,1,3) = matrix(1,1,3)+fact
            matrix(1,1,4) = matrix(1,1,4)-0.5d0*fact
            matrix(1,1,5) = matrix(1,1,5)+fact
            matrix(1,1,7) = matrix(1,1,7)-0.5d0*fact
            matrix(1,1,2) = 0d0

         else

c        --- Boundary 3 periodic, eliminate -1,-1 in direction 3

            matrix(1,1,3) = matrix(1,1,3)+2d0*fact
            matrix(1,1,4) = matrix(1,1,4)-fact
            matrix(1,1,2) = 0d0

         end if

      else if ( .not. multbn2 ) then

c     --- Boundary 1 periodic, eliminate -1,-1 in direction 1

         matrix(1,1,5) = matrix(1,1,5)+2d0*fact
         matrix(1,1,7) = matrix(1,1,7)-fact
         matrix(1,1,2) = 0d0

      end if

c     --- Vertex (1,ni)

      btype1 = ibinfor(ibipoint(2)+nvirtual)
      btype2 = ibinfor(ibipoint(3)+nvirtual+2*ni)
      multbn1 = btype1.ge.95 .and. btype1.le.99
      multbn2 = btype2.ge.95 .and. btype2.le.99
      if ( btype1.eq.99 .and. btype2.eq.99 ) then

c     --- Both sides are multiblock, treat as eliminate all

         multbn1 = .false.
         multbn2 = .false.

      end if
      fact = matrix(ni,1,4)
      if ( .not. multbn1 ) then
         if ( .not. multbn2 ) then

c        --- Standard situation, eliminate -1,-1 in two directions

            matrix(ni,1,3) = matrix(ni,1,3)+fact
            matrix(ni,1,2) = matrix(ni,1,2)-0.5d0*fact
            matrix(ni,1,6) = matrix(ni,1,6)+fact
            matrix(ni,1,9) = matrix(ni,1,9)-0.5d0*fact
            matrix(ni,1,4) = 0d0

         else

c        --- Boundary 3 periodic, eliminate -1,-1 in direction 3

            matrix(ni,1,3) = matrix(ni,1,3)+2d0*fact
            matrix(ni,1,2) = matrix(ni,1,2)-fact
            matrix(ni,1,4) = 0d0

         end if

      else if ( .not. multbn2 ) then

c     --- Boundary 4 periodic, eliminate -1,-1 in direction 2

         matrix(ni,1,6) = matrix(ni,1,6)+2d0*fact
         matrix(ni,1,9) = matrix(ni,1,9)-fact
         matrix(ni,1,4) = 0d0

      end if

c     --- Vertex (1,nj)

      btype1 = ibinfor(ibipoint(1)+nvirtual+2*nj)
      btype2 = ibinfor(ibipoint(4)+nvirtual)
      multbn1 = btype1.ge.95 .and. btype1.le.99
      multbn2 = btype2.ge.95 .and. btype2.le.99
      if ( btype1.eq.99 .and. btype2.eq.99 ) then

c     --- Both sides are multiblock, treat as eliminate all

         multbn1 = .false.
         multbn2 = .false.

      end if
      fact = matrix(1,nj,7)
      if ( .not. multbn1 ) then
         if ( .not. multbn2 ) then

c        --- Standard situation, eliminate -1,-1 in two directions

            matrix(1,nj,8) = matrix(1,nj,8)+fact
            matrix(1,nj,9) = matrix(1,nj,9)-0.5d0*fact
            matrix(1,nj,5) = matrix(1,nj,5)+fact
            matrix(1,nj,2) = matrix(1,nj,2)-0.5d0*fact
            matrix(1,nj,7) = 0d0

         else

c        --- Boundary 4 periodic, eliminate -1,-1 in direction 4

            matrix(1,nj,8) = matrix(1,nj,8)+2d0*fact
            matrix(1,nj,9) = matrix(1,nj,9)-fact
            matrix(1,nj,7) = 0d0

         end if

      else if ( .not. multbn2 ) then

c     --- Boundary 1 periodic, eliminate -1,-1 in direction 1

         matrix(1,nj,5) = matrix(1,nj,5)+2d0*fact
         matrix(1,nj,2) = matrix(1,nj,2)-fact
         matrix(1,nj,7) = 0d0

      end if

c     --- Vertex (ni,nj)

      btype1 = ibinfor(ibipoint(2)+nvirtual+2*nj)
      btype2 = ibinfor(ibipoint(4)+nvirtual+2*ni)
      multbn1 = btype1.ge.95 .and. btype1.le.99
      multbn2 = btype2.ge.95 .and. btype2.le.99
      if ( btype1.eq.99 .and. btype2.eq.99 ) then

c     --- Both sides are multiblock, treat as eliminate all

         multbn1 = .false.
         multbn2 = .false.

      end if
      fact = matrix(ni,nj,9)
      if ( .not. multbn1 ) then
         if ( .not. multbn2 ) then

c        --- Standard situation, eliminate -1,-1 in two directions

            matrix(ni,nj,8) = matrix(ni,nj,8)+fact
            matrix(ni,nj,7) = matrix(ni,nj,7)-0.5d0*fact
            matrix(ni,nj,6) = matrix(ni,nj,6)+fact
            matrix(ni,nj,4) = matrix(ni,nj,4)-0.5d0*fact
            matrix(ni,nj,9) = 0d0

         else

c        --- Boundary 4 periodic, eliminate -1,-1 in direction 4

            matrix(ni,nj,8) = matrix(ni,nj,8)+2d0*fact
            matrix(ni,nj,7) = matrix(ni,nj,7)-fact
            matrix(ni,nj,9) = 0d0

         end if

      else if ( .not. multbn2 ) then

c     --- Boundary 2 periodic, eliminate -1,-1 in direction 2

         matrix(ni,nj,6) = matrix(ni,nj,6)+2d0*fact
         matrix(ni,nj,4) = matrix(ni,nj,4)-fact
         matrix(ni,nj,9) = 0d0

      end if

c     --- loop over faces

      do iside = 1, 4

c     --- Compute tdir and ndir

         call isorient2 ( iside, tdir, ndir, tsign, nsign )

         ipibinf = ibipoint(iside)+nvirtual-1

c        --- loop over cells
c            Compute loop bounds

         iparea(1,1) = 1
         iparea(1,2) = 1
         iparea(2,1) = 1
         iparea(2,2) = 1

         if ( iside.le.2 ) then

c        --- iside = 1, 2:  left- and right-hand side

            iparea(2,2) = nj
            if ( iside.eq.2 ) then

c           --- iside = 2:  right-hand side

               iparea(1,1) = ni
               iparea(2,1) = ni

            end if

         else

c        --- iside = 3, 4:  lower- and upper side

            iparea(2,1) = ni
            if ( iside.eq.4 ) then

c           --- iside = 4:  upper side

               iparea(1,2) = nj
               iparea(2,2) = nj

            end if

         end if

         do k = -1, 1

            if ( tdir.eq.1 ) then
               iconcto = mct ( k, nsign, 0 )
               iconctc = mct ( k,     0, 0 )
               iconcti = mct ( k,-nsign, 0 )
            else
               iconcto = mct ( nsign, k, 0 )
               iconctc = mct (     0, k, 0 )
               iconcti = mct (-nsign, k, 0 )
            end if

c           --- loop over cells

            do jp = iparea(1,2), iparea(2,2)
               do ip = iparea(1,1), iparea(2,1)
                  xp(1) = ip
                  xp(2) = jp
                  ipoint = ipibinf+2*(xp(tdir)+k)
                  btype = ibinfor(ipoint)
                  fact = matrix(ip,jp,iconcto)
                  if ( btype.eq.diri ) then

c                 --- Dirichlet boundary condition

                     matrix(ip,jp,iconcto) = 0d0
                     matrix(ip,jp,iconctc) = matrix(ip,jp,iconctc) -
     +                                       fact
                     rhside(ip,jp) = rhside(ip,jp) -
     +                               2d0*fact*bndcon(1,ipoint)
                  else if ( btype.eq.robb .or. btype.eq.0 ) then

c                 --- Robbins boundary condition or no one at all

                     matrix(ip,jp,iconcto) = 0d0
                     matrix(ip,jp,iconctc) =
     +                   matrix(ip,jp,iconctc)+2d0*fact
                     matrix(ip,jp,iconcti) =
     +                   matrix(ip,jp,iconcti)-fact
                  else if ( btype.eq.dtur .and. k.eq.0 ) then

c                 --- Fill matrix elements with zero's, except main diagonal
c                     element which is filled with one, solut and rhside with
c                     bndcon when Dirichlet condition for turbulence is given

                     do i = 2, 9
                        matrix(ip,jp,i) = 0d0
                     end do
                     matrix(ip,jp,1) = 1d0
                     rhside(ip,jp) = bndcon(1,ipoint)

                  end if

               end do

            end do

         end do

      end do

      end
