      subroutine isextco2 ( iblock, ni, nj, nvirtual,
     +                      nblocks, nfaces, maxnsubf,
     +                      coor, nsubfaces, subfaces )
c=====================================================================
c
c
c        programmer   Erik Brakkee
c        version 1.0    27-03-95 (Renamed subroutine isexco02)
c
c   copyright (c) 1995-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c***********************************************************************
c
c                  DESCRIPTION
c
c     Initialize those virtual coordinates of the local coordinate array
c     of a certain subdomain that were not already initialized by istrco
c     by extrapolation.
c***********************************************************************
c
c                  KEYWORDS
c
c   mapping_array
c   topology
c***********************************************************************
c
c                  INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iblock, maxnsubf, nblocks, nfaces, ni, nj, nvirtual
      double precision coor(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual, *)
      integer nsubfaces(1:nblocks, 1:nfaces)
      integer subfaces(1:nblocks, 1:nfaces, 1:maxnsubf, *)

c     coor      i    reserved
c     iblock    i    Block sequence number
c     maxnsubf  i    Maximum number of subfaces encountered on all subfaces
c     nblocks   i    Number of blocks
c     nfaces    i    Number of faces
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nsubfaces i    Array containing the number of subfaces of each face,
c                    see array itopol for an extended description
c     nvirtual  i    Number of rows of virtual cells
c     subfaces  i    reserved
c***********************************************************************
c
c                   COMMON BLOCKS
c
      include 'IScommon/cisnas'
c***********************************************************************
c
c     LOCAL PARAMETERS
c
      integer i, iface, ihigh, ilow, isubface, iv, j, k
      double precision w1, w2

c     i         General loop variable
c     iface     Face number
c     ihigh     Upper index of subface
c     ilow      Lower index of subface
c     isubface  Subface number
c     iv        Indicates the number of the virtual row (0..nvirtual-1)
c     j         General loop variable
c     k         General loop variable
c     w1        Weight factor for extrapolation in direction 1 at the
c               faces/edges/vertices
c     w2        Weight factor for extrapolation in direction 2 at the
c               faces/edges/vertices
c**********************************************************************
c
c                   I/O
c
c      none
c
c**********************************************************************
c
c     CALLED SUBROUTINES
c
c     None.
c
c**********************************************************************
c
c                   ERROR MESSAGES :
c
c        -
c
c***********************************************************************
c
c     PSEUDO CODE    :
c
c     For all subfaces isubface of block iblock
c
c        If (subface isubface is external ) then
c
c          Determine index range [ilow,ihigh] along the subface at which
c          to compute virtual coordinates by extrapolation.
c
c          Do i = ilow, ihigh
c
c             extrapolate nvirtual rows of virtual coordinates at
c             index i
c
c          Enddo
c
c        Endif
c
c     Endfor
c=======================================================================
c
c     --- handle external subfaces

      do iface = 1, nfaces
         do isubface = 1, nsubfaces(iblock, iface)
            if ( subfaces(iblock, iface, isubface, 5) .eq. 0 ) then

c           --- determine index range of uninitialized coordinates

               ilow = subfaces(iblock,iface,isubface,1)
               ihigh = subfaces(iblock,iface,isubface,2)+1

c              --- if the previous subface was internal then exclude
c                  index ilow

               if ( isubface .gt. 1 ) then
                  if ( subfaces(iblock,iface,isubface-1,5) .ne. 0 )
     +               ilow = ilow + 1
               endif

c              --- if the next subface is internal then exclude
c                  index ihigh

               if ( isubface .lt. nsubfaces(iblock,iface) ) then
                  if ( subfaces(iblock,iface,isubface+1,5) .ne. 0 )
     +               ihigh = ihigh - 1
               endif

               if ( iface .eq. ileft) then

                  do k = 1, 2
                     do iv = 0, nvirtual-1
                        i = 1 - iv
                        do j = ilow, ihigh
                           coor(i-1,j,k) = 2d0*coor(i,j,k) -
     +                                         coor(i+1,j,k)
                        enddo
                     enddo
                  enddo

               else if ( iface .eq. iright) then

                  do k = 1, 2
                     do iv = 0, nvirtual-1
                        i = ni+1+iv
                        do j = ilow, ihigh
                           coor(i+1,j,k) = 2d0*coor(i,j,k) -
     +                                         coor(i-1,j,k)
                        enddo
                     enddo
                  enddo

               else if ( iface .eq. idown ) then

                  do k = 1, 2
                     do iv = 0, nvirtual-1
                        j = 1-iv
                        do i = ilow, ihigh
                           coor(i,j-1,k) = 2d0*coor(i,j,k) -
     +                                         coor(i,j+1,k)
                        enddo
                     enddo
                   enddo

               else

                  do k = 1, 2
                     do iv = 0, nvirtual-1
                        j = nj+1+iv
                        do i = ilow, ihigh
                           coor(i,j+1,k) = 2d0*coor(i,j,k) -
     +                                         coor(I,j-1,k)
                        enddo
                     enddo
                  enddo

               endif

            endif
         enddo
      enddo

c     --- handle corner points

      do k = 1, 2

c     --- bottom-left corner

         w1 = 5d-1
         if ( subfaces(iblock, idown, 1, 5 ) .ne. 0 ) w1 = w1 + 5d-1
         if ( subfaces(iblock, ileft, 1, 5 ) .ne. 0 ) w1 = w1 - 5d-1
         w2 = 1d0 - w1

         do i = 0, 1-nvirtual, -1
            do j = 0, 1-nvirtual, -1
               coor(i,j,k) =
     +            ( 2d0*coor(i+1,j,k) - coor(i+2,j,k) ) * w1 +
     +            ( 2d0*coor(i,j+1,k) - coor(i,j+2,k) ) * w2
            enddo
         enddo

c        --- bottom-right corner

         w1 = 5d-1
         if ( subfaces(iblock,idown,nsubfaces(iblock,idown), 5) .ne. 0)
     +           w1 = w1 + 5d-1
         if ( subfaces(iblock,iright,1,5) .ne. 0 ) w1 = w1 - 5d-1
         w2 = 1d0 - w1

         do i = ni+2, ni+1+nvirtual
            do j = 0, 1-nvirtual, -1
               coor(i,j,k) =
     +           ( 2d0*coor(i-1,j,k) - coor(i-2,j,k) ) * w1 +
     +           ( 2d0*coor(i,j+1,k) - coor(i,j+2,k) ) * w2
            enddo
         enddo

c        --- top-left corner

         w1 = 5d-1
         if ( subfaces(iblock,iup,1,5) .ne. 0 ) w1 = w1 + 5d-1
         if ( subfaces(iblock,ileft,nsubfaces(iblock,ileft),5) .ne. 0 )
     +           w1 = w1 -5d-1
         w2 = 1d0 - w1

         do i = 0, 1-nvirtual, -1
            do j = nj+2, nj+1+nvirtual
               coor(i,j,k) =
     +            ( 2d0*coor(i+1,j,k) - coor(i+2,j,k) ) * w1 +
     +            ( 2d0*coor(i,j-1,k) - coor(i,j-2,k) ) * w2
            enddo
         enddo

c        --- top-right corner

         w1 = 5d-1
         if ( subfaces(iblock,iup,nsubfaces(iblock,iup),5) .ne. 0 )
     +          w1 = w1 + 5d-1
         if ( subfaces(iblock,iright,nsubfaces(iblock,iright),5)
     +         .ne. 0)
     +          w1 = w1 - 5d-1
         w2 = 1d0 - w1

         do i = ni+2, ni+1+nvirtual
            do j = nj+2, nj+1+nvirtual
               coor(i,j,k) =
     +            ( 2d0*coor(i-1,j,k) - coor(i-2,j,k) ) * w1 +
     +            ( 2d0*coor(i,j-1,k) - coor(i,j-2,k) ) * w2
            enddo
         enddo

      enddo

      end
