      subroutine isfilv ( nblocks, isiinput, iscoor, isitopol )
c ======================================================================
c
c        programmer   Erik Brakkee
c        version 1.4    date 03-04-1997 Kees K.: call of istrco changed
c        version 1.3    date 27-03-1995 Guus: replace isexco by isextco2/3
c        version 1.2    date 30-01-1995 Kees K. integer decl. of isexco removed
c        version 1.1    date 27-07-1994 Michiel Kooper: First steps towards 3D:
c                                       Adaptation of ISEXCO, extra parameter NK
c        version 1.0    date 23-05-1994
c
c   copyright (c) 1994-1997  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Initializes the virtual points of the local coordinate
c     arrays of the subdomains.
c     First, virtual points are taken as much as possible from neighbouring
c     blocks.
c     The remaining virtual coordinates are obtained by extrapolation.
c **********************************************************************
c
c                       KEYWORDS
c
c   mapping_array
c   topology
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer nblocks
      integer iscoor(1:nblocks)
      integer isiinput(1:nblocks)
      integer isitopol

c     iscoor    i    Array of sequence numbers (nblocks) of array coor
c     isiinput  i    Array of sequence numbers (nblocks) of array iinput
c     isitopol  i    Sequence number of array itopol
c     nblocks   i    Number of blocks
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ipcoor, ipiinput, ipitopol, ipnsubf,
     +        ipsubf, isvco, ipvco, lenvco, maxnsubf,
     +        ndim, nfaces, ni, nj, nk, nvirtual

c     ipcoor    Starting address of array coor in buffr
c     ipiinput  Starting address of array iinput in ibuffr
c     ipitopol  Starting address of array itopol in ibuffr
c     ipnsubf   Starting address of the nsubfaces array
c     ipsubf    Starting address of the subfaces array
c     ipvco     Address of an array storing the intermediate representation
c               of virtual coordinates
c     isvco     Number of an array storing the intermediate representation
c               of virtual coordinates
c     lenvco    Length of the vco array
c     maxnsubf  Maximum number of subfaces encountered on all subfaces
c     ndim      Dimension of space (2 or 3)
c     nfaces    Number of faces
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     nvirtual  Number of rows of virtual cells
c **********************************************************************
c
c                       I/O
c
c      none
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isdpgetp, isingetp

c     ISACTIVE  Activate old permanent array
c     ISDCREAT  Create permanent space in array IBUFFR (Double precision array)
c     ISDCRPRI  Decrease priorities of all arrays in ibuffr
c     ISDELARR  Delete a permanent array
c     ISDPGETP  Get the pointer corresponding to a double precision array
c     ISEXTCO2  Compute virtual coordinates not coming from a
c               neighbouring block by extrapolation (2d case)
c     ISEXTCO3  Compute virtual coordinates not coming from a
c               neighbouring block by extrapolation (3d case)
c     ISINGETP  Get the pointer corresponding to an integer array
c     ISTRCO    Transport coordinates from a certain block
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c      1. compute some general parameters
c      2. activate all needed arrays and create an array vco
c         storing an intermediate representation of virtual coordinates
c      3. fill virtual coordinates as much as possible from neighbouring
c         blocks
c      4. fill the remaining virtual coordinates by extrapolation
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- fill virtual coordinates as much as possible using neighbouring
c         blocks
c         first compute nvirtual and ndim

      call isactive(isiinput(1))

      ipiinput     = isingetp(isiinput(1))
      nvirtual     = ibuffr(ipiinput + 54 - 1)
      ndim         = ibuffr(ipiinput + 57 - 1)

c     --- create an array vco containing an intermediate representation
c         of virtual coordinates

      lenvco = 0
      do iblockrf = 1, nblocks
         call isactive(isiinput(iblockrf))
         ipiinput = isingetp(isiinput(iblockrf))
         ni = ibuffr(ipiinput + 51 - 1)
         nj = ibuffr(ipiinput + 52 - 1)
         nk = ibuffr(ipiinput + 53 - 1)
         if ( ndim.eq.2) then
            lenvco = max(lenvco, (1+max(ni,nj,nk))*ndim*nvirtual)
         else
            lenvco = max(lenvco, (1+max(ni,nj,nk))*(1+max(ni,nj,nk))*
     +                            ndim*nvirtual)
         end if
      end do
      isvco = 0
      call isdcreat ( isvco, lenvco, 'vco' )

c     --- activate all coordinate and iinput arrays and the itopol array

      do iblockrf = 1, nblocks
         call isactive(iscoor(iblockrf))
         call isactive(isiinput(iblockrf))
      end do
      call isactive(isitopol)

c     --- compute pointers and some parameters for the itopol array

      ipvco    = isdpgetp(isvco)
      ipitopol = isingetp(isitopol)
      nfaces   = ibuffr(ipitopol + 2 - 1)
      maxnsubf = ibuffr(ipitopol + 3 - 1)
      ipnsubf  = ibuffr(ipitopol+5-1) + ipitopol - 1
      ipsubf   = ibuffr(ipitopol+6-1) + ipitopol - 1

c     --- fill virtual coordinates as much as possible using coordinates
c         from neighbouring blocks

      do iblockrf = 1, nblocks

c     --- determine some parameters of block iblockrf

         ipiinput = isingetp(isiinput(iblockrf))
         ni = ibuffr(ipiinput + 51 - 1)
         nj = ibuffr(ipiinput + 52 - 1)
         nk = ibuffr(ipiinput + 53 - 1)

         ipcoor       = isdpgetp(iscoor(iblockrf))

         call istrco ( iblockrf, ni, nj, nk, nvirtual, ndim,
     +                 nblocks, nfaces, maxnsubf,
     +                 iscoor, isiinput, buffer(ipcoor),
     +                 ibuffr(ipnsubf), ibuffr(ipsubf),
     +                 buffer(ipvco) )

      end do

c     --- fill the remaining virtual coordinates by extrapolation

      do iblockrf = 1, nblocks

         ipiinput = isingetp(isiinput(iblockrf))
         ni = ibuffr(ipiinput + 51 - 1)
         nj = ibuffr(ipiinput + 52 - 1)
         nk = ibuffr(ipiinput + 53 - 1)

         ipcoor       = isdpgetp(iscoor(iblockrf))

         if ( ndim.eq.2 ) then

c        --- ndim = 2

            call isextco2 ( iblockrf, ni, nj, nvirtual,
     +                      nblocks, nfaces, maxnsubf, buffer(ipcoor),
     +                      ibuffr(ipnsubf), ibuffr(ipsubf) )

         else

c        --- ndim = 3

            call isextco3 ( iblockrf, ni, nj, nk, nvirtual,
     +                      nblocks, nfaces, maxnsubf, buffer(ipcoor),
     +                      ibuffr(ipnsubf), ibuffr(ipsubf) )

         end if

      end do

      call isdelarr(isvco)
      call isdcrpri

      end
