      subroutine isior ( nblocks, nfaces, maxnsubf, nsubfaces,
     +                   subfaces, blockor )
c ======================================================================
c
c        programmer     Erik Brakkee
c        version 1.1    date 05-03-1997 Kees K. change for 3D
c        version 1.0    date 23-05-1994
c
c
c   copyright (c) 1994  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Initialize iorient fields of itopol
c
c **********************************************************************
c
c                       KEYWORDS
c
c   topology
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer maxnsubf, nblocks, nfaces
      integer blockor(1:nblocks)
      integer nsubfaces(1:nblocks, 1:nfaces)
      integer subfaces(1:nblocks, 1:nfaces, 1:maxnsubf, *)

c     blockor   i    array containing the absolute orientation
c                    of the subdomains (1 -> sqrt(g) > 0), -1 ->
c                    sqrt(g) < 0)
c     maxnsubf  i    Maximum number of subfaces encountered on all subfaces
c     nblocks   i    Number of blocks
c     nfaces    i    Number of faces
c     nsubfaces i    Array containing the number of subfaces of each face,
c                    see array itopol for an extended description
c     subfaces  i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer iblock, iface, isubface, ifto, nbto, relblckor
      integer vtsign(6), vnsign(6), nsignmb, tsignmb

c     iblock    Block sequence number
c     iface     Face number
c     ifto      Target face
c     isubface  Subface number
c     nbto      Number of a neighbouring block
c     nsignmb   Nsignmb = 0 if the normal direction of the subface
c               coincides with that of its neighbouring block, otherwise
c               nsignmb = 1
c     relblckor Relative block orientation. If the neighbouring
c               block has sqrt(g) with opposite sign to this block
c               then relblckor = 1 otherwise it is 0
c     tsignmb   If the tangential direction of the subface coincides with
c               that of its neighbouring block tsignmb = 0, otherwise
c               tsignmb = 1
c     vnsign    Vnsign(i) is the sign of the normal vector
c               with respect to the corresponding contravariant
c               component at side i
c     vtsign    Vtsign(i) is the sign of the tangential vector
c               with respect to the corresponding contravariant
c               component at side i
c **********************************************************************
c
c                       I/O
c
c      none
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c        -
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c     For iblock = 1, nblocks
c        For iface = 1, nfaces
c           For isubfaces = 1, nsubfaces(iblock,iface)
c              Set iorient field of subface (iblock,iface,isubface)
c           End
c        End
c     End
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c

      if ( nfaces.eq.4 ) then

        vtsign(ileft) = -1
        vnsign(ileft) = -1

        vtsign(iright) = 1
        vnsign(iright) = 1

        vtsign(idown) = 1
        vnsign(idown) = -1

        vtsign(iup) = -1
        vnsign(iup) = 1

      else

         do iface = 1, nfaces

           vtsign(iface) = 1
           vnsign(iface) = 1

         end do

      end if

      do iblock = 1, nblocks
         do iface = 1, nfaces
            do isubface = 1, nsubfaces(iblock, iface)

               nbto  = subfaces(iblock,iface,isubface,5)
               ifto  = subfaces(iblock,iface,isubface,6)

               relblckor = (-blockor(iblock)*blockor(nbto)+1)/2

               if ( nbto.eq.0 ) then

c              --- external subface

                  subfaces(iblock,iface,isubface,8) = 0

               else

                  if ( nfaces.eq.4 ) then

c                 --- 2-D

                     tsignmb = 1
                     if ( blockor(iblock)*vtsign(iface) .eq.
     +                    blockor(nbto)*vtsign(ifto) )
     +                  tsignmb = -tsignmb

                     nsignmb = 1
                     if ( vnsign(iface).eq.vnsign(ifto) )
     +                  nsignmb = -nsignmb

                     subfaces(iblock,iface,isubface,8) =
     +                  (-tsignmb+1)/2 + (-nsignmb+1) +
     +                  4*relblckor

                  else

c                 --- 3-D

                     subfaces(iblock,iface,isubface,8) = 1

c                    --- the right values for 3D will be set in
c                        subroutine isco3put

                  end if

               end if

            end do
         end do
      end do

      end
