      subroutine isnums02(nblocks, nfaces, maxnsubf,
     +                    nsubfaces, subfaces)
c=====================================================================
c
c     programmer   Erik Brakkee
c
c
c     version      1.0  6-5-94  Erik:  initial revision
c
c   copyright (c) 1994  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c***********************************************************************
c
c                  DESCRIPTION
c
c     Compute a unique numbering of subfaces
c
c***********************************************************************
c
c                  KEYWORDS
c
c***********************************************************************
c
c                  INPUT / OUTPUT PARAMETERS
c
c
      implicit none
      integer maxnsubf, nblocks, nfaces
      integer nsubfaces(1:nblocks, 1:nfaces)
      integer subfaces(1:nblocks, 1:nfaces, 1:maxnsubf, *)

c     maxnsubf  i    Maximum number of subfaces encoutered on all subfaces
c     nblocks   i    Number of blocks
c     nfaces    i    Number of faces
c     nsubfaces i    Array containing the number of subfaces of each face,
c                    see array itopol for an extended description
c     subfaces  i    reserved
c***********************************************************************
c
c                   COMMON BLOCKS
c
c
c
c***********************************************************************
c
c                   LOCAL PARAMETERS
c
      integer iblock, iblocknb, iface, isf, isubface
c
c     iblock    Block sequence number
c     iblocknb  Block number of neighbour
c     iface     Face number
c     isf       First unused subface number
c     isubface  Subface number
c**********************************************************************
c
c                   I/O
c
c      none
c
c**********************************************************************
c
c                   SUBROUTINES CALLED
c
c
c**********************************************************************
c
c                   ERROR MESSAGES :
c
c        -
c
c***********************************************************************
c
c     PSEUDO CODE    :
c
c     isf = first unused number of a subface
c
c     isf = 1
c
c     For iblock = 1, nblocks
c       For iface = 1, nfaces
c         For isubfaces = 1, nsubfaces(iblock,iface)
c           If ( subface is not already numbered )
c           Then
c             number_of_subface = isf
c             isf = isf + 1
c           Endif
c         End
c       End
c     End
c
c=======================================================================
c

      isf = 1

      do iblock = 1, nblocks
         do iface = 1, nfaces
            do isubface = 1, nsubfaces(iblock, iface)

               iblocknb =  subfaces(iblock,iface,isubface,5)

               if ( iblocknb .eq. 0 ) then

c              ---  external subface -> not already numbered

                  subfaces(iblock,iface,isubface,10) = isf
                  isf = isf + 1

               else

c              ---  internal subface

                  if ( iblocknb .gt. iblock ) then

                     subfaces(iblock,iface,isubface,10) = isf
                     isf = isf + 1

                  endif

               endif

            enddo
         enddo
      enddo

      end


