      subroutine isreadic ( iincnd, rincnd, ndegfd, dnumbr )
c ======================================================================
c
c
c        programmer     Guus Segal
c        version 1.0    date 19-05-94
c
c
c   copyright (c) 1994  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c ********************************************************************
c
c                      DESCRIPTION
c
c    Read all input concerning the initial conditions from the intermediate
c    file and store in the arrays iincnd and rincnd for one block
c ********************************************************************
c
c                      KEYWORDS
c
c    initial_conditions
c    read_cards
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer ndegfd, iincnd(ndegfd)
      double precision dnumbr(*), rincnd(ndegfd)

c     dnumbr         Work array to store a series numbers that have been read
c     iincnd    o    reserved
c     ndegfd    i    number of degrees of freedom
c     rincnd    o    reserved
c ********************************************************************
c
c                      COMMON BLOCKS
c
c ********************************************************************
c
c                      LOCAL PARAMETERS
c
      integer inum, j

c     inum      Indicates the number of numbers that have been read
c     j         General loop variable
c ********************************************************************
c
c                      SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c     ISREADRL  read a series of numbers which may be at several records
c               skip texts and other symbols
c ********************************************************************
c
c                      I/O
c
c     Input is read from the intermediate file
c     The following input is expected
c     ndegfd integers with respect to iincnd
c     ndegfd reals with respect to rincnd
c ********************************************************************
c
c                      ERROR MESSAGES
c
c     508   iincnd(i) in input file incorrrect
c ********************************************************************
c
c                      PSEUDO CODE
c
c     Read iincnd
c     Check iincnd
c     Read rincnd
c ======================================================================
c
      call iseropen ( 'isreadic' )

c     --- Read iincnd

      call isreadrl ( dnumbr, ndegfd, inum )
      do 100 j = 1, ndegfd
         iincnd(j) = nint(dnumbr(j))
100   continue
      do 110 j = 1, ndegfd
         if ( iincnd(j).lt.-1 .and. iincnd(j).gt.-100 .or.
     +        iincnd(j).lt.-10000 .and. iincnd(j).gt.10000 ) then
            call iserrint ( j, 1 )
            call iserrint ( iincnd(j), 2 )
            call iserrsub ( 508, 2, 0, 0 )
         end if
110   continue

c     --- Read rincnd

      call isreadrl ( rincnd, ndegfd, inum )

      call iserclos ( 'isreadic' )
      end
