      subroutine isreadif ( isiinput, isrinput, nblocks, nmachines,
     +                      ndim )
c ======================================================================
c
c        programmer     Guus Segal
c        version 2.22   date 29-04-1999 Guus: extension with structure
c        version 2.21   date 23-03-1999 Duncan: extension for newton
c                                       linearisation
c        version 2.20   date 18-03-1999 Guus: extension with free surfaces
c        version 2.19   date 15-12-1998 Guus: check on scriptfile in case of
c                                             parallel computing
c        version 2.18   date 04-06-1998 Charles: correction mturb, idiscret
c        version 2.17   date 27-05-1998 Guus: extension iintim/rintim
c        version 2.16   date 18-05-1998 Charles Bilinear discretization
c        version 2.15   date 09-04-1998 Guus: make maxcon dependend on stencil
c        version 2.14   date 27-03-1998 Kees K: leniintim=16
c        version 2.13   date 05-01-1998 Guus: increase ncoef to 10
c        version 2.12   date 25-11-1997 Guus: set singular parameter also in
c                                             decoupled case
c        version 2.11   date 13-10-1997 Guus: extra parameters ndim, icoupled
c        version 2.10   date 15-09-1997 Guus: extension of iintur and rintur
c        version 2.9    date 08-09-1997 Guus: introduction cavity
c        version 2.8    date 04-09-1997 Guus: introduction icoupled
c        version 2.7    date 02-09-1997 Guus: correction collocated
c        version 2.6    date 06-08-1997 Guus: return in case of error
c        version 2.5    date 10-06-1997 Guus: extension length iincom
c        version 2.4    date 17-02-1997 Marcel extend iintim with norm
c                                       version number 14
c        version 2.3    date 11-02-1997 Kees K. extend iintim with norm
c                                       version number 13
c        version 2.2    date 15-01-1997 Guus check presence of file
c                                       isnasinp.cmp
c        version 2.1    date 10-09-1996 Marcel extend leniintur
c                                       version number 12
c        version 2.0    date 23-08-1996 Marcel extend leniintur
c                                       version number 11
c
c   copyright (c) 1994-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Read intermediate file and store the input read into the arrays
c    corresponding to isiinput and isrinput
c    Warning: the parameters maxb, maxmat and maxcon are filled in
c             an ad-hoc way in this subroutine.
c             This is only temporary and should be replaced by a
c             definitive way later on
c **********************************************************************
c
c                       KEYWORDS
c
c    read
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer nblocks, isiinput(nblocks), isrinput(nblocks), nmachines
      integer ndim

c     isiinput i/o   Array of sequence numbers (nblocks) of array iinput
c     isrinput  i    Array of sequence numbers (nblocks) of array rinput
c     nblocks   i    Number of blocks
c     ndim      i    Dimension of space (2 or 3)
c     nmachines o    Number of nodes that are used
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'
      include 'IScommon/cisblank'
      include 'IScommon/cuser'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, igeo, inum, ipiinbc, ipiinblk, ipiincnd, ipiincof,
     +        ipiincop, ipiindsc, ipiinput, ipiinsol, ipiintim,
     +        ipiintur, iprinbc, iprinblk, iprincnd, iprincof, iprincop,
     +        iprindsc, iprinput, iprinsol, iprintim, iprintur, iptemp,
     +        iptemprl, irefsv, iversion, lenext, length, mcav, maxb,
     +        maxlenin, maxlenrl, maxvpnts, nbufol, itypepost, maxcon,
     +        ncoefs, ndegfd, ni, niuser, nj, nk, ntimlv, ntrans,
     +        ntrnsp, nturb, nuser, nvirtual, ipiincom, iprincom,
     +        leniinbc, leniinblk, leniincom, leniindsc, lenrintur,
     +        leniinsol, leniintim, leniintur, lenrinbc, lenrinblk,
     +        lenrincnd, lenrincom, lenrindsc, lenrinsol, lenrintim,
     +        ipiseqeq, leniseqeq, mshtyp, icoupled, leniincav,
     +        lenrincav, iprincav, ipiincav, lshift, mturb, idiscret,
     +        itypsten, lenrinfrs, igeotime, ipiinfrs, iprinfrs,
     +        leniinfrs, iversact, nclusters, ipiclust, ipiingls,
     +        ipiinlos, ipringls, iprinlos, leniclust, leniingls,
     +        leniinlos, lenringls, lenrinlos
      double precision dnumbr(1000)
      logical singular, check

c     check     Indication if a file has been opened (true) or not (false)
c     dnumbr    Work array to store a series numbers that have been read
c     i         General loop variable
c     icoupled  If 0 the momentum equations are coupled, if 1 they are not
c     idiscret  Type of discretization
c     igeo      Indicates type of geometry discretization
c     igeotime  If 1 a free surface model is used, if 0 not
c     inum      Indicates the number of numbers that have been read
c     ipiclust  Starting address of array ICLUSTER in array IINPUT
c     ipiinbc   Starting address of array iinbc in array iinput
c     ipiinblk  Starting address of array IINBLK in array IINPUT
c     ipiincav  Starting address of array IINCAV in array IINPUT
c     ipiincnd  Starting address of array iincnd in array iinput
c     ipiincof  Starting address of array IINCOF in array IINPUT
c     ipiincom  Starting address of array IINCOM in array IINPUT
c     ipiincop  Starting address of copy of array IINPUT in IBUFFR
c     ipiindsc  Starting address of array IINDSC in array IINPUT
c     ipiinfrs  Starting address of array IINFRSUR in array IINPUT
c     ipiingls  Starting address of array IGLOBSTR in array IINPUT
c     ipiinlos  Starting address of array ILOCSTR in array IINPUT
c     ipiinput  Starting address of array iinput in ibuffr
c     ipiinsol  Starting address of array IINSOL in array IINPUT
c     ipiintim  Starting address of array IINTIM in array IINPUT
c     ipiintur  Starting address of array IINTUR in array IINPUT
c     ipiseqeq  Starting address of array ISEQEQ in array IINPUT
c     iprinbc   Starting address of array rinbc in array iinput
c     iprinblk  Starting address of array RINBLK in array RINPUT
c     iprincav  Starting address of array RINCAV in array RINPUT
c     iprincnd  Starting address of array rincnd in array rinput
c     iprincof  Starting address of array RINCOF in array RINPUT
c     iprincom  Starting address of array RINCOM in array RINPUT
c     iprincop  Starting address of copy of array RINPUT in BUFFER
c     iprindsc  Starting address of array RINDSC in array RINPUT
c     iprinfrs  Starting address of array RINFRSUR in array RINPUT
c     ipringls  Starting address of array RGLOBSTR in array RINPUT
c     iprinlos  Starting address of array RLOCSTR in array RINPUT
c     iprinput  Starting address of array rinput in buffer
c     iprinsol  Starting address of array RINSOL in array RINPUT
c     iprintim  Starting address of array RINTIM in array RINPUT
c     iprintur  Starting address of array rintur in array RINPUT
c     iptemp    Starting address of temporary array in IBUFFR
c     iptemprl  Starting address of real temporary array in BUFFER
c     irefsv    Help variable to save the value of a reference number
c     itypepost Indicates the type of postprocessing required
c               0: standard Newton    1: linear in computational domain
c     itypsten  Indicates the type of stencil
c               Possible values
c               R^2
c               1:  13-point stencil
c               2:  21-point stencil
c               3:  17-point stencil
c               R^3
c               1:  51-point stencil
c               2:  59-point stencil
c     iversact  Actual version number
c     iversion  Version sequence number
c     lenext    Extra length required for array
c     length    Length of the array in words
c     leniclust Length of array icluster
c     leniinbc  Computed length of array iinbc
c     leniinblk Length of first component of array iinblk
c     leniincav Length of first component of array iincav
c     leniincom Length of first component of array iincom
c     leniindsc Length of first component of array iindsc
c     leniinfrs Length of array iinfrsur
c     leniingls Length of array IGLOBSTR
c     leniinlos Length of array ILOCSTR
c     leniinsol Length of first component of array iinsol
c     leniintim Length of first component of array iintim
c     leniintur Length of first component of array iintur
c     leniseqeq Length of the array iseqeq
c     lenrinbc  Computed length of array rinbc
c     lenrinblk Length of first component of array rinblk
c     lenrincav Length of first component of array rincav
c     lenrincnd Length of first component of array rincnd
c     lenrincom Length of first component of array rincom
c     lenrindsc Length of first component of array rindsc
c     lenrinfrs Length of array rinfrsur
c     lenringls Length of array RGLOBSTR
c     lenrinlos Length of array RLOCSTR
c     lenrinsol Length of first component of array rinsol
c     lenrintim Length of first component of array rintim
c     lenrintur Length of first component of array rintur
c     lshift    Local shift
c     maxb      Maximum number of real values for one boundary condition
c     maxcon    Maximum number of non zero diagonals in a matrix
c     maxlenin  Maximum length of integer array
c     maxlenrl  Maximum length of real array
c     maxvpnts  Maximum number of points per block including virtual ones
c     mcav      If 1 a cavity model is used, if 0 not
c     mshtyp    Type of grid arrangement
c     mturb     Indicates type of turbulence modelling
c     nbufol    Value of NBUFFR at input
c               This value may be used to reset NBUFFR in a later stage
c     nclusters Number of clusters
c     ncoefs    Number of coefficients
c     ndegfd    Number of degrees of freedom
c     ni        Number of non-virtual cells in the i-direction (one block)
c     niuser    Number of parameters stored in array iuser
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     ntimlv    Number of time levels
c     ntrans    Number of transport equations plus number of turbulence
c               equations
c     ntrnsp    Number of transport equations
c     nturb     Number of extra transport equations for turbulence
c     nuser     Number of parameters stored in array user
c     nvirtual  Number of rows of virtual cells
c     singular  If true the pressure matrix is singular since at all
c               boundaries the normal velocity has been prescribed
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isingetp, isdpgetp

c     DCOPY     Copy double precision array into other one (BLAS subroutine)
c     ISDCREAT  Create permanent space in array IBUFFR (Double precision array)
c     ISDEXTEN  Extend existing double precision array in BUFFER
c     ISDPGETP  get the pointer corresponding to a double precision array
c     ISENDSPC  Reserve space at the end of IBUFFR
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRCHR  Puts values of strings in character error array
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c     ISIEXTEN  Extend existing integer array in IBUFFR
c     ISINCOPY  Copy integer array to other integer array possibly itself
c     ISINGETP  get the pointer corresponding to an integer array
c     ISINIFIL  Open files
c     ISLRESET  Reset buffer length to old value
c     ISREADBC  Read boundary conditions from intermediate file
c     ISREADCF  Read coefficients from intermediate file
c     ISREADCM  Read information about compressibilty from intermediate file
c     ISREADCV  Read information about cavity from intermediate file
c     ISREADDS  Read input concerning discretization from intermediate file
c     ISREADFRS Read input concerning free surfaces from intermediate file
c     ISREADIC  Read initial conditions from intermediate file
c     ISREADMB  Read concerning multiblock from intermediate file
c     ISREADRL  read a series of numbers which may be at several records
c               skip texts and other symbols
c     ISREADSL  Read concerning linear solver from intermediate file
c     ISREADSQ  Read concerning sequence of equations from intermediate file
c     ISREADSTR Read input concerning the structure of the main program
c     ISREADTM  Read concerning time integration from intermediate file
c     ISREADTU  Read concerning turbulence modelling from intermediate file
c     ISWARSUB  Print warnings
c **********************************************************************
c
c                       I/O
c
c     Input is read from the intermediate file
c     See the document: Basis design ISNaS incompressible flow solver
c     for a description of the structure of the file
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     501   Version number incorrect
c     502   Parameter in input file incorrect
c     513   NUMSEG < 1 or NUMSEG > ni
c     514   i1 < 1 or i1 > ni
c     520   Parameter in input file incorrect
c     521   Number of machines larger than number of blocks
c     524   nturb read in dimension part not the same as in turbulence part
c     684   Version number incorrect
c    1712   parallel application without using isnasmpi
c **********************************************************************
c
c                       PSEUDO CODE
c
c     Open intermediate file
c     Read and check version number
c     Read and check parameters for common CISNAS
c     Read dimension parameters and put in all arrays iinput(iblock)
c     Read choice parameters and put in all arrays iinput(iblock)
c     Extend the arrays iinput(iblock) with space for iincof, iincnd, iintim,
c     iinsol, iindsc and iinblk
c     Extend the arrays rinput(iblock) with space for rincnd, rintim, rinsol,
c     rintur and rinblk
c     Read coefficients and put in all arrays IINCOF and RINCOF
c     Read information with respect to linear solvers and put in all arrays
c     IINSOL and RINSOL
c     Read information with respect to time integration and put in all arrays
c     IINTIM and RINTIM
c     Read information with respect to initial conditions and put in all arrays
c     IINCND and RINCND
c     Read information with respect to the discretization and put in all arrays
c     IINDSC and RINDSC
c     Read information with respect to the multi-block process and put in all
c     arrays IINBLK and RINBLK
c     Read information with respect to the turbulence modelling and put in
c     array rintur
c     Read information with respect to the compressible flow and put in
c     arrays iincom and rincom
c     Read information with respect to the cavity model and put in
c     arrays iincav and rincav
c     Read parameters with respect to the output
c     Create temporary arrays to store information about the boundary
c     conditions
c     Read information about the boundary conditions and store in temporary
c     arrays
c     Extend IINPUT and RINPUT with space with respect to the boundary
c     conditions
c     Copy information about boundary conditions in arrays IINBC and RINBC
c     Read information with respect to common cuser
c     Close intermediate file
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data iversact /17/
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen ( 'isreadif' )

c     --- Open intermediate file

      call isinifil(-75)
      if ( nerror.ne.0 ) go to 1000
      irefsv = irefre
      irefre = iref75

c     --- Read and check version number

      call isreadrl ( dnumbr, 1, inum )
      if ( nerror.ne.0 ) go to 1000
      iversion = nint(dnumbr(1))
      if ( iversion.lt.iversact ) then

c     --- Version number too small

         call iserrint ( iversion, 1 )
         call iserrint ( iversact, 2 )
         call iserrsub ( 501, 2, 0, 0 )

      else if ( iversion.gt.iversact ) then

c     --- Version number too large

         call iserrint ( iversion, 1 )
         call iserrint ( iversact, 2 )
         call iserrsub ( 684, 2, 0, 0 )

      end if
      if ( nerror.ne.0 ) go to 1000

c     --- Read and check parameters for common CISNAS
c         100 integers

      call isreadrl( dnumbr, 100, inum )
      if ( nerror.ne.0 ) go to 1000
      ioutpt = nint(dnumbr(1))
      if ( ioutpt.lt.-1 .or. ioutpt.gt.3 ) then
         call iserrint ( ioutpt, 1 )
         call iserrchr ( 'ioutpt', 1 )
         call iserrsub ( 502, 1, 0, 1 )
      end if
      itime = nint(dnumbr(2))
      if ( itime.lt.0 .or. itime.gt.1 ) then
         call iserrint ( itime, 1 )
         call iserrchr ( 'itime', 1 )
         call iserrsub ( 502, 1, 0, 1 )
      end if
      if ( nerror.ne.0 ) go to 1000

c     --- 100 reals

      call isreadrl( dnumbr, 100, inum )
      if ( nerror.ne.0 ) go to 1000

c     --- Read dimension parameters and put in all arrays iinput(iblock)
c         100 integers

      call isreadrl( dnumbr, 100, inum )
      if ( nerror.ne.0 ) go to 1000
      nvirtual = nint(dnumbr(1))
      if ( nvirtual.lt.0 .or. nvirtual.gt.2 ) then
         call iserrint ( nvirtual, 1 )
         call iserrchr ( 'nvirtual', 1 )
         call iserrsub ( 502, 1, 0, 1 )
      end if
      ndegfd = nint(dnumbr(2))
      ntrnsp = nint(dnumbr(3))
      if ( ntrnsp.lt.0 .or. ntrnsp.gt.10 ) then
         call iserrint ( ntrnsp, 1 )
         call iserrchr ( 'ntrnsp', 1 )
         call iserrsub ( 502, 1, 0, 1 )
      end if
      ntimlv = nint(dnumbr(4))
      if ( ntimlv.lt.0 .or. ntimlv.gt.2 ) then
         call iserrint ( ntimlv, 1 )
         call iserrchr ( 'ntimlv', 1 )
         call iserrsub ( 502, 1, 0, 1 )
      end if
      ncoefs = nint(dnumbr(5))
      if ( ncoefs.lt.6 .or. ncoefs.gt.10 ) then
         call iserrint ( ncoefs, 1 )
         call iserrchr ( 'ncoefs', 1 )
         call iserrsub ( 502, 1, 0, 1 )
      end if
      nturb = nint(dnumbr(6))
      if ( nturb.lt.0 .or. nturb.gt.2 ) then
         call iserrint ( nturb, 1 )
         call iserrchr ( 'nturb', 1 )
         call iserrsub ( 502, 1, 0, 1 )
      end if
      ntrans = ntrnsp+nturb
      if ( ndegfd.ne.ndim+1+ntrans ) then
         call iserrint ( ndegfd, 1 )
         call iserrint ( ndim+1+ntrans, 2 )
         call iserrchr ( 'ndegfd', 1 )
         call iserrsub ( 520, 2, 0, 1 )
      end if

      nmachines = nint(dnumbr(7))
      if ( nmachines.lt.1 .or. nmachines.gt.10000 ) then
         call iserrint ( nmachines, 1 )
         call iserrchr ( 'nmachines', 1 )
         call iserrsub ( 502, 1, 0, 1 )
      end if
      if ( nmachines.gt.nblocks ) then
         call iserrint(nmachines, 1)
         call iserrint(nblocks, 2)
         call iswarsub( 521, 2, 0, 0)
         nmachines = nblocks
      end if
      if ( nmachines.gt.1 ) then
         inquire ( file = 'isnasmpi_testfile', exist = check )
         if ( .not. check ) then
            call iserrint(nmachines, 1)
            call iserrsub ( 1712, 1, 0, 0 )
         end if
      end if
      nclusters = nint(dnumbr(8))
      if ( nclusters.lt.1 .or. nclusters.gt.ndegfd ) then
         call iserrint ( nclusters, 1 )
         call iserrchr ( 'nclusters', 1 )
         call iserrsub ( 502, 1, 0, 1 )
      end if
      leniclust = nint(dnumbr(9))
      if ( nerror.ne.0 ) go to 1000

c     --- Read choice parameters and put in all arrays iinput(iblock)
c         100 integers

      call isreadrl( dnumbr, 100, inum )
      if ( nerror.ne.0 ) go to 1000
      igeo = nint(dnumbr(1))
      if ( igeo.lt.1 .or. igeo.gt.2 ) then
         call iserrint ( igeo, 1 )
         call iserrchr ( 'igeo', 1 )
         call iserrsub ( 502, 1, 0, 1 )
      end if
      itypepost = nint(dnumbr(2))
      if ( itypepost.lt.0 .or. itypepost.gt.1 ) then
         call iserrint ( itypepost, 1 )
         call iserrchr ( 'itypepost', 1 )
         call iserrsub ( 502, 1, 0, 1 )
      end if
      mshtyp = nint(dnumbr(3))
      if ( mshtyp.lt.0 .or. mshtyp.gt.1 ) then
         call iserrint ( mshtyp, 1 )
         call iserrchr ( 'mshtyp', 1 )
         call iserrsub ( 502, 1, 0, 1 )
      end if
      icoupled = nint(dnumbr(4))
      if ( icoupled.lt.0 .or. icoupled.gt.1 ) then
         call iserrint ( icoupled, 1 )
         call iserrchr ( 'icoupled', 1 )
         call iserrsub ( 502, 1, 0, 1 )
      end if
      mcav = nint(dnumbr(5))
      if ( mcav.lt.0 .or. mcav.gt.1 ) then
         call iserrint ( mcav, 1 )
         call iserrchr ( 'mcav', 1 )
         call iserrsub ( 502, 1, 0, 1 )
      end if
      igeotime = nint(dnumbr(6))
      if ( igeotime.lt.0 .or. igeotime.gt.1 ) then
         call iserrint ( igeotime, 1 )
         call iserrchr ( 'igeotime', 1 )
         call iserrsub ( 502, 1, 0, 1 )
      end if
      if ( nerror.ne.0 ) go to 1000

c     --- Extend the arrays iinput(iblock) with space for iincof, iincnd,
c         iintim, iinsol, iindsc, iintur, iincom and iinblk

      leniintim = 20
      leniinsol = 25
      leniindsc = 20
      leniinblk = 10
      leniintur = 10
      leniincom = 10
      leniincav = 10
      leniinfrs = 10
      leniingls = 50
      leniinlos = nclusters*20
      lshift = icoupled*(ndim-1)
      leniseqeq = 2 + ntrans + lshift
      lenext = ncoefs*(ntrnsp+1+lshift) + ndegfd + leniintim +
     +         leniinsol*(ntrans+2+lshift) +
     +         leniindsc*(ntrans+1+lshift) +
     +         leniinblk*(ntrans+3+lshift) + leniintur + leniincom +
     +         leniseqeq + leniincav + leniinfrs + leniclust +
     +         leniingls + leniinlos
      ipiincof = 151
      do i = 1, nblocks
         call isiexten ( isiinput(i), lenext )
         ipiinput = isingetp(isiinput(i))
         ibuffr(ipiinput  ) = ipiincof
         ibuffr(ipiinput+100) = ncoefs
         ipiincnd = ipiincof+ncoefs*(ntrnsp+1+lshift)
         ibuffr(ipiinput+2) = ipiincnd
         ibuffr(ipiinput+102) = ndegfd
         ipiintim = ipiincnd+ndegfd
         ibuffr(ipiinput+3) = ipiintim
         ibuffr(ipiinput+103) = leniintim
         ipiinsol = ipiintim+leniintim
         ibuffr(ipiinput+4) = ipiinsol
         ibuffr(ipiinput+104) = leniinsol
         ipiindsc = ipiinsol+leniinsol*(ntrans+2+lshift)
         ibuffr(ipiinput+5) = ipiindsc
         ibuffr(ipiinput+105) = leniindsc
         ipiinblk = ipiindsc+leniindsc*(ntrans+1+lshift)
         ibuffr(ipiinput+6) = ipiinblk
         ibuffr(ipiinput+106) = leniinblk
         ipiintur = ipiinblk+leniinblk*(ntrans+3+lshift)
         ibuffr(ipiinput+7) = ipiintur
         ibuffr(ipiinput+107) = leniintur
         ipiincom = ipiintur+leniintur
         ibuffr(ipiinput+8) = ipiincom
         ibuffr(ipiinput+108) = leniincom
         ipiincav = ipiincom+leniincom
         ibuffr(ipiinput+10) = ipiincav
         ibuffr(ipiinput+110) = leniincav
         ipiinfrs = ipiincav+leniincav
         ibuffr(ipiinput+11) = ipiinfrs
         ibuffr(ipiinput+111) = leniinfrs
         ipiclust = ipiinfrs+leniinfrs
         ibuffr(ipiinput+12) = ipiclust
         ibuffr(ipiinput+112) = leniclust
         ipiingls = ipiclust+leniclust
         ibuffr(ipiinput+13) = ipiingls
         ibuffr(ipiinput+113) = leniingls
         ipiinlos = ipiingls+leniingls
         ibuffr(ipiinput+14) = ipiinlos
         ibuffr(ipiinput+114) = leniinlos
         ipiseqeq = ipiinlos+leniinlos
         ibuffr(ipiinput+9) = ipiseqeq
         ibuffr(ipiinput+109) = leniseqeq
         ipiinbc  = ipiseqeq+leniseqeq
         ibuffr(ipiinput+1) = ipiinbc
         ibuffr(ipiinput+53) = nvirtual
         ibuffr(ipiinput+54) = ndegfd
         ibuffr(ipiinput+55) = 0
         ibuffr(ipiinput+58) = igeo
         ibuffr(ipiinput+61) = 0
         ibuffr(ipiinput+62) = ntrnsp
         ibuffr(ipiinput+63) = ntimlv
         ibuffr(ipiinput+64) = 0
         ibuffr(ipiinput+65) = ncoefs
         ibuffr(ipiinput+66) = itypepost
         ibuffr(ipiinput+67) = nmachines
         ibuffr(ipiinput+69) = nturb
         ibuffr(ipiinput+74) = mshtyp
         ibuffr(ipiinput+75) = icoupled
         ibuffr(ipiinput+76) = mcav
         ibuffr(ipiinput+79) = igeotime
         ibuffr(ipiinput+80) = nclusters

c        --- maxmat = 5; temporarily
c            maxdx = 2; temporarily
c            maxdim = 3

         ibuffr(ipiinput+61) = 5
         ibuffr(ipiinput+64) = 2

      end do

c     --- Create the arrays rinput(iblock) with space for rincof, rincnd,
c         rintim, rinsol, rintur, rindsc and rinblk

      lenrincnd = ndegfd
      lenrintim = 60
      lenrinsol = 25
      lenrintur = 30
      lenrincom = 10
      lenrincav = 10
      lenrinblk = 10
      lenrindsc = 10
      lenrinfrs = 10
      lenringls = 10
      lenrinlos = 10*nclusters
      length = ncoefs*(ntrnsp+1+lshift) + lenrindsc + lenrintim +
     +         lenrinsol*(ntrans+2+lshift) + lenrintur +
     +         lenrinblk*(ntrans+2+lshift) +
     +         lenrindsc*(ntrans+1+lshift) + lenrincom +
     +         lenrincav + lenrinfrs + lenringls + lenrinlos
      iprincof = 1
      do i = 1, nblocks
         call isdcreat ( isrinput(i), length , 'rinput' )
         iprinput = isdpgetp(isrinput(i))
         ipiinput = isingetp(isiinput(i))
         ibuffr(ipiinput+25) = iprincof
         ibuffr(ipiinput+125) = ncoefs
         iprincnd = iprincof+ncoefs*(ntrnsp+1+lshift)
         ibuffr(ipiinput+27) = iprincnd
         ibuffr(ipiinput+127) = lenrincnd
         iprintim = iprincnd+lenrincnd
         ibuffr(ipiinput+28) = iprintim
         ibuffr(ipiinput+128) = lenrintim
         iprinsol = iprintim+lenrintim
         ibuffr(ipiinput+29) = iprinsol
         ibuffr(ipiinput+129) = lenrinsol
         iprintur = iprinsol+lenrinsol*(ntrans+2+lshift)
         ibuffr(ipiinput+30) = iprintur
         ibuffr(ipiinput+130) = lenrintur
         iprincom = iprintur+lenrintur
         ibuffr(ipiinput+33) = iprincom
         ibuffr(ipiinput+133) = lenrincom
         iprincav = iprincom+lenrincom
         ibuffr(ipiinput+34) = iprincav
         ibuffr(ipiinput+134) = lenrincav
         iprinfrs = iprincav+lenrincav
         ibuffr(ipiinput+35) = iprinfrs
         ibuffr(ipiinput+135) = lenrinfrs
         ipringls = iprinfrs+lenrinfrs
         ibuffr(ipiinput+36) = ipringls
         ibuffr(ipiinput+136) = lenringls
         iprinlos = ipringls+lenringls
         ibuffr(ipiinput+37) = iprinlos
         ibuffr(ipiinput+137) = lenrinlos
         iprinblk = iprinlos+lenrinlos
         ibuffr(ipiinput+31) = iprinblk
         ibuffr(ipiinput+131) = lenrinblk
         iprindsc = iprinblk+lenrinblk*(ntrans+2+lshift)
         ibuffr(ipiinput+32) = iprindsc
         ibuffr(ipiinput+132) = lenrindsc
         iprinbc  = iprindsc+lenrindsc*(ntrans+1+lshift)
         ibuffr(ipiinput+26) = iprinbc
      end do
      if ( nerror.ne.0 ) go to 1000

c     --- Read coefficients and put in arrays IINCOF and RINCOF

      ipiinput = isingetp(isiinput(1))
      iprinput = isdpgetp(isrinput(1))
      ndim = ibuffr(ipiinput+56)
      if ( nerror.ne.0 ) go to 1000
      call isreadcf ( ibuffr(ipiinput+ipiincof-1),
     +                buffer(iprinput+iprincof-1), ncoefs, ntrnsp,
     +                dnumbr, ndim, icoupled )
      if ( nerror.ne.0 ) go to 1000

c     --- Read information with respect to linear solvers and put in arrays
c         IINSOL and RINSOL

      call isreadsl ( ibuffr(ipiinput+ipiinsol-1),
     +                buffer(iprinput+iprinsol-1), ntrans, dnumbr, ndim,
     +                icoupled )
      if ( nerror.ne.0 ) go to 1000

c     --- Read information with respect to time integration and put in
c         arrays IINTIM and RINTIM

      call isreadtm ( ibuffr(ipiinput+ipiintim-1),
     +                buffer(iprinput+iprintim-1), dnumbr )
      if ( nerror.ne.0 ) go to 1000

c     --- Read information with respect to initial conditions and put in
c         arrays IINCND and RINCND

      call isreadic ( ibuffr(ipiinput+ipiincnd-1),
     +                buffer(iprinput+iprincnd-1), ndegfd, dnumbr )
      if ( nerror.ne.0 ) go to 1000

c     --- Read information with respect to the discretization and put in
c         arrays IINDSC and RINDSC

      call isreadds ( ibuffr(ipiinput+ipiindsc-1),
     +                buffer(iprinput+iprindsc-1), ntrans, dnumbr, ndim,
     +                icoupled, mcav )

      if ( nerror.ne.0 ) go to 1000

c     --- Read information with respect to the multi-block process and put in
c         all arrays IINBLK and RINBLK

      call isreadmb ( ibuffr(ipiinput+ipiinblk-1),
     +                buffer(iprinput+iprinblk-1), ntrans, dnumbr, ndim,
     +                icoupled )
      if ( nerror.ne.0 ) go to 1000

c     --- Read information with respect to the turbulence modelling and put
c         arrays IINTUR and RINTUR

      call isreadtu ( ibuffr(ipiinput+ipiintur-1),
     +                buffer(iprinput+iprintur-1), dnumbr )
      if ( nerror.ne.0 ) go to 1000
      if ( nturb.ne.ibuffr(ipiinput+ipiintur-1) ) then
         call iserrint ( nturb, 1 )
         call iserrint ( ibuffr(ipiinput+ipiintur-1), 2 )
         call iserrsub ( 524, 2, 0, 0 )
      end if
      if ( nerror.ne.0 ) go to 1000

c     --- Read information with respect to the compressibility and put
c         arrays IINCOM and RINCOM

      call isreadcm ( ibuffr(ipiinput+ipiincom-1),
     +                buffer(iprinput+iprincom-1), dnumbr )
      if ( nerror.ne.0 ) go to 1000

c     --- Read information with respect to the cavity model and put in
c         arrays IINCAV and RINCAV

      call isreadcv ( ibuffr(ipiinput+ipiincav-1),
     +                buffer(iprinput+iprincav-1), dnumbr, mcav )
      if ( nerror.ne.0 ) go to 1000

c     --- Read information with respect to the freesurface model and put in
c         arrays IINFRSUR and RINFRSUR

      call isreadfrs ( ibuffr(ipiinput+ipiinfrs-1),
     +                 buffer(iprinput+iprinfrs-1), dnumbr, igeotime )

c     --- Read parameters with respect to the output

      call isreadrl ( dnumbr, 50, inum )
      if ( nerror.ne.0 ) go to 1000

c     --- Read information with respect to the structure of the problem and put
c         in arrays IGLOBSTR, RGLOBSTR, ILOCSTR, RLOCSTR and ICLUSTER

      call isreadstr ( ibuffr(ipiinput+ipiingls-1),
     +                 buffer(iprinput+ipringls-1),
     +                 ibuffr(ipiinput+ipiinlos-1),
     +                 buffer(iprinput+iprinlos-1),
     +                 ibuffr(ipiinput+ipiclust-1),
     +                 dnumbr, nclusters, leniclust )
      if ( nerror.ne.0 ) go to 1000

c     --- Compute maxb and maxcon

      if ( ibuffr(ipiinput-1+ipiindsc+1).eq.2 .or.
     +     ibuffr(ipiinput-1+ipiindsc+1).eq.4 ) then
         itypsten = 2
      else if ( ibuffr(ipiinput-1+ipiincom).ge.1 .and.
     +          ibuffr(ipiinput-1+ipiindsc-1+3).eq.0 ) then
         itypsten = 3
      else
         itypsten = 1
      end if

      if (ndim.eq.2) then
         if ( itypsten.eq.1 ) then
            maxcon = 13
         else if ( itypsten.eq.2 ) then
            maxcon = 21
         else
            maxcon = 17
         end if
         maxb = 2
      else
         if ( itypsten.eq.1 ) then
            maxcon = 51
         else if ( itypsten.eq.2 ) then
            maxcon = 59
         end if
         maxb = 3
      end if

c     --- Copy information read before into all blocks

      do i = 2, nblocks
         ipiincop = isingetp(isiinput(i))
         call isincopy ( ibuffr, ibuffr, ipiinput+ipiincof-1,
     +                   ipiincop + ipiincof - 1, lenext, 0 )
         iprincop = isdpgetp(isrinput(i))
         call dcopy ( length, buffer(iprinput+iprincof-1), 1,
     +                buffer(iprincop+iprincof-1), 1 )
      end do

c     --- Fill iinput(77) and (78)

      mturb = ibuffr(ipiinput+ipiintur-1+2-1)
      idiscret = ibuffr(ipiinput+ipiindsc-1+2-1)

      if ( idiscret.eq.4 ) then
         if ( mturb.gt.0 ) then
            ibuffr(ipiinput+77) = 3
            ibuffr(ipiinput+78) = 1
         else
            ibuffr(ipiinput+77) = 4
            ibuffr(ipiinput+78) = 3
         end if
      end if

c     --- Create temporary arrays to store information about the boundary
c         conditions (6000 integer positions and 4000/intlen doubles)

      if ( nerror.ne.0 ) go to 1000
      call isendspc ( iptemp, 10000, nbufol )
      if ( nerror.ne.0 ) go to 1000
      iptemprl = (iptemp+6000-1)/intlen+1
      maxlenin = 6000
      maxlenrl = 4000/intlen
      maxvpnts = 0

      do iblockrf = 1, nblocks

c     --- Read information about the boundary conditions and store in
c         temporary arrays

         ipiinput = isingetp(isiinput(iblockrf))
         iprinput = isdpgetp(isrinput(iblockrf))
         ni = ibuffr(ipiinput+50)
         nj = ibuffr(ipiinput+51)
         nk = ibuffr(ipiinput+52)
         if ( ndim.eq.2 ) then
            maxvpnts = max ( maxvpnts, (ni+1+2*nvirtual)*
     +           (nj+1+2*nvirtual) )
         else
            maxvpnts = max ( maxvpnts, (ni+1+2*nvirtual)*
     +           (nj+1+2*nvirtual)*(nk+1+2*nvirtual) )
         end if
         if ( nerror.ne.0 ) go to 1000
         call isreadbc ( ibuffr(iptemp), buffer(iptemprl), maxlenin,
     +                   maxlenrl, leniinbc, lenrinbc, ndim, ntrans,
     +                   ni, nj, nk, dnumbr, ibuffr(ipiinput),
     +                   singular, ibuffr(ipiinput+ipiindsc-1) )
         if ( nerror.ne.0 ) go to 1000
         ipiinsol = ibuffr(ipiinput+4)
         if ( singular ) then

c        --- Pressure matrix is singular, fill position 21 in iinsol

            if ( ibuffr(ipiinput-1+76).eq.0 ) then

c           --- coupled approach

               ibuffr(ipiinput+ipiinsol-2+21+leniinsol) = 1

            else

c           --- decoupled approach

               ibuffr(ipiinput+ipiinsol-2+21+leniinsol*ndim) = 1

            end if

         end if

c        --- Extend IINPUT and RINPUT with space with respect to the boundary
c            conditions

         call isiexten ( isiinput(iblockrf), leniinbc )
         call isdexten ( isrinput(iblockrf), lenrinbc )
         if ( nerror.ne.0 ) go to 1000

c        --- Copy information about boundary conditions in arrays IINBC and
c            RINBC

         ipiinput = isingetp(isiinput(iblockrf))
         ipiinbc = ibuffr(ipiinput+1)
         ibuffr(ipiinput+101) = leniinbc
         call isincopy ( ibuffr, ibuffr, iptemp, ipiinput+ipiinbc-1,
     +                   leniinbc, 0 )
         iprinput = isdpgetp(isrinput(iblockrf))
         iprinbc = ibuffr(ipiinput+26)
         ibuffr(ipiinput+126) = lenrinbc
         call dcopy ( lenrinbc, buffer(iptemprl), 1,
     +                buffer(iprinput+iprinbc-1), 1 )

c        --- Check if boundary conditions are given at the complete boundary

      end do

c     --- Fill maxvpnts

      do iblockrf = 1, nblocks
         ipiinput = isingetp(isiinput(iblockrf))
         iprinput = isdpgetp(isrinput(iblockrf))
         ibuffr(ipiinput+68) = maxvpnts
         ibuffr(ipiinput+59) = maxcon
         ibuffr(ipiinput+60) = maxb
      end do
      if ( nerror.ne.0 ) go to 1000

c     --- Read information with respect to common cuser

      call isreadrl( dnumbr, 2, inum )
      if ( nerror.ne.0 ) go to 1000
      niuser = nint(dnumbr(1))
      nuser = nint(dnumbr(2))
      if ( niuser.lt.0 .or. niuser.gt.1000 ) then
         call iserrint ( niuser, 1 )
         call iserrsub ( 513, 1, 0, 0 )
         niuser = min ( niuser, 1000 )
      end if
      if ( nuser.lt.0 .or. nuser.gt.1000 ) then
         call iserrint ( nuser, 1 )
         call iserrsub ( 514, 1, 0, 0 )
         nuser = min ( nuser, 1000 )
      end if
      if ( nerror.ne.0 ) go to 1000
      if ( niuser.gt.0 ) then
         call isreadrl( dnumbr, niuser, inum )
         if ( nerror.ne.0 ) go to 1000
         do i = 1, niuser
            iuser(i) = nint(dnumbr(i))
         end do
      end if
      if ( nuser.gt.0 ) call isreadrl ( user, nuser, inum )
      if ( nerror.ne.0 ) go to 1000

c     --- Read sequence of equations

      ipiinput= isingetp(isiinput(1))
      call isreadsq( ibuffr(ipiinput+ipiseqeq-1),leniseqeq, dnumbr)

c     --- Close intermediate file and reset buffer array

      close ( unit=iref75 )
      irefre = irefsv
      call islreset ( nbufol )
      if ( nerror.ne.0 ) go to 1000

1000  call iserclos ( 'isreadif' )

      end
