      subroutine isreadmb ( iinblk, rinblk, ntrans, dnumbr, ndim,
     +                      icoupled )
c ======================================================================
c
c        programmer     Guus Segal
c        version 2.0    date 09-10-1997 Guus: Extra parameters ndim, icoupled
c        version 1.0    date 19-05-1994
c
c
c   copyright (c) 1994-1997  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Read all input concerning the multiblock solver from the intermediate
c    file and store in the arrays iinblk and rinblk for one block
c **********************************************************************
c
c                       KEYWORDS
c
c    multiblock
c    read_cards
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ntrans, iinblk(10,*), icoupled, ndim
      double precision dnumbr(*), rinblk(10,*)

c     dnumbr         Work array to store a series numbers that have been read
c     icoupled  i    If 0 the momentum equations are coupled, if 1 they are not
c     iinblk    o    reserved
c     ndim      i    Dimension of space (2 or 3)
c     ntrans    i    Number of transport equations plus number of turbulence
c                    equations
c     rinblk    o    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, inum, iortho, irestart, j, maxiter, methodmul
      double precision alpha, relaccur

c     alpha     Relaxation parameter alpha
c     i         General loop variable
c     inum      Indicates the number of numbers that have been read
c     iortho    Maximal number of vectors used in the orthogonalization process
c     irestart  Indicates after how many steps the method restarts
c     j         General loop variable
c     maxiter   Maximum number of iterations
c     methodmul Type of multi block algorithm
c     relaccur  Relative accuracy
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRCHR  Puts values of strings in character error array
c     ISERREAL  Put real value in error message
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c     ISREADRL  read a series of numbers which may be at several records
c               skip texts and other symbols
c **********************************************************************
c
c                       I/O
c
c     Input is read from the intermediate file
c     The following input is expected
c     For i := 1(1) ntrans+3
c        iinblk(j,i), j = 1, 10
c     For i := 1(1) ntrans+2
c        rinblk(j), j = 1, 10
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     510   iinblk(i) in input file incorrrect
c     511   rinblk(i) in input file incorrrect
c **********************************************************************
c
c                       PSEUDO CODE
c
c     For i := 1 (1) ntrans+3
c        Read iinblk
c        Check iinblk
c     For i := 1 (1) ntrans+2
c        Read rinblk
c        Check rinblk
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call iseropen ( 'isreadmb' )
      do 110 i = 1, ntrans+3+icoupled*(ndim-1)

c     --- Read iinblk

         call isreadrl ( dnumbr, 10, inum )
         do 100 j = 1, 10
            iinblk(j,i) = nint(dnumbr(j))
100      continue

110   continue

      do 140 i = 1, ntrans+2+icoupled*(ndim-1)

c     --- Read rinblk

         call isreadrl ( rinblk(1,i), 10, inum )

140   continue

      if ( iinblk(1,1).lt.0 .or. iinblk(1,1).gt.1 ) then
         call iserrint ( 1, 1 )
         call iserrint ( 1, 2 )
         call iserrint ( iinblk(1,1), 3 )
         call iserrchr ( 'type of parallel algorithm', 1 )
         call iserrsub ( 510, 3, 0, 1 )
      end if
      do 200 i = 2+icoupled*(ndim-1), ntrans+3+icoupled*(ndim-1)
         methodmul = iinblk(1,i)
         if ( methodmul.lt.0 .or. methodmul.gt.1 ) then
            call iserrint ( 1, 1 )
            call iserrint ( i, 2 )
            call iserrint ( methodmul, 3 )
            call iserrchr ( 'amount of output', 1 )
            call iserrsub ( 510, 3, 0, 1 )
         end if
         maxiter = iinblk(2,i)
         if ( maxiter.lt.1 .or. maxiter.gt.1000 ) then
            call iserrint ( 2, 1 )
            call iserrint ( i, 2 )
            call iserrint ( maxiter, 3 )
            call iserrchr ( 'maxiter', 1 )
            call iserrsub ( 510, 3, 0, 1 )
         end if
         irestart = iinblk(3,i)
         if ( irestart.lt.1 .or. irestart.gt.100 ) then
            call iserrint ( 3, 1 )
            call iserrint ( i, 2 )
            call iserrint ( irestart, 3 )
            call iserrchr ( 'irestart', 1 )
            call iserrsub ( 510, 3, 0, 1 )
         end if
         iortho = iinblk(4,i)
         if ( iortho.lt.1 .or. iortho.gt.100 ) then
            call iserrint ( 4, 1 )
            call iserrint ( i, 2 )
            call iserrint ( iortho, 3 )
            call iserrchr ( 'iortho', 1 )
            call iserrsub ( 510, 3, 0, 1 )
         end if

200   continue

      do 300 i = 1, ntrans+2+icoupled*(ndim-1)
         relaccur = rinblk(1,i)
         if ( relaccur.lt.1d-15 .or. relaccur.gt.1d-1 ) then
            call iserrint ( 1, 1 )
            call iserrint ( i, 2 )
            call iserreal ( relaccur, 1 )
            call iserrchr ( 'relaccur', 1 )
            call iserrsub ( 511, 2, 1, 1 )
         end if
         alpha = rinblk(2,i)
         if ( alpha.lt.1d-15 .or. alpha.gt.1d0 ) then
            call iserrint ( 2, 1 )
            call iserrint ( i, 2 )
            call iserreal ( alpha, 1 )
            call iserrchr ( 'alpha', 1 )
            call iserrsub ( 511, 2, 1, 1 )
         end if
300   continue

      call iserclos ( 'isreadmb' )
      end
