      subroutine isblkpre ( res, searchvc, nmbsol, iequa, isiinput,
     +                      isrinput, iscoor, issolut, isinfbnd,
     +                      isbndcon, ismatrix, isgmat, isrhside,
     +                      isgeom, isitopol, isdensup, iadmin )
c ======================================================================
c
c     programmer   Erik Brakkee
c
c     version      3.1  18-03-1999  Kees V: ishgmr02 rewritten to isblkpre
c     version      3.0  04-03-1999  Kees V: removed array usolinit
c     version      2.9  22-12-1998  Kees V: removed subroutine ishgmr03
c     version      2.8  23-11-1998  Jason: removed array iaction
c     version      2.7  22-07-1998  Duncan:      new call to ishdd
c                                                new call to ishgmr03
c     version      2.6  08-07-1998  Kees V: nusol, nsolut cleaned
c     version      2.5  24-04-1997  Kees K: call of ishdd changed
c     version      2.4  27-02-1995  Erik: nkrylin iterations are used
c                                   to determine approximate solution
c                                   instead of only one.
c     version      2.3  07-02-1995  Erik: do loop boundaries corrected
c     version      2.2  06-02-1995  Michiel K:array raction removed
c     version      2.1  31-01-1995  Erik: isol = 2 is now handled the
c                                   the same way as isol = 1
c     version      2.0  10-01-1995  Erik: general cleanup + extension
c                                   with inaccurate solution of subdomains
c     version      1.0  20-05-1994  Erik: initial revision
c
c   copyright (c) 1994-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c **********************************************************************
c
c                       DESCRIPTION
c
c     Compute a search direction for multiblock GCR. Approximately solve
c     A * searchvc = res
c **********************************************************************
c
c                       KEYWORDS
c
c     domain_decompostion
c     linear_solver
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer          nmbsol, iequa, iadmin(*), isbndcon(*), iscoor(*),
     +                 isgmat(*), isiinput(*), isinfbnd(*), isitopol,
     +                 ismatrix(*), isrhside(*), isrinput(*),
     +                 issolut(*), isgeom(*), isdensup(*)
      double precision res(nmbsol), searchvc(nmbsol)

c     iadmin    i    reserved
c     iequa     i    General equation sequence number
c     isbndcon  i    Array of sequence numbers (nblocks) of array bndcon
c     iscoor    i    Array of sequence numbers (nblocks) of array coor
c     isdensup  i    Array of sequence numbers (nblocks) of array densup
c     isgeom    i    Array of sequence numbers (nblocks) of array geom
c     isgmat    i    Array of sequence numbers (nblocks) of array gmat
c     isiinput  i    Array of sequence numbers (nblocks) of array iinput
c     isinfbnd  i    Array of sequence numbers (nblocks) of array infbnd
c     isitopol  i    Sequence number of array itopol
c     ismatrix  i    Array of sequence numbers (nblocks) of array matrix
c     isrhside  i    Array of sequence numbers (nblocks) of array rhside
c     isrinput  i    Array of sequence numbers (nblocks) of array rinput
c     issolut   i    Array of sequence numbers (nblocks) of array solut
c     nmbsol    i    Length of the multi-block solution vector
c     res       i    Array containing the residual vector.
c     searchvc  i    Vector containing a search direction
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisparal'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer          i, iblock, ipiinblk, ipiinput, iprhside, isol,
     +                 istartv, nblocks, ndim, ni, nj, nk, nusol
      double precision rnorm, dnrm2

c     i         General loop variable
c     iblock    Block sequence number
c     ipiinblk  Starting address of array IINBLK in array IINPUT
c     ipiinput  Starting address of array iinput in ibuffr
c     iprhside  Starting address of array rhside in buffer
c     isol      Accuracy of subdomain solution in relation with
c               domain decomposition.
c                 0 = accurate enough
c                 1 = inaccurate with respect to multi-block accuracy
c                 2 = inaccurate using ilu approximation
c     istartv   First free position in a vector
c     nblocks   Number of blocks
c     ndim      Dimension of space (2 or 3)
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     nusol     Number of unknowns for one degree of freedom in
c               one block
c     rnorm     Norm of the residual vector.
c **********************************************************************
c
c                       I/O
c
c      None
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isingetp, isdpgetp

c     DNRM2     Calculates the 2-norm of a given vector. This is
c               a BLAS routine.
c     ISACTIVE  Activate old permanent array
c     ISDDSOL   copy 1-D vector to inner regions of all subdomain
c               solution vectors
c     ISDPGETP  get the pointer corresponding to a double precision array
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISINGETP  get the pointer corresponding to an integer array
c     ISSDD     Single domain solver
c     ISSOLDD   copy inner regions of all subdomain solution vectors
c               to 1-D vector
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     None
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c

      call iseropen('isblkpre')

c     --- get type of subdomain solution

      call isactive(isiinput(1))
      ipiinput = isingetp(isiinput(1))
      ipiinblk = ibuffr(ipiinput + 7 - 1) + ipiinput - 1
      isol     = ibuffr(ipiinblk + 2 - 1)
      nblocks  = ibuffr(ipiinput + 73 - 1)
      ndim     = ibuffr(ipiinput + 57 - 1)

      if ( isol.eq.0 ) then

c     --- accurate solution of subdomains, a scaling is necessary

         rnorm = dnrm2 ( nmbsol, res, 1 )

         do i = 1, nmbsol
            searchvc(i) = res(i)/rnorm
         end do

      else

c     --- inaccurate solution of subdomains

c         --- put res into rhside of all blocks

         istartv = 1
         do iblock = 1, nblocks

         if ( iadmin(iblock).eq.inodenr ) then

            call isactive(isiinput(iblock))
            call isactive(isrhside(iblock))

            ipiinput = isingetp(isiinput(iblock))
            iprhside = isdpgetp(isrhside(iblock))

            ni       = ibuffr(ipiinput + 51 - 1)
            nj       = ibuffr(ipiinput + 52 - 1)
            nk       = ibuffr(ipiinput + 53 - 1)
            nusol    = (ni+1)*(nj+1)*(nk+1)
            if ( iequa.eq.1 ) nusol = ndim*nusol

            do i = 0, nusol-1
               buffer(iprhside+i) = res(istartv+i)
            end do

            istartv = istartv + nusol

         end if

         end do

c        --- take 0 as initial solution

         do i = 1, nmbsol
            searchvc(i) = 0d0
         end do

         call isddsol ( nblocks, iequa, isiinput, issolut, isitopol,
     +                  iadmin, searchvc )

c        --- Solve local subdomains

         call issdd ( iequa, nblocks, isiinput, isrinput, iscoor,
     +                issolut, ismatrix, isgmat, isrhside, isitopol,
     +                iadmin, isinfbnd, isbndcon, isgeom, isdensup )

c        --- copy solution to searchvc

         call issoldd ( nblocks, iequa, isiinput, issolut, iadmin,
     +                  searchvc )

      end if

      call iserclos('isblkpre')

      end
