      subroutine isblksol ( iequa, isiinput, isrinput, iscoor, issolut,
     +                      iscoefs, iseddy, isinfbnd, isbndcon,
     +                      ismatrix, isgmat, isrhside, isgeom,
     +                      isitopol, isdensup, iadmin, isol, nblocks )
c ======================================================================
c
c     programmer   Erik Brakkee
c     version      2.5  21-09-1999 Guus: extrapolate to virtual cells
c     version      2.4  14-07-1999 Kees V: general revision
c     version      2.3  04-03-1999 Kees V: remove ipsolin and ishgmr05
c     version      2.2  01-03-1999 Kees V: simplification of accurate
c     version      2.1  26-02-1999 Kees V: (in)accurate combined
c     version      2.0  20-01-1999 Kees V: (in)accurate subdomain combined
c     version      1.9  23-11-1998 Jason: removed array iaction
c     version      1.8  19-05-1998 Duncan:  extension for saving
c                                           coefficients dens upwind
c                                           new call to ishgmr01
c     version      1.7  date 08-07-1998 Kees V: nusol, nsolut cleaned
c     version      1.6  date 13-10-1997 Charles: uncoupled allowed
c     version      1.5  24-04-1997 Kees K.: 3D
c     version      1.4  13-02-1997 Kees V.: warning for reuse
c     version      1.3  16-01-1997 Erik: adapted for reuse
c     version      1.2  27-02-1995 Erik: adaptions for optimized
c                                  restarted GCR
c     version      1.1  06-02-1995 Michiel K:array raction removed
c     version      1.0  05-01-1995 Erik: initial revision
c
c
c   copyright (c) 1994-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     This routine solves a domain decomposition problem for a certain
c     type of unknown. Inaccurate solution of subdomains is permitted.
c     It uses isendspc to allocate memory needed
c     in the GCR algorithm that is not governed by the memory
c     management routines (put differently: is safe)
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iequa, nblocks, iadmin(*), isbndcon(nblocks,*), iscoor(*),
     +        iseddy(*)  , isgmat(*)  , isiinput(*), isinfbnd(*),
     +        isitopol   , ismatrix(*), isrhside(*), isrinput(*),
     +        issolut(*) , iscoefs(*) , isgeom(*)  , isdensup(*),
     +        isol

c     iadmin         i    contains information about
c                         which blocks on which computer are stored.
c     iequa          i    General equation sequence number
c     isbndcon       i    Array of sequence numbers (nblocks) of array bndcon
c     iscoefs        i    Array of sequence numbers (nblocks) of array coefs
c     iscoor         i    Array of sequence numbers (nblocks) of array coor
c     isdensup       i    Array of sequence numbers (nblocks) of array densup
c     iseddy         i    Array of sequence numbers (nblocks) of array eddy
c     isgeom         i    Array of sequence numbers (nblocks) of array geom
c     isgmat         i    Array of sequence numbers (nblocks) of array gmat
c     isiinput       i    Array of sequence numbers (nblocks) of array iinput
c     isinfbnd       i    Array of sequence numbers (nblocks) of array infbnd
c     isitopol       i    Sequence number of array itopol
c     ismatrix       i    Array of sequence numbers (nblocks) of array matrix
c     isol           i    Accuracy of subdomain solution in relation with
c                         domain decomposition.
c                           0 = accurate enough
c                           1 = inaccurate with respect to multi-block accuracy
c                           2 = inaccurate using ilu approximation
c     isrhside       i    Array of sequence numbers (nblocks) of array rhside
c     isrinput       i    Array of sequence numbers (nblocks) of array rinput
c     issolut        i    Array of sequence numbers (nblocks) of array solut
c     nblocks        i    Number of blocks
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisnas'
      include 'IScommon/cisparal'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, iblock, idummy, ifirst, ioutp, ipbetas, ipalphloc,
     +        ipalphglob, ipalphsum,
     +        ipiinblk, ipiinput, ipireuse, ipisearch, ipisval, ipphis,
     +        ipres, ipresid, iprhsd, iprhside, iprinblk, iprinput,
     +        ipsearsp, ipsearvc, ipsvalsp, ipsvalvc, ipusol,
     +        ipwork, irestart, ireusev, istartv, itmax, itrans, itrunc,
     +        lwork, nbufol, ndegfd, ndim, neq, ni, nj, nk,
     +        nmbsol, nreuse, nusol, icoupled, iseqpres, nusol2,
     +        idiscret, ilev, ipiindsc, ntimlv, ipsolut
      integer isdpgetp, isingetp
      logical usebndcn
      double precision eps2, rnorm
      save ifirst, ipisearch, ipisval, ipireuse
      data ifirst /1/

c     eps2           Determine the accuracy of the final approximation.
c                    The termination criterion is based upon
c                    ||b-Ax || <  eps1 + eps2 * (b-Ax ) .
c                          k                         0
c     i              General loop variable
c     iblock         Block sequence number
c     icoupled       If 0 the momentum equations are coupled,
c                    if 1 they are not
c     idiscret       Type of discretization
c     idummy         Integer dummy
c     ifirst         With this parameter it is indicated if the data to be sent
c                    or received is the first in a row, somewhere in the middle
c                    or the last of a row
c                    Possible values in case of receive:
c                    1  The data to be received is the first of a series of
c                       data.
c                       The actual receive is performed.
c                    2  The data to be received is not the first
c                       of a series of data.
c                    3  The data to be received is the first of a series of
c                       data. Only the information array info is read and the
c                       length of the next array is returned.
c                    4  The data to be received is not the first of a series of
c                       data. Only the information array info is read and the
c                       length of the next array is returned.
c                    5  The data to be received is not the first of a series of
c                       data. It is supposed that the information array info has
c                       already been read by a previous call of isreceiv.
c                       Only the actual data are read.
c                    Possible values in case of send:
c                    0  The data to be sent is the only data to be sent.
c                       The actual send is performed.
c                    1  The data to be sent is the first of a series of data.
c                       More data will be sent in the same actual send.
c                    2  The data to be sent is not the first nor the last of a
c                       series of data.
c                       More data will be sent in the same actual send.
c                    3  The data to be sent is the last of a series of data.
c                       The actual send is performed.
c     ilev           Time level of the multi-block transport of information
c     ioutp          Defines amount of output
c     ipalphglob     Starting address of array alphaGlob in buffer
c     ipalphloc      Starting address of array alphaLoc in buffer
c     ipalphsum      Starting address of array alphaSum in buffer
c     ipbetas        Starting address of array betas in buffer
c     ipiinblk       Starting address of array IINBLK in array IINPUT
c     ipiindsc       Starting address of array IINDSC in array IINPUT
c     ipiinput       Starting address of array iinput in ibuffr
c     ipireuse       Address of ireuse array in ibuffr. ireuse(iequa)
c                    contains the actual number of saved search
c                    directions for equation iequa.
c     ipisearch      Address of isearch array in ibuffr.
c                    isearch(iequa) is the address of the saved search
c                    space { s_1, s_2, ...., s_k } in buffer.
c                    If the search space is not saved for equation
c                    iequa, it has value 0. If isearch(iequa) has not been
c                    initialized yet, it has value -1.
c     ipisval        Address of isval array in ibuffr.
c                    isval(iequa) is the address of the saved search value
c                    space { v_1 = A s_1, v_2 = A s_2, ..., v_k } in buffer
c                    If the search space is not saved for equation
c                    iequa, it has value 0. If isearch(iequa) has not been
c                    initialized yet, it has value -1.
c     ipphis         Starting address of array phis in buffer
c     ipres          Address of the residual vector in buffer
c     ipresid        Address of the RESID array containing the norms
c                    of the residuals in the different iteration steps in
c                    buffer
c     iprhsd         Address of the right-hand side vector RHSD in
c                    buffer
c     iprhside       Starting address of array rhside in buffer
c     iprinblk       Starting address of array RINBLK in array RINPUT
c     iprinput       Starting address of array rinput in buffer
c     ipsearsp       Address of the search space SEARCHSP in buffer
c     ipsearvc       Address of the search vector SEARCHVC in buffer
c     ipsolut        Starting address of array solut in buffer
c     ipsvalsp       Address of the results of the search space SVALSP
c                    in buffer
c     ipsvalvc       Address of a vector containing the results of a
c                    search direction in buffer
c     ipusol         Address of the solution vector USOL in buffer
c     ipwork         Starting address of array work in buffer
c     irestart       Indicates after how many steps the method restarts
c     ireusev        Reuse search values v= A*s (=1) or not (=0)
c     iseqpres       Sequence number of pressure equation
c     istartv        First free position in a vector
c     itmax          The maximum number of iterations to be performed in
c                    the linear solver.
c     itrans         indicates the quantities to be transported
c                    for multi block
c     itrunc         Indicates after how many steps the method truncates
c     lwork          Length of array work
c     nbufol         Value of NBUFFR at input
c                    This value may be used to reset NBUFFR in a later
c                    stage
c     ndegfd         Number of degrees of freedom
c     ndim           Dimension of space (2 or 3)
c     neq            Number of equations
c     ni             Number of non-virtual cells in the i-direction
c                    (one block)
c     nj             Number of non-virtual cells in the j-direction
c                    (one block)
c     nk             Number of non-virtual cells in the k-direction
c                    (one block)
c     nmbsol         Length of the multi-block solution vector
c     nreuse         Number of search directions to reuse.
c     ntimlv         Number of time levels
c     nusol          Number of unknowns for one degree of freedom in
c                    one block
c     nusol2         Number of scalar unknowns for the solvers
c                    nusol2 = nusol * ndimso
c     rnorm          Norm of the residual vector.
c     usebndcn       If true boundary conditions are used in the
c                    extrapolation
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISACTIVE       Activate old permanent array
c     ISBLKGCR       Performs the GCR accelerated multi-block algorithm
c     ISDDREL        Get virtual unknowns from a certain block
c     ISDDSOL        Copy 1-D vector to inner regions of all subdomain
c                    solution vectors
c     ISDPGETP       get the pointer corresponding to a double precision array
c     ISENDSPC       Reserve space at the end of IBUFFR
c     ISERCLOS       Resets old name of previous subroutine of higher level
c     ISEROPEN       Produces concatenated name of local subroutine
c     ISERRCHR       Puts values of strings in character error array
c     ISFILVSO       Extrapolate solution at outer boundaries
c     ISHDD          Perform a single domain decomposition iteration on the host
c     ISINGETP       get the pointer corresponding to an integer array
c     ISLRESET       Reset buffer length to old value
c     ISLSOLPR       print the solution just after the computation by the linear
c                    solver
c     ISRELDD        Copy all virtual unknowns to a single one-dimensional
c                    array
c     ISSOLDD        Copy inner regions of all subdomain solution vectors
c                    to 1-D vector
c     ISWARSUB       Print warnings
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c    1609   The the search values (v_k = A s_k) cannot be reused.
c **********************************************************************
c
c                       PSEUDO CODE
c
c     For the accurate solution method we have the following remarks.
c     Given: a routine that computes  y = b - Ax = L(x) (routine ishgmr05)
c     Required: solution to b - A x = 0 using GMRES with initial guess
c               u0
c
c     The matrix vector product is computed using:
c
c        A'v = L(u0+v) - L(u0) = (-A) v                 (routine ishgmr04)
c
c     Algorithm:
c      1. compute   b = -L(u0)
c      2. solve     (-A) z = b     using GMRES with initial guess z = 0
c      3. compute   x = z + u0
c
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c

      call iseropen('isblksol')

c     --- compute pointers

      call isactive(isiinput(1))
      call isactive(isrinput(1))
      if ( nerror.ne.0 ) go to 1000

      ipiinput = isingetp(isiinput(1))
      iprinput = isdpgetp(isrinput(1))
      ipiinblk = ibuffr(ipiinput + 7 - 1) + ipiinput - 1
      iprinblk = ibuffr(ipiinput + 32 - 1) + iprinput - 1
      ipiindsc = ibuffr(ipiinput-1+6)
      idiscret = ibuffr(ipiinput-1+ipiindsc+1)

c     --- determine some general parameters for the multi-block process

      nblocks  = ibuffr(ipiinput + 73 - 1)
      ndim     = ibuffr(ipiinput + 57 - 1)
      ndegfd   = ibuffr(ipiinput + 55 - 1)
      icoupled = ibuffr(ipiinput + 76 - 1)
      neq      = ndegfd + ( 1 - ndim )*( 1 - icoupled )
      ntimlv   = ibuffr(ipiinput + 64 - 1)
      ilev     = ntimlv
      itmax    = ibuffr(ipiinblk + 2 + 10*iequa - 1)
      itrunc   = ibuffr(ipiinblk + 3 + 10*iequa - 1)
      ioutp    = ibuffr(ipiinblk + 5 + 10*iequa - 1)
      nreuse   = ibuffr(ipiinblk + 7 + 10*iequa - 1)
      ireusev  = ibuffr(ipiinblk + 8 + 10*iequa - 1)
      eps2     = buffer(iprinblk + 1 + 10*(iequa - 1) - 1)

      if ( icoupled.eq.0 ) then
         iseqpres = 2
      else
         iseqpres = ndim + 1
      end if

      if ( iequa.lt.iseqpres ) then
         itrans = 1
      else
         itrans = iequa + ( ndim - 1 )*( 1 - icoupled )
      end if

      if ( ibuffr(ipiinblk + 6 + 10*iequa - 1).eq.0 ) then

c     --- restarted GCR

         irestart = itrunc

      else

c     --- truncated GCR

         irestart = itmax

      end if

c     --- compute the dimension of the problem for GCR

      nmbsol = 0
      do iblock = 1, nblocks

         if ( isol.eq.0 ) then

c        --- set iblockrf for error messages

            iblockrf = iblock

c           --- compute required length for block iblock

            call isactive(isiinput(iblock))
            ipiinput = isingetp(isiinput(iblock))
            nmbsol   = nmbsol + ibuffr(ipiinput + 58 - 1)

         else

            if ( iadmin(iblock).eq.inodenr ) then

c           --- Count only local blocks
c               set iblockrf for error messages

               iblockrf = iblock

c              --- compute required length for block iblock

               call isactive(isiinput(iblock))
               ipiinput = isingetp(isiinput(iblock))
               ni       = ibuffr(ipiinput + 51 - 1)
               nj       = ibuffr(ipiinput + 52 - 1)
               nk       = ibuffr(ipiinput + 53 - 1)
               nmbsol   = nmbsol + (ni+1)*(nj+1)*(nk+1)

            end if

         end if

      end do

      if ( nerror.ne.0 ) go to 1000

      if ( isol.eq.0 ) then

c     --- twice as many virtual unknowns for the momentum equation

         if ( ndim.eq.2 ) then
            if ( idiscret.eq.2 .or. idiscret.eq.4 ) then

c           --- Wesbeek discretization

               if ( iequa.eq.1 ) nmbsol = 3*nmbsol
            else
               if ( iequa.eq.1 ) nmbsol = 2*nmbsol
            end if
         else
            if ( iequa.eq.1 ) nmbsol = 3*nmbsol
         end if

      else

c     --- ndim as many unknowns for the momentum equation

         if ( iequa.eq.1 ) nmbsol = ndim*nmbsol

      end if

c     --- When the value vectors (v=As) are reused for the momentum or
c         transport equation a warning is given and ireusev is set
c         equal to 0.

      if ( iequa.ne.2 .and. nreuse.gt.0 .and. ireusev.eq.1 ) then
         if ( ioutp.ge.0 ) then
            if ( iequa.eq.1 ) then
               call iserrchr ( 'momentum', 1 )
            else
               call iserrchr ( 'transport', 1 )
            end if
            call iswarsub ( 1609, 0, 0, 1 )
         end if
         ireusev  = 0
         ibuffr(ipiinblk + 8 + 10*iequa - 1) = ireusev
      end if

c     --- Reserve space for arrays isearch and isval
c         isearch(iequa) = start address of permanent storage
c                          of Krylov search directions for equation
c                          iequa. A negative value indicates that
c                          it is not yet determined if the search
c                          space is permanent or noet. The value 0
c                          indicates that the search space is not
c                          reused.
c         isval(iequa)  = similar to isearch but now for the sval space
c         ireuse(iequa) = number of search directions stored for
c                          equation iequa

      if ( ifirst.eq.1 ) then

         call isendspc(ipisearch, 3*neq, nbufol)
         ipisval = ipisearch + neq
         ipireuse = ipisval + neq

         do i = 1, 2*neq

            ibuffr(ipisearch + i - 1) = -1

         end do

         do i = 1, neq

            ibuffr(ipireuse + i - 1) = 0

         end do

         ifirst = 0

      end if

c     --- create permanent space for searchsp and svalsp if needed

      ipsearsp   = ibuffr(ipisearch + iequa -1)
      ipsvalsp   = ibuffr(ipisval   + iequa -1)

      if ( nerror.ne.0 ) go to 1000

      if ( ipsearsp.lt.0 ) then

         if ( nreuse.ne.0 ) then

c        --- search directions are reused, reserve permanent storage

            call isendspc(ipsearsp, nmbsol*itrunc*intlen, idummy)
            if ( nerror.ne.0 ) go to 1000
            ipsearsp = (ipsearsp + intlen - 1) / intlen
            ibuffr(ipisearch + iequa - 1) = ipsearsp
            nbufol = ipsearsp

            if ( ireusev.ne.0 ) then

c           --- reuse value vectors (v = A*s) are reused
c               reserve permanent storage

               call isendspc(ipsvalsp, nmbsol*itrunc*intlen, idummy)
               if ( nerror.ne.0 ) go to 1000
               ipsvalsp = (ipsvalsp + intlen - 1 ) / intlen
               ibuffr(ipisval + iequa - 1) = ipsvalsp
               nbufol = ipsvalsp

            else

c           --- reuse value vectors (v = A*s) are not reused

               ibuffr(ipisval + iequa - 1) = 0
               ipsvalsp = 0

            end if

         else

c        --- search directions are not reused

            ibuffr(ipisearch + iequa - 1) = 0
            ibuffr(ipisval   + iequa - 1) = 0
            ipsearsp = 0
            ipsvalsp = 0

         end if

      end if

c     --- number of reused vectors cannot be larger than the actual
c         number of stored search directions

      nreuse = min( ibuffr(ipireuse+iequa-1), nreuse)

c     --- create workspace using isendspc
c         rhs, usolin, usol, res, searchvc, svalvc

      lwork = nmbsol*6*intlen

c     --- betas, alphas, phis, resid

      lwork = lwork + ( itrunc*itrunc +
     +                  3*itrunc +
     +                  itrunc +
     +                  itrunc +
     +                  itmax ) * intlen

c     --- svalsp

      if ( ipsvalsp.eq.0 ) then

         lwork = lwork + nmbsol*itrunc*intlen

      end if

c     --- searchsp

      if ( ipsearsp.eq.0 ) then

         lwork = lwork + nmbsol*itrunc*intlen

      end if

c     --- reserve memory

      call isendspc(ipwork, lwork, nbufol)
      if ( nerror.ne.0 ) go to 1000
      ipwork = (ipwork+intlen-1) / intlen

      iprhsd   = ipwork
      ipusol   = iprhsd + nmbsol
      ipres    = ipusol + nmbsol
      ipsearvc = ipres + nmbsol
      ipsvalvc = ipsearvc + nmbsol

      ipbetas  = ipsvalvc + nmbsol
      ipalphloc  = ipbetas + itrunc*itrunc
      ipalphglob = ipalphloc + itrunc
      ipalphsum  = ipalphglob + itrunc
      ipphis   = ipalphsum + itrunc
      ipresid  = ipphis + itrunc

      if ( ipsvalsp.eq.0 ) then

         ipsvalsp = ipresid + itmax

         if ( ipsearsp.eq.0 ) then

            ipsearsp = ipsvalsp + nmbsol*itrunc

         end if

      end if

c     --- form the initial solution and compute the right-hand side

      if ( isol.eq.0 ) then

         call isddrel ( itrans, ilev, isiinput, issolut, isitopol,
     +                  buffer(ipusol) )
         if ( nerror.ne.0 ) go to 1000

c        --- res is here used as work array

         do i = 0, nmbsol-1
            buffer(ipres+i) = 0d0
         end do

c        --- copy res to subdomain solut arrays

         call isreldd( itrans, ilev, isiinput, issolut, isitopol,
     +                 buffer(ipres) )
         if ( nerror.ne.0 ) go to 1000

c        --- perform a single domain decomposition iteration

         call ishdd ( ndim, nblocks, iequa, isiinput, isrinput,
     +                iscoor, issolut, iscoefs, iseddy, isinfbnd,
     +                isbndcon, ismatrix, isgmat, isrhside, isgeom,
     +                isitopol, isdensup, iadmin )
         if ( nerror.ne.0 ) go to 1000

c        --- copy subdomain solut arrays to rhsd

         call isddrel ( itrans, ilev, isiinput, issolut, isitopol,
     +                  buffer(iprhsd) )
         if ( nerror.ne.0 ) go to 1000

      else

         call issoldd ( nblocks, iequa, isiinput, issolut, iadmin,
     +                  buffer(ipusol) )

c        --- compute the right-hand side

         istartv = iprhsd
         do iblock = 1, nblocks

            if ( iadmin(iblock).eq.inodenr ) then

c           --- only for local blocks

               call isactive(isiinput(iblock))
               call isactive(isrhside(iblock))

               ipiinput = isingetp(isiinput(iblock))
               iprhside = isdpgetp(isrhside(iblock))

               ni       = ibuffr(ipiinput + 51 - 1)
               nj       = ibuffr(ipiinput + 52 - 1)
               nk       = ibuffr(ipiinput + 53 - 1)
               nusol    = (ni+1)*(nj+1)*(nk+1)
               nusol2   = nusol
               if ( iequa.eq.1 ) nusol2 = ndim*nusol

               do i = 0, nusol2-1
                  buffer(istartv+i) = buffer(iprhside+i)
               end do

               istartv = istartv + nusol2

            end if

         end do

      end if

c     --- the right-hand side b and rout are identical on entry to
c         GCR. b and urout are still passed separately to improve
c         readability of the GCR code.
c          - b always means right-hand side of system of equations
c            and is used by GCR
c          - rout always means -L(uinit) and is used for computing
c            the matrix-vector product
c         usolin = uinit. This parameter must be passed to GCR because
c         it is needed for computing the matrix-vector product with
c         accurate solution of subdomains

      call isblkgcr ( nmbsol, itmax, irestart, itrunc, eps2,
     +                ioutp, rnorm, buffer(iprhsd), buffer(ipusol),
     +                buffer(ipres), buffer(ipsearsp), buffer(ipsvalsp),
     +                buffer(ipsearvc), buffer(ipsvalvc),
     +                buffer(ipbetas), buffer(ipalphloc),
     +                buffer(ipalphglob), buffer(ipalphsum),
     +                buffer(ipphis),   buffer(ipresid),
     +                iequa, isiinput, isrinput,
     +                iscoor, issolut, iscoefs, iseddy  , isinfbnd,
     +                isbndcon, ismatrix, isgmat, isrhside, isgeom,
     +                isitopol, isdensup, iadmin, nreuse, ireusev )

c     --- save nreuse

      ibuffr(ipireuse + iequa -1) = nreuse

      if ( isol.ne.0 ) then

c     --- place final solution back into the subdomain solution arrays

         call isddsol ( nblocks, iequa, isiinput, issolut,
     +                  isitopol, iadmin, buffer(ipusol) )

      end if

c     --- extrapolate solution to virtual points

      do iblock = 1, nblocks

         iblockrf = iblock
         if( iadmin(iblock).eq.inodenr) then

            if ( iequa.lt.iseqpres ) then
               itrans = iequa
            else
               itrans = iequa + ( ndim - 1 )*( 1 - icoupled )
            end if

            if ( nerror.ne.0 ) go to 1000

            usebndcn = iequa.ne.iseqpres
            call isfilvso ( iblock, itrans, ntimlv, nblocks,
     +                      isiinput(iblock), issolut(iblock),
     +                      isitopol, isinfbnd(iblock),
     +                      isbndcon(iblock,iequa), icoupled,
     +                      usebndcn )
            call isactive(isiinput(iblock))
            call isactive(issolut(iblock))
            ipiinput = isingetp(isiinput(iblock))
            ipsolut  = isdpgetp(issolut(iblock) )
            call islsolpr ( iequa, ibuffr(ipiinput), buffer(ipsolut) )

            if ( nerror.ne.0 ) go to 1000
         end if

      end do

c     --- reset ibuffr to its original length

      call islreset(nbufol)

1000  call iserclos('isblksol')

      return
      end
