      function isddre02 ( iblock, itrans, ilev, ni, nj, nk,
     +                    ntraq1, ntraq2, nvirtual,
     +                    nblocks, nfaces, maxnsubf, idiscret,
     +                    traq, nsubfaces, subfaces, relvar )
c ======================================================================
c
c     programmer   Erik Brakkee
c     version      1.4  27-02-1997 Kees K: extension 3D
c     version      1.3  12-12-1996 Kees K: idiscret added
c     version      1.2  13-03-1996 Guus: new call to isddvget
c     version      1.1  22-11-1994 Erik: extended for coefs.
c                                      changed solut into traq
c     version      1.0  07-04-1994
c
c   copyright (c) 1994-1996  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     This routine copies the virtual unknowns of certain type of
c     a certain block to a single one-dimensional array.
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iblock, ilev, isddre02, itrans, maxnsubf,
     +      nblocks, ntraq1, nfaces, ni, nj, nk, ntraq2, nvirtual
      integer nsubfaces(1:nblocks, 1:nfaces), idiscret
      double precision relvar(*)
      integer subfaces(1:nblocks, 1:nfaces, 1:maxnsubf, *)
      double precision traq(*)

c     iblock    i   Block sequence number
c     idiscret  i    Type of discretization
c     ilev      i   Time level of the multi-block transport of information
c     isddre02  o   Number of unknowns copied
c     itrans    i   reserved
c     maxnsubf  i   Maximum number of subfaces encountered on all subfaces
c     nblocks   i   Number of blocks
c     nfaces    i   Number of faces
c     ni        i   Number of non-virtual cells in the i-direction (one block)
c     nj        i   Number of non-virtual cells in the j-direction (one block)
c     nk        i   Number of non-virtual cells in the k-direction (one block)
c     nsubfaces i   Array containing the number of subfaces of each face,
c                   see array itopol for an extended description
c     ntraq1    i   Dimensioning parameter of the TRAQ array.
c                   (ntraq1 = ndegfd for solut, ncoefs for coefs)
c     ntraq2    i   Dimensioning parameter of the TRAQ array.
c                   (ntraq2 = ntimlv for solut, 1 for coefs)
c     nvirtual  i   Number of rows of virtual cells
c     relvar    o   Array containing copied virtual unknowns
c     subfaces  i   reserved
c     traq      i   reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer iface, istrtrel, isubface

c     iface     Face number
c     istrtrel  First free position in relvar
c     isubface  Subface number
c **********************************************************************
c
c                       I/O
c
c      None
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isddvget, isddvg03

c     ISDDVG03  Get virtual unknowns from a certain subface of a block 3D
c     ISDDVGET  Get virtual unknowns from the arrays of virtual unknowns
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c

      istrtrel = 1

      do iface = 1, nfaces
         do isubface = 1, nsubfaces(iblock,iface)
            if ( subfaces(iblock, iface, isubface, 5).ne.0 ) then

               if ( nk.eq.0 ) then
                  istrtrel = istrtrel +
     +            isddvget ( iblock, iface,  isubface, itrans, ilev, ni,
     +                    nj, ntraq1, ntraq2, nvirtual, nblocks,
     +                    nfaces, maxnsubf, idiscret, traq, subfaces,
     +                    relvar(istrtrel) )
               else
                  istrtrel = istrtrel +
     +            isddvg03 ( iblock, iface,  isubface, itrans, ilev, ni,
     +                    nj, nk, ntraq1, ntraq2, nvirtual, nblocks,
     +                    nfaces, maxnsubf, traq, subfaces,
     +                    relvar(istrtrel) )
               end if

            end if
         end do
      end do

      isddre02 = istrtrel - 1

      end
