      function isddvl02 ( iblock,   nblocks, nfaces, maxnsubf,
     +                    nsubfaces, subfaces, ndim)
c ======================================================================
c
c        programmer   Erik Brakkee
c        version 1.1    date 07-04-1997 Kees K. 3D
c        version 1.0    date 07-04-1994
c
c
c   copyright (c) 1994  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c **********************************************************************
c
c                       DESCRIPTION
c
c     Computes and return the parameter vlen, the number
c     of virtual tangential velocity unknowns, of a certain block.
c **********************************************************************
c
c                       KEYWORDS
c
c     topology
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iblock, isddvl02, maxnsubf, nblocks, nfaces, ndim
      integer nsubfaces(1:nblocks, 1:nfaces)
      integer subfaces(1:nblocks, 1:nfaces, 1:maxnsubf, *)

c     iblock       i   Block sequence number
c     isddvl02     o   Vlen parameter of block iblock
c     maxnsubf     i   Maximum number of subfaces encountered on all subfaces
c     nblocks      i   Number of blocks
c     ndim         i    Dimension of space (2 or 3)
c     nfaces       i   Number of faces
c     nsubfaces    i   Array containing the number of subfaces of each face,
c                      see array itopol for an extended description
c     subfaces    i/o  reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer iface, isubface, vlen

c     iface      Face number
c     isubface   Subface number
c     vlen       Number of virtual tangential velocities needed from
c                neighbouring blocks
c **********************************************************************
c
c                       I/O
c
c      None
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c      None
c **********************************************************************
c
c                       ERROR MESSAGES
c
c      None
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c

      vlen = 0

      do iface = 1, nfaces
         do isubface = 1, nsubfaces(iblock, iface)
            if ( subfaces(iblock, iface, isubface, 5 ).ne.0 ) then
               if( ndim.eq.2 ) then
                  vlen = vlen + subfaces(iblock, iface, isubface, 2) -
     +                        subfaces(iblock, iface, isubface, 1) + 2
               else
                  vlen = vlen + (subfaces(iblock, iface, isubface, 2) -
     +                        subfaces(iblock, iface, isubface, 1) + 2 )
     +                        * (subfaces(iblock, iface, isubface, 4) -
     +                        subfaces(iblock, iface, isubface, 3) + 2 )
               end if

            end if
         end do
      end do

      isddvl02 = vlen

      end
