      subroutine ishplain(ieq , isiinput, isrinput, iscoor  ,
     +                    issolut , iscoefs , iseddy  , isinfbnd,
     +                    isbndcon, ismatrix, isgmat  , isrhside,
     +                    isgeom  , isitopol, isdensup, iadmin )
c ======================================================================
c
c     programmer   Erik Brakkee
c     version      2.0  23-11-1998 Jason: Removed iaction
c     version      1.9  19-05-1998 Duncan  extension for saving
c                                          coefficients dens upwind
c                                          new call to ishdd
c     version      1.8  18-05-1998 Charles Bilinear discretization
c     version      1.7  13-10-1997 Charles uncoupled allowed
c     version      1.6  04-03-1997 Kees K. extension 3D
c     version      1.5  11-12-1996 Kees K. wesbeek discr.
c     version      1.4  18-07-1995 Erik: extended error message with
c                                  equation number
c     version      1.3  06-04-1995 Kees K. isitopol not an array
c     version      1.2  06-02-1995 Michiel K:array raction removed
c     version      1.1  03-11-1994 Kees K. coefs and geom added
c     version      1.0  20-05-1994 Erik: initial revision
c
c   copyright (c) 1994-1997  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Solve a domain decomposition problem for a certain type of unknown
c     using isendspc to allocate memory.
c     In case of single block method this subroutine calls the linear solver
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ieq, iadmin(*)  , isbndcon(*), iscoor(*)  ,
     +        iseddy(*)  , isgmat(*)  , isiinput(*), isinfbnd(*),
     +        isitopol   , ismatrix(*), isrhside(*), isrinput(*),
     +        issolut(*) , iscoefs(*) , isgeom(*), isdensup(*)

c     ieq       i    Equation number
c     iadmin    i    reserved
c     isbndcon  i    Array of sequence numbers (nblocks) of array bndcon
c     iscoefs   i    Array of sequence numbers (nblocks) of array coefs
c     iscoor    i    Array of sequence numbers (nblocks) of array coor
c     isdensup  i    Array of sequence numbers (nblocks) of array densup
c     iseddy    i    Array of sequence numbers (nblocks) of array eddy
c     isgeom    i    Array of sequence numbers (nblocks) of array geom
c     isgmat    i    Array of sequence numbers (nblocks) of array gmat
c     isiinput  i    Array of sequence numbers (nblocks) of array iinput
c     isinfbnd  i    Array of sequence numbers (nblocks) of array infbnd
c     isitopol  i    Sequence number of array itopol
c     ismatrix  i    Array of sequence numbers (nblocks) of array matrix
c     isrhside  i    Array of sequence numbers (nblocks) of array rhside
c     isrinput  i    Array of sequence numbers (nblocks) of array rinput
c     issolut   i    Array of sequence numbers (nblocks) of array solut
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, iblock, ipiinblk, ipiinput, ipnew, ipold,
     +      iprinblk, iprinput, ipwork, iter, itnlim,
     +      itrans, lenwork, nr, nblocks, nbufol, ntimlv,
     +      vlen, ipiindsc, idiscret, ndim, icoupled, iseqpres
      double precision diffdd, epsdd, epsabs

c     diffdd    2-norm of difference between 2 consecutive domain decomposition
c               iterations
c     epsabs    Required absolute accuracy
c     epsdd     Required relative accuracy
c     i         General loop variable
c     iblock    Block sequence number
c     icoupled  If 0 the momentum equations are coupled, if 1 they are not
c     idiscret  Type of discretization
c     ipiinblk  Starting address of array IINBLK in array IINPUT
c     ipiindsc  Starting address of array IINDSC in array IINPUT
c     ipiinput  Starting address of array iinput in ibuffr
c     ipnew     Starting address of new array in IBUFFR
c     ipold     Starting address of old array in IBUFFR
c     iprinblk  Starting address of array RINBLK in array RINPUT
c     iprinput  Starting address of array rinput in buffer
c     ipwork    Starting address of array work in buffer
c     iseqpres  Sequence number of pressure equation
c     iter      The current iteration number.
c     itnlim    Maximum number of iterations allowed
c     itrans    reserved
c     lenwork   Length of a work array
c     nblocks   Number of blocks
c     nbufol    Value of NBUFFR at input
c               This value may be used to reset NBUFFR in a later stage
c     ndim      Dimension of space (2 or 3)
c     nr        No. of relevant variables needed
c     ntimlv    Number of time levels
c     vlen      Number of virtual tangential velocities needed from
c               neighbouring blocks
c **********************************************************************
c
c                       I/O
c
c      None
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isdpgetp, isingetp
      double precision dnrm2

c     DAXPY     Vector a*X + Y
c     DNRM2     Compute 2-norm of a vector
c     ISACTIVE  Activate old permanent array
c     ISDDREL   Copy virtual unknowns from a single vector of relevant
c               unknowns to the subdomain solut arrays
c     ISDPGETP  get the pointer corresponding to a double precision array
c     ISENDSPC  Reserve space at the end of the buffer array
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERREAL  Put real value in error message
c     ISERRINT  Put integer value in error message
c     ISHDD     Perform a single domain decomposition iteration on the host
c     ISINGETP  get the pointer corresponding to an integer array
c     ISLRESET  Set the buffer array to its old length
c     ISRELDD   Copy virtual unknowns from the subdomain solut arrays to a
c               single vector of relevant unknowns
c     ISWARSUB  Print warnings
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     1604:  no convergence in the unaccelerated multi-block algorithm
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c

      call iseropen('ishplain')

c     --- compute pointers

      call isactive(isiinput(1))
      call isactive(isrinput(1))
      ipiinput = isingetp(isiinput(1))
      iprinput = isdpgetp(isrinput(1))
      ipiinblk = ibuffr(ipiinput + 7 - 1) + ipiinput - 1
      iprinblk = ibuffr(ipiinput + 32 - 1) + iprinput - 1

      ipiindsc = ibuffr(ipiinput-1+6)
      idiscret = ibuffr(ipiinput-1+ipiindsc+1)

c     --- determine some general parameters for the multi-block process

      ndim     = ibuffr(ipiinput + 57 - 1)
      nblocks  = ibuffr(ipiinput + 73 - 1)
      ntimlv   = ibuffr(ipiinput + 64 - 1)
      icoupled = ibuffr(ipiinput + 76 - 1)
      if ( icoupled.eq.0 ) then
         iseqpres = 2
      else
         iseqpres = ndim + 1
      end if
      itnlim   = ibuffr(ipiinblk + 2 + 10*ieq - 1)
      epsdd    = buffer(iprinblk + 1 + 10*(ieq - 1) - 1)
      if ( ieq.lt.iseqpres ) then
         itrans = 1
      else
         itrans = ieq + ( ndim - 1 )*( 1 - icoupled )
      end if

c     --- compute the dimension of the problem for GMRESR

      nr = 0
      do iblock = 1, nblocks

c     --- set iblockrf for error messages

         iblockrf = iblock

c        --- compute vlen of block iblock

         call isactive(isiinput(iblock))
         ipiinput = isingetp(isiinput(iblock))
         vlen     = ibuffr(ipiinput + 58 - 1)
         nr       = nr + vlen

      end do

c     --- twice as many virtual unknowns for the momentum equation
c         In case of single block nr = 0

      if ( ndim.eq.2) then
         if ( idiscret.eq.2 .or. idiscret.eq.4 ) then

c        --- Wesbeek discretization

            if ( ieq.eq.1 ) nr = 3*nr

         else

            if ( ieq.eq.1 ) nr = 2*nr

         end if

      else
         if ( ieq.eq.1 ) nr = 3*nr
      end if

c     --- create workspace using isendspc
c         Two vectors of size n are needed: one to store the initial guess
c         at each iteration and one to store the result after each iteration
c         Note: the +1 is there because n = 0 (single block without periodic
c         boundary conditions) gives problems with memory management.

      lenwork = (2*nr+1)*intlen
      call isendspc(ipwork, lenwork, nbufol)
      ipwork = (ipwork+intlen-1) / intlen
      ipold  = ipwork
      ipnew  = ipwork + nr

c     --- initialize the array containing old values

      if ( nblocks.gt.1 )
     +   call isddrel ( itrans, ntimlv, isiinput, issolut, isitopol,
     +                  buffer(ipold) )

      epsabs = epsdd*epsmac
      diffdd   = epsabs*10
      iter   = 1

c     --- main loop

      if ( nr.ne.0 ) then
         write(irefwr,*) ' '
         write(irefwr,*)
     +    'THE UNACCELERATED MULTI-BLOCK ALGORITHM FOR EQUATION ',
     +    ieq
         write(irefwr,*) ' '
      end if

      do while ( diffdd.gt.epsabs .and. iter.le.itnlim )

c     --- compute a single iteration
c         First the virtual unknowns are copied

         if ( nblocks.gt.1 )
     +      call isreldd ( itrans, ntimlv, isiinput, issolut, isitopol,
     +                     buffer(ipold) )

c        --- Actual solution of linear system

         call ishdd ( ndim, nblocks , ieq , isiinput, isrinput,
     +                iscoor  ,issolut , iscoefs , iseddy  , isinfbnd,
     +                isbndcon,ismatrix, isgmat  , isrhside, isgeom  ,
     +                isitopol, isdensup, iadmin )

         if ( nblocks.gt.1 ) then

c        --- Multi-block only

            call isddrel ( itrans, ntimlv, isiinput, issolut, isitopol,
     +                     buffer(ipnew) )

c           --- overwrite old array with difference between consecutive
c               iterations

            call daxpy(nr, -1d0, buffer(ipnew), 1, buffer(ipold), 1)

            diffdd = dnrm2(nr, buffer(ipold), 1)

c           --- compute required absolute accuracy

            if ( iter.eq.1 ) then
               epsabs = epsdd*diffdd
               if ( nr.ne.0 ) write(irefwr,1000) epsabs
            end if

            if ( nr.ne.0 ) write(irefwr,1010) iter, diffdd

c           --- copy result to old values at next iteration

            do i = 0, nr-1
               buffer(ipold + i) = buffer(ipnew + i)
            end do

         else

c        --- Single block

            diffdd = 0d0

         end if

c        --- next iteration

         iter = iter + 1

      end do

c     --- check if convergence

      if ( nblocks.gt.1 .and. iter.gt.itnlim ) then

         call iserrint(ieq, 1)
         call iserrint(itnlim, 2)
         call iserreal(epsdd, 1)
         call iserreal(epsabs, 2)
         call iswarsub(1604, 2, 2, 0)

      else

         if ( nr.ne.0 ) then
            write(irefwr,*) 'CONVERGED FOR EQUATION ', ieq
         end if

      end if

c     --- reset ibuffr to its original length

      call islreset(nbufol)

      call iserclos('ishplain')

1000  format(SS, 1P, 'REQUIRED ABSOLUTE ACCURACY IS ', E20.6E2)
1010  format(SS, 1P, 'ITERATION ', I4, ' 2-nrm of diff ', E20.6E2)

      end
