      subroutine isrmv ( iequa, ndim, ni, nj, nk, nvirtual, npointsnv,
     +                   npointsv, idiscret, ldegfd, nconct, ndegfd,
     +                   ntimlv, matonly, matrix, matvirt, solut,
     +                   rhside, rhsvirt, mctinv, mask,
     +                   icoupled, lmatvir )
c ======================================================================
c
c        programmer     Ronald van Nooyen
c
c        version 2.5    date 24-08-1998 Kees K mask from local to input
c        version 2.4    date 18-05-1998 Charles Bilinear discretization
c        version 2.3    date 22-12-1997 Kees V. automatic
c                                       determination of lmatvir
c        version 2.2    date 09-12-1997 Kees V. efficient memory use
c                                       for the virtual matrix elements
c        version 2.1    date 11-11-1997 Fred T. bug fixed for corner points
c        version 2.0    date 13-10-1997 Charles uncoupled allowed
c        version 1.3    date 07-01-1997 Kees K. idiscret added for
c                                       Wesbeek discretization
c        version 1.2    date 28-03-1995 Erik: rhs only copied and updated
c                                       if matonly is false.
c        version 1.1    date 22-02-1995 Erik: routine now also copies
c                                       matrix elements that are set
c                                       to zero to matvirt and boundary
c                                       rhside elements to rhsvirt
c        version 1.0    date 13-10-1994
c
c
c   copyright (c) 1994-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Fold virtual unkowns to the right-hand side.
c     Save virtual coefficients of the matrix and the unperturbed
c     right-hand side elements for reuse in subsequent domain
c     decomposition iterations.
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer maxdim
      parameter ( maxdim = 3 )
      integer iequa, ldegfd, nconct, ndegfd, ndim, ni, nj, nk,
     +   npointsnv, ntimlv, npointsv, idiscret, nvirtual, icoupled
      integer mctinv(0:maxdim,1:ldegfd,1:nconct), lmatvir
      double precision matrix(ni+1,nj+1,nk+1,1:ldegfd,1:nconct)
      double precision matvirt(lmatvir)
      double precision solut(1:npointsv,1:ndegfd,1:ntimlv)
      double precision rhside(1:npointsnv,1:ldegfd)
      double precision rhsvirt(1:npointsnv,1:ldegfd)
      logical mask(ni+1,nj+1,nk+1,1:ldegfd,1:nconct)
      logical matonly

c     icoupled  i    If 0 the momentum equations are coupled, if 1 they are not
c     idiscret  i    Type of discretization
c     iequa     i    General equation sequence number
c     ldegfd    i    Number of degrees of freedom in the matrix
c     lmatvir   i    Length of the array of virtual matrix elements
c     mask      o    logical variable to ensure that the virtual coefficients
c                    in the copy of the matrix are not overwritten
c     matonly   i    If true, only copy matrix elements and leave
c                    right-hand side alone.
c     matrix    i    reserved
c     matvirt   o    Storage for the virtual coefficients of a matrix
c     mctinv    i    Matrix information
c     nconct    i    Number of connections in a row of the matrix
c     ndegfd    i    Number of degrees of freedom
c     ndim      i    Dimension of space (2 or 3)
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     npointsnv i    Number of points in the mesh without virtual points
c     npointsv  i    Number of points in the mesh with virtual points
c     ntimlv    i    Number of time levels
c     rhside    i    reserved
c     rhsvirt   o    Storage for the elements of the right-hand side
c                    affected by virtual unknowns of neighbouring blocks
c     solut     i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      logical virtval
      integer iconct, idim, iface, ihisrc, ilosrc, jdim, jhisrc,
     +        jlosrc, khisrc, klosrc, ndir, nsig, varnr, iseqpres
      integer ntdir(1:maxdim), ntsig(1:maxdim), ilowloop, iupploop
      integer carea(1:2,1:maxdim), varea(1:2,1:maxdim),
     +        area(1:2,1:maxdim), dx(1:maxdim), i, j, k, icount

c     area      Summation index bounds
c     carea     Index bounds on set of grid cells adjacent to
c               this boundary
c     dx        Point location in cell relative to local
c               coordinates with (0,0) and (2,2) or
c               (0,0,0) and (2,2,2) as opposite cell corners
c     i         General loop variable
c     iconct    Loop variable for loop over all connections in the matrx
c     icount    Counts the number of positions necessary for the number
c     idim      Loop variable used for dimension
c     iface     Face number
c     ihisrc    Upper bound on i-index for source
c     ilosrc    Lower bound on i-index for source
c     ilowloop  Lower bound for loop
c     iseqpres  Sequence number of pressure equation
c     iupploop  Upper bound for loop
c     j         General loop variable
c     jdim      Loop variable used for dimension
c     jhisrc    Upper bound on j-index for source
c     jlosrc    Lower bound on j-index for source
c     k         General loop variable
c     khisrc    Upper bound on k-index for source
c     klosrc    Lower bound on k-index for source
c     maxdim    Highest physical space dimension for which ISNaS can work
c     ndir      Direction of normal vector on this side
c     nsig      Sign of inner product of the basis vector
c               for the above mentioned internal direction
c               and the normal
c     ntdir     Contains the ksi-alpha indices of the normal and
c               tangential directions
c               ntdir(1) refers to the normal direction
c               ntdir(2) refers to the first tangential direction
c               ntdir(3) refers to the second tangential direction
c     ntsig     Gives the direction of the normal or tangential vectors
c               with respect to the ksi co-ordinate axis
c               The normal is supposed to be the outward normal and
c               the tangential directions are taken into a counter
c               clockwise way
c               ntsig(1) refers to the normal direction
c               ntsig(2) refers to the first tangential direction
c               ntsig(3) refers to the second tangential direction
c     nvirtual  Number of rows of virtual cells
c     varea     Area of arrays
c     varnr     Variable position in solut
c     virtval   True if matrix element corresponds to virtual point
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISBNBVI   Gives information on the relation between the ISNaS
c               basis vectors and the normal and tangential vectors on
c               a given boundary. Moreover it gives lower and upper bou
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRSUB  Error messages
c     ISFILL    Fill ifillarea in dest with fillvalue
c     ISSCOPY   Copy operation
c     ISVSVVU  ?
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c    1610   Length of array matvirt is too small.
c **********************************************************************
c
c                       PSEUDO CODE
c
c     If ( lmatvir = 0 ) then
c
c        the correct value of lmatvir is determined,
c
c     else
c
c        For each boundary do:
c           Find matrix elements corresponding to values of the solution
c           in virtual points
c           Add the contribution of each virtual point to the right hand side
c           Set matrix elements corresponding to virtual points to zero
c
c     end if
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen ( 'isrmv' )

      icount = 0

c     --- Coupled or uncoupled

      if ( icoupled.eq.0 ) then
         iseqpres = 2
         ilowloop = 1
         iupploop = ndim
      else
         iseqpres = ndim + 1
         ilowloop = iequa
         iupploop = iequa
      end if

      do idim=1,ldegfd
         do iconct=1,nconct
            do k=1,nk+1
               do j=1,nj+1
                  do i=1,ni+1
                     mask(i,j,k,idim,iconct)=.TRUE.
                  end do
               end do
            end do
         end do
      end do

c     --- Get lower and upper array bounds for the solution with virtual points

      ilosrc = 1-nvirtual
      ihisrc = ni+1+nvirtual
      jlosrc = 1-nvirtual
      jhisrc = nj+1+nvirtual
      if (ndim.eq.2) then

c     --- 2D; third dimension is a dummy dimension

         klosrc = 1
         khisrc = 1
      else

c     --- bounds for third dimension

         klosrc = 1-nvirtual
         khisrc = nk+1+nvirtual
      end if

c     --- save right-hand side values

      if ( .not. matonly ) then
         do iface=1,2*ndim

c        --- Get the normal and tangential directions (ntdir,ntsig), the
c            grid cells adjacent to the boundary (carea) and the
c            vertices on the boundary (varea)

            call isbnbvi(ndim, ni, nj, nk, iface, ntdir, ntsig,
     +         carea, varea)

c           --- Get direction and sign of outward normal

            ndir = ntdir(1)
            nsig = ntsig(1)

            if ( iequa.lt.iseqpres ) then

c           --- Momentum matrix
c               If coupled, idim=1,ndim, else idim=iequa,iequa

               do idim = ilowloop, iupploop

c              --- Normal cell matrix elements

                  if ( idim.eq.ndir ) then

                     area(1,1) = carea(1,1)
                     area(2,1) = carea(2,1)
                     area(1,2) = carea(1,2)
                     area(2,2) = carea(2,2)
                     area(1,3) = carea(1,3)
                     area(2,3) = carea(2,3)

c                    --- with the ndir-index on the face:

                     area(1,ndir) = varea(1,ndir)
                     area(2,ndir) = varea(2,ndir)

                     if ( idiscret.eq.2 .or. idiscret.eq.4 ) then

c                    --- Wesbeek discretization , one extra row
c                        of normal cells at a distance one from
c                        the boundary

                        if ( nsig.lt.0 ) then
                           area(2,ndir) = area(2,ndir) + 1
                        else
                           area(1,ndir) = area(1,ndir) - 1
                        end if

                     end if

                  else

                     area(1,1) = carea(1,1)
                     area(2,1) = carea(2,1)
                     area(1,2) = carea(1,2)
                     area(2,2) = carea(2,2)
                     area(1,3) = carea(1,3)
                     area(2,3) = carea(2,3)

c                    --- and including the virtual cell at the upper
c                        edge for direction idim

                     area(1,idim) = varea(1,idim)
                     area(2,idim) = varea(2,idim)

                  end if

c                 --- save right-hand side values

                  call isscopy(1,ni+1,1,nj+1,1,nk+1,
     +                         1,ni+1,1,nj+1,1,nk+1,
     +                         1,area,rhside(1,idim),
     +                         rhsvirt(1,idim), 1d0)

               end do

            else

c           --- Pressure or transport matrix

               area(1,1) = carea(1,1)
               area(2,1) = carea(2,1)
               area(1,2) = carea(1,2)
               area(2,2) = carea(2,2)
               area(1,3) = carea(1,3)
               area(2,3) = carea(2,3)

c              --- save right-hand side values

               call isscopy(1,ni+1,1,nj+1,1,nk+1,
     +                      1,ni+1,1,nj+1,1,nk+1,
     +                      1,area,rhside(1,1),
     +                      rhsvirt(1,1), 1d0)

            end if

         end do

      end if

c     --- Fold virtual unkowns, Loop over boundaries

      do iface=1,2*ndim

c     --- Get the normal and tangential directions (ntdir,ntsig), the
c         grid cells adjacent to the boundary (carea) and the
c         vertices on the boundary (varea)

         call isbnbvi(ndim, ni, nj, nk, iface, ntdir, ntsig,
     +      carea, varea)

c        --- Get direction and sign of outward normal

         ndir = ntdir(1)
         nsig = ntsig(1)

c        --- Loop over connections

         do iconct=1,nconct

            if ( iequa.lt.iseqpres ) then

c           --- Momentum matrix
c               If coupled, idim=1,ndim, else idim=iequa,iequa

               do idim = ilowloop, iupploop

c              --- Velocity component corresponding to this matrix element

                  jdim  = mctinv(0,idim,iconct)

c                 --- Offset of grid cell containing this matrix element
c                     relative to the gird cell corresponding to the
c                     matrix element on the matrix diagonal

                  dx(1) = mctinv(1,idim,iconct)
                  dx(2) = mctinv(2,idim,iconct)
                  dx(3) = mctinv(3,idim,iconct)

                  if (idim.eq.ndir) then

c                 --- Normal cell matrix elements

                     if (nsig.lt.0) then

c                    --- Lower edge

                        if (jdim.eq.ndir) then

c                       --- Flux in normal direction

                           virtval = (dx(ndir).lt.0)
                        else

c                       --- Flux in tangential direction

                           virtval = (dx(ndir).lt.0)
                        end if
                     else

c                    --- Upper edge

                        if (jdim.eq.ndir) then

c                       --- Flux in normal direction

                           virtval = (dx(ndir).gt.0)
                        else

c                       --- Flux in tangential direction

                           virtval = (dx(ndir).ge.0)
                        end if
                     end if

c                    --- The cell indices of the diagonal matrix element
c                        along this edge are the grid cells along this
c                        edge

                     area(1,1) = carea(1,1)
                     area(2,1) = carea(2,1)
                     area(1,2) = carea(1,2)
                     area(2,2) = carea(2,2)
                     area(1,3) = carea(1,3)
                     area(2,3) = carea(2,3)

c                    --- with the ndir-index on the face:

                     area(1,ndir) = varea(1,ndir)
                     area(2,ndir) = varea(2,ndir)

                  else

c                 --- Tangential cell matrix elements

                     if (nsig.lt.0) then

c                    --- Lower edge

                        if (jdim.eq.ndir) then

c                       --- Flux in normal direction

                           virtval = (dx(ndir).lt.0)
                        else

c                       --- Flux in tangential direction

                           virtval = (dx(ndir).lt.0)
                        end if
                     else

c                    --- Upper edge

                        if (jdim.eq.ndir) then

c                       --- Flux in normal direction

                           virtval = (dx(ndir).gt.1)
                        else

c                       --- Flux in tangential direction

                           virtval = (dx(ndir).gt.0)
                        end if
                     end if

c                    --- The cell indices of the diagonal matrix element
c                        along this edge are the grid cells along this
c                        edge including the first cell in direction idim

                     area(1,1) = carea(1,1)
                     area(2,1) = carea(2,1)
                     area(1,2) = carea(1,2)
                     area(2,2) = carea(2,2)
                     area(1,3) = carea(1,3)
                     area(2,3) = carea(2,3)

c                    --- and including the virtual cell at the upper
c                        edge for direction idim

                     area(1,idim) = varea(1,idim)
                     area(2,idim) = varea(2,idim)

                  end if

                  if (virtval) then

c                 --- The current type of matrix element
c                     corresponds to a virtual value in solut
c                     Rhside(ip,jp,kp,idim) = Rhside(ip,jp,kp,idim) -
c                     Matrix(ip,jp,kp,idim,iconct)*
c                     Solut(ip+dx(1),jp+dx(2),kp+dx(3),jdim,ntimlv)

                     if ( .not. matonly .and. lmatvir.gt.0 ) then

                        call isvsvvu(ndim,.true.,rhside(1,idim),-1d0,
     +                   matrix(1,1,1,idim,iconct),solut(1,jdim,ntimlv),
     +                     1,ni+1,1,nj+1,1,nk+1,
     +                     1,ni+1,1,nj+1,1,nk+1,
     +                     ilosrc-dx(1), ihisrc-dx(1),
     +                     jlosrc-dx(2), jhisrc-dx(2),
     +                     klosrc-dx(3), khisrc-dx(3),
     +                     area)

                     end if

c                    --- save virtual coefficients

                     do k=area(1,3),area(2,3)
                        do j=area(1,2),area(2,2)
                           do i=area(1,1),area(2,1)
                              if(mask(i,j,k,idim,iconct)) then
                                 mask(i,j,k,idim,iconct)=.FALSE.
                                 icount = icount + 1
                                 if ( lmatvir.gt.0 ) then
                                    matvirt(icount)=
     +                               matrix(i,j,k,idim,iconct)
                                 end if
                              end if
                           end do
                        end do
                     end do

c                    --- Matrix(ip,jp,kp,idim,iconct) = 0d0

                     if ( lmatvir.gt.0 ) then
                        call isfill(1,ni+1,1,nj+1,1,nk+1,
     +                  1,area,matrix(1,1,1,idim,iconct),0d0)
                     end if
                  end if

                  if ( idiscret.eq.2 .or. idiscret.eq.4  ) then

c                 --- Wesbeek discretization

                     virtval = .false.

                     if (idim.eq.ndir) then

c                    --- Normal cell matrix elements

                        if (nsig.lt.0) then

c                       --- Lower edge

                           if (jdim.ne.ndir) then

c                          --- Flux in tangential direction

                              virtval = (dx(ndir).lt.-1)
                           end if
                        else

c                       --- Upper edge

                           if (jdim.ne.ndir) then

c                          --- Flux in tangential direction

                              virtval = (dx(ndir).gt.0)
                           end if
                        end if

c                       --- The cell indices of the diagonal matrix element
c                           along this edge are the grid cells along this
c                           edge

                        area(1,1) = carea(1,1)
                        area(2,1) = carea(2,1)
                        area(1,2) = carea(1,2)
                        area(2,2) = carea(2,2)
                        area(1,3) = carea(1,3)
                        area(2,3) = carea(2,3)

c                       --- with the ndir-index NOT on the face:

                        if ( nsig.lt.0 ) then
                           area(1,ndir) = area(1,ndir) + 1
                           area(2,ndir) = area(2,ndir) + 1
                        end if

                     end if

                     if (virtval) then

c                    --- The current type of matrix element
c                        corresponds to a virtual value in solut
c                        Rhside(ip,jp,kp,idim) = Rhside(ip,jp,kp,idim)
c                        Matrix(ip,jp,kp,idim,iconct)*
c                        Solut(ip+dx(1),jp+dx(2),kp+dx(3),jdim,ntimlv)

                        if ( .not. matonly .and. lmatvir.gt.0 ) then

                           call isvsvvu(ndim,.true.,rhside(1,idim),-1d0,
     +                        matrix(1,1,1,idim,iconct),
     +                        solut(1,jdim,ntimlv),
     +                        1,ni+1,1,nj+1,1,nk+1,
     +                        1,ni+1,1,nj+1,1,nk+1,
     +                        ilosrc-dx(1), ihisrc-dx(1),
     +                        jlosrc-dx(2), jhisrc-dx(2),
     +                        klosrc-dx(3), khisrc-dx(3),
     +                        area)

                        end if

c                       --- save virtual coefficients

                        do k=area(1,3),area(2,3)
                           do j=area(1,2),area(2,2)
                              do i=area(1,1),area(2,1)
                                 if(mask(i,j,k,idim,iconct)) then
                                    mask(i,j,k,idim,iconct)=.FALSE.
                                    icount = icount + 1
                                    if ( lmatvir.gt.0 ) then
                                       matvirt(icount)=
     +                                 matrix(i,j,k,idim,iconct)
                                    end if
                                 end if
                              end do
                           end do
                        end do

c                       --- Matrix(ip,jp,kp,idim,iconct) = 0d0

                        if ( lmatvir.gt.0 ) then
                           call isfill(1,ni+1,1,nj+1,1,nk+1,
     +                     1,area,matrix(1,1,1,idim,iconct),0d0)
                        end if
                     end if

                  end if

               end do
            else

c           --- Pressure or transport matrix
c               Get variable position in solut

               if ( icoupled.eq.0 ) then
                  varnr = iequa-1+ndim
               else
                  varnr = iequa
               end if

c              --- Offset of grid cell containing this matrix element
c                  relative to the gird cell corresponding to the
c                  matrix element on the matrix diagonal

               dx(1) = mctinv(1,1,iconct)
               dx(2) = mctinv(2,1,iconct)
               dx(3) = mctinv(3,1,iconct)

c              --- The cell indices of the diagonal matrix element
c                  along this edge are the grid cells along this edge

               area(1,1) = carea(1,1)
               area(2,1) = carea(2,1)
               area(1,2) = carea(1,2)
               area(2,2) = carea(2,2)
               area(1,3) = carea(1,3)
               area(2,3) = carea(2,3)
               virtval = (dx(ndir)*nsig.gt.0)
               if (virtval) then

c              --- The current type of matrix element
c                  corresponds to a virtual value in solut
c                  Rhside(ip,jp,kp,1) = Rhside(ip,jp,kp,1) -
c                  Matrix(ip,jp,kp,1,iconct)*
c                  Solut(ip+dx(1),jp+dx(2),kp+dx(3),nvarnr,ntimlv)

                  if ( .not. matonly .and. lmatvir.gt.0 ) then

                     call isvsvvu(ndim,.true.,rhside(1,1),-1d0,
     +                  matrix(1,1,1,1,iconct),solut(1,varnr,ntimlv),
     +                  1,ni+1,1,nj+1,1,nk+1,
     +                  1,ni+1,1,nj+1,1,nk+1,
     +                  ilosrc-dx(1), ihisrc-dx(1),
     +                  jlosrc-dx(2), jhisrc-dx(2),
     +                  klosrc-dx(3), khisrc-dx(3),
     +                  area)

                  end if

c                 --- save virtual coefficients

                  do k=area(1,3),area(2,3)
                     do j=area(1,2),area(2,2)
                        do i=area(1,1),area(2,1)
                           if(mask(i,j,k,1,iconct)) then
                              mask(i,j,k,1,iconct)=.FALSE.
                              icount = icount + 1
                              if ( lmatvir.gt.0 ) then
                                 matvirt(icount)=
     +                            matrix(i,j,k,1,iconct)
                              end if
                           end if
                        end do
                     end do
                  end do

c                 --- Matrix(ip,jp,kp,1,iconct) = 0d0

                  if ( lmatvir.gt.0 ) then
                     call isfill(1,ni+1,1,nj+1,1,nk+1,
     +               1,area,matrix(1,1,1,1,iconct),0d0)
                  end if

               end if
            end if
         end do
      end do

      if ( lmatvir.eq.0 ) then

         lmatvir = icount

      else

         if ( icount.gt.lmatvir ) then
            call iserrsub ( 1610, 0, 0, 0 )
         end if

      end if

      call iserclos ( 'isrmv' )
      end
