      subroutine issnddhs ( nblocks, itrans, ilev, ind, isiinput,
     +                      istraq, isitopol, iadmin, done )
c ======================================================================
c
c     programmer   Erik Brakkee
c     version      2.0 05-10-1999 Jason: send-requests added
c     version      1.9 23-11-1998 Jason: issenmpi version
c     version      1.8 31-08-1998 Jason: removed check nodeid
c     version      1.7 18-05-1998 Charles Bilinear discretization
c     version      1.6 27-02-1997 Kees K.: extension 3D
c     version      1.5 14-01-1997 Kees K.: arraylength changed for
c                                          wesbeek discretization
c     version      1.4 13-12-1996 Kees K: idiscret added
c     version      1.3 13-03-1996 Guus: new call to iscorn02
c     version      1.2 22-11-1994 Erik: extended for coefs
c                                     changed solut into traq
c     version      1.1 24-08-1994 Erik: internal representation of
c                                     virt arrays no longer used
c     version      1.0 02-05-1994
c
c   copyright (c) 1994-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     The purpose of this routine is to copy the arrays of virtual
c     unknowns for all blocks to the local copies at the node with
c     sequence number ind. If ind corresponds to the host, nothing
c     is done. The message to node ind should already be initialized
c     on entry to this subroutine. At return, the message is
c     not terminated and can still be appended to.
c     If a node does not reside at the host, routine issend is used.
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iadmin(*), ilev, ind, isiinput(*), isitopol,
     +        istraq(*), itrans, nblocks
      logical done

c     done      i    indicates the node domain decomposition should stop
c     iadmin    i    reserved
c     ilev      i    Time level of the multi-block transport of information
c     ind       i    Sequence number of node
c     isiinput  i    Array of sequence numbers (nblocks) of array iinput
c     isitopol  i    Sequence number of array itopol
c     istraq    i    Array of sequence numbers (nblocks) of array traq
c     itrans    i    reserved
c     nblocks   i    Number of blocks
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisnas'
      include 'IScommon/cisparal'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer  i, iarray(3), iblock, issmsg, ipiinput, ipitopol,
     +         ipnsubf, ipsmsg, ipsubf, iptraq, lenact,
     +         lengmes, lensmsg, maxnsubf, ntraq1, nfaces, ni, nj, nk,
     +         ntraq2, nvirtual, vlen, ipiindsc, idiscret, ndim,
     +         ireq(2*nblocks+1), isencnt

c     i         General loop variable
c     iarray    Integer array for issend and isreceiv
c     iblock    Block sequence number
c     idiscret  Type of discretization
c     ipiindsc  Starting address of array IINDSC in array IINPUT
c     ipiinput  Starting address of array iinput in ibuffr
c     ipitopol  Starting address of array itopol in ibuffr
c     ipnsubf   Starting address of the nsubfaces array
c     ipsmsg    Starting address of array smsg in buffer
c     ipsubf    Starting address of the subfaces array
c     iptraq    Starting address of array traq in buffer
c     ireq      Array of send requests
c     isencnt   Send request counter
c     issmsg    Sequence number of array smsg
c     lenact    Actual length of the submessage (only position 1..lenact)
c               are filled with significant values)
c     lengmes   Length of the message
c     lensmsg   Length of array smsg
c     maxnsubf  Maximum number of subfaces encountered on all subfaces
c     ndim      Dimension of space (2 or 3)
c     nfaces    Number of faces
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     ntraq1    Dimensioning parameter of the TRAQ array.
c               (ntraq1 = ndegfd for solut, ncoefs for coefs)
c     ntraq2    Dimensioning parameter of the TRAQ array.
c               (ntraq2 = ntimlv for solut, 1 for coefs)
c     nvirtual  Number of rows of virtual cells
c     vlen      Number of virtual tangential velocities needed from
c               neighbouring blocks
c **********************************************************************
c
c                       I/O
c
c      none
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isddre02, isdpgetp, isingetp

c     ISACTIVE       Activate permanent array
c     ISDCREAT       Create permanent space in array IBUFFR
c                    (Double precision array)
c     ISDCRPRI       Decrease priorities of all permanent arrays
c     ISDDRE02       Get virtual unknowns from a certain block
c     ISDELARR       Delete a permanent array
c     ISDPGETP       get the pointer corresponding to a double precision array
c     ISERCLOS       Resets old name of previous subroutine of higher level
c     ISEROPEN       Produces concatenated name of local subroutine
c     ISINGETP       get the pointer corresponding to an integer array
c     ISSENMPI       Send data to node or host
c     ISWAITALL      Wait for all sends to complete
c **********************************************************************
c
c                       ERROR MESSAGES
c
c        -
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c        IF ( node ind is not solved on the host ) THEN
c           initialize the send to node ind
c           DO iblock = 1, nblocks
c              IF ( iblock on node ind ) THEN
c                 Send virtual unknowns from block iblock to node ind
c                 using pvm
c              ENDIF
c           ENDDO
c           terminate the send to node ind
c        ENDIF
c
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen('issnddhs')

      iarray(1) = 0
      iarray(2) = 0
      iarray(3) = 0

c     --- create an array for storing a submessage and activate some arrays

      lensmsg = 0
      do iblock = 1, nblocks
         call isactive(isiinput(iblock))
         ipiinput = isingetp(isiinput(iblock))
         lensmsg  = max(lensmsg, 3*ibuffr(ipiinput + 58 - 1))
      end do
      lensmsg = lensmsg + 1
      issmsg = 0
      call isdcreat(issmsg, lensmsg, 'smsg' )

      ipiindsc     = ibuffr(ipiinput-1+6)
      idiscret     = ibuffr(ipiinput-1+ipiindsc+1)

      isencnt = 0

      if ( ind.ne.0 ) then

         do iblock = 1, nblocks

            if ( iadmin(iblock).eq.ind ) then

c           --- pass the block number to issend
c                Note: we still pass iblock explicitly in the
c                informative part of a multi-block submessage
c                for clarity of the code.

               iblockrf = iblock

c              --- activate the needed arrays

               call isactive(issmsg)
               call isactive(isitopol)
               call isactive(istraq(iblock))
               call isactive(isiinput(iblock))

c              --- compute addresses

               ipsmsg       = isdpgetp(issmsg)
               iptraq       = isdpgetp(istraq(iblock))
               ipiinput     = isingetp(isiinput(iblock))
               ipitopol     = isingetp(isitopol)
               ipnsubf  = ibuffr(ipitopol+5-1) + ipitopol - 1
               ipsubf   = ibuffr(ipitopol+6-1) + ipitopol - 1

c              --- compute some parameters

               nfaces       = ibuffr(ipitopol + 2 - 1)
               maxnsubf = ibuffr(ipitopol + 3 - 1)
               ni           = ibuffr(ipiinput + 51 - 1)
               nj           = ibuffr(ipiinput + 52 - 1)
               nk           = ibuffr(ipiinput + 53 - 1)
               ndim         = ibuffr(ipiinput + 57 - 1)
               if ( itrans.lt.0 ) then

c              --- coefs array: ntraq1 = ncoefs, ntraq2 = 0

                  ntraq1    = ibuffr(ipiinput + 66 - 1)
                  ntraq2    = 1

               else

c              --- solut array, ntraq1 = ndegfd, ntraq2 = ntimlv

                  ntraq1    = ibuffr(ipiinput + 55 - 1)
                  ntraq2    = ibuffr(ipiinput + 64 - 1)

               end if

               nvirtual     = ibuffr(ipiinput + 54 - 1)
               vlen         = ibuffr(ipiinput + 58 - 1)

c              --- send information about where the virtual unknowns
c                  are meant for

               iarray(1) = iblock
               iarray(2) = itrans
               iarray(3) = ilev
               lengmes = 3
               isencnt = isencnt + 1
               call issenmpi (iarray, lengmes, ISINT, ind, ICVCINFO,
     +             ireq(isencnt))

c              --- send virtual unknowns

               lenact =
     +            isddre02 ( iblock, itrans, ilev, ni, nj, nk, ntraq1,
     +                       ntraq2, nvirtual, nblocks, nfaces,
     +                       maxnsubf, idiscret, buffer(iptraq),
     +                       ibuffr(ipnsubf), ibuffr(ipsubf),
     +                       buffer(ipsmsg) )

               lengmes = vlen
               if ( ndim.eq.2 ) then
                  if ( idiscret.eq.2 .or. idiscret.eq.4 ) then
                     if ( itrans.eq.1 ) lengmes = lengmes*3
                  else
                     if ( itrans.eq.1 ) lengmes = lengmes*2
                  end if
               else
                  if ( itrans.eq.1 ) lengmes = lengmes*3
               end if

               do i = lenact+1, lengmes
                  buffer(ipsmsg+i-1) = 0d0
               end do

               isencnt = isencnt + 1
               call issenmpi (buffer(ipsmsg), lengmes, ISDOUBLE,
     +            ind, ICVCDATA, ireq(isencnt))

            end if

         end do

c        --- send sentinel

         iarray(1) = 0
         iarray(2) = 0
         iarray(3) = 0
         if (done) iarray(3)= -1
         lengmes = 3
         isencnt = isencnt + 1
         call issenmpi (iarray, lengmes, ISINT, ind, ICVCINFO,
     +      ireq(isencnt))
         call iswaitall (isencnt, ireq)

      end if

      call isdelarr(issmsg)
      call isdcrpri

      call iserclos('issnddhs')

      end
