      subroutine issoldd ( nblocks, iequa, isiinput, issolut, iadmin,
     +                    mbsol )
c ======================================================================
c
c     programmer    Erik Brakkee
c     version 1.2   date  27-07-1998 Guus update comment
c     version 1.1   date  08-07-1998 Kees V: nusol, nsolut cleaned
c     version 1.0   date  04-01-1995 Erik: initial revision
c
c   copyright (c) 1995-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c **********************************************************************
c
c                  DESCRIPTION
c
c     Copying of the inner region of the solution of a certain equation
c     from all blocks to a single one-dimensional array.
c
c     IMPORTANT!!!
c        The vector sol in the subroutine parameterlist may not be governed
c        by memory management routines since this routine also uses
c        memory management.
c
c **********************************************************************
c
c                  KEYWORDS
c
c **********************************************************************
c
c                  INPUT / OUTPUT PARAMETERS

      implicit none
      integer iequa, nblocks
      integer isiinput(1:nblocks), issolut(1:nblocks),
     +        iadmin(*)
      double precision mbsol(*)

c
c     iequa     i    General equation sequence number
c     isiinput  i    Array of sequence numbers (nblocks) of array iinput
c     issolut   i    Array of sequence numbers (nblocks) of array solut
c     mbsol     i    Multi-block solution vector
c     nblocks   i    Number of blocks
c **********************************************************************
c
c                   COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisparal'
c **********************************************************************
c
c
c                   LOCAL PARAMETERS
c
c
      integer i, iblock, idegfd, ipiinput, ipsolut, istartv

      integer ndegfd, ndim, ni, nj, nk, npointsv, ntimlv, nusol,
     +        nvirtual, offset, nusol2

      integer maxdim
      parameter (maxdim = 3)

      integer icparea(1:2,1:maxdim),
     +        ilosrc, ihisrc, jlosrc, jhisrc, klosrc, khisrc,
     +        ilodest, ihidest, jlodest, jhidest, klodest, khidest


c     i         General loop variable
c     iblock    Block sequence number
c     icparea   Area to be copied
c     idegfd    Loop variable with respect to a given unknown
c     ihidest   Upper bound on i-index for destination
c     ihisrc    Upper bound on i-index for source
c     ilodest   Lower bound on i-index for destination
c     ilosrc    Lower bound on i-index for source
c     ipiinput  Starting address of array iinput in ibuffr
c     ipsolut   Starting address of array solut in buffer
c     istartv   First free position in a vector
c     jhidest   Upper bound on j-index for destination
c     jhisrc    Upper bound on j-index for source
c     jlodest   Lower bound on j-index for destination
c     jlosrc    Lower bound on j-index for source
c     khidest   Upper bound on k-index for destination
c     khisrc    Upper bound on k-index for source
c     klodest   Lower bound on k-index for destination
c     klosrc    Lower bound on k-index for source
c     maxdim    Highest physical space dimension for which ISNaS can work
c     ndegfd    Number of degrees of freedom
c     ndim      Dimension of space (2 or 3)
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     npointsv  Number of points in the mesh with virtual points
c     ntimlv    Number of time levels
c     nusol     Number of unknowns for one degree of freedom in
c               one block
c     nusol2    Number of scalar unknowns for the solvers
c               nusol2 = nusol * ndimso
c     nvirtual  Number of rows of virtual cells
c     offset    General offset factor
c **********************************************************************
c
c                   I/O
c
c **********************************************************************
c
c                   SUBROUTINES CALLED
c
      integer isdpgetp, isingetp

c     ISACTIVE  Activate old permanent array
c     ISDPGETP  get the pointer corresponding to a double precision array
c     ISINGETP  get the pointer corresponding to an integer array
c     ISSCOPY   Copy operation
c **********************************************************************
c
c                   ERROR MESSAGES
c
c **********************************************************************
c
c             PSEUDO CODE
c
c     istart = 1
c
c     For iblock:= 1(1)nblocks
c
c        copy inner region of solution of equation iequa to
c        array sol starting from position istart
c
c        istart = istart + (ni+1)*(nj+1)*(nk+1)
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c

      istartv = 1

      do iblock = 1, nblocks
      
      if ( iadmin(iblock).eq.inodenr ) then

         call isactive(isiinput(iblock))
         call isactive(issolut(iblock))

         ipiinput = isingetp(isiinput(iblock))
         ipsolut  = isdpgetp(issolut(iblock))

         ni       = ibuffr(ipiinput + 51 - 1)
         nj       = ibuffr(ipiinput + 52 - 1)
         nk       = ibuffr(ipiinput + 53 - 1)
         ndim     = ibuffr(ipiinput + 57 - 1)
         ndegfd   = ibuffr(ipiinput + 55 - 1)
         ntimlv   = ibuffr(ipiinput + 64 - 1)
         nvirtual = ibuffr(ipiinput + 54 - 1)

c        --- fill background

         nusol  = (ni+1)*(nj+1)*(nk+1)
         nusol2 = nusol
         if ( iequa.eq.1 ) nusol2 = nusol*ndim
         do i = 1, nusol2
            mbsol(istartv + i - 1) = 0d0
         end do

c        --- determine degree of freedom to copy

         idegfd = iequa
         if ( iequa.gt.1 ) idegfd = idegfd + ndim - 1

c        --- initialize copy parameters
c            dimensions of source array

         ilosrc = 1-nvirtual
         ihisrc = ni+1+nvirtual
         jlosrc = 1-nvirtual
         jhisrc = nj+1+nvirtual
         if ( ndim.eq.2 ) then
            klosrc = 1
            khisrc = 1
         else
            klosrc = 1-nvirtual
            khisrc = nk+1+nvirtual
         end if

         npointsv = ( ihisrc - ilosrc + 1) *
     +              ( jhisrc - jlosrc + 1) *
     +              ( khisrc - klosrc + 1)

c        --- dimensions of target array

         ilodest = 1
         ihidest = ni+1
         jlodest = 1
         jhidest = nj+1
         klodest = 1
         khidest = nk+1

         if ( iequa.eq.1 ) then

c        --- area to be copied for V1

            icparea(1,1) = 1
            icparea(2,1) = ni+1
            icparea(1,2) = 1
            icparea(2,2) = nj
            if ( ndim.eq.2 ) then
               icparea(1,3) = 1
               icparea(2,3) = 1
            else
               icparea(1,3) = 1
               icparea(2,3) = nk
            end if

            offset = (ntimlv-1)*ndegfd + idegfd-1

            call isscopy ( ilosrc, ihisrc, jlosrc, jhisrc,
     +                     klosrc, khisrc,
     +                     ilodest, ihidest, jlodest, jhidest,
     +                     klodest, khidest,
     +                     1, icparea,
     +                     buffer(ipsolut+offset*npointsv),
     +                     mbsol(istartv), 1d0)

            istartv = istartv + (ni+1)*(nj+1)*(nk+1)
            idegfd  = idegfd + 1

c           --- area to be copied for V2

            icparea(1,1) = 1
            icparea(2,1) = ni
            icparea(1,2) = 1
            icparea(2,2) = nj+1
            if ( ndim.eq.2 ) then
               icparea(1,3) = 1
               icparea(2,3) = 1
            else
               icparea(1,3) = 1
               icparea(2,3) = nk
            end if

            offset = (ntimlv-1)*ndegfd + idegfd-1

            call isscopy ( ilosrc, ihisrc, jlosrc, jhisrc,
     +                     klosrc, khisrc,
     +                     ilodest, ihidest, jlodest, jhidest,
     +                     klodest, khidest,
     +                     1, icparea,
     +                     buffer(ipsolut + offset*npointsv),
     +                     mbsol(istartv), 1d0)

            istartv = istartv + (ni+1)*(nj+1)*(nk+1)
            idegfd  = idegfd + 1

c           --- area to be copied for V3

            if ( ndim.eq.3 ) then

               icparea(1,1) = 1
               icparea(2,1) = ni
               icparea(1,2) = 1
               icparea(2,2) = nj
               icparea(1,3) = 1
               icparea(2,3) = nk+1

               offset = (ntimlv-1)*ndegfd + idegfd-1

               call isscopy ( ilosrc, ihisrc, jlosrc, jhisrc,
     +                        klosrc, khisrc,
     +                        ilodest, ihidest, jlodest, jhidest,
     +                        klodest, khidest,
     +                        1, icparea,
     +                        buffer(ipsolut + offset*npointsv),
     +                        mbsol(istartv), 1d0)

               istartv = istartv + (ni+1)*(nj+1)*(nk+1)
               idegfd  = idegfd + 1

            end if

         else

c        --- area to be copied for pressure or transport unknown

            icparea(1,1) = 1
            icparea(2,1) = ni
            icparea(1,2) = 1
            icparea(2,2) = nj
            if ( ndim.eq.2 ) then
               icparea(1,3) = 1
               icparea(2,3) = 1
            else
               icparea(1,3) = 1
               icparea(2,3) = nk
            end if

            offset = (ntimlv-1)*ndegfd + idegfd-1

            call isscopy ( ilosrc, ihisrc, jlosrc, jhisrc,
     +                     klosrc, khisrc,
     +                     ilodest, ihidest, jlodest, jhidest,
     +                     klodest, khidest,
     +                     1, icparea,
     +                     buffer(ipsolut + offset*npointsv),
     +                     mbsol(istartv), 1d0)

            istartv = istartv + (ni+1)*(nj+1)*(nk+1)
            idegfd  = idegfd + 1

         end if
         
      end if

      end do

      end
