      subroutine isuprhs(iequa, iblock, matonly,
     +                   iinput, infmm, infmt,
     +                   solut, matrix, matvirt,
     +                   rhside, rhsvirt, work, lmatvir )
c ======================================================================
c
c     programmer Erik Brakkee
c
c     version    1.7    24-08-1998 Kees K. array work added
c     version    1.6    09-12-1997 Kees V. efficient memory use for the
c                                  virtual matrix elements
c     version    1.5    13-10-1997 Charles: uncoupled allowed
c     version    1.4    08-01-1997 Kees K.: idiscret added
c     version    1.3    28-03-1995 Erik: Only copying and update of
c                                      right-hand side if matonly
c                                      is false.
c     version    1.2    22-02-1995 Erik: routine now also copies
c                                      matrix elements that are set
c                                      to zero to matvirt and boundary
c                                      rhside elements to rhsvirt
c     version    1.1    13-10-1994 RvN:  routine now uses infmat
c     version    1.0    9-6-1994   Erik: initial revision
c
c
c     computer   HP - 9000/825
c     reference  -
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Update the right-hand side of a system of equations with linear
c     combinations of virtual fluxes.
c     The matrix elements corresponding to virtual unknowns are saved
c     in matvirt. The right-hand side elements that are updated
c     are saved to rhsvirt.
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iequa, iblock, iinput(*), infmm(*), infmt(*), lmatvir
      double precision matrix(*)
      double precision matvirt(*)
      double precision rhside(*)
      double precision rhsvirt(*)
      double precision solut(*)
      double precision work(*)
      logical matonly

c     iblock    i    Block sequence number
c     iequa     i    General equation sequence number
c     iinput    i    reserved
c     infmm     i    Integer information for Momentum matrix
c     infmt     i    Integer information for Pressure and Transport matrix
c     lmatvir   i    Length of the array of virtual matrix elements
c     matonly   i    If true, only copy matrix elements and leave
c                    right-hand side alone.
c     matrix   i/o   reserved
c     matvirt   o    Storage for the virtual coefficients of a matrix
c     rhside   i/o   reserved
c     rhsvirt   o    Storage for the elements of the right-hand side
c                    affected by virtual unknowns of neighbouring blocks
c     solut     i    reserved
c     work     i/o   Work array
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ni, nj, nk,
     +   ndim, ndegfd, ntimlv, nvirtual, nconct,
     +   ldegfd, npointsnv, npointsv, ipiindsc, idiscret,
     +   iseqpres, icoupled

c     icoupled  If 0 the momentum equations are coupled, if 1 they are not
c     idiscret  Type of discretization
c     ipiindsc  Starting address of array IINDSC in array IINPUT
c     iseqpres  Sequence number of pressure equation
c     ldegfd    Number of degrees of freedom in the matrix
c     nconct    Number of connections in a row of the matrix
c     ndegfd    Number of degrees of freedom
c     ndim      Dimension of space (2 or 3)
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     npointsnv Number of points in the mesh without virtual points
c     npointsv  Number of points in the mesh with virtual points
c     ntimlv    Number of time levels
c     nvirtual  Number of rows of virtual cells
c **********************************************************************
c
c                       I/O
c     none
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISRMV     Fold virtual unknowns to the right-hand side.
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c

      call iseropen('isuprhs')

      ni       = iinput(51)
      nj       = iinput(52)
      nk       = iinput(53)
      ndim     = iinput(57)
      ndegfd   = iinput(55)
      ntimlv   = iinput(64)
      nvirtual = iinput(54)
      icoupled = iinput(76)
      if ( icoupled.eq.0 ) then
         iseqpres = 2
      else
         iseqpres = ndim + 1
      end if

      ipiindsc = iinput(6)
      idiscret = iinput(ipiindsc+1)

      npointsnv = (ni+1)*(nj+1)*(nk+1)
      npointsv  = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)
      if (ndim.eq.3) npointsv = npointsv*(nk+1+2*nvirtual)

      if ( iequa.lt.iseqpres ) then

c     --- momentum matrix

         nconct = infmm(6)
         if ( icoupled.eq.0 ) then
            ldegfd = ndim
         else
            ldegfd = 1
         end if

         call isrmv( iequa, ndim, ni, nj, nk, nvirtual,
     +                   npointsnv, npointsv, idiscret,
     +                   ldegfd, nconct, ndegfd, ntimlv, matonly,
     +                   matrix, matvirt, solut,
     +                   rhside, rhsvirt, infmm(infmm(12)), work,
     +                   icoupled, lmatvir )

      else

c     --- pressure or transport matrix

         nconct = infmt(6)
         ldegfd = 1

         call isrmv( iequa, ndim, ni, nj, nk, nvirtual,
     +                   npointsnv, npointsv, idiscret,
     +                   ldegfd, nconct, ndegfd, ntimlv, matonly,
     +                   matrix, matvirt, solut,
     +                   rhside, rhsvirt, infmt(infmt(12)), work,
     +                   icoupled, lmatvir )

      end if

      call iserclos('isuprhs')

      return
      end
