      subroutine LU_all (mat, rbuf, ibuf)
c ======================================================================
c
c        programmer     Jos van Kan
c
c       version 1.1    10-04-1997
c         utilizes module mgpreset instead of common block cmgconst
c       version 1.0    27-03-1997
c
c
c   copyright (c) 1997
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   author.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c   calculates LU decomposition of the tridiagonal blocks on all levels.
c   Both LUx and LUy (for alternating line jacobi)
c   mklvlinfo must have been called previously.
c **********************************************************************
c
c                       KEYWORDS
c
c   multi_grid
c **********************************************************************
c
c                       MODULES USED
c
      use mgpreset
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      double precision mat (*), rbuf (*)
      integer ibuf(*)

c     ibuf      i    Integer array of dimension 300 that contains the
c                    administration of this particular multi grid block.
c     mat       i    The nx x ny x 9 coefficient matrix of the
c                    discrete Laplacian
c     rbuf      o    Buffer array used to store all coarse size info
c                    and as scratch space. The actaul bufferspace
c                    needed can be calculated using subr autosize.
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ilevel

c     ilevel    Levelcounter (multi grid levels)
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     LUX       Block LU decomposition for line Jacobi in x dir (MG)
c     LUY       Block LU decomposition for line Jacobi in y dir (MG)
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c   trivial
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- Both LUx and LUy on one level
c         This is independent of coarsening

        call LUx (mat, rbuf ( ibuf (LUxri)), ibuf (sizexi),
     +       ibuf (sizeyi),
     + ibuf(slacki),ibuf(slacki+1),ibuf(slacki+2),ibuf(slacki+3))
        call LUy (mat, rbuf ( ibuf (LUyri)), ibuf (sizexi),
     +       ibuf (sizeyi),
     + ibuf(slacki),ibuf(slacki+1),ibuf(slacki+2),ibuf(slacki+3))

        do ilevel = 1, ibuf (hileveli)
        call LUx (rbuf ( ibuf (matri + ilevel)),
     +            rbuf ( ibuf (LUxri + ilevel)),
     +            ibuf (sizexi + ilevel), ibuf (sizeyi + ilevel),
     +     0,0,0,0)
        call LUy (rbuf ( ibuf (matri + ilevel)),
     +            rbuf ( ibuf (LUyri + ilevel)),
     +            ibuf (sizexi + ilevel), ibuf (sizeyi + ilevel),
     +     0,0,0,0)
        end do
      end
