      subroutine RAPx3 ( finemat, coarsmat, nxf, nyf, nzf,
     +  sl1, sl2, sl3, sl4, sl5, sl6)
c ======================================================================
c
c        programmer     Jos van Kan
c        version 1.2    date   01-07-1997
c           correction of inter grid operators cell centered
c        version 1.1    date   04-06-1997
c           isnazification of all variables and subroutine calls
c           all truncated to max 8 characters.
c        version 1.0    date   11-03-1997
c
c   copyright (c) 1997
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   author.
c
c   pilotcode 3d mgpackage
c
c **********************************************************************
c
c                       MODULES USED
c
      use mgpreset
c **********************************************************************
c
c                       DESCRIPTION
c
c   Calculate RAP matrix with semicoarsening in x-direction
c   centring shall be decided by the parity of nxf.
c **********************************************************************
c
c                       KEYWORDS
c
c   multi_grid
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer sl1, sl2, sl3, sl4, sl5, sl6
      integer nxf, nyf, nzf
      double precision
     +finemat(1-sl1:nxf+sl2,1-sl3:nyf+sl4,
     + 1-sl5:nzf+sl6, 27),
     +coarsmat (nxf/2 + 1, nyf, nzf, 27)

c     coarsmat  o    Matrix on coarse level
c     finemat   i    Matrix on fine level
c     nxf       i    Number of grid x points on fine grid
c     nyf       i    Number of grid y points on fine grid
c     nzf       i    Number of grid z points on fine grid
c     sl1       i    Slack variable, x lower bound
c     sl2       i    Slack variable, x upper bound
c     sl3       i    Slack variable, y lower bound
c     sl4       i    Slack variable, y upper bound
c     sl5       i    Slack variable, z lower bound
c     sl6       i    Slack variable, z upper bound
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ixf, iyf, izf, ixc, nxc, centring
      double precision aa0, aa1

c     aa0       Interpolation weight in restriction/prolongation
c     aa1       Interpolation weight in restriction/prolongation
c     cellc     Cell centered (module mgpreset)
c     centring  0 = cellcentering, 1 = vertex centering
c     ixc       Counter to number of grid x points on coarse grid
c     ixf       Counter to number of grid x points on fine grid
c     iyf       Counter to number of grid y points on fine grid
c     izf       Counter to number of grid z points on fine grid
c     nxc       Number of grid x points on coarse grid
c     p000      Index of point in stencil 3D (module mgpreset)
c     p001      Index of point in stencil 3D (module mgpreset)
c     p002      Index of point in stencil 3D (module mgpreset)
c     p010      Index of point in stencil 3D (module mgpreset)
c     p011      Index of point in stencil 3D (module mgpreset)
c     p012      Index of point in stencil 3D (module mgpreset)
c     p020      Index of point in stencil 3D (module mgpreset)
c     p021      Index of point in stencil 3D (module mgpreset)
c     p022      Index of point in stencil 3D (module mgpreset)
c     p100      Index of point in stencil 3D (module mgpreset)
c     p101      Index of point in stencil 3D (module mgpreset)
c     p102      Index of point in stencil 3D (module mgpreset)
c     p110      Index of point in stencil 3D (module mgpreset)
c     p111      Index of point in stencil 3D (module mgpreset)
c     p112      Index of point in stencil 3D (module mgpreset)
c     p120      Index of point in stencil 3D (module mgpreset)
c     p121      Index of point in stencil 3D (module mgpreset)
c     p122      Index of point in stencil 3D (module mgpreset)
c     p200      Index of point in stencil 3D (module mgpreset)
c     p201      Index of point in stencil 3D (module mgpreset)
c     p202      Index of point in stencil 3D (module mgpreset)
c     p210      Index of point in stencil 3D (module mgpreset)
c     p211      Index of point in stencil 3D (module mgpreset)
c     p212      Index of point in stencil 3D (module mgpreset)
c     p220      Index of point in stencil 3D (module mgpreset)
c     p221      Index of point in stencil 3D (module mgpreset)
c     p222      Index of point in stencil 3D (module mgpreset)
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      nxc = nxf/2 + 1
      centring = mod (nxf, 2)
      if (centring.eq.CELLC) then

c     --- cell centered

      aa0 = 0.25d0
      aa1 = 0.75d0
      do izf = 1, nzf
      do iyf = 1, nyf

c     --- stencil left plane

        coarsmat (1, iyf, izf, p000) = 0d0
        coarsmat (1, iyf, izf, p010) = 0d0
        coarsmat (1, iyf, izf, p020) = 0d0
        coarsmat (1, iyf, izf, p001) = 0d0
        coarsmat (1, iyf, izf, p011) = 0d0
        coarsmat (1, iyf, izf, p021) = 0d0
        coarsmat (1, iyf, izf, p002) = 0d0
        coarsmat (1, iyf, izf, p012) = 0d0
        coarsmat (1, iyf, izf, p022) = 0d0
        do ixc = 2, nxc - 1
          ixf = 2 * ixc - 2
        coarsmat (ixc ,iyf,izf,p000) = aa1*finemat (ixf ,iyf,izf,p000) +
     +   aa0*(finemat (ixf,iyf,izf,p100) + finemat (ixf+1,iyf,izf,p000))
        coarsmat (ixc ,iyf,izf,p010) = aa1*finemat (ixf ,iyf,izf,p010) +
     +   aa0*(finemat (ixf,iyf,izf,p110) + finemat (ixf+1,iyf,izf,p010))
        coarsmat (ixc ,iyf,izf,p020) = aa1*finemat (ixf ,iyf,izf,p020) +
     +   aa0*(finemat (ixf,iyf,izf,p120) + finemat (ixf+1,iyf,izf,p020))
        coarsmat (ixc ,iyf,izf,p001) = aa1*finemat (ixf ,iyf,izf,p001) +
     +   aa0*(finemat (ixf,iyf,izf,p101) + finemat (ixf+1,iyf,izf,p001))
        coarsmat (ixc ,iyf,izf,p011) = aa1*finemat (ixf ,iyf,izf,p011) +
     +   aa0*(finemat (ixf,iyf,izf,p111) + finemat (ixf+1,iyf,izf,p011))
        coarsmat (ixc ,iyf,izf,p021) = aa1*finemat (ixf ,iyf,izf,p021) +
     +   aa0*(finemat (ixf,iyf,izf,p121) + finemat (ixf+1,iyf,izf,p021))
        coarsmat (ixc ,iyf,izf,p002) = aa1*finemat (ixf ,iyf,izf,p002) +
     +   aa0*(finemat (ixf,iyf,izf,p102) + finemat (ixf+1,iyf,izf,p002))
        coarsmat (ixc ,iyf,izf,p012) = aa1*finemat (ixf ,iyf,izf,p012) +
     +   aa0*(finemat (ixf,iyf,izf,p112) + finemat (ixf+1,iyf,izf,p012))
        coarsmat (ixc ,iyf,izf,p022) = aa1*finemat (ixf ,iyf,izf,p022) +
     +   aa0*(finemat (ixf,iyf,izf,p122) + finemat (ixf+1,iyf,izf,p022))
        end do

        coarsmat (nxc ,iyf,izf,p000) = aa1*finemat (nxf ,iyf,izf,p000) +
     +   aa0*(finemat (nxf,iyf,izf,p100))
        coarsmat (nxc ,iyf,izf,p010) = aa1*finemat (nxf ,iyf,izf,p010) +
     +   aa0*(finemat (nxf,iyf,izf,p110))
        coarsmat (nxc ,iyf,izf,p020) = aa1*finemat (nxf ,iyf,izf,p020) +
     +   aa0*(finemat (nxf,iyf,izf,p120))
        coarsmat (nxc ,iyf,izf,p001) = aa1*finemat (nxf ,iyf,izf,p001) +
     +   aa0*(finemat (nxf,iyf,izf,p101))
        coarsmat (nxc ,iyf,izf,p011) = aa1*finemat (nxf ,iyf,izf,p011) +
     +   aa0*(finemat (nxf,iyf,izf,p111))
        coarsmat (nxc ,iyf,izf,p021) = aa1*finemat (nxf ,iyf,izf,p021) +
     +   aa0*(finemat (nxf,iyf,izf,p121))
        coarsmat (nxc ,iyf,izf,p002) = aa1*finemat (nxf ,iyf,izf,p002) +
     +   aa0*(finemat (nxf,iyf,izf,p102))
        coarsmat (nxc ,iyf,izf,p012) = aa1*finemat (nxf ,iyf,izf,p012) +
     +   aa0*(finemat (nxf,iyf,izf,p112))
        coarsmat (nxc ,iyf,izf,p022) = aa1*finemat (nxf ,iyf,izf,p022) +
     +   aa0*(finemat (nxf,iyf,izf,p122))

c       --- end stencil left plane

c           --- stencil center plane

        coarsmat (1 ,iyf,izf,p100) =
     +   aa1*(finemat (1,iyf,izf,p000))
     + + aa0*finemat (1 ,iyf,izf,p200) +
     +   aa1*(finemat (1,iyf,izf,p100))
        coarsmat (1 ,iyf,izf,p110) =
     +   aa1*(finemat (1,iyf,izf,p010))
     + + aa0*finemat (1 ,iyf,izf,p210) +
     +   aa1*(finemat (1,iyf,izf,p110))
        coarsmat (1 ,iyf,izf,p120) =
     +   aa1*(finemat (1,iyf,izf,p020))
     + + aa0*finemat (1 ,iyf,izf,p220) +
     +   aa1*(finemat (1,iyf,izf,p120))
        coarsmat (1 ,iyf,izf,p101) =
     +   aa1*(finemat (1,iyf,izf,p001))
     + + aa0*finemat (1 ,iyf,izf,p201) +
     +   aa1*(finemat (1,iyf,izf,p101))
        coarsmat (1 ,iyf,izf,p111) =
     +   aa1*(finemat (1,iyf,izf,p011))
     + + aa0*finemat (1 ,iyf,izf,p211) +
     +   aa1*(finemat (1,iyf,izf,p111))
        coarsmat (1 ,iyf,izf,p121) =
     +   aa1*(finemat (1,iyf,izf,p021))
     + + aa0*finemat (1 ,iyf,izf,p221) +
     +   aa1*(finemat (1,iyf,izf,p121))
        coarsmat (1 ,iyf,izf,p102) =
     +   aa1*(finemat (1,iyf,izf,p002))
     + + aa0*finemat (1 ,iyf,izf,p202) +
     +   aa1*(finemat (1,iyf,izf,p102))
        coarsmat (1 ,iyf,izf,p112) =
     +   aa1*(finemat (1,iyf,izf,p012))
     + + aa0*finemat (1 ,iyf,izf,p212) +
     +   aa1*(finemat (1,iyf,izf,p112))
        coarsmat (1 ,iyf,izf,p122) =
     +   aa1*(finemat (1,iyf,izf,p022))
     + + aa0*finemat (1 ,iyf,izf,p222) +
     +   aa1*(finemat (1,iyf,izf,p122))
        do ixc = 2, nxc-1
          ixf = 2 * ixc - 2
        coarsmat (ixc ,iyf,izf,p100) = aa0*finemat (ixf ,iyf,izf,p000) +
     +   aa1*(finemat (ixf,iyf,izf,p100) + finemat (ixf+1,iyf,izf,p000))
     + + aa0*finemat (ixf+1 ,iyf,izf,p200) +
     +   aa1*(finemat (ixf,iyf,izf,p200) + finemat (ixf+1,iyf,izf,p100))
        coarsmat (ixc ,iyf,izf,p110) = aa0*finemat (ixf ,iyf,izf,p010) +
     +   aa1*(finemat (ixf,iyf,izf,p110) + finemat (ixf+1,iyf,izf,p010))
     + + aa0*finemat (ixf+1 ,iyf,izf,p210) +
     +   aa1*(finemat (ixf,iyf,izf,p210) + finemat (ixf+1,iyf,izf,p110))
        coarsmat (ixc ,iyf,izf,p120) = aa0*finemat (ixf ,iyf,izf,p020) +
     +   aa1*(finemat (ixf,iyf,izf,p120) + finemat (ixf+1,iyf,izf,p020))
     + + aa0*finemat (ixf+1 ,iyf,izf,p220) +
     +   aa1*(finemat (ixf,iyf,izf,p220) + finemat (ixf+1,iyf,izf,p120))
        coarsmat (ixc ,iyf,izf,p101) = aa0*finemat (ixf ,iyf,izf,p001) +
     +   aa1*(finemat (ixf,iyf,izf,p101) + finemat (ixf+1,iyf,izf,p001))
     + + aa0*finemat (ixf+1 ,iyf,izf,p201) +
     +   aa1*(finemat (ixf,iyf,izf,p201) + finemat (ixf+1,iyf,izf,p101))
        coarsmat (ixc ,iyf,izf,p111) = aa0*finemat (ixf ,iyf,izf,p011) +
     +   aa1*(finemat (ixf,iyf,izf,p111) + finemat (ixf+1,iyf,izf,p011))
     + + aa0*finemat (ixf+1 ,iyf,izf,p211) +
     +   aa1*(finemat (ixf,iyf,izf,p211) + finemat (ixf+1,iyf,izf,p111))
        coarsmat (ixc ,iyf,izf,p121) = aa0*finemat (ixf ,iyf,izf,p021) +
     +   aa1*(finemat (ixf,iyf,izf,p121) + finemat (ixf+1,iyf,izf,p021))
     + + aa0*finemat (ixf+1 ,iyf,izf,p221) +
     +   aa1*(finemat (ixf,iyf,izf,p221) + finemat (ixf+1,iyf,izf,p121))
        coarsmat (ixc ,iyf,izf,p102) = aa0*finemat (ixf ,iyf,izf,p002) +
     +   aa1*(finemat (ixf,iyf,izf,p102) + finemat (ixf+1,iyf,izf,p002))
     + + aa0*finemat (ixf+1 ,iyf,izf,p202) +
     +   aa1*(finemat (ixf,iyf,izf,p202) + finemat (ixf+1,iyf,izf,p102))
        coarsmat (ixc ,iyf,izf,p112) = aa0*finemat (ixf ,iyf,izf,p012) +
     +   aa1*(finemat (ixf,iyf,izf,p112) + finemat (ixf+1,iyf,izf,p012))
     + + aa0*finemat (ixf+1 ,iyf,izf,p212) +
     +   aa1*(finemat (ixf,iyf,izf,p212) + finemat (ixf+1,iyf,izf,p112))
        coarsmat (ixc ,iyf,izf,p122) = aa0*finemat (ixf ,iyf,izf,p022) +
     +   aa1*(finemat (ixf,iyf,izf,p122) + finemat (ixf+1,iyf,izf,p022))
     + + aa0*finemat (ixf+1 ,iyf,izf,p222) +
     +   aa1*(finemat (ixf,iyf,izf,p222) + finemat (ixf+1,iyf,izf,p122))
        end do
        coarsmat (nxc ,iyf,izf,p100) = aa0*finemat (nxf ,iyf,izf,p000) +
     +   aa1*(finemat (nxf,iyf,izf,p100))+
     +   aa1*(finemat (nxf,iyf,izf,p200))
        coarsmat (nxc ,iyf,izf,p110) = aa0*finemat (nxf ,iyf,izf,p010) +
     +   aa1*(finemat (nxf,iyf,izf,p110))+
     +   aa1*(finemat (nxf,iyf,izf,p210))
        coarsmat (nxc ,iyf,izf,p120) = aa0*finemat (nxf ,iyf,izf,p020) +
     +   aa1*(finemat (nxf,iyf,izf,p120))+
     +   aa1*(finemat (nxf,iyf,izf,p220))
        coarsmat (nxc ,iyf,izf,p101) = aa0*finemat (nxf ,iyf,izf,p001) +
     +   aa1*(finemat (nxf,iyf,izf,p101))+
     +   aa1*(finemat (nxf,iyf,izf,p201))
        coarsmat (nxc ,iyf,izf,p111) = aa0*finemat (nxf ,iyf,izf,p011) +
     +   aa1*(finemat (nxf,iyf,izf,p111))+
     +   aa1*(finemat (nxf,iyf,izf,p211))
        coarsmat (nxc ,iyf,izf,p121) = aa0*finemat (nxf ,iyf,izf,p021) +
     +   aa1*(finemat (nxf,iyf,izf,p121))+
     +   aa1*(finemat (nxf,iyf,izf,p221))
        coarsmat (nxc ,iyf,izf,p102) = aa0*finemat (nxf ,iyf,izf,p002) +
     +   aa1*(finemat (nxf,iyf,izf,p102))+
     +   aa1*(finemat (nxf,iyf,izf,p202))
        coarsmat (nxc ,iyf,izf,p112) = aa0*finemat (nxf ,iyf,izf,p012) +
     +   aa1*(finemat (nxf,iyf,izf,p112))+
     +   aa1*(finemat (nxf,iyf,izf,p212))
        coarsmat (nxc ,iyf,izf,p122) = aa0*finemat (nxf ,iyf,izf,p022) +
     +   aa1*(finemat (nxf,iyf,izf,p122))+
     +   aa1*(finemat (nxf,iyf,izf,p222))

c       --- end stencil center plane

c           --- stencil right plane

       coarsmat (1,iyf,izf,p200) = aa1*finemat (1,iyf,izf,p200) +
     +  aa0*(finemat (1,iyf,izf,p100))
       coarsmat (1,iyf,izf,p210) = aa1*finemat (1,iyf,izf,p210) +
     +  aa0*(finemat (1,iyf,izf,p110))
       coarsmat (1,iyf,izf,p220) = aa1*finemat (1,iyf,izf,p220) +
     +  aa0*(finemat (1,iyf,izf,p120))
       coarsmat (1,iyf,izf,p201) = aa1*finemat (1,iyf,izf,p201) +
     +  aa0*(finemat (1,iyf,izf,p101))
       coarsmat (1,iyf,izf,p211) = aa1*finemat (1,iyf,izf,p211) +
     +  aa0*(finemat (1,iyf,izf,p111))
       coarsmat (1,iyf,izf,p221) = aa1*finemat (1,iyf,izf,p221) +
     +  aa0*(finemat (1,iyf,izf,p121))
       coarsmat (1,iyf,izf,p202) = aa1*finemat (1,iyf,izf,p202) +
     +  aa0*(finemat (1,iyf,izf,p102))
       coarsmat (1,iyf,izf,p212) = aa1*finemat (1,iyf,izf,p212) +
     +  aa0*(finemat (1,iyf,izf,p112))
       coarsmat (1,iyf,izf,p222) = aa1*finemat (1,iyf,izf,p222) +
     +  aa0*(finemat (1,iyf,izf,p122))

        do ixc = 2, nxc - 1
          ixf = 2 * ixc - 2
       coarsmat (ixc,iyf,izf,p200) = aa1*finemat (ixf+1,iyf,izf,p200) +
     +  aa0*(finemat (ixf,iyf,izf,p200) + finemat (ixf+1,iyf,izf,p100))
       coarsmat (ixc,iyf,izf,p210) = aa1*finemat (ixf+1,iyf,izf,p210) +
     +  aa0*(finemat (ixf,iyf,izf,p210) + finemat (ixf+1,iyf,izf,p110))
       coarsmat (ixc,iyf,izf,p220) = aa1*finemat (ixf+1,iyf,izf,p220) +
     +  aa0*(finemat (ixf,iyf,izf,p220) + finemat (ixf+1,iyf,izf,p120))
       coarsmat (ixc,iyf,izf,p201) = aa1*finemat (ixf+1,iyf,izf,p201) +
     +  aa0*(finemat (ixf,iyf,izf,p201) + finemat (ixf+1,iyf,izf,p101))
       coarsmat (ixc,iyf,izf,p211) = aa1*finemat (ixf+1,iyf,izf,p211) +
     +  aa0*(finemat (ixf,iyf,izf,p211) + finemat (ixf+1,iyf,izf,p111))
       coarsmat (ixc,iyf,izf,p221) = aa1*finemat (ixf+1,iyf,izf,p221) +
     +  aa0*(finemat (ixf,iyf,izf,p221) + finemat (ixf+1,iyf,izf,p121))
       coarsmat (ixc,iyf,izf,p202) = aa1*finemat (ixf+1,iyf,izf,p202) +
     +  aa0*(finemat (ixf,iyf,izf,p202) + finemat (ixf+1,iyf,izf,p102))
       coarsmat (ixc,iyf,izf,p212) = aa1*finemat (ixf+1,iyf,izf,p212) +
     +  aa0*(finemat (ixf,iyf,izf,p212) + finemat (ixf+1,iyf,izf,p112))
       coarsmat (ixc,iyf,izf,p222) = aa1*finemat (ixf+1,iyf,izf,p222) +
     +  aa0*(finemat (ixf,iyf,izf,p222) + finemat (ixf+1,iyf,izf,p122))
        end do
        coarsmat (nxc, iyf, izf, p200) = 0d0
        coarsmat (nxc, iyf, izf, p210) = 0d0
        coarsmat (nxc, iyf, izf, p220) = 0d0
        coarsmat (nxc, iyf, izf, p201) = 0d0
        coarsmat (nxc, iyf, izf, p211) = 0d0
        coarsmat (nxc, iyf, izf, p221) = 0d0
        coarsmat (nxc, iyf, izf, p202) = 0d0
        coarsmat (nxc, iyf, izf, p212) = 0d0
        coarsmat (nxc, iyf, izf, p222) = 0d0

c       --- end stencil right column

      end do
      end do

c     --- end case cell centered

      else

c     --- begin case vertex centered

      aa0 = 0.25d0
      aa1 = 0.5d0

      do izf = 1, nzf
      do iyf = 1, nyf

c--- stencil center plane

c     --- region left boundary

        coarsmat (1,iyf,izf,p010) = 0d0
        coarsmat (1,iyf,izf,p210) = aa1* (finemat (2,iyf,izf,p210)
     +   +finemat (1,iyf,izf,p210)) +
     + aa0*finemat (2,iyf,izf,p110)
        coarsmat (1,iyf,izf,p110) = finemat (1,iyf,izf,p110) +
     + aa1*(finemat (1,iyf,izf,p210) +finemat (2,iyf,izf,p010))+
     +aa0*finemat (2,iyf,izf,p110)
        coarsmat (1,iyf,izf,p011) = 0d0
        coarsmat (1,iyf,izf,p211) = aa1* (finemat (2,iyf,izf,p211)
     +   +finemat (1,iyf,izf,p211)) +
     + aa0*finemat (2,iyf,izf,p111)
        coarsmat (1,iyf,izf,p111) = finemat (1,iyf,izf,p111) +
     + aa1*(finemat (1,iyf,izf,p211) +finemat (2,iyf,izf,p011))+
     +aa0*finemat (2,iyf,izf,p111)
        coarsmat (1,iyf,izf,p012) = 0d0
        coarsmat (1,iyf,izf,p212) = aa1* (finemat (2,iyf,izf,p212)
     +   +finemat (1,iyf,izf,p212)) +
     + aa0*finemat (2,iyf,izf,p112)
        coarsmat (1,iyf,izf,p112) = finemat (1,iyf,izf,p112) +
     + aa1*(finemat (1,iyf,izf,p212) +finemat (2,iyf,izf,p012))+
     +aa0*finemat (2,iyf,izf,p112)
        do ixc = 2,nxc-1
          ixf = 2*ixc - 1
       coarsmat (ixc,iyf,izf,p010)= aa1*(finemat (ixf-1,iyf,izf,p010)
     +  +finemat (ixf,iyf,izf,p010)) +
     +aa0*finemat (ixf-1,iyf,izf,p110)
       coarsmat (ixc,iyf,izf,p210) = aa1* (finemat (ixf +1,iyf,izf,p210)
     +  +finemat (ixf,iyf,izf,p210)) +
     +aa0*finemat (ixf+1,iyf,izf,p110)
       coarsmat (ixc,iyf,izf,p110) = finemat (ixf,iyf,izf,p110) +
     +  aa1*(finemat (ixf,iyf,izf,p010) +finemat (ixf-1,iyf,izf,p210))+
     +  aa1*(finemat (ixf,iyf,izf,p210) +finemat (ixf+1,iyf,izf,p010))+
     +  aa0*finemat (ixf-1,iyf,izf,p110) +
     +  aa0*finemat (ixf+1,iyf,izf,p110)
       coarsmat (ixc,iyf,izf,p011)= aa1*(finemat (ixf-1,iyf,izf,p011)
     +  +finemat (ixf,iyf,izf,p011)) +
     +aa0*finemat (ixf-1,iyf,izf,p111)
       coarsmat (ixc,iyf,izf,p211) = aa1* (finemat (ixf +1,iyf,izf,p211)
     +  +finemat (ixf,iyf,izf,p211)) +
     +aa0*finemat (ixf+1,iyf,izf,p111)
       coarsmat (ixc,iyf,izf,p111) = finemat (ixf,iyf,izf,p111) +
     +  aa1*(finemat (ixf,iyf,izf,p011) +finemat (ixf-1,iyf,izf,p211))+
     +  aa1*(finemat (ixf,iyf,izf,p211) +finemat (ixf+1,iyf,izf,p011))+
     +  aa0*finemat (ixf-1,iyf,izf,p111) +
     +  aa0*finemat (ixf+1,iyf,izf,p111)
       coarsmat (ixc,iyf,izf,p012)= aa1*(finemat (ixf-1,iyf,izf,p012)
     +  +finemat (ixf,iyf,izf,p012)) +
     +aa0*finemat (ixf-1,iyf,izf,p112)
       coarsmat (ixc,iyf,izf,p212) = aa1* (finemat (ixf +1,iyf,izf,p212)
     +  +finemat (ixf,iyf,izf,p212)) +
     +aa0*finemat (ixf+1,iyf,izf,p112)
       coarsmat (ixc,iyf,izf,p112) = finemat (ixf,iyf,izf,p112) +
     +  aa1*(finemat (ixf,iyf,izf,p012) +finemat (ixf-1,iyf,izf,p212))+
     +  aa1*(finemat (ixf,iyf,izf,p212) +finemat (ixf+1,iyf,izf,p012))+
     +  aa0*finemat (ixf-1,iyf,izf,p112) +
     +  aa0*finemat (ixf+1,iyf,izf,p112)
        end do

c       --- region right boundary

        nxf = 2*nxc - 1
        coarsmat(nxc,iyf,izf,p010)=aa1*(finemat(nxf-1,iyf,izf,p010)
     +  +finemat(nxf,iyf,izf,p010))+
     +aa0*finemat(nxf-1,iyf,izf,p110)
        coarsmat(nxc,iyf,izf,p210)=0d0
        coarsmat(nxc,iyf,izf,p110)=finemat(nxf,iyf,izf,p110)+
     +  aa1*(finemat(nxf,iyf,izf,p010)+finemat(nxf-1,iyf,izf,p210))+
     +  aa0*finemat(nxf-1,iyf,izf,p110)
        coarsmat(nxc,iyf,izf,p011)=aa1*(finemat(nxf-1,iyf,izf,p011)
     +  +finemat(nxf,iyf,izf,p011))+
     +aa0*finemat(nxf-1,iyf,izf,p111)
        coarsmat(nxc,iyf,izf,p211)=0d0
        coarsmat(nxc,iyf,izf,p111)=finemat(nxf,iyf,izf,p111)+
     +  aa1*(finemat(nxf,iyf,izf,p011)+finemat(nxf-1,iyf,izf,p211))+
     +  aa0*finemat(nxf-1,iyf,izf,p111)
        coarsmat(nxc,iyf,izf,p012)=aa1*(finemat(nxf-1,iyf,izf,p012)
     +  +finemat(nxf,iyf,izf,p012))+
     +aa0*finemat(nxf-1,iyf,izf,p112)
        coarsmat(nxc,iyf,izf,p212)=0d0
        coarsmat(nxc,iyf,izf,p112)=finemat(nxf,iyf,izf,p112)+
     +  aa1*(finemat(nxf,iyf,izf,p012)+finemat(nxf-1,iyf,izf,p212))+
     +  aa0*finemat(nxf-1,iyf,izf,p112)

c---end stencil centerrow

c--- stencil back plane

c       --- region left boundary

        coarsmat (1,iyf,izf,p020) = 0d0
        coarsmat (1,iyf,izf,p220) = aa1* (finemat (2,iyf,izf,p220)
     +   +finemat (1,iyf,izf,p220)) +
     + aa0*finemat (2,iyf,izf,p120)
        coarsmat (1,iyf,izf,p120) = finemat (1,iyf,izf,p120) +
     + aa1*(finemat (1,iyf,izf,p220) +finemat (2,iyf,izf,p020))+
     +aa0*finemat (1+1,iyf,izf,p120)
        coarsmat (1,iyf,izf,p021) = 0d0
        coarsmat (1,iyf,izf,p221) = aa1* (finemat (2,iyf,izf,p221)
     +   +finemat (1,iyf,izf,p221)) +
     + aa0*finemat (2,iyf,izf,p121)
        coarsmat (1,iyf,izf,p121) = finemat (1,iyf,izf,p121) +
     +aa1*(finemat (1,iyf,izf,p221) +finemat (2,iyf,izf,p021))+
     +aa0*finemat (1+1,iyf,izf,p121)
        coarsmat (1,iyf,izf,p022) = 0d0
        coarsmat (1,iyf,izf,p222) = aa1* (finemat (2,iyf,izf,p222)
     +   +finemat (1,iyf,izf,p222)) +
     + aa0*finemat (2,iyf,izf,p122)
        coarsmat (1,iyf,izf,p122) = finemat (1,iyf,izf,p122) +
     + aa1*(finemat (1,iyf,izf,p222) +finemat (2,iyf,izf,p022))+
     +aa0*finemat (2,iyf,izf,p122)
        do ixc = 2,nxc-1
          ixf = 2*ixc - 1
       coarsmat (ixc,iyf,izf,p020)= aa1*(finemat (ixf-1,iyf,izf,p020)
     +  +finemat (ixf,iyf,izf,p020)) +
     +aa0*finemat (ixf-1,iyf,izf,p120)
       coarsmat (ixc,iyf,izf,p220) = aa1* (finemat (ixf +1,iyf,izf,p220)
     +  +finemat (ixf,iyf,izf,p220)) +
     +aa0*finemat (ixf+1,iyf,izf,p120)
       coarsmat (ixc,iyf,izf,p120) = finemat (ixf,iyf,izf,p120) +
     +  aa1*(finemat (ixf,iyf,izf,p020) +finemat (ixf-1,iyf,izf,p220))+
     +  aa1*(finemat (ixf,iyf,izf,p220) +finemat (ixf+1,iyf,izf,p020))+
     +  aa0*finemat (ixf-1,iyf,izf,p120) +
     +  aa0*finemat (ixf+1,iyf,izf,p120)
       coarsmat (ixc,iyf,izf,p021)= aa1*(finemat (ixf-1,iyf,izf,p021)
     +  +finemat (ixf,iyf,izf,p021)) +
     +aa0*finemat (ixf-1,iyf,izf,p121)
       coarsmat (ixc,iyf,izf,p221) = aa1* (finemat (ixf +1,iyf,izf,p221)
     +  +finemat (ixf,iyf,izf,p221)) +
     +aa0*finemat (ixf+1,iyf,izf,p121)
       coarsmat (ixc,iyf,izf,p121) = finemat (ixf,iyf,izf,p121) +
     +  aa1*(finemat (ixf,iyf,izf,p021) +finemat (ixf-1,iyf,izf,p221))+
     +  aa1*(finemat (ixf,iyf,izf,p221) +finemat (ixf+1,iyf,izf,p021))+
     +  aa0*finemat (ixf-1,iyf,izf,p121) +
     +  aa0*finemat (ixf+1,iyf,izf,p121)
       coarsmat (ixc,iyf,izf,p022)= aa1*(finemat (ixf-1,iyf,izf,p022)
     +  +finemat (ixf,iyf,izf,p022)) +
     +aa0*finemat (ixf-1,iyf,izf,p122)
       coarsmat (ixc,iyf,izf,p222) = aa1* (finemat (ixf +1,iyf,izf,p222)
     +  +finemat (ixf,iyf,izf,p222)) +
     +aa0*finemat (ixf+1,iyf,izf,p122)
       coarsmat (ixc,iyf,izf,p122) = finemat (ixf,iyf,izf,p122) +
     +  aa1*(finemat (ixf,iyf,izf,p022) +finemat (ixf-1,iyf,izf,p222))+
     +  aa1*(finemat (ixf,iyf,izf,p222) +finemat (ixf+1,iyf,izf,p022))+
     +  aa0*finemat (ixf-1,iyf,izf,p122) +
     +  aa0*finemat (ixf+1,iyf,izf,p122)
        end do

c       --- region right boundary

        nxf = 2*nxc - 1
        coarsmat(nxc,iyf,izf,p020)=aa1*(finemat(nxf-1,iyf,izf,p020)
     +  +finemat(nxf,iyf,izf,p020))+
     +aa0*finemat(nxf-1,iyf,izf,p120)
        coarsmat(nxc,iyf,izf,p220)=0d0
        coarsmat(nxc,iyf,izf,p120)=finemat(nxf,iyf,izf,p120)+
     +  aa1*(finemat(nxf,iyf,izf,p020)+finemat(nxf-1,iyf,izf,p220))+
     +  aa0*finemat(nxf-1,iyf,izf,p120)
        coarsmat(nxc,iyf,izf,p021)=aa1*(finemat(nxf-1,iyf,izf,p021)
     +  +finemat(nxf,iyf,izf,p021))+
     +aa0*finemat(nxf-1,iyf,izf,p121)
        coarsmat(nxc,iyf,izf,p221)=0d0
        coarsmat(nxc,iyf,izf,p121)=finemat(nxf,iyf,izf,p121)+
     +  aa1*(finemat(nxf,iyf,izf,p021)+finemat(nxf-1,iyf,izf,p221))+
     +  aa0*finemat(nxf-1,iyf,izf,p121)
        coarsmat(nxc,iyf,izf,p022)=aa1*(finemat(nxf-1,iyf,izf,p022)
     +  +finemat(nxf,iyf,izf,p022))+
     +aa0*finemat(nxf-1,iyf,izf,p122)
        coarsmat(nxc,iyf,izf,p222)=0d0
        coarsmat(nxc,iyf,izf,p122)=finemat(nxf,iyf,izf,p122)+
     +  aa1*(finemat(nxf,iyf,izf,p022)+finemat(nxf-1,iyf,izf,p222))+
     +  aa0*finemat(nxf-1,iyf,izf,p122)

c---end stencil back plane

c--- stencil front plane

c       --- region left boundary

        coarsmat (1,iyf,izf,p000) = 0d0
        coarsmat (1,iyf,izf,p200) = aa1* (finemat (2,iyf,izf,p200)
     +   +finemat (1,iyf,izf,p200)) +
     + aa0*finemat (2,iyf,izf,p100)
        coarsmat (1,iyf,izf,p100) = finemat (1,iyf,izf,p100) +
     +aa1*(finemat (1,iyf,izf,p200) +finemat (2,iyf,izf,p000))+
     +aa0*finemat (1+1,iyf,izf,p100)
        coarsmat (1,iyf,izf,p001) = 0d0
        coarsmat (1,iyf,izf,p201) = aa1* (finemat (2,iyf,izf,p201)
     +   +finemat (1,iyf,izf,p201)) +
     + aa0*finemat (2,iyf,izf,p101)
        coarsmat (1,iyf,izf,p101) = finemat (1,iyf,izf,p101) +
     +aa1*(finemat (1,iyf,izf,p201) +finemat (2,iyf,izf,p001))+
     +aa0*finemat (1+1,iyf,izf,p101)
        coarsmat (1,iyf,izf,p002) = 0d0
        coarsmat (1,iyf,izf,p202) = aa1* (finemat (2,iyf,izf,p202)
     +   +finemat (1,iyf,izf,p202)) +
     + aa0*finemat (2,iyf,izf,p102)
        coarsmat (1,iyf,izf,p102) = finemat (1,iyf,izf,p102) +
     +aa1*(finemat (1,iyf,izf,p202) +finemat (2,iyf,izf,p002))+
     +aa0*finemat (2,iyf,izf,p102)
        do ixc = 2,nxc-1
          ixf = 2*ixc - 1
       coarsmat (ixc,iyf,izf,p000)= aa1*(finemat (ixf-1,iyf,izf,p000)
     +  +finemat (ixf,iyf,izf,p000)) +
     +aa0*finemat (ixf-1,iyf,izf,p100)
       coarsmat (ixc,iyf,izf,p200) = aa1* (finemat (ixf +1,iyf,izf,p200)
     +  +finemat (ixf,iyf,izf,p200)) +
     +aa0*finemat (ixf+1,iyf,izf,p100)
       coarsmat (ixc,iyf,izf,p100) = finemat (ixf,iyf,izf,p100) +
     +  aa1*(finemat (ixf,iyf,izf,p000) +finemat (ixf-1,iyf,izf,p200))+
     +  aa1*(finemat (ixf,iyf,izf,p200) +finemat (ixf+1,iyf,izf,p000))+
     +  aa0*finemat (ixf-1,iyf,izf,p100) +
     +  aa0*finemat (ixf+1,iyf,izf,p100)
       coarsmat (ixc,iyf,izf,p001)= aa1*(finemat (ixf-1,iyf,izf,p001)
     +  +finemat (ixf,iyf,izf,p001)) +
     +aa0*finemat (ixf-1,iyf,izf,p101)
       coarsmat (ixc,iyf,izf,p201) = aa1* (finemat (ixf +1,iyf,izf,p201)
     +  +finemat (ixf,iyf,izf,p201)) +
     +aa0*finemat (ixf+1,iyf,izf,p101)
       coarsmat (ixc,iyf,izf,p101) = finemat (ixf,iyf,izf,p101) +
     +  aa1*(finemat (ixf,iyf,izf,p001) +finemat (ixf-1,iyf,izf,p201))+
     +  aa1*(finemat (ixf,iyf,izf,p201) +finemat (ixf+1,iyf,izf,p001))+
     +  aa0*finemat (ixf-1,iyf,izf,p101) +
     +  aa0*finemat (ixf+1,iyf,izf,p101)
       coarsmat (ixc,iyf,izf,p002)= aa1*(finemat (ixf-1,iyf,izf,p002)
     +  +finemat (ixf,iyf,izf,p002)) +
     +aa0*finemat (ixf-1,iyf,izf,p102)
       coarsmat (ixc,iyf,izf,p202) = aa1* (finemat (ixf +1,iyf,izf,p202)
     +  +finemat (ixf,iyf,izf,p202)) +
     +aa0*finemat (ixf+1,iyf,izf,p102)
       coarsmat (ixc,iyf,izf,p102) = finemat (ixf,iyf,izf,p102) +
     +  aa1*(finemat (ixf,iyf,izf,p002) +finemat (ixf-1,iyf,izf,p202))+
     +  aa1*(finemat (ixf,iyf,izf,p202) +finemat (ixf+1,iyf,izf,p002))+
     +  aa0*finemat (ixf-1,iyf,izf,p102) +
     +  aa0*finemat (ixf+1,iyf,izf,p102)
        end do

c       --- region right boundary

        nxf = 2*nxc - 1
        coarsmat(nxc,iyf,izf,p000)=aa1*(finemat(nxf-1,iyf,izf,p000)
     +  +finemat(nxf,iyf,izf,p000))+
     +aa0*finemat(nxf-1,iyf,izf,p100)
        coarsmat(nxc,iyf,izf,p200)=0d0
        coarsmat(nxc,iyf,izf,p100)=finemat(nxf,iyf,izf,p100)+
     +  aa1*(finemat(nxf,iyf,izf,p000)+finemat(nxf-1,iyf,izf,p200))+
     +  aa0*finemat(nxf-1,iyf,izf,p100)
        coarsmat(nxc,iyf,izf,p001)=aa1*(finemat(nxf-1,iyf,izf,p001)
     +  +finemat(nxf,iyf,izf,p001))+
     +aa0*finemat(nxf-1,iyf,izf,p101)
        coarsmat(nxc,iyf,izf,p201)=0d0
        coarsmat(nxc,iyf,izf,p101)=finemat(nxf,iyf,izf,p101)+
     +  aa1*(finemat(nxf,iyf,izf,p001)+finemat(nxf-1,iyf,izf,p201))+
     +  aa0*finemat(nxf-1,iyf,izf,p101)
        coarsmat(nxc,iyf,izf,p002)=aa1*(finemat(nxf-1,iyf,izf,p002)
     +  +finemat(nxf,iyf,izf,p002))+
     +aa0*finemat(nxf-1,iyf,izf,p102)
        coarsmat(nxc,iyf,izf,p202)=0d0
        coarsmat(nxc,iyf,izf,p102)=finemat(nxf,iyf,izf,p102)+
     +  aa1*(finemat(nxf,iyf,izf,p002)+finemat(nxf-1,iyf,izf,p202))+
     +  aa0*finemat(nxf-1,iyf,izf,p102)

c---end stencil front plane
      end do
      end do

c--- end case vertex centered
      end if

      end
