      subroutine RAPxyz3 ( finemat, coarsmat, nxf, nyf, nzf,
     +   scratch1, scratch2,
     +  sl1, sl2, sl3, sl4, sl5, sl6)
c ======================================================================
c
c        programmer     Jos van Kan
c        version 1.2    date   01-07-1997
c           correction of inter grid operators cell centered
c        version 1.1    date   04-06-1997
c           isnazification of all variables and subroutine calls
c           all truncated to max 8 characters.
c        version 1.0    date   10-03-1997
c
c
c
c   copyright (c) 1997
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   author.
c
c   pilotcode 3d mgpackage
c
c **********************************************************************
c
c                       MODULES USED
c
      use mgpreset
c **********************************************************************
c
c                       DESCRIPTION
c
c    RAPxyz3 makes the coarsening in both x, y and direction
c    It is made specially to save computing time.
c **********************************************************************
c
c                       KEYWORDS
c
c    multi_grid
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer sl1, sl2, sl3, sl4, sl5, sl6
      integer nxf, nyf, nzf
      double precision
     +finemat(1-sl1:nxf+sl2,1-sl3:nyf+sl4,
     + 1-sl5:nzf+sl6, 27),
     +coarsmat (nxf/2+1, nyf/2+1, nzf/2+1, 27),
     +scratch1 (nxf/2+1, nyf, 0:8),
     +scratch2 (nxf/2+1, nyf/2+1, 0:8)

c     coarsmat  o    Matrix on coarse level
c     finemat   i    Matrix on fine level
c     nxf       i    Number of grid x points on fine grid
c     nyf       i    Number of grid y points on fine grid
c     nzf       i    Number of grid z points on fine grid
c     scratch1  i    Scratch array
c     scratch2  i    Scratch array
c     sl1       i    Slack variable, x lower bound
c     sl2       i    Slack variable, x upper bound
c     sl3       i    Slack variable, y lower bound
c     sl4       i    Slack variable, y upper bound
c     sl5       i    Slack variable, z lower bound
c     sl6       i    Slack variable, z upper bound
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ip, isp, izf, ispc, ispc0, ispc1
      integer centring, ixc, iyc, izc, nxc, nyc, nzc, odd, oddw
      integer pts (0:26)
      double precision ww (0:1)

c     cellc     Cell centered (module mgpreset)
c     centring  0 = cellcentering, 1 = vertex centering
c     ip        Local counting variable i-direction
c     isp       Counter to stencil plane
c     ispc      Counter to stencil plane (coarse grid)
c     ispc0     Counter to stencil plane (coarse grid)
c     ispc1     Counter to stencil plane (coarse grid)
c     ixc       Counter to number of grid x points on coarse grid
c     iyc       Counter to number of grid y points on coarse grid
c     izc       Counter to number of grid z points on coarse grid
c     izf       Counter to number of grid z points on fine grid
c     nxc       Number of grid x points on coarse grid
c     nyc       Number of grid y points on coarse grid
c     nzc       Number of grid z points on coarse grid
c     odd       Indicator which interpolation weight to use
c     oddw      Indicator which interpolation weight to use
c     pts       Array of indices of stencil points 3D
c     ww        Weight(s)
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     RAPXY3    Forms coarse grid oper semic in x & y-direction (MG3)
c     VECMAP    Maps stencil indices onto array pts
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      nxc = nxf/2+1
      nyc = nyf/2+1
      nzc = nzf/2+1
      centring = mod (nzf, 2)

c--- we map the stencil points onto a vector for ease of addressing
      call vecmap (pts)

c--- first zeroize the whole goddam coarse grid matrix:
      do ip = 1, 27
        do izc = 1, nzc
        do iyc = 1, nyc
        do ixc = 1, nxc
        coarsmat (ixc, iyc, izc, ip) = 0d0
        end do
        end do
        end do
      end do

      if (centring.eq.CELLC) then

c--- cell centered. We distribute the fine grid coefficients to the

c     --- coarse grid with the appropriate weight.
c             a@    b@    c@
c         A0           B0          C0
c                   a#    b#    c#
c         .75 a@ -> A0         .25 a@ -> B0
c         .25 b@ -> A0         .75 b@ -> B0
c                 .75 c@ -> B0        .25 c@ -> C0
c         .25 a# -> A0         .75 a# -> B0
c                 .75 b# -> B0        .25 b# -> C0
c                 .25 c# -> B0        .75 c# -> C0
c         @ = odd, #= even
c         First A must be 0, last C must be 0

c--- weights are .25 and .75

      ww (1) = 0.25d0
      ww (0) = 0.75d0

c     --- first the generic case

      do izf = 1, nzf

        odd = mod (izf, 2)
        izc = izf/2 + 1

c       --- all stencil planes must be distributed.
c           --- isp = 0 = a coefficient, isp = 1 = b, isp=2 = c

        do isp = 0, 2
          call RAPxy3 (finemat, scratch2, scratch1, nxf, nyf, nzf, izf,
     +         isp,
     +  sl1, sl2, sl3, sl4, sl5, sl6)

c---debug

c         --- write (*,*) "Z plane", izf, " Stencil Plane", isp
c             write (*, '(9f8.3)') scratch2
c             -- end debug

          ispc = (isp + odd)/2
          oddw = mod (isp + odd, 2)
          if ((izc.gt.1) .or. (ispc.eq.1)) then

c         --- on the z=1 plane the lower stencil plane remains 0

            do ip = 0, 8
            do iyc = 1, nyc
            do ixc = 1, nxc
               coarsmat (ixc, iyc, izc, pts (9*ispc + ip))=
     +         coarsmat (ixc, iyc, izc, pts (9*ispc + ip))  +
     +         ww (oddw) * scratch2 (ixc, iyc, ip)
            end do
            end do
            end do
          end if
          if ((izc.lt.nzc) .or. (ispc.eq.0)) then

c         --- on the izc = nzc plane the upper stencil plane remains 0

            do ip = 0, 8
            do iyc = 1, nyc
            do ixc = 1, nxc
               coarsmat (ixc, iyc, izc, pts (9*(ispc+1) + ip))=
     +         coarsmat (ixc, iyc, izc, pts (9*(ispc+1) + ip))  +
     +         ww (1-oddw) * scratch2 (ixc, iyc, ip)
            end do
            end do
            end do
          end if
        end do
      end do

c     --- end cell centered case

      else

c     --- vertex centered case
c         this falls apart in two separate cases: even and odd
c         fine grid lines
c         The even lines distribute between TWO coarse grid
c         lines each with parial weight 0.5
c         The odd lines distribute like:
c              A0   a@    B@   c@   C0
c         The notation B@ means that B0 and b@ coincide and b@ distributes
c         to B0 with weight 1.
c         a@ -> A0  and B0 each with weight 0.5
c         c@ -> C0  and B0 each with weight 0.5
c         No special measures at begin and end are necessary because of the
c         appropriate fine grid coefficients are already 0.

      do izf = 1, nzf, 2
      izc = (izf + 1)/2

c     --- odd gridlines

        do isp = 0, 2
          call RAPxy3 (finemat, scratch2, scratch1, nxf, nyf, nzf, izf,
     +         isp,
     +  sl1, sl2, sl3, sl4, sl5, sl6)

c---debug

c         --- write (*,*) "Z plane", izf, " Stencil Plane", isp
c             write (*, '(9f8.3)') scratch2
c             -- end debug

           ispc0 = (isp + 1) / 2
           ispc1 = (isp + 2) / 2
           do ip = 0, 8
              do iyc = 1, nyc
              do ixc = 1, nxc
                coarsmat (ixc, iyc,izc, pts (9*ispc0 + ip))=
     +          coarsmat (ixc, iyc,izc, pts (9*ispc0 + ip))  +
     +            0.5d0 * scratch2 (ixc, iyc, ip)
                coarsmat (ixc, iyc,izc, pts (9*ispc1 + ip))=
     +          coarsmat (ixc, iyc,izc, pts (9*ispc1 + ip))  +
     +            0.5d0 * scratch2 (ixc, iyc, ip)
              end do
              end do
           end do
         end do
      end do

c--- even lines

c     --- The even lines distribute to TWO coarse grid lines according to the
c         following pattern:
c             (A0)       B0       C0
c                        a@  b@   c@
c                        A1       B1      (C1)
c         a@ distributes to A1, B0 with weight .5
c         b@ distributes to all four with weight .25
c         c@ distributes to B1, C0 with weight .5

      do izf = 2, nzf, 2
        izc = izf / 2
        do isp = 0, 2
          call RAPxy3 (finemat, scratch2, scratch1, nxf, nyf, nzf, izf,
     +                 isp,
     +  sl1, sl2, sl3, sl4, sl5, sl6)

c---debug

c         --- write (*,*) "Z plane", izf, " Stencil Plane", isp
c             write (*, '(9f8.3)') scratch2
c             -- end debug

        if (isp.eq.1) then
        do ip = 0, 8
          do ixc = 1, nxc
          do iyc = 1, nyc
          coarsmat (ixc, iyc, izc, pts (9 + ip)) =
     +            coarsmat (ixc, iyc, izc, pts (9 + ip)) +
     +            0.25d0 * scratch2 (ixc, iyc, ip)
          coarsmat (ixc, iyc, izc, pts (18 + ip)) =
     +            coarsmat (ixc, iyc, izc, pts (18 + ip)) +
     +            0.25d0 * scratch2 (ixc, iyc, ip)
          coarsmat (ixc, iyc, izc + 1, pts (ip)) =
     +            coarsmat (ixc, iyc, izc + 1, pts (ip)) +
     +            0.25d0 * scratch2 (ixc, iyc, ip)
          coarsmat (ixc, iyc, izc + 1, pts (9 + ip)) =
     +            coarsmat (ixc, iyc, izc + 1, pts (9 + ip)) +
     +            0.25d0 * scratch2 (ixc, iyc, ip)
          end do
          end do
        end do
        else
        do ip = 0,8
          do ixc = 1, nxc
          do iyc = 1, nyc
          coarsmat (ixc, iyc, izc, pts (9*(isp/2 + 1) + ip))=
     +    coarsmat (ixc, iyc, izc, pts (9*(isp/2 + 1) + ip))+
     +            0.5d0 * scratch2 (ixc, iyc, ip)
          coarsmat (ixc, iyc, izc+1, pts (9*(isp/2) + ip))=
     +    coarsmat (ixc, iyc, izc+1, pts (9*(isp/2) + ip))+
     +            0.5d0 * scratch2 (ixc, iyc, ip)
          end do
          end do
        end do
        end if
        end do
      end do

c     --- end case vertex centered

      end if
      end
