      subroutine RAPy3 ( finemat, coarsmat, nxf, nyf, nzf,
     +  sl1, sl2, sl3, sl4, sl5, sl6)
c ======================================================================
c
c        programmer     Jos van Kan
c        version 1.2    date   01-07-1997
c           correction of inter grid operators cell centered
c        version 1.1    date   04-06-1997
c           isnazification of all variables and subroutine calls
c           all truncated to max 8 characters.
c        version 1.0    date   12-03-1997
c
c   copyright (c) 1997
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   author.
c
c   pilotcode 3d mgpackage
c
c **********************************************************************
c
c                       MODULES USED
c
      use mgpreset
c **********************************************************************
c
c                       DESCRIPTION
c
c   Calculate RAP matrix with semicoarsening in y-direction
c   centring shall be decided by the parity of nyf.
c **********************************************************************
c
c                       KEYWORDS
c
c    multi_grid
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer sl1, sl2, sl3, sl4, sl5, sl6
      integer nxf, nyf, nzf
      double precision
     +finemat(1-sl1:nxf+sl2,1-sl3:nyf+sl4,
     + 1-sl5:nzf+sl6, 27),
     + coarsmat (nxf, nyf/2+1, nzf, 27)

c     coarsmat  o    Matrix on coarse level
c     finemat   i    Matrix on fine level
c     nxf       i    Number of grid x points on fine grid
c     nyf       i    Number of grid y points on fine grid
c     nzf       i    Number of grid z points on fine grid
c     sl1       i    Slack variable, x lower bound
c     sl2       i    Slack variable, x upper bound
c     sl3       i    Slack variable, y lower bound
c     sl4       i    Slack variable, y upper bound
c     sl5       i    Slack variable, z lower bound
c     sl6       i    Slack variable, z upper bound
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ixf, iyf, izf, iyc, nyc, centring
      double precision aa0, aa1

c     aa0       Interpolation weight in restriction/prolongation
c     aa1       Interpolation weight in restriction/prolongation
c     cellc     Cell centered (module mgpreset)
c     centring  0 = cellcentering, 1 = vertex centering
c     ixf       Counter to number of grid x points on fine grid
c     iyc       Counter to number of grid y points on coarse grid
c     iyf       Counter to number of grid y points on fine grid
c     izf       Counter to number of grid z points on fine grid
c     nyc       Number of grid y points on coarse grid
c     p000      Index of point in stencil 3D (module mgpreset)
c     p001      Index of point in stencil 3D (module mgpreset)
c     p002      Index of point in stencil 3D (module mgpreset)
c     p010      Index of point in stencil 3D (module mgpreset)
c     p011      Index of point in stencil 3D (module mgpreset)
c     p012      Index of point in stencil 3D (module mgpreset)
c     p020      Index of point in stencil 3D (module mgpreset)
c     p021      Index of point in stencil 3D (module mgpreset)
c     p022      Index of point in stencil 3D (module mgpreset)
c     p100      Index of point in stencil 3D (module mgpreset)
c     p101      Index of point in stencil 3D (module mgpreset)
c     p102      Index of point in stencil 3D (module mgpreset)
c     p110      Index of point in stencil 3D (module mgpreset)
c     p111      Index of point in stencil 3D (module mgpreset)
c     p112      Index of point in stencil 3D (module mgpreset)
c     p120      Index of point in stencil 3D (module mgpreset)
c     p121      Index of point in stencil 3D (module mgpreset)
c     p122      Index of point in stencil 3D (module mgpreset)
c     p200      Index of point in stencil 3D (module mgpreset)
c     p201      Index of point in stencil 3D (module mgpreset)
c     p202      Index of point in stencil 3D (module mgpreset)
c     p210      Index of point in stencil 3D (module mgpreset)
c     p211      Index of point in stencil 3D (module mgpreset)
c     p212      Index of point in stencil 3D (module mgpreset)
c     p220      Index of point in stencil 3D (module mgpreset)
c     p221      Index of point in stencil 3D (module mgpreset)
c     p222      Index of point in stencil 3D (module mgpreset)
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      nyc = nyf/2+1
      centring = mod (nyf, 2)
      if (centring.eq.CELLC) then

c     --- BEGIN Case cell centered

      aa0 = 0.25d0
      aa1 = 0.75d0
      do izf = 1, nzf
      do ixf = 1, nxf

c     --- stencil front plane

        coarsmat (ixf, 1, izf,p000) = 0d0
        coarsmat (ixf, 1, izf,p100) = 0d0
        coarsmat (ixf, 1, izf,p200) = 0d0
        coarsmat (ixf, 1, izf,p001) = 0d0
        coarsmat (ixf, 1, izf,p101) = 0d0
        coarsmat (ixf, 1, izf,p201) = 0d0
        coarsmat (ixf, 1, izf,p002) = 0d0
        coarsmat (ixf, 1, izf,p102) = 0d0
        coarsmat (ixf, 1, izf,p202) = 0d0
        do iyc = 2, nyc-1
          iyf = 2 * iyc - 2
       coarsmat (ixf,iyc,izf,p000) = aa1* finemat (ixf,iyf,izf,p000) +
     + aa0 * (finemat (ixf,iyf,izf,p010) +finemat (ixf,iyf+1,izf,p000))
       coarsmat (ixf ,iyc,izf,p100) = aa1* finemat (ixf ,iyf,izf,p100) +
     + aa0 * (finemat (ixf,iyf,izf,p110) +finemat (ixf,iyf+1,izf,p100))
       coarsmat (ixf ,iyc,izf,p200) = aa1* finemat (ixf ,iyf,izf,p200) +
     + aa0 * (finemat (ixf,iyf,izf,p210) +finemat (ixf,iyf+1,izf,p200))
       coarsmat (ixf,iyc,izf,p001) = aa1* finemat (ixf,iyf,izf,p001) +
     + aa0 * (finemat (ixf,iyf,izf,p011) +finemat (ixf,iyf+1,izf,p001))
       coarsmat (ixf ,iyc,izf,p101) = aa1* finemat (ixf ,iyf,izf,p101) +
     + aa0 * (finemat (ixf,iyf,izf,p111) +finemat (ixf,iyf+1,izf,p101))
       coarsmat (ixf ,iyc,izf,p201) = aa1* finemat (ixf ,iyf,izf,p201) +
     + aa0 * (finemat (ixf,iyf,izf,p211) +finemat (ixf,iyf+1,izf,p201))
       coarsmat (ixf,iyc,izf,p002) = aa1* finemat (ixf,iyf,izf,p002) +
     + aa0 * (finemat (ixf,iyf,izf,p012) +finemat (ixf,iyf+1,izf,p002))
       coarsmat (ixf ,iyc,izf,p102) = aa1* finemat (ixf ,iyf,izf,p102) +
     + aa0 * (finemat (ixf,iyf,izf,p112) +finemat (ixf,iyf+1,izf,p102))
       coarsmat (ixf ,iyc,izf,p202) = aa1* finemat (ixf ,iyf,izf,p202) +
     + aa0 * (finemat (ixf,iyf,izf,p212) +finemat (ixf,iyf+1,izf,p202))
        end do

       coarsmat (ixf,nyc,izf,p000) = aa1* finemat (ixf,nyf,izf,p000) +
     + aa0 * (finemat (ixf,nyf,izf,p010))
       coarsmat (ixf ,nyc,izf,p100) = aa1* finemat (ixf ,nyf,izf,p100) +
     + aa0 * (finemat (ixf,nyf,izf,p110))
       coarsmat (ixf ,nyc,izf,p200) = aa1* finemat (ixf ,nyf,izf,p200) +
     + aa0 * (finemat (ixf,nyf,izf,p210))
       coarsmat (ixf,nyc,izf,p001) = aa1* finemat (ixf,nyf,izf,p001) +
     + aa0 * (finemat (ixf,nyf,izf,p011))
       coarsmat (ixf ,nyc,izf,p101) = aa1* finemat (ixf ,nyf,izf,p101) +
     + aa0 * (finemat (ixf,nyf,izf,p111))
       coarsmat (ixf ,nyc,izf,p201) = aa1* finemat (ixf ,nyf,izf,p201) +
     + aa0 * (finemat (ixf,nyf,izf,p211))
       coarsmat (ixf,nyc,izf,p002) = aa1* finemat (ixf,nyf,izf,p002) +
     + aa0 * (finemat (ixf,nyf,izf,p012))
       coarsmat (ixf ,nyc,izf,p102) = aa1* finemat (ixf ,nyf,izf,p102) +
     + aa0 * (finemat (ixf,nyf,izf,p112))
       coarsmat (ixf ,nyc,izf,p202) = aa1* finemat (ixf ,nyf,izf,p202) +
     + aa0 * (finemat (ixf,nyf,izf,p212))

c      --- end stencil front plane

c          --- stencil center plane

          coarsmat(ixf,1,izf,p010)=
     +      aa1*(finemat(ixf,1,izf,p000))
     +     +aa0*finemat(ixf,1,izf,p020)+
     +      aa1*(finemat(ixf,1,izf,p010))
          coarsmat(ixf,1,izf,p110)=
     +      aa1*(finemat(ixf,1,izf,p100))
     +     +aa0*finemat(ixf,1,izf,p120)+
     +      aa1*(finemat(ixf,1,izf,p110))
          coarsmat(ixf,1,izf,p210)=
     +      aa1*(finemat(ixf,1,izf,p200))
     +     +aa0*finemat(ixf,1,izf,p220)+
     +      aa1*(finemat(ixf,1,izf,p210))
          coarsmat(ixf,1,izf,p011)=
     +      aa1*(finemat(ixf,1,izf,p001))
     +     +aa0*finemat(ixf,1,izf,p021)+
     +      aa1*(finemat(ixf,1,izf,p011))
          coarsmat(ixf,1,izf,p111)=
     +      aa1*(finemat(ixf,1,izf,p101))
     +     +aa0*finemat(ixf,1,izf,p121)+
     +      aa1*(finemat(ixf,1,izf,p111))
          coarsmat(ixf,1,izf,p211)=
     +      aa1*(finemat(ixf,1,izf,p201))
     +     +aa0*finemat(ixf,1,izf,p221)+
     +      aa1*(finemat(ixf,1,izf,p211))
          coarsmat(ixf,1,izf,p012)=
     +      aa1*(finemat(ixf,1,izf,p002))
     +     +aa0*finemat(ixf,1,izf,p022)+
     +      aa1*(finemat(ixf,1,izf,p012))
          coarsmat(ixf,1,izf,p112)=
     +      aa1*(finemat(ixf,1,izf,p102))
     +     +aa0*finemat(ixf,1,izf,p122)+
     +      aa1*(finemat(ixf,1,izf,p112))
          coarsmat(ixf,1,izf,p212)=
     +      aa1*(finemat(ixf,1,izf,p202))
     +     +aa0*finemat(ixf,1,izf,p222)+
     +      aa1*(finemat(ixf,1,izf,p212))
        do iyc = 2, nyc - 1
          iyf = 2 * iyc - 2
          coarsmat(ixf,iyc,izf,p010)=aa0*finemat(ixf,iyf,izf,p000)+
     +      aa1*(finemat(ixf,iyf,izf,p010)+finemat(ixf,iyf+1,izf,p000))
     ++aa0*finemat(ixf,iyf+1,izf,p020)+
     +      aa1*(finemat(ixf,iyf,izf,p020)+finemat(ixf,iyf+1,izf,p010))
          coarsmat(ixf,iyc,izf,p110)=aa0*finemat(ixf,iyf,izf,p100)+
     +      aa1*(finemat(ixf,iyf,izf,p110)+finemat(ixf,iyf+1,izf,p100))
     ++aa0*finemat(ixf,iyf+1,izf,p120)+
     +      aa1*(finemat(ixf,iyf,izf,p120)+finemat(ixf,iyf+1,izf,p110))
          coarsmat(ixf,iyc,izf,p210)=aa0*finemat(ixf,iyf,izf,p200)+
     +      aa1*(finemat(ixf,iyf,izf,p210)+finemat(ixf,iyf+1,izf,p200))
     ++aa0*finemat(ixf,iyf+1,izf,p220)+
     +      aa1*(finemat(ixf,iyf,izf,p220)+finemat(ixf,iyf+1,izf,p210))
          coarsmat(ixf,iyc,izf,p011)=aa0*finemat(ixf,iyf,izf,p001)+
     +      aa1*(finemat(ixf,iyf,izf,p011)+finemat(ixf,iyf+1,izf,p001))
     ++aa0*finemat(ixf,iyf+1,izf,p021)+
     +      aa1*(finemat(ixf,iyf,izf,p021)+finemat(ixf,iyf+1,izf,p011))
          coarsmat(ixf,iyc,izf,p111)=aa0*finemat(ixf,iyf,izf,p101)+
     +      aa1*(finemat(ixf,iyf,izf,p111)+finemat(ixf,iyf+1,izf,p101))
     ++aa0*finemat(ixf,iyf+1,izf,p121)+
     +      aa1*(finemat(ixf,iyf,izf,p121)+finemat(ixf,iyf+1,izf,p111))
          coarsmat(ixf,iyc,izf,p211)=aa0*finemat(ixf,iyf,izf,p201)+
     +      aa1*(finemat(ixf,iyf,izf,p211)+finemat(ixf,iyf+1,izf,p201))
     ++aa0*finemat(ixf,iyf+1,izf,p221)+
     +      aa1*(finemat(ixf,iyf,izf,p221)+finemat(ixf,iyf+1,izf,p211))
          coarsmat(ixf,iyc,izf,p012)=aa0*finemat(ixf,iyf,izf,p002)+
     +      aa1*(finemat(ixf,iyf,izf,p012)+finemat(ixf,iyf+1,izf,p002))
     ++aa0*finemat(ixf,iyf+1,izf,p022)+
     +      aa1*(finemat(ixf,iyf,izf,p022)+finemat(ixf,iyf+1,izf,p012))
          coarsmat(ixf,iyc,izf,p112)=aa0*finemat(ixf,iyf,izf,p102)+
     +      aa1*(finemat(ixf,iyf,izf,p112)+finemat(ixf,iyf+1,izf,p102))
     ++aa0*finemat(ixf,iyf+1,izf,p122)+
     +      aa1*(finemat(ixf,iyf,izf,p122)+finemat(ixf,iyf+1,izf,p112))
          coarsmat(ixf,iyc,izf,p212)=aa0*finemat(ixf,iyf,izf,p202)+
     +      aa1*(finemat(ixf,iyf,izf,p212)+finemat(ixf,iyf+1,izf,p202))
     ++aa0*finemat(ixf,iyf+1,izf,p222)+
     +      aa1*(finemat(ixf,iyf,izf,p222)+finemat(ixf,iyf+1,izf,p212))
        end do

          coarsmat(ixf,nyc,izf,p010)=aa0*finemat(ixf,nyf,izf,p000)+
     +      aa1*(finemat(ixf,nyf,izf,p010))+
     +      aa1*(finemat(ixf,nyf,izf,p020))
          coarsmat(ixf,nyc,izf,p110)=aa0*finemat(ixf,nyf,izf,p100)+
     +      aa1*(finemat(ixf,nyf,izf,p110))+
     +      aa1*(finemat(ixf,nyf,izf,p120))
          coarsmat(ixf,nyc,izf,p210)=aa0*finemat(ixf,nyf,izf,p200)+
     +      aa1*(finemat(ixf,nyf,izf,p210))+
     +      aa1*(finemat(ixf,nyf,izf,p220))
          coarsmat(ixf,nyc,izf,p011)=aa0*finemat(ixf,nyf,izf,p001)+
     +      aa1*(finemat(ixf,nyf,izf,p011))+
     +      aa1*(finemat(ixf,nyf,izf,p021))
          coarsmat(ixf,nyc,izf,p111)=aa0*finemat(ixf,nyf,izf,p101)+
     +      aa1*(finemat(ixf,nyf,izf,p111))+
     +      aa1*(finemat(ixf,nyf,izf,p121))
          coarsmat(ixf,nyc,izf,p211)=aa0*finemat(ixf,nyf,izf,p201)+
     +      aa1*(finemat(ixf,nyf,izf,p211))+
     +      aa1*(finemat(ixf,nyf,izf,p221))
          coarsmat(ixf,nyc,izf,p012)=aa0*finemat(ixf,nyf,izf,p002)+
     +      aa1*(finemat(ixf,nyf,izf,p012))+
     +      aa1*(finemat(ixf,nyf,izf,p022))
          coarsmat(ixf,nyc,izf,p112)=aa0*finemat(ixf,nyf,izf,p102)+
     +      aa1*(finemat(ixf,nyf,izf,p112))+
     +      aa1*(finemat(ixf,nyf,izf,p122))
          coarsmat(ixf,nyc,izf,p212)=aa0*finemat(ixf,nyf,izf,p202)+
     +      aa1*(finemat(ixf,nyf,izf,p212))+
     +      aa1*(finemat(ixf,nyf,izf,p222))

c---end stencil center plane

c---stencil back plane
          coarsmat(ixf,1,izf,p020)=aa1*finemat(ixf,1,izf,p020)+
     + aa0*(finemat(ixf,1,izf,p010))
          coarsmat(ixf,1,izf,p120)=aa1*finemat(ixf,1,izf,p120)+
     + aa0*(finemat(ixf,1,izf,p110))
          coarsmat(ixf,1,izf,p220)=aa1*finemat(ixf,1,izf,p220)+
     + aa0*(finemat(ixf,1,izf,p210))
          coarsmat(ixf,1,izf,p021)=aa1*finemat(ixf,1,izf,p021)+
     + aa0*(finemat(ixf,1,izf,p011))
          coarsmat(ixf,1,izf,p121)=aa1*finemat(ixf,1,izf,p121)+
     + aa0*(finemat(ixf,1,izf,p111))
          coarsmat(ixf,1,izf,p221)=aa1*finemat(ixf,1,izf,p221)+
     + aa0*(finemat(ixf,1,izf,p211))
          coarsmat(ixf,1,izf,p022)=aa1*finemat(ixf,1,izf,p022)+
     + aa0*(finemat(ixf,1,izf,p012))
          coarsmat(ixf,1,izf,p122)=aa1*finemat(ixf,1,izf,p122)+
     + aa0*(finemat(ixf,1,izf,p112))
          coarsmat(ixf,1,izf,p222)=aa1*finemat(ixf,1,izf,p222)+
     + aa0*(finemat(ixf,1,izf,p212))
        do iyc=2,nyc-1
          iyf=2*iyc-2
          coarsmat(ixf,iyc,izf,p020)=aa1*finemat(ixf,iyf+1,izf,p020)+
     + aa0*(finemat(ixf,iyf,izf,p020)+finemat(ixf,iyf+1,izf,p010))
          coarsmat(ixf,iyc,izf,p120)=aa1*finemat(ixf,iyf+1,izf,p120)+
     + aa0*(finemat(ixf,iyf,izf,p120)+finemat(ixf,iyf+1,izf,p110))
          coarsmat(ixf,iyc,izf,p220)=aa1*finemat(ixf,iyf+1,izf,p220)+
     + aa0*(finemat(ixf,iyf,izf,p220)+finemat(ixf,iyf+1,izf,p210))
          coarsmat(ixf,iyc,izf,p021)=aa1*finemat(ixf,iyf+1,izf,p021)+
     + aa0*(finemat(ixf,iyf,izf,p021)+finemat(ixf,iyf+1,izf,p011))
          coarsmat(ixf,iyc,izf,p121)=aa1*finemat(ixf,iyf+1,izf,p121)+
     + aa0*(finemat(ixf,iyf,izf,p121)+finemat(ixf,iyf+1,izf,p111))
          coarsmat(ixf,iyc,izf,p221)=aa1*finemat(ixf,iyf+1,izf,p221)+
     + aa0*(finemat(ixf,iyf,izf,p221)+finemat(ixf,iyf+1,izf,p211))
          coarsmat(ixf,iyc,izf,p022)=aa1*finemat(ixf,iyf+1,izf,p022)+
     + aa0*(finemat(ixf,iyf,izf,p022)+finemat(ixf,iyf+1,izf,p012))
          coarsmat(ixf,iyc,izf,p122)=aa1*finemat(ixf,iyf+1,izf,p122)+
     + aa0*(finemat(ixf,iyf,izf,p122)+finemat(ixf,iyf+1,izf,p112))
          coarsmat(ixf,iyc,izf,p222)=aa1*finemat(ixf,iyf+1,izf,p222)+
     + aa0*(finemat(ixf,iyf,izf,p222)+finemat(ixf,iyf+1,izf,p212))
        end do
        coarsmat(ixf,nyc,izf,p020)=0d0
        coarsmat(ixf,nyc,izf,p120)=0d0
        coarsmat(ixf,nyc,izf,p220)=0d0
        coarsmat(ixf,nyc,izf,p021)=0d0
        coarsmat(ixf,nyc,izf,p121)=0d0
        coarsmat(ixf,nyc,izf,p221)=0d0
        coarsmat(ixf,nyc,izf,p022)=0d0
        coarsmat(ixf,nyc,izf,p122)=0d0
        coarsmat(ixf,nyc,izf,p222)=0d0

c---end stencil back plane

      end do
      end do

c---END Case cell centered

      else

c--- BEGIN Case vertex centered
      aa0=0.25d0
      aa1=0.5d0

      do izf = 1, nzf

c---BEGIN internal lines
       do iyc=2,nyc-1
         iyf=2*iyc-1

c---stencil center plane
        do ixf=1,nxf
      coarsmat(ixf,iyc,izf,p100)=aa1*(finemat(ixf,iyf-1,izf,p100)
     ++finemat(ixf,iyf,izf,p100))+
     + aa0*finemat(ixf,iyf-1,izf,p110)
      coarsmat(ixf,iyc,izf,p120)=aa1*(finemat(ixf,iyf+1,izf,p120)
     ++finemat(ixf,iyf,izf,p120))+
     + aa0*finemat(ixf,iyf+1,izf,p110)
      coarsmat(ixf,iyc,izf,p110)=finemat(ixf,iyf,izf,p110)+
     +aa1*(finemat(ixf,iyf-1,izf,p120)+finemat(ixf,iyf,izf,p100))+
     +aa1*(finemat(ixf,iyf+1,izf,p100)+finemat(ixf,iyf,izf,p120))+
     + aa0*finemat(ixf,iyf-1,izf,p110)+
     + aa0*finemat(ixf,iyf+1,izf,p110)
      coarsmat(ixf,iyc,izf,p101)=aa1*(finemat(ixf,iyf-1,izf,p101)
     ++finemat(ixf,iyf,izf,p101))+
     + aa0*finemat(ixf,iyf-1,izf,p111)
      coarsmat(ixf,iyc,izf,p121)=aa1*(finemat(ixf,iyf+1,izf,p121)
     ++finemat(ixf,iyf,izf,p121))+
     + aa0*finemat(ixf,iyf+1,izf,p111)
      coarsmat(ixf,iyc,izf,p111)=finemat(ixf,iyf,izf,p111)+
     +aa1*(finemat(ixf,iyf-1,izf,p121)+finemat(ixf,iyf,izf,p101))+
     +aa1*(finemat(ixf,iyf+1,izf,p101)+finemat(ixf,iyf,izf,p121))+
     + aa0*finemat(ixf,iyf-1,izf,p111)+
     + aa0*finemat(ixf,iyf+1,izf,p111)
      coarsmat(ixf,iyc,izf,p102)=aa1*(finemat(ixf,iyf-1,izf,p102)
     ++finemat(ixf,iyf,izf,p102))+
     + aa0*finemat(ixf,iyf-1,izf,p112)
      coarsmat(ixf,iyc,izf,p122)=aa1*(finemat(ixf,iyf+1,izf,p122)
     ++finemat(ixf,iyf,izf,p122))+
     + aa0*finemat(ixf,iyf+1,izf,p112)
      coarsmat(ixf,iyc,izf,p112)=finemat(ixf,iyf,izf,p112)+
     +aa1*(finemat(ixf,iyf-1,izf,p122)+finemat(ixf,iyf,izf,p102))+
     +aa1*(finemat(ixf,iyf+1,izf,p102)+finemat(ixf,iyf,izf,p122))+
     + aa0*finemat(ixf,iyf-1,izf,p112)+
     + aa0*finemat(ixf,iyf+1,izf,p112)
        end do

c---stencil left plane
        do ixf=1,nxf
          coarsmat(ixf,iyc,izf,p000)=aa1*(finemat(ixf,iyf-1,izf,p000)
     +    +finemat(ixf,iyf,izf,p000))+
     +aa0*finemat(ixf,iyf-1,izf,p010)
          coarsmat(ixf,iyc,izf,p020)=aa1*(finemat(ixf,iyf+1,izf,p020)
     +    +finemat(ixf,iyf,izf,p020))+
     +aa0*finemat(ixf,iyf+1,izf,p010)
          coarsmat(ixf,iyc,izf,p010)=finemat(ixf,iyf,izf,p010)+
     +    aa1*(finemat(ixf,iyf-1,izf,p020)+finemat(ixf,iyf,izf,p000))+
     +    aa1*(finemat(ixf,iyf+1,izf,p000)+finemat(ixf,iyf,izf,p020))+
     + aa0*finemat(ixf,iyf-1,izf,p010)+
     + aa0*finemat(ixf,iyf+1,izf,p010)
          coarsmat(ixf,iyc,izf,p001)=aa1*(finemat(ixf,iyf-1,izf,p001)
     +    +finemat(ixf,iyf,izf,p001))+
     +aa0*finemat(ixf,iyf-1,izf,p011)
          coarsmat(ixf,iyc,izf,p021)=aa1*(finemat(ixf,iyf+1,izf,p021)
     +    +finemat(ixf,iyf,izf,p021))+
     +aa0*finemat(ixf,iyf+1,izf,p011)
          coarsmat(ixf,iyc,izf,p011)=finemat(ixf,iyf,izf,p011)+
     +    aa1*(finemat(ixf,iyf-1,izf,p021)+finemat(ixf,iyf,izf,p001))+
     +    aa1*(finemat(ixf,iyf+1,izf,p001)+finemat(ixf,iyf,izf,p021))+
     + aa0*finemat(ixf,iyf-1,izf,p011)+
     + aa0*finemat(ixf,iyf+1,izf,p011)
          coarsmat(ixf,iyc,izf,p002)=aa1*(finemat(ixf,iyf-1,izf,p002)
     +    +finemat(ixf,iyf,izf,p002))+
     +aa0*finemat(ixf,iyf-1,izf,p012)
          coarsmat(ixf,iyc,izf,p022)=aa1*(finemat(ixf,iyf+1,izf,p022)
     +    +finemat(ixf,iyf,izf,p022))+
     +aa0*finemat(ixf,iyf+1,izf,p012)
          coarsmat(ixf,iyc,izf,p012)=finemat(ixf,iyf,izf,p012)+
     +    aa1*(finemat(ixf,iyf-1,izf,p022)+finemat(ixf,iyf,izf,p002))+
     +    aa1*(finemat(ixf,iyf+1,izf,p002)+finemat(ixf,iyf,izf,p022))+
     + aa0*finemat(ixf,iyf-1,izf,p012)+
     + aa0*finemat(ixf,iyf+1,izf,p012)
        end do

c--stencil right plane

        do ixf=1,nxf
      coarsmat(ixf,iyc,izf,p200)=aa1*(finemat(ixf,iyf-1,izf,p200)
     ++finemat(ixf,iyf,izf,p200))+
     + aa0*finemat(ixf,iyf-1,izf,p210)
      coarsmat(ixf,iyc,izf,p220)=aa1*(finemat(ixf,iyf+1,izf,p220)
     ++finemat(ixf,iyf,izf,p220))+
     + aa0*finemat(ixf,iyf+1,izf,p210)
      coarsmat(ixf,iyc,izf,p210)=finemat(ixf,iyf,izf,p210)+
     +aa1*(finemat(ixf,iyf-1,izf,p220)+finemat(ixf,iyf,izf,p200))+
     +aa1*(finemat(ixf,iyf+1,izf,p200)+finemat(ixf,iyf,izf,p220))+
     + aa0*finemat(ixf,iyf-1,izf,p210)+
     + aa0*finemat(ixf,iyf+1,izf,p210)
      coarsmat(ixf,iyc,izf,p201)=aa1*(finemat(ixf,iyf-1,izf,p201)
     ++finemat(ixf,iyf,izf,p201))+
     + aa0*finemat(ixf,iyf-1,izf,p211)
      coarsmat(ixf,iyc,izf,p221)=aa1*(finemat(ixf,iyf+1,izf,p221)
     ++finemat(ixf,iyf,izf,p221))+
     + aa0*finemat(ixf,iyf+1,izf,p211)
      coarsmat(ixf,iyc,izf,p211)=finemat(ixf,iyf,izf,p211)+
     +aa1*(finemat(ixf,iyf-1,izf,p221)+finemat(ixf,iyf,izf,p201))+
     +aa1*(finemat(ixf,iyf+1,izf,p201)+finemat(ixf,iyf,izf,p221))+
     + aa0*finemat(ixf,iyf-1,izf,p211)+
     + aa0*finemat(ixf,iyf+1,izf,p211)
      coarsmat(ixf,iyc,izf,p202)=aa1*(finemat(ixf,iyf-1,izf,p202)
     ++finemat(ixf,iyf,izf,p202))+
     + aa0*finemat(ixf,iyf-1,izf,p212)
      coarsmat(ixf,iyc,izf,p222)=aa1*(finemat(ixf,iyf+1,izf,p222)
     ++finemat(ixf,iyf,izf,p222))+
     + aa0*finemat(ixf,iyf+1,izf,p212)
      coarsmat(ixf,iyc,izf,p212)=finemat(ixf,iyf,izf,p212)+
     +aa1*(finemat(ixf,iyf-1,izf,p222)+finemat(ixf,iyf,izf,p202))+
     +aa1*(finemat(ixf,iyf+1,izf,p202)+finemat(ixf,iyf,izf,p222))+
     + aa0*finemat(ixf,iyf-1,izf,p212)+
     + aa0*finemat(ixf,iyf+1,izf,p212)
        end do
      end do

c---END internal planes

c--- Front and back region boundary still have to be done

c     --- BEGIN front region boundary

c---stencil left plane

      do ixf=1,nxf
        coarsmat(ixf,1,izf,p000)=0d0
        coarsmat(ixf,1,izf,p010)=finemat(ixf,1,izf,p010)+
     +    aa1*(finemat(ixf,2,izf,p000)+finemat(ixf,1,izf,p020))+
     + aa0*finemat(ixf,2,izf,p010)
        coarsmat(ixf,1,izf,p020)=aa1*(finemat(ixf,2,izf,p020)
     +    +finemat(ixf,1,izf,p020))+
     +aa0*finemat(ixf,2,izf,p010)
        coarsmat(ixf,1,izf,p001)=0d0
        coarsmat(ixf,1,izf,p011)=finemat(ixf,1,izf,p011)+
     +    aa1*(finemat(ixf,2,izf,p001)+finemat(ixf,1,izf,p021))+
     + aa0*finemat(ixf,2,izf,p011)
        coarsmat(ixf,1,izf,p021)=aa1*(finemat(ixf,2,izf,p021)
     +    +finemat(ixf,1,izf,p021))+
     +aa0*finemat(ixf,2,izf,p011)
        coarsmat(ixf,1,izf,p002)=0d0
        coarsmat(ixf,1,izf,p012)=finemat(ixf,1,izf,p012)+
     +    aa1*(finemat(ixf,2,izf,p002)+finemat(ixf,1,izf,p022))+
     + aa0*finemat(ixf,2,izf,p012)
        coarsmat(ixf,1,izf,p022)=aa1*(finemat(ixf,2,izf,p022)
     +    +finemat(ixf,1,izf,p022))+
     +aa0*finemat(ixf,2,izf,p012)
      end do

c---stencil center column
      do ixf=1,nxf
        coarsmat(ixf,1,izf,p100)=0d0
        coarsmat(ixf,1,izf,p110)=finemat(ixf,1,izf,p110)+
     +    aa1*(finemat(ixf,2,izf,p100)+finemat(ixf,1,izf,p120))+
     + aa0*finemat(ixf,2,izf,p110)
        coarsmat(ixf,1,izf,p120)=aa1*(finemat(ixf,2,izf,p120)
     +    +finemat(ixf,1,izf,p120))+
     +aa0*finemat(ixf,2,izf,p110)
        coarsmat(ixf,1,izf,p101)=0d0
        coarsmat(ixf,1,izf,p111)=finemat(ixf,1,izf,p111)+
     +    aa1*(finemat(ixf,2,izf,p101)+finemat(ixf,1,izf,p121))+
     + aa0*finemat(ixf,2,izf,p111)
        coarsmat(ixf,1,izf,p121)=aa1*(finemat(ixf,2,izf,p121)
     +    +finemat(ixf,1,izf,p121))+
     +aa0*finemat(ixf,2,izf,p111)
        coarsmat(ixf,1,izf,p102)=0d0
        coarsmat(ixf,1,izf,p112)=finemat(ixf,1,izf,p112)+
     +    aa1*(finemat(ixf,2,izf,p102)+finemat(ixf,1,izf,p122))+
     + aa0*finemat(ixf,2,izf,p112)
        coarsmat(ixf,1,izf,p122)=aa1*(finemat(ixf,2,izf,p122)
     +    +finemat(ixf,1,izf,p122))+
     +aa0*finemat(ixf,2,izf,p112)
      end do

c---stencil right plane
      do ixf=1,nxf
        coarsmat(ixf,1,izf,p200)=0d0
        coarsmat(ixf,1,izf,p210)=finemat(ixf,1,izf,p210)+
     +    aa1*(finemat(ixf,2,izf,p200)+finemat(ixf,1,izf,p220))+
     + aa0*finemat(ixf,2,izf,p210)
        coarsmat(ixf,1,izf,p220)=aa1*(finemat(ixf,2,izf,p220)
     +    +finemat(ixf,1,izf,p220))+
     +aa0*finemat(ixf,2,izf,p210)
        coarsmat(ixf,1,izf,p201)=0d0
        coarsmat(ixf,1,izf,p211)=finemat(ixf,1,izf,p211)+
     +    aa1*(finemat(ixf,2,izf,p201)+finemat(ixf,1,izf,p221))+
     + aa0*finemat(ixf,2,izf,p211)
        coarsmat(ixf,1,izf,p221)=aa1*(finemat(ixf,2,izf,p221)
     +    +finemat(ixf,1,izf,p221))+
     +aa0*finemat(ixf,2,izf,p211)
        coarsmat(ixf,1,izf,p202)=0d0
        coarsmat(ixf,1,izf,p212)=finemat(ixf,1,izf,p212)+
     +    aa1*(finemat(ixf,2,izf,p202)+finemat(ixf,1,izf,p222))+
     + aa0*finemat(ixf,2,izf,p212)
        coarsmat(ixf,1,izf,p222)=aa1*(finemat(ixf,2,izf,p222)
     +    +finemat(ixf,1,izf,p222))+
     +aa0*finemat(ixf,2,izf,p212)
      end do

c---END front region boundary

c---back regionboundary

c     --- BEGIN back region boundary

c---stencil left plane

      do ixf=1,nxf
      coarsmat(ixf,nyc,izf,p000)=aa1*(finemat(ixf,nyf-1,izf,p000)
     ++finemat(ixf,nyf,izf,p000))+
     + aa0*finemat(ixf,nyf-1,izf,p010)
      coarsmat(ixf,nyc,izf,p020)=0d0
      coarsmat(ixf,nyc,izf,p010)=finemat(ixf,nyf,izf,p010)+
     +aa1*(finemat(ixf,nyf-1,izf,p020)+finemat(ixf,nyf,izf,p000))+
     + aa0*finemat(ixf,nyf-1,izf,p010)
      coarsmat(ixf,nyc,izf,p001)=aa1*(finemat(ixf,nyf-1,izf,p001)
     ++finemat(ixf,nyf,izf,p001))+
     + aa0*finemat(ixf,nyf-1,izf,p011)
      coarsmat(ixf,nyc,izf,p021)=0d0
      coarsmat(ixf,nyc,izf,p011)=finemat(ixf,nyf,izf,p011)+
     +aa1*(finemat(ixf,nyf-1,izf,p021)+finemat(ixf,nyf,izf,p001))+
     + aa0*finemat(ixf,nyf-1,izf,p011)
      coarsmat(ixf,nyc,izf,p002)=aa1*(finemat(ixf,nyf-1,izf,p002)
     ++finemat(ixf,nyf,izf,p002))+
     + aa0*finemat(ixf,nyf-1,izf,p012)
      coarsmat(ixf,nyc,izf,p022)=0d0
      coarsmat(ixf,nyc,izf,p012)=finemat(ixf,nyf,izf,p012)+
     +aa1*(finemat(ixf,nyf-1,izf,p022)+finemat(ixf,nyf,izf,p002))+
     + aa0*finemat(ixf,nyf-1,izf,p012)
      end do

c---stencilcentercolumn
      do ixf=1,nxf
      coarsmat(ixf,nyc,izf,p100)=aa1*(finemat(ixf,nyf-1,izf,p100)
     ++finemat(ixf,nyf,izf,p100))+
     + aa0*finemat(ixf,nyf-1,izf,p110)
      coarsmat(ixf,nyc,izf,p120)=0d0
      coarsmat(ixf,nyc,izf,p110)=finemat(ixf,nyf,izf,p110)+
     +aa1*(finemat(ixf,nyf-1,izf,p120)+finemat(ixf,nyf,izf,p100))+
     + aa0*finemat(ixf,nyf-1,izf,p110)
      coarsmat(ixf,nyc,izf,p101)=aa1*(finemat(ixf,nyf-1,izf,p101)
     ++finemat(ixf,nyf,izf,p101))+
     + aa0*finemat(ixf,nyf-1,izf,p111)
      coarsmat(ixf,nyc,izf,p121)=0d0
      coarsmat(ixf,nyc,izf,p111)=finemat(ixf,nyf,izf,p111)+
     +aa1*(finemat(ixf,nyf-1,izf,p121)+finemat(ixf,nyf,izf,p101))+
     + aa0*finemat(ixf,nyf-1,izf,p111)
      coarsmat(ixf,nyc,izf,p102)=aa1*(finemat(ixf,nyf-1,izf,p102)
     ++finemat(ixf,nyf,izf,p102))+
     + aa0*finemat(ixf,nyf-1,izf,p112)
      coarsmat(ixf,nyc,izf,p122)=0d0
      coarsmat(ixf,nyc,izf,p112)=finemat(ixf,nyf,izf,p112)+
     +aa1*(finemat(ixf,nyf-1,izf,p122)+finemat(ixf,nyf,izf,p102))+
     + aa0*finemat(ixf,nyf-1,izf,p112)
      end do

c---stencil right plane
      do ixf=1,nxf
      coarsmat(ixf,nyc,izf,p200)=aa1*(finemat(ixf,nyf-1,izf,p200)
     ++finemat(ixf,nyf,izf,p200))+
     + aa0*finemat(ixf,nyf-1,izf,p210)
      coarsmat(ixf,nyc,izf,p220)=0d0
      coarsmat(ixf,nyc,izf,p210)=finemat(ixf,nyf,izf,p210)+
     +aa1*(finemat(ixf,nyf-1,izf,p220)+finemat(ixf,nyf,izf,p200))+
     + aa0*finemat(ixf,nyf-1,izf,p210)
      coarsmat(ixf,nyc,izf,p201)=aa1*(finemat(ixf,nyf-1,izf,p201)
     ++finemat(ixf,nyf,izf,p201))+
     + aa0*finemat(ixf,nyf-1,izf,p211)
      coarsmat(ixf,nyc,izf,p221)=0d0
      coarsmat(ixf,nyc,izf,p211)=finemat(ixf,nyf,izf,p211)+
     +aa1*(finemat(ixf,nyf-1,izf,p221)+finemat(ixf,nyf,izf,p201))+
     + aa0*finemat(ixf,nyf-1,izf,p211)
      coarsmat(ixf,nyc,izf,p202)=aa1*(finemat(ixf,nyf-1,izf,p202)
     ++finemat(ixf,nyf,izf,p202))+
     + aa0*finemat(ixf,nyf-1,izf,p212)
      coarsmat(ixf,nyc,izf,p222)=0d0
      coarsmat(ixf,nyc,izf,p212)=finemat(ixf,nyf,izf,p212)+
     +aa1*(finemat(ixf,nyf-1,izf,p222)+finemat(ixf,nyf,izf,p202))+
     + aa0*finemat(ixf,nyf-1,izf,p212)
      end do

      end do

c---END back region boundary

c-- END Case vertex centered

      end if
      end
