      subroutine v1step  ( mat, rhs, nsmooth, sol, rbuf, ibuf)
c ======================================================================
c
c        programmer     Jos van Kan
c
c       version 1.1    10-04-1997
c         utilizes module mgpreset instead of common block cmgconst
c       version 1.0    27-03-1997
c
c
c   copyright (c) 1997
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   author.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c   Preconditioner for mat based on a multigrid V-cycle
c   with alternating Line Jacobi as smoother.
c   Result in sol.
c **********************************************************************
c
c                       KEYWORDS
c
c   multi_grid
c   preconditioner v_cycle
c **********************************************************************
c
c         MODULES USED
c
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ibuf (*)
      double precision mat (*), rhs (*), sol(*), rbuf (*)
      integer nsmooth

c     ibuf      i    Integer array of dimension 300 that contains the
c                    administration of this particular multi grid block.
c     mat       i    The nx x ny x 9 coefficient matrix of the
c                    discrete Laplacian
c     nsmooth   i    Number of times smoother is applied (all levels)
c     rbuf      i    Buffer array used to store all coarse size info
c                    and as scratch space. The actaul bufferspace
c                    needed can be calculated using subr autosize.
c     rhs       i    Right hand side  (any level)
c     sol       i    Solution vector (any level)
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     CALCRES   Calculates residal on any level (MG)
c     ERRMSG    Reports and counts errors (MG)
c     FREESCRS  Gives reserved scratch space back to buffer (MG)
c     GETSCRS   Reserves a block of scratch space from buffer (MG)
c     LINEJACX  Calculates 1 step line jacobi preconditioner x-dir (MG)
c     LINEJACY  Calculates 1 step line jacobi preconditioner y-dir (MG)
c     PROLONGA  Prolongation (all coarsening types) (MG)
c     RESTRICT  Restriction (all coarsening types) (MG)
c     TRIADVEC  Adds x + b y with x and y vectors, b scalar (MG)
c     ZEROIZE   Initialize vector to all zeros (MG)
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      print *, ' You need a FORTRAN-90 compiler to use multigrid'
      stop
      end
