      subroutine multmatv ( mat, vec, bresul, nxa, nya,
     +           sl1, sl2, sl3, sl4)
c ======================================================================
c
c        programmer     Jos van Kan
c
c       version 1.1    10-04-1997
c         utilizes module mgpreset instead of common block cmgconst
c       version 1.0    27-03-1997
c
c
c   copyright (c) 1997
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   author.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c   Calculates Ax on any level
c
c **********************************************************************
c
c                       KEYWORDS
c
c   multi_grid
c **********************************************************************
c
c         MODULES USED
c
      use mgpreset
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer sl1, sl2, sl3, sl4
      integer nxa, nya
      double precision
     +  mat (1-sl1:nxa+sl2, 1-sl3:nya+sl4, 9),
     +  vec (1-sl1:nxa+sl2, 1-sl3:nya+sl4),
     +  bresul (1-sl1:nxa+sl2, 1-sl3:nya+sl4)

c     bresul    i    The result vector
c     mat       i    The nx x ny x 9 coefficient matrix of the
c                    discrete Laplacian
c     nxa       i    Number of grid x points on this level
c     nya       i    Number of grid y points on this level
c     sl1       i    Slack variable, x lower bound
c     sl2       i    Slack variable, x upper bound
c     sl3       i    Slack variable, y lower bound
c     sl4       i    Slack variable, y upper bound
c     vec       i    Generic name for quantity with the dimension of
c                    sol or rhs
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ixa, iya

c     ixa       Counter to number of grid x points on this level
c     iya       Counter to number of grid y points on this level
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c   trivial
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- You could do this in a sloppy way: multiply everything.
c         this would be all right, since all matrix coefficients that do not
c         contribute are 0.
c         We prefer to do it the right way. Somewhat more lines though.

c         --- lower left corner

      bresul (1,1) =
     +          mat (1,1, p12) * vec (1,2) + mat (1,1, p22) * vec (2,2)
     +        + mat (1,1, p11) * vec (1,1) + mat (1,1, p21) * vec (2,1)

c     --- lower region boundary

      do ixa = 2, nxa - 1
        bresul (ixa, 1) =
     +   mat (ixa,1, p02) * vec (ixa-1, 2)   + mat (ixa,1, p12) * vec
     +   (ixa, 2) + mat (ixa,1, p22) * vec (ixa+1, 2)
     + + mat (ixa,1, p01) * vec (ixa-1, 1)   + mat (ixa,1, p11) * vec
     +   (ixa, 1) + mat (ixa,1, p21) * vec (ixa+1, 1)
      end do

c     --- lower right corner

      bresul (nxa, 1) =
     +   mat (nxa,1, p02) * vec (nxa-1, 2)   + mat (nxa,1, p12) * vec
     +   (nxa, 2)
     + + mat (nxa,1, p01) * vec (nxa-1, 1)   + mat (nxa,1, p11) * vec
     +   (nxa, 1)

c     --- inner region

      do iya = 2, nya -1

c     --- left boundary

        bresul (1, iya) =
     +   mat(1,iya, p12)*vec (1,iya+1) + mat(1,iya, p22)*vec (2,iya+1)
     + + mat(1,iya, p11)*vec (1,iya  ) + mat(1,iya, p21)*vec (2,iya  )
     + + mat(1,iya, p10)*vec (1,iya-1) + mat(1,iya, p20)*vec (2,iya-1)

       do ixa = 2, nxa - 1

c      --- generic case

        bresul (ixa, iya) =
     +  mat (ixa, iya, p02) * vec(ixa - 1,iya + 1) + mat (ixa, iya, p12)
     +* vec(ixa, iya + 1) + mat (ixa, iya, p22) * vec (ixa + 1, iya + 1)
     ++ mat (ixa, iya, p01) * vec(ixa - 1,iya    ) + mat (ixa, iya, p11)
     +* vec(ixa, iya    ) + mat (ixa, iya, p21) * vec (ixa + 1, iya    )
     ++ mat (ixa, iya, p00) * vec(ixa - 1,iya - 1) + mat (ixa, iya, p10)
     +* vec(ixa, iya - 1) + mat (ixa, iya, p20) * vec (ixa + 1, iya - 1)
      end do

c     --- right boundary

        bresul (nxa, iya) =
     ++ mat (nxa, iya, p02) * vec(nxa - 1,iya + 1) + mat (nxa, iya, p12)
     +* vec(nxa, iya + 1)
     ++ mat (nxa, iya, p01) * vec(nxa - 1,iya    ) + mat (nxa, iya, p11)
     +* vec(nxa, iya    )
     ++ mat (nxa, iya, p00) * vec(nxa - 1,iya - 1) + mat (nxa, iya, p10)
     +* vec(nxa, iya - 1)

      end do

c     --- end inner region

c         --- upper left corner

        bresul (1, nya) =
     ++ mat (1, nya, p11) * vec(1, nya    ) + mat (1, nya, p21) *
     +  vec (2, nya) + mat (1, nya, p10) * vec(1, nya - 1)
     ++ mat (1, nya, p20) * vec (2, nya - 1)

c       --- upper region boundary

      do ixa = 2, nxa - 1
        bresul (ixa, nya) =
     ++ mat (ixa, nya, p01) * vec(ixa - 1,nya    ) + mat (ixa, nya, p11)
     +* vec(ixa, nya    ) + mat (ixa, nya, p21) * vec (ixa + 1, nya    )
     ++ mat (ixa, nya, p00) * vec(ixa - 1,nya - 1) + mat (ixa, nya, p10)
     +* vec(ixa, nya - 1) + mat (ixa, nya, p20) * vec (ixa + 1, nya - 1)
      end do

c     --- upper right corner

        bresul (nxa, nya) =
     ++ mat (nxa, nya, p01) * vec(nxa - 1,nya    ) + mat (nxa, nya, p11)
     +* vec(nxa, nya    )
     ++ mat (nxa, nya, p00) * vec(nxa - 1,nya - 1) + mat (nxa, nya, p10)
     +* vec(nxa, nya - 1)
      end
