      subroutine prolonx ( finevec, coarsvec, nxf, nyf,
     +           sl1, sl2, sl3, sl4)
c ======================================================================
c
c        programmer     Jos van Kan
c
c       version 1.2    15-05-97
c         new cell centered R and P
c         nxc = nxf/2 + 1 instead of (nxf+1)/2
c       version 1.1    10-04-1997
c         utilizes module mgpreset instead of common block cmgconst
c       version 1.0    27-03-1997
c
c
c   copyright (c) 1997
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   author.
c
c
c **********************************************************************
c
c         MODULES USED
c
      use mgpreset
c
c **********************************************************************
c
c                       DESCRIPTION
c
c   Prolongation by standard methods in x direction
c   Centering method is decided by parity of nfx
c **********************************************************************
c
c                       KEYWORDS
c
c   multi_grid
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer nxf, nyf
      integer sl1, sl2, sl3, sl4
      double precision
     + finevec (1-sl1:nxf+sl2, 1-sl3:nyf+sl4),
     + coarsvec (nxf/2+1, nyf)

c     coarsvec  i    Vector on coarse level
c     finevec   o    Vector on fine level
c     nxf       i    Number of grid x points on fine grid
c     nyf       i    Number of grid y points on fine grid
c     sl1       i    Slack variable, x lower bound
c     sl2       i    Slack variable, x upper bound
c     sl3       i    Slack variable, y lower bound
c     sl4       i    Slack variable, y upper bound
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ixf, iyf, ixc, nxc
      integer centring
      double precision aa0, aa1

c     aa0       Interpolation weight in restriction/prolongation
c     aa1       Interpolation weight in restriction/prolongation
c     centring  0 = cellcentering, 1 = vertex centering
c     ixc       Counter to number of grid x points on coarse grid
c     ixf       Counter to number of grid x points on fine grid
c     iyf       Counter to number of grid y points on fine grid
c     nxc       Number of grid x points on coarse grid
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      centring = mod (nxf, 2)
      nxc = nxf/2 + 1
      aa0 = 0.25d0
      aa1 = 0.75d0
      if (centring.eq.CELLC) then

c     --- cell centered

        do iyf = 1, nyf
          do ixc = 1, nxc-1
           ixf = 2 * ixc - 1
           finevec (ixf, iyf) = aa1 * coarsvec (ixc, iyf) +
     +                           aa0 * coarsvec (ixc + 1, iyf)
           finevec (ixf+1, iyf) = aa0 * coarsvec (ixc, iyf) +
     +                          aa1 * coarsvec (ixc + 1, iyf)
          end do
        end do

c       --- end cell centered

      else

c     --- vertex centered

        aa0 = 0.5d0
        aa1 = 0.5d0
        do iyf = 1, nyf
          finevec (1, iyf) = coarsvec (1, iyf)
          do ixc = 2, nxc
            ixf = 2*ixc - 1
            finevec (ixf - 1, iyf) = aa0 * (coarsvec (ixc - 1, iyf)
     +                + coarsvec (ixc, iyf))
            finevec (ixf, iyf) = coarsvec (ixc, iyf)
          end do
        end do

c       --- end vertex centered

      end if
      end
