      subroutine isout ( time, isiinput, isrinput, iscoor, issolut,
     +                   isitopol, isgeom, iseddy, indorien )
c ======================================================================
c
c        programmer     Erik Brakkee
c        version 3.17   date 19-05-1999 Kees K: write coordinates
c        version 3.16   date 06-02-1999 Guus: split subroutine
c        version 3.15   date 02-02-1999 Kees K: axisymmetric
c        version 3.14   date 20-01-1999 Guus: use virtual unknowns
c        version 3.13   date 27-07-1998 Duncan: new call to isout07
c        version 3.12   date 29-06-1998 Charles: adaption of the stream-
c                                                function for colocated
c        version 3.11   date 09-06-1998 Charles: nvpoints changed to npointsv
c        version 3.10   date 11-03-1998 Guus: correction mscal incompressible
c        version 3.9    date 05-03-1998 Duncan: call to isout07 changed
c        version 3.8    date 12-11-1997 Hester: call to isout07 changed
c        version 3.7    date 29-09-1997 Charles: adaptation for collocated grid
c        version 3.6    date 22-09-1997 Fred T. adaptation for LES
c        version 3.5    date 19-07-1997 Guus: change ifirst
c        version 3.4    date 11-06-1997 Guus: layout
c        version 3.3    date 02-04-1997 Guus: new call to isout01
c
c   copyright (c) 1994-1999 "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Write the before last time level in the subdomain solution arrays
c     to the output file.
c **********************************************************************
c
c                       KEYWORDS
c
c   output
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iscoor(*), isiinput(*), isrinput(*), isitopol,
     +        issolut(*), isgeom(*), iseddy(*), indorien(*)
      double precision time

c     indorien  i    In this array it is stored if the block orientation is
c                    positive (indorien(iblock)=1) or
c                    negative(indorien(iblock)=-1)
c     iscoor    i    Array of sequence numbers (nblocks) of array coor
c     iseddy    i    Array of sequence numbers (nblocks) of array eddy
c     isgeom    i    Array of sequence numbers (nblocks) of array geom
c     isiinput  i    Array of sequence numbers (nblocks) of array iinput
c     isitopol  i    Sequence number of array itopol
c     isrinput  i    Array of sequence numbers (nblocks) of array rinput
c     issolut   i    Array of sequence numbers (nblocks) of array solut
c     time      i    Actual time of time step
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer maxblocki
      parameter ( maxblocki = 100 )
      integer iblock, ipiinput, ipitopol, iseq1,
     +        isorder, isoutput,
     +        isstream, isvel(maxblocki), lenoutput,
     +        lenstream, maxnsubf, maxpoints,
     +        nblocks, ndegfd, ndim, nfaces, nvec,
     +        mturb, ipitur, ifirst, mcom, ipiincom, ipiingls
       logical freesurf

c     freesurf  If true information about the free surface has been read
c     iblock    Block sequence number
c     ifirst    reserved
c     ipiincom  Starting address of array IINCOM in array IINPUT
c     ipiingls  Starting address of array IGLOBSTR in array IINPUT
c     ipiinput  Starting address of array iinput in ibuffr
c     ipitopol  Starting address of array itopol in ibuffr
c     ipitur    Starting address of array iintur in array iinput
c     iseq1     Sequence number of output quantity
c     isorder   Sequence number of array order
c     isoutput  Pointer to a local output array containing quantities in
c               the vertices
c     isstream  Pointer to the stream array containing for each subface
c               the value of the stream function at the beginning and
c               ending of that subface
c     isvel     Pointer to a local array for storing Cartesian velocity
c               components at the vertices
c     lenoutput Upper bound for the length of a local output array containing
c               quantities in the vertices
c     lenstream Length of the stream array containing for each subface
c               the value of the stream function at the beginning and
c               ending of that subface
c     maxblocki Maximum number of blocks
c     maxnsubf  Maximum number of subfaces encountered on all subfaces
c     maxpoints Maximum number of points in a block
c     mcom      If zero the flow is incompressible else (1) compressible
c     mturb     Indicates type of turbulence modelling
c     nblocks   Number of blocks
c     ndegfd    Number of degrees of freedom
c     ndim      Dimension of space (2 or 3)
c     nfaces    Number of faces
c     nvec      Number of vectors to be written to the output file
c **********************************************************************
c
c                       I/O
c
c      This solution and some related quantities are written to the file
c      isnascomp.out
c      Depending on the parameters in the file isnas.env this file is
c      formatted or unformatted
c      The records in the file isnascomp.out are filled in the following way:
c      -  ndim, nblocks
c      For each array to be written:
c         -  Name of vector
c         -  time, iseq1, itype1    (itype1=1:scalar, 2:vector)
c         For each block:
c            - iblock, nx, ny, nz
c            For all components
c               - values of vector in the sequence (1:nx+1,1:ny+1,1:nz+1)
c     The blocks are written in the natural sequence, except for the stream
c     function, which uses a special block ordering
c     The sequence of the vectors to be written is:
c        velocity
c        pressure
c        stream function
c        transport quantities
c        turbulent quantities
c        extra quantities
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isingetp

c     ISACTIVE  Activate old permanent array
c     ISDCREAT  Create permanent space in array IBUFFR (Double precision array)
c     ISDCRPRI  Decrease priorities of all arrays in ibuffr
c     ISDELARR  Delete a permanent array
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c     ISINGETP  get the pointer corresponding to an integer array
c     ISINIFIL  Open files
c     ISORD     Compute an ordering of blocks needed for computation
c               of the streamfunction
c     ISOUTCOMP Write special information for compressible flows
c     ISOUTCOR  Write the coordinates to the output file
c     ISOUTPRES Write the pressure to the output file
c     ISOUTRANS Write the transport and turbulent quantitities to the output
c               file
c     ISOUTSTRM Write the stream function to the output file
c     ISOUTTURB Write special information for turbulent flows
c     ISOUTVEL  Write the velocity to the output file
c     ISSTOP    Stop ISNAS
c **********************************************************************
c
c                       ERROR MESSAGES
c
c    1802   Maximum number of blocks too small
c **********************************************************************
c
c                       PSEUDO CODE
c
c     Allocate local arrays
c     DO iblock = 1, nblocks
c        determine Cartesian velocity components in the cell-centers (isout02)
c        interpolate these velocity components to the vertices (isout01)
c        Adapt results to given boundary conditions (...in later versions)
c        interpolate the pressure unknowns to the vertices (isout01)
c        DO itrnsp = ndim+2, ndegfd
c           interpolate quantity itrnsp to the vertices (isout01)
c        ENDDO
c        IF TURBULENCE
c          DO i = 1, 3*ndim
c            fill work with specified quantities in centroids (isout06)
c            interpolate these quantities to the vertices (isout01)
c          ENDDO
c        END IF
c     ENDDO
c     Delete local arrays
c
c
c     Important remark:
c
c     To interpolate from cell centers to vertices use is made of virtual cells
c     except for the turbulent quantities
c     The reason is that array eddy is not extrpolated for turbulent quantities
c     As a consequence turbulent quantities on multiblock and periodical
c     boundaries are incorrect
c     This must be improved in the future!!!!
c **********************************************************************
c
c                       DATA STATEMENTS
c
      save isorder, ifirst
      data isorder /0/, ifirst/0/
c ======================================================================
c
      call iseropen('isout')

c     --- compute an array containing an ordering of blocks needed for
c         computation of the streamfunction

      if ( ifirst.eq.0 ) then
         call isinifil ( 74 )
         call isord ( isitopol, isorder )
      end if

c     --- determine nblocks

      call isactive(isitopol)
      ipitopol = isingetp(isitopol)
      nblocks  = ibuffr(ipitopol)
      nfaces   = ibuffr(ipitopol + 1)
      maxnsubf = ibuffr(Ipitopol + 2)
      if ( nblocks.gt.maxblocki ) then
         call iserrint ( nblocks, 1 )
         call iserrint ( maxblocki, 2 )
         call iserrsub ( 1802, 2, 0, 0 )
         call isstop
      end if

c     --- compute some general parameters

      call isactive(isiinput(1))
      ipiinput   = isingetp(isiinput(1))
      call isactive(isrinput(1))
      ipitur     = ibuffr(ipiinput +  8 - 1)
      ipiincom   = ibuffr(ipiinput +  9 - 1)
      ipiingls   = ibuffr(ipiinput + 14 - 1)
      ndegfd     = ibuffr(ipiinput + 55 - 1)
      ndim       = ibuffr(ipiinput + 57 - 1)
      maxpoints  = ibuffr(ipiinput + 71 - 1)
      mturb      = ibuffr(ipiinput + ipitur -1 + 2 - 1)
      mcom       = ibuffr(ipiinput + ipiincom - 1 )
      freesurf   = ibuffr( ipiinput -1 + ipiingls-1 +2).gt.0
      if ( ndim.eq.2 ) then
         nvec       = ndegfd - ndim + 2
      else
         nvec       = ndegfd - ndim + 1
      end if
      if ( mturb.ne.0 ) nvec = nvec + 3*(ndim-1) + 3
      if ( mcom .ne.0 ) nvec = nvec + 2

c     --- compute dimensions of the local arrays

      lenoutput = maxpoints*ndim
      lenstream = nblocks*nfaces*maxnsubf*2

      if ( mturb.ne.0 ) then
         lenoutput = 3*ndim*maxpoints
      end if

      if ( mcom.ne.0 ) then
        lenoutput = lenoutput + 2*ndim*maxpoints
      end if

c     --- create arrays

      isoutput = 0
      isstream = 0

      call isdcreat(isoutput, lenoutput, 'output' )
      call isdcreat(isstream, lenstream, 'stream' )

c     --- write output for the different blocks
c         Note: a special ordering of blocks is used for computation of
c         the streamfunction

      if ( ifirst.eq.0 ) then

c     --- ifirst = 0, write ndim and nblocks

         if ( iref74.gt.0 ) then
            write(iref74,100 ) ndim, nblocks,'# ndim, nblocks'
100         format(i3,i5,12x,a)
         else
            write(-iref74) ndim, nblocks
         end if
      end if

      if ( freesurf ) then

c     --- Write coordinates

         call isoutcor ( time, isiinput, iscoor, isitopol,
     +                    isoutput, iseq1 )

      end if

c     --- Write velocities

      call isoutvel ( time, isiinput, isrinput, iscoor, issolut,
     +                isitopol, indorien, isoutput, isvel, iseq1 )

c     --- Write pressure

      call isoutpres ( time, isiinput, isrinput, iscoor, issolut,
     +                 isitopol, indorien, isoutput, iseq1 )

      if ( ndim.eq.2 ) then

c     --- Write stream function if ndim = 2

         call isoutstrm ( time, isiinput, isrinput, iscoor, issolut,
     +                    isitopol, indorien, isoutput, isvel, isorder,
     +                    isstream, iseq1 )

      end if

c     --- Write transport quantities including turbulent primary quantities

      call isoutrans ( time, isiinput, isrinput, iscoor, issolut,
     +                 isitopol, indorien, isoutput, iseq1 )

      if ( mturb.gt.0 ) then

c     --- mturb>0, compute and print specific quantities with respect to
c         turbulence

         call isoutturb ( time, isiinput, isrinput, iscoor, issolut,
     +                    isitopol, indorien, isoutput, iseq1,
     +                    iseddy, isgeom )

      end if

      if ( mcom.ne.0 ) then

c     --- mcom>0, compute and print specific quantities with respect to
c         compressible flow, both cavitating and fully wetted

         call isoutcomp ( time, isiinput, isrinput, iscoor, issolut,
     +                    isitopol, indorien, isoutput, iseq1, isvel,
     +                    iseddy )

      end if

c     --- delete arrays

      call isdelarr(isstream)
      do iblock = nblocks, 1, -1
         call isdelarr(isvel(iblock))
      end do
      call isdelarr(isoutput)

      call iserclos('isout')
      call isdcrpri
      ifirst = 1

      end
