      subroutine isoutcomp ( time, isiinput, isrinput, iscoor, issolut,
     +                       isitopol, indorien, isoutput, iseq1, isvel,
     +                       iseddy )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.1    date 03-03-1999  Duncan: new call to isout07
c        version 1.0    date 27-01-1999
c
c
c   copyright (c) 1999-1999 "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Write the before last time level in the subdomain solution arrays
c     to the output file. This subroutine writes special output for the
c     compresible case
c **********************************************************************
c
c                       KEYWORDS
c
c   output
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iscoor(*), isiinput(*), isrinput(*), isitopol,
     +        issolut(*), isoutput, indorien(*), iseq1, isvel(*),
     +        iseddy(*)
      double precision time

c     indorien       i    In this array it is stored if the block orientation is
c                         positive (indorien(iblock)=1) or
c                         negative(indorien(iblock)=-1)
c     iscoor         i    Array of sequence numbers (nblocks) of array coor
c     iseddy         i    Array of sequence numbers (nblocks) of array eddy
c     iseq1          o    Sequence number of output quantity
c     isiinput       i    Array of sequence numbers (nblocks) of array iinput
c     isitopol       i    Sequence number of array itopol
c     isoutput       i    Pointer to a local output array containing quantities
c                         in the vertices
c     isrinput       i    Array of sequence numbers (nblocks) of array rinput
c     issolut        i    Array of sequence numbers (nblocks) of array solut
c     isvel          i    Pointer to a local array for storing Cartesian
c                         velocity components at the vertices
c     time           i    Actual time of time step
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer j, iblock, ipcoor, ipiinput, ipitopol, ipoutput, ipeddy,
     +        itype1, ipsolut, nblocks, ndegfd, ndim, ni, nj, nk,
     +        ntimlv, nvirtual, npointsv, ipwork1, ipnsubf, ipiinbc,
     +        itypepost, istartup, iprincom, mcom, ipiincom,
     +        iprinput, ipsubf, ipvel, ipwork, iswork, maxnsubf, mcav,
     +        mscal, nfaces, ipiincav, iprincav, meqstate
      character * 30 name

c     iblock         Block sequence number
c     ipcoor         Starting address of array coor in buffer
c     ipeddy         Starting address of array eddy in buffer
c     ipiinbc        Starting address of array iinbc in array iinput
c     ipiincav       Starting address of array IINCAV in array IINPUT
c     ipiincom       Starting address of array IINCOM in array IINPUT
c     ipiinput       Starting address of array iinput in ibuffr
c     ipitopol       Starting address of array itopol in ibuffr
c     ipnsubf        Starting address of the nsubfaces array
c     ipoutput       Starting address of a local output array containing
c                    quantities in the vertices
c     iprincav       Starting address of array RINCAV in array RINPUT
c     iprincom       Starting address of array RINCOM in array RINPUT
c     iprinput       Starting address of array rinput in buffer
c     ipsolut        Starting address of array solut in buffer
c     ipsubf         Starting address of the subfaces array
c     ipvel          Starting address of a local array for storing
c                    Cartesian velocity components at the vertices
c     ipwork         Starting address of array work in buffer
c     ipwork1        Starting address of array work1 in buffer
c     istartup       If 1 the cavity calculation is in the startup
c                    procedure, if 0 not
c     iswork         Sequence number of array work
c     itype1         Indicates the type of output vector
c                    1: scalar,  2: vector,  3: tensor
c     itypepost      Indicates the type of postprocessing required
c                    0: standard Newton
c                    1: linear in computational domain
c     j              General loop variable
c     maxnsubf       Maximum number of subfaces encountered on all subfaces
c     mcav           If 1 a cavity model is used, if 0 not
c     mcom           If zero the flow is incompressible else (1)
c                    compressible
c     meqstate       Defines the type of equation of state
c                    See the programmers Guide array IINCOM for the
c                    possible values
c     mscal          Indicates use of non-dimensionalisation for
c                    compr. flow:
c                    0:   Equations are non-dimensionalised
c                    1:   Equations are in dimensional formulation
c     name           Name of file
c     nblocks        Number of blocks
c     ndegfd         Number of degrees of freedom
c     ndim           Dimension of space (2 or 3)
c     nfaces         Number of faces
c     ni             Number of non-virtual cells in the i-direction
c                    (one block)
c     nj             Number of non-virtual cells in the j-direction
c                    (one block)
c     nk             Number of non-virtual cells in the k-direction
c                    (one block)
c     npointsv       Number of points in the mesh with virtual points
c     ntimlv         Number of time levels
c     nvirtual       Number of rows of virtual cells
c **********************************************************************
c
c                       I/O
c
c      This solution and some related quantities are written to the file
c      isnascomp.out
c      Depending on the parameters in the file isnas.env this file is
c      formatted or unformatted
c      The records in the file isnascomp.out are filled in the following way:
c      -  ndim, nblocks
c      For each array to be written:
c         -  Name of vector
c         -  time, iseq1, itype1    (itype1=1:scalar, 2:vector)
c         For each block:
c            - iblock, nx, ny, nz
c            For all components
c               - values of vector in the sequence (1:nx+1,1:ny+1,1:nz+1)
c     The blocks are written in the natural sequence, except for the stream
c     function, which uses a special block ordering
c     The sequence of the vectors to be written is:
c        velocity
c        pressure
c        stream function
c        transport quantities
c        turbulent quantities
c        extra quantities
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isdpgetp, isingetp

c     ISACTIVE       Activate old permanent array
c     ISCPPER2       Compute mean values on common sides of periodical boundarie
c     ISDCREAT       Create permanent space in array IBUFFR (Double precision ar
c     ISDELARR       Delete a permanent array
c     ISDIMSC        Dimensionalise the unknowns for postprocessing
c     ISDPGETP       get the pointer corresponding to a double precision array
c     ISERCLOS       Resets old name of previous subroutine of higher level
c     ISEROPEN       Produces concatenated name of local subroutine
c     ISINGETP       get the pointer corresponding to an integer array
c     ISOUT01        interpolate quantities from centroids to vertices
c     ISOUT07        Compute Mach number for output purpose for compressible
c                    computations
c     ISPOSOR1       Change directions in solution array
c     ISPRINS2       Print one block of scalars in 2D
c     ISTMSPAC       Create space for temporary array
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     Allocate local arrays
c     DO iblock = 1, nblocks
c        interpolate the transport unknowns to the vertices (isout01)
c     ENDDO
c     Delete local arrays
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call iseropen('isoutcomp')

c     --- determine nblocks

      call isactive(isitopol)
      ipitopol = isingetp(isitopol)
      nblocks  = ibuffr(ipitopol)
      nfaces   = ibuffr(ipitopol + 1)
      maxnsubf = ibuffr(ipitopol + 2)

c     --- compute some general parameters

      call isactive(isiinput(1))
      ipiinput   = isingetp(isiinput(1))
      call isactive(isrinput(1))
      iprinput   = isdpgetp(isrinput(1))
      ipiinbc    = ibuffr(ipiinput +  2 - 1)
      ipiincom   = ibuffr(ipiinput +  9 - 1)
      iprincom   = ibuffr(ipiinput + 34 - 1)
      iprincav   = ibuffr(ipiinput + 35 - 1)
      ndegfd     = ibuffr(ipiinput + 55 - 1)
      ndim       = ibuffr(ipiinput + 57 - 1)
      ntimlv     = ibuffr(ipiinput + 64 - 1)
      itypepost  = ibuffr(ipiinput + 67 - 1)
      mcom       = ibuffr(ipiinput + ipiincom - 1 )
      mcav       = ibuffr(ipiinput + 77 - 1 )
      mscal      = ibuffr(ipiinput + ipiincom + 4 - 1 )
      meqstate   = ibuffr(ipiinput-1 + ipiincom-1+9)
      if( mcav.eq.1) then
         ipiincav = ibuffr(ipiinput -1 + 11)
         istartup = ibuffr(ipiinput -1 + ipiincav -1 + 2)
      else
         istartup = 0
      end if

      nvirtual   = ibuffr(ipiinput + 54 - 1)
      iswork   = 0

c     --- compute and print specific quantities with respect to
c         compressible flow, both cavitating and fully wetted

      itype1 = 1

      do j = 1, 2

         iseq1 = iseq1+1
         name = ' '
         if ( j.eq.1 ) then
            name = 'density'
         else if ( meqstate.eq.0 ) then
            name = 'mach'
         else
            name = 'temperature'
         end if
         if ( iref74.gt.0 ) then
            write(iref74,110 ) name, time, iseq1, itype1
110         format(a/f12.5,2i3,12x,a)
         else
            write(-iref74) name
            write(-iref74) time, iseq1, itype1
         end if

         do iblock = 1, nblocks

c        --- determine subdomain parameters

            call isactive(isiinput(iblock))
            ipiinput = isingetp(isiinput(iblock))
            ni       = ibuffr(ipiinput + 51 - 1)
            nj       = ibuffr(ipiinput + 52 - 1)
            nk       = ibuffr(ipiinput + 53 - 1)
            if ( ndim.eq.2 ) then
               npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)
            else
               npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)*
     +                    (nk+1+2*nvirtual)
            end if

c           --- Write first record of block

            if ( iref74.gt.0 ) then
               write(iref74,120 ) iblock, ni, nj, nk
120            format(4i8,12x,a)
            else
               write(-iref74) iblock, ni, nj, nk
            end if

c           --- activate arrays

            call isactive(iscoor(iblock))
            call isactive(isitopol)
            call isactive(isoutput)
            if ( j.eq.1 ) call isactive(iseddy(iblock))
            if ( j.eq.2 ) then
               if(mcav.eq.1) call isactive(iseddy(iblock))
               call isactive(issolut(iblock))
               call isactive(isvel(iblock))
               call isdcreat ( iswork, npointsv, 'work' )
               ipwork   = isdpgetp(iswork)
            end if
            if ( indorien(iblock).eq.-1 )
     +         call istmspac((ni+1)*(nj+1)*intlen,ipwork1)
            ipcoor   = isdpgetp(iscoor(iblock))
            ipoutput = isdpgetp(isoutput)
            iprinput = isdpgetp(isrinput(1))
            ipitopol = isingetp(isitopol)
            ipnsubf  = ibuffr(ipitopol+5-1) + ipitopol - 1
            ipsubf   = ibuffr(ipitopol+6-1) + ipitopol - 1
            if ( j.eq.1 ) then

c           --- Use second array in eddy

               ipeddy = isdpgetp(iseddy(iblock))
               ipwork = ipeddy+npointsv

            else if ( meqstate.eq.1 ) then

c           --- Use seventh array in eddy

               ipeddy = isdpgetp(iseddy(iblock))
               ipwork = ipeddy+npointsv*6

            else

c           --- Use ndegfd-th array in new time of solut
c               for fully wetted flow Mach is based on density
c               for cavitating flow Mach is based on the derivative
c               of the equation of state rho(p)

               ipvel = isdpgetp(isvel(iblock))
               iprincom = ibuffr(ipiinput + 34 - 1)
               if ( mcav.eq.1 ) then
                  ipsolut = isdpgetp(issolut(iblock))+
     +                        (ntimlv-2)*npointsv*ndegfd
                  call isout07 ( buffer(ipwork),
     +                           buffer(ipsolut+npointsv*ndim),
     +                           buffer(ipvel), mcom, mcav, ni, nj,
     +                           buffer(iprinput+iprincom-1),
     +                           nvirtual, mscal, istartup,
     +                           buffer(iprinput+iprincav-1))

               else
                  ipsolut = isdpgetp(issolut(iblock)) +
     +                        npointsv*ndegfd+npointsv*(ndegfd-1)
                  call isout07 ( buffer(ipwork), buffer(ipsolut),
     +                           buffer(ipvel), mcom, mcav, ni, nj,
     +                           buffer(iprinput+iprincom-1),
     +                           nvirtual, mscal, istartup,
     +                           buffer(iprinput+iprincav-1))

               end if
            end if
            if ( ndim.eq.2 ) then

c           --- interpolate quantities to the vertices

               call isout01 ( iblock, ni, nj, nvirtual,
     +                        nblocks, nfaces, maxnsubf,
     +                        ibuffr(ipnsubf), ibuffr(ipsubf),
     +                        buffer(ipcoor), buffer(ipwork),
     +                        buffer(ipoutput), itypepost )
               if (j.eq.1.and.mscal.eq.0) then
                   call isdimsc ( buffer(ipoutput), buffer(ipoutput),
     +                            buffer(iprinput+iprincom-1), ni,
     +                            nj, 2 )
               end if

c              --- Compute mean value for periodical boundaries

               call iscpper2 ( ibuffr(ipiinput),
     +                         ibuffr(ipiinput-1+ipiinbc),
     +                         buffer(ipoutput),  ni, nj, iblock )

               if ( indorien(iblock).eq.-1 ) then
                  call isposor1 ( buffer(ipoutput),
     +                            buffer(ipwork1), ni, nj )
                  call isprins2 ( nj, ni, buffer(ipoutput) )
               else
                  call isprins2 ( ni, nj, buffer(ipoutput) )
               end if

            end if

         end do

         if ( j.eq.2 ) call isdelarr(iswork)

      end do

      call iserclos('isoutcomp')

      end
