      subroutine isoutv13 ( ni, nj, nk, nvirtual, coor, scalar, output )
c ======================================================================
c
c     programmer   Kees Kassels
c     version      1.0  20-01-1999
c
c   copyright (c) 1999-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c   This subroutine performs the interpolation of a scalar quantity
c   defined in the centroids of the 3D cells, to the vertices.
c   Single block only.
c   For the values on the boundary the values in the vertices are
c   extrapolated from the inner area, the virtual values are not
c   used.
c   Bilinear interpolation  is used.
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nk, nvirtual
      double precision coor(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual,3)
      double precision output(1:ni+1,1:nj+1,1:nk+1)
      double precision scalar(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,
     +                        1-nvirtual:nk+1+nvirtual)

c     coor      i    reserved
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     output    o    Cartesian components of the flow field in the vertices
c     scalar    i    Quantities in the centroids
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j, k
      double precision uci, vci, wci

c     i         General loop variable
c     j         General loop variable
c     k         General loop variable
c     uci       Local coordinate for the interpolation in the i-direction
c     vci       Local coordinate for the interpolation in the j-direction
c     wci       Local coordinate for the interpolation in the k-direction
c **********************************************************************
c
c                       I/O
c
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISNT3D    Compute local coordinates for the interpolation
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- inner area

      do k = 1, nk+1
         do j = 1, nj+1
            do i = 1, ni+1
               call isnt3d ( ni, nj, nk, nvirtual, coor, i, j, k,
     +                       i, j, k, uci, vci, wci )
               output(i,j,k) = (1d0-uci)*(1d0-vci)*(1d0-wci) *
     +                          scalar(i-1,j-1,k-1)      +
     +                          uci*(1d0-vci)*(1d0-wci)    *
     +                          scalar(i  ,j-1,k-1)      +
     +                          (1d0-uci)*vci*(1d0-wci)    *
     +                          scalar(i-1,j  ,k-1)      +
     +                          uci*vci*(1d0-wci)        *
     +                          scalar(i  ,j  ,k-1)      +
     +                          (1d0-uci)*(1d0-vci)*wci    *
     +                          scalar(i-1,j-1,k  )      +
     +                          uci*(1d0-vci)*wci        *
     +                          scalar(i  ,j-1,k  )      +
     +                          (1d0-uci)*vci*wci        *
     +                          scalar(i-1,j  ,k  )      +
     +                          uci*vci*wci            *
     +                          scalar(i  ,j  ,k  )

            end do
         end do
      end do

      end
