      subroutine ispri101 ( iinput, rinput, text )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.4    date 10-09-1996 Marcel: extension turbulence
c                                               parameters
c        version 1.3    date 23-08-1996 Marcel: extension turbulence
c                                               parameter
c        version 1.2    date 23-05-1996 Marcel: extension turbulence
c                                               parameter
c        version 1.1    date 22-05-1996 Marcel: extension k-omega and
c                                               Kato-Launder coefficients
c        version 1.0    date 16-08-1994
c
c
c   copyright (c) 1994-1996  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Print the contents of the arrays iinput and rinput for one block
c     Actual printing subroutine
c **********************************************************************
c
c                       KEYWORDS
c
c    print
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iinput(*)
      double precision rinput(*)
      character *(*) text

c     iinput    i    reserved
c     rinput    i    reserved
c     text      i    String in which a text is stored
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, iboun, ipiinbc, ipiinblk, ipiincnd, ipiincof, ipiindsc,
     +        ipiinsol, ipiintim, iprinbc, iprinblk, iprincnd, iprincof,
     +        iprinsol, iprintim, iprintur, iseg, istrtinp, j, jstart,
     +        ncoefs, ndegfd, ndim, ntrans, nseg, iprindsc,  ipiintur

c     i         General loop variable
c     iboun     Counting variable: sequence number of boundary
c     ipiinbc   Starting address of array iinbc in array iinput
c     ipiinblk  Starting address of array IINBLK in array IINPUT
c     ipiincnd  Starting address of array iincnd in array iinput
c     ipiincof  Starting address of array IINCOF in array IINPUT
c     ipiindsc  Starting address of array IINDSC in array IINPUT
c     ipiinsol  Starting address of array IINSOL in array IINPUT
c     ipiintim  Starting address of array IINTIM in array IINPUT
c     ipiintur  Starting address of array IINTUR in array IINPUT
c     iprinbc   Starting address of array rinbc in array iinput
c     iprinblk  Starting address of array RINBLK in array RINPUT
c     iprincnd  Starting address of array rincnd in array rinput
c     iprincof  Starting address of array RINCOF in array RINPUT
c     iprindsc  Starting address of array RINDSC in array RINPUT
c     iprinsol  Starting address of array RINSOL in array RINPUT
c     iprintim  Starting address of array RINTIM in array RINPUT
c     iprintur  Starting address of array rintur in array RINPUT
c     iseg      Counting variable: sequence number of segment
c     istrtinp  Starting address in array iinput
c     j         General loop variable
c     jstart    Starting address in array iinput
c     ncoefs    Number of coefficients
c     ndegfd    Number of degrees of freedom
c     ndim      Dimension of space (2 or 3)
c     nseg      Number of segments the boundary is divided in
c     ntrans    Number of transport equations plus number of turbulence
c               equations
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c    Print heading
c    Print first 75 postions of array iinput
c    Print the coefficient arrays IINCOF and RINCOF
c    Print arrays iinbc and rinbc
c    Print the arrays with respect to the initial conditions RINCND and IINCND
c    Print the arrays with respect to the time-integration RINTIM and IINTIM
c    Print the arrays with respect to the linear solver RINSOL and IINSOL
c    Print the arrays with respect to the discretization RINDSC and IINDSC
c    Print the arrays with respect to the multiblock process RINBLK and IINBLK
c    Print the array with turbulence coefficients rintur
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- Print heading

      write(irefwr,100) text
100   format(//1x,a)
      write(irefwr,110) iblockrf
110   format(//' Contents of array iinput for block number', i4 //)

c     --- Set constants

      ndegfd = iinput(55)
      ndim   = iinput(57)
      ntrans = iinput(63) + iinput(70)

c     --- Print first 75 postions of array iinput

      write(irefwr,120) (iinput(i),i=1,25)
120   format(/' Local starting addresses integer arrays:'/
     +       ' ipiincof =', i4/' ipiinbc  =', i4/' ipiincnd =', i4/
     +       ' ipiintim =', i4/' ipiinsol =', i4/' ipiindsc =', i4/
     +       ' ipiinblk =', i4/' Rest = '/(10i4))
      write(irefwr,130) (iinput(i),i=26,50)
130   format(/' Local starting addresses double precision arrays:'/
     +       ' iprincof =', i4/' iprinbc  =', i4/' iprincnd =', i4/
     +       ' iprintim =', i4/' iprinsol =', i4/' iprintur =', i4/
     +       ' iprinblk =', i4/' Rest = '/(10i4))
      if ( iblockrf.eq.1 ) write(irefwr,140) (iinput(i),i=51,75)
140   format(/' Parameters'/
     +       ' ni         =', i4/' nj         =',i4/' nk         =',i4/
     +       ' nvirtual   =', i4/' ndegfd     =',i4/' -          =',i4/
     +       ' ndim       =', i4/' vlen       =',i4/' igeo       =',i4/
     +       ' maxcon     =', i4/' maxb       =',i4/' -          =',i4/
     +       ' ntrnsp     =', i4/' ntimlv     =',i4/' -          =',i4/
     +       ' ncoefs     =', i4/' -          =',i4/' nmachines  =',i4/
     +       ' maxvpnts =', i4/' nturb      =',i4/
     +       ' maxpoints  =', i4/
     +       ' maxcells   =', i4/' nblocks    =',i4/
     +       ' npoint_global =', i4/
     +       ' model_turb =', i4 )

c     --- Print the coefficient arrays IINCOF and RINCOF

      if ( iblockrf.eq.1 ) then

         ipiincof = iinput(1)
         iprincof = iinput(26)
         ncoefs = iinput(66)
         do 200 i = 1, 1+ntrans
            write(irefwr,150) i,
     +         (iinput(ipiincof+(i-1)*ncoefs+j-1),j=1,ncoefs)
150         format(/' Array iincof for equation',i3/(10i6))
            write(irefwr,160)
     +         (rinput(iprincof+(i-1)*ncoefs+j-1),j=1,ncoefs)
160         format(/' Array rincof '/ss,(5(1pe12.3e2)))
200      continue

      end if

c     --- Print arrays iinbc and rinbc

      ipiinbc = iinput(2)
      iprinbc = iinput(27)
      istrtinp = ipiinbc-1
      jstart = iprinbc-1
      write(irefwr,210) (i,iinput(istrtinp+i),i=1,2*ndim)
210   format(/' array iinbc'//' Starting addresses'/(5(1x,i3,': ',i6)))
      istrtinp = istrtinp+2*ndim+1
      if ( ndim.eq.2 ) then

c     --- ndim = 2

         do iboun = 1, 4
            nseg = iinput(istrtinp)
            write(irefwr,220) nseg, iboun
220         format(' number of segments is ',i3,
     +             '  at boundary ', i1 )
            do iseg = 1, nseg
               write(irefwr,230) iseg,
     +            (iinput(istrtinp+i),i=1,2+3*(ntrans+1))
230            format(' segment ', i3,'  i1 =',i3,'  i2 =',i3/
     +                ' Array itype'/(1x,3i6))
               write(irefwr,240) (rinput(jstart+i),i=1,2*(ntrans+1))
240            format(' rinbc '/(1x,ss,5(1pe12.3e2)))
               istrtinp = istrtinp + 2+3*(ntrans+1)
               jstart = jstart + 2*(ntrans+1)
            end do
            istrtinp = istrtinp + 1
         end do
      else

c     --- ndim = 3

         do iboun = 1, 6
            nseg = iinput(istrtinp)
            write(irefwr,220) nseg, iboun
            do iseg = 1, nseg
               write(irefwr,250) iseg,
     +            (iinput(istrtinp+i),i=1,4+4*(ntrans+1))
250            format(' segment ', i3,'  i1 =',i3,
     +                 '  j1 =',i3,'  i2 =',i3,'  j2 =',i3/
     +                ' Array itype'/(1x,4i6))
               write(irefwr,240) (rinput(jstart+i),i=1,3*(ntrans+1))
               istrtinp = istrtinp + 4+4*(ntrans+1)
               jstart = jstart + 3*(ntrans+1)
            end do
            istrtinp = istrtinp + 1
         end do

      end if

c     --- Print the arrays with respect to the initial conditions
c         RINCND and IINCND

      if ( iblockrf.eq.1 ) then

         ipiincnd = iinput(3)
         iprincnd = iinput(28)
         write(irefwr,300)  (iinput(ipiincnd+j-1),j=1,ndegfd)
300      format(/' Array iincnd '/(10i6))
         write(irefwr,310) (rinput(iprincnd+j-1),j=1,ndegfd)
310      format(/' Array rincnd '/ss,(5(1pe12.3e2)))

      end if

c     --- Print the arrays with respect to the time-integration
c         RINTIM and IINTIM

      if ( iblockrf.eq.1 ) then

         ipiintim = iinput(4)
         iprintim = iinput(29)
         write(irefwr,320)  (iinput(ipiintim+j-1),j=1,10)
320      format(/' Array iintim '/
     +           ' method = ', i3/ ' -      = ', i3/ ' -      = ', i3/
     +           ' itime  = ', i3/ ' -      = ', i3/ ' -      = ', i3/
     +           ' -      = ', i3/ ' -      = ', i3/ ' -      = ', i3/
     +           ' istationary = ', i3 )
         write(irefwr,330) (rinput(iprintim+j-1),j=1,28)
330      format(/' Array rintim '/
     +           ' t        = ', 1pe12.3e2/ ' t0       = ', 1pe12.3e2/
     +           ' tend     = ', 1pe12.3e2/ ' dt       = ', 1pe12.3e2/
     +           ' theta    = ', 1pe12.3e2/ ' toutinit = ', 1pe12.3e2/
     +           ' toutend  = ', 1pe12.3e2/ ' toutstep = ', 1pe12.3e2/
     +           6(1pe12.3e2)/ 6(1pe12.3e2)/
     +           ' absaccuracy_u = ', 1pe12.3e2/
     +           ' relaccuracy_u = ', 1pe12.3e2/
     +           ' absaccuracy_p = ', 1pe12.3e2/
     +           ' relaccuracy_p = ', 1pe12.3e2/
     +           ' absaccuracy_trans = ', 1pe12.3e2/
     +           ' relaccuracy_trans = ', 1pe12.3e2/
     +           ' absaccuracy_turbulence = ', 1pe12.3e2/
     +           ' relaccuracy_turbulence = ', 1pe12.3e2 )

      end if

c     --- Print the arrays with respect to the linear solver
c         RINSOL and IINSOL

      if ( iblockrf.eq.1 ) then

         ipiinsol = iinput(5)
         iprinsol = iinput(30)
         do 400 i = 1, 1+ntrans
            write(irefwr,350) i,
     +         (iinput(ipiinsol+(i-1)*25+j-1),j=1,25)
350         format(/' Array iinsol for equation',i3/(10i6))
            write(irefwr,360)
     +         (rinput(iprinsol+(i-1)*25+j-1),j=1,25)
360         format(/' Array rinsol '/ss,(5(1pe12.3e2)))
400      continue

      end if

c     --- Print the arrays with respect to the discretization IINDSC and RINDSC

      if ( iblockrf.eq.1 ) then

         ipiindsc = iinput(6)
         iprindsc = iinput(33)
         do 450 i = 1, 1+ntrans
            write(irefwr,410) i,
     +         (iinput(ipiindsc+(i-1)*20+j-1),j=1,10)
410         format(/' Array iindsc for equation',i3/
     +              ' iterm               =', i3/,
     +              ' idiscret_convection =', i3/
     +              ' idiscret_viscosity  =', i3/
     +              ' idiscret_pressure   =', i3/
     +              ' iupwind             =',i3/
     +              ' imonoton_diffusion  =', i3/ 3i6 )
            write(irefwr,420)
     +         (iinput(iprindsc+(i-1)*10+j-1),j=1,10)
420         format(/' Array rindsc'/ss,(5(1pe12.3e2)))
450      continue

      end if

c     --- Print the arrays with respect to the multiblock process
c         RINBLK and IINBLK

      if ( iblockrf.eq.1 ) then

         ipiinblk = iinput(7)
         iprinblk = iinput(32)
         write(irefwr,460) (iinput(ipiinblk+(i-1)*10+j-1),j=1,10)
460      format(/' Array iinblk general information'/(1x,10i6))
         do 500 i = 1, 2+ntrans
            write(irefwr,470) i, (iinput(ipiinblk+i*10+j-1),j=1,10)
470         format(/' Array iinblk for equation',i3/(1x,10i6))
            write(irefwr,480) i, (rinput(iprinblk+i*10+j-11),j=1,10)
480         format(/' Array rinblk for equation',i3/ss,(5(1pe12.3e2)))
500      continue

      end if

c     --- Print the arrays with turbulence coefficients iintur and rintur

      if ( iblockrf.eq.1 ) then

         ipiintur = iinput(8)
         iprintur = iinput(31)
         write(irefwr,510) (iinput(ipiintur+j-1),j=1,4)
510      format(/' Array iintur'/
     +           ' nturb = ', i6/, ' mturb = ',i6/, ' viscorr = ', i6/,
     +           ' maniso = ', i6 )
         write(irefwr,520) (rinput(iprintur+j-1),j=1,24)
520      format(/' Array rintur'/ ' E         =', f9.2/
     +           ' kappa     =', f9.2/,' c_mu      =', f9.2/
     +           ' sigma_k   =', f9.2/,' sigma_eps =', f9.2/
     +           ' c_1eps    =', f9.2/,' c_2eps    =', f9.2/
     +           ' c_D       =', f9.2/,' sigma_T   =', f9.2/
     +           ' A         =', f9.2/,' E_rough   =', f9.2/
     +           ' eta_0     =', f9.2/,' betarng   =', f9.2/
     +           ' c_s       =', f9.2/,' alpha     =', f9.2/
     +           ' beta^*    =', f9.2/,' beta      =', f9.2/
     +           ' sigma     =', f9.2/,' sigma^*   =', f9.2/
     +           ' mod_prod  =', f9.2/,' c_3eps    =', f9.2/
     +           ' ctau_one  =', f9.2/,' ctau_two  =', f9.2/
     +           ' ctau_three=', f9.2 )

      end if

      end
