      subroutine ispri103 ( iinput, solut, text )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.2    date 08-07-1998 Kees V: nusol, nsolut cleaned
c        version 1.1    date 17-02-1995 (Print all components)
c        version 1.0    date 01-06-1994
c
c
c   copyright (c) 1994-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Print the contents of array solut for one block
c     Actual printing subroutine
c **********************************************************************
c
c                       KEYWORDS
c
c    print
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iinput(*)
      double precision solut(*)
      character *(*) text

c     iinput    i    reserved
c     solut     i    reserved
c     text      i    String in which a text is stored
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, ipsolut, ndim, ni, nj, nk, ntrnsp, nturb,
     +        nusol, nvirtual

c     i         General loop variable
c     ipsolut   Starting address of array solut in buffer
c     ndim      Dimension of space (2 or 3)
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     ntrnsp    Number of transport equations
c     nturb     Number of extra transport equations for turbulence
c     nusol     Number of unknowns for one degree of freedom in
c               one block
c     nvirtual  Number of rows of virtual cells
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISPRAR01  Print general solution array defined in R2
c     ISPRAR02  Print general solution array defined in R3
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c    Print heading
c    Extract ndim, ni, nj, nk, nvirtual
c    Print solut
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- Print heading

      write(irefwr,100) text
100   format(//1x,a)
      write(irefwr,110) iblockrf
110   format(//' Contents of array solut for block number', i4 //)

c     --- Extract ndim, ni, nj, nk, nvirtual

      ni       = iinput(51)
      nj       = iinput(52)
      nk       = iinput(53)
      nvirtual = iinput(54)
      ndim     = iinput(57)
      ntrnsp   = iinput(63)
      nturb    = iinput(70)

c     --- Print solut

      if ( ndim.eq.2 ) then

c     --- ndim = 2
c         First print the velocity

         nusol = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)
         write(irefwr,*) 'velocity'
         call isprar01 ( solut, nvirtual, ni, nj, 2 )

c        --- Next the pressure

         write(irefwr,*) 'pressure'
         ipsolut = 1+2*nusol
         call isprar01 ( solut(ipsolut), nvirtual, ni, nj, 1 )

c        --- The transport equations

         do 200 i = 1, ntrnsp
             write(irefwr,150) i
150          format ( 'transport quantity ', i2 )
             ipsolut = ipsolut+nusol
             call isprar01 ( solut(ipsolut), nvirtual, ni, nj, 1 )
200      continue

c        --- The turbulence equations

         do 220 i = 1, nturb
             write(irefwr,210) i
210          format ( 'turbulent quantity ', i2 )
             ipsolut = ipsolut+nusol
             call isprar01 ( solut(ipsolut), nvirtual, ni, nj, 1 )
220      continue

      else

c     --- ndim = 3
c         First print the velocity

         nusol = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)*(nk+1+2*nvirtual)
         write(irefwr,*) 'velocity'
         call isprar02 ( solut, nvirtual, ni, nj, nk, 3 )

c        --- Next the pressure

         write(irefwr,*) 'pressure'
         ipsolut = 1+3*nusol
         call isprar02 ( solut(ipsolut), nvirtual, ni, nj, nk, 1 )

c        --- The transport equations

         do 300 i = 1, ntrnsp
             write(irefwr,150) i
             ipsolut = ipsolut+nusol
             call isprar02 ( solut(ipsolut), nvirtual, ni, nj, nk, 1 )
300      continue

c        --- The turbulence equations

         do 320 i = 1, nturb
             write(irefwr,210) i
             ipsolut = ipsolut+nusol
             call isprar02 ( solut(ipsolut), nvirtual, ni, nj, nk, 1 )
320      continue

      end if

      end
