      subroutine ispri108 ( initer, resid )
c ======================================================================
c
c        programmer     Kees Vuik
c        version 1.6    date 22-09-1997 Kees V. changes for uncoupled
c        version 1.5    date 17-04-1997 Kees V. Multi-grid prec added
c        version 1.4    date 07-02-1997 Kees V. adding neumayer and
c                                               cgsrelax
c        version 1.3    date 21-01-1997 Kees V. inclusion of ispri109
c        version 1.2    date 18-09-1995 Kees V. adding of ILU_Fill
c        version 1.1    date 28-08-1995 Kees V. printing of amod
c        version 1.0    date 21-12-1994
c
c
c   copyright (c) 1994-1997  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Prints information from the linear solver.
c
c **********************************************************************
c
c                       KEYWORDS
c
c    print
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer          initer(*)
      double precision resid(*)

c     initer    i    reserved
c     resid     i    Array containing the norm of the residuals
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      logical          neumayer, coupled
      integer          i, iprinlin, method, mkind, nfill, ahilevel,
     +                 nsmooth
      double precision amod, cgsrelax

c     ahilevel  Maximum level of coarse grids (level 0 is finest)
c               (formal parameter)
c     amod      Parameter used in the average of ILU and MILU
c     cgsrelax  Relaxation parameter used in Bi-CSTAB
c     coupled   Logical which is true if the momentum equations
c               are coupled
c     i         General loop variable
c     iprinlin  Control parameter indicating the amount of output.
c     method    Control parameter indicating the iterative method.
c     mkind     Control parameter indicating the preconditioner.
c     neumayer  If true the Neumayer adaptation is used in CGS
c     nfill     The amount of fill in allowed in the preconditioner
c     nsmooth   Number of times smoother is applied (all levels)
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c **********************************************************************
c
c                       I/O
c
c     Writes information from the linear solver to standard output
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     If iprinlin is less than zero nothing is printed. If iprinlin is
c     equal to zero but no convergence occurred or iprinlin is equal to
c     one the name of the iteration method is printed. For iprinlin
c     greater than one all information about the iterative solver is given.
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen ( 'ispri108' )

      mkind     = initer(1)
      iprinlin  = initer(2)
      method    = initer(9)
      nfill     = initer(22)
      ahilevel  = initer(26)
      nsmooth   = initer(28)
      amod      = resid(initer(4)+1+7)
      cgsrelax  = resid(initer(4)+1+8)
      coupled  = initer(29).eq.0

      if ( initer(24).eq.0 ) then
         neumayer = .false.
      else
         neumayer = .true.
      end if

c     --- The start of the linear solver output

      if ( iprinlin.ge.1 .or. iprinlin.eq.0 .and.
     +     initer(3).ne.0 ) then
         if ( method.eq.0 ) write ( irefwr, 1000 ) 'LSQR.'
         if ( method.eq.1 .and. neumayer )  then
            write ( irefwr, 1000 ) 'CGS with Neumayer adaptation.'
         else if ( method.eq.1 ) then
            write ( irefwr, 1000 ) 'CGS without Neumayer adaptation.'
         end if
         if ( method.eq.2 ) write ( irefwr, 1000 ) 'GMRES.'
         if ( method.eq.3 ) write ( irefwr, 1000 ) 'GCR.'
         if ( method.eq.4 ) write ( irefwr, 1005 )  cgsrelax
      end if

c     --- Further start information is given

      if ( iprinlin.ge.2 .or. iprinlin.eq.0 .and.
     +     initer(3).ne.0 ) then
         if ( initer(11).eq.1 ) then
            if ( coupled ) then
               write ( irefwr, 1040 )
            else
               write ( irefwr, 1045 )
            end if
         end if
         if ( initer(11).eq.3 ) write ( irefwr, 1050 )
         if ( initer(11).eq.4 ) write ( irefwr, 1060 )

         if ( method.eq.2 ) then
            write ( irefwr, 1010 ) 'GMRES', initer(6)
         end if

         if ( mkind.eq.0 ) write ( irefwr, 1020 )
         if ( mkind.eq.1 ) write ( irefwr, 1030 ) ' A diagonal pre'
         if ( mkind.eq.2 ) write ( irefwr, 1070 ) ' An ILUD pre',
     +      amod
         if ( mkind.eq.3 ) write ( irefwr, 1070 ) ' An ILU pre',
     +      amod
         if ( mkind.eq.4 ) write ( irefwr, 1080 )
     +      ' An ILU_Fill pre', amod, nfill
         if ( mkind.eq.5 ) write ( irefwr, 1090 )
     +      ' A Multi-grid pre', ahilevel, nsmooth
         if ( mkind.eq.-1 ) write ( irefwr, 1030 ) ' A diagonal post'
         if ( mkind.eq.-2 ) write ( irefwr, 1070 ) ' An ILUD post',
     +      amod
         if ( mkind.eq.-3 ) write ( irefwr, 1070 ) ' An ILU post',
     +      amod
         if ( mkind.eq.-4 ) write ( irefwr, 1080 )
     +      ' An ILU_Fill post', amod, nfill
         if ( mkind.eq.-5 ) write ( irefwr, 1090 )
     +      ' A Multi-grid post', ahilevel, nsmooth

      end if

c     --- The initial and final residual are given

      if ( iprinlin.eq.1 ) then
         write( irefwr, 2000) resid(1), initer(4), resid(initer(4)+1)
      end if

c     --- All the residuals are given

      if ( iprinlin.ge.2 .or. iprinlin.eq.0 .and.
     +     initer(3).ne.0) then
         write ( irefwr, 2010 )
         do i = 1, initer(4)+1
            write ( irefwr, 2020) i-1, resid(i)
         end do
         write( irefwr, 2000) resid(1), initer(4),
     +                        resid(initer(4)+1)
      end if

1000  format(/, ' Output of subroutine ', a)
1005  format(/, ' Output of subroutine Bi-CGSTAB with',
     +          ' relaxation equal to ', e13.5)
1010  format(/, ' The ', a, ' method restarts after ', i4,
     +       ' iterations.')
1020  format(' No pre- nor postconditioning is used.')
1030  format(a,'conditioner is used.')
1040  format(' The linear solver is used to solve the coupled momentum',
     +       ' equations.')
1045  format(' The linear solver is used to solve the uncoupled',
     +       ' momentum equations.')
1050  format(' The linear solver is used to solve the pressure',
     +       ' equation.')
1060  format(' The linear solver is used to solve a transport',
     +       ' equation.')
1070  format(a,'conditioner is used, with average parameter',/,
     +       ' equal to', e15.5,'.')
1080  format(a,'conditioner is used, with average parameter',/,
     +       ' equal to', e15.5, ' and fill_in is', i4,'.')
1090  format(a,'conditioner is used with', i2,' levels.',/,
     +       ' The smoother is used', i2, ' times at all levels.')

2000  format(/,' The 2-norm of the initial residual is:', e15.5, /,
     +         ' The number of executed iterations is :', i5, /,
     +         ' The 2-norm of the final residual is  :', e15.5,/)
2010  format(/,' Iteration number |    residual',/
     +         ' -----------------| ---------------')
2020  format(i9,'         | ',e15.5)

      call iserclos ( 'ispri108' )

      end
