      subroutine issnflnd(nblocks, isiinput, issolut, iadmin )
c ======================================================================
c
c        programmer     Kees Kassels
c        version 1.4    date 05-01-2000 Guus: fix size of ireq
c        version 1.3    date 05-10-1999 Jason: added send-requests
c        version 1.2    date 23-11-1998 Jason: issenmpi version
c        version 1.1    date 09-06-1998 Charles: nvpoints changed to npointsv
c        version 1.0    date 20-04-1994
c
c   copyright (c) 1994-2000  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Send field from node to host
c
c **********************************************************************
c
c                       KEYWORDS
c
c     parallel
c     send
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer nblocks,  isiinput(*), issolut(*), iadmin(*)

c     iadmin    i    contains information about
c                    which blocks on which computer are stored.
c     isiinput  i    Array of sequence numbers (nblocks) of array iinput
c     issolut   i    Array of sequence numbers (nblocks) of array solut
c     nblocks   i    Number of blocks
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisparal'
      include 'IScommon/cisblank'
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer iblock, ipiinput, ndim,
     +        ni, nj, nk, nvirtual, npointsv,
     +        ipsolut, length, ireq(10000), isndcnt

c     iblock      block counting variable
c     ipiinput    address of an iinput array in ibuffr
c     ipsolut     pointer of array solut  in array ibuffr
c     ireq        array of send requests
c     isndcnt     send request counter
c     length      length of the array in words
c     ndim        dimension
c     ni          i-dimension of a block
c     nj          j-dimension of a block
c     nk          k-dimension of a block
c     npointsv    Number of points in the mesh with virtual points
c     nvirtual    number of virtual rows of coordinates
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isingetp, isdpgetp, isdpleng

c     ISACTIVE       Activate old permanent array
c     ISDPGETP       get pointer of double precision array
c     ISDPLENG       get length of double precision array
c     ISERCLOS       removes name of subroutine from error list
c     ISEROPEN       adds name of subroutine to error list
c     ISINGETP       get pointer of integer array
c     ISSENMPI       Send data to node or host
c     ISWAITALL      Wait for all sends to complete
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     for all local blocks do
c
c        send field
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call iseropen('issnflnd')

      isndcnt = 0

      if ( nblocks.gt.10000 ) then
         call iserrint ( nblocks, 1 )
         call iserrint ( 10000, 2 )
         call iserrchr ( 'ireq', 1 )
         call iserrsub ( 17, 2, 0, 1 )
         stop
      end if
      do 10 iblock =1, nblocks

         iblockrf = iblock

         if ( iadmin(iblock).eq.inodenr ) then

            call isactive(isiinput(iblock))
            call isactive(issolut(iblock))

            ipiinput = isingetp(isiinput(iblock))
            ipsolut = isdpgetp( issolut(iblock))

            ni       = ibuffr(ipiinput + 51 - 1)
            nj       = ibuffr(ipiinput + 52 - 1)
            nk       = ibuffr(ipiinput + 53 - 1)
            ndim     = ibuffr(ipiinput + 57 - 1)
            nvirtual = ibuffr(ipiinput + 54 - 1)
            npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)
            if ( ndim.eq.3 ) then
               npointsv = npointsv*(nk+1+2*nvirtual)
            end if

c           --- Only a single time level needs to be sent.
c               However, pressure is stored at time level ntimlv-1 instead of
c               ntimlv
c                          length = isdpleng( issolut(iblock) ) /ntimlv
c                          ipsolut = ipsolut + (ntimlv-1)*npointsv

c                         --- inefficient: send entire solut array

            length = isdpleng( issolut(iblock) )

            isndcnt = isndcnt + 1
	    ireq(isndcnt) = 0
            call issenmpi (buffer(ipsolut), length, ISDOUBLE, ISHOST,
     +         ICFIELD, ireq(isndcnt) )

         end if

10    continue

      call iswaitall ( isndcnt, ireq )

      call iserclos('issnflnd')

      end
