      subroutine ischkbnd ( iinbc, maxtrans, maxturb, ndim, compress,
     +                      sepran, icurvinf, ncurvs, nblocks, idimens,
     +                      iindsc )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.2    date   18-09-1997 Guus: extension because of decoupled
c                                         approach
c        version 1.1    date   11-04-1996 Kees K. dimension iinbc plus 1
c                                         for the pressure
c        version 1.0    date   18-03-1996
c
c   copyright (c) 1996-1997  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Check whether boundary conditions are given at all outer boundaries
c
c **********************************************************************
c
c                       KEYWORDS
c
c     boundary_conditions
c     error_manipulation
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer nblocks, iinbc(nblocks,*), maxtrans, maxturb, ndim,
     +        ncurvs, icurvinf(4,ncurvs), idimens(*),
     +        iindsc(100,maxtrans+1+maxturb)
      logical compress, sepran

c     compress  i    If true compressible has been read
c     icurvinf  i    reserved
c     idimens   i    Integer dimension parameters for the intermediate file
c     iinbc     i    reserved
c     iindsc    i    reserved
c     maxtrans  i    Maximum number of transport equations allowed
c     maxturb   i    Maximum number of turbulence equations allowed
c     nblocks   i    Number of blocks
c     ncurvs    i    Number of curves (SEPRAN environment)
c     ndim      i    Dimension of space (2 or 3)
c     sepran    i    If true the grid is created by SEPRAN
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer icurve, iface, ipstart, isubface, itrans,
     +        iturb, nsubf, ntrnsp, nturb, iblock

c     iblock    Block sequence number
c     icurve    SEPRAN curve sequence number
c     iface     Face number
c     ipstart   Starting address
c     isubface  Subface number
c     itrans    reserved
c     iturb     Turbulence sequence number
c     nsubf     Number of subfaces
c     ntrnsp    Number of transport equations
c     nturb     Number of extra transport equations for turbulence
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isfincur

c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c     ISFINCUR  Find curve number
c     ISWARSUB  Print warnings
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     602   No momentum boundary condition given along boundary
c     603   No transport boundary condition given along boundary
c     604   No turbulence boundary condition given along boundary
c **********************************************************************
c
c                       PSEUDO CODE
c
c      For iblock := 1 (1) nblocks do
c         For iface := 1 (1) 2*ndim do
c            istart := iinbc(iblock,i)
c            nsubfaces := iinbc(iblock,istart)
c            For isubface := 1 (1) nsubfaces do
c               Check if all boundary conditions are given
c               raise istart
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen ( 'ischkbnd' )

c     --- Get ntrnsp and nturb

      ntrnsp = idimens(3)
      nturb  = idimens(6)

c     --- Loop over blocks

      do iblock = 1, nblocks

         iblockrf = iblock

c        --- Loop over faces

         do iface = 1, 2*ndim
            ipstart = iinbc(iblock,iface)
            nsubf = iinbc(iblock,ipstart)
            ipstart = ipstart+1
            do isubface = 1, nsubf
               if ( iindsc(1,1).gt.0 .and.
     +              iinbc(iblock,ipstart+2*(ndim-1)).eq.0 ) then

c              --- no boundary conditions have been given for the momentum
c                  equations

                  call iserrint ( isubface, 1 )
                  call iserrint ( iface, 2 )
                  call iserrint ( iblock, 3 )
                  if ( sepran ) then
                     icurve = isfincur ( icurvinf, iblock,
     +                                   ipstart+2*ndim-2, ncurvs )
                     call iserrint ( icurve, 4 )
                  else
                     call iserrint ( 0, 4 )
                  end if
                  if ( compress ) then
                     call iswarsub ( 602, 4, 0, 0 )
                  else
                     call iserrsub ( 602, 4, 0, 0 )
                  end if
               end if
               do itrans = 1, ntrnsp
                  if ( iindsc(1,1+itrans).gt.0 .and. iinbc(iblock,
     +               ipstart+2*(ndim-1)+(ndim+1)*(itrans+ndim-1)).eq.0 )
     +               then

c                 --- no boundary conditions have been given for the transport
c                     equations

                     call iserrint ( itrans, 1 )
                     call iserrint ( isubface, 2 )
                     call iserrint ( iface, 3 )
                     call iserrint ( iblock, 4 )
                     if ( sepran ) then
                        icurve = isfincur ( icurvinf, iblock,
     +                                      ipstart+2*ndim-2, ncurvs )
                        call iserrint ( icurve, 5 )
                     else
                        call iserrint ( 0, 5 )
                     end if
                     call iswarsub ( 603, 5, 0, 0 )
                  end if
               end do
               do iturb = 1, nturb
                  if ( iindsc(1,1+maxtrans+iturb).gt.0 .and.
     +                 iinbc(iblock,ipstart+2*(ndim-1)+
     +                       (ndim+1)*(iturb+ndim-1+maxtrans)).eq.0 )
     +               then

c                 --- no boundary conditions have been given for the turbulent
c                     equations

                     call iserrint ( itrans, 1 )
                     call iserrint ( isubface, 2 )
                     call iserrint ( iface, 3 )
                     call iserrint ( iblock, 4 )
                     if ( sepran ) then
                        icurve = isfincur ( icurvinf, iblock,
     +                                      ipstart+2*ndim-2, ncurvs )
                        call iserrint ( icurve, 5 )
                     else
                        call iserrint ( 0, 5 )
                     end if
                     call iserrsub ( 604, 5, 0, 0 )
                  end if
               end do
               ipstart = ipstart+(ndim+1)*(maxtrans+maxturb+ndim)+
     +                   2*(ndim-1)
            end do

         end do

      end do

      call iserclos ( 'ischkbnd' )

      end
