      subroutine isinitbc ( iinbc, nblocks, nfaces, maxnsubf, nsubfaces,
     +                      subfaces, maxtrans, icurvinf, ineigh,
     +                      ncurvs, iworkgen, ndim, sepran )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.4    date 01-10-1997 Guus: adaptation for decoupled
c        version 1.3    date 03-04-1996 Kees K. dimension itypebc, rtypebc
c                                       plus 1 for the pressure
c        version 1.2    date 19-03-1996 Layout
c        version 1.1    date 19-01-1995 Kees K. 3D
c        version 1.0    date 20-09-1994
c
c
c   copyright (c) 1994-1996  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Initialize array iinbc with default information of boundaries
c    Internal boundaries are considered as multiblock boundaries
c    All other (external) boundaries are set equal to 0
c **********************************************************************
c
c                       KEYWORDS
c
c     boundary_conditions
c     initializations
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer nblocks, nfaces, maxnsubf, iinbc(nblocks,*), ncurvs,
     +        nsubfaces(nblocks,nfaces), maxtrans, icurvinf(4,ncurvs),
     +        subfaces(1:nblocks,1:nfaces,1:maxnsubf,*), ndim,
     +        ineigh(10+(ndim-2)*4,ncurvs), iworkgen(nblocks,*)
      logical sepran

c     icurvinf  o    reserved
c     iinbc     o    reserved
c     ineigh    i    reserved
c     iworkgen       General integer work array.
c                    See pseudo code for the contents
c     maxnsubf  i    Maximum number of subfaces encountered on all subfaces
c     maxtrans  i    Maximum number of transport equations allowed
c     nblocks   i    Number of blocks
c     ncurvs    i    Number of curves (SEPRAN environment)
c     ndim      i    Dimension of space (2 or 3)
c     nfaces    i    Number of faces
c     nsubfaces i    Array containing the number of subfaces of each face,
c                    see array itopol for an extended description
c     sepran    i    If true the grid is created by SEPRAN
c     subfaces  i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, iblock, iblocknb, icurve, iface, ipos, iseg,
     +        nseg, ipos1

c     i         General loop variable
c     iblock    Block sequence number
c     iblocknb  Block number of neighbour
c     icurve    SEPRAN curve sequence number
c     iface     Face number
c     ipos      Position in array
c     ipos1     Position in array
c     iseg      Counting variable: sequence number of segment
c     nseg      Number of segments the boundary is divided in
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     For iblock := 1 (1) nblocks do
c        ipos := nfaces+1
c        ipos1 := 1
c        For iface := 1 (1) nfaces do
c           iinbc(iblock,iface) := ipos
c           iworkgen(iblock,iface) := ipos1
c           nseg = nsubfaces(iblock,iface)
c           iinbc(iblock,ipos) := nseg
c           ipos := ipos+1
c           For iseg := 1 (1) nseg do
c              if ( ndim = 2 ) then
c              iinbc(iblock,ipos) := subfaces(iblock,iface,iseg,1)
c              iinbc(iblock,ipos+1) := subfaces(iblock,iface,iseg,2)
c              iblocknb := subfaces(iblock,iface,iseg,5)
c              if ( iblocknb=0 ) then
c                 Clear itype part
c              else ( iblocknb>0 ) then
c                 Set itype part
c              ipos := ipos+3*(maxtrans+2)+2
c              ipos1 := ipos1+2*(maxtrans+2)
c    Clear array icurvinf
c    For icurve := 1 (1) ncurvs do
c       iblock   := first block icurve is lying in
c       iblocknb := second block icurve is lying in
c       if ( iblock>0 and iblocknb=0 ) then
c          iface := face number of curve in iblock
c          iseg := subface number of curve in iblock
c          ipos := iinbc(iblock,iface)+1+(iseg-1)*(3*(maxtrans+2)+2)
c          ipos1 := iworkgen(iblock,iface)+(iseg-1)*2*(maxtrans+2)
c          icurvinf(1,icurve) := iblock
c          icurvinf(2,icurve) := ipos
c          icurvinf(3,icurve) := ipos1
c   Remark: in iworkgen the starting positions of the faces per block in
c           rinbc are stored
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c

      do iblock = 1, nblocks
         ipos = nfaces+1
         ipos1 = 1
         do iface = 1, nfaces
            iinbc(iblock,iface) = ipos
            iworkgen(iblock,iface) = ipos1
            nseg = nsubfaces(iblock,iface)
            iinbc(iblock,ipos) = nseg
            ipos = ipos+1
            do iseg = 1, nseg
               if ( ndim.eq.2 ) then

c              --- ndim = 2

                  iinbc(iblock,ipos) = subfaces(iblock,iface,iseg,1)
                  iinbc(iblock,ipos+1) = subfaces(iblock,iface,iseg,2)
                  iblocknb = subfaces(iblock,iface,iseg,5)
                  ipos = ipos+2
                  if ( iblocknb.eq.0 ) then

c                 --- No neighbouring block, hence external boundary
c                     Clear itype part

                  do i = 0, 3*(maxtrans+2)-1
                     iinbc(iblock,ipos+i) = 0
                  end do

                  else

c                 --- Neighbouring block, hence internal boundary
c                     Set itype part

                  do i = 1, maxtrans+2
                     iinbc(iblock,ipos+3*(i-1)  ) = 99
                     iinbc(iblock,ipos+3*(i-1)+1) = 0
                     iinbc(iblock,ipos+3*(i-1)+2) = 0
                  end do

                  end if
                  ipos = ipos+3*(maxtrans+2)
                  ipos1 = ipos1+2*(maxtrans+2)

               else

c              --- ndim = 3

                  iinbc(iblock,ipos) = subfaces(iblock,iface,iseg,1)
                  iinbc(iblock,ipos+1) = subfaces(iblock,iface,iseg,3)
                  iinbc(iblock,ipos+2) = subfaces(iblock,iface,iseg,2)
                  iinbc(iblock,ipos+3) = subfaces(iblock,iface,iseg,4)
                  iblocknb = subfaces(iblock,iface,iseg,5)
                  ipos = ipos+4
                  if ( iblocknb.eq.0 ) then

c                 --- No neighbouring block, hence external boundary
c                     Clear itype part

                  do i = 0, 4*(maxtrans+3)-1
                     iinbc(iblock,ipos+i) = 0
                  end do

                  else

c                 --- Neighbouring block, hence internal boundary
c                     Set itype part

                  do i = 1, maxtrans+3
                     iinbc(iblock,ipos+4*(i-1)  ) = 99
                     iinbc(iblock,ipos+4*(i-1)+1) = 0
                     iinbc(iblock,ipos+4*(i-1)+2) = 0
                     iinbc(iblock,ipos+4*(i-1)+3) = 0
                  end do

                  end if
                  ipos = ipos+4*(maxtrans+3)
                  ipos1 = ipos1+3*(maxtrans+3)
               end if
            end do
         end do
      end do

c     --- Clear array icurvinf

      if ( sepran ) then
         do icurve = 1, ncurvs
            icurvinf(1,icurve) = 0
            icurvinf(2,icurve) = 0
         end do
      end if

c     --- Fill array icurvinf

      if ( sepran ) then
         do icurve = 1, ncurvs

            if ( ndim.eq.2 ) then

c           --- ndim = 2

               iblock = ineigh(1,icurve)
               iblocknb = ineigh(6,icurve)

               if ( iblock.gt.0 .and. iblocknb.eq.0 ) then

c              --- Curve does not consist of subcurves and curve is outer curve

                  iface = abs(ineigh(2,icurve))
                  iseg  = ineigh(3,icurve)
                  ipos  = iinbc(iblock,iface)+3+
     +                          (iseg-1)*(3*(maxtrans+2)+2)
                  ipos1 = iworkgen(iblock,iface)+(iseg-1)*2*(maxtrans+2)
                  icurvinf(1,icurve) = iblock
                  icurvinf(2,icurve) = ipos
                  icurvinf(3,icurve) = ipos1

               end if

            else

c           --- ndim = 3

               iblock = ineigh(1,icurve)
               iblocknb = ineigh(8,icurve)

               if ( iblock.gt.0 .and. iblocknb.eq.0 ) then

c              --- Curve does not consist of subcurves and curve is outer curve

                  iface = abs(ineigh(2,icurve))
                  iseg  = ineigh(3,icurve)
                  ipos  = iinbc(iblock,iface)+5+
     +                          (iseg-1)*(4*(maxtrans+3)+4)
                  ipos1 = iworkgen(iblock,iface)+(iseg-1)*3*(maxtrans+3)
                  icurvinf(1,icurve) = iblock
                  icurvinf(2,icurve) = ipos
                  icurvinf(3,icurve) = ipos1

               end if

            end if

         end do

      end if

c     --- fill positions 9 and 10 of the subfaces array with the starting
c         positions in iinbc and rinbc

      do iblock = 1, nblocks
         do iface = 1, nfaces
            nseg = nsubfaces(iblock,iface)
            do iseg = 1, nseg
               iblocknb = subfaces(iblock,iface,iseg,5)
               if ( iblocknb.eq.0 ) then

                  if ( ndim.eq.2 ) then

c                 --- ndim = 2

                     ipos  =
     +               iinbc(iblock,iface)+3+(iseg-1)*(3*(maxtrans+2)+2)
                     ipos1 =
     +               iworkgen(iblock,iface)+(iseg-1)*2*(maxtrans+2)
                     subfaces(iblock,iface,iseg,9) = ipos
                     subfaces(iblock,iface,iseg,10) = ipos1

                  else

c                 --- ndim = 3

                     ipos  =
     +               iinbc(iblock,iface)+5+(iseg-1)*(4*(maxtrans+3)+4)
                     ipos1 =
     +               iworkgen(iblock,iface)+(iseg-1)*3*(maxtrans+3)
                     subfaces(iblock,iface,iseg,9) = ipos
                     subfaces(iblock,iface,iseg,10) = ipos1

                  end if

               end if
            end do
         end do
      end do

      end
