      subroutine isintx02 ( code, ncode, nncode, itype, table, codemn,
     +                      nmcode, nmain, iretur, tabmin, tabmax,
     +                      tabout, nout, strout, nstrng, ncodmx )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.4    date   28-01-1998 Guus: Extra error message
c        version 1.3    date   01-10-1997 Guus: Better error message
c        version 1.2    date   13-02-1995 Kees K. itype=26
c        version 1.1    date   18-01-1995 Correction tabmax=n in case keyword
c                                         does not appear
c        version 1.0    date   24-08-1994 Subroutine extracted from the SEPRAN
c                                         subroutine INTX02 version 1.5
c
c
c   copyright (c) 1994-1998  "Ingenieursbureau SEPRA"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "Ingenieursbureau SEPRA".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c   read rest of record and check if keywords from the code table followed
c   by a number are available
c   It is assumed that either the record has alrady been read or that ityprd
c   is set equal to 1
c   The subroutine starts with reading the next item but reads only one record
c   If a keyword from the code table is found, the value following it is
c   read and put in the right position of table
c   It is checked if the value read is within a given range
c   If an item is not recognized an error message is given
c   If a main keyword is found or end of record the subroutine returns
c **********************************************************************
c
c                       KEYWORDS
c
c  interpret
c  keyword
c  read_cards
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer nncode, ncode(nncode), itype(*), nmain, nmcode(*), iretur,
     +        nout, nstrng, ncodmx(nncode)
      character *(*) code(nncode), codemn(*), strout(nstrng)
      double precision table(nncode), tabmin(nncode), tabmax(nncode),
     +                 tabout(nout)

c     code    i    Table containing the subkeywords corresponding to this
c                  subroutine.
c     codemn  i    Table of main keywords
c     iretur  o    Return code, indicates the sequence number of the new
c                  main keyword found
c                  If -1 an error has occurred
c                  If -2 the end of file has been reached
c     itype   i    Array containing the type of actions to be undertaken for
c                  each subkeyword found.
c                  See subroutine isintx02 for the possible values of itype
c     ncode   i    Array containing the number of significant characters for
c                  each keyword in code
c     ncodmx  i    Array containing the maximum number of characters for
c                  each keyword in code
c     nmain   i    Number of keywords in array codemn
c     nmcode  i    Array containing the number of significant characters
c                  for each main keyword in codemn
c     nncode  i    Number of subkeywords stored in array code
c     nout    i    Length of array tabout
c     nstrng  i    Length of array strout
c     strout  o    Contains series of strings, see itype
c     table  i/o   Array containing the real or integer values corresponding to
c                  the subkeywords in code that have been read by isintx02
c     tabmax  i    Array containing the maximal values of the parameters read
c                  corresponding to the subkeywords in code
c     tabmin  i    Array containing the minimum values of the parameters read
c                  corresponding to the subkeywords in code
c     tabout  o    Array of output reals provided by isintx02
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/csepch'
      include 'IScommon/cseprd'
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      logical ready, readnx, close, found, seqk1, seqk2
      integer icode, i, ivalrd, minval, maxval, imin, imax, iseqkey,
     +        iseqmn, iseqmx, num, iposut, itypsv, iminsv, istartkey,
     +        minusg, ih, ispecs, ihelp(100), ifunc, numkey, inumlc,
     +        nsubbrack, nmax, kposut, irep
      double precision value
      character *80 subkey

c     close     Indicates if the end of a sequence of numbers has been found
c               (true) or not (false)
c     found     Indicates if already a etxt or number has been found after
c               plane (true) or not (false)
c     i         General loop variable
c     icode     Position number of keyword found in table code
c     ifunc     Parameter k after FUNC =
c     ih        Help variable to store a constant temporarily
c     ihelp     Help array to store number of appearances of keywords that
c               may be used more than once or number of numbers read
c     imax      Maximum position in table
c     imin      Minimum position in table
c     iminsv    Help variable to store the previous value of imin
c     inumlc    Number of numbers read in a record
c     iposut    last position used in array tabout
c     irep      Number of local repetitions
c     iseqkey   Sequence number read after keyword
c     iseqmn    Minimum value allowed of sequence number read after keyword
c     iseqmx    Maximum value allowed of sequence number read after keyword
c     ispecs    Help variable to save the value of ispecl
c     istartkey Starting address of information about second keyword in
c               itype
c     itypsv    Help variable to store the old value of ityprd
c     ivalrd    Integer value read
c     kposut    last position used in array strout
c     maxval    Maximum value allowed for the integer read
c     minusg    Indicates if a minus sign has been found (-1) or not (1)
c     minval    Minimum value allowed for the integer read
c     nmax      Maximum number of numbers allowed
c     nsubbrack Number of numbers between subbrackets
c     num       Number of numbers read between brackets
c     numkey    Number of significant characters in subkeyword
c     readnx    If true the next item must be read
c     ready     Indicates if a while loop is finished or not
c     seqk1     Indicates if sequence number 1 has been found
c     seqk2     Indicates if sequence number 2 has been found
c     subkey    Sub (second) keyword
c     value     Real number read by isreadfn
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISASKNXT  Read next item from the standard input file
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRCHR  Puts values of strings in character error array
c     ISERRCRD  Prints the last record read and underlines the errorneous part
c     ISERREAL  Put real value in error message
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c     ISINTX04  Read a series of numbers until ) or test is found
c     ISREADFN  Read items of the shape number, VALUE=number or FUNC=i
c     ISWARSUB  Print warnings
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     528   Not an integer number found after the keyword
c     534   string could not be recognized
c     536   No character found after number in the part following keyword
c     537   Character could not be recognized in the part following keyword
c     538   Number of items more than 100
c     539   Keyword occurs too many times
c     540   not a series of numbers found after keyword
c     541   number of numbers between brackets incorrect
c     542   incorrect keyword found after keyword
c     543   too much series of numbers read
c     544   The number of numbers found after the keyword too large
c     545   Number found after keyword out of range
c     546   Letter found in the part ax+by+cz=d more than once
c     547   keyword has been found more than once
c     548   Array tabout too small
c     549   Array strout too small
c     551   Text not recognized
c     552   No number found after keyword
c     553   Integer number read out of range
c     554   Real number read out of range
c     555   Not a text found after the keyword
c     556   Not a sequence number found after the keyword
c     557   The sequence number found after the keyword is incorrect
c     558   Not a number of the correct type found after keyword followed
c           by sequence number
c     559   Too few or too much numbers read
c     560   Not a second keyword found after keyword =
c     588   Keyword contains too much characters
c **********************************************************************
c
c                       PSEUDO CODE
c
c    ready := false
c    Clear array ihelp
c    While ( not ready ) do
c       read next item
c       if ( end of file found ) then
c          iretur := -2
c          ready := true
c       else if ( end of record found ) then
c          iretur := 0
c          ready := true
c       else if ( item # keyword ) then
c          Error message
c          iretur := -1
c          ready := true
c       else if ( item = main keyword ) then
c          set iretur
c          ready := true
c       else if ( item = keyword from code ) then
c          icode := sequence number of keyword in code table
c          if ( itype(icode) # 2 and # -1 ) then
c             read next item
c             if ( itype(icode) = 0, 1, 3 or 4 and next item is a number ) then
c                Check type of number
c                Fill number in table
c             else if ( itype(icode) = 5 or 13 and next item is a text ) then
c                Check type of text
c                Fill number in table
c                if ( itype(icode) = 13 ) then
c                   If ( second keyword should be present ) then
c                      Read next keyword
c                      Check type of text
c                      Fill number in table
c             else if ( itype(icode) = 6 ) then
c                Read and store text
c             else if ( itype(icode) = 3 or 4 and no number found ) then
c                skip
c             else if ( itype(icode) = 10 and next item is integer number ) then
c                Check sequence number
c                Read next item
c                Check if this is a number in the correct range
c                Fill number in table
c             else if ( itype(icode) = 11 ) then
c                Read and check series of integers between brackets
c                Store integers in tabout
c                Set table(i) and tabmax(i)
c             else if ( itype(icode) = 14  or itype(icode) = 18 ) then
c                if ( itype(icode) = 18 ) then
c                   Set keyword to subkeyword
c                While ( next item = keyword with number icode ) do
c                   Read next item (must be integer) and store in tabout
c                   Read next item
c                Set table(i) and tabmax(i)
c             else if ( itype(icode) = 15 ) then
c                if ( ihelp(icode)=0 ) then
c                   Fill table(icode)
c                Read next item (must be integer) and store in tabout
c                Raise ihelp
c             else if ( itype(icode) = 16 ) then
c                Read next item
c                if ( item = 'value', 'func' or 'time_func' ) then
c                   read next item
c                Fill table and if necessary tabout
c             else if ( itype(icode) = 17 ) then
c                Read series of numbers until new item is found
c                Fill table and tabout
c             else if ( itype(icode) = 19 ) then
c                Read series of numbers until new item is found
c                Fill table and tabout
c             else if ( itype(icode) = 20 ) then
c                Check if keyword is followed by special keyword
c                If ( not special keyword found ) then
c                   Set table(icode)
c                else
c                   Read and store integers
c             else if ( itype(icode) = 21 ) then
c                Read ax+by+sz=d
c                Store result in posout
c             else if ( itype(icode) = 22 or 24 ) then
c                Read next item
c                if ( next item = special keyword ) then
c                   read next item
c                if ( next item = special keyword ) then
c                   read next item
c                Read numbers and store in posout
c             else if ( itype(icode) = 23 ) then
c                Read next item
c                If ( next item is special keyword ) then
c                   skip
c                Set table(icode)
c             else if ( itype(icode) = 25 and next item is integer number ) then
c                Check sequence number
c                Read next item
c                if ( item = 'value', 'func' or 'time_func' ) then
c                   read next item
c                Fill table and if necessary tabout
c             else if ( itype(icode) = 26 and next item is integer number ) then
c                Check sequence number
c                Read next item
c                if ( item = 'value', 'func' or 'time_func' ) then
c                   read next item
c                Fill table and if necessary tabout
c             else
c                error message
c                iretur := -1
c                ready := true
c          else if ( itype(icode) = 2 ) then
c             Put a 1 in the table
c       else ( keyword not recognized )
c          iretur := -1
c          ready := true
c
c **********************************************************************
c
c
c
c                    LOCAL DESCRIPTION OF SOME ARRAYS
c
c    itype        Possible values (short description):
c                  -1, keyword, skip
c                   0, keyword = integer
c                   1, keyword = real
c                   2, keyword, Set table to 1
c                   3, keyword or keyword = integer
c                   4, keyword or keyword = integer, indicate both situations
c                   5, keyword = keyword
c                   6, keyword = 'text'
c                  10, keyword iseq1 = j1, keyword iseq2 = j2, ...
c                  11, keyword = i1, i2, i3, ...  all positive integer
c                  12, keyword iseq1 = j1, keyword iseq2 = j2, ...
c                      or keyword = j1 (iseq1 is set to 1)
c                  13, keyword = keyword1 [keyword2], the last may be empty
c                  14, c1, -c3, c4, -c9, c5, i.e. list of keywords followed
c                      by sequence number
c                  15, c1 v1 c2 v2 List of keyword followed by sequence number
c                      followed by another keyword followed by sequence number
c                      and this repeatedly
c                  16, keyword = real
c                      or keyword (value = real)
c                      or keyword (func = i)
c                      or keyword (time_func = i)
c                  17, keyword = ( (x1,y1,..), (x2,y2,..)  ... (xn,yn,..) )
c                  18, keyword = P1, P2, ...
c                  19, keyword = ( x1, y1, z1 )
c                  20, keyword or keyword keyword = i1, i2, ...
c                  21, keyword (ax+by+cz=d)
c                  22, keyword [keyword1] [keyword2] = ( x1, y1, z1 )
c                  23, keyword keyword  set tabel to 1
c                  24, keyword [keyword1] [keyword2] = ( i1, i2, i3 )
c                  25, keyword iseq1 = real
c                      or keyword iseq1 (value = real)
c                      or keyword iseq1 (func = i)
c                      or keyword iseq1 (time_func = i)
c                  26, keyword iseq1 = real keyword iseq2 = real
c                      or keyword iseq1 (value = real)
c                      or keyword iseq1 (func = i)
c                      or keyword iseq1 (time_func = i)
c                      or keyword iseq2 (value = real)
c                      or keyword iseq2 (func = i)
c                      or keyword iseq2 (time_func = i)
c
c
c                  Possible values (long description):
c                  itype(i) = -1, no number, the information must be skipped
c                                input:  code(i) keyword
c                                output: -
c                  itype(i) = 0, number i is of type integer
c                                input:  code(i) keyword
c                                        tabmin(i) minimum value
c                                        tabmax(i) maximum value
c                                output: table(i)  value
c                  itype(i) = 1, number i is of type real
c                                input:  code(i) keyword
c                                        tabmin(i) minimum value
c                                        tabmax(i) maximum value
c                                output: table(i)  value
c                  itype(i) = 2, no number but in table the value 1 must be set
c                                input:  code(i) keyword
c                                output: table(i): 1
c                  itype(i) = 3, either an integer number or no number at all
c                                input:  code(i) keyword
c                                        tabmin(i) minimum value
c                                        tabmax(i) maximum value
c                                output: table(i)  value or nothing
c                  itype(i) = 4, either an integer number or no number at all
c                                input:  code(i) keyword
c                                        tabmin(i) minimum value
c                                        tabmax(i) maximum value
c                                output: table(i) = -1000 keyword without number
c                                        table(i) = iposut keyword with number
c                                        tabout(iposut) = value
c                  itype(i) = 5, number is given in the form of a keyword
c                                input:  code(i) keyword
c                                        tabmin(i) starting position allowed
c                                                  keywords in codemn
c                                        tabmax(i) end position allowed
c                                                  keywords in codemn
c                                        codemn(tabmin(i) to tabmax(i))
c                                output: table(i) = relative position of
c                                                   keyword found in codemn
c                                                   i.e. from 1 to
c                                                   tabmax(i)-tabmin(i)+1
c                  itype(i) = 6, text between quotes
c                                input:  code(i) keyword
c                                output: table(i) kposut
c                                        tabmax(i) number of characters in text
c                                        strout(kposut) = string
c                  itype(i) =10, special situation of keyword followed
c                                by sequence number followed by number,
c                                for example inpvc i = j
c                                In this case the number positions
c                                in the arrays code itype etc. is equal to the
c                                range of the sequence numbers
c                                input:  code(i to i+max-1) keyword
c                                        max is tabmax(i)-tabmin(i)+1
c                                        tabmin(i) minimum value i
c                                        tabmax(i) maximum value i
c                                        tabmin(i+1) minimum value j
c                                        tabmax(i+1) maximum value j
c                                        itype(i+1) = 0: j is integer
c                                        itype(i+1) = 1: j is real
c                                output: table(i to i+max-1) contain the
c                                        values of j for respectively
c                                        k=tabmin(i) to k=tabmax(i)
c                  itype(i) =11, special situation of keyword followed by
c                                series of positive integer numbers
c                                Brackets are treated as separator
c                                input:  code(i) keyword
c                                        tabmin(i) maximum number of integers
c                                        tabmax(i) maximum value of integers
c                                output: table(i) = iposut
c                                        tabmax(i) = m (number of integers read)
c                                        tabout(iposut to iposut+m-1) = values
c                  itype(i) =12, special situation of keyword followed
c                                by sequence number followed by number,
c                                for example inpvc i = j or alternatively
c                                keyword followed by number
c                                for example inpvc = j. i = 1 in that case
c                                Rest see itype(i) = 10
c                  itype(i) =13, special situation of keyword followed
c                                by one or two keywords, for example:
c                                renumber = cuthill profile
c                                input:  code(i) keyword
c                                        code(i+1) same keyword
c                                        itype(i+1) ipostyp
c                                        itype(ipostyp+2*j-2) ipmn1_j
c                                        itype(ipostyp+2*j-1) ipmn2_j
c                                        tabmin(i) ipmn1_1
c                                        tabmax(i) ipmn2_1
c                                        codemn(ipmn1_1 to ipmn2_1) contains
c                                        the list of first keywords allowed
c                                        codemn(ipmn1_j to ipmn2_j) contains
c                                        the list of second keywords allowed
c                                        corresponding to first keyword "j",
c                                        where j is the relative seq. number
c                                        If to first keyword j no second
c                                        keyword corresponds ipmn1_j and ipmn2_j
c                                        must be 0
c                                output: table(i) = j1
c                                        table(i+1) = j2
c                                        j1, j2 denote the relative ref. number
c                                        corresponding to first resp. second
c                                        keyword
c                  itype(i) =14, special situation of keyword followed
c                                by number, followed by same keyword followed
c                                by number and so on, for example:
c                                p1, p3, p5, p8, p2
c                                The keywords may be preceded by a minus sign,
c                                hence c1, c3, -c5, c8 etc., in which the
c                                sequence number gets the minus sign
c                                input:  code(i) keyword
c                                        tabmin(i) maximum number of integers
c                                        tabmax(i) maximum value of integers
c                                output: table(i) = iposut
c                                        tabmax(i) = m (number of integers read)
c                                        tabout(iposut to iposut+m-1) = values
c                  itype(i) =15, special situation of keyword followed
c                                by number, followed by same keyword followed
c                                by number etc. However, in this case it is
c                                possible that the keyword number combination
c                                is followed by other keywords, before it
c                                is followed by the next keyword number
c                                combination
c                                Example: c1 v1 + c2 v2
c                                In this case both c and v get type number 15
c                                In this case in tabout exactly tabmin(i)
c                                positions are reserved. The rest of the input
c                                is exactly the same as for itype(i) = 14
c                  itype(i) =16, special situation of keyword followed
c                                by either a value or the possibilities
c                                value = v, func = ifunc, or time_func = ifunc
c                                input:  code(i) keyword
c                                output: table(i) = ifunc (>0) or -iposut
c                                        tabout(iposut) = value
c                                        if time_func = ifunc is given
c                                        table(i) gets the value -ifunc-100
c                  itype(i) =17, special situation of keyword followed by
c                                series of series of numbers between
c                                brackets. Each series of local numbers itself
c                                must be between brackets. The number of
c                                numbers in the inner series must be constant.
c                                input:  code(i) keyword
c                                        tabmin(i) number of numbers in inner
c                                                  series
c                                        tabmax(i) maximum number of series
c                                output: table(i)  iposut
c                                        tabout(iposut to iposut+m-1) values
c                                        tabmax(i) m
c                  itype(i) =18, special situation of keyword followed by
c                                followed by a series of the same keywords
c                                followed by sequence numbers
c                                hence we have the situation keyword followed
c                                by possibility 14
c                                input:  code(i) keyword
c                                        table(i) -iposmain
c                                        codemn(iposmain) special sub keyword
c                                        tabmin(i) maximum number of numbers
c                                                  that may be read
c                                        tabmax(i) maximum value of |numbers|
c                                output: table(i)  iposut
c                                        tabout(iposut to iposut+m-1) values
c                                        tabmax(i) m
c                  itype(i) =19, special situation of keyword followed by
c                                a series of numbers possibly enclosed by
c                                brackets
c                                input:  code(i) keyword
c                                        tabmin(i) maximum number of numbers
c                                                  that may be read
c                                        tabmax(i) maximum value of |numbers|
c                                output: table(i)  iposut
c                                        tabout(iposut to iposut+m-1) values
c                                        tabmax(i) m
c                  itype(i) =20, special situation of keyword which has to
c                                be set or the same keyword followed by
c                                another special keyword followed by integers
c                                input:  code(i) keyword
c                                        table(i) -iposmain
c                                        codemn(iposmain) special sub keyword
c                                        tabmin(i) maximum number of numbers
c                                                  that may be read
c                                        tabmax(i) maximum value of |numbers|
c                                output: table(i)  iposut
c                                        or -100000 if subkeyword is not present
c                                        tabout(iposut to iposut+m-1) values
c                                        tabmax(i) m
c                  itype(i) =21, special situation of keyword followed by
c                                (ax+by+cz=d), with a, b, c and d numbers
c                                if a, b or c is omitted 1 or -1 is supposed
c                                At output table(i) contains the starting
c                                position of the 4 numbers a,b,c and d in that
c                                sequence in tabout
c                  itype(i) =22, special situation of keyword followed by
c                                possibly some other special keywords followed
c                                by a series of numbers
c                                input:  code(i) keyword
c                                        table(i) -iposmain
c                                        codemn(iposmain) special sub keyword
c                                        tabmin(i) maximum number of numbers
c                                                  that may be read
c                                        tabmax(i) maximum value of |numbers|
c                                output: table(i)  iposut
c                                        tabout(iposut to iposut+m) values read
c                                        tabmax(i) m
c                  itype(i) =23, special situation of keyword followed by
c                                by a special keyword that must be skipped
c                                input:  code(i) keyword
c                                        table(i) -iposmain
c                                        codemn(iposmain) special sub keyword
c                                output: table(i): 1
c                  itype(i) =24, special situation of keyword followed by
c                                possibly some other special keywords followed
c                                by a series of positive integers
c                                See 22
c                  itype(i) =25, special situation of keyword followed by
c                                sequence number followed by either a value or
c                                the possibilities value = v, func = ifunc,
c                                or time_func = ifunc (See 10 and 16)
c                                input:  code(i to i+max-1) keyword
c                                        max is tabmax(i)-tabmin(i)+1
c                                        tabmin(i) minimum value i
c                                        tabmax(i) maximum value i
c                                output: table(k) = ifunc (>0)
c                                        or -iposut
c                                        tabout(iposut) = value
c                                        if time_func = ifunc is given
c                                        table(k) gets the value -ifunc-100
c                                        for respectively
c                                        k=tabmin(i) to k=tabmax(i)
c                  itype(i) =26, special situation of keyword followed by
c                                sequence number followed by either a value or
c                                the possibilities value = v, func = ifunc,
c                                or time_func = ifunc (See 10 and 16)
c                                then
c                                followed by the same keyword followed by
c                                sequence number followed by either a value or
c                                the possibilities value = v, func = ifunc,
c                                or time_func = ifunc (See 10 and 16) then
c                                input:  code(i ) keyword
c                                        tabmin(i) minimum value i
c                                        tabmax(i) maximum value i
c                                output: table(i) = iposut
c                                        tabout(iposut to iposut+3) = values
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call iseropen ( 'isintx02' )
      ready = .false.
      readnx = .false.
      seqk1 =  .false.
      seqk2 =  .false.
      call isasknxt
      iposut = 0
      kposut = 0
      ispecs = ispecl
      if ( nncode.gt.100 ) then
         call iserrsub ( 538, 0, 0, 0 )
         go to 1000
      end if
      do i = 1, nncode
        ihelp(i) = 0
      end do

100   if ( .not. ready ) then

c     --- read next item

         minusg = 1
         ispecl = 4
         if ( readnx ) then
            ityprd = 4
            call isasknxt
         end if
         readnx = .true.
         if ( ityprd.eq.6 ) then

c        --- minus sign found

            minusg = -1
            call isasknxt

         end if
         if ( ityprd.eq.7 ) call isasknxt

         if ( ityprd.eq.0 ) then

c        --- end of file found

            iretur = -2
            ready = .true.

         else if ( ityprd.eq.1 ) then

c        --- end of record found

            iretur = 0
            ready = .true.

         else if ( ityprd.ne.4 ) then

c        --- item # keyword

            call iserrsub ( 551, 0, 0, 0 )
            call iserrcrd
            iretur = -1
            ready = .true.

         else

c        --- Keyword found, check if it is a main keyword

            do i = 1, nmain
               if ( string(1:nmcode(i)).eq.codemn(i)(1:nmcode(i)) )
     +              then

c              --- Main keyword found

                  iretur = i
                  ready = .true.
                  go to 100
               end if
            end do

            do i = 1, nncode
               if ( string(1:ncode(i)).eq.code(i)(1:ncode(i)) ) then

c              --- Text found

                  icode = i
                  if ( nchar.gt.ncodmx(i) ) then

c                 --- Too much characters in keyword

                     call iserrint ( nchar, 1 )
                     call iserrint ( ncodmx(i), 2 )
                     call iserrchr ( string(1:nchar), 1 )
                     call iswarsub ( 588, 2, 0, 1 )
                     call iserrcrd
                  end if
                  go to 200

               end if

            end do

c           --- keyword not recognized

            call iserrchr ( string(1:nchar), 1 )
            do i = 1, nmain
               call iserrchr ( codemn(i), 1+i )
            end do
            do i = 1, nncode
               call iserrchr ( code(i), 1+i+nmain )
            end do
            call iserrsub ( 534, 0, 0, 1+nmain+nncode )
            call iserrcrd
            iretur = -1
            ready = .true.
            go to 100

c           --- keyword found, read next item

200         go to ( 100, 210, 210, 230, 210, 210, 240, 280, 100, 100,
     +              100, 300, 350, 300, 240, 400, 450, 500, 550, 400,
     +              600, 650, 700, 750, 800, 750, 850, 1050)
     +              itype(icode)+2

c           --- itype(icode) = 0, 1, 3, 4

210            call isasknxt
               if ( itype(icode).eq.1 .and. ityprd.eq.3 .or.
     +              ityprd.eq.2 .and. itype(icode).ge.0 ) then

c              --- Number of correct type found

                  if ( itype(icode).eq.4 ) then
                     iposut = iposut + 1
                     if ( iposut.gt.nout ) go to 950
                     tabout(iposut) = realrd
                     table(icode) = iposut
                  else
                     table(icode) = realrd
                  end if

c                 --- Check values

                  if ( itype(icode).eq.0 ) then

c                 --- Integer value read

                     minval = nint(tabmin(icode))
                     maxval = nint(tabmax(icode))
                     ivalrd   = nint(realrd)
                     if ( ivalrd.lt.minval .or. ivalrd.gt.maxval ) then
                        call iserrchr ( code(icode), 1 )
                        call iserrint ( ivalrd, 1 )
                        call iserrint ( minval, 2 )
                        call iserrint ( maxval, 3 )
                        call iserrsub ( 553, 3, 0, 1 )
                        call iserrcrd
                     end if

                  else

c                 --- Real value read

                     if ( realrd.lt.tabmin(icode) .or.
     +                  realrd.gt.tabmax(icode) ) then
                        call iserreal ( realrd, 1 )
                        call iserreal ( tabmin(icode), 2 )
                        call iserreal ( tabmax(icode), 3 )
                        call iserrsub ( 554, 0, 3, 0 )
                        call iserrcrd
                     end if

                  end if

               else if ( itype(icode).eq.0 ) then

c              --- Not an integer number found

                  call iserrchr ( code(icode), 1 )
                  call iserrsub ( 528, 0, 0, 1 )
                  call iserrcrd
                  ready = .true.

               else if ( itype(icode).eq.1 ) then

c              --- Not a real number found

                  call iserrchr ( code(icode), 1 )
                  call iserrsub ( 552, 0, 0, 1 )
                  call iserrcrd
                  ready = .true.

               else if ( itype(icode).eq.3 ) then

c              --- No number found, permitted in this case

                  readnx = .false.

               else if ( itype(icode).eq.4 ) then

c              --- No number found, permitted in this case

                  table(icode) = -1d3
                  readnx = .false.

               end if
               go to 100

c              --- itype(icode) = 2, Put 1 in table

230            table(icode) = 1d0
               go to 100

c              --- itype(icode) = 5 or 13, hence the keyword must be followed by
c                  one or two keywords

240            call isasknxt
               if ( ityprd.ne.4 ) then

c              --- Not a keyword found after this keyword

                  call iserrchr ( code(icode), 1 )
                  call iserrsub ( 555, 0, 0, 1 )
                  call iserrcrd

               else

c              --- keyword found, check if it is in the table

                  imin = tabmin(icode)
                  imax = tabmax(icode)
                  do i = imin, imax
                     if ( string(1:nmcode(i)).eq.
     +                    codemn(i)(1:nmcode(i)) ) then

c                    --- Text found

                        table(icode) = i-imin+1
                        if ( itype(icode).eq.5 ) then
                           go to 100
                        else
                           go to 260
                        end if

                     end if

                  end do

c                 --- keyword not recognized

                  call iserrchr ( string(1:nchar), 1 )
                  call iserrsub ( 534, 0, 0, 1 )
                  call iserrcrd
                  iretur = -1
                  ready = .true.
                  go to 100

c                 --- itype(icode) = 13, check if second keyword should be
c                     present

260               iminsv = imin
                  istartkey = itype(icode+1)
                  imin = itype(istartkey+(i-iminsv)*2)
                  if ( imin.eq.0 ) go to 100
                  imax = itype(istartkey+(i-iminsv)*2+1)

c                 --- Read next keyword

                  call isasknxt
                  if ( ityprd.ne.4 ) then

c                 --- Not a keyword found after this keyword

                     call iserrchr ( code(icode), 1 )
                     iminsv = nint(table(icode))+iminsv-1
                     call iserrchr ( codemn(iminsv), 2 )
                     call iserrsub ( 560, 0, 0, 2 )
                     call iserrcrd

                  else

c                 --- keyword found, check if it is in the table

                     do i = imin, imax
                        if ( string(1:nmcode(i)).eq.
     +                       codemn(i)(1:nmcode(i)) ) then

c                       --- Text found

                           table(icode+1) = i-imin+1
                           go to 100

                        end if

                     end do

c                    --- keyword not recognized

                     call iserrchr ( string(1:nchar), 1 )
                     call iserrsub ( 534, 0, 0, 1 )
                     call iserrcrd
                     iretur = -1
                     ready = .true.
                     go to 100

                  end if

               end if
               go to 100

c              --- itype(icode) = 6, text

280            call isasknxt
               if ( ityprd.ne.5 ) then

c              --- not a text found

                  call iserrchr ( code(icode), 1 )
                  call iserrsub ( 555, 0, 0, 1 )
                  iretur = -1
                  ready = .true.

               else

                  kposut = kposut + 1
                  if ( kposut.gt.nstrng ) go to 960
                  strout(kposut)(1:nchar) = string(1:nchar)
                  tabmax(icode) = nchar
                  table(icode) = kposut

               end if
               go to 100

c              --- itype(icode) = 10, hence the keyword must be followed by
c                  a sequence number and a number
c                  or itype(icode) = 12, the sequence number may be not present

300            call isasknxt
               if ( itype(icode).eq.10 .and. ityprd.ne.2 ) then

c              --- Not a sequence number found after the keyword

                  call iserrchr ( code(icode), 1 )
                  call iserrsub ( 556, 0, 0, 1 )
                  call iserrcrd
                  iretur = -1
                  ready = .true.
                  go to 100

               else if ( ityprd.ne.2 .and. ityprd.ne.3 ) then

c              --- Not a number found after the keyword

                  call iserrchr ( code(icode), 1 )
                  call iserrsub ( 552, 0, 0, 1 )
                  call iserrcrd
                  iretur = -1
                  ready = .true.
                  go to 100

               end if
               iseqkey = 1
               if ( ityprd.eq.2 ) iseqkey = nint(realrd)
               if ( itype(icode).eq.12 ) then

c              --- read next item if itype(icode) = 12

                  itypsv = ityprd
                  call isasknxt
                  if ( ityprd.ne.2 .and. ityprd.ne.3 ) then
                     readnx = .false.
                     iseqkey = 1
                  else
                     itypsv = ityprd
                  end if

               end if

c              --- Sequence number found, check if it is in the correct range

               iseqmn = nint(tabmin(icode))
               iseqmx = nint(tabmax(icode))
               if ( iseqkey.lt.iseqmn .or. iseqkey.gt.iseqmx ) then

                  call iserrchr ( code(icode), 1 )
                  call iserrint ( iseqkey, 1 )
                  call iserrint ( iseqmn, 2 )
                  call iserrint ( iseqmx, 3 )
                  call iserrsub ( 557, 3, 0, 1 )
                  call iserrcrd
                  iseqkey = iseqmn

               end if

c              --- Read next item

               if ( itype(icode).eq.10 ) then
                  call isasknxt
                  itypsv = ityprd
               end if
               if ( itypsv.eq.2 .or. itypsv.eq.3 .and.
     +              itype(icode+1).eq.1 ) then

                  table(icode+iseqkey-iseqmn) = realrd

c                 --- Check values

                  if ( itype(icode+1).eq.0 ) then

c                 --- Integer value read

                     minval = nint(tabmin(icode+1))
                     maxval = nint(tabmax(icode+1))
                     ivalrd   = nint(realrd)
                     if ( ivalrd.lt.minval .or. ivalrd.gt.maxval ) then
                        call iserrint ( ivalrd, 1 )
                        call iserrint ( minval, 2 )
                        call iserrint ( maxval, 3 )
                        call iserrchr ( code(icode), 1 )
                        call iserrsub ( 553, 3, 0, 1 )
                        call iserrcrd
                     end if

                  else

c                 --- Real value read

                     if ( realrd.lt.tabmin(icode+1) .or.
     +                  realrd.gt.tabmax(icode+1) ) then
                        call iserreal ( realrd, 1 )
                        call iserreal ( tabmin(icode+1), 2 )
                        call iserreal ( tabmax(icode+1), 3 )
                        call iserrsub ( 554, 0, 3, 0 )
                        call iserrcrd
                     end if

                  end if

               else

                  call iserrchr ( code(icode), 1 )
                  call iserrsub ( 558, 0, 0, 1 )
                  call iserrcrd

               end if
               go to 100

c              --- itype(icode) = 11, hence the keyword must be followed by
c                  a series of positive integers

350            num = 0
               table(icode) = iposut+1

c              --- Next read integer numbers until closing bracket or
c                  other type of separator is found

               close = .false.
360            if ( .not. close ) then

c              --- Close bracket not yet found

                  if ( readnx ) call isasknxt
                  readnx = .true.
                  if ( ityprd.eq.2 ) then

c                 --- Integer number found

                     ivalrd = nint(realrd)
                     maxval = nint(tabmax(icode))
                     if ( itype(icode).eq.11 ) then

c                    --- itype(icode) = 11, only positive integers

                        if ( ivalrd.lt.1 .or. ivalrd.gt.maxval ) then

c                       --- Error integer out of range

                           call iserrint ( ivalrd, 1 )
                           call iserrint ( 1, 2 )
                           call iserrint ( maxval, 3 )
                           call iserrchr ( code(icode), 1 )
                           call iserrsub ( 553, 3, 0, 1 )
                           call iserrcrd

                        end if

                     else

c                    --- itype(icode) = 20, positive and negative integers

                        if ( ivalrd.lt.1 .or. ivalrd.gt.maxval ) then

c                       --- Error integer out of range

                           call iserrint ( ivalrd, 1 )
                           call iserrint ( -maxval, 2 )
                           call iserrint ( maxval, 3 )
                           call iserrchr ( code(icode), 1 )
                           call iserrsub ( 553, 3, 0, 1 )
                           call iserrcrd

                        end if

                     end if
                     num = num + 1
                     ihelp(icode) = num
                     iposut = iposut + 1
                     if ( iposut.gt.nout ) go to 950
                     tabout(iposut) = ivalrd

                  else if ( ityprd.eq.7 ) then

c                 --- end bracket found

                     close = .true.
                     inumlc = nint(tabmin(icode))
                     if ( num.eq.0 .or. num.gt.inumlc ) then

c                    --- Too much numbers read

                        call iserrint ( nncode, 1 )
                        call iserrint ( 1, 2 )
                        call iserrint ( inumlc, 3 )
                        call iserrsub ( 559, 3, 0, 0 )

                     end if

                  else

c                 --- new item

                     readnx = .false.
                     go to 100

                  end if
                  go to 360

               end if
               go to 100

c              --- itype(icode) = 14, 18 hence the keyword must be followed by
c                  a positive integer and possibly the same keyword with another
c                  integer and so on

400            if ( itype(icode).eq.18 ) then

c              --- the keyword must be followed by the next keyword
c                  possibly provided with minus sign

                  ih = nint(-table(icode))
                  if ( ih.lt.0 ) then
                     call iserrchr ( code(icode), 1 )
                     call iserrsub ( 547, 0, 0, 1 )
                     call iserrcrd
                     go to 100
                  end if
                  subkey = codemn(ih)
                  numkey = nmcode(ih)
                  call isasknxt
                  if ( ityprd.eq.6 ) then

c                 --- minus sign found

                     minusg = -1
                     call isasknxt

                  end if
                  if ( ityprd.ne.4 ) then

c                 --- Not a keyword found

                     call iserrchr ( code(icode), 1 )
                     call iserrchr ( subkey, 2 )
                     call iserrsub ( 560, 0, 0, 2 )
                     call iserrcrd
                     ready = .true.

                  else if ( string(1:numkey).ne.subkey(1:numkey) ) then

c                 --- incorrect keyword found

                     call iserrchr ( code(icode), 1 )
                     call iserrchr ( string(1:nchar), 2 )
                     call iserrchr ( subkey, 3 )
                     call iserrsub ( 542, 0, 0, 3 )
                     call iserrcrd
                     ready = .true.

                  end if

               else

c              --- icode = 14, set subkey

                  subkey = code(icode)
                  numkey = ncode(icode)

               end if
               close = .false.
               num = 0
               readnx = .false.
               if ( iposut.gt.nout ) go to 950
               table(icode) = iposut+1
410            if ( .not. close ) then

c              --- End of items not yet found

                  call isasknxt
                  if ( ityprd.eq.2 ) then

c                 --- Integer number found

                     ivalrd = nint(realrd)
                     if ( ivalrd.lt.1 .or. ivalrd.gt.tabmax(icode) )
     +                  then

c                    --- Error integer out of range

                        call iserrint ( ivalrd, 1 )
                        call iserrint ( 1, 2 )
                        call iserrint ( nint(tabmax(icode)), 3 )
                        call iserrchr ( code(icode), 1 )
                        call iserrsub ( 553, 3, 0, 1 )
                        call iserrcrd

                     end if
                     num = num + 1
                     iposut = iposut + 1
                     if ( iposut.gt.nout ) go to 950
                     tabout(iposut) = minusg * ivalrd

c                    --- Read next item

                     minusg = 1
                     call isasknxt
                     if ( ityprd.eq.6 ) then
                        minusg = -1
                        call isasknxt
                     end if
                     if ( ityprd.eq.7 ) call isasknxt
                     if ( ityprd.eq.4 ) then
                        if (string(1:ncode(i)).ne.subkey(1:numkey)) then

c                       --- String not found

                           close = .true.
                           readnx = .false.

                        end if

                     else

c                    --- not a string

                        close = .true.
                        readnx = .false.

                     end if
                     go to 410

                  else

c                 --- Not an integer number found after the keyword

                     call iserrchr ( code(icode), 1 )
                     call iserrsub ( 528, 0, 0, 1 )
                     call iserrcrd
                     ready = .true.

                  end if

               end if
               ihelp(icode) = num
               go to 100

c              --- itype(icode) = 15, hence the keyword must be followed by
c                  an integer. This situation may be repeated

450            if ( ihelp(icode).eq.0 ) then

c              --- ihelp(icode) = 0, keyword has been found for the
c                  first time

                  table(icode) = iposut + 1
                  iposut = iposut + nint(tabmin(icode))
                  if ( iposut.gt.nout ) go to 950

               end if

               call isasknxt
               if ( ityprd.eq.2 ) then

c              --- Integer number found

                  ivalrd = nint(realrd)
                  maxval = nint(tabmax(icode))
                  if ( ivalrd.lt.1 .or. ivalrd.gt.maxval ) then

c                 --- Error integer out of range

                     call iserrint ( ivalrd, 1 )
                     call iserrint ( 1, 2 )
                     call iserrint ( maxval, 3 )
                     call iserrchr ( code(icode), 1 )
                     call iserrsub ( 553, 3, 0, 1 )
                     call iserrcrd

                  end if
                  tabout(nint(table(icode))+ihelp(icode)) =
     +               minusg*ivalrd
                  ihelp(icode) = ihelp(icode) + 1

               else

c              --- Not an integer number found after the keyword

                  call iserrchr ( code(icode), 1 )
                  call iserrsub ( 528, 0, 0, 1 )
                  call iserrcrd
                  ready = .true.

               end if
               go to 100

c              --- itype(icode) = 16, hence the keyword must be followed by
c                                     a value or value = v, or func = i

500            call isreadfn ( ifunc, value )
               if ( ifunc.gt.0 .or. ifunc.lt.-100 ) then

c              --- function found

                  table(icode) = ifunc

               else

c              --- value found

                  iposut = iposut+1
                  if ( iposut.gt.nout ) go to 950
                  table(icode) = -iposut
                  tabout(iposut) = value

               end if
               go to 100

c              --- itype(icode) = 17, hence the keyword must be followed by
c                                     a series of series of numbers

550            nsubbrack = nint(tabmin(icode))
               nmax = nint(tabmax(icode))
               num = 0
               close = .false.
               table(icode) = iposut+1
               readnx = .false.
560            if ( .not. close ) then

c              --- end of series has not yet been found
c                  Read next series

                  call isintx04 ( tabout(iposut+1), nsubbrack+1,
     +                            inumlc )
                  if ( inumlc.eq.0 ) then

c                 --- end of series found

                     if ( num.eq.0 ) then

c                    --- No series found

                        call iserrchr ( code(icode), 1 )
                        call iserrsub ( 540, 0, 0, 1 )
                        call iserrcrd
                        ihelp(icode) = 0

                     else

c                    --- Fill table

                        ihelp(icode) = num * nsubbrack

                     end if
                     close = .true.

                  else

c                 --- next series found

                     if ( inumlc.ne.nsubbrack ) then

c                    --- Not exactly nsubbrack numbers between brackets

                        call iserrint ( inumlc, 1 )
                        call iserrint ( nsubbrack, 2 )
                        call iserrsub ( 541, 2, 0, 0 )
                        call iserrcrd

                     end if
                     iposut = iposut + nsubbrack
                     if ( iposut.gt.nout ) go to 950
                     num = num + 1
                     if ( num.gt.nmax ) then

c                    --- Too much series of numbers read

                        call iserrchr ( code(icode), 1 )
                        call iserrint ( num, 1 )
                        call iserrint ( nmax, 2 )
                        call iserrsub ( 543, 2, 0, 1 )
                        call iserrcrd
                        ready = .true.

                     end if

                  end if
                  go to 560

               end if
               readnx = .false.
               go to 100

c              --- itype(icode) = 19, hence the keyword must be followed by
c                                     a series of numbers

600            nmax = nint(tabmin(icode))
               table(icode) = iposut+1
               call isintx04 ( tabout(iposut+1), nmax+1, inumlc )
610            if ( inumlc.eq.0 .or. inumlc.gt.nmax ) then

c              --- Too few or too much numbers found after keyword

                  call iserrchr ( code(icode), 1 )
                  call iserrint ( inumlc, 1 )
                  call iserrint ( nmax, 2 )
                  call iserrsub ( 544, 2, 0, 1 )
                  call iserrcrd

               end if
               do i = 1, inumlc
                  if ( abs(tabout(iposut+i)).gt.tabmax(icode) ) then
                     call iserrchr ( code(icode), 1 )
                     call iserrint ( i, 1 )
                     call iserreal ( tabout(iposut+i), 1 )
                     call iserreal ( tabmax(icode), 2 )
                     call iserrsub ( 545, 1, 2, 1 )
                     call iserrcrd
                  end if
               end do
               iposut = iposut+inumlc
               ihelp(icode) = inumlc
               if ( iposut.gt.nout ) go to 950
               readnx = .false.
               go to 100

c              --- itype(icode) = 20, hence the keyword is a single keyword in
c                                     which case the table must be set to
c                  -100000
c                                     or the keyword is followed by a given
c                  other
c                                     keyword and a series of integers
c                  Read next item

650            call isasknxt
               if ( ityprd.ne.4 ) then

c              --- not a keyword found, hence single keyword situation

                  table(icode) = -100000
                  readnx = .false.
                  go to 100

               else

c              --- keyword found, check if it is the special one

                  ih = nint(-table(icode))
                  if ( ih.lt.0 ) then
                     call iserrchr ( code(icode), 1 )
                     call iserrsub ( 547, 0, 0, 1 )
                     call iserrcrd
                     go to 100
                  end if
                  subkey = codemn(ih)
                  numkey = nmcode(ih)
                  if ( string(1:numkey).ne.subkey(1:numkey) ) then

c                 --- not special keyword found, hence single keyword situation

                     table(icode) = -100000
                     readnx = .false.
                     go to 100

                  else

c                 --- Special keyword found, read integers
c                     This is the situation of itype(icode) = 11, except
c                     for positiveness
c                     Jump to itype(icode) = 11

                     go to 350

                  end if

               end if

c              --- itype(icode) = 21, hence the keyword is followed by
c                                     (ax+by+cz=d)
c                  Read next item, the equals sign is treated as special symbol

700            table(icode) = iposut+1
               found = .false.
               do i = 1, 4
                  tabout(iposut+i) = 0d0
               end do
720            if ( found ) ispecl = 8
               call isasknxt
               if ( ityprd.eq.6 ) then
                  value = -1d0
               else
                  value = 1d0
               end if
               if ( ityprd.eq.2 .or. ityprd.eq.3 ) then

c              --- number found, read next item (must be text)

                  value = realrd
                  found = .true.
                  ispecl = 8
                  call isasknxt

               end if
               if ( ityprd.ne.4 .and. ityprd.ne.9 ) then

c              --- next item not text or equals sign

                  call iserrchr ( code(icode), 1 )
                  call iserrsub ( 536, 0, 0, 1 )
                  call iserrcrd
                  go to 100

               else if ( ityprd.eq.4 ) then

                  if ( string(1:1).eq.'X' ) then
                     iseqkey = 1
                  else if ( string(1:1).eq.'Y' ) then
                     iseqkey = 2
                  else if ( string(1:1).eq.'Z' ) then
                     iseqkey = 3
                  else

c                 --- text not recognized

                     call iserrchr ( string(1:nchar), 1 )
                     call iserrchr ( code(icode), 2 )
                     call iserrsub ( 537, 0, 0, 2 )
                     call iserrcrd
                     go to 100

                  end if
                  found = .true.
                  ispecl = 8
                  if ( nint(tabout(iposut+iseqkey)).ne.0 ) then

c                 --- text is used repeatedly

                     call iserrchr ( code(icode), 1 )
                     call iserrchr ( string(1:1), 2 )
                     call iserrsub ( 546, 0, 0, 2 )
                     call iserrcrd
                     go to 100

                  end if
                  tabout(iposut+iseqkey) = value
                  go to 720

               else if ( ityprd.eq.9 ) then

c              --- equals sign found, hence one number remains to be read

                  ispecl = 4
                  call isasknxt
                  if ( ityprd.ne.2 .and. ityprd.ne.3 ) then

c                 --- Not a number found

                     call iserrchr ( code(icode), 1 )
                     call iserrsub ( 552, 0, 0, 1 )
                     call iserrcrd

                  else

c                 --- Last number read

                     tabout(iposut+4) = realrd
                     iposut = iposut+ 4
                     if ( iposut.gt.nout ) go to 950

                  end if
                  go to 100

               else

c              --- text not recognized

                  call iserrsub ( 551, 0, 0, 0 )
                  call iserrcrd
                  go to 100

               end if

c              --- itype(icode) = 22, 24 hence the keyword is followed by
c                  --- possibly
c                                        one or two special keyword that are
c                                        skipped and then by a series of numbers

750            call isasknxt
               irep = 0
760            if ( ityprd.eq.4 ) then

c              --- keyword found, check if it is the special one

                  ih = nint(-table(icode))
                  if ( ih.lt.0 ) then
                     call iserrchr ( code(icode), 1 )
                     call iserrsub ( 547, 0, 0, 1 )
                     call iserrcrd
                     go to 100
                  end if
                  subkey = codemn(ih+irep)
                  numkey = nmcode(ih+irep)
                  if ( string(1:numkey).ne.subkey(1:numkey) ) then

c                 --- string not recognized

                     call iserrchr ( code(icode), 1 )
                     call iserrchr ( string(1:nchar), 2 )
                     call iserrchr ( subkey, 3 )
                     call iserrsub ( 542, 0, 0, 3 )
                     call iserrcrd
                     go to 100

                  end if
                  irep = irep + 1
                  call isasknxt
                  if ( irep.eq.1 ) go to 760

               end if

c              --- The rest of the record should contain the numbers to be
c                  read. In fact this is the same situation as for
c                  itype(icode) = 19, so we jump to that part of the code
c                  or in the case 24 as 11, so we jump to that part of the code

               if ( itype(icode).eq.24 ) then
                  readnx = .false.
                  go to 350
               end if
               nmax = nint(tabmin(icode))
               table(icode) = iposut+1
               tabout(iposut+1) = realrd
               if ( ityprd.eq.2 .or. ityprd.eq.3 ) then
                  call isintx04 ( tabout(iposut+2), nmax, inumlc )
                  inumlc = inumlc + 1
               else
                  inumlc = 0
               end if
               go to 610

c              --- itype(icode) = 23, hence the keyword is followed by possibly
c                                     a special keyword that is skipped
c                                     table(icode) is set equal to 1

800            call isasknxt
               ih = nint(-table(icode))
               if ( ih.lt.0 ) then
                  call iserrchr ( code(icode), 1 )
                  call iserrsub ( 547, 0, 0, 1 )
                  call iserrcrd
                  go to 100
               end if
               subkey = codemn(ih)
               numkey = nmcode(ih)
               if ( ityprd.eq.4 ) then
                  if ( string(1:numkey).ne.subkey(1:numkey) )
     +                  readnx = .false.
               else
                  readnx = .false.
               end if
               table(icode) = 1
               go to 100

         end if
         go to 100

c        --- itype(icode) = 25, hence the keyword must be followed by
c                               a sequence number followed by
c                               a value or value = v, or func = i

850         call isasknxt
            if ( ityprd.ne.2 ) then

c           --- Not a sequence number found after the keyword

               call iserrchr ( code(icode), 1 )
               call iserrsub ( 556, 0, 0, 1 )
               call iserrcrd
               iretur = -1
               ready = .true.
               go to 100

            end if
            iseqkey = 1
            if ( ityprd.eq.2 ) iseqkey = nint(realrd)

c           --- Sequence number found, check if it is in the correct range

            iseqmn = nint(tabmin(icode))
            iseqmx = nint(tabmax(icode))
            if ( iseqkey.lt.iseqmn .or. iseqkey.gt.iseqmx ) then

               call iserrchr ( code(icode), 1 )
               call iserrint ( iseqkey, 1 )
               call iserrint ( iseqmn, 2 )
               call iserrint ( iseqmx, 3 )
               call iserrsub ( 557, 3, 0, 1 )
               call iserrcrd
               iseqkey = iseqmn

            end if

c           --- Read next item

            call isreadfn ( ifunc, value )
            if ( ifunc.gt.0 .or. ifunc.lt.-100 ) then

c           --- function found

               table(icode+iseqkey-iseqmn) = ifunc

            else

c           --- value found

               iposut = iposut+1
               if ( iposut.gt.nout ) go to 950
               table(icode+iseqkey-iseqmn) = -iposut
               tabout(iposut) = value

            end if
            go to 100

c           --- itype(icode) = 26, hence the keyword must be followed by
c                                  a sequence number followed by
c                                  a value or value = v, or func = i
c                                  and again the same keyword followed  by
c                                  a sequence number followed by
c                                  a value or value = v, or func = i

1050         call isasknxt
            if ( ityprd.ne.2 ) then

c           --- Not a sequence number found after the keyword

               call iserrchr ( code(icode), 1 )
               call iserrsub ( 556, 0, 0, 1 )
               call iserrcrd
               iretur = -1
               ready = .true.
               go to 100

            end if
            iseqkey = 1
            if ( ityprd.eq.2 ) iseqkey = nint(realrd)

c           --- Sequence number found, check if it is in the correct range

            iseqmn = nint(tabmin(icode))
            iseqmx = nint(tabmax(icode))
            if ( iseqkey.lt.iseqmn .or. iseqkey.gt.iseqmx ) then

               call iserrchr ( code(icode), 1 )
               call iserrint ( iseqkey, 1 )
               call iserrint ( iseqmn, 2 )
               call iserrint ( iseqmx, 3 )
               call iserrsub ( 557, 3, 0, 1 )
               call iserrcrd
               iseqkey = iseqmn

            end if

            if ( iseqkey.eq.1) then
               seqk1 = .true.
            else if ( iseqkey.eq.2 ) then
               seqk2 = .true.
            end if

c           --- Read next item

            if( seqk1 .and. .not. seqk2   .or.
     +          seqk2 .and. .not. seqk1) then
               iposut = iposut + 1
               table(icode) = iposut
               do i=1,4
                  tabout(iposut-1+i) =0d0
               end do
            end if

            call isreadfn ( ifunc, value )
            if ( ifunc.gt.0 .or. ifunc.lt.-100 ) then

c           --- function found

               tabout(iposut+(iseqkey-1)*2) = ifunc

            else

c           --- value found

               tabout(iposut+(iseqkey-1)*2) = -1
               if ( iposut.gt.nout ) go to 950
               tabout(iposut+1+(iseqkey-1)*2) = value

            end if

            if ( seqk1 .and. seqk2 ) then
               iposut = iposut + 3
            end if

            goto 100

      end if

      do i = 1, nncode

         if ( ihelp(i).gt.0 ) then

c        --- ihelp(i) > 0, fill tabmax(i)

            tabmax(i) = ihelp(i)
            if ( ihelp(i).gt.nint(tabmin(i)) ) then

c           --- Error: more numbers found than permitted

               call iserrchr ( code(icode), 1 )
               call iserrint ( ihelp(i), 1 )
               call iserrint ( nint(tabmin(i)), 2 )
               call iserrsub ( 539, 2, 0, 1 )
               call iserrcrd

            end if

         else if ( itype(i).eq.11 .or. itype(i).eq.14 .or.
     +             itype(i).eq.15 .or. itype(i).ge.17 .and.
     +             itype(i).le.20 .or. itype(i).eq.22 .or.
     +             itype(i).eq.24 ) then

            tabmax(i) = 0d0

         end if

      end do

      go to 1000

c     --- Error: tabout too small

950   call iserrint ( nout, 1 )
      call iserrsub ( 548, 1, 0, 0 )
      go to 1000

c     --- Error: strout too small

960   call iserrint ( nstrng, 1 )
      call iserrsub ( 549, 1, 0, 0 )

1000  call iserclos ( 'isintx02' )
      if ( ierror.ne.0 ) iretur = -1
      ispecl = ispecs

      end
