      subroutine isprmain ( nbuffr )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.12   date 21-07-1999 Guus: initialize isicurvnf
c        version 1.11   date 27-05-1999 Guus: new call to iswrmpi
c        version 1.10   date 18-01-1999 Guus: Only write isnas.mpi.script if
c                                             no errors
c        version 1.9    date 08-09-1998 Kees K: call of isreadsp with
c                                               indorien
c        version 1.8    date 28-01-1998 Guus: Jump to end in case of error
c        version 1.7    date 10-10-1996 Guus: new call to isprmain
c        version 1.6    date 25-06-1996 Guus: introduction indorien
c        version 1.5    date 07-03-1996 Guus: Remove mapar
c        version 1.4    date 02-10-1995 Guus: Correction length isfread
c        version 1.3    date 13-09-1995 Guus: Allow for unformatted files
c        version 1.2    date 13-07-1995 Guus: remove file isnasinp.cmp
c        version 1.1    date 08-02-1995 Kees K. 3D
c        version 1.0    date 28-07-1994
c
c
c   copyright (c) 1994-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    General ISNaS Pre-processor.  Given an ISNaS grid and an ISNaS input file
c    the intermediate file is created.
c    Restriction: at this moment the pre-processor recognizes only SEPRAN
c    and LISS grid files or files that are made according to exactly
c    the same definitions
c **********************************************************************
c
c                       KEYWORDS
c
c    pre_processor
c    read
c    read_cards
c    read_mesh
c    write
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer nbuffr

c     nbuffr    Length of buffer array ibuffr
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cseprd'
      include 'IScommon/csepch'
      include 'IScommon/cisnas'
      include 'IScommon/cisblank'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer maxblocki, maxtrans, maxturb
      parameter ( maxblocki = 100, maxtrans=100, maxturb=10 )
      integer isiinput(maxblocki), iscotmp(maxblocki),
     +        indorien(maxblocki)
      integer iblock, ipicurvnf, ipiinbc, ipineigh, ipisfread, ipitopol,
     +        ipiuser,
     +        ipkmeshq, ipnsubf, iprinbc, ipsubf, iptemp, ipuser,
     +        irefsv, isicurvnf, isiinbc, i, nmachines,
     +        isineigh, isisfread, isitopol, isiuser, iskmeshq, isrinbc,
     +        isuser, leniinbc, lenrinbc, maxnsubf,
     +        maxuser, nblocks, ncurvs, ndim, nfaces
      logical sepran, check

c     check     Indication if a file has been opened (true) or not (false)
c     i         General loop variable
c     iblock    Block sequence number
c     indorien  In this array it is stored if the block orientation is
c               positive (indorien(iblock)=1) or
c               negative(indorien(iblock)=-1)
c     ipicurvnf Starting address of array icurvnf in IBUFFR
c     ipiinbc   Starting address of array iinbc in array iinput
c     ipineigh  Starting address of array ineigh in IBUFFR
c     ipisfread Address of the isfread array in ibuffr
c     ipitopol  Starting address of array itopol in ibuffr
c     ipiuser   Starting address of array iuser in IBUFFR
c     ipkmeshq  Starting address of SEPRAN array kmeshq in IBUFFR
c     ipnsubf   Starting address of the nsubfaces array
c     iprinbc   Starting address of array rinbc in array iinput
c     ipsubf    Starting address of the subfaces array
c     iptemp    Starting address of temporary array in IBUFFR
c     ipuser    Starting address of array user in BUFFER
c     irefsv    Help variable to save the value of a reference number
c     iscotmp   Array of sequence numbers (nblocks) of array cotmp
c     isicurvnf Sequence number of array icurvnf
c     isiinbc   Sequence number of array iinbc
c     isiinput  Array of sequence numbers (nblocks) of array iinput
c     isineigh  Sequence number of array ineigh
c     isisfread Sequence number of the isfread array
c     isitopol  Sequence number of array itopol
c     isiuser   Sequence number of array iuser
c     iskmeshq  Reference to SEPRAN array kmeshq
c     isrinbc   Sequence number of array rinbc
c     isuser    Sequence number of array user
c     leniinbc  Computed length of array iinbc
c     lenrinbc  Computed length of array rinbc
c     maxblocki Maximum number of blocks
c     maxnsubf  Maximum number of subfaces encountered on all subfaces
c     maxtrans  Maximum number of transport equations allowed
c     maxturb   Maximum number of turbulence equations allowed
c     maxuser   Maximal length of array user
c     nblocks   Number of blocks
c     ncurvs    Number of curves (SEPRAN environment)
c     ndim      Dimension of space (2 or 3)
c     nfaces    Number of faces
c     nmachines Number of nodes that are used
c     sepran    If true the grid is created by SEPRAN
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isingetp, isdpgetp

c     ISACTIVE  Activate old permanent array
c     ISASKNXT  Read next item from the standard input file
c     ISCOMLBC  Compute maximal length of arrays iinbc and rinbc as
c               function of the topology
c     ISDCREAT  Create permanent space in array IBUFFR (Double precision array)
c     ISDPGETP  get the pointer corresponding to a double precision array
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRCHR  Puts values of strings in character error array
c     ISERRSUB  Error messages
c     ISHSTART  Start ISNaS host program, store machine dependent quantities
c     ISICREAT  Create permanent space in array IBUFFR (Integer array)
c     ISINGETP  get the pointer corresponding to an integer array
c     ISINIFIL  Open files
c     ISINITBC  Initialize array iinbc with default information of boundaries
c     ISINTMDF  Read user input files and store result in output arrays
c               Write output arrays to intermediate file
c     ISRDGRID  Read the grid input file describing completely the subdomain
c               grids and block topology
c     ISREADSP  Read last part of the grid file finvol.new in case this file is
c               created by SEPRAN
c     ISREMFIL  Remove file
c     ISSTOP    Stop ISNAS
c     ISTMSPAC  Create space for temporary array
c     ISWRMPI   Write the isnas.mpi.script file
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     526   The first item read in the grid input file is not a keyword.
c     527   The first keyword read in the grid input file is incorrect
c **********************************************************************
c
c                       PSEUDO CODE
c
c     Start ISNaS including reading environment file
c     Read file with grid information finvol.new
c     Create arrays for output
c     Read user input file and store result in output arrays
c     Write output arrays to intermediate file
c     Close program
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data indorien / maxblocki*1 /
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- Remove the file isnas.mpi.script

      inquire ( file = 'isnas.mpi.script', exist = check )
      if ( check ) then
         open ( unit = 93, file = 'isnas.mpi.script' )
         close ( unit  = 93, status = 'delete' )
      end if

c     --- Start ISNaS including reading environment file
c         initialize all pointers to zero

      isitopol = 0

      do 100 iblock = 1, maxblocki

         isiinput( iblock ) = 0

100   continue

      call ishstart ( nbuffr )
      call iseropen ( 'isnaspre' )

c     --- Read file with grid information finvol.new

      call isinifil ( -10 )
      if ( nerror.gt.0 ) go to 1000

c     --- Remove file isnasinp.cmp

      call isremfil ( 75 )
      irefsv = irefre
      irefre = abs(iref10)

c     --- Start with reading first keyword

      if ( iref10.gt.0 ) then

c     --- Formatted case

         ityprd = 1
         call isasknxt
         if ( ityprd.ne.4 ) then

c        --- The first item in the grid file is not a keyword

            call iserrsub ( 526, 0, 0, 0 )

         end if

      else

c     --- Unformatted case

         read(irefre) string(1:6)

      end if
      if ( nerror.gt.0 ) go to 1000

c     --- Keyword found, check if it is the correct one

      if ( string(1:6).eq.'SEPRAN' ) then
         sepran = .true.
      else if ( string(1:4).eq.'LISS' .or.
     +          string(1:5).eq.'ISNAS' ) then
         sepran = .false.
      else
         call iserrchr ( string(1:6), 1 )
         call iserrsub ( 527, 0, 0, 1 )
      end if
      if ( nerror.gt.0 ) go to 1000
      ileft  = 1
      iright = 2
      idown  = 3
      iup    = 4
      call isrdgrid ( maxblocki, ndim, nblocks, isiinput, iscotmp,
     +                isitopol, indorien )
      if ( nerror.gt.0 ) go to 1000

      iskmeshq = 0
      if ( sepran ) then

c     --- In case of sepran read array ineigh

         call isreadsp ( ndim, ncurvs, isineigh, iskmeshq , indorien)

      else

c     --- no sepran so set ncurvs = 0 and create ineigh array
c         of length 1 and contents 0

         ncurvs = 1
         isineigh = 0
         call isicreat(isineigh, 1, 'ineigh')
         ipineigh = isingetp(isineigh)
         ibuffr(ipineigh) = 0

      end if

      if ( iskmeshq.eq.0 ) then

c     --- Array kmeshq has not been filled
c         Create vector of length 1 with contents 0

         call isicreat ( iskmeshq, 1 , 'kmeshq' )
         ipkmeshq = isingetp(iskmeshq)
         ibuffr(ipkmeshq) = 0

      end if

c     --- close grid input file

      close(irefre)
      irefre = irefsv

c     --- Create arrays for output

      isiinbc = 0
      isrinbc = 0
      isiuser = 0
      isuser  = 0
      isicurvnf = 0
      isisfread = 0
      maxuser = 10000
      call isactive ( isitopol )
      ipitopol = isingetp(isitopol)
      nfaces   = ibuffr(ipitopol + 2 - 1)
      maxnsubf = ibuffr(ipitopol + 3 - 1)

c     --- Compute length of arrays iinbc and rinbc

      call iscomlbc ( ibuffr(ipitopol), leniinbc, lenrinbc,
     +                maxtrans+maxturb, nfaces, maxnsubf )
      call isicreat ( isiinbc, leniinbc , 'iinbc' )
      call isdcreat ( isrinbc, lenrinbc , 'rinbc' )

c     --- create other arrays needed for reading the problem file

      call isicreat ( isiuser, maxuser , 'iuser' )
      call isdcreat ( isuser, maxuser , 'user' )
      call isicreat ( isicurvnf, 4*ncurvs , 'icurvnf' )
      call isicreat ( isisfread, nblocks*nfaces*maxnsubf*10, 'isfread' )
      call istmspac ( max(nblocks*maxnsubf,ncurvs), iptemp )
      if ( nerror.gt.0 ) go to 1000

c     --- initialize array rinbc

      iprinbc   = isdpgetp(isrinbc)
      do i = 0, lenrinbc-1
         buffer(iprinbc+i) = 0
      end do

c     --- initialize array iinbc

      ipiinbc   = isingetp(isiinbc)
      ipitopol  = isingetp(isitopol)
      ipineigh  = isingetp(isineigh)
      ipicurvnf = isingetp(isicurvnf)
      ipnsubf   = ipitopol+ibuffr(ipitopol+4)-1
      ipsubf    = ipitopol+ibuffr(ipitopol+5)-1
      call isinitbc ( ibuffr(ipiinbc), nblocks, nfaces, maxnsubf,
     +                ibuffr(ipnsubf), ibuffr(ipsubf),
     +                maxtrans+maxturb, ibuffr(ipicurvnf),
     +                ibuffr(ipineigh), ncurvs, ibuffr(iptemp), ndim,
     +                sepran )
      if ( nerror.gt.0 ) go to 1000
      iblockrf = 1

c     --- Read user input file and store result in output arrays
c         Write output arrays to intermediate file

      ipiinbc  = isingetp(isiinbc)
      iprinbc  = isdpgetp(isrinbc)
      ipineigh = isingetp(isineigh)
      ipisfread= isingetp(isisfread)
      ipitopol = isingetp(isitopol)
      ipiuser  = isingetp(isiuser)
      ipuser   = isdpgetp(isuser)
      ipkmeshq = isingetp(iskmeshq)
      call isintmdf ( ndim, nblocks, ncurvs, ibuffr(ipiinbc),
     +                buffer(iprinbc), ibuffr(ipiuser), buffer(ipuser),
     +                ibuffr(ipicurvnf), ibuffr(ipkmeshq),
     +                ibuffr(ipisfread), ibuffr(ipitopol),
     +                ibuffr(ipineigh),  ibuffr(iptemp),
     +                sepran, maxtrans, maxblocki, maxturb, indorien,
     +                nmachines )

c     --- Close program

1000  call iserclos ( 'isnaspre' )
      if ( nerror.eq.0 ) call iswrmpi ( nmachines, machin )
      call isstop
      end
