      subroutine isrdbou2 ( nsfread, isfread, iworkgen, ncurvs,
     +                      icurvinf, kmeshq, ineigh, ndim,
     +                      code, ncode, nncode, itype, table, codemn,
     +                      nmcode, nmain, iretur, tabmin, tabmax,
     +                      tabout, nout, strout, nstrng, ncodmx )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.3    11-10-1995 Guus: correction for negative subcurves
c        version 1.2    08-02-1995 Kees K. 3D
c        version 1.1    09-11-1994 Erik: the code in this file has
c                                     been taken from an old
c                                     version of isrdboun
c        version 1.0    28-07-1994
c
c
c   copyright (c) 1994-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Read a set of curve numbers.
c    Such a set can be described in two ways:
c          CURVE <curvenumber>
c    or
c          CURVE <curvenumber> to <curvenumber>
c    On input to this subroutine the 'CURVE' keyword has already been
c    read.
c    On output from this subroutine, the next main keyword is stored in
c    iretur.
c **********************************************************************
c
c                       KEYWORDS
c
c       boundary_conditions
c       read
c       read_cards
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer nsfread, isfread(10,*), iworkgen(*), ncurvs, ndim,
     +        icurvinf(4,ncurvs), kmeshq(*),
     +        ineigh(10+(ndim-2)*4,ncurvs)
      integer nncode, ncode(nncode), itype(*), nmain, nmcode(*), iretur,
     +        nout, nstrng, ncodmx(nncode)
      character *(*) code(nncode), codemn(*), strout(nstrng)
      double precision table(nncode), tabmin(nncode), tabmax(nncode),
     +                 tabout(nout)

c     code    i    Table containing the subkeywords corresponding to this
c                  subroutine.
c     codemn  i    Table of main keywords
c     icurvinf  i    reserved
c     ineigh  i    reserved
c     iretur i/o   Return code, indicates the sequence number of the new
c                  main keyword found
c                  If -1 an error has occurred
c                  If -2 the end of file has been reached
c     isfread o    reserved
c     itype   i    Array containing the type of actions to be undertaken for
c                  each subkeyword found.
c                  See subroutine isintx02 for the possible values of itype
c     iworkgen i/o General integer work array.
c                  See pseudo code for the contents
c     kmeshq  i    SEPRAN array KMESH part q. Contains information about
c                  curves of curves. See the SEPRAN Programmers Guide.
c     ncode   i    Array containing the number of significant characters for
c                  each keyword in code
c     ncodmx  i    Array containing the maximum number of characters for
c                  each keyword in code
c     ncurvs  i    Number of curves (SEPRAN environment)
c     ndim    i    Dimension of space (2 or 3)
c     nmain   i    Number of keywords in array codemn
c     nmcode  i    Array containing the number of significant characters
c                  for each main keyword in codemn
c     nncode  i    Number of subkeywords stored in array code
c     nout    i    Length of array tabout
c     nsfread i    Number of subfaces read
c     nstrng  i    Length of array strout
c     strout  o    Contains series of strings, see itype
c     table  i/o   Array containing the real or integer values corresponding to
c                  the subkeywords in code that have been read by isintx02
c     tabmax  i    Array containing the maximal values of the parameters read
c                  corresponding to the subkeywords in code
c     tabmin  i    Array containing the minimum values of the parameters read
c                  corresponding to the subkeywords in code
c     tabout  o    Array of output reals provided by isintx02
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cseprd'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, icurve, icurve1, icurve2, nsubcurvs

c     i         General loop variable
c     icurve    SEPRAN curve sequence number
c     icurve1   First curve number of a set
c     icurve2   Last curve number of a set
c     nsubcurvs Number of subcurves of a curve
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISASKNXT  Read next item from the standard input file
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRCHR  Puts values of strings in character error array
c     ISERRCRD  Prints the last record read and underlines the erroneous part
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c     ISINTX02  Interpret a record containing keywords followed by numbers
c **********************************************************************
c
c                       I/O
c
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     528   No integer number has been found after the keyword %
c     530   integer number read out of range
c     570   Boundary conditions prescribed at inner curve
c     579   Curve number has been used more than once to prescribe
c           boundary conditions
c **********************************************************************
c
c                       PSEUDO CODE
c
c    icurve1 := 0
c
c    get curve number icurve1
c
c    read boundary conditions and possibly 'TO' keyword
c
c    if ( 'TO' keyword present ) then
c
c       set icurve2
c
c    else
c
c       set icurve2 = icurve1
c
c    end if
c
c    fill array iworkgen and check if each curve is used at most one time
c
c    check if all curves are outer curves
c
c    fill isfread array with the resulting subfaces
c
c
c    Remark:
c
c      The first array iinbc/rinbc(.,1) corresponds to the momentum equations
c      The arrays iinbc/rinbc(.,2 to maxtrans+1) correspond to the transport
c      equations 1 to maxtrans
c      The arrays iinbc/rinbc(maxtrans+2 to maxtrans+1+maxturb) correspond to
c      the turbulence equations 1 to maxturb
c
c      Array iworkgen positions 1 to nsfread contains the actual curve
c      numbers. If a curve contains subcurves, only the subcurve numbers are
c      stored
c **********************************************************************
c
c                       DATA STATEMENTS
c
c     None
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen ( 'isrdbou2' )

c     --- get curve number

      nsfread = 0

      call isasknxt
      if ( ityprd.eq.2 ) then

c     --- number found

         icurve1 = nint(realrd)
         if ( icurve1.lt.1 .or. icurve1.gt.ncurvs ) then

c        --- icurve1 out of range

            call iserrchr ( codemn(iretur), 1 )
            call iserrint ( icurve1, 1 )
            call iserrint ( 1, 2 )
            call iserrint ( ncurvs, 3 )
            call iserrsub ( 530, 3, 0, 1 )
            call iserrcrd
            icurve1 = 1

         end if

      else

c     --- Error not an integer number found

         call iserrchr ( codemn(iretur), 1 )
         call iserrsub ( 528, 0, 0, 1 )
         call iserrcrd

      end if

c     --- read 'TO' if present and the momentum boundary conditions

      call isintx02 ( code, ncode, nncode, itype, table, codemn,
     +                nmcode, nmain, iretur, tabmin, tabmax,
     +                tabout, nout, strout, nstrng, ncodmx )

      if ( table(31).gt.0d0 ) then

c     --- TO found, read icurve2

         icurve2 = nint(table(31))

         if ( icurve2.lt.icurve1 .or. icurve2.gt.ncurvs ) then

c        --- icurve2 out of range

            call iserrchr ( code(30), 1 )
            call iserrint ( icurve2, 1 )
            call iserrint ( icurve1, 2 )
            call iserrint ( ncurvs, 3 )
            call iserrsub ( 530, 3, 0, 1 )
            call iserrcrd
            icurve2 = icurve1

         end if

         table(31) = -1d20

      else

c     --- TO not found

         icurve2 = icurve1

      end if

c     --- Set curve numbers into array iworkgen

      do icurve = icurve1, icurve2

         if ( kmeshq(1).gt.0 ) then

c        --- Possibly subcurves

            nsubcurvs = kmeshq(icurve+1) - kmeshq(icurve)

            if ( nsubcurvs.gt.0 ) then

c           --- There are nsubcurvs subcurves

               do i = 1, nsubcurvs
                  iworkgen(nsfread+i) =
     +               kmeshq(kmeshq(icurve)+i)
               end do
               nsfread = nsfread+nsubcurvs

            else

c           --- There are no subcurves

               nsfread = nsfread+1
               iworkgen(nsfread) = icurve

            end if

         else

c        --- No subcurves

            nsfread = nsfread+1
            iworkgen(nsfread) = icurve

         end if

         if ( icurvinf(4,icurve).ne.0 ) then

c        --- Curve number has been used more than once

            call iserrint ( icurve, 1 )
            call iserrsub ( 579, 1, 0, 0 )

         end if
         icurvinf(4,icurve) = 1

      end do

c     --- Check if all curves in iworkgen are outer curves

      do i = 1, nsfread
         icurve = abs(iworkgen(i))
         if ( icurvinf(1,icurve).eq.0 ) then

c        --- Curve is inner curve

            call iserrint ( icurve, 1 )
            call iserrsub ( 570, 1, 0, 0 )
            iworkgen(i) = 0

         end if
      end do

c     --- fill isfread

      do i = 1, nsfread
         icurve = abs(iworkgen(i))
         isfread(1,i) = ineigh(1,icurve)

c        --- note: faces have a sign in sepran

         isfread(2,i) = abs(ineigh(2,icurve))
         isfread(3,i) = abs(ineigh(3,icurve))
         isfread(4,i) = iworkgen(i)
      end do

      call iserclos ( 'isrdbou2' )

      end
