      subroutine isrdbou3 ( nsfread, isfread,
     +                      nblocks, nfaces, maxnsubf,
     +                      nsubfaces, subfaces,
     +                      code, ncode, nncode, itype, table, codemn,
     +                      nmcode, nmain, iretur, tabmin, tabmax,
     +                      tabout, nout, strout, nstrng, ncodmx )
c ======================================================================
c
c        programmer     Erik Brakkee
c        version 1.1    03-02-1998 Guus: set tabmax(14)
c        version 1.0    10-11-1994
c
c
c   copyright (c) 1994-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Read a set of subfaces numbers.
c    On input to this subroutine the 'CURVE' keyword has already been
c    read.
c    On output from this subroutine, the next main keyword is stored in
c    iretur.
c **********************************************************************
c
c                       KEYWORDS
c
c       boundary_conditions
c       read
c       read_cards
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer nsfread, isfread(10,*), nblocks, nfaces, maxnsubf
      integer nsubfaces(1:nblocks, 1:nfaces)
      integer subfaces(1:nblocks, 1:nfaces, 1:maxnsubf, *)
      integer nncode, ncode(nncode), itype(*), nmain, nmcode(*), iretur,
     +        nout, nstrng, ncodmx(nncode)
      character *(*) code(nncode), codemn(*), strout(nstrng)
      double precision table(nncode), tabmin(nncode), tabmax(nncode),
     +                 tabout(nout)

c     code    i    Table containing the subkeywords corresponding to this
c                  subroutine.
c     codemn  i    Table of main keywords
c     iretur i/o   Return code, indicates the sequence number of the new
c                  main keyword found
c                  If -1 an error has occurred
c                  If -2 the end of file has been reached
c     isfread i/o  reserved
c     itype   i    Array containing the type of actions to be undertaken for
c                  each subkeyword found.
c                  See subroutine isintx02 for the possible values of itype
c     maxnsubf i   Maximum number of subfaces encountered on all subfaces
c     nblocks i    Number of blocks
c     ncode   i    Array containing the number of significant characters for
c                  each keyword in code
c     ncodmx  i    Array containing the maximum number of characters for
c                  each keyword in code
c     nfaces  i    Number of faces
c     nmain   i    Number of keywords in array codemn
c     nmcode  i    Array containing the number of significant characters
c                  for each main keyword in codemn
c     nncode  i    Number of subkeywords stored in array code
c     nout    i    Length of array tabout
c     nsfread i/o  Number of subfaces read
c     nstrng  i    Length of array strout
c     nsubfaces i  Array containing the number of subfaces of each face,
c                  see array itopol for an extended description
c     strout  o    Contains series of strings, see itype
c     subfaces i   reserved
c     table   i/o  Array containing the real or integer values corresponding
c                  to the subkeywords in code that have been read by isintx02
c     tabmax  i    Array containing the maximal values of the parameters read
c                  corresponding to the subkeywords in code
c     tabmin  i    Array containing the minimum values of the parameters read
c                  corresponding to the subkeywords in code
c     tabout  o    Array of output reals provided by isintx02
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cseprd'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer dummy1, dummy2, i, iblock, iblock1, iblock2,
     +        iface, iface1, iface2,
     +        iretur2, iseg, iseg1, iseg2

c     dummy1    Dummy
c     dummy2    Dummy
c     i         General loop variable
c     iblock    Block sequence number
c     iblock1   Block sequence number
c     iblock2   Block sequence number
c     iface     Face number
c     iface1    Face number
c     iface2    Face number
c     iretur2   Saved value of iretur
c     iseg      Counting variable: sequence number of segment
c     iseg1     Counting variable: sequence number of segment
c     iseg2     Counting variable: sequence number of segment
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      logical isbcread

c     ISASKNXT  Read next item from the standard input file
c     ISBCREAD  Check is isintx02 has read boundary conditions
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRCHR  Puts values of strings in character error array
c     ISERRCRD  Prints the last record read and underlines the erroneous part
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c     ISINTX02  Interpret a record containing keywords followed by numbers
c     ISWARSUB  Print warnings
c **********************************************************************
c
c                       I/O
c
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     528   No integer number has been found after the keyword %
c           boundary conditions
c     692   CURVE keyword found while not in Sepran mode
c     694   Invalid range of blocks, faces or subfaces given.
c     695   Block number out of range
c     696   FACE keyword unexpected
c     697   Face number out of range
c     698   Subface keyword unexpected
c     699   Subface number larger than maxnsubfaces
c     700   Subface number out of range
c     701   Block number undefined
c     702   Face  number undefined
c     703   Boundary condition specified at an internal subface
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c    Remark:
c
c      The first array iinbc/rinbc(.,1) corresponds to the momentum equations
c      The arrays iinbc/rinbc(.,2 to maxtrans+1) correspond to the transport
c      equations 1 to maxtrans
c      The arrays iinbc/rinbc(maxtrans+2 to maxtrans+1+maxturb) correspond to
c      the turbulence equations 1 to maxturb
c
c      Array iworkgen positions 1 to nsfread contains the actual curve
c      numbers. If a curve contains subcurves, only the subcurve numbers are
c      stored
c **********************************************************************
c
c                       DATA STATEMENTS
c
c     None
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c

      call iseropen ( 'isrdbou3' )

      if ( nsfread.eq.0 ) then

         if ( nblocks.eq.1 ) then

c        --- 'BLOCK' keyword not mandatory in single block case

            iblock1 = 1
            iblock2 = 1

         else

c        --- 'BLOCK' keyword is mandatory in general multi-block case

            iblock1 = 0
            iblock2 = 0

         end if

c        --- 'FACE' and 'SUBFACE' always mandatory

         iface1  = 0
         iface2  = 0
         iseg1   = 0
         iseg2   = 0

      else

c     --- not the first call to this subroutine, fill iblock*, iface* and
c         iseg* using nsfread and isfread

         iblock1 = nblocks+1
         iblock2 = -1
         iface1  = nfaces+1
         iface2  = -1
         iseg1   = maxnsubf+1
         iseg2   = -1

         do i = 1, nsfread
            iblock1 = MIN( iblock1, isfread(1,i) )
            iblock2 = MAX( iblock2, isfread(1,i) )
            iface1  = MIN( iface1,  isfread(2,i) )
            iface2  = MAX( iface2,  isfread(2,i) )
            iseg1   = MIN( iseg1,   isfread(3,i) )
            iseg2   = MAX( iseg2,   isfread(3,i) )
         end do

      end if

c     --- while boundary conditions have not been read and the
c         next main keyword is either 'BLOCK', 'FACE' or 'SUBFACE'
c         interpret keyword

c         --- do while ( .not. isbcread(nncode, table) .and. iretur.le.4 )

20    if  ( .not. isbcread(nncode, table) .and. iretur.le.4 ) then

         if ( iretur.eq.1 ) then

c        --- 'CURVE' found in non-sepran mode

            call iserrsub ( 692, 0, 0, 0 )

         end if

         call isasknxt

         if ( ityprd.eq.2 ) then

c        --- number found

            dummy1 = nint(realrd)

         else

c        --- Error not an integer number found

            call iserrchr ( codemn(iretur), 1 )
            call iserrsub ( 528, 0, 0, 1 )
            call iserrcrd

         end if

c        --- save old keyword

         iretur2 = iretur

c        --- read 'TO' if present and the next keyword
c            (and boundary conditions if present)

         tabmax(14) = max(nblocks,nfaces, maxnsubf)
         call isintx02 ( code, ncode, nncode, itype, table, codemn,
     +                   nmcode, nmain, iretur, tabmin, tabmax,
     +                   tabout, nout, strout, nstrng, ncodmx )

         if ( table(31).gt.0d0 ) then

c        --- TO found, read icurve2

            dummy2 = nint(table(31))

            table(31) = -1d20

         else

            dummy2 = dummy1

         end if

         if ( dummy2.lt.dummy1 ) then

            call iserrint ( dummy1,          1 )
            call iserrint ( dummy2,          2 )
            call iserrchr ( codemn(iretur2), 1 )
            call iserrsub ( 694, 2, 0, 1 )

         end if

         if ( iretur2.eq.2 ) then

c        --- 'BLOCK' keyword

            iblock1 = dummy1
            iblock2 = dummy2
            iface1  = 0
            iface2  = 0
            iseg1   = 0
            iseg2   = 0

            if ( iblock1.lt.1 .or. iblock2.lt.1 .or.
     +           iblock2.gt.nblocks .or. iblock2.gt.nblocks ) then

               call iserrint ( iblock1, 1 )
               call iserrint ( iblock2, 2 )
               call iserrint ( nblocks, 3 )
               call iserrsub ( 695, 3, 0, 0 )

            end if

         else if ( iretur2.eq.3 ) then

c        --- 'FACE' keyword

            if ( iblock1.eq.0 ) then

               call iserrsub ( 696, 0, 0, 0 )

            end if

            iface1 = dummy1
            iface2 = dummy2
            iseg1  = 0
            iseg2  = 0

            if ( iface1.lt.1      .or. iface2.lt.1 .or.
     +           iface1.gt.nfaces .or. iface2.gt.nfaces ) then

               call iserrint ( iblock1, 1 )
               call iserrint ( iblock2, 2 )
               call iserrint ( iface1,  3 )
               call iserrint ( iface2,  4 )
               call iserrint ( nfaces,  5 )
               call iserrsub ( 697, 5, 0, 0 )

            end if

         else if ( iretur2.eq.4 ) then

c        --- 'SUBFACE' keyword

            if ( iface1.eq.0 ) then

               call iserrsub ( 698, 0, 0, 0 )

            end if

            iseg1 = dummy1
            iseg2 = dummy2

            if ( iseg1.lt.1 .or. iseg2.lt.1 .or.
     +           iseg1.gt.maxnsubf .or. iseg2.gt.maxnsubf ) then

               call iserrint ( iseg1, 1 )
               call iserrint ( iseg2, 2 )
               call iserrint ( maxnsubf, 3 )
               call iserrsub ( 699, 3, 0, 0 )

            end if

            do iblock = iblock1, iblock2
               do iface = iface1, iface2

                  if ( iseg2.gt.nsubfaces(iblock, iface) ) then

                     call iserrint ( iface, 1 )
                     call iserrint ( iblock, 2 )
                     call iserrint ( iseg1, 3 )
                     call iserrint ( iseg2, 4 )
                     call iserrint ( nsubfaces(iblock, iface), 5 )
                     call iserrsub ( 700, 5, 0, 0 )

                  end if

               end do
            end do

         end if

         goto 20
      end if

c     --- fill nsfread and isfread

      if ( iblock1.eq.0 ) then
         call iserrsub ( 701, 0, 0, 0 )
      end if

      if ( iface1.eq.0 ) then
         call iserrsub ( 702, 0, 0, 0 )
      end if

      if ( iseg1.eq.0 ) then

c     --- BLOCK a to b,  FACE c to d  given

         nsfread = 0
         do iblock = iblock1, iblock2
            do iface = iface1, iface2
               do iseg = 1, nsubfaces(iblock,iface)
                  if ( subfaces(iblock,iface,iseg,5).ne.0 ) then
                     call iserrint ( iseg, 1 )
                     call iserrint ( iface, 2 )
                     call iserrint ( iblock, 3 )
                     call iswarsub ( 703, 3, 0, 0 )
                  end if
                  nsfread = nsfread + 1
                  isfread(1,nsfread) = iblock
                  isfread(2,nsfread) = iface
                  isfread(3,nsfread) = iseg
                  isfread(4,nsfread) = 0
               end do
            end do
         end do

      else

c     --- BLOCK a to b, FACE c to d, SUBFACE e to f given

         nsfread = 0
         do iblock = iblock1, iblock2
            do iface = iface1, iface2
               do iseg = iseg1, iseg2
                  if ( subfaces(iblock,iface,iseg,5).ne.0 ) then
                     call iserrint ( iseg, 1 )
                     call iserrint ( iface, 2 )
                     call iserrint ( iblock, 3 )
                     call iswarsub ( 703, 3, 0, 0 )
                  end if
                  nsfread = nsfread + 1
                  isfread(1,nsfread) = iblock
                  isfread(2,nsfread) = iface
                  isfread(3,nsfread) = iseg
                  isfread(4,nsfread) = 0
               end do
            end do
         end do

      end if

      call iserclos ( 'isrdbou3' )

      end
