      subroutine isrdbou4 ( nsfread, isfread, nncode,  nout, code,
     +                      table, tabout, uset, maxtrans, maxturb,
     +                      ndim, itypebc, rtypebc, ineigh, sepran,
     +                      nblocks, nsubfaces, tabmax )
c ======================================================================
c
c        programmer     Guus Segal
c        version 3.0    date 02-02-1998 Guus, length itypbc
c                                       extra parameters sepran, nblocks,
c                                       nsubfaces
c        version 2.1    date 28-01-1998 Guus: extension array ineigh for 3D
c        version 2.0    date 28-10-1997 Kees K: periodic changed
c        version 1.10   date 17-09-1997 Guus: no-slip and so on for 3D
c        version 1.9    date 11-07-1996 Marcel eps_asymptote
c        version 1.8    date 11-04-1996 Kees K. dimension itypebc, rtypebc
c                                       plus 1 for the pressure
c        version 1.7    date 29-03-1996 Kees K. allow negative function numbers
c                                       for time_func
c        version 1.6    date 09-01-1996 Marcel Z. periodic for turbulence
c        version 1.5    date 08-01-1996 Marcel: error message with respect to
c                                       bndtype = 5 deleted
c        version 1.4    date 04-04-1995 Guus: correction periodic/anti-periodic
c        version 1.3    date 10-01-1995 Kees K 3D
c        version 1.2    date 30-11-1994 Introduce parallel_outflow
c        version 1.1    date 09-11-1994 Erik: extracted from isrdboun
c        version 1.0    date 24-08-1994 Subroutine extracted from the SEPRAN
c                                       subroutine INTX02 version 1.5
c
c   copyright (c) 1994-1998  "Ingenieursbureau SEPRA"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "Ingenieursbureau SEPRA".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c   Interpret the boundary conditions read in the table and tabout
c   arrays for the momentum equation.
c **********************************************************************
c
c                       KEYWORDS
c
c  interpret
c  keyword
c  read_cards
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer nsfread, isfread(10,1:nsfread),
     +        maxtrans, maxturb, ndim, ineigh(10+(ndim-2)*4,*)
      integer nncode, nout, nblocks, nsubfaces(1:nblocks,*)
      integer itypebc(maxtrans+maxturb+ndim,4)
      double precision table(nncode), tabout(nout), tabmax(nncode)
      double precision rtypebc(maxtrans+maxturb+ndim,ndim)
      character *16 code(nncode)
      logical uset, sepran

c     code      i    Table containing the subkeywords corresponding to this
c                    subroutine.
c     ineigh    i    reserved
c     isfread   i    reserved
c     itypebc  i/o   Contains the part itype in array iinbc corresponding to
c                    one block and one segment
c     maxtrans  i    Maximum number of transport equations allowed
c     maxturb   i    Maximum number of turbulence equations allowed
c     nblocks   i    Number of blocks
c     ndim      i    Dimension of space (2 or 3)
c     nncode    i    Number of subkeywords stored in array code
c     nout      i    Length of array tabout
c     nsfread   i    Number of subfaces read
c     nsubfaces i    Array containing the number of subfaces of each face,
c                    see array itopol for an extended description
c     rtypebc  i/o   Contains the part in array rinbc corresponding to
c                    one block and one segment
c     sepran    i    If true the grid is created by SEPRAN
c     table     i    Array containing the real or integer values corresponding
c                    to the subkeywords in code that have been read by isintx02
c     tabmax    i    Array containing the maximal values of the parameters read
c                    corresponding to the subkeywords in code
c     tabout    i    Array of output reals provided by isintx02
c     uset      o    If true a boundary conditions for the momentum equation has
c                    been set for the present segment
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      logical kset, epsset, omegaset
      integer iblock, iposut, iface, isubface, m

c     epsset    If true a boundary conditions for eps has been set for the
c               present segment
c     iblock    Block sequence number
c     iface     Face number
c     iposut    last position used in array tabout
c     isubface  Subface number
c     kset      If true a boundary conditions for k has been set for the
c               present segment
c     m         General loop variable
c     omegaset  If true a boundary conditions for omega has been set for the
c               present segment
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c     ISRDERR   Give an error message for a set of subfaces
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     572   Normal and/or tangential velocity component has already been
c           prescribed
c     573   Normal velocity component has been prescribed, but not ut
c           or sigmant
c     574   Tangential velocity component has been prescribed, but not un
c           or sigmann
c     575   ux has been prescribed, but not uy
c     576   uy has been prescribed, but not ux
c     582   k has already been prescribed
c     583   eps has already been prescribed
c     591   Already a boundary condition has been given for omega-turbulence
c           equation
c     704   keyword uz not found
c     705   functions not yet allowed for this boundary condition
c     714   Not 3 numbers after periodic
c     715   Block number is larger than number of blocks
c     716   Face number is larger than number of faces
c     717   Subface number is larger than number of subfaces
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen ( 'isrdbou4' )

      uset     = .false.
      kset     = .false.
      epsset   = .false.
      omegaset = .false.

      if ( table(1).gt.0d0 ) then

c     --- noslip

         itypebc(1,1) = 4
         itypebc(1,2) = -1
         itypebc(1,3) = -1
         if ( ndim.eq.3 ) itypebc(1,4) = -1
         uset = .true.

      end if
      if ( table(2).gt.-1d19 ) then

c     --- inflow umax

         itypebc(1,1) = 12
         itypebc(1,2) = -1
         rtypebc(1,1) = table(2)
         if ( uset ) then
            call isrderr( 572, code(2), 0, nsfread, isfread )
         end if
         uset = .true.

      end if
      if ( table(3).gt.0d0 ) then

c     --- outflow

         itypebc(1,1) = 7
         itypebc(1,2) = -1
         itypebc(1,3) = -1
         if ( ndim.eq.3 ) itypebc(1,4) = -1
         if ( uset ) then
            call isrderr( 572, code(3), 0, nsfread, isfread )
         end if
         uset = .true.

      end if
      if ( table(4).gt.0d0 .or. table(33).gt.0d0) then

c     --- parallel_outflow or outstream

         itypebc(1,1) = 6
         itypebc(1,2) = -1
         itypebc(1,3) = -1
         if ( uset ) then
            call isrderr( 572, code(4), 0, nsfread, isfread )
         end if
         uset = .true.

      end if
      if ( table(5).gt.0d0 ) then

c     --- freeslip

         itypebc(1,1) = 5
         itypebc(1,2) = -1
         itypebc(1,3) = -1
         if ( ndim.eq.3 ) itypebc(1,4) = -1
         if ( uset ) then
            call isrderr( 572, code(5), 0, nsfread, isfread )
         end if
         uset = .true.

      end if
      if ( table(6).gt.-1d19 ) then

c     --- un given, check if either ut or sigmant is given

         if ( table(6).gt.0 .or. table(6).lt.-100) then
            itypebc(1,2) = nint(table(6))
         else
            itypebc(1,2) = -1
            rtypebc(1,1) = tabout(-nint(table(6)))
         end if

         if ( table(7).gt.-1d19 ) then

c        --- un, ut given

            itypebc(1,1) = 4

            if ( ndim.eq.2) then

c           --- ndim = 2

               if ( table(7).gt.0 .or. table(7).lt.-100) then
                  itypebc(1,3) = nint(table(7))
               else
                  itypebc(1,3) = -1
                  rtypebc(1,2) = tabout(-nint(table(7)))
               end if

            else

c           --- ndim = 3

               if ( tabout(nint(table(7))).gt.0
     +            .or. tabout(nint(table(7))).lt.-100) then
                  itypebc(1,3) = nint(tabout(nint(table(7))))
                  call isrderr( 705, code(7), 0, nsfread, isfread )
               else
                  itypebc(1,3) = -1
                  rtypebc(1,2) = tabout(nint(table(7)) +1 )
               end if

               if ( tabout(nint(table(7))+2).gt.0
     +            .or. tabout(nint(table(7))+2).lt.-100)  then
                  itypebc(1,4) = nint(tabout(nint(table(7))+2))
                  call isrderr( 705, code(7), 0, nsfread, isfread )
               else
                  itypebc(1,4) = -1
                  rtypebc(1,3) = tabout(nint(table(7)) +3 )
               end if

               if( rtypebc(1,2).ne.0 .or. rtypebc(1,3).ne.0 ) then
                  call isrderr( 705, code(7), 0, nsfread, isfread )
               end if

            end if

         else if ( table(11).gt.-1d19 ) then

c        --- un, sigmant given

            itypebc(1,1) = 5

            if ( ndim.eq.2) then

c           --- ndim = 2

               if ( table(11).gt.0 .or. table(11).lt.-100) then
                  itypebc(1,3) = nint(table(11))
               else
                  itypebc(1,3) = -1
                  rtypebc(1,2) = tabout(-nint(table(11)))
               end if

            else

c           --- ndim = 3

               if ( tabout(nint(table(11))).gt.0
     +              .or. tabout(nint(table(11))).lt.-100) then
                  itypebc(1,3) = nint(tabout(nint(table(11))))

c                 --- call isrderr( 705, code(11), 0, nsfread, isfread )

               else
                  itypebc(1,3) = -1
                  rtypebc(1,2) = tabout(nint(table(11)) +1 )
               end if

               if ( tabout(nint(table(11))+2).gt.0
     +              .or. tabout(nint(table(11))+2).lt.-100) then
                  itypebc(1,4) = nint(tabout(nint(table(11))+2))

c                 --- call isrderr( 705, code(11), 0, nsfread, isfread )

               else
                  itypebc(1,4) = -1
                  rtypebc(1,3) = tabout(nint(table(11)) +3 )
               end if

c              --- if( rtypebc(1,2).ne.0 .or. rtypebc(1,3).ne.0 ) then
c                     call isrderr( 705, code(11), 0, nsfread, isfread )
c                  end if

            end if

         else

c        --- neither ut or sigmant given

            call isrderr( 573, code(6), 0, nsfread, isfread )

         end if

         uset = .true.

      end if
      if ( table(7).gt.-1d19 .and. table(6).lt.-1d19 ) then

c     --- ut given, check if sigmann is given

         if( ndim.eq.2 ) then

c        --- ndim = 2

            if ( table(7).gt.0 .or. table(7).lt.-100) then
               itypebc(1,3) = nint(table(7))
            else
               itypebc(1,3) = -1
               rtypebc(1,2) = tabout(-nint(table(7)))
            end if

         else

c        --- ndim = 3

               if ( tabout(nint(table(7))).gt.0
     +              .or. tabout(nint(table(7))).lt.-100) then
                  itypebc(1,3) = nint(tabout(nint(table(7))))
                  call isrderr( 705, code(7), 0, nsfread, isfread )
               else
                  itypebc(1,3) = -1
                  rtypebc(1,2) = tabout(nint(table(7)) +1 )
               end if

               if ( tabout(nint(table(7))+2).gt.0
     +              .or.tabout(nint(table(7))+2).lt.-100) then
                  itypebc(1,4) = nint(tabout(nint(table(7))+2))
                  call isrderr( 705, code(7), 0, nsfread, isfread )
               else
                  itypebc(1,4) = -1
                  rtypebc(1,3) = tabout(nint(table(7)) +3 )
               end if
               if( rtypebc(1,2).ne.0 .or. rtypebc(1,3).ne.0 ) then
                  call isrderr( 705, code(7), 0, nsfread, isfread )
               end if

         end if

         if ( table(10).gt.-1d19 ) then

c        --- ut, sigmann given

            itypebc(1,1) = 6
            if ( table(10).gt.0 ) then
               itypebc(1,2) = nint(table(10))
               if( ndim.eq.3 ) then
                  call isrderr( 705, code(10), 0, nsfread, isfread )
               end if
            else
               itypebc(1,2) = -1
               rtypebc(1,1) = tabout(-nint(table(10)))
               if( ndim.eq.3 .and. rtypebc(1,1).ne.0 ) then
                  call isrderr( 705, code(10), 0, nsfread, isfread )
               end if
            end if

         else

c        --- sigmann not given

            call isrderr( 574, code(7), 0, nsfread, isfread )

         end if
         if ( uset ) then
            call isrderr( 572, code(5), 0, nsfread, isfread )
         end if
         uset = .true.

      end if
      if ( table(8).gt.-1d19 ) then

c     --- ux given, check if uy is given

         if ( table(8).gt.0 .or. table(8).lt.-100) then
            itypebc(1,2) = nint(table(8))
         else
            itypebc(1,2) = -1
            rtypebc(1,1) = tabout(-nint(table(8)))
         end if
         if ( ndim.eq.2 ) then

c        --- ndim = 2

            if ( table(9).gt.-1d19 ) then

c           --- ux, uy given

               itypebc(1,1) = 10
               if ( table(9).gt.0 .or. table(9).lt.-100) then
                  itypebc(1,3) = nint(table(9))
               else
                  itypebc(1,3) = -1
                  rtypebc(1,2) = tabout(-nint(table(9)))
               end if

            else

c           --- uy not given

               call isrderr( 575, code(8), 0, nsfread, isfread )

            end if
         else

c        --- ndim = 3

            if ( table(9).gt.-1d19 .and. table(34).gt.-1d19 ) then

c           --- ux, uy  and uz given

               itypebc(1,1) = 10
               if ( table(9).gt.0 .or. table(9).lt.-100) then
                  itypebc(1,3) = nint(table(9))
               else
                  itypebc(1,3) = -1
                  rtypebc(1,2) = tabout(-nint(table(9)))
               end if

               if ( table(34).gt.0 .or. table(34).lt.-100) then
                  itypebc(1,4) = nint(table(34))
               else
                  itypebc(1,4) = -1
                  rtypebc(1,3) = tabout(-nint(table(34)))
               end if
            else

               if ( table(9).lt.-1d19 ) then

c              --- uy not given

                  call isrderr( 575, code(8), 0, nsfread, isfread )

               end if

               if ( table(34).lt.-1d19 ) then

c              --- uz not given

                  call isrderr( 704, code(8), 0, nsfread, isfread )

               end if

            end if
         end if

         if ( uset ) then
            call isrderr( 572, code(8), 0, nsfread, isfread )
         end if
         uset = .true.

      end if
      if ( table(9).gt.-1d19 .and. table(8).lt.-1d19 ) then

c     --- ux not given

         call isrderr( 576, code(9), 0, nsfread, isfread )

      end if
      if ( table(34).gt.-1d19 .and. table(8).lt.-1d19 ) then

c     --- ux not given

         call isrderr( 576, code(34), 0, nsfread, isfread )

      end if
      if ( table(10).gt.-1d19 .and. table(7).lt.-1d19 ) then

c     --- sigmann given, check if sigmant is given

         if ( table(10).gt.0 .or. table(10).lt.-100) then
            itypebc(1,2) = nint(table(10))
            if( ndim.eq.3 ) then
               call isrderr( 705, code(10), 0, nsfread, isfread )
            end if
         else
            itypebc(1,2) = -1
            rtypebc(1,1) = tabout(-nint(table(10)))
            if( ndim.eq.3 .and. rtypebc(1,1).ne.0 ) then
               call isrderr( 705, code(10), 0, nsfread, isfread )
            end if
         end if
         if ( table(11).gt.-1d19 ) then

c        --- sigmann, sigmant given

            itypebc(1,1) = 7

            if ( ndim.eq.2 ) then

c           --- ndim = 2

               if ( table(11).gt.0 .or. table(11).lt.-100 ) then
                  itypebc(1,3) = nint(table(11))
               else
                  itypebc(1,3) = -1
                  rtypebc(1,2) = tabout(-nint(table(11)))
               end if

            else

c           --- ndim = 3

               if ( tabout(nint(table(11))).gt.0
     +              .or. tabout(nint(table(11))).lt.-100) then
                  itypebc(1,3) = nint(tabout(nint(table(11))))
                  call isrderr( 705, code(11), 0, nsfread, isfread )
               else
                  itypebc(1,3) = -1
                  rtypebc(1,2) = tabout(nint(table(11)) +1 )
               end if

               if ( tabout(nint(table(11))+2).gt.0
     +              .or. tabout(nint(table(11))+2).lt.-100) then
                  itypebc(1,4) = nint(tabout(nint(table(11))+2))
                  call isrderr( 705, code(11), 0, nsfread, isfread )
               else
                  itypebc(1,4) = -1
                  rtypebc(1,3) = tabout(nint(table(11)) +3 )
               end if
               if( rtypebc(1,2).ne.0 .or. rtypebc(1,3).ne.0 ) then
                  call isrderr( 705, code(11), 0, nsfread, isfread )
               end if

            end if

         else

c        --- sigmant not given

            call isrderr( 573, code(10), 0, nsfread, isfread )

         end if
         if ( uset ) then
            call isrderr( 572, code(10), 0, nsfread, isfread )
         end if
         uset = .true.

      end if
      if ( table(11).gt.-1d19 .and. table(10).lt.-1d19 .and.
     +        table(6).lt.-1d19 ) then

c     --- un or sigmann not given

         call isrderr( 574, code(11), 0, nsfread, isfread )

      end if
      if ( table(12).gt.-1d19 ) then

c     --- wall functions

         itypebc(1,1) = 9
         itypebc(1,2) = nint(table(12))
         itypebc(maxtrans+ndim+1,1) = 2
         itypebc(maxtrans+ndim+1,2) = -1
         itypebc(maxtrans+ndim+1,3) = -1
         itypebc(maxtrans+ndim+2,1) = 3
         itypebc(maxtrans+ndim+2,2) = -1
         itypebc(maxtrans+ndim+2,3) = -1
         if ( itypebc(1,2).eq.2 ) then

c        --- rough found, fill roughness

            if ( table(32).lt.-1d19 ) then

c           --- roughness has not been given

               call isrderr( 572, code(32), 0, nsfread, isfread)

            end if
            rtypebc(1,1) = table(32)

         end if

         if ( uset ) then
            call isrderr( 572, code(12), 0, nsfread, isfread )
         end if
         uset = .true.

      end if
      if ( table(13).gt.-1d19 .or. table(16).gt.-1d19 ) then

c     --- periodic or anti-symmetric pressjump found

         if ( table(13).gt.-1d19 ) then
            itypebc(1,1) = 96
         else
            itypebc(1,1) = 95
         end if
         if ( table(15).lt.-1d19 ) then

c        --- flux = 0

            itypebc(1,2) = -1

         else

            if ( table(15).gt.0 .or. table(15).lt.-100) then
               itypebc(1,2) = nint(table(15))
            else
               itypebc(1,2) = -1
               rtypebc(1,1) = tabout(-nint(table(15)))
            end if
         end if

         if ( uset .or. table(13).gt.-1d19 .and.
     +           table(16).gt.-1d19 ) then
            call isrderr( 572, code(13), 0, nsfread, isfread )
         end if
         uset = .true.

      end if
      if ( table(14).gt.-1d19 .or. table(17).gt.-1d19 ) then

c     --- periodic or anti-symmetric periodic found

         if ( table(14).gt.-1d19 ) then

c        --- periodic found

            itypebc(1,1) = 98
            itypebc(ndim+1+maxtrans,1) = 98
            itypebc(ndim+2+maxtrans,1) = 98
            if ( sepran ) then
               iblock   = abs(ineigh(1,nint(table(14))) )
               iface    = abs(ineigh(2,nint(table(14))) )
               isubface = abs(ineigh(3,nint(table(14))) )
               itypebc(1,2) = iface+10*(iblock-1)
               itypebc(1,3) = isubface
            else
               iposut =nint(table(14))
               m = nint(tabmax(14))
               if ( m.ne.3 ) then
                  call iserrint ( m, 1 )
                  call iserrsub ( 714, 1, 0, 0 )
               else
                  iblock   = nint(tabout(iposut))
                  iface    = nint(tabout(iposut+1))
                  isubface = nint(tabout(iposut+2))
                  if ( iblock.gt.nblocks ) then
                     call iserrint ( iblock, 1 )
                     call iserrint ( nblocks, 2 )
                     call iserrsub ( 715, 2, 0, 0 )
                     iblock = 1
                  end if
                  if ( iface.gt.ndim*2 ) then
                     call iserrint ( iface, 1 )
                     call iserrint ( ndim*2, 2 )
                     call iserrsub ( 716, 2, 0, 0 )
                     iface = 1
                  end if
                  if ( isubface.gt.nsubfaces(iblock,iface) ) then
                     call iserrint ( isubface, 1 )
                     call iserrint ( nsubfaces(iblock,iface), 2 )
                     call iserrsub ( 717, 2, 0, 0 )
                     isubface = 1
                  end if
                  itypebc(1,2) = iface+10*(iblock-1)
                  itypebc(1,3) = isubface
               end if
            end if

         else

c        --- anti-symmetric periodic found

            itypebc(1,1) = 97
            itypebc(ndim+1+maxtrans,1) = 97
            itypebc(ndim+2+maxtrans,1) = 97

         end if

         if ( uset .or. table(14).gt.-1d19 .and.
     +           table(17).gt.-1d19 ) then
            call isrderr( 572, code(17), 0, nsfread, isfread )
         end if
         uset = .true.

         if ( kset .or. table(14).gt.-1d19 .and.
     +           table(17).gt.-1d19 ) then
            call isrderr( 572, code(17), 0, nsfread, isfread )
         end if
         kset = .true.

         if ( epsset .or. table(14).gt.-1d19 .and.
     +           table(17).gt.-1d19 ) then
            call isrderr( 572, code(17), 0, nsfread, isfread )
         end if
         epsset = .true.

         if ( omegaset .or. table(14).gt.-1d19 .and.
     +           table(17).gt.-1d19 ) then
            call isrderr( 572, code(17), 0, nsfread, isfread )
         end if
         omegaset = .true.

      end if

      if ( table(24).gt.-1d19 ) then

c     --- k-dirichlet found

         itypebc(ndim+1+maxtrans,1) = 1
         if ( table(24).gt.0 .or. table(24).lt.-100) then
            itypebc(ndim+1+maxtrans,2) = nint(table(24))
         else
            itypebc(ndim+1+maxtrans,2) = -1
            rtypebc(ndim+1+maxtrans,1) = tabout(-nint(table(24)))
         end if
         kset = .true.

      end if
      if ( table(25).gt.-1d19 ) then

c     --- k-neumann found

         itypebc(ndim+1+maxtrans,1) = 2
         itypebc(ndim+1+maxtrans,2) = -1
         if ( table(25).gt.0 .or. table(25).lt.-100) then
            itypebc(ndim+1+maxtrans,3) = nint(table(25))
         else
            itypebc(ndim+1+maxtrans,3) = -1
            rtypebc(ndim+1+maxtrans,2) = tabout(-nint(table(25)))
         end if
         if ( kset ) then
            call isrderr( 582, code(25), 0, nsfread, isfread )
         end if
         kset = .true.

      end if
      if ( table(26).gt.-1d19 ) then

c     --- eps-dirichlet found

         itypebc(ndim+2+maxtrans,1) = 1
         if ( table(26).gt.0 .or. table(26).lt.-100) then
            itypebc(ndim+2+maxtrans,2) = nint(table(26))
         else
            itypebc(ndim+2+maxtrans,2) = -1
            rtypebc(ndim+2+maxtrans,1) = tabout(-nint(table(26)))
         end if
         epsset = .true.

      end if
      if ( table(27).gt.-1d19 ) then

c     --- eps-neumann found

         itypebc(ndim+2+maxtrans,1) = 2
         itypebc(ndim+2+maxtrans,2) = -1
         if ( table(27).gt.0 .or. table(27).lt.-100) then
            itypebc(ndim+2+maxtrans,3) = nint(table(27))
         else
            itypebc(ndim+2+maxtrans,3) = -1
            rtypebc(ndim+2+maxtrans,2) = tabout(-nint(table(27)))
         end if
         if ( epsset ) then
            call isrderr( 583, code(27), 0, nsfread, isfread )
         end if
         epsset = .true.

      end if
      if ( table(28).gt.-1d19 ) then

c     --- omega-dirichlet found

         itypebc(ndim+2+maxtrans,1) = 1
         if ( table(28).gt.0 .or. table(28).lt.-100) then
            itypebc(ndim+2+maxtrans,2) = nint(table(28))
         else
            itypebc(ndim+2+maxtrans,2) = -1
            rtypebc(ndim+2+maxtrans,1) = tabout(-nint(table(28)))
         end if
         omegaset = .true.

      end if
      if ( table(29).gt.-1d19 ) then

c     --- omega-neumann found

         itypebc(ndim+2+maxtrans,1) = 2
         itypebc(ndim+2+maxtrans,2) = -1
         if ( table(29).gt.0 .or. table(29).lt.-100) then
            itypebc(ndim+2+maxtrans,3) = nint(table(29))
         else
            itypebc(ndim+2+maxtrans,3) = -1
            rtypebc(ndim+2+maxtrans,2) = tabout(-nint(table(29)))
         end if
         omegaset = .true.

      end if
      if ( table(30).gt.-1d19 ) then

c     --- omega-asymptote found

         itypebc(maxtrans+ndim+2,1) = 27
         itypebc(maxtrans+ndim+2,2) = -1
         itypebc(maxtrans+ndim+2,3) = -1

         if ( omegaset ) then
            call isrderr( 591, code(30), 0, nsfread, isfread )
         end if
         omegaset = .true.

      end if
      if ( table(36).gt.-1d19 ) then

c     --- eps-asymptote found

         itypebc(maxtrans+ndim+2,1) = 48
         itypebc(maxtrans+ndim+2,2) = -1
         itypebc(maxtrans+ndim+2,3) = -1

         if ( epsset ) then
            call isrderr( 583, code(36), 0, nsfread, isfread )
         end if
         epsset = .true.

      end if

      call iserclos ( 'isrdbou4' )

      end
