      subroutine isrdbou5 ( itrans, nsfread, isfread,
     +                      nncode,  nout, code, table, tabout,
     +                      maxtrans, maxturb, ndim, itypebc, rtypebc,
     +                      ineigh )
c ======================================================================
c
c        programmer     Guus Segal
c        version 2.1    date 20-11-1997 Kees K: periodic changed
c        version 2.0    date 15-09-1997 Guus: Use ndim for itypebc and rtypebc
c                                       remove uset
c        version 1.4    date 04-06-1996 Marcel wall_temperature
c        version 1.3    date 03-04-1996 Kees K. dimension itypebc, rtypebc
c                                       plus 1 for the pressure
c        version 1.2    date 01-04-1996 Kees K. allow negative function numbers
c                                       for time_func
c        version 1.1    date 04-04-1995 Guus: correction periodic/anti-periodic
c        version 1.0    date 24-08-1994 (Subroutine extracted from the SEPRAN
c                                        subroutine INTX02 version 1.5)
c
c
c   copyright (c) 1994-1997  "Ingenieursbureau SEPRA"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "Ingenieursbureau SEPRA".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c   Interpret the boundary conditions read in the table and tabout
c   arrays for a transport equation.
c **********************************************************************
c
c                       KEYWORDS
c
c  interpret
c  keyword
c  read_cards
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer itrans, nsfread, isfread(10,1:nsfread),
     +        maxtrans, maxturb, ndim, ineigh(10,*)
      integer nncode, nout
      integer itypebc(maxtrans+maxturb+ndim,ndim+1)
      double precision table(nncode), tabout(nout)
      double precision rtypebc(maxtrans+maxturb+ndim,ndim)
      character *16 code(nncode)

c     code    i    Table containing the subkeywords corresponding to this
c                  subroutine.
c     isfread i    reserved
c     itrans  i    reserved
c     itypebc i    Contains the part itype in array iinbc corresponding to
c                  one block and one segment
c     maxtrans  i  Maximum number of transport equations allowed
c     maxturb   i  Maximum number of turbulence equations allowed
c     ndim    i    Dimension of space (2 or 3)
c     nncode  i    Number of subkeywords stored in array code
c     nout    i    Length of array tabout
c     nsfread i    Number of subfaces read
c     rtypebc i    Contains the part in array rinbc corresponding to
c                  one block and one segment
c     table  i/o   Array containing the real or integer values corresponding to
c                  the subkeywords in code that have been read by isintx02
c     tabout  o    Array of output reals provided by isintx02
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      logical sset

c     sset      If true a boundary conditions for the present transport
c               equation has been set for the present segment
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISRDERR   Give an error message for a set of subfaces
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     572   Normal and/or tangential velocity component has already been
c           prescribed
c     580   Scalar in transport equation has already been prescribed
c     581   Keyword alpha or h not found after robbins or mixed
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c

      call iseropen ( 'isrdbou5' )

      sset = .false.

      if ( table(14).gt.-1d19 .or. table(17).gt.-1d19 ) then

c     --- periodic or anti-symmetric periodic found

         if ( table(14).gt.-1d19 ) then
            itypebc(ndim+itrans,1) = 98
            itypebc(ndim+itrans,2) = abs(ineigh(2,nint(table(14))) )
            itypebc(ndim+itrans,3) = abs(ineigh(3,nint(table(14))) )
         else
            itypebc(ndim+itrans,1) = 97
         end if
         if ( table(14).gt.-1d19 .and.
     +        table(17).gt.-1d19 ) then
            call isrderr( 572, code(17), itrans, nsfread, isfread)
         end if
         sset = .true.

      end if

      if ( table(18).gt.-1d19 ) then

c     --- dirichlet found

         itypebc(ndim+itrans,1) = 1
         if ( table(18).gt.0 .or. table(18).lt.-100) then
            itypebc(ndim+itrans,2) = nint(table(18))
         else
            itypebc(ndim+itrans,2) = -1
            rtypebc(ndim+itrans,1) = tabout(-nint(table(18)))
         end if
         sset = .true.

      end if

      if ( table(19).gt.-1d19 ) then

c     --- neumann found

         itypebc(ndim+itrans,1) = 2
         itypebc(ndim+itrans,2) = -1
         if ( table(19).gt.0 .or. table(19).lt.-100) then
            itypebc(ndim+itrans,3) = nint(table(19))
         else
            itypebc(ndim+itrans,3) = -1
            rtypebc(ndim+itrans,2) = tabout(-nint(table(19)))
         end if
         if ( sset ) then
            call isrderr( 580, code(19), itrans, nsfread, isfread)
         end if
         sset = .true.

      end if

      if ( table(20).gt.-1d19 .or. table(23).gt.-1d19 ) then

c     --- robbins or mixed found

         itypebc(ndim+itrans,1) = 2
         itypebc(ndim+itrans,2) = -1
         if ( table(21).lt.-1d19 ) then
            call isrderr( 581, code(21), itrans, nsfread, isfread)
         else if ( table(21).gt.0 .or. table(21).lt.-100) then
            itypebc(ndim+itrans,2) = nint(table(21))
         else
            itypebc(ndim+itrans,2) = -1
            rtypebc(ndim+itrans,1) = tabout(-nint(table(21)))
         end if
         if ( table(22).lt.-1d19 ) then
            call isrderr( 581, code(21), itrans, nsfread, isfread)
         else if ( table(22).gt.0 .or. table(22).lt.-100) then
            itypebc(ndim+itrans,3) = nint(table(22))
         else
            itypebc(ndim+itrans,3) = -1
            rtypebc(ndim+itrans,2) = tabout(-nint(table(22)))
         end if
         if ( sset .or. (table(20).gt.-1d19 .and.
     +      table(23).gt.-1d19) ) then
            call isrderr( 580, code(20), itrans, nsfread, isfread)
         end if
         sset = .true.

      end if

      if ( table(35).gt.-1d19 ) then

c     --- wall_temperature found

         itypebc(ndim+itrans,1) = 34
         if ( table(35).gt.0 .or. table(35).lt.-100) then
            itypebc(ndim+itrans,2) = nint(table(35))
         else
            itypebc(ndim+itrans,2) = -1
            itypebc(ndim+itrans,3) = -1
            rtypebc(ndim+itrans,1) = tabout(-nint(table(35)))
         end if
         sset = .true.

      end if

      call iserclos ( 'isrdbou5' )

      end
