      subroutine isrdcav ( nmain, codemn, nmcode, iretur, iincav,
     +                     rincav )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.5    date 03-04-1999 Duncan: set nncode to 7
c                                       change default values
c        version 1.4    date 03-03-1999 New definition of rincav
c        version 1.3    date 23-11-1998 New definition of rincav
c        version 1.2    date 29-07-1998 Extension with iincav(2)
c        version 1.1    date 31-04-1998 Extension with iincav(1)
c        version 1.0    date 07-09-1997
c
c   copyright (c) 1997-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Read cavity part of input files and store result in
c    arrays iincav and rincav
c **********************************************************************
c
c                       KEYWORDS
c
c       read
c       read_cards
c       cavity
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer nmain, nmcode(nmain), iretur, iincav(100)
      double precision rincav(100)
      character *(*) codemn(nmain)

c     codemn    i    Table of main keywords
c     iincav   i/o   reserved
c     iretur    o    Return code, indicates the sequence number of the new
c                    main keyword found
c                    If -1 an error has occurred
c                    If -2 the end of file has been reached
c     nmain     i    Number of keywords in array codemn
c     nmcode    i    Array containing the number of significant characters
c                    for each main keyword in codemn
c     rincav   i/o   reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer nncode
      parameter ( nncode=7 )
      integer itype(nncode), ncodmx(nncode), nout,
     +        ncode(nncode),  nstrng, i
      double precision table(nncode), tabmin(nncode), tabmax(nncode),
     +                 tabout(1)
      character *21 code(nncode)
      character *1 strout(1)

c     code      Table containing the subkeywords corresponding to this
c               subroutine.
c     i         General loop variable
c     itype     Array containing the type of actions to be undertaken for
c               each subkeyword found.
c               See subroutine isintx02 for the possible values of itype
c     ncode     Array containing the number of significant characters for
c               each keyword in code
c     ncodmx    Array containing the maximum number of characters for
c               each keyword in code
c     nncode    Number of subkeywords stored in array code
c     nout      Length of array tabout
c     nstrng    Length of array strout
c     strout    Contains series of strings, see itype
c     table     Array containing the real or integer values corresponding
c               to the subkeywords in code that have been read by isintx02
c     tabmax    Array containing the maximal values of the parameters read
c               corresponding to the subkeywords in code
c     tabmin    Array containing the minimum values of the parameters read
c               corresponding to the subkeywords in code
c     tabout    Array of output reals provided by isintx02
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERREAL  Put real value in error message
c     ISERRSUB  Error messages
c     ISINTX02  Interpret a record containing keywords followed by numbers
c **********************************************************************
c
c                       I/O
c
c    This subroutine reads the input after the main keyword COMPRESSIBLE
c    until a new main keyword has been found
c
c    The following input is recognized:
c
c    CVAPOUR = p
c    DELTARHO = d
c    CLIQUID = a
c    LOWERPRESSTRANSIT = a
c    UPPERPRESSTRANSIT = a
c    PRESSURE_GAUSS_SEIDEL
c    START_UP
c **********************************************************************
c
c                       ERROR MESSAGES
c
c    2613   lower bound pressure transition larger than upper bound
c **********************************************************************
c
c                       PSEUDO CODE
c
c    Read subkeywords until a new main keyword is found
c    interpret subkeywords
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data code  / 'CLIQUID', 'CVAPOUR', 'LOWERPRESSTRANSIT',
     +             'UPPERPRESSTRANSIT', 'DELTARHO',
     +             'PRESSURE_GAUSS_SEIDEL', 'START_UP' /
      data ncode  / 4, 4, 7,  7,  5,11, 4 /
      data ncodmx / 7, 8,17, 17,  8,21, 8 /
      data itype  / 1, 1, 1,  1,  1, 2, 2 /
      data tabmin /  1d-6, 1d-6, -1d6, -1d6,  0.1d0, 0d0,  0d0 /
      data tabmax /   1d6,  1d6,  1d6,  1d6,   1d3,  0d0,  0d0 /

c     code      Table containing the subkeywords corresponding to this
c               subroutine. Contents:
c               1:  CLIQUID
c               2:  CVAPOUR
c               3:  LOWERPRESSTRANSIT
c               4:  UPPERPRESSTRANSIT
c               5:  DELTARHO
c               6:  PRESSURE_GAUSS_SEIDEL
c               7:  START_UP
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen ( 'isrdcav' )

c     --- Read subkeywords until a main keyword is found

      table(1) = 4.4444d-1
      table(2) = 4.0d0
      table(3) = 0.4d0
      table(4) = 0.7d0
      table(5) = 4d0
      table(6) = 0d0
      table(7) = 0d0
      nout = 1
      nstrng = 1
      call isintx02 ( code, ncode, nncode, itype, table, codemn,
     +                nmcode, nmain, iretur, tabmin, tabmax,
     +                tabout, nout, strout, nstrng, ncodmx )

c     --- interpret subkeywords

      do i = 1, 5
         rincav(i) = table(i)
      end do
      if ( rincav(4).lt.rincav(3) ) then

c     --- Error upper bound smaller than lower bound

         call iserreal ( rincav(4), 1 )
         call iserreal ( rincav(3), 2 )
         call iserrsub ( 2613, 0, 2, 0 )

      end if
      if ( table(6).gt.0d0 ) iincav(1) = 1
      if ( table(7).gt.0d0 ) iincav(2) = 1

      call iserclos ( 'isrdcav' )
      end
