      subroutine isrdcomp ( nmain, codemn, nmcode, iretur, iincom,
     +                      rincom )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.16   date 15-07-1999 Guus: extra keywords
c        version 1.15   date 04-04-1999 Duncan: Extension with DEFCORR
c        version 1.14   date 23-03-1999 Duncan: Extension with SUPERSCN
c        version 1.13   date 19-03-1999 Duncan: Extension with SUPERSONIC
c        version 1.12   date 11-08-1998 Guus: Extension with explicit/implicit
c        version 1.11   date 29-07-1998 Guus: Extension with subkeywords
c                                             corresponding to dens_upwind
c        version 1.10   date 03-03-1998 Guus: Extension of RINCOM
c        version 1.9    date 01-12-1997 Hester: Extension with PRES_CORR_METHOD
c        version 1.8    date 12-11-1997 Hester: Extension with NO_SCALING
c        version 1.7    date 29-07-1997 Guus: Extension with NO_ENTHALPY
c        version 1.6    date 03-07-1997 Hester: correction compr. density upw.
c        version 1.5    date 09-06-1997 Guus: Extension with multiphase gas flow
c        version 1.4    date 19-02-1996 Guus: UPWIND added
c        version 1.3    date 12-08-1995 Guus: initialize ncodmx
c        version 1.2    date 30-05-1995 Hester: changed some values.
c        version 1.1    date 08-03-1995 Kees K:values of table(5),table(6)
c                                            interchanged
c        version 1.0    date 07-02-1995
c
c   copyright (c) 1995-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Read compressible part of input files and store result in
c    arrays iincom and rincom
c **********************************************************************
c
c                       KEYWORDS
c
c       read
c       read_cards
c       compressible
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer nmain, nmcode(nmain), iretur, iincom(100)
      double precision rincom(100)
      character *(*) codemn(nmain)

c     codemn         i    Table of main keywords
c     iincom        i/o   Array IINCOM is part of array IINPUT and hence
c                         coupled to one block only. It contains information
c                         with respect to the compressible flow
c                         For a description see the programmers guide
c     iretur        i/o   Return code, indicates the sequence number of the new
c                         main keyword found
c                         If -1 an error has occurred
c                         If -2 the end of file has been reached
c     nmain          i    Number of keywords in array codemn
c     nmcode         i    Array containing the number of significant characters
c                         for each main keyword in codemn
c     rincom        i/o   Array RINCOM is part of array IINPUT and hence
c                         coupled to one block only. It contains information
c                         with respect to the compressible flow
c                         For a description see the programmers guide
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer nncode, nmainnw, nncod2, nout, lenchar
      parameter ( nmainnw=75, nncode=24, nncod2=5, nout=1, lenchar=14 )
      integer itype(nncode), nmcoda(nmainnw), ncodmx(nncode),
     +        ncode(nncode), nmcode2(nncod2), nstrng, i, nmmaintot
      double precision table(nncode), tabmin(nncode), tabmax(nncode),
     +                 tabout(nout)
      character *20 code(nncode), code2(nncod2)
      character *14 codema(nmainnw)
      character *1 strout(nout)

c     code           Table containing the subkeywords corresponding to this
c                    subroutine.
c     code2          Table containing the  subsubkeywords corresponding to
c                    this subroutine.
c     codema         Table consisting of the local main keywords followed
c                    by the general main keywords possibly followed by the
c                    subsubkeywords
c     i              General loop variable
c     itype          Array containing the type of actions to be undertaken
c                    for each subkeyword found.
c                    See subroutine isintx02 for the possible values
c                    of itype
c     lenchar        Number of characters in main keywords
c     ncode          Array containing the number of significant characters
c                    for each keyword in code
c     ncodmx         Array containing the maximum number of characters for
c                    each keyword in code
c     nmainnw        Maximal number of keywords in codema
c     nmcoda         Array containing the number of significant characters
c                    for each keyword in codema
c     nmcode2        Array containing the number of significant characters
c                    for each keyword in code2
c     nmmaintot      Total number of keywords in codema
c     nncod2         Number of subsubkeywords stored in array code2
c     nncode         Number of subkeywords stored in array code
c     nout           Length of array tabout
c     nstrng         Length of array strout
c     strout         Contains series of strings, see itype
c     table          Array containing the real or integer values
c                    corresponding to the subkeywords in code that
c                    have been read by isintx02
c     tabmax         Array containing the maximal values of the parameters
c                    read corresponding to the subkeywords in code
c     tabmin         Array containing the minimum values of the parameters
c                    read corresponding to the subkeywords in code
c     tabout         Array of output reals provided by isintx02
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS       Resets old name of previous subroutine of higher level
c     ISEROPEN       Produces concatenated name of local subroutine
c     ISERRCHR       Puts values of strings in character error array
c     ISERRINT       Put integer value in error message
c     ISERRSUB       Error messages
c     ISINTX02       Interpret a record containing keywords followed by numbers
c     ISSTOP         Stop ISNAS
c **********************************************************************
c
c                       I/O
c
c    This subroutine reads the input after the main keyword COMPRESSIBLE
c    until a new main keyword has been found
c
c    The following input is recognized:
c
c    GAMMA = g
c    MACH = m
c    VEL_SOUND = v
c    ALPHA = a
c    P_V = p
c    P_OUT = p
c    DENS_UPWIND = n,m
c       with n = GRADIENT_BASED
c                MACH_BASED      (default)
c                UNCONDITIONAL
c                SUPERSONIC
c                SUPERSCN
c                SUPERSDC
c       with m = IMPLICIT        (default)
c                EXPLICIT
c    NO_DENS_UPWIND
c    NO_ENTHALPY
c    NO_SCALING
c    PRES_CORR_METHOD = i
c    P_REF = p
c    RHO_REF = rho
c    VEL_REF = v
c    ENERGY_EQUATION = m
c       with m = ENTHALPY      (default)
c                TOTAL_ENTHALPY
c                INTERNAL_ENERGY
c    EQUATION_OF_STATE = m
c       with m = PERFECT_GAS     (default)
c                USER_PROVIDED
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     531   Internal error in subroutine; character array too small
c     587   two keywords found that are mutually exclusive
c     592   mutually exclusive keywords of a set of three were found
c     718   Subkeyword is used without the combination of the corresponding
c           keyword
c **********************************************************************
c
c                       PSEUDO CODE
c
c    Read subkeywords until a new main keyword is found
c    interpret subkeywords
c    Fill nturb in idimens
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data code  / 'GAMMA', 'MACH_BASED', 'VEL_SOUND', 'ALPHA', 'P_V',
     +             'P_OUT', 'DENS_UPWIND', 'NO_DENS_UPWIND',
     +             'NO_ENTHALPY', 'NO_SCALING', 'PRES_CORR_METHOD',
     +             'P_REF', 'RHO_REF', 'VEL_REF', 'GRADIENT_BASED',
     +             'MACH', 'UNCONDITIONAL', 'IMPLICIT', 'EXPLICIT',
     +             'SUPERSONIC', 'SUPERSCN', 'SUPERSDC',
     +             'ENERGY_EQUATION', 'EQUATION_OF_STATE' /
      data ncode  / 5, 8, 5, 3, 3,   5, 8, 11,  6, 6,
     +              8, 5, 7, 7,10,   4, 8,  5,  5 ,10,
     +              8, 8, 6, 8/
      data ncodmx / 5,10, 9, 5, 3,   5,11, 14, 11,10,
     +             16, 5, 7, 7,14,   4,13,  8,  8,10,
     +              8, 8,15,17/
      data itype  / 1, 2, 1, 1, 1,   1, 2,  2,  2, 2,
     +              0, 1, 1, 1, 2,   1, 2,  2,  2, 2,
     +              2, 2, 5, 5/
      data tabmin / 1d0, 0d0, 0d0,-90d0, 0d0,
     +              0d0, 0d0, 0d0,  0d0, 0d0,
     +              0d0, 0d0,1d-4, 1d-4, 0d0,
     +              0d0, 0d0, 0d0, 0d0, 0d0,
     +              0d0, 0d0, 0d0, 0d0/
      data tabmax / 2d0, 0d0, 6d3, 90d0, 1d5,
     +              3d7, 0d0, 0d0,  0d0, 0d0,
     +              1d0, 1d7, 1d4,  1d4, 0d0,
     +             10d0, 0d0, 0d0, 0d0, 0d0,
     +              0d0, 0d0, 0d0, 0d0/
      data code2 / 'ENTHALPY', 'TOTAL_ENTHALPY', 'INTERNAL_ENERGY',
     +             'PERFECT_GAS', 'USER_PROVIDED' /
      data nmcode2 / 4,10, 5, 8, 8 /

c     code      Table containing the subkeywords corresponding to this
c               subroutine. Contents:
c               1:  GAMMA
c               2:  MACH_BASED
c               3:  VEL_SOUND
c               4:  ALPHA
c               5:  P_V
c               6:  P_OUT
c               7:  DENS_UPWIND
c               8:  NO_DENS_UPWIND
c               9:  NO_ENTHALPY
c              10:  NO_SCALING
c              11:  PRES_CORR_METHOD
c              12:  P_REF
c              13:  RHO_REF
c              14:  VEL_REF
c              15:  GRADIENT_BASED
c              16:  MACH
c              17:  UNCONDITIONAL
c              18:  IMPLICIT
c              19:  EXPLICIT
c              20:  SUPERSONIC
c              21:  SUPERSCN
c              22:  SUPERSDC
c              23:  ENERGY_EQUATION
c              24:  EQUATION_OF_STATE
c     code2     Table containing the subkeywords corresponding to this
c               subroutine. Contents:
c               1:  ENTHALPY
c               2:  TOTAL_ENTHALPY
c               3:  INTERNAL_ENERGY
c               4:  PERFECT_GAS
c               5:  USER_PROVIDED
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen ( 'isrdcomp' )
      nmmaintot = nmain+nncod2
      if ( nmmaintot.gt.nmainnw ) then

c     --- Error nmainnw too small

         call iserrint ( nmainnw, 1 )
         call iserrint ( nmain, 2 )
         call iserrint ( 0, 3 )
         call iserrsub ( 531, 3, 0, 0 )
         call isstop

      end if

      do i = 1, nmain
         codema(i)(1:lenchar) = codemn(i)(1:lenchar)
         nmcoda(i) = nmcode(i)
      end do
      do i = 1, nncod2
         codema(i+nmain)(1:lenchar) = code2(i)(1:lenchar)
         nmcoda(i+nmain) = nmcode2(i)
      end do
      if ( iretur.eq.14 ) then
         iincom(1) = 1
      else
         iincom(1) = 3
      end if

c     --- Read subkeywords until a main keyword is found

      do i = 1, nncode
         table(i) = 0d0
      end do
      table(1) = 1.4d0
      table(3) = 330d0
      table(5) = 1d0
      table(6) = 1d5
      table(13) = 1d0
      table(14) = 1d0
      table(23) = 1d0
      table(24) = 1d0
      tabmin(23) = nmain+1
      tabmax(23) = tabmin(23)+2
      tabmin(24) = tabmax(23)+1
      tabmax(24) = tabmin(24)+1
      nstrng = 1
      call isintx02 ( code, ncode, nncode, itype, table, codema,
     +                nmcoda, nmain, iretur, tabmin, tabmax,
     +                tabout, nout, strout, nstrng, ncodmx )

c     --- interpret subkeywords

      rincom(1) = table(1)
      rincom(2) = table(16)
      do i = 3, 6
         rincom(i) = table(i)
      end do
      if ( nint(table(7)).eq.1 ) then
         if ( nint(table(8)).eq.1 ) then
            call iserrchr ( code(7), 1 )
            call iserrchr ( code(8), 2 )
            call iserrsub ( 587, 0, 0, 2 )
         end if
         iincom(2) = 1
      else
         iincom(2) = 0
      end if
      iincom(4) = 0
      if ( nint(table(9)).eq.1 ) iincom(4) = 1
      if ( nint(table(10)).eq.1 ) iincom(5) = 1
      iincom(6) = nint(table(11))
      do i = 7, 9
         rincom(i) = table(i+5)
      end do
      iincom(7) = 0
      if ( table(15).gt.0 ) then
         if ( iincom(2).eq.0 ) then
            call iserrchr ( code(15), 1 )
            call iserrchr ( code(7), 2 )
            call iserrsub ( 718, 0, 0, 2 )
         end if
         iincom(7) = 1
      end if
      if ( table(2).gt.0 ) then
         if ( iincom(2).eq.0 ) then
            call iserrchr ( code(2), 1 )
            call iserrchr ( code(7), 2 )
            call iserrsub ( 718, 0, 0, 2 )
         end if
         if ( iincom(7).gt.0 ) then
            call iserrchr ( code(15), 1 )
            call iserrchr ( code( 2), 2 )
            call iserrchr ( code(17), 3 )
            call iserrchr ( code(20), 4 )
            call iserrchr ( code(21), 5 )
            call iserrchr ( code(22), 6 )
            call iserrsub ( 592, 0, 0, 5 )
         end if
         iincom(7) = 2
      end if
      if ( table(17).gt.0 ) then
         if ( iincom(2).eq.0 ) then
            call iserrchr ( code(17), 1 )
            call iserrchr ( code(7), 2 )
            call iserrsub ( 718, 0, 0, 2 )
         end if
         if ( iincom(7).gt.0 ) then
            call iserrchr ( code(15), 1 )
            call iserrchr ( code( 2), 2 )
            call iserrchr ( code(17), 3 )
            call iserrchr ( code(20), 4 )
            call iserrchr ( code(21), 5 )
            call iserrchr ( code(22), 6 )
            call iserrsub ( 592, 0, 0, 6)
         end if
         iincom(7) = 3
      end if
      if ( table(20).gt.0 ) then
         if ( iincom(2).eq.0 ) then
            call iserrchr ( code(20), 1 )
            call iserrchr ( code(7), 2 )
            call iserrsub ( 718, 0, 0, 2 )
         end if
         if ( iincom(7).gt.0 ) then
            call iserrchr ( code(15), 1 )
            call iserrchr ( code(2),  2 )
            call iserrchr ( code(17), 3 )
            call iserrchr ( code(20), 4 )
            call iserrchr ( code(21), 5 )
            call iserrchr ( code(22), 6 )
            call iserrsub ( 592, 0, 0, 6 )
         end if
         iincom(7) = 4
      end if
      if ( table(21).gt.0 ) then
         if ( iincom(2).eq.0 ) then
            call iserrchr ( code(21), 1 )
            call iserrchr ( code(7), 2 )
            call iserrsub ( 718, 0, 0, 2 )
         end if
         if ( iincom(7).gt.0 ) then
            call iserrchr ( code(15), 1 )
            call iserrchr ( code(2),  2 )
            call iserrchr ( code(17), 3 )
            call iserrchr ( code(20), 4 )
            call iserrchr ( code(21), 5 )
            call iserrchr ( code(22), 6 )
            call iserrsub ( 592, 0, 0, 6 )
         end if
         iincom(7) = 5
      end if
      if ( table(22).gt.0 ) then
         if ( iincom(2).eq.0 ) then
            call iserrchr ( code(22), 1 )
            call iserrchr ( code(7), 2 )
            call iserrsub ( 718, 0, 0, 2 )
         end if
         if ( iincom(7).gt.0 ) then
            call iserrchr ( code(15), 1 )
            call iserrchr ( code(2),  2 )
            call iserrchr ( code(17), 3 )
            call iserrchr ( code(20), 4 )
            call iserrchr ( code(21), 5 )
            call iserrchr ( code(22), 6 )
            call iserrsub ( 592, 0, 0, 6 )
         end if
         iincom(7) = 6
      end if
      if ( iincom(7).eq.0 ) iincom(7) = 2
      iincom(8) = 0
      if ( table(18).gt.0 ) then
         if ( iincom(2).eq.0 ) then
            call iserrchr ( code(18), 1 )
            call iserrchr ( code(7), 2 )
            call iserrsub ( 718, 0, 0, 2 )
         end if
      end if
      if ( table(19).gt.0 ) then
         if ( iincom(2).eq.0 ) then
            call iserrchr ( code(19), 1 )
            call iserrchr ( code(7), 2 )
            call iserrsub ( 718, 0, 0, 2 )
         end if
         if ( table(18).gt.0 ) then
            call iserrchr ( code(18), 1 )
            call iserrchr ( code(19), 2 )
            call iserrsub ( 587, 0, 0, 2 )
         end if
         iincom(8) = 1
      end if
      iincom(10) = nint(table(23))
      iincom(9) = nint(table(24))-1
      if ( iincom(4).eq.1 ) iincom(10) = 0

      call iserclos ( 'isrdcomp' )
      end
