      subroutine isrdfree ( nmain, codemn, nmcode, iretur, iinfrsur,
     +                      rinfrsur )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.1    date 29-04-1999 Extension with keyword relaxation
c        version 1.0    date 18-03-1999
c
c   copyright (c) 1999-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Read free surface part of input files and store result in
c    arrays iinfrsur and rinfrsur
c **********************************************************************
c
c                       KEYWORDS
c
c       read
c       read_cards
c       free_surface
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer nmain, nmcode(nmain), iretur, iinfrsur(100)
      double precision rinfrsur(100)
      character *(*) codemn(nmain)

c     codemn    i    Table of main keywords
c     iinfrsur i/o   Contains integer input with respect to the free surface
c                    computation
c     iretur    o    Return code, indicates the sequence number of the new
c                    main keyword found
c                    If -1 an error has occurred
c                    If -2 the end of file has been reached
c     nmain     i    Number of keywords in array codemn
c     nmcode    i    Array containing the number of significant characters
c                    for each main keyword in codemn
c     rinfrsur i/o   Contains real input with respect to the free surface
c                    computation
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer nmainnw, lenchar, nncode, nncod2
      parameter ( nmainnw=45, nncode=2, lenchar=14, nncod2=2 )
      integer nmcoda(nmainnw), itype(nncode), nmmaintot,
     +        nmcode2(nncod2), ncodmx(nncode), nout,
     +        ncode(nncode),  nstrng, i
      double precision table(nncode), tabmin(nncode), tabmax(nncode),
     +                 tabout(1)
      character *14 codema(nmainnw), code2(nncod2)
      character *21 code(nncode)
      character *1 strout(1)

c     code      Table containing the subkeywords corresponding to this
c               subroutine.
c     code2     Table containing the  subsubkeywords corresponding to this
c               subroutine.
c     codema    Table consisting of the local main keywords followed by the
c               general main keywords possibly followed by the subsubkeywords
c     i         General loop variable
c     itype     Array containing the type of actions to be undertaken for
c               each subkeyword found.
c               See subroutine isintx02 for the possible values of itype
c     lenchar   Number of characters in main keywords
c     ncode     Array containing the number of significant characters for
c               each keyword in code
c     ncodmx    Array containing the maximum number of characters for
c               each keyword in code
c     nmainnw   Maximal number of keywords in codema
c     nmcoda    Array containing the number of significant characters for
c               each keyword in codema
c     nmcode2   Array containing the number of significant characters for
c               each keyword in code2
c     nmmaintot Total number of keywords in codema
c     nncod2    Number of subsubkeywords stored in array code2
c     nncode    Number of subkeywords stored in array code
c     nout      Length of array tabout
c     nstrng    Length of array strout
c     strout    Contains series of strings, see itype
c     table     Array containing the real or integer values corresponding
c               to the subkeywords in code that have been read by isintx02
c     tabmax    Array containing the maximal values of the parameters read
c               corresponding to the subkeywords in code
c     tabmin    Array containing the minimum values of the parameters read
c               corresponding to the subkeywords in code
c     tabout    Array of output reals provided by isintx02
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c     ISINTX02  Interpret a record containing keywords followed by numbers
c     ISSTOP    Stop ISNAS
c **********************************************************************
c
c                       I/O
c
c    This subroutine reads the input after the main keyword COMPRESSIBLE
c    until a new main keyword has been found
c
c    The following input is recognized:
c
c    METHOD = m                    Default: none
c             with m = none
c                      plain
c    RELAXATION = a                Default: 1
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     531   Internal error in subroutine; character array too small
c **********************************************************************
c
c                       PSEUDO CODE
c
c    Read subkeywords until a new main keyword is found
c    interpret subkeywords
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data code  / 'METHOD', 'RELAXATION' /
      data ncode  / 4, 5 /
      data ncodmx / 6,10 /
      data itype  / 5, 1 /
      data tabmin / 0d0, 1d-10 /
      data tabmax / 0d0, 2d0 /
      data code2 / 'NONE', 'PLAIN' /
      data nmcode2 / 3, 3 /

c     code      Table containing the subkeywords corresponding to this
c               subroutine. Contents:
c               1:  METHOD
c     code2     Table containing the  subsubkeywords corresponding to this
c               subroutine. Contents:
c               1:  NONE
c               2:  PLAIN
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen ( 'isrdfree' )

c     --- Make table of new main keywords consisting of the local main keywords
c         and the global main keywords and the subsubkeywords

      nmmaintot = nmain+nncod2
      if ( nmmaintot.gt.nmainnw ) then

c     --- Error nmainnw too small

         call iserrint ( nmainnw, 1 )
         call iserrint ( nmain, 2 )
         call iserrint ( 0, 3 )
         call iserrsub ( 531, 3, 0, 0 )
         call isstop

      end if

      do i = 1, nmain
         codema(i)(1:lenchar) = codemn(i)(1:lenchar)
         nmcoda(i) = nmcode(i)
      end do
      do i = 1, nncod2
         codema(i+nmain)(1:lenchar) = code2(i)(1:lenchar)
         nmcoda(i+nmain) = nmcode2(i)
      end do

c     --- Set tabmin and tabmax with references of sub subkeywords
c         stored in codema

      tabmin(1) = nmain+1
      tabmax(1) = tabmin(1)+1

c     --- Read subkeywords until a main keyword is found

      table(1) = 0d0
      table(2) = 1d0
      nout = 1
      nstrng = 1
      call isintx02 ( code, ncode, nncode, itype, table, codema,
     +                nmcoda, nmain, iretur, tabmin, tabmax,
     +                tabout, nout, strout, nstrng, ncodmx )

c     --- interpret subkeywords

      if ( table(1).gt.0d0 ) then

c     --- METHOD has been found

         iinfrsur(1) = nint(table(1))-1

      end if
      rinfrsur(1) = table(2)

      call iserclos ( 'isrdfree' )
      end
