      subroutine isrdstruc ( nmain, codemn, nmcode, iretur, iglobstr,
     +                       rglobstr, ilocstr, rlocstr, maxclust,
     +                       iseqeq, nclusters, icluster, lncluster )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.0    date 19-04-1999
c
c
c   copyright (c) 1999-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Read part of input files with respect to the structure of the main program
c    Store information in arrays iglobstr, rglobstr, ilocstr and rlocstr
c **********************************************************************
c
c                       KEYWORDS
c
c       structure
c       read
c       read_cards
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer nmain, nmcode(nmain), iretur, maxclust, nclusters,
     +        iglobstr(50), ilocstr(maxclust,20), lncluster,
     +        icluster(*), iseqeq(*)
      double precision rglobstr(10), rlocstr(maxclust,10)
      character *(*) codemn(nmain)

c     codemn    i    Table of main keywords
c     icluster  o    Array in which information about the clusters is stored
c                    icluster(i), i=1(1)nclusters contains the starting
c                    position of information about cluster i
c                    In this starting position the number of equations is
c                    stored, followed by the equation sequence numbers
c     iglobstr  o    Contains information of the global structure of the program
c                    See the programmers Guide for a description
c     ilocstr   o    Contains information of the structure of the main program
c                    for each cluster
c                    See the programmers Guide for a description
c     iretur    i    Return code, indicates the sequence number of the new
c                    main keyword found
c                    If -1 an error has occurred
c                    If -2 the end of file has been reached
c     iseqeq    i    Array with sequence numbers of equations
c     lncluster i    Maximum length of array icluster
c     maxclust  i    Maximum number of clusters
c     nclusters o    Number of clusters
c     nmain     i    Number of keywords in array codemn
c     nmcode    i    Array containing the number of significant characters
c                    for each main keyword in codemn
c     rglobstr  o    Contains information of the global structure of the program
c                    See the programmers Guide for a description
c     rlocstr   o    Contains information of the structure of the main program
c                    for each cluster
c                    See the programmers Guide for a description
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cseprd'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer nmainnw, nncod1, lenchar, nncod3, nncod2, nout, maxeq
      parameter ( nmainnw=45, nncod1=2, nncod2=8, nncod3=7,
     +            lenchar=14, nout=112, maxeq=112 )
      integer nmcoda(nmainnw), nstrng,
     +        itype(1), nmcode1(nncod1), ncode1(1),
     +        itype2(nncod2), ncodmx2(nncod2), ncode2(nncod2),
     +        itype3(nncod3), ncodmx3(nncod3), ncode3(nncod3),
     +        i, ih, iseq, ihelp(maxeq), nmain1, numsub, ihelp1(maxeq),
     +        icheckeq(maxeq), ipcluster, m, iposut
      double precision table1(1), tabmin1(1), tabmax1(1),
     +                 table2(nncod2), tabmin2(nncod2), tabmax2(nncod2),
     +                 table3(nncod3), tabmin3(nncod3), tabmax3(nncod3),
     +                 tabout(nout)
      character *14 codema(nmainnw), code1(nncod1)
      character *25 code2(nncod2), code3(nncod3)
      character *1 strout(1)
      logical finished, globloop

c     code1     Table containing the  main keywords corresponding to this
c               subroutine.
c     code2     Table containing the  subsubkeywords corresponding to this
c               subroutine.
c     code3     Table containing the  subsubkeywords corresponding to this
c               subroutine.
c     codema    Table consisting of the local main keywords followed by the
c               general main keywords possibly followed by the subsubkeywords
c     finished  if finished=1 convergence is achieved to the required
c               accuracy, if  finished=0 iteration has to be continued
c     globloop  If true the keyword GLOBAL_LOOP has been read
c     i         General loop variable
c     icheckeq  Work array to indicate if an equation has been considered
c               (icheckeq(i)=1 or not icheckeq(i)=0)
c     ih        Help variable to store a constant temporarily
c     ihelp     Help array to store number of appearances of keywords that
c               may be used more than once or number of numbers read
c     ihelp1    Help array to store some quantities
c     ipcluster Starting address of cluster in array icluster
c     iposut    last position used in array tabout
c     iseq      Sequence number read after transport or turbulence equation
c     itype     Array containing the type of actions to be undertaken for
c               each subkeyword found.
c               See subroutine isintx02 for the possible values of itype
c     itype2    Array containing the type of actions to be undertaken for
c               each subkeyword corresponding to code2 found.
c               See subroutine isintx02 for the possible values of itype2
c     itype3    Array containing the type of actions to be undertaken for
c               each subkeyword corresponding to code3 found.
c               See subroutine isintx02 for the possible values of itype3
c     lenchar   Number of characters in main keywords
c     m         General loop variable
c     maxeq     Maximum number of equations allowed
c     ncode1    Array containing the number of significant characters for
c               each keyword in code1
c     ncode2    Array containing the number of significant characters for
c               each keyword in code2
c     ncode3    Array containing the number of significant characters for
c               each keyword in code3
c     ncodmx2   Array containing the maximum number of characters for
c               each keyword in code2
c     ncodmx3   Array containing the maximum number of characters for
c               each keyword in code3
c     nmain1    Total number of main keywords (newmain+nmain)
c     nmainnw   Maximal number of keywords in codema
c     nmcoda    Array containing the number of significant characters for
c               each keyword in codema
c     nmcode1   Array containing the number of significant characters for
c               each keyword in code1
c     nncod1    Number of subsubkeywords stored in array code1
c     nncod2    Number of subsubkeywords stored in array code2
c     nncod3    Number of subsubkeywords stored in array code3
c     nout      Length of array tabout
c     nstrng    Length of array strout
c     numsub    Number of subkeywords that may be detected by isintx02
c               In the first step numsub=0 for any other step numsub=nncode
c     strout    Contains series of strings, see itype
c     table1    Array containing the real or integer values corresponding
c               to the subkeywords in code1  that have been read by isintx02
c     table2    Array containing the real or integer values corresponding
c               to the subkeywords in code2 that have been read by isintx02
c     table3    Array containing the real or integer values corresponding
c               to the subkeywords in code3 that have been read by isintx02
c     tabmax1   Array containing the maximal values of the parameters read
c               corresponding to the subkeywords in code1
c     tabmax2   Array containing the maximal values of the parameters read
c               corresponding to the subkeywords in code2
c     tabmax3   Array containing the maximal values of the parameters read
c               corresponding to the subkeywords in code3
c     tabmin1   Array containing the minimum values of the parameters read
c               corresponding to the subkeywords in code1
c     tabmin2   Array containing the minimum values of the parameters read
c               corresponding to the subkeywords in code2
c     tabmin3   Array containing the minimum values of the parameters read
c               corresponding to the subkeywords in code3
c     tabout    Array of output reals provided by isintx02
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISASKNXT  Read next item from the standard input file
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRCHR  Puts values of strings in character error array
c     ISERRCRD  Prints the last record read and underlines the errorneous part
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c     ISINTX02  Interpret a record containing keywords followed by numbers
c     ISWARSUB  Print warnings
c **********************************************************************
c
c                       I/O
c
c    This subroutine reads the input after the main keyword main_structure
c    until a new main keyword has been found
c
c    The following input is recognized:
c
c    GLOBAL_LOOP, followed by subkeywords about the global loop
c       These subkeywords must start on a new record
c    CLUSTER i, followed by subkeywords with information about the cluster
c       These subkeywords must start on a new record
c
c    The following subkeywords for GLOBAL_LOOP are recognized:
c
c       sequence_of_equations = ( i,j,k,... )
c       iterate_initial_condition
c       min_initial_iter = m
c       max_initial_iter = m
c       abs_initial_accuracy = e
c       rel_initial_accuracy = e
c       min_stationary_iter = m
c       max_stationary_iter = m
c
c    The following subkeywords for CLUSTER i are recognized:
c
c       equations = ( i,j,k,... )
c       stationary
c       abs_stationary_accuracy = e
c       rel_stationary_accuracy = e
c       min_stationary_iter = m
c       max_stationary_iter = m
c       nsubsteps = n
c **********************************************************************
c
c                       ERROR MESSAGES
c
c      17   internal error: array size too large
c     528   No integer number has been found after the keyword %
c     529   Keyword found more than once
c     530   integer number read out of range
c     531   Internal error in subroutine; character array too small
c     614   number appears more than once in the list of equation numbers
c     615   cluster number appears more than once
c     616   equation number appears more than once
c     617   Cluster has not been filled
c **********************************************************************
c
c                       PSEUDO CODE
c
c    Make table of new main keywords consisting of the local main keywords
c    and the global main keywords
c    Read first record (main keyword)
c    finished := false
c    while ( not finished ) do
c       if ( main keyword = CLUSTER ) then
c          Read next subkeywords until a new main keyword is found
c          interpret subkeywords
c       else if ( main keyword = CLUSTER ) then
c          Read sequence number iseq
c          Read next subkeywords until a new main keyword is found
c          interpret subkeywords
c       else
c          finished := true
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data code1 / 'GLOBAL_LOOP', 'CLUSTER' /
      data nmcode1  / 4, 5 /
      data code2  / 'SEQUENCE_OF_EQUATIONS',
     +              'ITERATE_INITIAL_CONDITION',
     +              'MIN_INITIAL_ITER', 'MAX_INITIAL_ITER',
     +              'ABS_INITIAL_ACCURACY', 'REL_INITIAL_ACCURACY',
     +              'MIN_STATIONARY_ITER', 'MAX_STATIONARY_ITER' /
      data ncode2  / 3, 4,  8,  8,  8,    8,  8,  8 /
      data ncodmx2 /21,25, 16, 16, 20,   20, 19, 19 /
      data itype2  /11, 2,  0,  0,  1,    1,  0,  0 /
      data tabmin2 / 0d0, 0d0, 1d0, 1d0, 1d-10, 1d-10, 1d0, 1d0 /
      data tabmax2 / 0d0, 0d0, 1d2, 1d4, 1d0,   1d0,   1d2, 1d4 /
      data code3  / 'EQUATIONS', 'STATIONARY',
     +              'ABS_STATIONARY_ACCURACY',
     +              'REL_STATIONARY_ACCURACY',
     +              'MIN_STATIONARY_ITER', 'MAX_STATIONARY_ITER',
     +              'NSUBSTEPS' /
      data ncode3  / 4, 4,  8,  8,  8,    8,  4 /
      data ncodmx3 / 9,10, 23, 23, 20,   20,  9 /
      data itype3  /11, 2,  1,  1,  0,    0,  0 /
      data tabmin3 / 0d0, 0d0, 1d-10, 1d-10, 1d0, 1d0, 1d0 /
      data tabmax3 / 0d0, 0d0, 1d0,   1d0,   1d2, 1d4, 1d3 /

c     code1     Table containing the  main keywords corresponding to this
c               subroutine. Contents:
c               1:  GLOBAL_LOOP
c               2:  CLUSTER
c     code2     Table containing the subkeywords corresponding to GLOBAL_LOOP
c               Contents:
c               1:  SEQUENCE_OF_EQUATIONS
c               2:  ITERATE_INITIAL_CONDITION
c               3:  MIN_INITIAL_ITER
c               4:  MAX_INITIAL_ITER
c               5:  ABS_INITIAL_ACCURACY
c               6:  REL_INITIAL_ACCURACY
c               7:  MIN_STATIONARY_ITER
c               8:  MAX_STATIONARY_ITER
c     code3     Table containing the  subsubkeywords corresponding to this
c               subroutine. Contents:
c               1:  equations
c               2:  stationary
c               3:  abs_stationary_accuracy
c               4:  rel_stationary_accuracy
c               5:  min_stationary_iter
c               6:  max_stationary_iter
c               7:  nsubsteps
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen ( 'isrdstruc' )

c     --- Make table of new main keywords consisting of the local main keywords
c         and the global main keywords

      nmain1 = nncod1+nmain
      if ( nmain1.gt.nmainnw ) then

c     --- Error nmainnw too small

         call iserrint ( nmainnw, 1 )
         call iserrint ( nmain, 2 )
         call iserrint ( nncod1, 3 )
         call iserrsub ( 531, 3, 0, 0 )
         go to 1000

      end if

      do i = 1, nncod1
         codema(i)(1:lenchar) = code1(i)(1:lenchar)
         nmcoda(i) = nmcode1(i)
      end do
      do i = 1, nmain
         codema(i+nncod1)(1:lenchar) = codemn(i)(1:lenchar)
         nmcoda(i+nncod1) = nmcode(i)
      end do

c     --- Clear array icheckeq

      do i = 1, maxeq
         icheckeq(i) = 0
         ihelp(i) = 0
         ihelp1(i) = 0
      end do

c     --- Read first record (main keyword)

      numsub = 0
      nclusters = 0
      ipcluster = 0
      call isintx02 ( code1, ncode1, numsub, itype, table1, codema,
     +                nmcoda, nmain1, iretur, tabmin1, tabmax1,
     +                tabout, nout, strout, nstrng, nmcode1 )
      if ( iretur.lt.0 ) go to 1000
      finished = .false.
      globloop = .false.

200   if ( .not. finished ) then

c     --- while loop finished

         iseq = 0

c        --- Act depending on keyword found

         if ( iretur.eq.1 ) then

c        --- keyword_global loop found, read subkeywords

            if ( globloop ) then

c           --- Keyword has been found before

               call iserrchr ( code1(1), 1 )
               call iswarsub ( 529, 0, 0, 1 )

            else

               globloop = .true.

            end if
            do i = 1, nncod2
               table2(i) = 0d0
            end do
            tabmin2(1) = maxeq
            tabmax2(1) = maxeq
            call isintx02 ( code2, ncode2, nncod2, itype2, table2,
     +                      codema, nmcoda, nmain1, iretur, tabmin2,
     +                      tabmax2, tabout, nout, strout, nstrng,
     +                      ncodmx2 )

            if ( table2(1).gt.0d0 ) then

c           --- sequence_of_equations found, check contents
c               Store result in iseqeq

               iposut =  nint(table2(1))
               m = nint(tabmax2(1))
               do i = 1, m
                  ih = nint(tabout(iposut-1+i))
                  if ( ihelp(ih).eq.0 ) then
                     ihelp(ih) = i
                     if ( iseqeq(ih).gt.0 ) iseqeq(ih) = i
                  else
                     call iserrint ( ih, 1 )
                     call iserrsub ( 614, 1, 0, 0 )
                  end if
               end do

            end if

            if ( table2(2).gt.0d0 .or. table2(3).gt.0d0 .or.
     +           table2(4).gt.0d0 .or. table2(5).gt.0d0 )
     +         iglobstr(3) = 1
            if ( table2(3).gt.0d0 ) iglobstr(7) = nint(table2(3))
            if ( table2(4).gt.0d0 ) iglobstr(8) = nint(table2(4))
            rglobstr(3) = table2(5)
            rglobstr(4) = table2(6)
            if ( table2(7).gt.0d0 ) iglobstr(4) = nint(table2(7))
            if ( table2(8).gt.0d0 ) iglobstr(5) = nint(table2(8))
            if ( iglobstr(3).gt.0 ) then
               if ( rglobstr(3).le.0d0 .and. rglobstr(4).le.0d0 ) then
                  rglobstr(4) = 1d-2
               end if
            end if

         else if ( iretur.eq.2 ) then

c        --- keyword cluster found, read sequence number followed by subkeywords

            call isasknxt
            if ( ityprd.eq.2 ) then

c           --- number found

               iseq = nint(realrd)
               if ( iseq.lt.1 .or. iseq.gt.maxeq ) then

c              --- iseq out of range

                  call iserrchr ( codema(iretur), 1 )
                  call iserrint ( iseq, 1 )
                  call iserrint ( 1, 2 )
                  call iserrint ( maxeq, 3 )
                  call iserrsub ( 530, 3, 0, 1 )
                  call iserrcrd
                  iseq = 1

               end if
               if ( ihelp1(iseq).eq.0 ) then
                  ihelp1(iseq) = 1
               else
                  call iserrint ( iseq, 1 )
                  call iserrsub ( 615, 1, 0, 0 )
                  go to 200
               end if
               nclusters = max(nclusters,iseq)
               do i = 1, nncod3
                  table3(i) = 0d0
               end do
               tabmin3(1) = maxeq
               tabmax3(1) = maxeq
               call isintx02 ( code3, ncode3, nncod3, itype3, table3,
     +                         codema, nmcoda, nmain1, iretur, tabmin3,
     +                         tabmax3, tabout, nout, strout, nstrng,
     +                         ncodmx3 )
               if ( table3(1).le.0d0 ) then

c              --- Not the keyword equations found, hence use iseq

                  if ( icheckeq(iseq).ne.0 ) then

c                 --- Equation already in use

                     call iserrint ( iseq, 1 )
                     call iserrsub ( 616, 1, 0, 0 )

                  else

c                 --- Store information in icluster

                     icheckeq(iseq) = 1
                     icluster(ipcluster+1 ) = iseq
                     icluster(ipcluster+2 ) = 1
                     icluster(ipcluster+3 ) = iseq
                     ipcluster = ipcluster+3

                  end if

               else

c              --- The keyword equations has been found

                  iposut =  nint(table3(1))
                  m = nint(tabmax3(1))
                  icluster(ipcluster+1 ) = iseq
                  icluster(ipcluster+2 ) = m
                  ipcluster = ipcluster+2
                  do i = 1, m
                     ih = nint(tabout(iposut-1+i))
                     if ( icheckeq(ih).eq.0 ) then
                        icheckeq(ih) = i
                     else
                        call iserrint ( ih, 1 )
                        call iserrsub ( 616, 1, 0, 0 )
                     end if
                     icluster(ipcluster+i ) = ih
                  end do
                  ipcluster = ipcluster+m

               end if
               if ( ipcluster.gt.lncluster ) then

c              --- Array icluster too small

                  call iserrchr ( 'icluster', 1 )
                  call iserrint ( ipcluster, 1 )
                  call iserrint ( lncluster, 2 )
                  call iserrsub ( 17, 2, 0, 1 )
                  go to 1000

               end if

               if ( table3(2).gt.0d0 .or. table3(3).gt.0d0 .or.
     +              table3(4).gt.0d0 .or. table3(5).gt.0d0 .or.
     +              table3(6).gt.0d0 ) ilocstr(iseq,1) = 1
               if ( table3(3).gt.0d0 ) then
                   ilocstr(iseq,4) = 0
                   rlocstr(iseq,1) = table3(3)
               end if
               if ( table3(4).gt.0d0 ) then
                   ilocstr(iseq,4) = 1
                   rlocstr(iseq,1) = table3(4)
               end if
               if ( table3(5).gt.0d0 ) ilocstr(iseq,2) = nint(table3(5))
               if ( table3(6).gt.0d0 ) ilocstr(iseq,3) = nint(table3(6))
               if ( table3(7).gt.0d0 ) ilocstr(iseq,5) = nint(table3(7))

            else

c           --- Error not an integer number found

               call iserrchr ( codema(iretur), 1 )
               call iserrsub ( 528, 0, 0, 1 )
               call iserrcrd

            end if

         else

c        --- Main keyword found, return

            finished = .true.
	    iretur = iretur - nncod1

         end if
         go to 200

      end if

c     --- Check if all clusters have been filled

      do i = 1, nclusters
         if ( ihelp1(i).eq.0 ) then
            call iserrint ( i, 1 )
            call iserrint ( nclusters, 2 )
            call iserrsub ( 617, 2, 0, 0 )
         end if
      end do

1000  call iserclos ( 'isrdstruc' )
      end
