      subroutine isrdtime ( nmain, codemn, nmcode, iretur, iintim,
     +                      rintim )
c ======================================================================
c
c        programmer     Guus Segal
c        version 2.3    date 29-04-1999 Guus: extension of codemn
c        version 2.2    date 27-05-1998 Guus: extension with time iteration
c        version 2.1    date 22-04-1998 Kees K: tabmax for outsession
c                                               changed
c        version 2.0    date 26-03-1998 Kees K: nsubsteps changed in
c                                               nsubsteps_turbulence
c        version 1.9    date 26-03-1998 Kees K: extension with nsubsteps
c        version 1.8    date 14-07-1997 Guus: extension of nmainnw
c        version 1.7    date 10-02-1997 Kees K extension with norm
c        version 1.6    date 23-04-1996 Kees K changes for restart file
c        version 1.5    date 10-10-1995 Guus: extend max TEND
c        version 1.4    date 10-05-1995 Kees: extension with restart
c        version 1.3    date 08-02-1995 Guus: extension of nmainnw
c        version 1.2    date 18-01-1995 Michiel Kooper corrections made
c        version 1.1    date 11-01-1995 (Extension with fractional time method)
c        version 1.0    date 27-08-1994
c
c
c   copyright (c) 1994-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Read time integration part of input files and store result in
c    arrays iintim and rintim
c **********************************************************************
c
c                       KEYWORDS
c
c       read
c       read_cards
c       time_integration
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer nmain, nmcode(nmain), iretur, iintim(100)
      double precision rintim(100)
      character *(*) codemn(nmain)

c     codemn    i    Table of main keywords
c     iintim    o    Contains information of the time integration
c                    See the programmers Guide for a description
c     iretur    o    Return code, indicates the sequence number of the new
c                    main keyword found
c                    If -1 an error has occurred
c                    If -2 the end of file has been reached
c     nmain     i    Number of keywords in array codemn
c     nmcode    i    Array containing the number of significant characters
c                    for each main keyword in codemn
c     rintim    o    Contains information of the time integration
c                    See the programmers Guide for a description
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer nmainnw, lenchar, nncode, nncod2, nout
      parameter ( nmainnw=40, nncode=22, nncod2=12, lenchar=14,
     +            nout=30 )
      integer nmcoda(nmainnw), itype(nncode), ndt1, ndt2, nfrac,
     +        nmcode2(nncod2), ncodmx(nncode), nstrng, iposut,
     +        ncode(nncode), i, nmmaintot
      double precision table(nncode), tabmin(nncode), tabmax(nncode),
     +                 tabout(nout), alpha1, beta1,
     +                 testfrac1, testfrac2, testfrac3, theta
      character *14 codema(nmainnw), code2(nncod2)
      character *25 code(nncode)
      character *1 strout(1)

c     alpha1    General constant alpha
c     beta1     General constant beta
c     code      Table containing the subkeywords corresponding to this
c               subroutine.
c     code2     Table containing the  subsubkeywords corresponding to this
c               subroutine.
c     codema    Table consisting of the local main keywords followed by the
c               general main keywords possibly followed by the subsubkeywords
c     i         General loop variable
c     iposut    last position used in array tabout
c     itype     Array containing the type of actions to be undertaken for
c               each subkeyword found.
c               See subroutine isintx02 for the possible values of itype
c     lenchar   Number of characters in main keywords
c     ncode     Array containing the number of significant characters for
c               each keyword in code
c     ncodmx    Array containing the maximum number of characters for
c               each keyword in code
c     ndt1      Number of end times read
c     ndt2      Number of different time steps
c     nfrac     Number of fractional time steps
c     nmainnw   Maximal number of keywords in codema
c     nmcoda    Array containing the number of significant characters for
c               each keyword in codema
c     nmcode2   Array containing the number of significant characters for
c               each keyword in code2
c     nmmaintot Total number of keywords in codema
c     nncod2    Number of subsubkeywords stored in array code2
c     nncode    Number of subkeywords stored in array code
c     nout      Length of array tabout
c     nstrng    Length of array strout
c     strout    Contains series of strings, see itype
c     table     Array containing the real or integer values corresponding
c               to the subkeywords in code that have been read by isintx02
c     tabmax    Array containing the maximal values of the parameters read
c               corresponding to the subkeywords in code
c     tabmin    Array containing the minimum values of the parameters read
c               corresponding to the subkeywords in code
c     tabout    Array of output reals provided by isintx02
c     testfrac1 Counter to test consistency in the choice of the thetas
c               when METHOD.eq.4
c     testfrac2 Counter to test consistency in the choice of the thetas
c               when METHOD.eq.4
c     testfrac3 Counter to test consistency in the choice of the thetas
c               when METHOD.eq.4
c     theta     Parameter theta for theta method
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRCHR  Puts values of strings in character error array
c     ISERREAL  Put real value in error message
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c     ISINTX02  Interpret a record containing keywords followed by numbers
c     ISSTOP    Stop ISNAS
c **********************************************************************
c
c                       I/O
c
c    This subroutine reads the input after the main keyword TIME_INTEGRATION
c    until a new main keyword has been found
c    The following input is recognized:
c    METHOD = type1
c    TINIT  = rintim(2)
c    TEND   = (t1, t2, t3, ... ,tN)
c    THETA  = (theta1, theta2, ... , thetaM)
c    TSTEP  = (dt1, dt2, dt3, ... ,dtN)
c    TOUTInit = t
c    TOUTEnd  = t
c    TOUTStep = t
c    NSUBSTEPS_TURBULENCE = t
c    ABS_STATIONARY_ACCURACY = eps
c    REL_STATIONARY_ACCURACY = eps
c    OUTSESSION = t
c    FORCE_DELETION = YES/NO
c    RESTART = APPEND/OVERWRITE
c    OUTPUT  =  DUMP/CARTESIAN
c    NORM    =  LTWO/MAX
c    PRESS_CORR_TIME_ITERATION
c    ABS_PRESS_CORR_ACCURACY = eps
c    RELAXATION_PRESS_CORR = r
c    type1 may take one of the following values:
c       THETA
c       FRACTIONAL_STEP
c       GENERALIZED_THETA
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     507   rintim(i) in input file incorrrect
c     531   Internal error in subroutine; character array too small
c     594   Number of time steps not equal to number of end times
c     595   nfrac incorrect for method
c     596   nfrac is not even
c **********************************************************************
c
c                       PSEUDO CODE
c
c    Make table of new main keywords consisting of the global main keywords
c    and the subsubkeywords
c    Read subkeywords until a new main keyword is found
c    interpret subkeywords
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data code  / 'METHOD', 'TINIT', 'TEND', 'THETA', 'TSTEP',
     +             'TOUTINIT', 'TOUTEND', 'TOUTSTEP',
     +             'ABS_STATIONARY_ACCURACY', 'REL_STATIONARY_ACCURACY',
     +             'OUTSESSION', 'FORCE_DELETION', 'RESTART',
     +             'RINIT', 'REND', 'RSTEP', 'OUTPUT', 'NORM',
     +             'NSUBSTEPS_TURBULENCE','PRESS_CORR_TIME_ITERATION',
     +             'ABS_PRESS_CORR_ACCURACY','RELAXATION_PRESS_CORR' /
      data ncode  /  6,  5,  4,  5,  5, 5, 5, 5,  8,  8,
     +               4,  7,  7,  5,  4, 5, 6, 4,  4,  8,
     +               8,  8 /
      data ncodmx /  6,  5,  4,  5,  5, 8, 7, 8, 23, 23,
     +              10, 14,  7,  5,  4, 5, 6, 4, 20, 25,
     +              23, 21 /
      data itype  /  5,  1, 19, 19, 19, 1, 1, 1,  1,  1,
     +               0,  5,  5,  1,  1, 1, 5, 5,  0,  2,
     +               1,  1 /
      data tabmin / 0d0, -1d5,  10d0,  10d0, 10d0,
     +             -1d5, -1d5, 1d-10, 1d-10, 1d-8,
     +              0d0,  0d0,   0d0,  -1d5,  0d0,
     +              0d0,  0d0,   0d0,   0d0,  0d0,
     +           -0.5d0,1d-15 /
      data tabmax / 0d0,  1d5,   1d9,   1d0,  1d5,
     +              1d5,  1d5,   1d5,   1d0,  1d0,
     +              1d2,  0d0,   0d0,   1d5,  1d5,
     +              1d5,  0d0,   0d0,   1d2,  0d0,
     +            0.5d0,  1d0 /
      data code2 / 'THETA', 'FRACTIONAL_STE', 'GENERALIZED_TH',
     +             'DUMP', 'CARTESIAN', 'YES', 'NO', 'APPEND',
     +             'OVERWRITE', 'NONE', 'LTWO', 'MAX'/
      data nmcode2 / 5, 4, 5, 4, 4, 3, 2, 6, 5, 4, 4, 3 /

c     code      Table containing the subkeywords corresponding to this
c               subroutine. Contents:
c               1:  METHOD
c               2:  TINIT
c               3:  TEND
c               4:  THETA
c               5:  TSTEP
c               6:  TOUTINIT
c               7:  TOUTEND
c               8:  TOUTSTEP
c               9:  ABS_STATIONARY_ACCURACY
c              10:  REL_STATIONARY_ACCURACY
c              11:  OUTSESSION
c              12:  FORCE_DELETION
c              13:  RESTART
c              14:  RINIT
c              15:  REND
c              16:  RSTEP
c              17:  OUTPUT
c              18:  NORM
c              19:  NSUBSTEPS_TURBULENCE
c              20:  PRESS_CORR_TIME_ITERATION
c              21:  ABS_PRESS_CORR_ACCURACY
c              22:  RELAXATION_PRESS_CORR
c     code2     Table containing the  subsubkeywords corresponding to this
c               subroutine. Contents:
c               1:  THETA
c               2:  FRACTIONAL_STEP
c               3:  GENERALIZED_THETA
c               4:  DUMP
c               5:  CARTESIAN
c               6:  YES
c               7:  NO
c               8:  APPEND
c               9:  OVERWRITE
c              10:  NONE
c              11:  LTWO
c              12:  MAX
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen ( 'isrdtime' )

c     --- Make table of new main keywords consisting of the global main keywords
c         and the subsubkeywords

      nmmaintot = nmain+nncod2
      if ( nmmaintot.gt.nmainnw ) then

c     --- Error nmainnw too small

         call iserrint ( nmainnw, 1 )
         call iserrint ( nmain, 2 )
         call iserrint ( nncod2, 3 )
         call iserrsub ( 531, 3, 0, 0 )
         call isstop

      end if

      do i = 1, nmain
         codema(i)(1:lenchar) = codemn(i)(1:lenchar)
         nmcoda(i) = nmcode(i)
      end do
      do i = 1, nncod2
         codema(i+nmain)(1:lenchar) = code2(i)(1:lenchar)
         nmcoda(i+nmain) = nmcode2(i)
      end do

c     --- Set tabmin and tabmax with references of sub subkeywords
c         stored in codema

      tabmin(1) = nmain+1
      tabmax(1) = nmain+3
      tabmax(3) = 1d5
      tabmax(4) = 1d5
      tabmax(5) = 1d5
      tabmin(17) = nmain+4
      tabmax(17) = nmain+5
      tabmin(12) = nmain+6
      tabmax(12) = nmain+7
      tabmin(13) = nmain+8
      tabmax(13) = nmain+10
      tabmin(18) = nmain+11
      tabmax(18) = nmain+12

c     --- Read subkeywords until a main keyword is found

      do i = 1, nncode
         table(i) = -1d20
      end do
      table(1) = 1d0
      table(2) = 0d0
      nstrng = 1
      call isintx02 ( code, ncode, nncode, itype, table, codema,
     +                nmcoda, nmain, iretur, tabmin, tabmax,
     +                tabout, nout, strout, nstrng, ncodmx )

c     --- interpret subkeywords

      iintim(1) =  nint(table(1))
      if ( iintim(1).gt.1 ) iintim(1) = iintim(1)+1
      rintim(2) =  table(2)

c     --- Time steps and end time

      ndt1 = nint(tabmax(3))
      ndt2 = nint(tabmax(5))
      if ( max(1,ndt1).ne.max(1,ndt2) ) then

c     --- ndt1 and ndt2 are different

         call iserrint ( ndt1, 1 )
         call iserrint ( ndt2, 2 )
         call iserrsub ( 594, 2, 0, 0 )

      end if
      if ( ndt1.gt.0 ) then

c     --- ndt1 > 0

         if ( ndt2.eq.0 ) then

c        --- ndt1=1, ndt2=0

            iposut     = nint(table(3))
            rintim( 3) = tabout(iposut)
            rintim(51) = tabout(iposut)
         else

c        --- ndt1>0, ndt2>0

            iposut = nint(table(3))
            do i = 1, ndt1
               rintim(50+i) = tabout(iposut+i-1)
               if ( i.eq.1 .and. rintim(50+i).lt.rintim(2) .or.
     +              i.gt.1 .and. i.lt.ndt1                 .and.
     +              rintim(50+i).le.rintim(50+i-1)           .or.
     +              i.eq.ndt1 .and. rintim(50+i).gt.1d6 )  then
                    call iserrint ( 50+i, 1 )
                    call iserreal ( rintim(50+i), 1 )
                    call iserrchr ( 'tstepend', 1 )
                    call iserrsub ( 507, 1, 1, 1 )
               end if
            end do
            rintim(3) = rintim(50+ndt1)
            iposut = nint(table(5))
            do i = 1, ndt2
               rintim(40+i) = tabout(iposut+i-1)
               if ( rintim(40+i).le.0d0                         .or.
     +              i.eq.1 .and.  rintim(40+i) .gt.
     +                 (rintim(50+i)-rintim(2))*(1d0+1d-6)        .or.
     +              i.gt.1 .and.  rintim(40+i) .gt.
     +                 (rintim(50+i)-rintim(50+i-1))*(1d0+1d-6) ) then
                    call iserrint ( 40+i, 1 )
                    call iserreal ( rintim(40+i), 1 )
                    call iserrchr ( 'tstep', 1 )
                    call iserrsub ( 507, 1, 1, 1 )
               end if
            end do
            rintim(4) = rintim(41)
         end if

      else if ( ndt2.eq.1 ) then

c     --- ndt1=0, ndt2=1

         iposut = nint(table(5))
         rintim(4) = tabout(iposut)
         rintim(41) = tabout(iposut)

      end if
      iintim(8) = max(1,ndt1)
      if ( ndt1.gt.1 ) iintim(7) = 1

c     --- Thetas

      nfrac = nint(tabmax(4))
      if ( iintim(1).eq.1 .and. nfrac.gt.1 ) then

c     --- theta method, with more fraction steps given

         call iserrint ( nfrac, 1 )
         call iserrint ( 1, 2 )
         call iserrchr ( 'theta', 1 )
         call iserrsub ( 595, 2, 0, 1 )

      else if ( iintim(1).eq.3 .and. nfrac.gt.0 .and. nfrac.ne.6 ) then

c     --- fractional step method, with more fraction steps given

         call iserrint ( nfrac, 1 )
         call iserrint ( 6, 2 )
         call iserrchr ( 'fractional step', 1 )
         call iserrsub ( 595, 2, 0, 1 )

      else if ( iintim(1).gt.1 .and. mod(nfrac,2).ne.0 ) then

c     --- nfrac is not even

         call iserrint ( nfrac, 1 )
         call iserrsub ( 596, 1, 0, 0 )

      end if
      if ( nfrac.eq.0 ) then

c     --- No thetas given, use default values

         if ( iintim(1).eq.1 ) then

c        --- theta method

            nfrac = 1
            rintim(31) = 0d0
            rintim(32) = 1d0
            rintim(5) = 1d0

         else if ( iintim(1).eq.3 ) then

c        --- fractional step method

            nfrac = 3
            theta = 1d0-0.5d0*sqrt(2d0)
            alpha1 = (1d0-2d0*theta)/(1d0-theta)
            beta1 = theta/(1d0-theta)
            rintim(31) = beta1*theta
            rintim(32) = alpha1*theta
            rintim(33) = alpha1*(1d0-2d0*theta)
            rintim(34) = beta1*(1d0-2d0*theta)
            rintim(35) = rintim(31)
            rintim(36) = rintim(32)

         else

c        --- generalized theta method

            nfrac = 3
            alpha1 = 1d0/(1d0+2d0/sqrt(3d0))
            rintim(31) = 0.5d0*alpha1
            rintim(32) = sqrt(3d0)/6d0*alpha1
            rintim(33) = 0d0
            rintim(34) = sqrt(3d0)/3d0*alpha1
            rintim(35) = rintim(31)
            rintim(36) = rintim(32)

         end if

      else

c     --- thetas have been given, use actual values

         iposut = nint(table(4))
         if ( iintim(1).eq.1 ) then

c        --- theta method

            nfrac = 1
            rintim(5)  = tabout(iposut)
            rintim(31) = 1d0-rintim(5)
            rintim(32) = rintim(5)

         else

c        --- fractional step or generalized theta method

            do i = 1, nfrac
               rintim(30+i) = tabout(iposut+i-1)
            end do
            testfrac1 = 0d0
            testfrac2 = 0d0
            testfrac3 = 0d0

c           --- Test consistency in choice of the thetas when METHOD.eq.4
c               Sum theta_i = 1
c               Sum ( theta_2i )**2 = Sum ( theta_(2i-1) )**2

            do i=1, nfrac
               testfrac1    = testfrac1 + rintim(30+2*i-1)
     +                                  + rintim(30+2*i  )
               testfrac2    = testfrac2 + rintim(30+2*i  )**2
               testfrac3    = testfrac3 + rintim(30+2*i-1)**2
            end do
            if ( testfrac1.lt.(1d0-1d-6)             .or.
     +           testfrac1.gt.(1d0+1d-6)             .or.
     +           testfrac2.lt.testfrac3*(1d0-1d-6)   .or.
     +           testfrac2.gt.testfrac3*(1d0+1d-6) ) then
               call iserrint ( 30+2*nfrac, 1 )
               call iserreal ( rintim(30+2*nfrac), 1 )
               call iserrchr ( 'choice of thetas', 1 )
               call iserrsub ( 507, 1, 1, 1 )
            end if

            nfrac = nfrac/2

         end if

      end if
      iintim(6) = nfrac

c     --- toutinit, toutend, toutstep

      if ( table(6).lt.-1d19 ) then
         rintim(6) = rintim(50+ndt1)
      else
         rintim(6) =  table(6)
      end if
      if ( table(7).lt.-1d19 ) then
         rintim(7) = rintim(50+ndt1)
      else
         rintim(7) =  table(7)
      end if
      if ( table(8).lt.-1d19 ) then
         rintim(8) = rintim(41)
      else
         rintim(8) =  table(8)
      end if

c     --- outsession number

      if ( table(11).lt.-1d19 ) then
         iintim(14) = 1
      else
         iintim(14) =  nint(table(11))
      end if

c     --- force_deletion

      if ( table(12).lt.-1d19 ) then
         iintim(13) = 0
      else if ( nint(table(12)).eq.1 ) then
         iintim(13) = 1
      else if ( nint(table(12)).eq.2 ) then
         iintim(13) = 0
      end if

c     --- restart : append/overwrite

      if ( table(13).lt.-1d19 ) then
         iintim(11) = -1
      else if ( nint(table(13)).eq.1 ) then
         iintim(11) = 1
      else if ( nint(table(13)).eq.2 ) then
         iintim(11) = 0
      else if ( nint(table(13)).eq.3 ) then
         iintim(11) = -1
      end if

c     --- output : dump/cartesian

      if ( table(17).lt.-1d19 ) then
         iintim(12) = 0
      else if ( nint(table(17)).eq.1 ) then
         iintim(12) = 0
      else if ( nint(table(17)).eq.2 ) then
         iintim(12) = 1
      end if

c     --- nsubsteps_turbulence

      if ( table(19).lt.-1d19 ) then
         iintim(16) = 1
      else
         iintim(16) =  nint(table(19))
      end if

c     --- rinit, rend, rstep

      if ( table(14).lt.-1d19 ) then
         rintim(9) = rintim(7)
      else
         rintim(9) =  table(14)
      end if
      if ( table(15).lt.-1d19 ) then
         rintim(10) = rintim(7)
      else
         rintim(10) =  table(15)
      end if
      if ( table(16).lt.-1d19 ) then
         rintim(11) = rintim(8)
      else
         rintim(11) =  table(16)
      end if

c     --- norm : ltwo/max

      if ( table(18).lt.-1d19 ) then
         iintim(15) = 0
      else if ( nint(table(18)).eq.1 ) then
         iintim(15) = 0
      else if ( nint(table(18)).eq.2 ) then
         iintim(15) = 1
      end if

c     --- Stationary stop

      if ( table(9).gt.-1d0 .or. table(10).gt.-1d0 ) then

c     --- absolute and or relative accuracy required for stopping criterion

         if ( table(9).gt.-1d0 ) then
            rintim(29) = table(9)
         else
            rintim(29) = 0d0
         end if
         if ( table(10).gt.-1d0 ) then
            rintim(30) = table(10)
         else
            rintim(30) = 0d0
         end if
         iintim(10) = 1

      end if

c     --- Pressure correction time iteration

      if ( table(20).gt.0d0 ) iintim(17) = 1
      if ( table(21).gt.-1d10 ) rintim(19) = table(21)
      if ( table(22).gt.-1d10 ) rintim(18) = table(22)

      call iserclos ( 'isrdtime' )
      end
