      subroutine isreadsp ( ndim, ncurvs, isineigh, iskmeshq,
     +                       indorien )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.3    date 08-09-1998 Kees K: change orientation
c        version 1.2    date 13-09-1995 Guus: Allow for unformatted files
c        version 1.1    date 19-01-1995 Kees K. 3D
c        version 1.0    date 28-07-1994
c
c
c   copyright (c) 1994-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Read last part of the grid file finvol.new in case this file is created
c    by SEPRAN. The information of the curves is read and stored in isineigh
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer isineigh, ncurvs, iskmeshq, ndim, indorien(*)

c     indorien  i    In this array it is stored if the block orientation is
c                    positive (indorien(iblock)=1) or
c                    negative(indorien(iblock)=-1)
c     isineigh  o   Sequence number of array ineigh
c     iskmeshq  i   Reference to SEPRAN array kmeshq
c     ncurvs    o   Number of curves (SEPRAN environment)
c     ndim      i   Dimension of space (2 or 3)
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, inum, ipineigh, ipkmeshq, iptemp, lenkmeshq, nsurfs,
     +          iss
      double precision dnumbr(1)

c     dnumbr    Work array to store a series numbers that have been read
c     i         General loop variable
c     inum      Indicates the number of numbers that have been read
c     ipineigh  Starting address of array ineigh in IBUFFR
c     ipkmeshq  Starting address of SEPRAN array kmeshq in IBUFFR
c     iptemp    Starting address of temporary array in IBUFFR
c     iss       Help variable to save sign
c     lenkmeshq Length of SEPRAN array kmeshq
c     nsurfs    Number of surfaces (SEPRAN environment)
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isingetp

c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISICREAT  Create permanent space in array IBUFFR (Integer array)
c     ISINGETP  get the pointer corresponding to an integer array
c     ISREADRL  read a series of numbers which may be at several records
c               skip texts and other symbols
c     ISTMSPAC  Create space for temporary array
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     Read nsurfs
c     Read and skip isurfc of length 4*nsurfs
c     Read ncurvs
c     Reserve space for array ineigh
c     Read ineigh of length 10*ncurvs
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call iseropen ( 'isreadsp' )

c     --- Read nsurfs

      if ( iref10.gt.0 ) then
         call isreadrl ( dnumbr(1), 1, inum )
         nsurfs = nint ( dnumbr(1) )
      else
         read(irefre) nsurfs
      end if

c     --- Read and skip isurfc of length 4*nsurfs ( ndim = 2 )
c                                 length 6*nsurfs ( ndim = 3 )

      if ( ndim.eq.2 ) then
         call istmspac ( 4*nsurfs*intlen, iptemp )
         if ( iref10.gt.0 ) then
            iptemp = (iptemp-1)/intlen+1
            call isreadrl ( buffer(iptemp), 4*nsurfs, inum )
         else
            read(irefre) ibuffr(iptemp)
         end if
      else
         call istmspac ( 6*nsurfs*intlen, iptemp )
         if ( iref10.gt.0 ) then
            iptemp = (iptemp-1)/intlen+1
            call isreadrl ( buffer(iptemp), 6*nsurfs, inum )
         else
            read(irefre) ibuffr(iptemp)
         end if
      end if

c     --- Read ncurvs

      if ( iref10.gt.0 ) then
         call isreadrl ( dnumbr(1), 1, inum )
         ncurvs = nint ( dnumbr(1) )
      else
         read(irefre) ncurvs
      end if

c     --- Reserve space for array ineigh and temporary space

      isineigh = 0

      if ( ndim.eq.2 ) then

c     --- ndim = 2

         call isicreat ( isineigh, 10*ncurvs , 'ineigh' )
         call istmspac ( 10*ncurvs*intlen, iptemp )
         ipineigh = isingetp ( isineigh )

c        --- Read ineigh of length 10*ncurvs

         if ( iref10.gt.0 ) then
            iptemp = (iptemp-1)/intlen+1
            call isreadrl ( buffer(iptemp), 10*ncurvs, inum )
            do i = 0, 10*ncurvs-1
               ibuffr(ipineigh+i) = nint(buffer(iptemp+i))
            end do
         else
            read(irefre) (ibuffr(ipineigh+i),i = 0, 10*ncurvs-1)
         end if

c        --- change orientation
c            face 1 ---> face 3   mod(1+1,4) + 1
c            face 2 ---> face 4   mod(2+1,4) + 1
c            face 3 ---> face 1   mod(3+1,4) + 1
c            face 4 ---> face 2   mod(4+1,4) + 1

         do i = 1, ncurvs

            if( indorien( ibuffr(ipineigh-1+(i-1)*10+1)).eq.-1)
     +          then
                 iss = sign(1,ibuffr(ipineigh-1+(i-1)*10+2) )
                 ibuffr(ipineigh-1+(i-1)*10+2) = iss * (
     +            mod( abs(ibuffr(ipineigh-1+(i-1)*10+2))+1,4)+1)
            end if
         end do

      else

c     --- ndim = 3

         call isicreat ( isineigh, 14*ncurvs , 'ineigh' )
         call istmspac ( 14*ncurvs*intlen, iptemp )
         ipineigh = isingetp ( isineigh )

c        --- Read ineigh of length 14*ncurvs

         if ( iref10.gt.0 ) then
            iptemp = (iptemp-1)/intlen+1
            call isreadrl ( buffer(iptemp), 14*ncurvs, inum )
            do i = 0, 14*ncurvs-1
               ibuffr(ipineigh+i) = nint(buffer(iptemp+i))
            end do
         else
            read(irefre) (ibuffr(ipineigh+i),i = 0, 14*ncurvs-1)
         end if

      end if

c     --- Read length of array kmeshq

      if ( iref10.gt.0 ) then
         call isreadrl ( dnumbr(1), 1, inum )
         lenkmeshq = nint ( dnumbr(1) )
      else
         read(irefre) lenkmeshq
      end if
      iskmeshq = 0

      if ( lenkmeshq.gt.0 ) then

c     --- lenkmeshq > 0, hence kmeshq exists

         call isicreat ( iskmeshq, lenkmeshq , 'kmeshq' )
         call istmspac ( lenkmeshq*intlen, iptemp )
         ipkmeshq = isingetp ( iskmeshq )

c        --- Read kmeshq

         if ( iref10.gt.0 ) then
            iptemp = (iptemp-1)/intlen+1
            call isreadrl ( buffer(iptemp), lenkmeshq, inum )
            do i = 0, lenkmeshq-1
               ibuffr(ipkmeshq+i) = nint(buffer(iptemp+i))
            end do
         else
            read(irefre) (ibuffr(ipkmeshq+i),i = 0, lenkmeshq-1)
         end if

      end if

      call iserclos ( 'isreadsp' )
      end
