      subroutine iswrtif4 ( idimens, maxtrans, maxturb, iindsc,
     +                      rindsc, ndim, icoupled )
c ======================================================================
c
c        programmer     Guus Segal
c        version 2.2    date 02-02-1999 Kees K: Extra keyword axisymmetric
c        version 2.1    date 03-03-1998 Guus: Extra keyword conservative
c        version 2.0    date 09-10-1997 Guus: Extra parameters ndim, icoupled
c        version 1.5    date 10-03-1997 Guus: change parameter iindsc(7,1)
c        version 1.4    date 17-02-1997 Marcel: extra parameter limiter
c        version 1.3    date 18-10-1995 Marcel: modified flux-limiting
c                                             parameters
c        version 1.2    date 06-03-1995 Guus: error correction transport
c        version 1.1    date 22-02-1995 Guus: added 'print solution'
c        version 1.0    date 27-09-1994
c
c
c   copyright (c) 1994-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Write fifth part of the intermediate file
c    It concerns the blocks 9 to 9
c **********************************************************************
c
c                       KEYWORDS
c
c    intermediate_file
c    write
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer idimens(100), maxturb, maxtrans,
     +        iindsc(100,maxtrans+1+maxturb), ndim, icoupled
      double precision rindsc(100,maxtrans+1+maxturb)

c     icoupled  i    If 0 the momentum equations are coupled, if 1 they are not
c     idimens   i    Integer dimension parameters for the intermediate file
c     iindsc    i    reserved
c     maxtrans  i    Maximum number of transport equations allowed
c     maxturb   i    Maximum number of turbulence equations allowed
c     ndim      i    Dimension of space (2 or 3)
c     rindsc    i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, itrans, ntrnsp, nturb

c     i         General loop variable
c     itrans    reserved
c     ntrnsp    Number of transport equations
c     nturb     Number of extra transport equations for turbulence
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c     This subroutine  writes the following parts to the intermediate file
c
c     Block 9: Parameters with respect to discretization
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     Get ntrnsp and nturb
c     write integer information
c        write discretization parameters for momentum equation
c        write discretization parameters for transport equations
c        write discretization parameters for turbulence equations
c     write real information
c        write discretization parameters for momentum equation
c        write discretization parameters for transport equations
c        write discretization parameters for turbulence equations
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- Get ntrnsp and nturb

      ntrnsp = idimens(3)
      nturb  = idimens(6)

c     --- write integer information
c         write information for momentum equation

      if ( icoupled.eq.0 ) then

c     --- Coupled equations

         write(irefwr,100)
         write(irefwr,110) (iindsc(i,1),i=1,10)
         write(irefwr,120) (iindsc(i,1),i=11,100)

      else

c     --- Decoupled equations
c         First u-momentum

         write(irefwr,130)
         write(irefwr,110) (iindsc(i,1),i=1,10)
         write(irefwr,120) (iindsc(i,1),i=11,100)

c        --- Next v-momentum

         write(irefwr,140)
         write(irefwr,110) (iindsc(i,1),i=1,10)
         write(irefwr,120) (iindsc(i,1),i=11,100)

         if ( ndim.eq.3 ) then

c        --- Finally w-momentum

            write(irefwr,150)
            write(irefwr,110) (iindsc(i,1),i=1,10)
            write(irefwr,120) (iindsc(i,1),i=11,100)

         end if

      end if

100   format ( /'**** BLOCK 9: Parameters with respect to the',
     +          ' discretization'//
     +          '** Subblock 9.1 momentum equations'// )
110   format ( /'* iindsc:'//
     +          'iterm               =', i5/
     +          'idiscret_method     =', i5/
     +          'stress/diffusion    =', i5/
     +          'idiscret_pressure   =', i5/
     +          'iupwind             =', i5/
     +          'imonoton_diffusion  =', i5/
     +          'coupled/decoupled   =', i5/
     +          'linearization       =', i5/
     +          'conservative        =', i5/
     +          'axisymmetric        =', i5)
120   format (  'iprint_matrix       =', i5/
     +          'iprint_rhsd         =', i5/
     +          'iprint_time_one     =', i5/
     +          'iprint_time_two     =', i5/
     +          'iprint_timestep     =', i5/
     +           5i2,4x,' # Positions 16-20 not yet defined'/
     +          10i2,4x,' # Positions 21-30 not yet defined'/
     +          10i2,4x,' # Positions 31-40 not yet defined'/
     +          10i2,4x,' # Positions 41-50 not yet defined'/
     +          10i2,4x,' # Positions 51-60 not yet defined'/
     +          10i2,4x,' # Positions 61-70 not yet defined'/
     +          10i2,4x,' # Positions 71-80 not yet defined'/
     +          10i2,4x,' # Positions 81-90 not yet defined'/
     +          10i2,4x,' # Positions 91-100 not yet defined'// )
130   format ( /'**** BLOCK 9: Parameters with respect to the',
     +          ' discretization'//
     +          '** Subblock 9.1a u- momentum equations'// )
140   format ( /'**** Subblock 9.1b v-momentum equations'// )
150   format ( /'**** Subblock 9.1c w-momentum equations'// )

c     --- write information about the transport equations

      do itrans = 1, ntrnsp
         write(irefwr,160) itrans+1,itrans
160      format ( /'** Subblock 9.',i2,' transport equation',i2// )
         write(irefwr,110) (iindsc(i,1+itrans),i=1,10)
         write(irefwr,120) (iindsc(i,1+itrans),i=11,100)
      end do

c     --- write information about the turbulence equations

      do itrans = 1, nturb
         write(irefwr,170) itrans+1+ntrnsp,itrans
170      format ( /'** Subblock 9.',i2,' turbulence equation',i2// )
         write(irefwr,110) (iindsc(i,1+itrans+maxtrans),i=1,10)
         write(irefwr,120) (iindsc(i,1+itrans+maxtrans),i=11,100)
      end do

c     --- write real information
c         write information for momentum equation

      if ( icoupled.eq.0 ) then

c     --- Coupled equations

         write(irefwr,200)
         write(irefwr,210) (rindsc(i,1),i=1,100)

      else

c     --- Decoupled equations
c         First u-momentum

         write(irefwr,220)
         write(irefwr,210) (rindsc(i,1),i=1,100)

c        --- Next v-momentum

         write(irefwr,230)
         write(irefwr,210) (rindsc(i,1),i=1,100)

         if ( ndim.eq.3 ) then

c        --- Finally w-momentum

            write(irefwr,240)
            write(irefwr,210) (rindsc(i,1),i=1,100)

         end if

      end if

200   format ( /'** Subblock 9.1 momentum equations'// )
210   format ( / '* rindsc :'//
     +          'kappa      =', d15.7/
     +          'phi        =', d15.7/
     +          'mbound     =', d15.7/
     +          'alpha      =', d15.7/
     +           6f4.0,
     + '                 # Positions  5-10 not yet defined'/
     +          10f4.0,' # Positions 11-20 not yet defined'/
     +          10f4.0,' # Positions 21-30 not yet defined'/
     +          10f4.0,' # Positions 31-40 not yet defined'/
     +          10f4.0,' # Positions 41-50 not yet defined'/
     +          10f4.0,' # Positions 51-60 not yet defined'/
     +          10f4.0,' # Positions 61-70 not yet defined'/
     +          10f4.0,' # Positions 71-80 not yet defined'/
     +          10f4.0,' # Positions 81-90 not yet defined'/
     +          10f4.0,' # Positions 91-100 not yet defined'// )
220   format ( /'** Subblock 9.1a u-momentum equations'// )
230   format ( /'** Subblock 9.1b v-momentum equations'// )
240   format ( /'** Subblock 9.1c w-momentum equations'// )

c     --- write information about the transport equations

      do itrans = 1, ntrnsp
         write(irefwr,160) itrans+1,itrans
         write(irefwr,210) (rindsc(i,1+itrans),i=1,100)
      end do

c     --- write information about the turbulence equations

      do itrans = 1, nturb
         write(irefwr,170) itrans+1+ntrnsp,itrans
         write(irefwr,210) (rindsc(i,1+itrans+maxtrans),i=1,100)
      end do

      end
