      subroutine isdexten ( index, lengext )
c ======================================================================
c
c
c        programmer     Guus Segal
c        version 1.0    date   21-06-94
c
c
c   copyright (c) 1994  "Ingenieursbureau SEPRA"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "Ingenieursbureau SEPRA".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c ********************************************************************
c
c                         DESCRIPTION
c
c     Extend the length of an existing double precision array
c     in buffer array ibuffr
c ********************************************************************
c
c                      KEYWORDS
c
c     memory_management
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer index, lengext

c     index   i  Integer variable containing the sequence number of the
c                array to be processed
c     lengext i  Extra length of the array in double precisions
c ********************************************************************
c
c                      COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisnas'
      include 'IScommon/cdebug'
c ********************************************************************
c
c                    LOCAL PARAMETERS
c
      integer iextra, ipcopy, ipindx, iplast, ipstar, isave, itemp, k,
     +        lennew, lenold, numarr

c     iextra    Extra space needed at both sides of the array for control
c               integers
c     ipcopy    Starting address in IBUFFR of new array
c     ipindx    Help variable to store a starting address
c     iplast    Ending address of array in IBUFFR
c     ipstar    Computed starting position in array IBUFFR
c     isave     Help variable to store the value of index
c     itemp     Temporary infor entry of array
c     k         General loop variable
c     lennew    New length of array in integers
c     lenold    Old length of array in integers
c     numarr    Number of arrays that are stored in IBUFFR
c ********************************************************************
c
c             SUBROUTINES CALLED
c
      integer isinleng, isingetp

c     ISACTIVE  Activate old permanent array
c     ISDELARR  Delete a permanent array
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c     ISICREAT  Create permanent space in array IBUFFR (Integer array)
c     ISINCOPY  Copy integer array to other integer array possibly itself
c     ISINGETP  get the pointer corresponding to an integer array
c     ISINLENG  Return with the length of the array in integers
c     ISSTOP    Stop ISNAS
c ********************************************************************
c
c                    I/O
c
c     If necessary arrays are written to and/or read from file 3 (iref3)
c ********************************************************************
c
c                    ERROR MESSAGES
c
c    1426   Extra length non-positive
c ********************************************************************
c
c                    PSEUDO CODE
c
c     Activate old array
c     Compute space after old array
c     If (sufficient space after old array)
c        then
c           Raise ibfree if necessary
c           Set new length
c        if ( idebug>0 ) then
c           Set new control integer
c     else
c        create new array with new length
c        copy old array to new array
c        Reset index and control integers
c ======================================================================
c
      if ( idebug.eq.0 ) then
         iextra = 0
      else
         iextra = intlen
      end if
      if ( lengext.le.0 ) then

c     --- lengext <= 0, error

         call iserrint ( lengext, 1 )
         call iserrsub ( 1426, 1, 0, 0 )
         call isstop

      end if
      call isactive ( index )
      lenold = isinleng(index)
      lennew = lengext*intlen + lenold

c     --- Compute space after old array

      ipstar = isingetp(index)
      iplast = ibuffr(7)-iextra-1
      numarr = ibuffr(3)
      do 100 k = 1, numarr
         ipindx = ibuffr(11+3*(k-1))
         if ( ipindx.gt.ipstar ) iplast = min(iplast,ipindx-iextra-1)
100   continue

      if ( lennew+iextra.lt.iplast-ipstar ) then

c     --- sufficient space is available

         ibuffr(12+3*(index-1)) = lennew
         ibuffr(4) = max(ibuffr(4),ipstar+lennew+2*iextra)
         if ( idebug.gt.1 ) ibuffr(ipstar+lenold) = iinfin
         if ( idebug.gt.0 ) ibuffr(ipstar+lennew) = index

      else

c     --- not enough space available
c         create new array with new length

         itemp = 0
         call isicreat ( itemp, lennew , 'itemp' )

c        --- copy old array to new array

         ipstar = isingetp(index)
         ipcopy = isingetp(itemp)
         call isincopy ( ibuffr, ibuffr, ipstar, ipcopy, lenold, 0 )
         isave = index
         call isdelarr(isave)

c        --- Reset index and control integers

         ibuffr(11+3*(index-1)) = ipcopy
         ibuffr(12+3*(index-1)) = lennew
         ibuffr(13+3*(index-1)) = 10
         ibuffr(11+3*(itemp-1)) = 0
         ibuffr(12+3*(itemp-1)) = 0
         ibuffr(13+3*(itemp-1)) = 0
         if ( ibuffr(3).eq.itemp ) ibuffr(3) = ibuffr(3) - 1
         if ( idebug.gt.0 ) then
            ibuffr(ipcopy-1) = index
            ibuffr(ipcopy+lennew) = index
         end if

      end if

      end
