      subroutine isfreesp ( ipstarfr, maxlen )
c ======================================================================
c
c
c        programmer     Guus Segal
c        version 1.0    date   21-06-94 (Extracted from the SEPRAN subroutine
c                                        ini058 version 1.1)
c
c
c   copyright (c) 1994  "Ingenieursbureau SEPRA"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "Ingenieursbureau SEPRA".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c ********************************************************************
c
c                         DESCRIPTION
c
c     Check how much contiguous space is available in array IBUFFR
c     from position IPSTAR
c ********************************************************************
c
c                      KEYWORDS
c
c     memory_management
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer ipstarfr, maxlen

c     ipstarfr i  Position in array IBUFFR from which the available space
c                 must be found
c     maxlen   o  Number of words available in array IBUFFR from position IPSTAR
c ********************************************************************
c
c                    COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisnas'
      include 'IScommon/cdebug'
      include 'IScommon/cismessg'
      include 'IScommon/cismessc'
c ********************************************************************
c
c                    LOCAL PARAMETERS

      integer i, last, iextra, ifirsv, nbuffr, iinfor, ibfree, nvectr

c     i       General loop variable
c     ibfree  Next free position in array IBUFFR
c     iextra  Extra space needed at both sides of the array for control
c             integers
c     ifirsv  Help variable to save the value of ifirsw temporarily
c     iinfor  Number of arrays that has been referenced
c     last    Last position used by array in IBUFFR corresponding to index
c     nbuffr  Length of buffer array ibuffr
c     nvectr  Maximum number of arrays that may be stored in IBUFFR
c ********************************************************************
c
c             SUBROUTINES CALLED
c
c     ISINICHK  Check IBUFFR
c ********************************************************************
c
c                              I/O
c
c     none
c ********************************************************************
c
c                ERROR MESSAGES:
c
c     -
c ********************************************************************
c
c                    PSEUDO CODE
c
c     If ( idebug > 0 ) then
c        Set iextra = extra space needed
c        check all control integers
c     if ( ibfree=ipstar ) then
c        maxlen = nbuffr-ipstar+1
c     else
c        Find smallest starting address in IBUFFR that may not be used
c ======================================================================
c
      nvectr = ibuffr(2)
      iinfor = ibuffr(3)
      ibfree = ibuffr(4)
      nbuffr = ibuffr(7)
       if ( idebug.eq.0 ) then
         iextra = 0
      else
         iextra = intlen
         ifirsv = ifirws
         ifirws = 1
         call isinichk ( ibuffr, ibuffr(11) )
         ifirws = ifirsv
      end if

      if ( ibfree.eq.ipstarfr ) then

c     --- ipstar is ibfree, hence all space after ipstar is available

         maxlen = nbuffr - ipstarfr + 1 - iextra

      else

c     --- Find smallest starting address in IBUFFR that may not be used

         last = nbuffr + 1
         do 100 i = 1, iinfor
            if ( ibuffr(11+3*(i-1)).gt.ipstarfr )
     +         last = min ( last,ibuffr(11+3*(i-1)) )
100      continue
         maxlen = last - ipstarfr - 2* iextra

      end if

      if ( idebug.gt.0 ) then
         nbufw3 = ipstarfr + maxlen
         ibuffr(nbufw3) = nvectr+1
      end if
      end
