      subroutine isini102 ( iwork, numarr, ibuffr, infor, iarstr, ksort,
     +                      lspace )
c ======================================================================
c
c
c        programmer     Guus Segal
c        version 1.0    date   21-06-94 (Extracted from the SEPRAN subroutine
c                                        ini102 version 1.4)
c
c
c   copyright (c) 1994  "Ingenieursbureau SEPRA"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "Ingenieursbureau SEPRA".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c ********************************************************************
c
c                         DESCRIPTION
c
c   sort the sequence in array infor and store in array iwork
c   at output numarr is the number of arrays stored in iwork
c ********************************************************************
c
c                      KEYWORDS
c
c    memory_management
c    sort
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer iwork(*), numarr, infor(3,*), iarstr(*), ksort(*),
     +        lspace(*), ibuffr(*)

c     iarstr       Contains the starting addresses of the arrays in IBUFFR
c     ibuffr  i    reserved
c     infor   i    Information concerning the storage of arrays in IBUFFR
c                  is stored in the following way:
c                  Pos. (1,i)  If > 0: array i is stored in IBUFFR from position
c                              stored in infor(1,i)
c                              If < 0: array i is stored in file 3 from record
c                              number -infor(1,i)
c                  Pos. (2,i)  Length of the array in words
c                  Pos. (3,i)  Priority number. Is used to decide which arrays
c                              are put to file 3. 0 <= prio <= 10
c     iwork   i    Work array of length nvectr, in which the entries in infor
c                  are sorted for increasing starting address
c                  iwork(1) contains the index of the array with the smallest
c                           starting address,
c                  iwork(2) with the next starting address etc.
c     ksort        Contains the sorted sequence of the arrays
c     lspace       Integer work space for subroutine chsort
c     numarr  o    Number of arrays that are stored in IBUFFR
c ********************************************************************
c
c                      COMMON BLOCKS
c
c ********************************************************************
c
c                    LOCAL PARAMETERS
c
      integer i, iinfor

c     i       General loop variable
c     iinfor  Number of arrays that has been referenced
c ********************************************************************
c
c             SUBROUTINES CALLED
c
c     ISCHSORT  Sort integer array
c ********************************************************************
c
c                    I/O
c
c    none
c ********************************************************************
c
c                    ERROR MESSAGES
c
c   -
c ********************************************************************
c
c                    PSEUDO CODE
c
c   Store sequence numbers of all arrays in IBUFFR in iwork and corresponding
c   starting addresses in keysrt
c   Sort keysrt, the result is stored in KSORT
c   Copy iwork into lspace
c   Put final ordering in IWORK
c ======================================================================
c
c     --- Store sequence numbers into IWORK and starting addresses in KEYSRT

      iinfor = ibuffr(3)
      numarr = 0
      do 100 i = 1,iinfor
         if ( infor(1,i).gt.0 .and. infor(2,i).gt.0) then

c        --- infor(1,i) > 0   array "i" is stored in ibuffr

            numarr = numarr+1
            iwork(numarr)  = i
            iarstr(numarr) = infor(1,i)

         end if

100   continue

c     --- Sort KEYSRT

      call ischsort ( iarstr, ksort, numarr, lspace, numarr )

c     --- Copy iwork into lspace

      do 110 i = 1, numarr
         lspace(i) = iwork(i)
110   continue

c     --- Fill iwork finally

      do 120 i = 1, numarr
         iwork(i) = lspace(ksort(i))
120   continue

      end
