      subroutine isreadfl ( indref, iunder, iupper, ireffile, inparray )
c ======================================================================
c
c
c        programmer     Guus Segal
c        version 1.1    date   01-05-95 iref=2 added
c        version 1.0    date   21-06-94 (Extracted from the SEPRAN subroutine
c                                        readfl version 3.7)
c
c
c   copyright (c) 1994  "Ingenieursbureau SEPRA"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "Ingenieursbureau SEPRA".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c ********************************************************************
c
c                         DESCRIPTION
c
c   read inparray from direct access file into core
c
c ********************************************************************
c
c                      KEYWORDS
c
c   direct_access
c   read
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer inparray(*), indref, iunder, iupper, ireffile

c     indref    i  Record number from which inparray has to be read
c     inparray  i  Array from which the input is read depending on the
c                  values of ichois and iref
c     ireffile  i  Dataset reference number indicating the type of file
c                  ireffile = 1, 2, 3, 4 or 17
c     iunder    i  First element in inparray to be read
c     iupper    i  Last element in inparray to be read
c ********************************************************************
c
c                      COMMON BLOCKS
c
      include 'IScommon/cisnas'
      include 'IScommon/cismessg'
      include 'IScommon/cismessc'
      include 'IScommon/cfile3'
c ********************************************************************
c
c                    LOCAL PARAMETERS
c
      integer jref, nrec, numrec, iho, i, iht, k, ios, irec2

c     i         General loop variable
c     iho       Lower index of array to be written in present record
c     iht       Upper index of array to be written in present record
c     ios       Iostatus variable
c     irec2     Last reccord used plus one of file 2
c     jref      Reference number
c     k         General loop variable
c     nrec      Record length of file
c     numrec    Number of records to be written
c ********************************************************************
c
c             SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRINT  Put integer value in error message file
c     ISERRSUB  Error messages
c     ISSTOP    Stop ISNAS
c ********************************************************************
c
c                    I/O
c
c    none
c ********************************************************************
c
c                    ERROR MESSAGES
c
c    1420   Error in reading direct access file
c ********************************************************************
c
c                    PSEUDO CODE
c
c   trivial
c ======================================================================
      if ( ireffile.eq.2 ) then

c     --- iref = 2

         jref = iref2
         nrec = nrec2
         irec2 = indref+(iupper-iunder)/nrec2+1
         ifree2 = max(ifree2,irec2)

      else       if ( ireffile.eq.3 ) then

c     --- iref = 3

         jref = iref3
         nrec = nrec3
         irec3 = indref+(iupper-iunder)/nrec3+1
         ifree3 = max(ifree3,irec3)

      else if ( ireffile.eq.4 ) then

c     --- iref = 4

         jref = irefer
         nrec = nrec4

      end if

      if ( nrec.eq.0 ) then
         write(*,110) ireffile
110      format ( ///' ************************************'//
     +               ' fatal error in subroutine isreadfl '/
     +               ' File with symbolic reference number',i2/
     +               ' has not been opened, whereas you try to'/
     +               ' read from this file'//
     +               ' Execution is terminated' //
     +               ' ************************************')
         call isstop
      end if
      numrec=(iupper-iunder)/nrec+1
      iho=iunder

      if ( ioutpt.ge.2 ) then

c     --- Extra writes to print all record numbers that are used

         call iseropen ( 'isreadfl' )
         write(irefwr,120) namsubs(levelsub), ireffile, nrec, indref,
     +      indref+numrec-1
120      format(' Reading from file in subroutine ',a/
     +          ' File with symbolic ref. number',i2,'  record length',
     +          i5, '  records', i4, ' to ', i4 )
         call iserclos ( 'isreadfl' )

      end if

      do 200 i=1,numrec
         iht=min(iupper,iho+nrec-1)
         read(jref,rec=indref+i-1,iostat=ios) (inparray(k),k=iho,iht)
         if ( ios.ne.0 ) then

c        --- Error in reading file

            if ( jref .ne. irefer ) then

               call iseropen ( 'isreadfl' )
               call iserrint ( jref, 1 )
               call iserrint ( indref+i-1, 2 )
               call iserrint ( ios, 3 )
               call iserrsub ( 1420, 3, 0, 0 )
               call iserclos ( 'isreadfl' )

            else

c           --- Error in reading file4,  avoid the infinite recursion

               write ( *,      '( // a / )')
     +         ' ********* fatal error in isreadfl, when reading file4'
               write ( irefwr, 130 ) numrec, jref, indref, ios
130            format(//' Error occcured when reading ',i3, ' records',
     +                ' from file with reference number',i3/
     +                ' starting record',i7/
     +                ' iostat parameter has value', i5 )
               call isstop

            endif

         end if
         iho=iht+1
200   continue
      end
