      subroutine istmspac ( length, ipstar )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.0    date   21-06-1994 (Extracted from the SEPRAN subroutine
c                                        ini072 version 1.4)
c
c
c   copyright (c) 1994  "Ingenieursbureau SEPRA"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "Ingenieursbureau SEPRA".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                         DESCRIPTION
c
c     compute starting position of a temporary array of length LENGTH in
c     array ibuffr
c     If space is available IBFREE is not adjusted, which means that ISTMSPAC
c     may only be called once during a process because otherwise the same
c     start address might be found.
c     If more arrays must be reserved in one time it is necessary to give the
c     sum of the length of all these arrays.
c
c     Remember: use first all reals and then the integers!
c
c     if no space is available in array IBUFFR, ISTMSPAC tries to
c     create space in the following ways:
c
c       -  by writing low priority arrays to backing storage
c
c       -  if this is not sufficient by compressing
c          array ibuffr after writing of all arrays with priority
c          smaller than 10 to backing storage
c
c     As a consequence starting positions that have been computed before may
c     be changed and should be recomputed
c     The name of the calling subroutine is extracted from common blocks
c     CISMESSC and CISMESSG, so it is supposed that the subroutine ISEROPEN has
c     been used
c **********************************************************************
c
c                      KEYWORDS
c
c     memory_management
c **********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer length, ipstar

c     ipstar  o  Computed starting position in array IBUFFR
c     length  o  Length of the array in words
c **********************************************************************
c
c                      COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisnas'
      include 'IScommon/cdebug'
      include 'IScommon/cismessg'
      include 'IScommon/cismessc'
c **********************************************************************
c
c                    LOCAL PARAMETERS
c
      integer iextra, nbuffr, ibfree, nvectr

c     ibfree  Next free position in array IBUFFR
c     iextra  Extra space needed at both sides of the array for control
c             integers
c     nbuffr  Length of buffer array ibuffr
c     nvectr  Maximum number of arrays that may be stored in IBUFFR
c **********************************************************************
c
c             SUBROUTINES CALLED
c
c     ISERRINT  Put integer value in error message file
c     ISERRSUB  Error messages
c     ISINI049  Create space in IBUFFR by writing low priority arrays to
c               file 3, or to compress IBUFFR
c     ISINICHK  Check IBUFFR
c     ISPRINBF  Print and check length of array IBUFFR
c     ISSTOP    Stop ISNAS
c **********************************************************************
c
c                    I/O
c
c     If necessary arrays are written to file 3 (iref3)
c
c **********************************************************************
c
c                    ERROR MESSAGES
c
c    1401   length of array less than one
c **********************************************************************
c
c                    PSEUDO CODE
c
c     If ( idebug > 0 ) then
c        Set iextra = extra space needed
c        check all control integers
c     If ( enough space in IBUFFR after IBFREE ) then
c         IPSTAR = IBFREE
c     else
c         Create space by writing low priority arrays to file 3
c         or compressing IBUFFR
c     If ( idebug > 0 ) then
c        place control integers for this array
c ======================================================================
c
      nvectr = ibuffr(2)
      ibfree = ibuffr(4)
      nbuffr = ibuffr(7)
      if ( idebug.eq.0 ) then
         iextra = 0
      else
         iextra = intlen
         ifirws = 0
         call isinichk ( ibuffr, ibuffr(11) )
      end if
      if ( length.le.0 ) then

c     --- length <= 0, error

         call iserrint ( length, 1 )
         call iserrsub ( 1401, 1, 0, 0 )
         call isstop

      end if

      ipstar=1
      if ( ibfree+length+iextra*2-1.le.nbuffr ) then

c     --- enough free space available   ( standard situation )

         ipstar = ibfree

      else

c     --- try to create free space by writing low priority arrays
c         to backing storage ( file 3 ) or compressing ibuffr

         call isini049 ( length, ipstar, ibuffr )

      end if

      if ( idebug.gt.0 ) then

         nbufw1 = ipstar-1
         ibuffr(nbufw1) = nvectr+1
         nbufw2 = ipstar+length
         ibuffr(nbufw2) = nvectr+1

      end if
      call isprinbf ( ibuffr, ibfree-1 )

      if ( ioutpt.gt.2 ) then
         write(irefwr,200) ipstar, namsubs(levelsub)//':istmspac'
200      format ( ' Create work space with starting address',
     +            i9/' in subroutine ', a )
      end if

      end
