      function isbrzo ( abr ,bbr ,machep ,tolnln , cons1nl,
     +                  cons2nl, cons3nl, rincom , mscal, istartup,
     +                  rincav)
c ======================================================================
c
c     programmer     Duncan van der Heul
c     version      2.2    date 23-02-2000  Duncan: added no convergence
c     version      2.1    date 01-02-1999  Duncan: revision for efficiency
c     version      2.0    date 06-08-1998  Duncan: extra parameter istartup
c     version      1.1    date 13-04-1998  Duncan: fixed bug for mscal=0
c     version      1.0    date 01-04-1998
c
c
c   copyright (c) 1998-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    isbrezo is an adaptation of the algorithm by Brent for the
c    solution of a non linear equation f(x)=0 of the Dekker-algorithm
c    type. The algorithm is a combination of bisection steps and
c    interpolation. See:
c    Brent, Richard P., Algorithms for minimization without Derivatives,
c    Prentice-Hall series in automatic computation,
c    Prentice-Hall, Inc, Englewood Cliffs, N.J., 1973:pp 47-60.
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer mscal, istartup
      double precision abr, bbr, machep, tolnln, cons1nl, cons2nl,
     +                 cons3nl, rincom(*), rincav(*)

c     abr       i  End point of the search interval,
c                  sign(f(abr.ne.sign(f(bbr
c     bbr       i  End point of the search interval,
c                  sign(f(abr.ne.sign(f(bbr
c     cons1nl   i  Constant 1 in non linear equation
c     cons2nl   i  Constant 2 in non linear equation
c     cons3nl   i  Constant 3 in non linear equation
c     isbrzo    o  Function value f(isbrezo)=0
c     istartup  i  If 1 the cavity calculation is in the startup procedure,
c                  if 0 not
c     machep    i  Machine precision
c     mscal        Indicates use of non-dimensionalisation for compr. flow:
c                  0:   Equations are non-dimensionalised
c                  1:   Equations are in dimensional formulation
c     rincav    i  reserved
c     rincom    i  reserved
c     tolnln    i  Tolerance for solution of nonlinear equation
c                  without machine precision
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      double precision sabr, sbbr, cbr, dbr, ebr, fabr, fbbr, fcbr,
     +                 toltot, mbr, pbr, qbr, rbr, sbr, isbrzo
      integer iter, maxiter

c     cbr       General help variable
c     dbr       General help variable
c     ebr       General help variable
c     fabr      Function value of function f at a
c     fbbr      Function value of function f at b
c     fcbr      Function value of function f at c
c     iter      The current iteration number.
c     maxiter   Maximum number of iterations
c     mbr       General help variable
c     pbr       General help variable
c     qbr       General help variable
c     rbr       General help variable
c     sabr      General help variable
c     sbbr      General help variable
c     sbr       General help variable
c     toltot    Tolerance for solution of non linear equation
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      double precision isstdfl, isstdls

c     ISSTDFL    dimensionfull equation of state
c     ISSTDLS    Dimensionless equation of state
c     ISSTOP     Stop ISNAS
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      if ( mscal.eq.0) then

         sabr =    abr
         sbbr =    bbr
         fabr = isstdls(cons3nl+abr,rincom, istartup, rincav)+
     +                             cons2nl*abr + cons1nl
         fbbr = isstdls(cons3nl+bbr,rincom, istartup, rincav)+
     +                             cons2nl*bbr + cons1nl
         cbr  = sabr
         fcbr = fabr
         ebr  = sbbr - sabr
         dbr  = ebr
         if(abs(fcbr).lt.abs(fbbr)) then
           sabr = sbbr
           sbbr =  cbr
           cbr  = sabr
           fabr = fbbr
           fbbr = fcbr
           fcbr = fabr
         end if

         toltot = 2.0d0*machep*abs(sbbr) + tolnln
         mbr    = 0.5d0*(cbr-sbbr)
         maxiter=30000
         iter=0
 5       if (abs(mbr).gt.toltot.and.iter.lt.maxiter) then
           iter=iter+1
           if (abs(ebr).lt.toltot.or.abs(fabr).le.abs(fbbr)) then
             ebr=mbr
             dbr=ebr
           else
             sbr=fbbr/fabr
             if (abs(sabr-cbr).lt.machep) then
               pbr=2.0d0*mbr*sbr
               qbr=1.0d0-sbr
             else
               qbr=fabr/fcbr
               rbr=fbbr/fcbr
               pbr=sbr*(2.0d0*mbr*qbr*(qbr-rbr)-(sbbr-sabr)*(rbr-1.0d0))
               qbr=(qbr-1.0d0)*(rbr-1.0d0)*(sbr-1.0d0)
             end if
             if (pbr.gt.0.0d0) then
               qbr=-qbr
             else
               pbr=-pbr
             end if
             sbr=ebr
             ebr=dbr
             if ((2.0d0*pbr).lt.(3.0d0*mbr*qbr*abs(toltot*qbr)).
     +                 and.pbr.lt.abs(0.5d0*sbr*qbr)) then
               dbr=pbr/qbr
             else
               ebr=mbr
               dbr=ebr
             end if
           end if
           sabr=sbbr
           fabr=fbbr
           if (abs(dbr).gt.toltot) then
             sbbr=sbbr+dbr
           else
             if (mbr.gt.0.0d0) then
               sbbr=sbbr+toltot
             else
               sbbr=sbbr-toltot
             end if
           end if
           fbbr=isstdls(cons3nl+sbbr,rincom, istartup, rincav)+
     +                             cons2nl*sbbr + cons1nl
           if ((fbbr.le.0d0.and.fcbr.le.0d0).or.
     +               (fbbr.gt.0d0.and.fcbr.gt.0d0))  then
             cbr  = sabr
             fcbr = fabr
             ebr  = sbbr - sabr
             dbr  = ebr
           end if
           if (abs(fcbr).lt.abs(fbbr)) then
             sabr = sbbr
             sbbr =  cbr
             cbr  = sabr
             fabr = fbbr
             fbbr = fcbr
             fcbr = fabr
           end if
           toltot = 2.0d0*machep*abs(sbbr) + tolnln
           mbr    = 0.5d0*(cbr-sbbr)
         go to 5
         end if

      else if( mscal.eq.1) then

         sabr =    abr
         sbbr =    bbr
         fabr = isstdfl(cons3nl+abr, istartup, rincav)+
     +                             cons2nl*abr + cons1nl
         fbbr = isstdfl(cons3nl+bbr, istartup, rincav)+
     +                             cons2nl*bbr + cons1nl
         cbr  = sabr
         fcbr = fabr
         ebr  = sbbr - sabr
         dbr  = ebr
         if(abs(fcbr).lt.abs(fbbr)) then
           sabr = sbbr
           sbbr =  cbr
           cbr  = sabr
           fabr = fbbr
           fbbr = fcbr
           fcbr = fabr
         end if

         toltot = 2.0d0*machep*abs(sbbr) + tolnln
         mbr    = 0.5d0*(cbr-sbbr)
         maxiter=30000
         iter=0
 15      if (abs(mbr).gt.toltot.and.iter.lt.maxiter) then
           iter=iter+1
           if (abs(ebr).lt.toltot.or.abs(fabr).le.abs(fbbr)) then
             ebr=mbr
             dbr=ebr
           else
             sbr=fbbr/fabr
             if (abs(sabr-cbr).lt.machep) then
               pbr=2.0d0*mbr*sbr
               qbr=1.0d0-sbr
             else
               qbr=fabr/fcbr
               rbr=fbbr/fcbr
               pbr=sbr*(2.0d0*mbr*qbr*(qbr-rbr)-(sbbr-sabr)*(rbr-1.0d0))
               qbr=(qbr-1.0d0)*(rbr-1.0d0)*(sbr-1.0d0)
             end if
             if (pbr.gt.0.0d0) then
               qbr=-qbr
             else
               pbr=-pbr
             end if
             sbr=ebr
             ebr=dbr
             if ((2.0d0*pbr).lt.(3.0d0*mbr*qbr*abs(toltot*qbr)).
     +                 and.pbr.lt.abs(0.5d0*sbr*qbr)) then
               dbr=pbr/qbr
             else
               ebr=mbr
               dbr=ebr
             end if
           end if
           sabr=sbbr
           fabr=fbbr
           if (abs(dbr).gt.toltot) then
             sbbr=sbbr+dbr
           else
             if (mbr.gt.0.0d0) then
               sbbr=sbbr+toltot
             else
               sbbr=sbbr-toltot
             end if
           end if
           fbbr=isstdfl(cons3nl+sbbr, istartup, rincav)+
     +                             cons2nl*sbbr + cons1nl
           if ((fbbr.le.0d0.and.fcbr.le.0d0).or.
     +               (fbbr.gt.0d0.and.fcbr.gt.0d0))  then
             cbr  = sabr
             fcbr = fabr
             ebr  = sbbr - sabr
             dbr  = ebr
           end if
           if (abs(fcbr).lt.abs(fbbr)) then
             sabr = sbbr
             sbbr =  cbr
             cbr  = sabr
             fabr = fbbr
             fbbr = fcbr
             fcbr = fabr
           end if
           toltot = 2.0d0*machep*abs(sbbr) + tolnln
           mbr    = 0.5d0*(cbr-sbbr)
         go to 15

         end if

      end if
      if (abs(mbr).gt.toltot) then
        write(*,*) 'no convergence in nonlinear solver',sbbr
        call isstop
      end if
      isbrzo=sbbr

      end
