      subroutine isdiag ( amat, nusol, ndimso, nconct, prec, nprec )
c ======================================================================
c
c        programmer     Kees Vuik
c        version 1.1    date 06-07-1998 Kees V: nusol, nsolut cleaned
c        version 1.0    date 10-07-1995 extracted from old ISNaS
c                       subroutine diag
c
c   copyright (c) 1994-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c      Makes a diagonal scaling of the matrix in case of a momentum
c      equations, a transport equation, or a pressure equation.
c **********************************************************************
c
c                       KEYWORDS
c
c      linear_solver
c      preconditioner
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer          nusol, ndimso, nconct, nprec
      double precision amat(1:nusol,1:ndimso,1:nconct),
     +                 prec(1:nusol,1:ndimso,1:nprec)

c     amat      i    The coefficients matrix used in the linear system
c     nconct    i    Number of connections in a row of the matrix
c     ndimso    i    Integer indicating the dimension of the space in
c                    which the problem must be solved (ndimso = 1 or ndim)
c     nprec     i    Maximum number of diagonals in preconditioning matrix
c     nusol     i    Number of unknowns for one degree of freedom in
c                    one block
c     prec      i    The preconditioning matrix
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j

c     i         General loop variable
c     j         General loop variable
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     To obtain a diagonal scaling of the matrix. This is done for
c     the momentum and pressure equation. The inverse of the main
c     diagonal is stored in the array prec. For transport equations
c     we use the same structure as the pressure equation.
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      do j = 1, ndimso
         do i = 1, nusol
            if ( amat(i,j,1).eq.0d0 ) then
               prec(i,j,1) = 1d0
            else
               prec(i,j,1) = 1d0/amat(i,j,1)
            end if
         end do
      end do

      end
