      subroutine isdmlu4(amat,n,ndimso,nconct,prec,nprec,initer,
     +                 reiter, work )
c ==================================================================
c
c
c     programmer       Kees Vuik
c     version 1.3     date 02-04-1997  Kees K. literal constants in
c                                       double precision for f90
c     version 1.2      date 04-08-1992  Zdenek: ndimso instead of ndim
c     version 1.1      date 01-06-1992
c     developed at     Convex, HP700
c
c ******************************************************************
c
c                       DESCRIPTION
c
c     Calculates an upper triangular matrix U and a lower triangular
c     matrix L, which form an incomplete decomposition of A.
c
c ******************************************************************
c
c                       KEYWORDS
c
c     linear_solver
c     ilu
c     preconditioner
c
c ******************************************************************
c
c                       INPUT / OUTPUT   PARAMETERS
c
c     amat    i  the coefficient matrix for the momentum equations
c                or an equation similar to the pressure equation.
c
c     n       i  number of unknowns in the solution vector.
c
c     nconct  i  number of connections in one row of the matrix
c
c     ndimso  i  integer indicating the dimension of the space in
c                which the problem must be solved (ndimso = 1 or ndim).
c
c     nprec   i  number of diagonals, which are used in the pre-
c                conditioning. In this subroutine nprec = nconct.
c
c     prec    o  the preconditioning matrix.
c
c ******************************************************************
c
c                       COMMON BLOCKS
c
c
c ******************************************************************
c
c                       LOCAL PARAMETERS
c
c     a       this parameter is used to make a combination of ILU
c             and MILU preconditioning.
c
c     i       loop counter.
c
c     n1      n1 is needed to describe the matrix elements.
c
c ******************************************************************
c
c                       SUBROUTINES CALLED
c
c ******************************************************************
c
c                       I/O
c
c ******************************************************************
c
c                       ERROR MESSAGES
c
c ******************************************************************
c
c                       PSEUDO CODE
c
c     This subroutine calculates an upper triangular matrix U and a lower
c     triangular matrix L, These matrices form an incomplete decomposition
c     for the matrix A using the following rules:
c
c            A = L U - R,
c
c     (a) diag(L) = I,
c
c     (b) The nonzero pattern of L and U are equal to the nonzero pattern
c         of A,
c
c     (c) If a(i,j) # 0 then L*U(i,j) = a(i,j)
c
c     The off-diagonal elements of U are stored in prec(1:n,1:ndimso,6:9),
c     whereas inverse(diag(U)) is stored in prec(1:n,1:ndimso,1).
c     The off-diagonal elements of L are stored in prec(1:n,1:ndimso,2:5),
c
c     **************************************************************
c
c     Remarks
c
c     There are two difficulties to obtain the incomplete decomposition
c
c     - firstly, since the index of the matrix element can be less than
c       1 we discriminate 9 different situations:(n1 = ni+3)
c              i = 1,
c              i = 2,...,n1-1
c              i = n1,
c              i = n1+1,
c              i = n1+2,...,n-n1-2,
c              i = n-n1-1
c              i = n-n1
c              i = n-n1+1,...,n-1
c              i = n
c
c     - secondly, it is known that in the virtual cells the corresponding
c       row and collumn in the matrix consists of zero elements. So it is
c       possible that d(i) = 0. To circumvent this possibility we imple-
c       ment the following check:
c          if(a(i,1).eq.0d0) then prec(i,1) = 1 else ...
c
c ==================================================================
c
c     Parameters
c
      implicit none
      integer          initer(*),n,ndimso,nconct,nprec
      double precision amat(1:n,1:nconct),
     +                 prec(1:n,1:nprec), reiter(*),
     +                 work(1:n,1:2)
c
c     local parameters
c
      integer          k,nfill,n1,infshi(100),mrel(9),m,i,p,j,
     1                 nfilon,nfilbo, mhulp, ninv
      double precision a, uhulp

c     statement hierna moet weg bij opschonen

      mhulp = ndimso

c     statement hierboven moet weg bij opschonen

c     maak bij opschonen de waarde van a zodanig dat hij ook goed
c     is bij IBLU

      a = 0.975d0
      a = 1.0d0
      reiter(7) = a
      nfill = (nprec-9)/2
      nfilon = nfill/2
      nfilbo = nfill-nfilon
      n1    = initer(12)+1
      if(nfill.gt.(n1-3)) then
         write(6,1010) nfill, n1-3
1010     format(' De waarde van nfill is' , i5,
     +          ' groter dan n1-3 : ', i5)
         stop
      endif
      if(nprec.gt.100) then
         write(6,2010) nprec
2010     format(' De waarde van nprec is', i5,
     +          ' groter dan 100')
         stop
      endif
c
c     fill the array infshi
c
      infshi(1) = 0
      do k = 1, nfilon+3
         infshi(k+1) = -n1-2+k
      enddo
      do k = 1, nfilbo+1
         infshi(k+nfilon+4) = -nfilbo-2+k
      enddo
      do k = 1, nfilbo+1
         infshi(k+nfill+5) = k
      enddo
      do k = 1, nfilon+3
         infshi(k+nfill+nfilbo+6) = n1-nfilon-2+k
      enddo
c
c     initialization
c
      do m = 1, 2*nfill+9
         do i = 1, n
            prec(i,m) =0d0
         enddo
      enddo
      mrel(1) = 1
      mrel(2) = 2
      mrel(3) = 3
      mrel(4) = 4
      mrel(5) = nfill+5
      mrel(6) = nfill+6
      mrel(7) = 2*nfill+7
      mrel(8) = 2*nfill+8
      mrel(9) = 2*nfill+9
      do i = 1, n
         work(i,1) = 0d0
         work(i,2) = 0d0
      enddo
      do m = 1, 9
         do i = 1, n
            prec(i,mrel(m)) =amat(i,m)
            work(i,1) = work(i,1)+amat(i,m)
         enddo
      enddo
      if(nfill.lt.n1-3) then
         do i = 1, n
c
c        compute U
c
            do m = nfill+6, 2*nfill+9
               do p = 2, nfill+5
                  if(i+infshi(p).ge.1) then
                     mhulp = infshi(m)-infshi(p)
                     ninv  = 0
                     if((1.le.mhulp).and.(mhulp.le.nfilbo+1)) then
                        ninv = nfill+5+mhulp
                     endif
                     if(((n1-nfilon-1).le.mhulp).and.(mhulp.le.(n1+1)))
     +                  then
                        ninv = 2*nfill+8-n1+mhulp
                     endif
                     if(ninv.gt.0) then
                        prec(i,m) = 
     +                  prec(i,m)-prec(i,p)*prec(i+infshi(p),ninv)
                     endif
                  endif
c
c              end of loop p
c
               enddo
               work(i,2) = work(i,2)+prec(i,m)
c
c           end of loop m
c
            enddo
            m = 1
            do p = 2, nfill+5
               if(i+infshi(p).ge.1) then
                  mhulp = infshi(m)-infshi(p)
                  ninv  = 0
                  if((1.le.mhulp).and.(mhulp.le.nfilbo+1)) then
                     ninv = nfill+5+mhulp
                  endif
                  if(((n1-nfilon-1).le.mhulp).and.(mhulp.le.(n1+1)))
     +               then
                     ninv = 2*nfill+8-n1+mhulp
                  endif
                  if(ninv.gt.0) then
                     prec(i,m) =
     +               prec(i,m)-prec(i,p)*prec(i+infshi(p),ninv)
                  endif
               endif
c
c           end of loop p
c
            enddo
c
c           The milu approach
c
            uhulp = work(i,1)-work(i,2)
            do p = 2, nfill+5
               if(i+infshi(p).ge.1) then
                  uhulp = uhulp-prec(i,p)*work(i+infshi(p),2)
               endif
            enddo
            prec(i,1) = (1-a)*prec(i,1)+a*uhulp
            work(i,2) = work(i,2)+prec(i,1)
            if(prec(i,1).eq.0d0) then
               prec(i,1) = dble(1)
            else
               prec(i,1) = dble(1)/prec(i,1)
            endif
c
c           compute L
c
            do m = 2, nfill+5
               if((i-infshi(m)).le.n) then
                  do p = nfill+6, 2*nfill+9
                     if((i-infshi(p)).ge.1) then
                        mhulp = infshi(m)-infshi(p)
                        ninv  = 0
                        if(((-nfilbo-1).le.mhulp).and.(mhulp.le.(-1)))
     +                     then
                           ninv = nfill+6+mhulp
                        endif
                        if(((-n1-1).le.mhulp).and.
     +                     (mhulp.le.(-n1+nfilon+1))) then
                           ninv = n1+3+mhulp
                        endif
                        if(ninv.gt.0) then
                           prec(i-infshi(m),m) = prec(i-infshi(m),m)
     +                     -prec(i-infshi(m),ninv)*prec(i-infshi(p),p)
                        endif
                     endif
c
c                 end of loop p
c
                  enddo
                  prec(i-infshi(m),m) = prec(i-infshi(m),m)*prec(i,1)
               endif
c
c           end of loop m
c
            enddo
c
c        end of loop i
c
         enddo
      else
         do i = 1, n
c
c        compute U
c
         do p = 2, n1+2
            if(i-n1-3+p.ge.1) then
               prec(i,1) = prec(i,1)-prec(i,p)*prec(i-n1-3+p,2*n1+5-p)
            endif
         enddo
         if(prec(i,1).eq.0d0) then
            prec(i,1) = dble(1)
         else
            prec(i,1) = dble(1)/prec(i,1)
         endif
         do m = n1+3, 2*n1+3
            do p = m-n1, n1+2
                if(i-n1-3+p.ge.1) then
                   prec(i,m) = prec(i,m)-
     +                         prec(i,p)*prec(i-n1-3+p,m+n1+3-p)
                endif
            enddo
         enddo
c
c        compute L
c
         do m = 2, n1+2
            if(i+n1+3-m.le.n) then
               do p = n1+3, n1+m
                  if(i+n1+2-p.ge.1) then
                     prec(i+n1+3-m,m) = prec(i+n1+3-m,m)-
     +                   prec(i+n1+3-m,n1+m-p+2)*prec(i+n1+2-p,p)
                  endif
               enddo
               prec(i+n1+3-m,m) = prec(i+n1+3-m,m)*prec(i,1)
            endif
         enddo
c
c        end of loop i
c
         enddo
      endif
      do j = 1, 2
         do i = 1, n
            work(i,j) = 0d0
         enddo
      enddo
      end
