      subroutine isgsdpsm ( dpold, dpsave, dpiter, iinput, itergs )
c ======================================================================
c
c     programmer     Duncan van der Heul
c
c     version      1.3    date 03-01-2000 Duncan: added itergs to
c                                                 pam list.
c     version      1.2    date 08-07-1998 Kees V: nusol, nsolut cleaned
c     version      1.1    date 12-04-1998 Duncan:save old pressure
c                                         correction for matrix update
c     version      1.0    date 01-04-1998
c
c
c   copyright (c) 1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c  Subroutine isgsdpsm calculates the sum of the pressure correction
c   due to the non-linear Gauss-Seidel step and the GMRES step.
c **********************************************************************
c
c                       KEYWORDS
c
c     compressible
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iinput(*), itergs
      double precision dpiter(*), dpold(*), dpsave(*)

c     dpiter    i   work array for non linear Gauss-Seidel algorithm
c     dpold    i/o  youngest estimate for the total pressure correction
c                   due to the non linear Gauss-Seidel step and the
c                   GMRES acceleration step
c     dpsave    i   work array which saves the pressure correction
c     iinput    i   reserved
c     itergs    i   Number of the iteration in the mixed non linear
c                   Gauss-Seidel/GMRES algorithm
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ni, nj, nk, nusol, j
      double precision facsum

c     facsum    underrelaxation factor for accelerated Gauss-Seidel
c     j         General loop variable
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     nusol     Number of unknowns for one degree of freedom in
c               one block
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c   for all non-virtual points do:
c
c   dp      = dp      + dp
c     total     gauss     gmres
c
c   dp      = dp
c     save      total
c   end do
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      ni        = iinput(51)
      nj        = iinput(52)
      nk        = iinput(53)
      nusol     = (ni+1)*(nj+1)*(nk+1)
      facsum =   1d0

      if( itergs.gt.4 .and. itergs.le.20) then
          facsum = 0.5d0
      else if( itergs.gt.20 ) then
          facsum =   0.2d0
      end if

      do j=1, nusol
         dpold(j)=facsum*dpold(j)+dpiter(j)
         dpsave(j)=dpold(j)
      end do

      end
