      subroutine ismtvc21 ( matrix, solut, rhside, ni, nj, nvirtual,
     +                      nconct, mmctinv )
c ======================================================================
c
c        programmer     Guus Segal
c        version 2.2    date   06-08-1998 Guus: remove superfluous parameters
c        version 2.1    date   19-11-1997 Kees  K.: iperiod removed
c        version 2.0    date   02-10-1995 Kees  K. solution array contains
c                                         virtual points now
c        version 1.0    date   04-04-1995
c
c
c   copyright (c) 1995-1998  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     This subroutine performs the of the matrix vector
c     multiplication for the 2D momentum equations in case of periodical bc's
c       A x = b, where b and x have the same structure
c **********************************************************************
c
c                       KEYWORDS
c
c      multiplication
c      matrix
c      vector
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, nconct, mmctinv(0:3,2,nconct)
      double precision matrix(ni+1,nj+1,2,nconct),
     +                 rhside(ni+1,nj+1,2),
     +                 solut(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,2)

c     matrix    i    reserved
c     mmctinv   i    Matrix code table inverse transport equation see
c                    array infmt
c     nconct    i    Number of connections in a row of the matrix
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     rhside    o    reserved
c     solut     i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, idegfd, imhi, imlo, ishift, j, jdegfd, jmhi, jmlo,
     +        jshift, k, jshi, jslo, ishi, islo

c     i         General loop variable
c     idegfd    Loop variable with respect to a given unknown
c     imhi      Upper bound on i-index for matrix and rhside
c     imlo      Lower bound on i-index for matrix and rhside
c     ishi      Upper bound on i-index for solution vector
c     ishift    Shift in i-direction
c     islo      Lower bound on i-index for solution vector
c     j         General loop variable
c     jdegfd    Loop variable with respect to a given unknown
c     jmhi      Upper bound on j-index for matrix and rhside
c     jmlo      Lower bound on j-index for matrix and rhside
c     jshi      Upper bound on j-index for solution vector
c     jshift    Shift in j-direction
c     jslo      Lower bound on j-index for solution vector
c     k         General loop variable
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     The matrix vector multiplication is defined by:  a   x  = f
c                                                       ij  j    i
c
c     Since the matrix has a different structure for both degrees of freedom
c     we have to perform different loops
c     Due to the periodical boundary conditions it is necessary to perform
c     a loop in both i and j directions
c     As a consequence the matrix-vector multiplication becomes less efficient
c     than for the standard multiplication defined in ismtvc01
c
c     Algorithm:
c
c     a. In case no periodical boundary conditions are present
c
c     For idegfd := 1 (1) 2 do
c         Compute imli, imhi, jmlo, jmhi
c         For i := iunder (1) imhi do
c            For j := jmlo (1) jmhi do
c               f(i,j,idegfd) := a(i,j,idegfd,1)*x(i,j,idegfd)
c     For idegfd := 1 (1) 2 do
c        For k := 1, nconct do
c           jdegfd := mmctinv(0,idegfd,k)
c           ishift := mmctinv(1,idegfd,k)
c           jshift := mmctinv(2,idegfd,k)
c           Compute imli, imhi, jmlo, jmhi
c           For i := imli (1) imhi do
c              For j := jmlo (1) jmhi do
c                 f(i,j,idegfd) := f(i,j,idegfd)+a(i,j,idegfd,k)*
c                                  x(i+ishift,j+jshift,jdegfd)
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- Multiply by diagonal

      do idegfd = 1, 2

         do j = 1, nj+1
            do i = 1, ni+1
               rhside(i,j,idegfd) = matrix(i,j,idegfd,1) *
     +                              solut(i,j,idegfd)
            end do
         end do

      end do

c     --- Multiply rest of matrix
c         Loop over the degrees of freedom

      do idegfd = 1, 2

         if ( idegfd.eq.1 ) then
            imhi = ni+1
            jmhi = nj
         else
            imhi = ni
            jmhi = nj+1
         end if
         imlo = 1
         jmlo = 1

c        --- Loop over the connections

         do k = 2, nconct

            jdegfd = mmctinv(0,idegfd,k)
            ishift = mmctinv(1,idegfd,k)
            jshift = mmctinv(2,idegfd,k)
            if ( jdegfd.eq.1 ) then
               ishi = ni+1-ishift
               jshi = nj-jshift
            else
               ishi = ni-ishift
               jshi = nj+1-jshift
            end if
            islo = 1-ishift
            jslo = 1-jshift

            islo = imlo
            ishi = imhi
            jslo = jmlo
            jshi = jmhi

c           --- Corrections for periodical boundary conditions

            do j = max(jmlo,jslo), min(jmhi,jshi)
               do i = max(imlo,islo), min(imhi,ishi)
                  rhside(i,j,idegfd) = rhside(i,j,idegfd) +
     +                                 matrix(i,j,idegfd,k) *
     +                                 solut(i+ishift,j+jshift,jdegfd)
               end do
            end do

         end do

      end do

      end
