      subroutine ismtvc22 ( matrix, solut, rhside, ni, nj, nvirtual,
     +                      nconct, smctinv )
c ======================================================================
c
c
c        programmer     Guus Segal
c        version 2.1    date   19-10-97 Kees  K. : iperiod removed
c        version 2.0    date   02-10-95 Kees  K. solution array contains
c                                       virtual points now
c        version 1.0    date   04-04-95
c
c
c   copyright (c) 1995-1995  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c ********************************************************************
c
c                         DESCRIPTION
c
c     This subroutine performs the of the matrix vector
c     multiplication for the 2D transport equations in case of periodical bc's
c
c       A x = b, where b and x have the same structure
c
c ********************************************************************
c
c                      KEYWORDS
c
c      multiplication
c      matrix
c      vector
c ********************************************************************
c
c           INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, nconct, smctinv(0:3,nconct)
      double precision matrix(ni+1,nj+1,nconct),
     +                 rhside(ni+1,nj+1),
     +                 solut(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual)


c     matrix    i    reserved
c     nconct    i    Number of connections in a row of the matrix
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     rhside    o    reserved
c     smctinv   i    Matrix code table inverse for the transport
c                    equation see array infmt
c     solut     i    reserved
c***********************************************************************
c
c                   COMMON BLOCKS
c
c
c ********************************************************************
c
c                    LOCAL PARAMETERS
c
      integer i, imhi, imlo, ishift, j, jmhi, jmlo,
     +        jshift, k, jshi, jslo, ishi, islo

c     i         General loop variable
c     imhi      Upper bound on i-index for matrix and rhside
c     imlo      Lower bound on i-index for matrix and rhside
c     ishi      Upper bound on i-index for solution vector
c     ishift    Shift in i-direction
c     islo      Lower bound on i-index for solution vector
c     j         General loop variable
c     jmhi      Upper bound on j-index for matrix and rhside
c     jmlo      Lower bound on j-index for matrix and rhside
c     jshi      Upper bound on j-index for solution vector
c     jshift    Shift in j-direction
c     jslo      Lower bound on j-index for solution vector
c     k         General loop variable
c
c ********************************************************************
c
c                         I/O
c
c ********************************************************************
c
c                      SUBROUTINES CALLED
c
c ********************************************************************
c
c                     ERROR MESSAGES
c
c ********************************************************************
c
c                     PSEUDO CODE
c
c     The matrix vector multiplication is defined by:  a   x  = f
c                                                       ij  j    i
c
c     Since the matrix has a different structure for both degrees of freedom
c     we have to perform different loops
c     Due to the periodical boundary conditions it is necessary to perform
c     a loop in both i and j directions
c     As a consequence the matrix-vector multiplication becomes less efficient
c     than for the standard multiplication defined in ismtvc01
c
c     Algorithm:
c
c     a. In case no periodical boundary conditions are present
c
c     For i := imlo (1) imhi do
c        For j := jmlo (1) jmhi do
c           f(i,j) := a(i,j,1)*x(i,j)
c     For k := 1, nconct do
c        ishift := smctinv(1,idegfd,k)
c        jshift := smctinv(2,idegfd,k)
c        Compute imlo, imhi, jmlo, jmhi
c        For i := imlo (1) imhi do
c           For j := jmlo (1) jmhi do
c              f(i,j) := f(i,j)+a(i,j,k)*x(i+ishift,j+jshift)
c ======================================================================
c
c     --- Detect type of periodicity


c     --- Multiply by diagonal

      do j = 1, nj+1
         do i = 1, ni+1
            rhside(i,j) = matrix(i,j,1) * solut(i,j)
         end do
      end do

c     --- Multiply rest of matrix

      imlo = 1
      jmlo = 1
      imhi = ni
      jmhi = nj

c     --- Loop over the connections

      do k = 2, nconct

         ishift = smctinv(1,k)
         jshift = smctinv(2,k)
         islo = 1-ishift
         jslo = 1-jshift
         ishi = ni-ishift
         jshi = nj-jshift


         islo = imlo
         ishi = imhi
         jslo = jmlo
         jshi = jmhi


         do j = max(jmlo,jslo), min(jmhi,jshi)
            do i = max(imlo,islo), min(imhi,ishi)
               rhside(i,j) = rhside(i,j) + matrix(i,j,k) *
     +                       solut(i+ishift,j+jshift)
            end do
         end do

      end do

      end
