      subroutine ismtvc31 ( matrix, solut, rhside, ni, nj, nk, nvirtual,
     +                      nconct, mmctinv, iperiod )
c ======================================================================
c
c
c        programmer     Kees Kassels
c        version 2.1    date 25-10-95 Kees  K. bug fixed
c        version 2.0    date 04-10-95 Kees  K. solution array contains
c                                       virtual points now
c        version 1.1    date 03-08-95 Michiel K: Corrections for
c                       3D periodic bc.
c        version 1.0    date 04-04-95 (Renamed subroutine mtvc1)
c
c
c   copyright (c) 1995-1995  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c ********************************************************************
c
c                         DESCRIPTION
c
c     This subroutine performs the matrix vector multiplication for
c     the 3D momentum equations in case of periodical bc's
c
c       A x = b, where b and x have the same structure
c
c ********************************************************************
c
c                      KEYWORDS
c
c      multiplication
c      matrix
c      vector
c ********************************************************************
c
c           INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nk, nvirtual, nconct, mmctinv(0:3,3,nconct),
     +        iperiod
      double precision matrix(ni+1,nj+1,nk+1,3,nconct),
     +                 rhside(ni+1,nj+1,nk+1,3),
     +                 solut (1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,
     +                        1-nvirtual:nk+1+nvirtual,3)

c     iperiod   i    Parameter indicating if periodical boundary conditions
c                    are present per boundary
c                    iperiod is a 3 digit number with the sequence 321
c                    Each digit refers to 1 direction
c                    The digits have the following meaning:
c                    0:  no periodical boundary conditions
c                    1:  symmetrical periodical boundary conditions
c                    2:  anti-symmetrical periodical boundary conditions
c     matrix    i    reserved
c     mmctinv   i    Matrix code table inverse transport equation see
c                    array infmt
c     nconct    i    Number of connections in a row of the matrix
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     rhside    o    reserved
c     solut     i    reserved
c***********************************************************************
c
c                   COMMON BLOCKS
c
c
c ********************************************************************
c
c                    LOCAL PARAMETERS
c
      integer i, idegfd, imhi, imlo, ishi, ishift, islo,
     +        j, jdegfd, jmhi, jmlo, jshi, jshift, jslo,
     +        k,         kmhi, kmlo, kshi, kshift, kslo,
     +        l
      logical period1, period2, period3

c     i         General loop variable
c     idegfd    Loop variable with respect to a given unknown
c     imhi      Upper bound on i-index for matrix and rhside
c     imlo      Lower bound on i-index for matrix and rhside
c     ishi      Upper bound on i-index for solution vector
c     ishift    Shift in i-direction
c     islo      Lower bound on i-index for solution vector
c     j         General loop variable
c     jdegfd    Loop variable with respect to a given unknown
c     jmhi      Upper bound on j-index for matrix and rhside
c     jmlo      Lower bound on j-index for matrix and rhside
c     jshi      Upper bound on j-index for solution vector
c     jshift    Shift in j-direction
c     jslo      Lower bound on j-index for solution vector
c     k         General loop variable
c     kmhi      Upper bound on k-index for matrix and rhside
c     kmlo      Lower bound on k-index for matrix and rhside
c     kshi      Upper bound on k-index for solution vector
c     kshift    Shift in k-direction
c     kslo      Lower bound on k-index for solution vector
c     l         General loop variable
c     period1   If true there are periodical boundary conditions in the
c               1 direction
c     period2   If true there are periodical boundary conditions in the
c               2 direction
c     period3   If true there are periodical boundary conditions in the
c               3 direction
c ********************************************************************
c
c                         I/O
c
c ********************************************************************
c
c                      SUBROUTINES CALLED
c
c ********************************************************************
c
c                     ERROR MESSAGES
c
c ********************************************************************
c
c                     PSEUDO CODE
c
c     The matrix vector multiplication is defined by:  a   x  = f
c                                                       ij  j    i
c
c     Due to the periodical boundary conditions it is not possible to
c     perform the regular matrix-vector multiplication ISMTVC01. In 2D
c     we need to take special care of one outer layer of the domain per side,
c     in 3D general coordinates we need to take care of two outer layers per
c     side of the domain.
c     As a consequence the matrix-vector multiplication becomes less efficient
c     than for the standard multiplication defined in ismtvc01
c
c     Algorithm:
c
c     a. In case no periodical boundary conditions are present
c
c     For idegfd := 1 (1) 3 do
c         Compute imli, imhi, jmlo, jmhi, kmlo, kmhi
c         For i := imlo (1) imhi do
c            For j := jmlo (1) jmhi do
c               For k := kmlo (1) kmhi do
c                  f(i,j,k,idegfd) := a(i,j,k,idegfd,1)*x(i,j,k,idegfd)
c     For idegfd := 1 (1) 3 do
c        For l := 1, nconct do
c           jdegfd := mmctinv(0,idegfd,l)
c           ishift := mmctinv(1,idegfd,l)
c           jshift := mmctinv(2,idegfd,l)
c           kshift := mmctinv(3,idegfd,l)
c           Compute imli, imhi, jmlo, jmhi, kmlo, kmhi
c           For i := imli (1) imhi do
c              For j := jmlo (1) jmhi do
c                 For k := kmlo (1) kmhi do
c                    f(i,j,idegfd) := f(i,j,k,idegfd)+a(i,j,k,idegfd,k)*
c                                     x(i+ishift,j+jshift,k+kshift,jdegfd)
c
c ======================================================================
c
c     --- Detect type of periodicity

      period1 = .false.
      period2 = .false.
      period3 = .false.
      if ( mod(iperiod,10).eq.1 ) period1 = .true.
      if ( mod(iperiod/10,10).eq.1 ) period2 = .true.
      if ( mod(iperiod/100,10).eq.1 ) period3 = .true.

c     --- Multiply by diagonal
c
      do idegfd = 1, 3

         do k = 1, nk+1
            do j = 1, nj+1
               do i = 1, ni+1
                  rhside(i,j,k,idegfd) = matrix(i,j,k,idegfd,1) *
     +                                   solut (i,j,k,idegfd)
               end do
            end do
         end do

      end do

c     --- Multiply rest of matrix
c         Loop over the degrees of freedom

      do idegfd = 1, 3

         if ( idegfd.eq.1 ) then
            imhi = ni+1
            jmhi = nj
            kmhi = nk
         else if ( idegfd.eq.2 ) then
            imhi = ni
            jmhi = nj+1
            kmhi = nk
         else
            imhi = ni
            jmhi = nj
            kmhi = nk+1
         end if
         imlo = 1
         jmlo = 1
         kmlo = 1


c        --- Loop over the connections

         do l = 2, nconct

            jdegfd = mmctinv(0,idegfd,l)
            ishift = mmctinv(1,idegfd,l)
            jshift = mmctinv(2,idegfd,l)
            kshift = mmctinv(3,idegfd,l)
            if ( jdegfd.eq.1 ) then
               ishi = ni+1-ishift
               jshi = nj-jshift
               kshi = nk-kshift
            else if ( jdegfd.eq.2 ) then
               ishi = ni-ishift
               jshi = nj+1-jshift
               kshi = nk-kshift
            else
               ishi = ni-ishift
               jshi = nj-jshift
               kshi = nk+1-kshift
            end if
            islo = 1-ishift
            jslo = 1-jshift
            kslo = 1-kshift

            if ( period1 ) then
               islo = imlo
               ishi = imhi
            end if
            if ( period2 ) then
               jslo = jmlo
               jshi = jmhi
            end if
            if ( period3 ) then
               kslo = kmlo
               kshi = kmhi
            end if
c           --- Corrections for periodical boundary conditions

            do k = max(kmlo,kslo), min(kmhi,kshi)
               do j = max(jmlo,jslo), min(jmhi,jshi)
                  do i = max(imlo,islo), min(imhi,ishi)
                     rhside(i,j,k,idegfd) = rhside(i,j,k,idegfd) +
     +                                    matrix(i,j,k,idegfd,l) *
     +                                    solut(i+ishift,j+jshift,
     +                                          k+kshift,jdegfd  )
                  end do
               end do
            end do

         end do

      end do


      if ( period1 ) then

c     --- periodical boundary conditions in 1-direction
c         Copy left-hand boundary from right-hand boundary

         do k = 1, nk
            do j = 1, nj
               rhside(1,j,k,1) = .5d0*(rhside(ni+1,j,k,1)+
     +                                 rhside(1,j,k,1) )
               rhside(ni+1,j,k,1) = .5d0*(rhside(ni+1,j,k,1)+
     +                                 rhside(1,j,k,1) )
            end do
         end do

      end if

      if ( period2 ) then

c     --- periodical boundary conditions in 2-direction
c         Copy lower boundary from upper boundary

         do k = 1, nk
            do i = 1, ni
               rhside(i,1,k,2) =.5d0*( rhside(i,nj+1,k,2)+
     +                                 rhside(i,1,k,2) )
               rhside(i,nj+1,k,2) =.5d0*( rhside(i,nj+1,k,2)+
     +                                 rhside(i,1,k,2) )
            end do
         end do

      end if

      if ( period3 ) then

c     --- periodical boundary conditions in 3-direction
c         Copy lower boundary from upper boundary

         do j = 1, nj
            do i = 1, ni
               rhside(i,j,1,3) =.5d0*( rhside(i,j,nk+1,3)+
     +                                 rhside(i,j,1,3) )
               rhside(i,j,nk+1,3) =.5d0*( rhside(i,j,nk+1,3)+
     +                                 rhside(i,j,1,3) )
            end do
         end do

      end if

      end
