      subroutine isprefin ( nusol2, xresul, bresul, amat, infmat,
     +                      prec, initer )
c ======================================================================
c
c        programmer     Kees Vuik
c        version 1.3    date 07-07-1998 Kees V: nusol, nsolut cleaned
c        version 1.2    date 28-04-1995 Kees V replace tinvu by istinvu
c        version 1.1    date 16-02-1995 Kees V: 3D preconditioner added
c        version 1.0    date 11-11-1994
c
c
c   copyright (c) 1994-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Scales the solution if a postconditioning is used.
c
c **********************************************************************
c
c                       KEYWORDS
c
c     linear_solver
c     preconditioner
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer          nusol2, infmat(*), initer(*)
      double precision xresul(nusol2), bresul(nusol2), amat(nusol2,*),
     +                 prec(nusol2,*)

c     amat      i    The coefficients matrix used in the linear system
c     bresul    o    The result vector
c     infmat    i    Integer information for the matrix
c     initer    i    reserved
c     nusol2    i    Number of scalar unknowns for the solvers
c                    nusol2 = nusol * ndimso
c     prec      i    The preconditioning matrix
c     xresul    i    The original vector
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ndim, ndimso, nconct

c     nconct    Number of connections in a row of the matrix
c     ndim      Dimension of space (2 or 3)
c     ndimso    Integer indicating the dimension of the space in
c               which the problem must be solved (ndimso = 1 or ndim)
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     DCOPY     Copy double precision array into other one (BLAS subroutine)
c     ISDINVU   Computes the multiplication of the inverse of an upper
c               triangular matrix stored in arrays amat and prec
c               and a vector. Prec should be filled by isdmlu.f.
c     ISPREVC   Computes the product of the preconditioner, stored in
c               amat and prec.
c     ISTINVU   Computes the multiplication of the inverse of an upper
c               triangular matrix stored in arrays amat and prec
c               and a vector. Prec should be filled by istmlu.f.
c               This subroutine works for ndim = 3.
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     If the ILUD preconditioner is used then b = U^(-1) * x. If a
c     postconditioner is used b = U^(-1) * L^(-1) * x.
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c

      ndim   = initer(15)
      ndimso = initer(16)
      nconct = initer(18)

      if(initer(1).eq.2) then
         call dcopy ( nusol2, xresul, 1, bresul, 1 )
         if ( ndim.eq.2 ) then
            call isdinvu ( bresul, amat, nusol2/ndimso, ndimso,
     +                     nconct, initer )
         else
            call istinvu ( bresul, amat, nusol2/ndimso, ndimso,
     +                     nconct, infmat, initer )
         end if
      else
         call isprevc ( nusol2, xresul, bresul, amat,
     +                  prec, infmat, initer )
      end if

      end
