      subroutine ispremat ( nusol2, xresul, bresul, amat, infmat,
     +                      prec, work, solutv, initer, iinmtvc )
c ======================================================================
c
c        programmer     Kees Vuik
c        version 1.6    date 07-07-1998 Kees V: nusol, nsolut cleaned
c        version 1.5    date 20-10-1997 Kees K. added array iinmtvc
c        version 1.4    date 03-10-1995 Kees K. added array solutv
c        version 1.3    date 26-04-1995 Kees V replace tinvl and tinvu
c                                     by istinvl and istinvu
c        version 1.2    date 04-04-1995 Guus replace mtvc1 by ismatvec
c        version 1.1    date 16-02-1995 Kees V: 3D preconditioner added
c        version 1.0    date 11-11-1994
c
c
c   copyright (c) 1994-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Computes a matrix vector product combined with a pre- or postcon-
c     ditioning matrix.
c **********************************************************************
c
c                       KEYWORDS
c
c     linear_solver
c     preconditioner
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer          nusol2, infmat(*), initer(*), iinmtvc(*)
      double precision xresul(nusol2), bresul(nusol2), amat(nusol2,*),
     +                 prec(nusol2,*), work(nusol2), solutv(*)

c     amat      i    The coefficients matrix used in the linear system
c     bresul    o    The result vector
c     iinmtvc   i    reserved
c     infmat    i    Integer information for the matrix
c     initer    i    reserved
c     nusol2    i    Number of scalar unknowns for the solvers
c                    nusol2 = nusol * ndimso
c     prec      i    The preconditioning matrix
c     solutv    i    Work array for solut with virtual points
c     work      i    Work array
c     xresul    i    The original vector
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, ndim, ndimso, nconct

c     i         General loop variable
c     nconct    Number of connections in a row of the matrix
c     ndim      Dimension of space (2 or 3)
c     ndimso    Integer indicating the dimension of the space in
c               which the problem must be solved (ndimso = 1 or ndim)
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     DAXPY     Computes a vector + a constant * other vector. This
c               is a BLAS routine.
c     DCOPY     Copy double precision array into other one (BLAS subroutine)
c     ISDINVL   Computes the multiplication of the inverse of a lower
c               triangular matrix stored in arrays amat and prec
c               and a vector. Prec should be filled by isdmlu.f.
c     ISDINVU   Computes the multiplication of the inverse of an upper
c               triangular matrix stored in arrays amat and prec
c               and a vector. Prec should be filled by isdmlu.f.
c     ISMATVEC  Computes the product of the sparse matrix, stored in amat,
c               and a vector.
c     ISPREVC   Computes the product of the preconditioner, stored in
c               amat and prec.
c     ISTINVL   Computes the multiplication of the inverse of a lower
c               triangular matrix stored in arrays amat and prec
c               and a vector. Prec should be filled by istmlu.f.
c               This subroutine works for ndim = 3.
c     ISTINVU   Computes the multiplication of the inverse of an upper
c               triangular matrix stored in arrays amat and prec
c               and a vector. Prec should be filled by istmlu.f.
c               This subroutine works for ndim = 3.
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     If no preconditioning is used then b = A*x. If the ILUD precondi-
c     tioner is used, we calculate the preconditioned matrix vector
c     product by the Eisenstat trick.
c
c     Literature
c
c     S. Eisenstat,
c     Efficient implementation of a class of preconditioned conjugate
c     gradient methods,
c     SIAM J. Sci. Statist. Comput. 2 pp.1-4 (1981).
c
c     H.A. van der Vorst,
c     Iterative solution methods for certain sparse linear systems with
c     a non-symmetric matrix arising from PDE-problems,
c     J. Comput. Phys. 44 pp.1-19 (1981).
c
c     Finally in the other cases the preconditioned matrix vector pro-
c     duct is calculated as b = P*A*x, and the postconditioned matrix
c     vector product is calculated as b = A*P*x.
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c

      ndim   = initer(15)
      ndimso = initer(16)
      nconct = initer(18)

      if(initer(1).eq.2) then
         call dcopy ( nusol2, xresul, 1, bresul, 1 )
         if ( ndim.eq.2 ) then
            call isdinvu ( bresul, amat, nusol2/ndimso, ndimso,
     +                     nconct, initer )
         else
            call istinvu ( bresul, amat, nusol2/ndimso, ndimso,
     +                  nconct, infmat, initer )
         end if
         do i = 1, nusol2
            work(i) = amat(i,1)*bresul(i)+xresul(i)
         end do
         if ( ndim.eq.2 ) then
            call isdinvl ( work, amat, nusol2/ndimso, ndimso,
     +                     nconct, initer )
         else
            call istinvl ( work, amat, nusol2/ndimso, ndimso,
     +                     nconct, infmat, initer )
         end if
         call daxpy ( nusol2, 1d0, work, 1 ,bresul, 1 )
      else
         if(initer(1).gt.0) then

            call ismatvec ( amat, xresul, solutv, work, infmat, initer,
     +                      iinmtvc)

            call isprevc ( nusol2, work, bresul, amat,
     +                     prec, infmat, initer )
         else
            call isprevc ( nusol2, xresul, work, amat,
     +                     prec, infmat, initer )

            call ismatvec ( amat, work, solutv, bresul, infmat, initer,
     +                      iinmtvc)

         end if
      end if
      end
