      subroutine istmlu3 ( amat, nusol, ndimso, nconct, prec, nprec,
     +                     initer, reiter, work )
c ======================================================================
c
c        programmer     Kees Vuik
c        version 1.3    date 07-07-1998 Kees V: nusol, nsolut cleaned
c        version 1.2    date 02-04-1997 Kees K. literal constants in
c                                     double precision for f90
c        version 1.1    date 23-02-1996 Kees V. default for IBLU changed
c        version 1.0    date 11-08-1995 extracted from old ISNaS
c                       subroutine tmlu2.
c
c   copyright (c) 1994-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Calculates a diagonal matrix D to obtain an incomplete decomposi-
c     tion for the matrix A.
c **********************************************************************
c
c                       KEYWORDS
c
c      linear_solver
c      preconditioner
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer          initer(*),nusol,ndimso,nconct,nprec
      double precision amat(1:nusol,1:ndimso,1:nconct),
     +                 prec(1:nusol,1:ndimso,1:nprec), reiter(*),
     +                 work(1:nusol,1:2)

c     amat      i    The coefficients matrix used in the linear system
c     initer    i    reserved
c     nconct    i    Number of connections in a row of the matrix
c     ndimso    i    Integer indicating the dimension of the space in
c                    which the problem must be solved (ndimso = 1 or ndim)
c     nprec     i    Maximum number of diagonals in preconditioning matrix
c     nusol     i    Number of unknowns for one degree of freedom in
c                    one block
c     prec      i    The preconditioning matrix
c     reiter    i    reserved
c     work      i    Work array
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer          i,j,nx,ny,npxy
      double precision amod, thulp

c     amod      Parameter used in the average of ILU and MILU
c     i         General loop variable
c     j         General loop variable
c     npxy      Product of nx and ny
c     nx        Number of grid x points on finest grid
c     ny        Number of grid y points on finest grid
c     thulp     Auxiliary variable.
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     This subroutine calculates a diagonal matrix D to obtain an in-
c     complete decomposition, for the matrix A using the following
c     rules defined in:
c
c     H.A. van der Vorst
c     Preconditioning by Incomplete Decompositions (p. 65)
c     Thesis, Utrecht State University (1982)
c
c                   -1
c            A = L D  U - R,
c
c     (a) diag(L) = diag(U) = D,
c
c     (b) The off-diagonal parts of L and U are equal to the cor-
c         responding parts of A,
c
c                 -1
c     (c) diag(L D  U) = diag(A).
c
c     This subroutine is a combination of the above mentioned procedure
c     and the MILU decomposition as described in:
c
c     I. Gustafsson
c     A Class of First Order Factorization Methods
c     BIT 18 (1978) 142-156
c
c     The contributions on the diagonal are weighted by the scalar
c     amod, amod = 0 corresponds with ILU, whereas amod = 1 corresponds
c     with MILU.
c
c     The matrix inverse(D) is stored in prec(1:n,1:ndimso,2)
c
c     If this is used a preconditioner the matrix is scaled.
c     Scale the system Ax = b with D. This leads to
c
c                 -1      -1
c                D  Ax = D  b,
c
c     and
c                 -1      -1  -1     -1
c                D  A  = D  LD  U - D  R.
c
c     From (a), (b), and (c) it follows that:
c
c                -1           -1
c     (a') diag(D  L) = diag(D  U) = I,
c
c                                     -1       -1
c     (b') The off-diagonal parts of D  L and D  U are equal to the cor-
c                               -1
c          responding parts of D  A,
c
c                -1   -1           -1                              -1
c     (c') diag(D  L D  U) = diag(D  A), because r   = 0, implies D  r = 0.
c                                                 ii                  ii
c
c     With respect of the MILU-preconditioning we note that
c             -1             -1  -1
c     rowsum(D  A) = rowsum(D  LD  U), because rowsum(R) = 0 implies
c             -1
c     rowsum(D  R) = 0.
c
c     The actual scaling of the matrix is done in the subroutine
c     issolv.f by a call to isscalma.f
c
c
c     Remark
c
c     - It is known that in the virtual cells the corresponding
c       row and column in the matrix consists of zero elements. So it is
c       possible that d(i) = 0. To circumvent this possibility we imple-
c       ment the following check:
c          if(a(i,1).eq.0d0) then prec(i,1) = 1 else ...
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- Filling of the parameter amod

      amod    = 0d0
      if ( reiter(7).eq.-1d0 ) then
         if ( initer(11).eq.3 ) then
            if ( initer(21).eq.0 ) then
               amod = 1d0
            else
               amod = 0.975d0
            end if
         end if
         if ( initer(11).ge.4 ) amod = 1
         if ( initer(9).eq.6 )  amod = 0.9d0
         reiter(7) = amod
      else
         amod = reiter(7)
      end if

      nx   = initer(12)+1
      ny   = initer(13)+1
      npxy = nx*ny

c     --- Initialization

      do i = 1, nusol
         work(i,1) = 0d0
      end do

      do j = 1, nconct
         do i = 1, nusol
            work(i,1) = work(i,1)+amat(i,1,j)
         end do
      end do

c     --- Compute the incomplete decomposition

      do i = 1, npxy+nx

         prec(i,1,1) = amat(i,1,1)
         prec(i,1,6) = amat(i,1,6)
         prec(i,1,7) = amat(i,1,7)
         prec(i,1,8) = amat(i,1,8)
         prec(i,1,9) = amat(i,1,9)
         prec(i,1,15) = amat(i,1,15)
         prec(i,1,16) = amat(i,1,16)
         prec(i,1,17) = amat(i,1,17)
         prec(i,1,18) = amat(i,1,18)
         prec(i,1,19) = amat(i,1,19)
         prec(i+1   ,1,5) = amat(i+1,1,5)
         prec(i+nx-1,1,4) = amat(i+nx-1,1,4)
         prec(i+nx  ,1,3) = amat(i+nx  ,1,3)
         prec(i+nx+1,1,2) = amat(i+nx+1,1,2)
         prec(i+npxy-nx,1,14) = amat(i+npxy-nx,1,14)
         prec(i+npxy-1 ,1,13) = amat(i+npxy-1 ,1,13)
         prec(i+npxy   ,1,12) = amat(i+npxy   ,1,12)
         prec(i+npxy+1 ,1,11) = amat(i+npxy+1 ,1,11)
         prec(i+npxy+nx,1,10) = amat(i+npxy+nx,1,10)

         if ( i.gt.1 ) then

c        --- The matrix U

            prec(i,1,1) = prec(i,1,1)
     +                    -prec(i,1,5)*prec(i-1   ,1,6)
            prec(i,1,7) = prec(i,1,7)
     +                    -prec(i,1,5)*prec(i-1   ,1,8)
            prec(i,1,8) = prec(i,1,8)
     +                    -prec(i,1,5)*prec(i-1   ,1,9)
            prec(i,1,16) = prec(i,1,16)
     +                     -prec(i,1,5)*prec(i-1   ,1,17)
            prec(i,1,17) = prec(i,1,17)
     +                     -prec(i,1,5)*prec(i-1   ,1,18)

c           --- The matrix L

            prec(i+nx-1,1,4) = prec(i+nx-1,1,4)
     +                         -prec(i+nx-1,1,3 )*prec(i-1,1,6)
            prec(i+nx  ,1,3) = prec(i+nx  ,1,3)
     +                         -prec(i+nx  ,1,2 )*prec(i-1,1,6)
            prec(i+npxy-1,1,13) = prec(i+npxy-1,1,13)
     +                      -prec(i+npxy-1,1,12)*prec(i-1   ,1,6)
            prec(i+npxy,1,12) = prec(i+npxy,1,12)
     +                      -prec(i+npxy,1,11)*prec(i-1   ,1,6)

         end if

         if ( i.gt.nx-1 ) then

c        --- The matrix U

            prec(i,1,1) = prec(i,1,1)
     +                    -prec(i,1,4)*prec(i-nx+1,1,7)
            prec(i,1,6) = prec(i,1,6)
     +                    -prec(i,1,4)*prec(i-nx+1,1,8)
            prec(i,1,15) = prec(i,1,15)
     +                     -prec(i,1,4)*prec(i-nx+1,1,16)
            prec(i,1,18) = prec(i,1,18)
     +                     -prec(i,1,4)*prec(i-nx+1,1,19)

c           --- The matrix L

            prec(i+1   ,1,5) = prec(i+1   ,1,5)
     +                         -prec(i+1,1,3)*prec(i-nx+1,1,7)
            prec(i+npxy-nx,1,14) = prec(i+npxy-nx,1,14)
     +                      -prec(i+npxy-nx,1,13)*prec(i-nx+1,1,7)
            prec(i+npxy+1,1,11) = prec(i+npxy+1,1,11)
     +                      -prec(i+npxy+1,1,10)*prec(i-nx+1,1,7)

         end if

         if ( i.gt.nx ) then

c        --- The matrix U

            prec(i,1,1) = prec(i,1,1)
     +                    -prec(i,1,3)*prec(i-nx  ,1,8)
            prec(i,1,6) = prec(i,1,6)
     +                    -prec(i,1,3)*prec(i-nx  ,1,9)
            prec(i,1,15) = prec(i,1,15)
     +                     -prec(i,1,3)*prec(i-nx  ,1,17)
            prec(i,1,17) = prec(i,1,17)
     +                     -prec(i,1,3)*prec(i-nx  ,1,19)

c           --- The matrix L

            prec(i+1   ,1,5) = prec(i+1   ,1,5)
     +                         -prec(i+1,1,2)*prec(i-nx  ,1,8)
            prec(i+npxy-nx,1,14) = prec(i+npxy-nx,1,14)
     +                      -prec(i+npxy-nx,1,12)*prec(i-nx  ,1,8)
            prec(i+npxy,1,12) = prec(i+npxy,1,12)
     +                      -prec(i+npxy,1,10)*prec(i-nx  ,1,8)

         end if

         if ( i.gt.nx+1 ) then

c        --- The matrix U

            prec(i,1,1) = prec(i,1,1)
     +                    -prec(i,1,2)*prec(i-nx-1,1,9)
            prec(i,1,15) = prec(i,1,15)
     +                     -prec(i,1,2)*prec(i-nx-1,1,18)
            prec(i,1,16) = prec(i,1,16)
     +                     -prec(i,1,2)*prec(i-nx-1,1,19)

c           --- The matrix L

            prec(i+npxy-nx,1,14) = prec(i+npxy-nx,1,14)
     +                      -prec(i+npxy-nx,1,11)*prec(i-nx-1,1,9)
            prec(i+npxy-1,1,13) = prec(i+npxy-1,1,13)
     +                      -prec(i+npxy-1,1,10)*prec(i-nx-1,1,9)

         end if

         if ( i.gt.npxy-nx ) then

c        --- The matrix U

            prec(i,1,1) = prec(i,1,1)
     +                    -prec(i,1,14)*prec(i-npxy+nx,1,15)
            prec(i,1,7) = prec(i,1,7)
     +                    -prec(i,1,14)*prec(i-npxy+nx,1,16)
            prec(i,1,8) = prec(i,1,8)
     +                    -prec(i,1,14)*prec(i-npxy+nx,1,17)
            prec(i,1,9) = prec(i,1,9)
     +                    -prec(i,1,14)*prec(i-npxy+nx,1,18)

c           --- The matrix L

            prec(i+nx-1,1,4) = prec(i+nx-1,1,4)
     +                         -prec(i+nx-1,1,13)*prec(i-npxy+nx,1,15)
            prec(i+nx  ,1,3) = prec(i+nx  ,1,3)
     +                         -prec(i+nx  ,1,12)*prec(i-npxy+nx,1,15)
            prec(i+nx+1,1,2) = prec(i+nx+1,1,2)
     +                         -prec(i+nx+1,1,11)*prec(i-npxy+nx,1,15)

         end if

         if ( i.gt.npxy-1 ) then

c        --- The matrix U

            prec(i,1,1) = prec(i,1,1)
     +                    -prec(i,1,13)*prec(i-npxy+1 ,1,16)
            prec(i,1,6) = prec(i,1,6)
     +                    -prec(i,1,13)*prec(i-npxy+1 ,1,17)
            prec(i,1,9) = prec(i,1,9)
     +                    -prec(i,1,13)*prec(i-npxy+1 ,1,19)

c           --- The matrix L

            prec(i+1   ,1,5) = prec(i+1   ,1,5)
     +                         -prec(i+1,1,12)*prec(i-npxy+1,1,16)
            prec(i+nx+1,1,2) = prec(i+nx+1,1,2)
     +                         -prec(i+nx+1,1,10)*prec(i-npxy+1 ,1,16)

         end if

         if ( i.gt.npxy ) then

c        --- The matrix U

            prec(i,1,1) = prec(i,1,1)
     +                    -prec(i,1,12)*prec(i-npxy   ,1,17)
            prec(i,1,6) = prec(i,1,6)
     +                    -prec(i,1,12)*prec(i-npxy   ,1,18)
            prec(i,1,8) = prec(i,1,8)
     +                    -prec(i,1,12)*prec(i-npxy   ,1,19)

c           --- The matrix L

            prec(i+1   ,1,5) = prec(i+1   ,1,5)
     +                         -prec(i+1,1,11)*prec(i-npxy  ,1,17)
            prec(i+nx  ,1,3) = prec(i+nx  ,1,3)
     +                         -prec(i+nx  ,1,10)*prec(i-npxy   ,1,17)

         end if

         if ( i.gt.npxy+1 ) then

c        --- The matrix U

            prec(i,1,1) = prec(i,1,1)
     +                    -prec(i,1,11)*prec(i-npxy-1 ,1,18)
            prec(i,1,7) = prec(i,1,7)
     +                    -prec(i,1,11)*prec(i-npxy-1 ,1,19)

c           --- The matrix L

            prec(i+nx-1,1,4) = prec(i+nx-1,1,4)
     +                         -prec(i+nx-1,1,10)*prec(i-npxy-1 ,1,18)

         end if

c        --- The MILU approach

         work(i,2) = prec(i,1,6)+prec(i,1,7)+prec(i,1,8)+
     +               prec(i,1,9)+prec(i,1,15)+prec(i,1,16)+
     +               prec(i,1,17)+prec(i,1,18)+prec(i,1,19)

         thulp = work(i,1)-work(i,2)

         if ( i.gt.1 )
     +      thulp = thulp-prec(i,1,5)*work(i-1   ,2)

         if ( i.gt.nx-1 )
     +      thulp = thulp-prec(i,1,4)*work(i-nx+1,2)

         if ( i.gt.nx )
     +      thulp = thulp-prec(i,1,3)*work(i-nx  ,2)

         if ( i.gt.nx+1 )
     +      thulp = thulp-prec(i,1,2)*work(i-nx-1,2)

         if ( i.gt.npxy-nx )
     +      thulp = thulp-prec(i,1,14)*work(i-npxy+nx,2)

         if ( i.gt.npxy-1 )
     +      thulp = thulp-prec(i,1,13)*work(i-npxy+1 ,2)

         if ( i.gt.npxy )
     +      thulp = thulp-prec(i,1,12)*work(i-npxy   ,2)

         if ( i.gt.npxy+1 )
     +      thulp = thulp-prec(i,1,11)*work(i-npxy-1 ,2)

         prec(i,1,1) = (1d0-amod)*prec(i,1,1)+amod*thulp
         work(i,2)   = work(i,2)+prec(i,1,1)

         if ( prec(i,1,1).eq.0d0 ) then
            prec(i,1,1) = 1d0
         else
            prec(i,1,1) = 1d0/prec(i,1,1)
         end if

         prec(i+1   ,1,5) = prec(i+1,1,5)*prec(i,1,1)
         prec(i+nx-1,1,4) = prec(i+nx-1,1,4)*prec(i,1,1)
         prec(i+nx  ,1,3) = prec(i+nx  ,1,3)*prec(i,1,1)
         prec(i+nx+1,1,2) = prec(i+nx+1,1,2)*prec(i,1,1)
         prec(i+npxy-nx,1,14) = prec(i+npxy-nx,1,14)*prec(i,1,1)
         prec(i+npxy-1 ,1,13) = prec(i+npxy-1 ,1,13)*prec(i,1,1)
         prec(i+npxy   ,1,12) = prec(i+npxy   ,1,12)*prec(i,1,1)
         prec(i+npxy+1 ,1,11) = prec(i+npxy+1 ,1,11)*prec(i,1,1)
         prec(i+npxy+nx,1,10) = prec(i+npxy+nx,1,10)*prec(i,1,1)
      end do

c     --- i = npxy+nx+1, nusol-npxy-nx

      do i = npxy+nx+1, nusol-npxy-nx

c     --- The matrix U

         if ( amat(i,1,1).eq.0d0 ) then
            prec(i,1,1) = 1d0
         else
            prec(i,1,1) = amat(i,1,1)
     +                    -prec(i,1,2)*prec(i-nx-1,1,9)
     +                    -prec(i,1,3)*prec(i-nx  ,1,8)
     +                    -prec(i,1,4)*prec(i-nx+1,1,7)
     +                    -prec(i,1,5)*prec(i-1   ,1,6)
     +                    -prec(i,1,10)*prec(i-npxy-nx,1,19)
     +                    -prec(i,1,11)*prec(i-npxy-1 ,1,18)
     +                    -prec(i,1,12)*prec(i-npxy   ,1,17)
     +                    -prec(i,1,13)*prec(i-npxy+1 ,1,16)
     +                    -prec(i,1,14)*prec(i-npxy+nx,1,15)

         end if
         prec(i,1,6) = amat(i,1,6)
     +                 -prec(i,1,3)*prec(i-nx  ,1,9)
     +                 -prec(i,1,4)*prec(i-nx+1,1,8)
     +                 -prec(i,1,12)*prec(i-npxy   ,1,18)
     +                 -prec(i,1,13)*prec(i-npxy+1 ,1,17)
         prec(i,1,7) = amat(i,1,7)
     +                 -prec(i,1,5)*prec(i-1   ,1,8)
     +                 -prec(i,1,11)*prec(i-npxy-1 ,1,19)
     +                 -prec(i,1,14)*prec(i-npxy+nx,1,16)
         prec(i,1,8) = amat(i,1,8)
     +                 -prec(i,1,5)*prec(i-1   ,1,9)
     +                 -prec(i,1,12)*prec(i-npxy   ,1,19)
     +                 -prec(i,1,14)*prec(i-npxy+nx,1,17)
         prec(i,1,9) = amat(i,1,9)
     +                 -prec(i,1,13)*prec(i-npxy+1 ,1,19)
     +                 -prec(i,1,14)*prec(i-npxy+nx,1,18)
         prec(i,1,15) = amat(i,1,15)
     +                  -prec(i,1,2)*prec(i-nx-1,1,18)
     +                  -prec(i,1,3)*prec(i-nx  ,1,17)
     +                  -prec(i,1,4)*prec(i-nx+1,1,16)
         prec(i,1,16) = amat(i,1,16)
     +                  -prec(i,1,2)*prec(i-nx-1,1,19)
     +                  -prec(i,1,5)*prec(i-1   ,1,17)
         prec(i,1,17) = amat(i,1,17)
     +                  -prec(i,1,3)*prec(i-nx  ,1,19)
     +                  -prec(i,1,5)*prec(i-1   ,1,18)
         prec(i,1,18) = amat(i,1,18)
     +                  -prec(i,1,4)*prec(i-nx+1,1,19)
         prec(i,1,19) = amat(i,1,19)

c        --- The MILU approach

         work(i,2) = prec(i,1,6)+prec(i,1,7)+prec(i,1,8)+
     +               prec(i,1,9)+prec(i,1,15)+prec(i,1,16)+
     +               prec(i,1,17)+prec(i,1,18)+prec(i,1,19)

         thulp = work(i,1)-work(i,2)
         thulp = thulp-prec(i,1,2)*work(i-nx-1,2)
     +                -prec(i,1,3)*work(i-nx  ,2)
     +                -prec(i,1,4)*work(i-nx+1,2)
     +                -prec(i,1,5)*work(i-1   ,2)
     +                -prec(i,1,10)*work(i-npxy-nx,2)
     +                -prec(i,1,11)*work(i-npxy-1 ,2)
     +                -prec(i,1,12)*work(i-npxy   ,2)
     +                -prec(i,1,13)*work(i-npxy+1 ,2)
     +                -prec(i,1,14)*work(i-npxy+nx,2)

         prec(i,1,1) = (1d0-amod)*prec(i,1,1)+amod*thulp
         work(i,2)   = work(i,2)+prec(i,1,1)

         if ( prec(i,1,1).eq.0d0 ) then
            prec(i,1,1) = 1d0
         else
            prec(i,1,1) = 1d0/prec(i,1,1)
         end if

c        --- The matrix L

         prec(i+1   ,1,5) = (amat(i+1,1,5)
     +                      -prec(i+1,1,2)*prec(i-nx  ,1,8)
     +                      -prec(i+1,1,3)*prec(i-nx+1,1,7)
     +                      -prec(i+1,1,11)*prec(i-npxy  ,1,17)
     +                      -prec(i+1,1,12)*prec(i-npxy+1,1,16)
     +                      )*prec(i,1,1)
         prec(i+nx-1,1,4) = (amat(i+nx-1,1,4)
     +                      -prec(i+nx-1,1,3 )*prec(i-1,1,6)
     +                      -prec(i+nx-1,1,10)*prec(i-npxy-1 ,1,18)
     +                      -prec(i+nx-1,1,13)*prec(i-npxy+nx,1,15)
     +                      )*prec(i,1,1)
         prec(i+nx  ,1,3) = (amat(i+nx  ,1,3)
     +                      -prec(i+nx  ,1,2 )*prec(i-1,1,6)
     +                      -prec(i+nx  ,1,10)*prec(i-npxy   ,1,17)
     +                      -prec(i+nx  ,1,12)*prec(i-npxy+nx,1,15)
     +                      )*prec(i,1,1)
         prec(i+nx+1,1,2) = (amat(i+nx+1,1,2)
     +                      -prec(i+nx+1,1,10)*prec(i-npxy+1 ,1,16)
     +                      -prec(i+nx+1,1,11)*prec(i-npxy+nx,1,15)
     +                      )*prec(i,1,1)
         prec(i+npxy-nx,1,14) = (amat(i+npxy-nx,1,14)
     +                      -prec(i+npxy-nx,1,11)*prec(i-nx-1,1,9)
     +                      -prec(i+npxy-nx,1,12)*prec(i-nx  ,1,8)
     +                      -prec(i+npxy-nx,1,13)*prec(i-nx+1,1,7)
     +                      )*prec(i,1,1)
         prec(i+npxy-1,1,13) = (amat(i+npxy-1,1,13)
     +                      -prec(i+npxy-1,1,10)*prec(i-nx-1,1,9)
     +                      -prec(i+npxy-1,1,12)*prec(i-1   ,1,6)
     +                      )*prec(i,1,1)
         prec(i+npxy,1,12) = (amat(i+npxy,1,12)
     +                      -prec(i+npxy,1,10)*prec(i-nx  ,1,8)
     +                      -prec(i+npxy,1,11)*prec(i-1   ,1,6)
     +                      )*prec(i,1,1)
         prec(i+npxy+1,1,11) = (amat(i+npxy+1,1,11)
     +                      -prec(i+npxy+1,1,10)*prec(i-nx+1,1,7)
     +                      )*prec(i,1,1)
         prec(i+npxy+nx,1,10) = amat(i+npxy+nx,1,10)*prec(i,1,1)
      end do

c     --- i = nusol-npxy-nx+1, nusol

      do i = nusol-npxy-nx+1, nusol

c     --- The matrix U

         work(i,2) = 0d0

         if ( amat(i,1,1).eq.0d0 ) then
            prec(i,1,1) = 1d0
         else
            prec(i,1,1) = amat(i,1,1)
     +                    -prec(i,1,2)*prec(i-nx-1,1,9)
     +                    -prec(i,1,3)*prec(i-nx  ,1,8)
     +                    -prec(i,1,4)*prec(i-nx+1,1,7)
     +                    -prec(i,1,5)*prec(i-1   ,1,6)
     +                    -prec(i,1,10)*prec(i-npxy-nx,1,19)
     +                    -prec(i,1,11)*prec(i-npxy-1 ,1,18)
     +                    -prec(i,1,12)*prec(i-npxy   ,1,17)
     +                    -prec(i,1,13)*prec(i-npxy+1 ,1,16)
     +                    -prec(i,1,14)*prec(i-npxy+nx,1,15)
         end if

         if ( i.le.nusol-1 ) then
            prec(i,1,6) = amat(i,1,6)
     +                    -prec(i,1,3)*prec(i-nx  ,1,9)
     +                    -prec(i,1,4)*prec(i-nx+1,1,8)
     +                    -prec(i,1,12)*prec(i-npxy   ,1,18)
     +                    -prec(i,1,13)*prec(i-npxy+1 ,1,17)
            work(i,2) = work(i,2) + prec(i,1,6)
         end if

         if ( i.le.nusol-nx+1 ) then
            prec(i,1,7) = amat(i,1,7)
     +                    -prec(i,1,5)*prec(i-1   ,1,8)
     +                    -prec(i,1,11)*prec(i-npxy-1 ,1,19)
     +                    -prec(i,1,14)*prec(i-npxy+nx,1,16)
            work(i,2) = work(i,2) + prec(i,1,7)
         end if

         if ( i.le.nusol-nx ) then
            prec(i,1,8) = amat(i,1,8)
     +                    -prec(i,1,5)*prec(i-1   ,1,9)
     +                    -prec(i,1,12)*prec(i-npxy   ,1,19)
     +                    -prec(i,1,14)*prec(i-npxy+nx,1,17)
            work(i,2) = work(i,2) + prec(i,1,8)
         end if

         if ( i.le.nusol-nx-1 ) then
            prec(i,1,9) = amat(i,1,9)
     +                    -prec(i,1,13)*prec(i-npxy+1 ,1,19)
     +                    -prec(i,1,14)*prec(i-npxy+nx,1,18)
            work(i,2) = work(i,2) + prec(i,1,9)
         end if

         if ( i.le.nusol-npxy+nx ) then
            prec(i,1,15) = amat(i,1,15)
     +                     -prec(i,1,2)*prec(i-nx-1,1,18)
     +                     -prec(i,1,3)*prec(i-nx  ,1,17)
     +                     -prec(i,1,4)*prec(i-nx+1,1,16)
            work(i,2) = work(i,2) + prec(i,1,15)
         end if

         if ( i.le.nusol-npxy+1 ) then
            prec(i,1,16) = amat(i,1,16)
     +                     -prec(i,1,2)*prec(i-nx-1,1,19)
     +                     -prec(i,1,5)*prec(i-1   ,1,17)
            work(i,2) = work(i,2) + prec(i,1,16)
         end if

         if ( i.le.nusol-npxy ) then
            prec(i,1,17) = amat(i,1,17)
     +                     -prec(i,1,3)*prec(i-nx  ,1,19)
     +                     -prec(i,1,5)*prec(i-1   ,1,18)
            work(i,2) = work(i,2) + prec(i,1,17)
         end if

         if ( i.le.nusol-npxy-1 ) then
            prec(i,1,18) = amat(i,1,18)
     +                     -prec(i,1,4)*prec(i-nx+1,1,19)
            work(i,2) = work(i,2) + prec(i,1,18)
         end if

c        --- The MILU approach

         thulp = work(i,1)-work(i,2)
         thulp = thulp-prec(i,1,2)*work(i-nx-1,2)
     +                -prec(i,1,3)*work(i-nx  ,2)
     +                -prec(i,1,4)*work(i-nx+1,2)
     +                -prec(i,1,5)*work(i-1   ,2)
     +                -prec(i,1,10)*work(i-npxy-nx,2)
     +                -prec(i,1,11)*work(i-npxy-1 ,2)
     +                -prec(i,1,12)*work(i-npxy   ,2)
     +                -prec(i,1,13)*work(i-npxy+1 ,2)
     +                -prec(i,1,14)*work(i-npxy+nx,2)

         prec(i,1,1) = (1d0-amod)*prec(i,1,1)+amod*thulp
         work(i,2)   = work(i,2)+prec(i,1,1)

         if ( prec(i,1,1).eq.0d0 ) then
            prec(i,1,1) = 1d0
         else
            prec(i,1,1) = 1d0/prec(i,1,1)
         end if

c        --- The matrix L

         if ( i.le.nusol-1 )
     +   prec(i+1   ,1,5) = (amat(i+1,1,5)
     +                      -prec(i+1,1,2)*prec(i-nx  ,1,8)
     +                      -prec(i+1,1,3)*prec(i-nx+1,1,7)
     +                      -prec(i+1,1,11)*prec(i-npxy  ,1,17)
     +                      -prec(i+1,1,12)*prec(i-npxy+1,1,16)
     +                      )*prec(i,1,1)
         if ( i.le.nusol-nx+1 )
     +    prec(i+nx-1,1,4) = (amat(i+nx-1,1,4)
     +                      -prec(i+nx-1,1,3 )*prec(i-1,1,6)
     +                      -prec(i+nx-1,1,10)*prec(i-npxy-1 ,1,18)
     +                      -prec(i+nx-1,1,13)*prec(i-npxy+nx,1,15)
     +                      )*prec(i,1,1)
         if ( i.le.nusol-nx )
     +   prec(i+nx  ,1,3) = (amat(i+nx  ,1,3)
     +                      -prec(i+nx  ,1,2 )*prec(i-1,1,6)
     +                      -prec(i+nx  ,1,10)*prec(i-npxy   ,1,17)
     +                      -prec(i+nx  ,1,12)*prec(i-npxy+nx,1,15)
     +                      )*prec(i,1,1)
         if ( i.le.nusol-nx-1 )
     +   prec(i+nx+1,1,2) = (amat(i+nx+1,1,2)
     +                      -prec(i+nx+1,1,10)*prec(i-npxy+1 ,1,16)
     +                      -prec(i+nx+1,1,11)*prec(i-npxy+nx,1,15)
     +                      )*prec(i,1,1)
         if ( i.le.nusol-npxy+nx )
     +   prec(i+npxy-nx,1,14) = (amat(i+npxy-nx,1,14)
     +                      -prec(i+npxy-nx,1,11)*prec(i-nx-1,1,9)
     +                      -prec(i+npxy-nx,1,12)*prec(i-nx  ,1,8)
     +                      -prec(i+npxy-nx,1,13)*prec(i-nx+1,1,7)
     +                      )*prec(i,1,1)
         if ( i.le.nusol-npxy+1 )
     +   prec(i+npxy-1,1,13) = (amat(i+npxy-1,1,13)
     +                      -prec(i+npxy-1,1,10)*prec(i-nx-1,1,9)
     +                      -prec(i+npxy-1,1,12)*prec(i-1   ,1,6)
     +                      )*prec(i,1,1)
         if ( i.le.nusol-npxy )
     +   prec(i+npxy,1,12) = (amat(i+npxy,1,12)
     +                      -prec(i+npxy,1,10)*prec(i-nx  ,1,8)
     +                      -prec(i+npxy,1,11)*prec(i-1   ,1,6)
     +                      )*prec(i,1,1)
         if ( i.le.nusol-npxy-1 )
     +   prec(i+npxy+1,1,11) = (amat(i+npxy+1,1,11)
     +                      -prec(i+npxy+1,1,10)*prec(i-nx+1,1,7)
     +                      )*prec(i,1,1)
      end do

      end
