      subroutine isvpper3 ( solut, ni, nj, nk, nvirtual,iperiod )
c ======================================================================
c
c
c        programmer     Kees Kassels
c        version 1.0    date   05-10-95
c
c
c   copyright (c) 1995-1995  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c ********************************************************************
c
c                         DESCRIPTION
c
c     Fills virtual periodical pressure, transport unknowns 
c     in solution array
c
c ********************************************************************
c
c                      KEYWORDS
c
c ********************************************************************
c
c           INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nk, nvirtual, iperiod
      double precision solut(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1-nvirtual:nk+1+nvirtual)

c     iperiod   i    Parameter indicating if periodical boundary conditions
c                    are present per boundary
c                    iperiod is a 3 digit number with the sequence 321
c                    Each digit refers to 1 direction
c                    The digits have the following meaning:
c                    0:  no periodical boundary conditions
c                    1:  symmetrical periodical boundary conditions
c                    2:  anti-symmetrical periodical boundary conditions
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     solut     i    reserved
c***********************************************************************
c
c                   COMMON BLOCKS
c
c
c ********************************************************************
c
c                    LOCAL PARAMETERS
c
      integer i,  j, k
      logical period1, period2, period3

c     i         General loop variable
c     j         General loop variable
c     k         General loop variable
c     period1   If true there are periodical boundary conditions in the
c               1 direction
c     period2   If true there are periodical boundary conditions in the
c               2 direction
c     period3   If true there are periodical boundary conditions in the
c               3 direction
c ********************************************************************
c
c                         I/O
c
c ********************************************************************
c
c                      SUBROUTINES CALLED
c
c ********************************************************************
c
c                     ERROR MESSAGES
c
c ********************************************************************
c
c                     PSEUDO CODE
c
c ======================================================================
c
c     --- Detect type of periodicity

      period1 = .false.
      period2 = .false.
      period3 = .false.
      if ( mod(iperiod,10).eq.1 ) period1 = .true.
      if ( mod(iperiod/10,10).eq.1 ) period2 = .true.
      if ( mod(iperiod/100,10).eq.1 ) period3 = .true.


      if ( period1 ) then

c     --- periodical boundary conditions in 1-direction

         do j = 1, nj
            do k = 1, nk
               solut(ni+1,j,k) = solut(1 ,j,k)
               solut(0   ,j,k) = solut(ni,j,k)
            end do
         end do

      end if


      if ( period2 ) then

c     --- periodical boundary conditions in 2-direction


         do i = 1, ni
            do k = 1, nk
               solut(i,nj+1,k) = solut(i,1 ,k)
               solut(i,0   ,k) = solut(i,nj,k)
            end do
         end do

      end if

      if ( period3 ) then

c     --- periodical boundary conditions in 3-direction


         do i = 1, ni
            do j = 1, nj
               solut(i,j,nk+1) = solut(i,j,1 )
               solut(i,j,0   ) = solut(i,j,nk)
            end do
         end do

      end if
      end
