      subroutine uscalcrho ( eddy, solut, ni, nj, nk, nvirtual, ndim,
     +                       ndegfd, ilowloop, iupploop, jlowloop,
     +                       jupploop, klowloop, kupploop )
c ======================================================================
c
c        programmer     Philipe Reynier
c        version 1.0    date 06-07-1999
c
c
c   copyright (c) 1999-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c   Sample user subroutine to compute the density rho
c   The density rho is calculated from the internal energy equation
c   The density rho is put into eddy(.,2)
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nk, ndim, nvirtual, ndegfd, ilowloop, iupploop,
     +        jlowloop, jupploop, klowloop, kupploop
      double precision solut(1-nvirtual:1+ni+nvirtual,
     +                       1-nvirtual:1+nj+nvirtual,1:ndegfd),
     +                 eddy(1-nvirtual:1+ni+nvirtual,
     +                      1-nvirtual:1+nj+nvirtual,1:7),
     +                 phiint(1-nvirtual:1+ni+nvirtual,
     +                        1-nvirtual:1+nj+nvirtual),
     +                 a, b, c, rk2, R, rn

c     eddy           o    array EDDY contains the values of the
c                         eddy viscosity of that block in the centroids
c                         of the elements. The storage of such
c                         an array EDDY is identical to that of all other
c                         coefficients.
c     ilowloop         i    ?
c     iupploop          i    upploop position in ibuffr that may be used
c     jlowloop         i    ?
c     jupploop          i    changed upper j-index for a subface
c     klowloop         i    ?
c     kupploop          i    upploop blank on position 80 or before, first non-b
c                         beyond found upploop blank
c     ndegfd         i    Number of degrees of freedom
c     ndim           i    Dimension of space (2 or 3)
c     ni             i    Number of non-virtual cells in the i-direction
c                         (one block)
c     nj             i    Number of non-virtual cells in the j-direction
c                         (one block)
c     nk             i    Number of non-virtual cells in the k-direction
c                         (one block)
c     nvirtual       i    Number of rows of virtual cells
c     solut          i    Solution array
c                         For a description see the programmers guide
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      double precision const, den, vol
      integer i, j

c     const
c     den
c     i
c     j
c     rho
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c         usccoefhydr
c         uscompphiint
c
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c

      call uscoefhydr(a,b,c,rk2,R,rn)

c Computation of the constant in the second term

      const = c*(rn +1d0)

c Computation of the integral of phi

      call uscompphiint(eddy,phiint,ni,nj,nk,nvirtual,ndim)

c     --- Calculation of the density from the thermodynamic equation
c         for the internal energy
c         eddy(i,j,2) is the density
c         eddy(i,j,7) is the temperature
c         solut(i,j,4) is the internal energy

      do i=ilowloop,iupploop
         do j=jlowloop,jupploop
            den = rk2 + phiint(i,j) - solut(i,j,4)
            vol = const/(den*(eddy(i,j,7)**rn)) - b
            eddy(i,j,2) = 1d0/vol
         end do
      end do
      end
