      subroutine usfilsrc(f,ni,nj,nk,ndim,nvirtual,coor,solut,ndegfd,
     +                    time,eddy,axisym,mturb)
c ======================================================================
c
c        programmer     Philipe Reynier
c        version 1.0    date 06-07-1999
c
c
c   copyright (c) 1999-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c   Sample user subroutine to calculate the internal energy rhs
c   (equation written in 2D).
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer axisym,ni,nj,nk,ndim,nvirtual,ndegfd,mturb
      double precision f(1-nvirtual:1+ni+nvirtual,
     +                   1-nvirtual:1+nj+nvirtual),
     +                 coor(1-nvirtual:1+ni+nvirtual,
     +                      1-nvirtual:1+nj+nvirtual,1:2),
     +                 solut(1-nvirtual:1+ni+nvirtual,
     +                       1-nvirtual:1+nj+nvirtual,1:ndegfd),
     +                 eddy(1-nvirtual:1+ni+nvirtual,
     +                      1-nvirtual:1+nj+nvirtual,1:7),
     +                 dx(1-nvirtual:1+ni+nvirtual,
     +                    1-nvirtual:1+nj+nvirtual),
     +                 dy(1-nvirtual:1+ni+nvirtual,
     +                    1-nvirtual:1+nj+nvirtual),
     +                 tauxi(1-nvirtual:1+ni+nvirtual,
     +                       1-nvirtual:1+nj+nvirtual),
     +                 tauxx(1-nvirtual:1+ni+nvirtual,
     +                       1-nvirtual:1+nj+nvirtual),
     +                 tauxy(1-nvirtual:1+ni+nvirtual,
     +                       1-nvirtual:1+nj+nvirtual),
     +                 tauyy(1-nvirtual:1+ni+nvirtual,
     +                       1-nvirtual:1+nj+nvirtual),
     +                 tauteta(1-nvirtual:1+ni+nvirtual,
     +                         1-nvirtual:1+nj+nvirtual),
     +                 dkdx(1-nvirtual:1+ni+nvirtual,
     +                      1-nvirtual:1+nj+nvirtual),
     +                 dkdy(1-nvirtual:1+ni+nvirtual,
     +                      1-nvirtual:1+nj+nvirtual),
     +                 dudx(1-nvirtual:1+ni+nvirtual,
     +                      1-nvirtual:1+nj+nvirtual),
     +                 dudy(1-nvirtual:1+ni+nvirtual,
     +                      1-nvirtual:1+nj+nvirtual),
     +                 dvdx(1-nvirtual:1+ni+nvirtual,
     +                      1-nvirtual:1+nj+nvirtual),
     +                 dvdy(1-nvirtual:1+ni+nvirtual,
     +                      1-nvirtual:1+nj+nvirtual),
     +                 div(1-nvirtual:1+ni+nvirtual,
     +                      1-nvirtual:1+nj+nvirtual),
     +                 vy(1-nvirtual:1+ni+nvirtual,
     +                    1-nvirtual:1+nj+nvirtual),
     +                 dTdx(1-nvirtual:1+ni+nvirtual,
     +                      1-nvirtual:1+nj+nvirtual),
     +                 dTdy(1-nvirtual:1+ni+nvirtual,
     +                      1-nvirtual:1+nj+nvirtual),
     +                 diffx(1-nvirtual:1+ni+nvirtual,
     +                       1-nvirtual:1+nj+nvirtual),
     +                 diffy(1-nvirtual:1+ni+nvirtual,
     +                       1-nvirtual:1+nj+nvirtual),
     +                 rmu(1-nvirtual:1+ni+nvirtual,
     +                     1-nvirtual:1+nj+nvirtual),
     +                 rmut(1-nvirtual:1+ni+nvirtual,
     +                     1-nvirtual:1+nj+nvirtual),
     +                 rkap(1-nvirtual:1+ni+nvirtual,
     +                      1-nvirtual:1+nj+nvirtual),
     +                 rkapt(1-nvirtual:1+ni+nvirtual,
     +                      1-nvirtual:1+nj+nvirtual),
     +                 sourck(1-nvirtual:1+ni+nvirtual,
     +                      1-nvirtual:1+nj+nvirtual),
     +                 vist(1-nvirtual:1+ni+nvirtual,
     +                     1-nvirtual:1+nj+nvirtual),
     +                 visk(1-nvirtual:1+ni+nvirtual,
     +                     1-nvirtual:1+nj+nvirtual),
     +                 time

c     coor           i    Array containing the coordinates for one block
c                         In 2d coor(i,j,.) contains the x resp y-coordinate
c                         of point i,j
c                         in 3d: coor(i,j,k,.) corresponds to (i,j,k)
c     eddy           i    array EDDY contains the values of the
c                         eddy viscosity of that block in the centroids
c                         of the elements. The storage of such
c                         an array EDDY is identical to that of all other
c                         coefficients.
c     f              o    Array containing the force term
c     axisym              Parameter equal to 1 in axisymmetric and 0
c                         in 2D
c     mturb          i    Indicates type of turbulence modelling
c     ndegfd         i    Number of degrees of freedom
c     ndim           i    Dimension of space (2 or 3)
c     ni             i    Number of non-virtual cells in the i-direction
c                         (one block)
c     nj             i    Number of non-virtual cells in the j-direction
c                         (one block)
c     nk             i    Number of non-virtual cells in the k-direction
c                         (one block)
c     nvirtual       i    Number of rows of virtual cells
c     solut          i    Solution array
c                         For a description see the programmers guide
c     time           i    Actual time of time step
c     diffx          diffusive term along x ou r
c     diffy          diffusive term along y ou z
c     div
c     dTdx           derivative of T along x calculated at the pressure points
c     dTdy           derivative of T along y calculated at the pressure points
c     dkdx           derivative of k along x calculated at the pressure points
c     dkdy           derivative of k along y calculated at the pressure points
c     dudx           derivative of u along x calculated at the pressure points
c     dudy           derivative of u along y calculated at the pressure points
c     dvdx           derivative of v along x calculated at the pressure points
c     dvdy           derivative of v along y calculated at the pressure points
c     dx             space step along x of a pressure point
c     dy             space step along y of a pressure point
c     rkap           thermal conductivity
c     rmu            viscosity (dynamic)
c     rmut           turbulent kinematic viscosity
c     tauxx          component of the stress tensor calculated at the
c                    pressure points
c     tauxy          component of the stress tensor calculated at the
c                    pressure points
c     tauyy          component of the stress tensor calculated at the
c                    pressure points
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      double precision diftx, difty, g, Pk, Des, Dif, sigmak
      integer i, j

c     g
c     i              General loop variable
c     j              General loop variable
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- dynamic viscosity
      call uscompvisc(rmu,eddy,ni,nj,nk,nvirtual,ndim)

c     --- thermic conductivity
      call uscompkap(rkap,eddy,ni,nj,nk,nvirtual,ndim)

c eddy(i,j,1) is the turbulent kinematic viscosity
c Calculation of the total viscosity and conductivity

      print *,'in usfilsrc: mturb',mturb
      if (mturb.ne.0) then
         sigmak = 1d0
         do i=1-nvirtual,ni+nvirtual-1
            do j=1-nvirtual,nj+nvirtual-1
               vist(i,j) = eddy(i,j,2)*eddy(i,j,1)
               rmut(i,j) = rmu(i,j) + vist(i,j)
               rkapt(i,j) = (1d0 + vist(i,j)/rmu(i,j))*rkap(i,j)
               visk(i,j) = rmu(i,j) + vist(i,j)/sigmak
            end do
         end do
      else
         do i=1-nvirtual,ni+nvirtual-1
            do j=1-nvirtual,nj+nvirtual-1
               rmut(i,j) = rmu(i,j)
               rkapt(i,j) = rkap(i,j)
            end do
         end do
      endif

c Cas 2D plan

      if (axisym.eq.0) then

c     --- space steps along x and y

         do i=1-nvirtual,ni+nvirtual-1
            do j=1-nvirtual,nj+nvirtual-1
               dx(i,j) = coor(i+1,j,1) - coor(i,j,1)
               dy(i,j) = coor(i,j+1,2) - coor(i,j,2)
            end do
         end do
         i=ni+nvirtual
         do j=1-nvirtual,nj+nvirtual-1
            dx(i,j) = dx(i-1,j)
            dy(i,j) = coor(i,j+1,2) - coor(i,j,2)
         end do
         j=nj+nvirtual
         do i=1-nvirtual,ni+nvirtual-1
            dx(i,j) = coor(i,j,1) - coor(i-1,j,1)
            dy(i,j) = dy(i,j-1)
         end do
         i=ni+nvirtual
         j=nj+nvirtual
         dx(i,j) = dx(i-1,j)
         dy(i,j) = dy(i,j-1)

c     --- calculation of the derivatives along x and y
c         solut(i,j,1) is the u velocity
c         solut(i,j,2) is the v velocity
c         solut(i,j,5) is the turbulent energy
c         eddy(i,j,7) is the temperature

         do i=1-nvirtual+1,ni+nvirtual-1
            do j=1-nvirtual+1,nj+nvirtual-1
               vy(i,j) = 0.d0
               dudx(i,j) = (solut(i+1,j,1)-solut(i,j,1))/dx(i,j)
               dkdx(i,j) = (solut(i+1,j,5)-solut(i-1,j,5))/
     +                   (0.5d0*dx(i-1,j)+dx(i,j)+0.5d0*dx(i+1,j))
               dvdy(i,j) = (solut(i,j+1,2)-solut(i,j,2))/dy(i,j)
               div(i,j) = dudx(i,j) + dvdy(i,j)
               dkdy(i,j) = (solut(i,j+1,5)-solut(i,j-1,5))/
     +                   (0.5d0*dy(i,j-1)+dy(i,j)+0.5d0*dx(i,j+1))
               dudy(i,j) = 0.5d0*(solut(i+1,j+1,1)+solut(i,j+1,1)
     +                    -solut(i+1,j-1,1)-solut(i,j-1,1))/
     +                    (0.5d0*dy(i,j-1)+dy(i,j)+0.5d0*dy(i,j+1))
               dvdx(i,j) = 0.5d0*(solut(i+1,j+1,2)+solut(i+1,j,2)
     +                    -solut(i-1,j+1,2)-solut(i-1,j,2))/
     +                    (0.5d0*dx(i-1,j)+dx(i,j)+0.5d0*dy(i+1,j))
               dTdx(i,j) = (eddy(i+1,j,7)-eddy(i-1,j,7))/
     +                   (0.5d0*dx(i-1,j)+dx(i,j)+0.5d0*dx(i+1,j))
               dTdy(i,j) = (eddy(i,j+1,7)-eddy(i,j-1,7))/
     +                   (0.5d0*dy(i,j-1)+dy(i,j)+0.5d0*dx(i,j+1))
            end do
         end do

c     --- calculation of diffusive terms
         do i=1,ni
            do j=1,nj
               diffx(i,j) = (rkapt(i+1,j)*dTdx(i+1,j)
     +                       -rkapt(i-1,j)*dTdx(i-1,j))/
     +                      (0.5d0*dx(i-1,j)+dx(i,j)+0.5d0*dx(i+1,j))
               diffy(i,j) = (rkapt(i,j+1)*dTdy(i,j+1)
     +                      -rkapt(i,j-1)*dTdy(i,j-1))/
     +                 (0.5d0*dy(i,j-1)+dy(i,j)+0.5d0*dy(i,j+1))
            end do
         end do


c Cas 2D axisymmetrique: coor1 represente r et coor2, x. Tout est
c inverse Il en est de meme pour les  vitesses solut1 est V tandis
c que solut2 est U.

      elseif (axisym.eq.1) then

c     --- space steps along x and y

         do i=1-nvirtual,ni+nvirtual-1
            do j=1-nvirtual,nj+nvirtual-1
               dy(i,j) = coor(i+1,j,1) - coor(i,j,1)
               dx(i,j) = coor(i,j+1,2) - coor(i,j,2)
            end do
         end do
         j=nj+nvirtual
         do i=1-nvirtual,ni+nvirtual-1
            dx(i,j) = dx(i,j-1)
            dy(i,j) = coor(i+1,j,1) - coor(i,j,1)
         end do
         i=ni+nvirtual
         do j=1-nvirtual,nj+nvirtual-1
            dx(i,j) = coor(i,j,2) - coor(i,j-1,2)
            dy(i,j) = dy(i-1,j)
         end do
         i=ni+nvirtual
         j=nj+nvirtual
         dx(i,j) = dx(i,j-1)
         dy(i,j) = dy(i-1,j)

c     --- calculation of the derivatives along x and y
c         solut(i,j,1) is the v velocity
c         solut(i,j,2) is the u velocity
c         eddy(i,j,7) is the temperature

         do i=1-nvirtual+1,ni+nvirtual-1
            do j=1-nvirtual+1,nj+nvirtual-1
               dudx(i,j) = (solut(i,j+1,2)-solut(i,j,2))/dx(i,j)
               dvdy(i,j) = (solut(i+1,j,1)-solut(i,j,1))/dy(i,j)
               if (coor(i,j,1).ne.0d0) then
                  vy(i,j) = solut(i,j,1)/coor(i,j,1)
               else
                  vy(i,j) = 0d0
               endif
               div(i,j) = dudx(i,j) + dvdy(i,j) + vy(i,j)
               dudy(i,j) = 0.5d0*(solut(i+1,j+1,2)+solut(i+1,j,2)
     +                 -solut(i-1,j+1,2)-solut(i-1,j,2))/
     +                 (0.5d0*dy(i-1,j)+dy(i,j)+0.5d0*dy(i+1,j))
               dvdx(i,j) = 0.5d0*(solut(i+1,j+1,1)+solut(i,j+1,1)
     +                 -solut(i+1,j-1,1)-solut(i,j-1,1))/
     +                 (0.5d0*dx(i,j-1)+dx(i,j)+0.5d0*dy(i,j+1))
               dTdx(i,j) = (eddy(i,j+1,7)-eddy(i,j-1,7))/
     +                   (0.5d0*dx(i,j-1)+dx(i,j)+0.5d0*dx(i,j+1))
               dTdy(i,j) = (eddy(i+1,j,7)-eddy(i-1,j,7))/
     +                   (0.5d0*dy(i-1,j)+dy(i,j)+0.5d0*dx(i+1,j))
               dkdx(i,j) = (solut(i,j+1,5)-solut(i,j-1,5))/
     +                   (0.5d0*dx(i,j-1)+dx(i,j)+0.5d0*dx(i,j+1))
               dkdy(i,j) = (solut(i+1,j,5)-solut(i-1,j,5))/
     +                   (0.5d0*dy(i-1,j)+dy(i,j)+0.5d0*dx(i+1,j))
            end do
         end do

c     --- calculation of diffusive terms and extra term used in
c         axisymmetric coordinates

         do i=1,ni
            do j=1,nj
               diffx(i,j) = (rkapt(i,j+1)*dTdx(i,j+1)
     +                    -rkapt(i,j-1)*dTdx(i,j-1))/
     +                   (0.5d0*dx(i,j-1)+dx(i,j)+0.5d0*dx(i,j+1))
               if (coor(i,j,1).ne.0d0) then
                  diffy(i,j) = (rkapt(i+1,j)*coor(i+1,j,1)*dTdy(i+1,j)
     +                       -rkapt(i-1,j)*coor(i-1,j,1)*dTdy(i-1,j))/
     +                        ((0.5d0*dy(i-1,j)+dy(i,j)
     +                        + 0.5d0*dy(i+1,j))*coor(i,j,1))
               elseif (coor(i,j,1).eq.0d0) then
                  diffy(i,j) = 0d0
               endif
            end do
         end do
      endif

c     --- calculation of stress tenssor
      do i=1,ni
         do j=1,nj
            tauxx(i,j) = rmut(i,j)*(2d0*dudx(i,j)-(2d0/3d0)*div(i,j))
            tauyy(i,j) = rmut(i,j)*(2d0*dvdy(i,j)-(2d0/3d0)*div(i,j))
            tauxy(i,j) = rmut(i,j)*(dudy(i,j)+dvdx(i,j))
            if (axisym.eq.1) then
               tauteta(i,j) = rmut(i,j)*(2d0*vy(i,j)-(2d0/3d0)*div(i,j))
               tauxi(i,j) = vy(i,j)*(solut(i,j,3) - tauteta(i,j))
            else
               tauteta(i,j) = 0d0
               tauxi(i,j) = 0d0
            endif
         end do
      end do

c     --- calculation of sourve terms for turbulent flows ---
c         k-eps turbulence model is assumed

      if (mturb.eq.0) then
          do i=1,ni
             do j=1,nj
                sourck(i,j) = 0d0
             end do
          end do
      else
          do i=1,ni
             do j=1,nj
c Production term
                Pk = vist(i,j)*(2d0*(dudx(i,j)**2 + dvdy(i,j)**2
     +              + vy(i,j)**2) + (dvdx(i,j) + dudy(i,j))**2)
     +              - (2d0/3d0)*div(i,j)*(vist(i,j)*div(i,j)
     +              + eddy(i,j,2)*solut(i,j,5))

c Destruction term
                Des = - eddy(i,j,2)*solut(i,j,6)

c Diffusion term
                if (axisym.eq.0) then
                   diftx = (visk(i+1,j)*dkdx(i+1,j)
     +                     -visk(i-1,j)*dkdx(i-1,j))/
     +                     (0.5d0*dx(i-1,j)+dx(i,j) + 0.5d0*dx(i+1,j))
                   difty = (visk(i,j+1)*dkdy(i,j+1)
     +                      - visk(i,j-1)*dkdy(i,j-1))/
     +                      (0.5d0*dy(i,j-1)+dy(i,j) + 0.5d0*dy(i,j+1))
                elseif (axisym.eq.1) then
                   diftx = (visk(i,j+1)*dkdx(i,j+1)
     +                    -visk(i,j-1)*dkdx(i,j-1))/
     +                   (0.5d0*dx(i,j-1)+dx(i,j)+0.5d0*dx(i,j+1))
                   if (coor(i,j,1).ne.0d0) then
                       difty = (visk(i+1,j)*coor(i+1,j,1)*dkdy(i+1,j)
     +                       -visk(i-1,j)*coor(i-1,j,1)*dkdy(i-1,j))/
     +                        ((0.5d0*dy(i-1,j)+dy(i,j)
     +                        + 0.5d0*dy(i+1,j))*coor(i,j,1))
                   elseif (coor(i,j,1).eq.0d0) then
                       difty = 0d0
                   endif
                endif
                Dif =  diftx + difty
                sourck(i,j) = Pk + Des + Dif
            end do
         end do
      endif
c     --- computation of right and side : f
c         solut(i,j,3) is the pressure + 2/3 of the turbulent energy
c         eddy(i,j,7) is the temperature
c         eddy(i,j,2) is the density

      do i=1,ni
         do j=1,nj
            g = dudx(i,j)*(solut(i,j,3)-tauxx(i,j))
     +          - tauxy(i,j)*(dvdx(i,j)+dudy(i,j))
     +          + dvdy(i,j)*(solut(i,j,3)-tauyy(i,j))
     +          - diffx(i,j)-diffy(i,j)+tauxi(i,j)
            f(i,j) = - g / eddy(i,j,2)
         end do
      end do
      end
