      subroutine isinitmd
c ======================================================================
c
c        programmer    Guus Segal
c        version 1.0    date   08-02-94 (Subroutine extracted from the ISNAS
c                                        subroutine INITMD version 6.2)
c
c
c
c   copyright (c) 1994  "Ingenieursbureau SEPRA"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "Ingenieursbureau SEPRA".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c ********************************************************************
c
c                         DESCRIPTION
c
c    Store machine dependent quantities in common blocks
c    must be adapted for each local computer
c    Important quantities to be updated are the quantities:
c
c    SPHOME and SPRENV
c
c    The concatenation of these two texts decides where the global
c    environment file is positioned
c    SPRENV gives the name of the local environment file
c    For unix computers as well as MSDOS computers running under FTN77/386
c    these names are set during the installation procedure
c    For other computers the system manager must set the parameters himself
c
c    This subroutine is machine dependent and must be adapted for
c    each local machine
c
c
c      Available versions:
c
c      Computer     Operating system     Comment    machine
c
c      IBM                               cibm         1
c      Cyber        NOS/VE               cybr         2
c      Apollo       Aegis/Unix           capo         3
c      HP 9000      HP/UX                chpu cuni    4
c      ....         Unix                 cuni         5
c      VAX          VMS                  cvax         6
c      Harris                            char         7
c      Cray                              cray         8
c      IBM PC 386   MSDOS+FTN77/386      cipc         9
c      Convex       Unix/Concentrix      ccon        10
c      ALLIANT      Unix/Concentrix      cali        13
c
c
c ********************************************************************
c
c                      KEYWORDS
c
c   initializations
c   machine_dependent
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none

c ********************************************************************
c
c                      COMMON BLOCKS
c
      include 'IScommon/cisnasch'
c ********************************************************************
c
c                      LOCAL PARAMETERS
c
      integer iref
      character * 100 name
      integer length

c    iref     Reference number for the environment file
c    length   Number of characters in the name SPHOME
c ********************************************************************
c
c                      SUBROUTINES CALLED

      integer isgenv
c
c     ISREADMD  Reads the ISNaS environment file and sets constants
c     ISSTMACH  Machine (installation) dependent subroutine that may be used
c               to perform local actions not standard for the ISNaS package
c     ISSTMSDS  MSDOS extra starting subroutine to set some names of files
c     ISSTUNIX  UNIX extra starting subroutine to set some names of files
c ********************************************************************
c
c                    I/O
c
c ********************************************************************
c
c                    ERROR MESSAGES
c
c   -
c ********************************************************************
c
c                    PSEUDO CODE
c
c ======================================================================
c
c     --- Read all parameters from the environment file
c         First the temporary reference number for this file is set
c         This reference number is made equal to 73 and is only used in ISREADMD
c         The file is opened and closed in ISREADMD, so the reference number
c         is free afterwards
c         If necessary 73 should be changed in some other number
c         Next the parameters SPHOME and SPRENV are set
c
c       SPHOME is only used to define the directory where the global environment
c       file is positioned. This name must contain the / or \ extension
c       if this extension is used to identify a subdirectory
c       SPHOME is concatenated with SPRENV to define the name of the
c       global environment file
c       SPRENV gives the name of the local environment file
c       Implicitly it also defines the name of the global environment file
c
c       The names of SPHOME and SPRENV must be adapted at installation
c       For unix and msdos computers this is done automatically by the
c       installation procedure

      iref = 73
      SPHOME = ' '
      SPRENV = 'isnas.env'
cybr      SPRENV = 'isnas_env'

cipcc    --- For DOS the installation is based upon the DOS parameter SPHOME
cipcc        name of the DOS_parameter must be in capitals !
cipc      call dosparam@ ( 'SPHOME', sphome )
cipc      length = index ( sphome, ' ' )-1
cipc      if ( length.le.0 ) then
cipc         print *, 'warning in inifil: DOS_variable SPHOME is empty'
cipc         sphome = 'c:\isnas'
cipc         print *, 'default setting applied:  SPHOME=', sphome
cipc         length = 9
cipc      end if
cipc      sphome(length+1:length+8) = '\instal\'
cuni      length = isgenv(1,name)
cuni      if ( length.gt.0 ) then
cuni         sphome(1:length) = name(1:length)
cuni         sphome(length+1:length+8) = '/instal/'
cuni      else
cuni         write(6,1)
cuni  1      format ( ' No environment variable SPHOME found'/
cuni     +            ' Please make an environment variable SPHOME'/
cuni     +            ' and give this parameter the name of the ISNAS',
cuni     +            ' home directory'//
cuni     +            ' For example in c-shell:',
cuni     +            ' put the line'//' setenv SPHOME isdir '//
cuni     +            ' in your .cshrc file, where sepdir is the name of',
cuni     +            ' the isnas home dir'/
cuni     +            ' In bourne shell change the .profile:'//
cuni     +            ' SPHOME=isdir'/
cuni     +            ' export isdir'//
cuni     +            ' Login out and login after changing this file'//
cuni     +            ' Execution is terminated' )
cuni         stop
cuni      end if

c     --- Read the environment file

      call isreadmd ( iref )

c     --- Call a special procedure for local adaptation of names due to
c         batch files (DOS) or shell scripts with settings (unix)
c         These subroutines are special for the operating system

      if ( stoper(1:5).eq.'msdos' ) then

c     --- DOS operating system

cipc         call isstmsds

      else if ( stoper(1:4).eq.'unix' ) then

c     --- UNIX operating system

         call isstunix

      end if

c     --- Call machine (installation) dependent subroutine:

c      call isstmach

      end
