      subroutine issync ( ibufptr, icount, itype, isource )
c ======================================================================
c
c        Programmer     Jason Frank
c
c     copyright (c) 1994-1996  "TU-Delft"
c     Permission to copy or distribute this software or documentation
c     in hard copy or soft copy granted only by written license
c     obtained from "TU-Delft"
c     All rights reserved. No part of this publication may be
c     reproduced, stored in a retrieval system (e.g., in memory, disk,
c     or core) or be transmitted by any means, electronic, mechanical,
c     photocopy, recording, or otherwise, without written Permission
c     from the publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     This function ensures that all nodes have the same value
c     for an array as isource
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ibufptr, icount, itype, isource

c     ibufptr  i    pointer to first element of array to be sent
c     icount   i    the length of array to be sent
c     itype    i    MPI datatype
c     isource  i    rank of the value to synchronize with
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisparal'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ierr
cpar      include 'mpif.h'

c     ierr      error status of an MPI call ( .eq.MPI_SUCCESS if ok)
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRINT  Put integer value in error message file
c     ISERRSUB  Error message subroutine
c     MPI_BCAST MPI broadcast function
c **********************************************************************
c
c                       ERROR MESSAGES
c
c    1750   MPI error
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
cpar      call iseropen ( 'issync' )

cpar      call MPI_BCAST ( ibufptr, icount, itype, isource, 
cpar     +   MPI_COMM_WORLD, ierr)
cpar      if ( ierr.ne.MPI_SUCCESS ) then
cpar         call iserrint ( ierr, 1 )
cpar         call iserrsub ( 1750, 1, 0, 0 )
cpar      end if

cpar      call iserclos ( 'issync' )
      end
