      subroutine isrecsub ( ifirst, itypec, lengrc, rarray, iarray,
     +                      carray, inode, iblock )
c ======================================================================
c
c        Programmer     Guus Segal
c        version 1.4    date 18-09-1998 Jason: MPI version
c        version 1.3    date 17-09-1998 bug fixed fortran 90 case
c        version 1.2    date 20-02-1996 Correction node number
c        version 1.1    date 08-02-1996 Adaptations for multiple errors
c        version 1.0    date 01-02-1996 Body copied from isreceiv
c
c
c     copyright (c) 1996-1996  "TU-Delft"
c     Permission to copy or distribute this software or documentation
c     in hard copy or soft copy granted only by written license
c     obtained from "TU-Delft"
c     All rights reserved. No part of this publication may be
c     reproduced, stored in a retrieval system (e.g., in memory, disk,
c     or core) or be transmitted by any means, electronic, mechanical,
c     photocopy, recording, or otherwise, without written Permission
c     from the publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Receive data from node or host.
c     This implementation is completely based on MPI
c     Body of subroutine isreceiv
c **********************************************************************
c
c                       KEYWORDS
c
c     parallel
c     receive
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ifirst, itypec, lengrc, iarray(*), inode, iblock
      double precision rarray(*)
      character *(*) carray(*)

c     carray    i    Character array for issend and isreceiv
c     iarray    i    Integer array for issend and isreceiv
c     iblock    o    Block sequence number
c     ifirst    i    reserved
c     inode     o    Node number of node from which the message has been sent
c                    inode = 0, means host
c     itypec    i    reserved
c     lengrc   i/o   Number of data to be received, i.e. number of integers,
c                    strings or reals.
c     rarray    i    Real array to be used for sending and receiving of data
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'
      include 'IScommon/ciscom'
      include 'IScommon/cismessg'
      include 'IScommon/cismessc'
      include 'IScommon/cisparal'
      include 'IScommon/cisblank'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      character * 8 namsubcp(20)
      logical firstrec, inforec, getinfo, getdata, receive, specdata,
     +        ready, stopstat
      integer, dimension(:), allocatable :: loc_ints, loc_reals
      integer msgtype, ierr, nints, nreal, nchar, ityperf, i,
     +        levelcop, iblocksv
cpar      include 'mpif.h'
      integer status(MPI_STATUS_SIZE)
      save firstrec, inforec, stopstat
      data firstrec /.true./, inforec /.false./, stopstat /.false./

c     firstrec  If true the next data to be received is a new series of data
c     getdata   If true data must be unpacked
c     getinfo   If true the information array must be unpacked
c     i         General loop variable
c     iblocksv  Help variable to store the value of iblock
c     ierr      Return code of a subroutine (<0 is error)
c     inforec   If true the next data to be unpacked is the information array
c     ityperf   Reference value with respect to itypec
c     levelcop  Help variable to save the value of levelsub
c     loc_ints  Local dimensioned integer array for the print subroutine
c     loc_reals Local dimensioned real array for the print subroutine
c     msgtype   Message tag for MPI, may be used for identification
c     namsubcp  Help array to store the names in namsubs temporarily
c     nchar     Number of strings in a message
c     nints     Number of integers in a message
c     nreal     Number of reals in a message
c     ready     Indicates if a while loop is finished or not
c     receive   If true new data must be received
c     specdata  If true the data to be received is of special type
c     status    MPI receive status array
c     stopstat  If true the program must be stopped after closing MPI
c **********************************************************************
c
c                       I/O
c
c     Information is received host or node depending on the parameter
c     ifirst
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISCLOSSB  Closing subroutine
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRINT  Put integer value in error message file
c     ISERRSUB  Error messages
c     ISEXITPM  exit parallel machine (MPI)
c     ISPRINT   ISNaS general print subroutine
c     ISSEND    Send data to node or host
c     ISWARSUB  Print warnings
c     MPI_RECV  Blocks until a message with specified message tag has arrived
c               from the specified source, and places it in a new active
c               receive buffer. MPI subroutine.
c     MPI_UNPACK  Unpack the active message buffer into arrays of prescribed
c               data type. MPI subroutine.
c **********************************************************************
c
c                       ERROR MESSAGES
c
c    1708   first_receive = false, whereas ifirst = 1 or 3
c    1709   No data has been read after the previous reading of the
c           information arra
c    1710   itypec is not info(1)
c    1711   length is not info(2)
c    1750   Impossible to clear buffer, the process is halted
c **********************************************************************
c
c                       PSEUDO CODE
c
c     At initialization:
c        first_receive := true
c        info_receive := false
c     receive := ifirst=1 or ifirst=3
c     get_info := ifirst < 5
c     get_data := ifirst <= 2 or ifirst=5
c     if ( receive ) then
c        if ( not first_receive ) error
c        first_receive := false
c        receive data
c     ready := false
c     while ( not ready ) do
c        special_data := false
c        if ( get_info ) then
c           if ( info_receive ) error
c           unpack info array and put in common ciscom
c           info_receive := true
c           itypec_ref := info(1)
c           if ( itypec_ref<0 ) then
c              Unpack and print message depending on itypec
c              if ( itypec_ref=-1 ) then
c                 set ierror
c              info_receive := false
c              special_data := true
c        if ( get_data and not special_data ) then
c           Check itypec and lengrc
c           Unpack array and put in correct array
c           info_receive := false
c        if ( not special_data ) then
c           ready := true
c     if ( info(10)=1 ) then
c        first_receive := true
c        info_receive := false
c     if ( ifirst=3 or ifirst=4 ) then
c        lengrc := info(2)
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
cpar      call iseropen ( 'isrecsub' )

cpar      receive = ifirst.eq.1 .or. ifirst.eq.3 .or. ifirst.eq.6
cpar      getinfo = ifirst.lt.5 .or. ifirst.eq.6
cpar      getdata = ifirst.le.2 .or. ifirst.eq.5
cpar      ityperf = itypec
cpar 50   if ( receive ) then

c     --- receive is true, receive data

cpar         if ( ifirst.ne.6 .and. .not. firstrec ) then

c        --- Error not all previous information has been unpacked

cpar            call iserrint ( ifirst, 1 )
cpar            call iserrsub ( 1708, 1, 0, 0 )

cpar         end if
cpar         firstrec = .false.

c        --- receive data

cpar         msgtype = 1
cpar         call MPI_RECV ( irecbuf, mxmsglen, MPI_PACKED,
cpar     +      MPI_ANY_SOURCE, msgtype, MPI_COMM_WORLD, status, ierr)

cpar         if ( ierr.ne.MPI_SUCCESS ) then

c        --- Error in MPI

cpar            write (6,*) 'MPI error ', ierr, ' in function isrecsub'
cpar            call iserrsub ( 1750, 0, 0, 0 )

cpar         end if
cpar         irecpos = 1

cpar      end if
cpar      ready = .false.

cpar100   if ( .not. ready ) then

c     --- while loop ready

cpar         specdata = .false.
cpar         if ( getinfo ) then

c        --- getinfo = true, unpack information array

cpar            if ( ifirst.ne.6 .and. inforec )  then

c           --- Error no data has been read after the previous reading of
c               the information array

cpar               call iserrint ( ifirst, 1 )
cpar               call iserrsub ( 1709, 1, 0, 0 )

cpar            end if

c           --- unpack info array and put in common ciscom

cpar            call MPI_UNPACK ( irecbuf, mxmsglen, irecpos, info, 10,
cpar     +          MPI_INTEGER, MPI_COMM_WORLD, ierr )
cpar            iblocksv = iblockrf
cpar            iblockrf = info(8)
cpar            if ( ierr.ne.MPI_SUCCESS ) then

c           --- Error in MPI

cpar               write (6,*) 'MPI error ', ierr,
cpar     +             ' in function isrecsub'
cpar               call iserrsub ( 1750, 0, 0, 0 )

cpar            end if
cpar            inforec = .true.
cpar            ityperf = info(1)
cpar            if ( ityperf.eq.-10 ) then

c           --- Stop signal received by host

cpar               stopstat = .true.
cpar               getdata = .false.

cpar            else if ( ityperf.eq.-9 ) then

c           --- Stop signal received by node
c               Send receive signal and close application

cpar               info(1) = -10
cpar               call issend ( 0, -4, 0, rarray, iarray, carray, 0 )
cpar               call isexitpm
cpar               call isclossb

cpar            else if ( ityperf.lt.0 ) then

c           --- ityperf<0, Unpack and print message depending on ityperf

cpar               nints = info(3)
cpar               nreal = info(4)
cpar               nchar = info(5)
cpar               if ( ityperf.gt.-3 ) then

c              --- ityperf = -1 or -2, error or warning
c                  Get traceback information

cpar                  levelcop = levelsub
cpar                  do i = 1, levelsub
cpar                     namsubcp(i) = namsubs(i)
cpar                  end do
cpar                  levelsub = info(6)
cpar                  call MPI_UNPACK ( irecbuf, mxmsglen, irecpos,
cpar     +               namsubs, levelsub*8, MPI_CHARACTER,
cpar     +               MPI_COMM_WORLD, ierr )

cpar               end if

cpar               if ( nints.gt.1000 ) then

c              --- Special situation, use FORTRAN 90 extension allocate

cpar                  allocate (loc_ints(1:nints))
cpar                  call MPI_UNPACK ( irecbuf, mxmsglen, irecpos,
cpar     +               loc_ints, nints, MPI_INTEGER, MPI_COMM_WORLD,
cpar     +               ierr )

cpar               else if ( nints.gt.0 ) then

c              --- Standard situation, use cismessg

cpar                  call MPI_UNPACK ( irecbuf, mxmsglen, irecpos,
cpar     +               ints, nints, MPI_INTEGER, MPI_COMM_WORLD,
cpar     +               ierr )

cpar               end if
cpar               if ( ierr.ne.MPI_SUCCESS ) then

c              --- Error in mpi

cpar                  write (6,*) 'MPI error ', ierr,
cpar     +                ' in function isrecsub'
cpar                  call iserrsub ( 1750, 0, 0, 0 )

cpar               end if

cpar               if ( nreal.gt.1000 ) then

c              --- Special situation, use FORTRAN 90 extension allocate

cpar                  allocate (loc_reals(nreal))
cpar                  call MPI_UNPACK ( irecbuf, mxmsglen, irecpos,
cpar     +               loc_reals, nreal, MPI_DOUBLE_PRECISION,
cpar     +               MPI_COMM_WORLD, ierr )

cpar               else if ( nreal.gt.0 ) then

c              --- Standard situation, use cismessg

cpar                  call MPI_UNPACK ( irecbuf, mxmsglen, irecpos,
cpar     +               reals, nreal, MPI_DOUBLE_PRECISION,
cpar     +               MPI_COMM_WORLD, ierr )

cpar               end if
cpar               if ( ierr.ne.MPI_SUCCESS ) then

c              --- Error in mpi

cpar                  write (6,*) 'MPI error ', ierr,
cpar     +               ' in function isrecsub'
cpar                  call iserrsub ( 1750, 0, 0, 0 )

cpar               end if

cpar               call MPI_UNPACK ( irecbuf, mxmsglen, irecpos,
cpar     +               chars, nchar*80, MPI_CHARACTER,
cpar     +               MPI_COMM_WORLD, ierr )

cpar               if ( ierr.ne.MPI_SUCCESS ) then

c              --- Error in mpi

cpar                  write (6,*) 'MPI error ', ierr,
cpar     +               ' in function isrecsub'
cpar                  call iserrsub ( 1750, 0, 0, 0 )

cpar               end if
cpar               if ( ityperf.eq.-1 ) then

c              --- ityperf = -1: error

cpar                  inodenr = info(9)
cpar                  call iserrsub ( -info(2), nints, nreal, nchar )
cpar                  inodenr = 0
cpar                  stopstat = .true.
cpar                  if ( ifirst.eq.6 ) go to 50

cpar               else if ( ityperf.eq.-2 ) then

c              --- ityperf = -2: warning

cpar                  inodenr = info(9)
cpar                  call iswarsub ( -info(2), nints, nreal, nchar )
cpar                  inodenr = 0

cpar               else

c              --- ityperf = -3: output

cpar                  if ( nints.le.1000 .and. nreal.le.1000 ) then
cpar                     call isprint ( info(2), nints, ints, nreal, reals,
cpar     +                              nchar, chars )
cpar                  else if ( nints.gt.1000 .and. nreal.le.1000 ) then
cpar                     call isprint ( info(2), nints, loc_ints, nreal,
cpar     +                              reals, nchar, chars )
cpar                     deallocate (loc_ints)
cpar                  else if ( nints.le.1000 .and. nreal.gt.1000 ) then
cpar                     call isprint ( info(2), nints, ints, nreal,
cpar     +                              loc_reals, nchar, chars )
cpar                     deallocate (loc_reals)
cpar                  else
cpar                     call isprint ( info(2), nints, loc_ints, nreal,
cpar     +                              loc_reals, nchar, chars )
cpar                     deallocate (loc_ints, loc_reals)
cpar                   end if
cpar               end if
cpar               inodenr = 0
cpar               if ( ityperf.gt.-3 ) then

c              --- ityperf = -1 or -2, error or warning
c                  Reset traceback information

cpar                  levelsub = levelcop
cpar                  do i = 1, levelsub
cpar                     namsubs(i) = namsubcp(i)
cpar                  end do

cpar               end if

cpar               inforec = .false.
cpar               specdata = .true.

cpar            end if

cpar         end if

cpar         if ( getdata .and. .not. specdata ) then

c        --- Check itypec and lengrc

cpar            if ( itypec.ne.info(1) ) then

c           --- itypec has incorrect value

cpar               call iserrint ( itypec, 1 )
cpar               call iserrint ( info(1), 2 )
cpar               call iserrsub ( 1710, 2, 0, 0 )

cpar            end if
cpar            if ( lengrc.ne.info(2) ) then

c           --- lengrc has incorrect value

cpar               call iserrint ( lengrc, 1 )
cpar               call iserrint ( info(2), 2 )
cpar               call iserrsub ( 1711, 2, 0, 0 )

cpar            end if

c           --- Unpack array and put in correct array

cpar            if ( ierror.ne.0 ) go to 1000
cpar
cpar            if ( stopstat ) then
cpar               if ( ityperf.eq.0 .and. lengrc.gt.0 ) then
cpar                  call MPI_UNPACK ( irecbuf, mxmsglen, irecpos,
cpar     +               chars, lengrc*len(carray(1)), MPI_CHARACTER,
cpar     +               MPI_COMM_WORLD, ierr)
cpar               else if ( ityperf.eq.3 .and. lengrc.gt.0 ) then
cpar                  call MPI_UNPACK ( irecbuf, mxmsglen, irecpos,
cpar     +               ibuffr, lengrc, MPI_INTEGER, MPI_COMM_WORLD,
cpar     +               ierr )
cpar               else if ( lengrc.gt.0 ) then
cpar                  call MPI_UNPACK ( irecbuf, mxmsglen, irecpos,
cpar     +               buffer, lengrc, MPI_DOUBLE_PRECISION,
cpar     +               MPI_COMM_WORLD, ierr )
cpar               end if
cpar            else if ( ityperf.eq.0 .and. lengrc.gt.0 ) then
cpar               call MPI_UNPACK ( irecbuf, mxmsglen, irecpos,
cpar     +            carray, lengrc*len(carray(1)), MPI_CHARACTER,
cpar     +            MPI_COMM_WORLD, ierr)
cpar            else if ( ityperf.eq.3 .and. lengrc.gt.0 ) then
cpar               call MPI_UNPACK ( irecbuf, mxmsglen, irecpos,
cpar     +            iarray, lengrc, MPI_INTEGER, MPI_COMM_WORLD,
cpar     +            ierr )
cpar            else if ( lengrc.gt.0 ) then
cpar               call MPI_UNPACK ( irecbuf, mxmsglen, irecpos,
cpar     +            rarray, lengrc, MPI_DOUBLE_PRECISION,
cpar     +            MPI_COMM_WORLD, ierr )
cpar            end if
cpar            if ( ierr.ne.MPI_SUCCESS ) then

c           --- Error in mpi

cpar               write (6,*) 'MPI error ', ierr,
cpar     +            ' in function isrecsub'
cpar               call iserrsub ( 1750, 0, 0, 0 )

cpar            end if

cpar            inforec = .false.

cpar         end if

cpar         if ( .not. specdata ) ready = .true.
cpar         go to 100

cpar      end if

cpar      if ( info(10).eq.1 ) then

c     --- Final read

cpar         firstrec = .true.
cpar         inforec = .false.

cpar      end if
cpar      if ( ifirst.eq.3 .or. ifirst.eq.4 ) lengrc = info(2)
cpar      iblock = info(8)
cpar      inode  = info(9)
cpar      iblockrf = iblocksv

cpar1000  if ( ierror.ne.0 ) then
cpar          stopstat = .true.
cpar      else
cpar          call iserclos ( 'isrecsub' )
cpar      end if
      end
