      subroutine istpar
c ======================================================================
c
c        Programmer     Jason Frank
c        version 1.0    date 22-07-1998 Jason: MPI combined version of
c                                         istparnd and istparhs
c
c
c     copyright (c) 1994-1995  "TU-Delft"
c     Permission to copy or distribute this software or documentation
c     in hard copy or soft copy granted only by written license
c     obtained from "TU-Delft"
c     All rights reserved. No part of this publication may be
c     reproduced, stored in a retrieval system (e.g., in memory, disk,
c     or core) or be transmitted by any means, electronic, mechanical,
c     photocopy, recording, or otherwise, without written Permission
c     from the publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Join parallel application.
c     Initialize the common cisparal.
c **********************************************************************
c
c                       KEYWORDS
c
c     initializations
c     parallel
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisparal'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ierr
      include 'mpif.h'

c     ierr      Return code of an MPI subroutine
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERRINT  Put integer value in error message file
c     ISERRSUB  Error messages
c     ISSTOP    Stop ISNAS
c     MPI_COMM_RANK Get rank of process in MPI communication context
c     MPI_COMM_SIZE Get number of processes in MPI communication context
c     MPI_INIT  Enroll in MPI
c **********************************************************************
c
c                       ERROR MESSAGES
c
c    1700   Failure enrolling in MPI
c    1750   MPI error
c **********************************************************************
c
c                       PSEUDO CODE
c
c     Initialize common cisparal
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c

c     --- Enroll in MPI computation

      call MPI_Init ( ierr )

      if ( ierr.ne.MPI_SUCCESS ) then

c     --- an error occurred:

         call iserrint ( inodenr, 1 )
         call iserrsub ( 1750, 1, 0, 0 )
         call isstop
      end if

c     --- Initialize cisparal

      call MPI_COMM_RANK ( MPI_COMM_WORLD, inodenr, ierr)

      if ( ierr.ne.MPI_SUCCESS ) then

c     --- an error occurred:

         call iserrint ( inodenr, 1 )
         call iserrsub ( 1750, 1, 0, 0 )
         call isstop
      end if

      call MPI_COMM_SIZE ( MPI_COMM_WORLD, iacnodes, ierr)

      if ( ierr.ne.MPI_SUCCESS ) then

c     --- an error occurred:

         call iserrint ( inodenr, 1 )
         call iserrsub ( 1750, 1, 0, 0 )
         call isstop
      end if

c     --- in keeping with original definition of nodes:

      iacnodes = iacnodes - 1

      end
